@@ -4,9 +4,11 @@
 
 #include "android_webview/browser/aw_gl_surface.h"
 
+#include "android_webview/browser/scoped_app_gl_state_restore.h"
+
 namespace android_webview {
 
-AwGLSurface::AwGLSurface() : fbo_(0) {}
+AwGLSurface::AwGLSurface() {}
 
 AwGLSurface::~AwGLSurface() {}
 
@@ -17,8 +19,8 @@ bool AwGLSurface::IsOffscreen() {
   return false;
 }
 
-unsigned int AwGLSurface::GetBackingFrameBufferObject() {
-  return fbo_;
+unsigned int AwGLSurface::GetBackingFramebufferObject() {
+  return ScopedAppGLStateRestore::Current()->framebuffer_binding_ext();
 }
 
 gfx::SwapResult AwGLSurface::SwapBuffers() {
@@ -37,8 +39,4 @@ void* AwGLSurface::GetDisplay() {
   return NULL;
 }
 
-void AwGLSurface::SetBackingFrameBufferObject(unsigned int fbo) {
-  fbo_ = fbo;
-}
-
 }  // namespace android_webview
