@@ -7,8 +7,8 @@
 #include <utility>
 
 #include "android_webview/browser/browser_view_renderer_client.h"
-#include "android_webview/browser/child_frame.h"
 #include "android_webview/browser/compositor_frame_consumer.h"
+#include "android_webview/common/aw_switches.h"
 #include "base/auto_reset.h"
 #include "base/command_line.h"
 #include "base/logging.h"
@@ -18,7 +18,6 @@
 #include "base/supports_user_data.h"
 #include "base/trace_event/trace_event_argument.h"
 #include "cc/output/compositor_frame.h"
-#include "cc/output/compositor_frame_ack.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/render_view_host.h"
 #include "content/public/browser/web_contents.h"
@@ -94,12 +93,15 @@ BrowserViewRenderer::BrowserViewRenderer(
     const scoped_refptr<base::SingleThreadTaskRunner>& ui_task_runner)
     : client_(client),
       ui_task_runner_(ui_task_runner),
+      sync_on_draw_hardware_(base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kSyncOnDrawHardware)),
       current_compositor_frame_consumer_(nullptr),
       compositor_(nullptr),
       is_paused_(false),
       view_visible_(false),
       window_visible_(false),
       attached_to_window_(false),
+      was_attached_(false),
       hardware_enabled_(false),
       dip_scale_(0.f),
       page_scale_factor_(1.f),
@@ -107,7 +109,8 @@ BrowserViewRenderer::BrowserViewRenderer(
       max_page_scale_factor_(0.f),
       on_new_picture_enable_(false),
       clear_view_(false),
-      offscreen_pre_raster_(false) {}
+      offscreen_pre_raster_(false),
+      allow_async_draw_(false) {}
 
 BrowserViewRenderer::~BrowserViewRenderer() {
   DCHECK(compositor_map_.empty());
@@ -224,46 +227,54 @@ bool BrowserViewRenderer::OnDrawHardware() {
   ReturnResourceFromParent(current_compositor_frame_consumer_);
   UpdateMemoryPolicy();
 
-  gfx::Size surface_size(size_);
-  gfx::Rect viewport(surface_size);
-  gfx::Rect clip = viewport;
   gfx::Transform transform_for_tile_priority =
       external_draw_constraints_.transform;
 
-  // If the WebView is on a layer, WebView does not know what transform is
-  // applied onto the layer so global visible rect does not make sense here.
-  // In this case, just use the surface rect for tiling.
-  gfx::Rect viewport_rect_for_tile_priority;
+  gfx::Rect viewport_rect_for_tile_priority =
+      ComputeViewportRectForTilePriority();
 
-  // Leave viewport_rect_for_tile_priority empty if offscreen_pre_raster_ is on.
-  if (!offscreen_pre_raster_ && !external_draw_constraints_.is_layer) {
-    viewport_rect_for_tile_priority = last_on_draw_global_visible_rect_;
+  scoped_refptr<content::SynchronousCompositor::FrameFuture> future; // Async.
+  content::SynchronousCompositor::Frame frame; // Sync.
+  bool async = !sync_on_draw_hardware_ && allow_async_draw_;
+  if (async) {
+    future = compositor_->DemandDrawHwAsync(
+        size_, viewport_rect_for_tile_priority, transform_for_tile_priority);
+  } else {
+    frame = compositor_->DemandDrawHw(size_, viewport_rect_for_tile_priority,
+                                      transform_for_tile_priority);
   }
 
-  content::SynchronousCompositor::Frame frame =
-      compositor_->DemandDrawHw(surface_size,
-                                gfx::Transform(),
-                                viewport,
-                                clip,
-                                viewport_rect_for_tile_priority,
-                                transform_for_tile_priority);
-  if (!frame.frame.get()) {
+  if (!frame.frame && !future) {
     TRACE_EVENT_INSTANT0("android_webview", "NoNewFrame",
                          TRACE_EVENT_SCOPE_THREAD);
     return current_compositor_frame_consumer_->HasFrameOnUI();
   }
 
-  std::unique_ptr<ChildFrame> child_frame = base::WrapUnique(new ChildFrame(
-      frame.output_surface_id, std::move(frame.frame), compositor_id_,
-      viewport_rect_for_tile_priority.IsEmpty(), transform_for_tile_priority,
-      offscreen_pre_raster_, external_draw_constraints_.is_layer));
+  allow_async_draw_ = true;
+  std::unique_ptr<ChildFrame> child_frame = base::MakeUnique<ChildFrame>(
+      std::move(future), frame.compositor_frame_sink_id, std::move(frame.frame),
+      compositor_id_, viewport_rect_for_tile_priority.IsEmpty(),
+      transform_for_tile_priority, offscreen_pre_raster_,
+      external_draw_constraints_.is_layer);
 
   ReturnUnusedResource(
-      current_compositor_frame_consumer_->PassUncommittedFrameOnUI());
-  current_compositor_frame_consumer_->SetFrameOnUI(std::move(child_frame));
+      current_compositor_frame_consumer_->SetFrameOnUI(std::move(child_frame)));
   return true;
 }
 
+gfx::Rect BrowserViewRenderer::ComputeViewportRectForTilePriority() {
+  // If the WebView is on a layer, WebView does not know what transform is
+  // applied onto the layer so global visible rect does not make sense here.
+  // In this case, just use the surface rect for tiling.
+  // Leave viewport_rect_for_tile_priority empty if offscreen_pre_raster_ is on.
+  gfx::Rect viewport_rect_for_tile_priority;
+
+  if (!offscreen_pre_raster_ && !external_draw_constraints_.is_layer) {
+    viewport_rect_for_tile_priority = last_on_draw_global_visible_rect_;
+  }
+  return viewport_rect_for_tile_priority;
+}
+
 void BrowserViewRenderer::OnParentDrawConstraintsUpdated(
     CompositorFrameConsumer* compositor_frame_consumer) {
   DCHECK(compositor_frame_consumer);
@@ -286,24 +297,31 @@ void BrowserViewRenderer::RemoveCompositorFrameConsumer(
   // At this point the compositor frame consumer has to hand back all resources
   // to the child compositor.
   compositor_frame_consumer->DeleteHardwareRendererOnUI();
-  ReturnUnusedResource(compositor_frame_consumer->PassUncommittedFrameOnUI());
+  ReturnUncommittedFrames(
+      compositor_frame_consumer->PassUncommittedFrameOnUI());
   ReturnResourceFromParent(compositor_frame_consumer);
   compositor_frame_consumer->SetCompositorFrameProducer(nullptr);
 }
 
+void BrowserViewRenderer::ReturnUncommittedFrames(
+    ChildFrameQueue child_frames) {
+  for (auto& child_frame : child_frames)
+    ReturnUnusedResource(std::move(child_frame));
+}
+
 void BrowserViewRenderer::ReturnUnusedResource(
     std::unique_ptr<ChildFrame> child_frame) {
   if (!child_frame.get() || !child_frame->frame.get())
     return;
 
-  cc::CompositorFrameAck frame_ack;
-  cc::TransferableResource::ReturnResources(
-      child_frame->frame->delegated_frame_data->resource_list,
-      &frame_ack.resources);
+  cc::ReturnedResourceArray resources;
+  cc::TransferableResource::ReturnResources(child_frame->frame->resource_list,
+                                            &resources);
   content::SynchronousCompositor* compositor =
       FindCompositor(child_frame->compositor_id);
-  if (compositor && !frame_ack.resources.empty())
-    compositor->ReturnResources(child_frame->output_surface_id, frame_ack);
+  if (compositor && !resources.empty())
+    compositor->ReturnResources(child_frame->compositor_frame_sink_id,
+                                resources);
 }
 
 void BrowserViewRenderer::ReturnResourceFromParent(
@@ -313,11 +331,12 @@ void BrowserViewRenderer::ReturnResourceFromParent(
   for (auto& pair : returned_resource_map) {
     CompositorID compositor_id = pair.first;
     content::SynchronousCompositor* compositor = FindCompositor(compositor_id);
-    cc::CompositorFrameAck frame_ack;
-    frame_ack.resources.swap(pair.second.resources);
+    cc::ReturnedResourceArray resources;
+    resources.swap(pair.second.resources);
 
-    if (compositor && !frame_ack.resources.empty()) {
-      compositor->ReturnResources(pair.second.output_surface_id, frame_ack);
+    if (compositor && !resources.empty()) {
+      compositor->ReturnResources(pair.second.compositor_frame_sink_id,
+                                  resources);
     }
   }
 }
@@ -343,7 +362,10 @@ sk_sp<SkPicture> BrowserViewRenderer::CapturePicture(int width,
     {
       // Reset scroll back to the origin, will go back to the old
       // value when scroll_reset is out of scope.
-      compositor_->DidChangeRootLayerScrollOffset(gfx::ScrollOffset());
+      base::AutoReset<gfx::Vector2dF> scroll_reset(&scroll_offset_dip_,
+                                                   gfx::Vector2dF());
+      compositor_->DidChangeRootLayerScrollOffset(
+          gfx::ScrollOffset(scroll_offset_dip_));
       CompositeSW(rec_canvas);
     }
     compositor_->DidChangeRootLayerScrollOffset(
@@ -423,6 +445,8 @@ void BrowserViewRenderer::OnAttachedToWindow(int width, int height) {
                "height",
                height);
   attached_to_window_ = true;
+  was_attached_ = true;
+
   size_.SetSize(width, height);
   if (offscreen_pre_raster_)
     UpdateMemoryPolicy();
@@ -449,8 +473,9 @@ void BrowserViewRenderer::OnComputeScroll(base::TimeTicks animation_time) {
 }
 
 void BrowserViewRenderer::ReleaseHardware() {
-  for (auto compositor_frame_consumer : compositor_frame_consumers_) {
-    ReturnUnusedResource(compositor_frame_consumer->PassUncommittedFrameOnUI());
+  for (auto* compositor_frame_consumer : compositor_frame_consumers_) {
+    ReturnUncommittedFrames(
+        compositor_frame_consumer->PassUncommittedFrameOnUI());
     ReturnResourceFromParent(compositor_frame_consumer);
     DCHECK(compositor_frame_consumer->ReturnedResourcesEmptyOnUI());
   }
@@ -464,7 +489,13 @@ bool BrowserViewRenderer::IsVisible() const {
 }
 
 bool BrowserViewRenderer::IsClientVisible() const {
-  return !is_paused_ && (!attached_to_window_ || window_visible_);
+  // When WebView is not paused, we declare it visible even before it is
+  // attached to window to allow for background operations. If it ever gets
+  // attached though, the WebView is visible as long as it is attached
+  // to a window and the window is visible.
+  return is_paused_
+             ? false
+             : !was_attached_ || (attached_to_window_ && window_visible_);
 }
 
 gfx::Rect BrowserViewRenderer::GetScreenRect() const {
@@ -707,6 +738,10 @@ void BrowserViewRenderer::DidOverscroll(
   client_->DidOverscroll(rounded_overscroll_delta, fling_velocity_pixels);
 }
 
+ui::TouchHandleDrawable* BrowserViewRenderer::CreateDrawable() {
+  return client_->CreateDrawable();
+}
+
 void BrowserViewRenderer::PostInvalidate(
     content::SynchronousCompositor* compositor) {
   TRACE_EVENT_INSTANT0("android_webview", "BrowserViewRenderer::PostInvalidate",
