@@ -8,7 +8,6 @@
 #include "base/memory/ref_counted.h"
 
 namespace content {
-class ExternalVideoSurfaceContainer;
 class WebContents;
 class WebContentsViewDelegate;
 }  // namespace content
@@ -16,7 +15,6 @@ class WebContentsViewDelegate;
 namespace android_webview {
 
 class AwBrowserContext;
-class AwMessagePortService;
 class AwQuotaManagerBridge;
 class AwWebPreferencesPopulater;
 class AwLocaleManager;
@@ -31,12 +29,7 @@ class JniDependencyFactory {
   virtual content::WebContentsViewDelegate* CreateViewDelegate(
       content::WebContents* web_contents) = 0;
   virtual AwWebPreferencesPopulater* CreateWebPreferencesPopulater() = 0;
-  virtual AwMessagePortService* CreateAwMessagePortService() = 0;
   virtual AwLocaleManager* CreateAwLocaleManager() = 0;
-#if defined(VIDEO_HOLE)
-  virtual content::ExternalVideoSurfaceContainer*
-      CreateExternalVideoSurfaceContainer(content::WebContents* contents) = 0;
-#endif
 };
 
 }  // namespace android_webview
