@@ -7,10 +7,6 @@
 
 #include "base/macros.h"
 
-namespace gl {
-class GLContext;
-}
-
 namespace android_webview {
 
 namespace internal {
@@ -44,6 +40,8 @@ class ScopedAppGLStateRestore {
     MODE_RESOURCE_MANAGEMENT,
   };
 
+  static ScopedAppGLStateRestore* Current();
+
   explicit ScopedAppGLStateRestore(CallMode mode);
   ~ScopedAppGLStateRestore();
 
