@@ -82,6 +82,9 @@ class WebViewDelegateFactory {
 
         /** @see android.webkit.WebViewDelegate#addWebViewAssetPath */
         void addWebViewAssetPath(Context context);
+
+        /** @see android.webkit.WebViewDelegate#isMultiProcessEnabled */
+        boolean isMultiProcessEnabled();
     }
 
     /**
@@ -110,7 +113,7 @@ class WebViewDelegateFactory {
      * A {@link WebViewDelegate com.android.webview.chromium.WebViewDelegate} that proxies requests
      * to a {@link android.webkit.WebViewDelegate android.webkit.WebViewDelegate}.
      */
-    private static class ProxyDelegate implements WebViewDelegate {
+    static class ProxyDelegate implements WebViewDelegate {
         android.webkit.WebViewDelegate mDelegate;
 
         ProxyDelegate(android.webkit.WebViewDelegate delegate) {
@@ -152,7 +155,7 @@ class WebViewDelegateFactory {
         @Override
         public void callDrawGlFunction(
                 Canvas canvas, long nativeDrawGLFunctor, Runnable releasedRunnable) {
-            throw new RuntimeException("Call not supported");
+            mDelegate.callDrawGlFunction(canvas, nativeDrawGLFunctor, releasedRunnable);
         }
 
         @Override
@@ -198,6 +201,11 @@ class WebViewDelegateFactory {
                 }
             });
         }
+
+        @Override
+        public boolean isMultiProcessEnabled() {
+            throw new UnsupportedOperationException();
+        }
     }
 
     /**
@@ -383,5 +391,10 @@ class WebViewDelegateFactory {
                 throw new RuntimeException("Invalid reflection", e);
             }
         }
+
+        @Override
+        public boolean isMultiProcessEnabled() {
+            throw new UnsupportedOperationException();
+        }
     }
 }
