@@ -4,7 +4,7 @@
 
 package org.chromium.android_webview.test;
 
-import android.test.suitebuilder.annotation.SmallTest;
+import android.support.test.filters.SmallTest;
 import android.view.View;
 import android.view.ViewGroup.LayoutParams;
 import android.widget.LinearLayout;
@@ -14,9 +14,9 @@ import org.chromium.android_webview.AwContentsClient;
 import org.chromium.android_webview.AwLayoutSizer;
 import org.chromium.android_webview.test.util.CommonResources;
 import org.chromium.android_webview.test.util.GraphicsTestUtils;
+import org.chromium.base.test.util.CallbackHelper;
 import org.chromium.base.test.util.Feature;
-import org.chromium.content.browser.test.util.CallbackHelper;
-import org.chromium.ui.gfx.DeviceDisplayInfo;
+import org.chromium.ui.display.DisplayAndroid;
 
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -85,7 +85,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
     private AwTestContainerView createCustomTestContainerViewOnMainSync(
             final AwContentsClient awContentsClient, final int visibility) throws Exception {
         final AtomicReference<AwTestContainerView> testContainerView =
-                new AtomicReference<AwTestContainerView>();
+                new AtomicReference<>();
         getInstrumentation().runOnMainSync(new Runnable() {
             @Override
             public void run() {
@@ -100,7 +100,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
     private AwTestContainerView createDetachedTestContainerViewOnMainSync(
             final AwContentsClient awContentsClient) {
         final AtomicReference<AwTestContainerView> testContainerView =
-                new AtomicReference<AwTestContainerView>();
+                new AtomicReference<>();
         getInstrumentation().runOnMainSync(new Runnable() {
             @Override
             public void run() {
@@ -121,7 +121,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
     }
 
     private int getRootLayoutWidthOnMainThread() throws Exception {
-        final AtomicReference<Integer> width = new AtomicReference<Integer>();
+        final AtomicReference<Integer> width = new AtomicReference<>();
         getInstrumentation().runOnMainSync(new Runnable() {
             @Override
             public void run() {
@@ -150,7 +150,8 @@ public class AndroidViewIntegrationTest extends AwTestBase {
         assertZeroHeight(testContainerView);
 
         final int contentSizeChangeCallCount = mOnContentSizeChangedHelper.getCallCount();
-        loadUrlAsync(testContainerView.getAwContents(), CommonResources.ABOUT_HTML);
+        loadDataAsync(
+                testContainerView.getAwContents(), CommonResources.ABOUT_HTML, "text/html", false);
         mOnContentSizeChangedHelper.waitForCallback(contentSizeChangeCallCount);
         assertTrue(mOnContentSizeChangedHelper.getHeight() > 0);
     }
@@ -170,7 +171,8 @@ public class AndroidViewIntegrationTest extends AwTestBase {
         assertZeroHeight(testContainerView);
 
         final int contentSizeChangeCallCount = mOnContentSizeChangedHelper.getCallCount();
-        loadUrlAsync(testContainerView.getAwContents(), CommonResources.ABOUT_HTML);
+        loadDataAsync(
+                testContainerView.getAwContents(), CommonResources.ABOUT_HTML, "text/html", false);
         mOnContentSizeChangedHelper.waitForCallback(contentSizeChangeCallCount);
         assertTrue(mOnContentSizeChangedHelper.getHeight() > 0);
 
@@ -195,7 +197,8 @@ public class AndroidViewIntegrationTest extends AwTestBase {
 
         final int contentSizeChangeCallCount = mOnContentSizeChangedHelper.getCallCount();
         final int pageScaleChangeCallCount = mOnPageScaleChangedHelper.getCallCount();
-        loadUrlAsync(testContainerView.getAwContents(), CommonResources.ABOUT_HTML);
+        loadDataAsync(
+                testContainerView.getAwContents(), CommonResources.ABOUT_HTML, "text/html", false);
         mOnPageScaleChangedHelper.waitForCallback(pageScaleChangeCallCount);
         mOnContentSizeChangedHelper.waitForCallback(contentSizeChangeCallCount);
         assertTrue(mOnContentSizeChangedHelper.getHeight() > 0);
@@ -305,7 +308,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
         assertZeroHeight(testContainerView);
 
         final double deviceDIPScale =
-                DeviceDisplayInfo.create(testContainerView.getContext()).getDIPScale();
+                DisplayAndroid.getNonMultiDisplay(testContainerView.getContext()).getDipScale();
 
         final int contentWidthCss = 142;
         final int contentHeightCss = 180;
@@ -332,7 +335,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
         assertZeroHeight(testContainerView);
 
         final double deviceDIPScale =
-                DeviceDisplayInfo.create(testContainerView.getContext()).getDIPScale();
+                DisplayAndroid.getNonMultiDisplay(testContainerView.getContext()).getDipScale();
 
         final int contentWidthCss = 142;
         final int contentHeightCss = 180;
@@ -360,7 +363,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
         assertZeroHeight(testContainerView);
 
         final double deviceDIPScale =
-                DeviceDisplayInfo.create(testContainerView.getContext()).getDIPScale();
+                DisplayAndroid.getNonMultiDisplay(testContainerView.getContext()).getDipScale();
 
         final int contentWidthCss = 142;
         final int contentHeightCss = 180;
@@ -386,7 +389,7 @@ public class AndroidViewIntegrationTest extends AwTestBase {
         final AwContents awContents = testContainerView.getAwContents();
 
         final double deviceDIPScale =
-                DeviceDisplayInfo.create(testContainerView.getContext()).getDIPScale();
+                DisplayAndroid.getNonMultiDisplay(testContainerView.getContext()).getDipScale();
         final int physicalWidth = 600;
         final int spanWidth = 42;
         final int expectedWidthCss =
