@@ -11,16 +11,14 @@ import android.graphics.Canvas;
 import android.os.Handler;
 import android.os.Looper;
 import android.os.Message;
+import android.support.test.filters.LargeTest;
+import android.support.test.filters.SmallTest;
 import android.test.UiThreadTest;
-import android.test.suitebuilder.annotation.LargeTest;
-import android.test.suitebuilder.annotation.SmallTest;
 import android.util.Pair;
 import android.view.KeyEvent;
 import android.view.View;
 import android.webkit.JavascriptInterface;
 
-import org.apache.http.Header;
-import org.apache.http.HttpRequest;
 import org.chromium.android_webview.AwContents;
 import org.chromium.android_webview.AwSettings;
 import org.chromium.android_webview.AwSwitches;
@@ -28,13 +26,14 @@ import org.chromium.android_webview.test.TestAwContentsClient.OnDownloadStartHel
 import org.chromium.android_webview.test.util.CommonResources;
 import org.chromium.android_webview.test.util.JSUtils;
 import org.chromium.base.annotations.SuppressFBWarnings;
+import org.chromium.base.test.util.CallbackHelper;
 import org.chromium.base.test.util.CommandLineFlags;
 import org.chromium.base.test.util.Feature;
 import org.chromium.base.test.util.parameter.ParameterizedTest;
 import org.chromium.content.browser.BindingManager;
 import org.chromium.content.browser.ChildProcessConnection;
 import org.chromium.content.browser.ChildProcessLauncher;
-import org.chromium.content.browser.test.util.CallbackHelper;
+import org.chromium.net.test.EmbeddedTestServer;
 import org.chromium.net.test.util.TestWebServer;
 
 import java.io.InputStream;
@@ -68,8 +67,9 @@ public class AwContentsTest extends AwTestBase {
     public void testCreateLoadPageDestroy() throws Throwable {
         AwTestContainerView awTestContainerView =
                 createAwTestContainerViewOnMainSync(mContentsClient);
-        loadUrlSync(awTestContainerView.getAwContents(),
-                mContentsClient.getOnPageFinishedHelper(), CommonResources.ABOUT_HTML);
+        loadDataSync(awTestContainerView.getAwContents(), mContentsClient.getOnPageFinishedHelper(),
+                CommonResources.ABOUT_HTML, "text/html", false);
+
         destroyAwContentsOnMainSync(awTestContainerView.getAwContents());
         // It should be safe to call destroy multiple times.
         destroyAwContentsOnMainSync(awTestContainerView.getAwContents());
@@ -149,8 +149,8 @@ public class AwContentsTest extends AwTestBase {
         AwTestContainerView awTestContainerView =
                 createAwTestContainerViewOnMainSync(mContentsClient);
         AwSettings awSettings = getAwSettingsOnUiThread(awTestContainerView.getAwContents());
-        loadUrlSync(awTestContainerView.getAwContents(),
-                mContentsClient.getOnPageFinishedHelper(), CommonResources.ABOUT_HTML);
+        loadDataSync(awTestContainerView.getAwContents(), mContentsClient.getOnPageFinishedHelper(),
+                CommonResources.ABOUT_HTML, "text/html", false);
         destroyAwContentsOnMainSync(awTestContainerView.getAwContents());
 
         // AwSettings should still be usable even after native side is destroyed.
@@ -438,31 +438,35 @@ public class AwContentsTest extends AwTestBase {
                 createAwTestContainerViewOnMainSync(mContentsClient);
         final AwContents awContents = testContainer.getAwContents();
 
-        TestWebServer webServer = TestWebServer.start();
+        enableJavaScriptOnUiThread(awContents);
+
+        EmbeddedTestServer testServer =
+                EmbeddedTestServer.createAndStartServer(getInstrumentation().getContext());
+
         try {
-            final String pagePath = "/test_can_inject_headers.html";
-            final String pageUrl = webServer.setResponse(
-                    pagePath, "<html><body>foo</body></html>", null);
+            String url = testServer.getURL("/echoheader?X-foo");
 
             final Map<String, String> extraHeaders = new HashMap<String, String>();
-            extraHeaders.put("Referer", "foo");
             extraHeaders.put("X-foo", "bar");
-            loadUrlSync(awContents, mContentsClient.getOnPageFinishedHelper(),
-                    webServer.getResponseUrl(pagePath), extraHeaders);
+            loadUrlSync(awContents, mContentsClient.getOnPageFinishedHelper(), url, extraHeaders);
 
-            assertEquals(1, webServer.getRequestCount(pagePath));
+            String xfoo = maybeStripDoubleQuotes(JSUtils.executeJavaScriptAndWaitForResult(this,
+                    awContents, mContentsClient.getOnEvaluateJavaScriptResultHelper(),
+                    "document.body.textContent"));
+            assertEquals("bar", xfoo);
 
-            HttpRequest request = webServer.getLastRequest(pagePath);
-            assertNotNull(request);
+            url = testServer.getURL("/echoheader?Referer");
 
-            for (Map.Entry<String, String> value : extraHeaders.entrySet()) {
-                String header = value.getKey();
-                Header[] matchingHeaders = request.getHeaders(header);
-                assertEquals("header " + header + " not found", 1, matchingHeaders.length);
-                assertEquals(value.getValue(), matchingHeaders[0].getValue());
-            }
+            extraHeaders.clear();
+            extraHeaders.put("Referer", "http://www.example.com/");
+            loadUrlSync(awContents, mContentsClient.getOnPageFinishedHelper(), url, extraHeaders);
+
+            String referer = maybeStripDoubleQuotes(JSUtils.executeJavaScriptAndWaitForResult(this,
+                    awContents, mContentsClient.getOnEvaluateJavaScriptResultHelper(),
+                    "document.body.textContent"));
+            assertEquals("http://www.example.com/", referer);
         } finally {
-            webServer.shutdown();
+            testServer.stopAndDestroyServer();
         }
     }
 
