@@ -4,15 +4,15 @@
 
 package org.chromium.android_webview.test;
 
-import android.test.suitebuilder.annotation.MediumTest;
+import android.support.test.filters.MediumTest;
 
 import org.chromium.android_webview.AwContents;
 import org.chromium.android_webview.AwContentsClient;
 import org.chromium.android_webview.AwSettings;
 import org.chromium.base.annotations.SuppressFBWarnings;
+import org.chromium.base.test.util.CallbackHelper;
 import org.chromium.base.test.util.Feature;
-import org.chromium.content.browser.test.util.CallbackHelper;
-import org.chromium.ui.gfx.DeviceDisplayInfo;
+import org.chromium.ui.display.DisplayAndroid;
 
 import java.util.Locale;
 import java.util.concurrent.Callable;
@@ -41,14 +41,14 @@ public class AwLegacyQuirksTest extends AwTestBase {
 
         settings.setJavaScriptEnabled(true);
 
-        DeviceDisplayInfo deviceInfo =
-                DeviceDisplayInfo.create(getInstrumentation().getTargetContext());
+        DisplayAndroid displayAndroid = DisplayAndroid.getNonMultiDisplay(
+                getInstrumentation().getTargetContext());
         loadDataSync(awContents, onPageFinishedHelper, pageDeviceDpi, "text/html", false);
         int actualWidth = Integer.parseInt(getTitleOnUiThread(awContents));
-        assertEquals(deviceInfo.getDisplayWidth(), actualWidth, 10f);
+        assertEquals(displayAndroid.getDisplayWidth(), actualWidth, 10f);
 
-        float displayWidth = (deviceInfo.getDisplayWidth());
-        float deviceDpi = (float) (160f * deviceInfo.getDIPScale());
+        float displayWidth = (displayAndroid.getDisplayWidth());
+        float deviceDpi = 160f * displayAndroid.getDipScale();
 
         loadDataSync(awContents, onPageFinishedHelper, pageHighDpi, "text/html", false);
         actualWidth = Integer.parseInt(getTitleOnUiThread(awContents));
@@ -76,10 +76,11 @@ public class AwLegacyQuirksTest extends AwTestBase {
         settings.setJavaScriptEnabled(true);
         settings.setUseWideViewPort(true);
 
-        DeviceDisplayInfo deviceInfo =
-                DeviceDisplayInfo.create(getInstrumentation().getTargetContext());
+        DisplayAndroid displayAndroid = DisplayAndroid.getNonMultiDisplay(
+                getInstrumentation().getTargetContext());
         loadDataSync(awContents, onPageFinishedHelper, page, "text/html", false);
-        float displayWidth = (float) (deviceInfo.getDisplayWidth() / deviceInfo.getDIPScale());
+        float displayWidth =
+                displayAndroid.getDisplayWidth() / displayAndroid.getDipScale();
         int actualWidth = Integer.parseInt(getTitleOnUiThread(awContents));
         assertEquals(displayWidth, actualWidth, 10f);
         assertEquals(1.0f, getScaleOnUiThread(awContents));
@@ -102,10 +103,11 @@ public class AwLegacyQuirksTest extends AwTestBase {
 
         settings.setJavaScriptEnabled(true);
 
-        DeviceDisplayInfo deviceInfo =
-                DeviceDisplayInfo.create(getInstrumentation().getTargetContext());
+        DisplayAndroid displayAndroid = DisplayAndroid.getNonMultiDisplay(
+                getInstrumentation().getTargetContext());
         loadDataSync(awContents, onPageFinishedHelper, page, "text/html", false);
-        float displayWidth = (float) (deviceInfo.getDisplayWidth() / deviceInfo.getDIPScale());
+        float displayWidth =
+                displayAndroid.getDisplayWidth() / displayAndroid.getDipScale();
         int actualWidth = Integer.parseInt(getTitleOnUiThread(awContents));
         assertEquals(displayWidth, actualWidth, 10f);
         assertEquals(1.0f, getScaleOnUiThread(awContents));
@@ -132,16 +134,16 @@ public class AwLegacyQuirksTest extends AwTestBase {
 
         loadUrlSync(awContents, onPageFinishedHelper, "about:blank");
 
-        DeviceDisplayInfo deviceInfo =
-                DeviceDisplayInfo.create(getInstrumentation().getTargetContext());
-        float dipScale = (float) deviceInfo.getDIPScale();
-        float physicalDisplayWidth = deviceInfo.getPhysicalDisplayWidth() != 0
-                                     ? deviceInfo.getPhysicalDisplayWidth()
-                                     : deviceInfo.getDisplayWidth();
+        DisplayAndroid displayAndroid = DisplayAndroid.getNonMultiDisplay(
+                getInstrumentation().getTargetContext());
+        float dipScale = displayAndroid.getDipScale();
+        float physicalDisplayWidth = displayAndroid.getPhysicalDisplayWidth() != 0
+                                     ? displayAndroid.getPhysicalDisplayWidth()
+                                     : displayAndroid.getDisplayWidth();
         float cssDisplayWidth = physicalDisplayWidth / dipScale;
-        float physicalDisplayHeight = deviceInfo.getPhysicalDisplayHeight() != 0
-                                      ? deviceInfo.getPhysicalDisplayHeight()
-                                      : deviceInfo.getDisplayHeight();
+        float physicalDisplayHeight = displayAndroid.getPhysicalDisplayHeight() != 0
+                                      ? displayAndroid.getPhysicalDisplayHeight()
+                                      : displayAndroid.getDisplayHeight();
         float cssDisplayHeight = physicalDisplayHeight / dipScale;
 
         float screenWidth = Integer.parseInt(
