@@ -13,18 +13,15 @@ class AwContentsBackgroundThreadClient {
  public:
   static base::android::ScopedJavaLocalRef<jobject> shouldInterceptRequest(
       JNIEnv* env,
-      jobject obj,
-      jstring url,
+      const base::android::JavaRef<jobject>& obj,
+      const base::android::JavaRef<jstring>& url,
       jboolean isMainFrame,
       jboolean hasUserGesture,
-      jstring method,
-      jobjectArray requestHeaderNames,
-      jobjectArray requestHeaderValues);
+      const base::android::JavaRef<jstring>& method,
+      const base::android::JavaRef<jobjectArray>& requestHeaderNames,
+      const base::android::JavaRef<jobjectArray>& requestHeaderValues);
 };
 
-// JNI registration method.
-bool RegisterAwContentsBackgroundThreadClient(JNIEnv* env);
-
 }
 
 #endif  // ANDROID_WEBVIEW_NATIVE_AW_CONTENTS_BACKGROUND_THREAD_CLIENT_H_
