@@ -4,11 +4,9 @@
 
 package org.chromium.android_webview.shell;
 
-import org.chromium.android_webview.R;
 import org.chromium.base.CommandLine;
 import org.chromium.base.annotations.SuppressFBWarnings;
 import org.chromium.content.app.ContentApplication;
-import org.chromium.ui.base.ResourceBundle;
 
 /**
  * The android_webview shell Application subclass.
@@ -18,16 +16,6 @@ public class AwShellApplication extends ContentApplication {
         super(false /* mShouldInitializeApplicationStatusTracking */);
     }
 
-    @Override
-    public void onCreate() {
-        super.onCreate();
-    }
-
-    @Override
-    protected void initializeLibraryDependencies() {
-        ResourceBundle.initializeLocalePaks(this, R.array.locale_paks);
-    }
-
     @SuppressFBWarnings("DMI_HARDCODED_ABSOLUTE_FILENAME")
     @Override
     public void initCommandLine() {
