@@ -37,8 +37,8 @@ public class AwTestContainerView extends FrameLayout {
     private AwContents mAwContents;
     private AwContents.InternalAccessDelegate mInternalAccessDelegate;
 
-    private HardwareView mHardwareView = null;
-    private boolean mAttachedContents = false;
+    private HardwareView mHardwareView;
+    private boolean mAttachedContents;
 
     private class HardwareView extends GLSurfaceView {
         private static final int MODE_DRAW = 0;
@@ -50,23 +50,24 @@ public class AwTestContainerView extends FrameLayout {
         // and drawGL on the rendering thread. The variables following
         // are protected by it.
         private final Object mSyncLock = new Object();
-        private boolean mFunctorAttached = false;
-        private boolean mNeedsProcessGL = false;
-        private boolean mNeedsDrawGL = false;
-        private boolean mWaitForCompletion = false;
-        private int mLastScrollX = 0;
-        private int mLastScrollY = 0;
+        private boolean mFunctorAttached;
+        private boolean mNeedsProcessGL;
+        private boolean mNeedsDrawGL;
+        private boolean mWaitForCompletion;
+        private int mLastScrollX;
+        private int mLastScrollY;
 
         // Only used by drawGL on render thread to store the value of scroll offsets at most recent
         // sync for subsequent draws.
-        private int mCommittedScrollX = 0;
-        private int mCommittedScrollY = 0;
+        private int mCommittedScrollX;
+        private int mCommittedScrollY;
 
-        private boolean mHaveSurface = false;
-        private Runnable mReadyToRenderCallback = null;
+        private boolean mHaveSurface;
+        private Runnable mReadyToRenderCallback;
+        private Runnable mReadyToDetachCallback;
 
-        private long mDrawGL = 0;
-        private long mViewContext = 0;
+        private long mDrawGL;
+        private long mViewContext;
 
         public HardwareView(Context context) {
             super(context);
@@ -74,8 +75,8 @@ public class AwTestContainerView extends FrameLayout {
             getHolder().setFormat(PixelFormat.OPAQUE);
             setPreserveEGLContextOnPause(true);
             setRenderer(new Renderer() {
-                private int mWidth = 0;
-                private int mHeight = 0;
+                private int mWidth;
+                private int mHeight;
 
                 @Override
                 public void onDrawFrame(GL10 gl) {
@@ -112,11 +113,14 @@ public class AwTestContainerView extends FrameLayout {
             mReadyToRenderCallback = runner;
         }
 
+        public void setReadyToDetachCallback(Runnable runner) {
+            mReadyToDetachCallback = runner;
+        }
+
         @Override
         public void surfaceCreated(SurfaceHolder holder) {
-            boolean didHaveSurface = mHaveSurface;
             mHaveSurface = true;
-            if (!didHaveSurface && mReadyToRenderCallback != null) {
+            if (mReadyToRenderCallback != null) {
                 mReadyToRenderCallback.run();
                 mReadyToRenderCallback = null;
             }
@@ -126,6 +130,10 @@ public class AwTestContainerView extends FrameLayout {
         @Override
         public void surfaceDestroyed(SurfaceHolder holder) {
             mHaveSurface = false;
+            if (mReadyToDetachCallback != null) {
+                mReadyToDetachCallback.run();
+                mReadyToDetachCallback = null;
+            }
             super.surfaceDestroyed(holder);
         }
 
@@ -224,7 +232,7 @@ public class AwTestContainerView extends FrameLayout {
         }
     }
 
-    private static boolean sCreatedOnce = false;
+    private static boolean sCreatedOnce;
     private HardwareView createHardwareViewOnlyOnce(Context context) {
         if (sCreatedOnce) return null;
         sCreatedOnce = true;
@@ -287,19 +295,37 @@ public class AwTestContainerView extends FrameLayout {
         mAwContents.onConfigurationChanged(newConfig);
     }
 
+    private void attachedContentsInternal() {
+        assert !mAttachedContents;
+        mAwContents.onAttachedToWindow();
+        mAttachedContents = true;
+    }
+
+    private void detachedContentsInternal() {
+        assert mAttachedContents;
+        mAwContents.onDetachedFromWindow();
+        mAttachedContents = false;
+    }
+
     @Override
     public void onAttachedToWindow() {
         super.onAttachedToWindow();
         if (mHardwareView == null || mHardwareView.isReadyToRender()) {
-            mAwContents.onAttachedToWindow();
-            mAttachedContents = true;
+            attachedContentsInternal();
         } else {
             mHardwareView.setReadyToRenderCallback(new Runnable() {
                 @Override
                 public void run() {
-                    assert !mAttachedContents;
-                    mAwContents.onAttachedToWindow();
-                    mAttachedContents = true;
+                    attachedContentsInternal();
+                }
+            });
+        }
+
+        if (mHardwareView != null) {
+            mHardwareView.setReadyToDetachCallback(new Runnable() {
+                @Override
+                public void run() {
+                    detachedContentsInternal();
                 }
             });
         }
@@ -308,11 +334,14 @@ public class AwTestContainerView extends FrameLayout {
     @Override
     public void onDetachedFromWindow() {
         super.onDetachedFromWindow();
-        mAwContents.onDetachedFromWindow();
-        if (mHardwareView != null) {
-            mHardwareView.setReadyToRenderCallback(null);
+        if (mHardwareView == null || mHardwareView.isReadyToRender()) {
+            detachedContentsInternal();
+
+            if (mHardwareView != null) {
+                mHardwareView.setReadyToRenderCallback(null);
+                mHardwareView.setReadyToDetachCallback(null);
+            }
         }
-        mAttachedContents = false;
     }
 
     @Override
