@@ -2,42 +2,43 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "ash/accelerators/accelerator_controller.h"
+#include "ash/common/accelerators/accelerator_controller.h"
 
-#include "ash/accelerators/accelerator_table.h"
-#include "ash/aura/wm_window_aura.h"
+#include "ash/common/accelerators/accelerator_table.h"
 #include "ash/common/accessibility_delegate.h"
 #include "ash/common/accessibility_types.h"
 #include "ash/common/ash_switches.h"
-#include "ash/common/shell_window_ids.h"
+#include "ash/common/ime_control_delegate.h"
+#include "ash/common/session/session_state_delegate.h"
+#include "ash/common/system/brightness_control_delegate.h"
+#include "ash/common/system/keyboard_brightness_control_delegate.h"
 #include "ash/common/system/tray/system_tray_delegate.h"
-#include "ash/common/system/volume_control_delegate.h"
+#include "ash/common/test/test_shelf_delegate.h"
 #include "ash/common/wm/panels/panel_layout_manager.h"
 #include "ash/common/wm/window_positioning_utils.h"
 #include "ash/common/wm/window_state.h"
 #include "ash/common/wm/wm_event.h"
 #include "ash/common/wm_shell.h"
-#include "ash/display/display_manager.h"
-#include "ash/ime_control_delegate.h"
-#include "ash/screen_util.h"
+#include "ash/common/wm_window.h"
+#include "ash/public/cpp/shell_window_ids.h"
 #include "ash/shell.h"
-#include "ash/system/brightness_control_delegate.h"
-#include "ash/system/keyboard_brightness/keyboard_brightness_control_delegate.h"
 #include "ash/test/ash_test_base.h"
-#include "ash/test/display_manager_test_api.h"
+#include "ash/test/lock_state_controller_test_api.h"
 #include "ash/test/test_screenshot_delegate.h"
 #include "ash/test/test_session_state_animator.h"
-#include "ash/test/test_shelf_delegate.h"
-#include "ash/test/test_shell_delegate.h"
-#include "ash/test/test_volume_control_delegate.h"
 #include "ash/wm/lock_state_controller.h"
 #include "ash/wm/window_state_aura.h"
 #include "ash/wm/window_util.h"
 #include "base/command_line.h"
+#include "base/test/user_action_tester.cc"
+#include "services/ui/public/interfaces/window_manager_constants.mojom.h"
+#include "ui/app_list/presenter/app_list.h"
+#include "ui/app_list/presenter/test/test_app_list_presenter.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/test/test_window_delegate.h"
 #include "ui/aura/test/test_windows.h"
 #include "ui/aura/window.h"
+#include "ui/display/manager/display_manager.h"
 #include "ui/display/screen.h"
 #include "ui/events/event.h"
 #include "ui/events/event_processor.h"
@@ -208,10 +209,6 @@ class AcceleratorControllerTest : public test::AshTestBase {
   ~AcceleratorControllerTest() override {}
 
  protected:
-  void EnableInternalDisplay() {
-    test::DisplayManagerTestApi().SetFirstDisplayAsInternalDisplay();
-  }
-
   static AcceleratorController* GetController();
 
   static bool ProcessInController(const ui::Accelerator& accelerator) {
@@ -257,24 +254,30 @@ class AcceleratorControllerTest : public test::AshTestBase {
   aura::Window* CreatePanel() {
     aura::Window* window = CreateTestWindowInShellWithDelegateAndType(
         NULL, ui::wm::WINDOW_TYPE_PANEL, 0, gfx::Rect(5, 5, 20, 20));
-    test::TestShelfDelegate* shelf_delegate =
-        test::TestShelfDelegate::instance();
-    shelf_delegate->AddShelfItem(window);
-    PanelLayoutManager* manager =
-        PanelLayoutManager::Get(WmWindowAura::Get(window));
-    manager->Relayout();
+    WmWindow* wm_window = WmWindow::Get(window);
+    test::TestShelfDelegate::instance()->AddShelfItem(wm_window);
+    PanelLayoutManager::Get(wm_window)->Relayout();
     return window;
   }
 
+  void SetBrightnessControlDelegate(
+      std::unique_ptr<BrightnessControlDelegate> delegate) {
+    WmShell::Get()->brightness_control_delegate_ = std::move(delegate);
+  }
+
+  void SetKeyboardBrightnessControlDelegate(
+      std::unique_ptr<KeyboardBrightnessControlDelegate> delegate) {
+    WmShell::Get()->keyboard_brightness_control_delegate_ = std::move(delegate);
+  }
+
  private:
   DISALLOW_COPY_AND_ASSIGN(AcceleratorControllerTest);
 };
 
 AcceleratorController* AcceleratorControllerTest::GetController() {
-  return Shell::GetInstance()->accelerator_controller();
+  return WmShell::Get()->accelerator_controller();
 }
 
-#if !defined(OS_WIN)
 // Double press of exit shortcut => exiting
 TEST_F(AcceleratorControllerTest, ExitWarningHandlerTestDoublePress) {
   ui::Accelerator press(ui::VKEY_Q, ui::EF_SHIFT_DOWN | ui::EF_CONTROL_DOWN);
@@ -330,7 +333,6 @@ TEST_F(AcceleratorControllerTest, LingeringExitWarningBubble) {
 
   // Exit ash and there should be no crash
 }
-#endif  // !defined(OS_WIN)
 
 TEST_F(AcceleratorControllerTest, Register) {
   const ui::Accelerator accelerator_a(ui::VKEY_A, ui::EF_NONE);
@@ -432,13 +434,13 @@ TEST_F(AcceleratorControllerTest, WindowSnap) {
   {
     GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
     gfx::Rect expected_bounds = wm::GetDefaultLeftSnappedWindowBoundsInParent(
-        WmWindowAura::Get(window.get()));
+        WmWindow::Get(window.get()));
     EXPECT_EQ(expected_bounds.ToString(), window->bounds().ToString());
   }
   {
     GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_RIGHT);
     gfx::Rect expected_bounds = wm::GetDefaultRightSnappedWindowBoundsInParent(
-        WmWindowAura::Get(window.get()));
+        WmWindow::Get(window.get()));
     EXPECT_EQ(expected_bounds.ToString(), window->bounds().ToString());
   }
   {
@@ -476,7 +478,64 @@ TEST_F(AcceleratorControllerTest, WindowSnap) {
   }
 }
 
-TEST_F(AcceleratorControllerTest, WindowSnapLeftDockLeftRestore) {
+// Tests that when window docking is disabled, only snapping windows works.
+TEST_F(AcceleratorControllerTest, WindowSnapWithoutDocking) {
+  ASSERT_FALSE(ash::switches::DockedWindowsEnabled());
+  std::unique_ptr<aura::Window> window(
+      CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
+
+  wm::WindowState* window_state = wm::GetWindowState(window.get());
+  window_state->Activate();
+
+  // Snap right.
+  GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_RIGHT);
+  gfx::Rect normal_bounds = window_state->GetRestoreBoundsInParent();
+  gfx::Rect expected_bounds = wm::GetDefaultRightSnappedWindowBoundsInParent(
+      WmWindow::Get(window.get()));
+  EXPECT_EQ(expected_bounds.ToString(), window->bounds().ToString());
+  EXPECT_TRUE(window_state->IsSnapped());
+  // Snap right again ->> becomes normal.
+  GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_RIGHT);
+  EXPECT_TRUE(window_state->IsNormalStateType());
+  EXPECT_FALSE(window_state->IsDocked());
+  EXPECT_EQ(normal_bounds.ToString(), window->bounds().ToString());
+  // Snap right.
+  GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_RIGHT);
+  EXPECT_TRUE(window_state->IsSnapped());
+  EXPECT_FALSE(window_state->IsDocked());
+  // Snap left.
+  GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
+  EXPECT_TRUE(window_state->IsSnapped());
+  EXPECT_FALSE(window_state->IsDocked());
+  expected_bounds = wm::GetDefaultLeftSnappedWindowBoundsInParent(
+      WmWindow::Get(window.get()));
+  EXPECT_EQ(expected_bounds.ToString(), window->bounds().ToString());
+  // Snap left again ->> becomes normal.
+  GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
+  EXPECT_TRUE(window_state->IsNormalStateType());
+  EXPECT_FALSE(window_state->IsDocked());
+  EXPECT_EQ(normal_bounds.ToString(), window->bounds().ToString());
+}
+
+// Test class used for testing docked windows.
+class EnabledDockedWindowsAcceleratorControllerTest
+    : public AcceleratorControllerTest {
+ public:
+  EnabledDockedWindowsAcceleratorControllerTest() = default;
+  ~EnabledDockedWindowsAcceleratorControllerTest() override = default;
+
+  void SetUp() override {
+    base::CommandLine::ForCurrentProcess()->AppendSwitch(
+        ash::switches::kAshEnableDockedWindows);
+    AcceleratorControllerTest::SetUp();
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(EnabledDockedWindowsAcceleratorControllerTest);
+};
+
+TEST_F(EnabledDockedWindowsAcceleratorControllerTest,
+       WindowSnapLeftDockLeftRestore) {
   std::unique_ptr<aura::Window> window0(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
   std::unique_ptr<aura::Window> window1(
@@ -487,7 +546,7 @@ TEST_F(AcceleratorControllerTest, WindowSnapLeftDockLeftRestore) {
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
   gfx::Rect normal_bounds = window1_state->GetRestoreBoundsInParent();
   gfx::Rect expected_bounds = wm::GetDefaultLeftSnappedWindowBoundsInParent(
-      WmWindowAura::Get(window1.get()));
+      WmWindow::Get(window1.get()));
   EXPECT_EQ(expected_bounds.ToString(), window1->bounds().ToString());
   EXPECT_TRUE(window1_state->IsSnapped());
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
@@ -498,7 +557,8 @@ TEST_F(AcceleratorControllerTest, WindowSnapLeftDockLeftRestore) {
   EXPECT_EQ(normal_bounds.ToString(), window1->bounds().ToString());
 }
 
-TEST_F(AcceleratorControllerTest, WindowSnapRightDockRightRestore) {
+TEST_F(EnabledDockedWindowsAcceleratorControllerTest,
+       WindowSnapRightDockRightRestore) {
   std::unique_ptr<aura::Window> window0(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
   std::unique_ptr<aura::Window> window1(
@@ -510,7 +570,7 @@ TEST_F(AcceleratorControllerTest, WindowSnapRightDockRightRestore) {
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_RIGHT);
   gfx::Rect normal_bounds = window1_state->GetRestoreBoundsInParent();
   gfx::Rect expected_bounds = wm::GetDefaultRightSnappedWindowBoundsInParent(
-      WmWindowAura::Get(window1.get()));
+      WmWindow::Get(window1.get()));
   EXPECT_EQ(expected_bounds.ToString(), window1->bounds().ToString());
   EXPECT_TRUE(window1_state->IsSnapped());
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_RIGHT);
@@ -521,7 +581,8 @@ TEST_F(AcceleratorControllerTest, WindowSnapRightDockRightRestore) {
   EXPECT_EQ(normal_bounds.ToString(), window1->bounds().ToString());
 }
 
-TEST_F(AcceleratorControllerTest, WindowSnapLeftDockLeftSnapRight) {
+TEST_F(EnabledDockedWindowsAcceleratorControllerTest,
+       WindowSnapLeftDockLeftSnapRight) {
   std::unique_ptr<aura::Window> window0(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
   std::unique_ptr<aura::Window> window1(
@@ -532,9 +593,9 @@ TEST_F(AcceleratorControllerTest, WindowSnapLeftDockLeftSnapRight) {
 
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
   gfx::Rect expected_bounds = wm::GetDefaultLeftSnappedWindowBoundsInParent(
-      WmWindowAura::Get(window1.get()));
+      WmWindow::Get(window1.get()));
   gfx::Rect expected_bounds2 = wm::GetDefaultRightSnappedWindowBoundsInParent(
-      WmWindowAura::Get(window1.get()));
+      WmWindow::Get(window1.get()));
   EXPECT_EQ(expected_bounds.ToString(), window1->bounds().ToString());
   EXPECT_TRUE(window1_state->IsSnapped());
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
@@ -546,7 +607,8 @@ TEST_F(AcceleratorControllerTest, WindowSnapLeftDockLeftSnapRight) {
   EXPECT_EQ(expected_bounds2.ToString(), window1->bounds().ToString());
 }
 
-TEST_F(AcceleratorControllerTest, WindowDockLeftMinimizeWindowWithRestore) {
+TEST_F(EnabledDockedWindowsAcceleratorControllerTest,
+       WindowDockLeftMinimizeWindowWithRestore) {
   std::unique_ptr<aura::Window> window0(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
   std::unique_ptr<aura::Window> window1(
@@ -593,7 +655,8 @@ TEST_F(AcceleratorControllerTest, WindowDockLeftMinimizeWindowWithRestore) {
   EXPECT_EQ(window3_docked_bounds.ToString(), window3->bounds().ToString());
 }
 
-TEST_F(AcceleratorControllerTest, WindowPanelDockLeftDockRightRestore) {
+TEST_F(EnabledDockedWindowsAcceleratorControllerTest,
+       WindowPanelDockLeftDockRightRestore) {
   std::unique_ptr<aura::Window> window0(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
 
@@ -604,7 +667,7 @@ TEST_F(AcceleratorControllerTest, WindowPanelDockLeftDockRightRestore) {
   gfx::Rect window_restore_bounds2 = window->bounds();
   GetController()->PerformActionIfEnabled(WINDOW_CYCLE_SNAP_DOCK_LEFT);
   gfx::Rect expected_bounds = wm::GetDefaultLeftSnappedWindowBoundsInParent(
-      WmWindowAura::Get(window.get()));
+      WmWindow::Get(window.get()));
   gfx::Rect window_restore_bounds = window_state->GetRestoreBoundsInScreen();
   EXPECT_NE(expected_bounds.ToString(), window->bounds().ToString());
   EXPECT_FALSE(window_state->IsSnapped());
@@ -620,7 +683,7 @@ TEST_F(AcceleratorControllerTest, WindowPanelDockLeftDockRightRestore) {
   EXPECT_EQ(window_restore_bounds.ToString(), window->bounds().ToString());
 }
 
-TEST_F(AcceleratorControllerTest, CenterWindowAccelerator) {
+TEST_F(EnabledDockedWindowsAcceleratorControllerTest, CenterWindowAccelerator) {
   std::unique_ptr<aura::Window> window(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
   wm::WindowState* window_state = wm::GetWindowState(window.get());
@@ -723,7 +786,8 @@ TEST_F(AcceleratorControllerTest, DontRepeatToggleFullscreen) {
   widget->Init(params);
   widget->Show();
   widget->Activate();
-  widget->GetNativeView()->SetProperty(aura::client::kCanMaximizeKey, true);
+  widget->GetNativeView()->SetProperty(aura::client::kResizeBehaviorKey,
+                                       ui::mojom::kResizeBehaviorCanMaximize);
 
   ui::test::EventGenerator& generator = GetEventGenerator();
   wm::WindowState* window_state = wm::GetWindowState(widget->GetNativeView());
@@ -742,15 +806,8 @@ TEST_F(AcceleratorControllerTest, DontRepeatToggleFullscreen) {
 }
 
 // TODO(oshima): Fix this test to use EventGenerator.
-#if defined(OS_WIN)
-// crbug.com/317592
-#define MAYBE_ProcessOnce DISABLED_ProcessOnce
-#else
-#define MAYBE_ProcessOnce ProcessOnce
-#endif
-
-#if defined(OS_WIN) || defined(USE_X11)
-TEST_F(AcceleratorControllerTest, MAYBE_ProcessOnce) {
+#if defined(USE_X11)
+TEST_F(AcceleratorControllerTest, ProcessOnce) {
   // The IME event filter interferes with the basic key event propagation we
   // attempt to do here, so we disable it.
   DisableIME();
@@ -761,22 +818,7 @@ TEST_F(AcceleratorControllerTest, MAYBE_ProcessOnce) {
   // The accelerator is processed only once.
   ui::EventProcessor* dispatcher =
       Shell::GetPrimaryRootWindow()->GetHost()->event_processor();
-#if defined(OS_WIN)
-  MSG msg1 = {NULL, WM_KEYDOWN, ui::VKEY_A, 0};
-  ui::KeyEvent key_event1(msg1);
-  ui::EventDispatchDetails details = dispatcher->OnEventFromSource(&key_event1);
-  EXPECT_TRUE(key_event1.handled() || details.dispatcher_destroyed);
 
-  MSG msg2 = {NULL, WM_CHAR, L'A', 0};
-  ui::KeyEvent key_event2(msg2);
-  details = dispatcher->OnEventFromSource(&key_event2);
-  EXPECT_FALSE(key_event2.handled() || details.dispatcher_destroyed);
-
-  MSG msg3 = {NULL, WM_KEYUP, ui::VKEY_A, 0};
-  ui::KeyEvent key_event3(msg3);
-  details = dispatcher->OnEventFromSource(&key_event3);
-  EXPECT_FALSE(key_event3.handled() || details.dispatcher_destroyed);
-#elif defined(USE_X11)
   ui::ScopedXI2Event key_event;
   key_event.InitKeyEvent(ui::ET_KEY_PRESSED, ui::VKEY_A, 0);
   ui::KeyEvent key_event1(key_event);
@@ -791,7 +833,6 @@ TEST_F(AcceleratorControllerTest, MAYBE_ProcessOnce) {
   ui::KeyEvent key_event3(key_event);
   details = dispatcher->OnEventFromSource(&key_event3);
   EXPECT_FALSE(key_event3.handled() || details.dispatcher_destroyed);
-#endif
   EXPECT_EQ(1, target.accelerator_pressed_count());
 }
 #endif
@@ -807,7 +848,6 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
   EXPECT_TRUE(ProcessInController(
       ui::Accelerator(ui::VKEY_MEDIA_LAUNCH_APP1, ui::EF_NONE)));
 
-#if defined(OS_CHROMEOS)
   // The "Take Screenshot", "Take Partial Screenshot", volume, brightness, and
   // keyboard brightness accelerators are only defined on ChromeOS.
   {
@@ -836,21 +876,23 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
   const ui::Accelerator volume_down(ui::VKEY_VOLUME_DOWN, ui::EF_NONE);
   const ui::Accelerator volume_up(ui::VKEY_VOLUME_UP, ui::EF_NONE);
   {
-    TestVolumeControlDelegate* delegate = new TestVolumeControlDelegate;
-    WmShell::Get()->system_tray_delegate()->SetVolumeControlDelegate(
-        std::unique_ptr<VolumeControlDelegate>(delegate));
-    EXPECT_EQ(0, delegate->handle_volume_mute_count());
+    base::UserActionTester user_action_tester;
+    ui::AcceleratorHistory* history = GetController()->accelerator_history();
+
+    EXPECT_EQ(0, user_action_tester.GetActionCount("Accel_VolumeMute_F8"));
     EXPECT_TRUE(ProcessInController(volume_mute));
-    EXPECT_EQ(1, delegate->handle_volume_mute_count());
-    EXPECT_EQ(volume_mute, delegate->last_accelerator());
-    EXPECT_EQ(0, delegate->handle_volume_down_count());
+    EXPECT_EQ(1, user_action_tester.GetActionCount("Accel_VolumeMute_F8"));
+    EXPECT_EQ(volume_mute, history->current_accelerator());
+
+    EXPECT_EQ(0, user_action_tester.GetActionCount("Accel_VolumeDown_F9"));
     EXPECT_TRUE(ProcessInController(volume_down));
-    EXPECT_EQ(1, delegate->handle_volume_down_count());
-    EXPECT_EQ(volume_down, delegate->last_accelerator());
-    EXPECT_EQ(0, delegate->handle_volume_up_count());
+    EXPECT_EQ(1, user_action_tester.GetActionCount("Accel_VolumeDown_F9"));
+    EXPECT_EQ(volume_down, history->current_accelerator());
+
+    EXPECT_EQ(0, user_action_tester.GetActionCount("Accel_VolumeUp_F10"));
     EXPECT_TRUE(ProcessInController(volume_up));
-    EXPECT_EQ(1, delegate->handle_volume_up_count());
-    EXPECT_EQ(volume_up, delegate->last_accelerator());
+    EXPECT_EQ(volume_up, history->current_accelerator());
+    EXPECT_EQ(1, user_action_tester.GetActionCount("Accel_VolumeUp_F10"));
   }
   // Brightness
   // ui::VKEY_BRIGHTNESS_DOWN/UP are not defined on Windows.
@@ -859,7 +901,7 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
   {
     DummyBrightnessControlDelegate* delegate =
         new DummyBrightnessControlDelegate;
-    GetController()->SetBrightnessControlDelegate(
+    SetBrightnessControlDelegate(
         std::unique_ptr<BrightnessControlDelegate>(delegate));
     EXPECT_EQ(0, delegate->handle_brightness_down_count());
     EXPECT_TRUE(ProcessInController(brightness_down));
@@ -881,7 +923,7 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
     EXPECT_TRUE(ProcessInController(alt_brightness_up));
     DummyKeyboardBrightnessControlDelegate* delegate =
         new DummyKeyboardBrightnessControlDelegate;
-    GetController()->SetKeyboardBrightnessControlDelegate(
+    SetKeyboardBrightnessControlDelegate(
         std::unique_ptr<KeyboardBrightnessControlDelegate>(delegate));
     EXPECT_EQ(0, delegate->handle_keyboard_brightness_down_count());
     EXPECT_TRUE(ProcessInController(alt_brightness_down));
@@ -892,9 +934,7 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
     EXPECT_EQ(1, delegate->handle_keyboard_brightness_up_count());
     EXPECT_EQ(alt_brightness_up, delegate->last_accelerator());
   }
-#endif
 
-#if !defined(OS_WIN)
   // Exit
   ExitWarningHandler* ewh = GetController()->GetExitWarningHandlerForTest();
   ASSERT_TRUE(ewh);
@@ -909,7 +949,6 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
   EXPECT_TRUE(is_idle(ewh));
   EXPECT_FALSE(is_ui_shown(ewh));
   Reset(ewh);
-#endif
 
   // New tab
   EXPECT_TRUE(
@@ -931,7 +970,6 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
   EXPECT_TRUE(
       ProcessInController(ui::Accelerator(ui::VKEY_ESCAPE, ui::EF_SHIFT_DOWN)));
 
-#if defined(OS_CHROMEOS)
   // Open file manager
   EXPECT_TRUE(ProcessInController(
       ui::Accelerator(ui::VKEY_M, ui::EF_SHIFT_DOWN | ui::EF_ALT_DOWN)));
@@ -940,26 +978,27 @@ TEST_F(AcceleratorControllerTest, GlobalAccelerators) {
   // NOTE: Accelerators that do not work on the lock screen need to be
   // tested before the sequence below is invoked because it causes a side
   // effect of locking the screen.
-  EXPECT_TRUE(ProcessInController(
-      ui::Accelerator(ui::VKEY_L, ui::EF_SHIFT_DOWN | ui::EF_CONTROL_DOWN)));
-#endif
+  EXPECT_TRUE(
+      ProcessInController(ui::Accelerator(ui::VKEY_L, ui::EF_COMMAND_DOWN)));
 }
 
 TEST_F(AcceleratorControllerTest, GlobalAcceleratorsToggleAppList) {
-  AccessibilityDelegate* delegate = WmShell::Get()->GetAccessibilityDelegate();
-  EXPECT_FALSE(Shell::GetInstance()->GetAppListTargetVisibility());
+  app_list::test::TestAppListPresenter test_app_list_presenter;
+  WmShell::Get()->app_list()->SetAppListPresenter(
+      test_app_list_presenter.CreateInterfacePtrAndBind());
+  AccessibilityDelegate* delegate = WmShell::Get()->accessibility_delegate();
 
-  // The press event should not open the AppList, the release should instead.
+  // The press event should not toggle the AppList, the release should instead.
   EXPECT_FALSE(
       ProcessInController(ui::Accelerator(ui::VKEY_LWIN, ui::EF_NONE)));
+  RunAllPendingInMessageLoop();
   EXPECT_EQ(ui::VKEY_LWIN, GetCurrentAccelerator().key_code());
-
-  EXPECT_FALSE(Shell::GetInstance()->GetAppListTargetVisibility());
+  EXPECT_EQ(0u, test_app_list_presenter.toggle_count());
 
   EXPECT_TRUE(
       ProcessInController(ReleaseAccelerator(ui::VKEY_LWIN, ui::EF_NONE)));
-  EXPECT_TRUE(Shell::GetInstance()->GetAppListTargetVisibility());
-
+  RunAllPendingInMessageLoop();
+  EXPECT_EQ(1u, test_app_list_presenter.toggle_count());
   EXPECT_EQ(ui::VKEY_LWIN, GetPreviousAccelerator().key_code());
 
   // When spoken feedback is on, the AppList should not toggle.
@@ -969,32 +1008,26 @@ TEST_F(AcceleratorControllerTest, GlobalAcceleratorsToggleAppList) {
   EXPECT_FALSE(
       ProcessInController(ReleaseAccelerator(ui::VKEY_LWIN, ui::EF_NONE)));
   delegate->ToggleSpokenFeedback(A11Y_NOTIFICATION_NONE);
-  EXPECT_TRUE(Shell::GetInstance()->GetAppListTargetVisibility());
+  RunAllPendingInMessageLoop();
+  EXPECT_EQ(1u, test_app_list_presenter.toggle_count());
 
+  // Turning off spoken feedback should allow the AppList to toggle again.
   EXPECT_FALSE(
       ProcessInController(ui::Accelerator(ui::VKEY_LWIN, ui::EF_NONE)));
   EXPECT_TRUE(
       ProcessInController(ReleaseAccelerator(ui::VKEY_LWIN, ui::EF_NONE)));
-  EXPECT_FALSE(Shell::GetInstance()->GetAppListTargetVisibility());
+  RunAllPendingInMessageLoop();
+  EXPECT_EQ(2u, test_app_list_presenter.toggle_count());
 
-  // When spoken feedback is on, the AppList should not toggle.
-  delegate->ToggleSpokenFeedback(A11Y_NOTIFICATION_NONE);
-  EXPECT_FALSE(
-      ProcessInController(ui::Accelerator(ui::VKEY_LWIN, ui::EF_NONE)));
-  EXPECT_FALSE(
-      ProcessInController(ReleaseAccelerator(ui::VKEY_LWIN, ui::EF_NONE)));
-  delegate->ToggleSpokenFeedback(A11Y_NOTIFICATION_NONE);
-  EXPECT_FALSE(Shell::GetInstance()->GetAppListTargetVisibility());
-
-#if defined(OS_CHROMEOS)
   // The press of VKEY_BROWSER_SEARCH should toggle the AppList
   EXPECT_TRUE(ProcessInController(
       ui::Accelerator(ui::VKEY_BROWSER_SEARCH, ui::EF_NONE)));
-  EXPECT_TRUE(Shell::GetInstance()->GetAppListTargetVisibility());
+  RunAllPendingInMessageLoop();
+  EXPECT_EQ(3u, test_app_list_presenter.toggle_count());
   EXPECT_FALSE(ProcessInController(
       ReleaseAccelerator(ui::VKEY_BROWSER_SEARCH, ui::EF_NONE)));
-  EXPECT_TRUE(Shell::GetInstance()->GetAppListTargetVisibility());
-#endif
+  RunAllPendingInMessageLoop();
+  EXPECT_EQ(3u, test_app_list_presenter.toggle_count());
 }
 
 TEST_F(AcceleratorControllerTest, ImeGlobalAccelerators) {
@@ -1049,13 +1082,12 @@ TEST_F(AcceleratorControllerTest, ImeGlobalAcceleratorsWorkaround139556) {
 }
 
 TEST_F(AcceleratorControllerTest, PreferredReservedAccelerators) {
-#if defined(OS_CHROMEOS)
   // Power key is reserved on chromeos.
   EXPECT_TRUE(GetController()->IsReserved(
       ui::Accelerator(ui::VKEY_POWER, ui::EF_NONE)));
   EXPECT_FALSE(GetController()->IsPreferred(
       ui::Accelerator(ui::VKEY_POWER, ui::EF_NONE)));
-#endif
+
   // ALT+Tab are not reserved but preferred.
   EXPECT_FALSE(GetController()->IsReserved(
       ui::Accelerator(ui::VKEY_TAB, ui::EF_ALT_DOWN)));
@@ -1112,18 +1144,23 @@ TEST_F(PreferredReservedAcceleratorsTest, AcceleratorsWithFullscreen) {
   ASSERT_TRUE(w1_state->IsFullscreen());
 
   ui::test::EventGenerator& generator = GetEventGenerator();
-#if defined(OS_CHROMEOS)
+
   // Power key (reserved) should always be handled.
-  LockStateController::TestApi test_api(
+  test::LockStateControllerTestApi test_api(
       Shell::GetInstance()->lock_state_controller());
   EXPECT_FALSE(test_api.is_animating_lock());
   generator.PressKey(ui::VKEY_POWER, ui::EF_NONE);
   EXPECT_TRUE(test_api.is_animating_lock());
-#endif
+
+  auto press_and_release_alt_tab = [&generator]() {
+    generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+    // Release the alt key to trigger the window activation.
+    generator.ReleaseKey(ui::VKEY_MENU, ui::EF_NONE);
+  };
 
   // A fullscreen window can consume ALT-TAB (preferred).
   ASSERT_EQ(w1, wm::GetActiveWindow());
-  generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  press_and_release_alt_tab();
   ASSERT_EQ(w1, wm::GetActiveWindow());
   ASSERT_NE(w2, wm::GetActiveWindow());
 
@@ -1138,7 +1175,7 @@ TEST_F(PreferredReservedAcceleratorsTest, AcceleratorsWithFullscreen) {
   ASSERT_FALSE(w1_state->IsFullscreen());
 
   EXPECT_EQ(w1, wm::GetActiveWindow());
-  generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  press_and_release_alt_tab();
   ASSERT_NE(w1, wm::GetActiveWindow());
   ASSERT_EQ(w2, wm::GetActiveWindow());
 }
@@ -1156,14 +1193,13 @@ TEST_F(PreferredReservedAcceleratorsTest, AcceleratorsWithPinned) {
   }
 
   ui::test::EventGenerator& generator = GetEventGenerator();
-#if defined(OS_CHROMEOS)
+
   // Power key (reserved) should always be handled.
-  LockStateController::TestApi test_api(
+  test::LockStateControllerTestApi test_api(
       Shell::GetInstance()->lock_state_controller());
   EXPECT_FALSE(test_api.is_animating_lock());
   generator.PressKey(ui::VKEY_POWER, ui::EF_NONE);
   EXPECT_TRUE(test_api.is_animating_lock());
-#endif
 
   // A pinned window can consume ALT-TAB (preferred), but no side effect.
   ASSERT_EQ(w1, wm::GetActiveWindow());
@@ -1173,7 +1209,6 @@ TEST_F(PreferredReservedAcceleratorsTest, AcceleratorsWithPinned) {
   ASSERT_NE(w2, wm::GetActiveWindow());
 }
 
-#if defined(OS_CHROMEOS)
 TEST_F(AcceleratorControllerTest, DisallowedAtModalWindow) {
   std::set<AcceleratorAction> all_actions;
   for (size_t i = 0; i < kAcceleratorDataLength; ++i)
@@ -1181,29 +1216,30 @@ TEST_F(AcceleratorControllerTest, DisallowedAtModalWindow) {
   std::set<AcceleratorAction> all_debug_actions;
   for (size_t i = 0; i < kDebugAcceleratorDataLength; ++i)
     all_debug_actions.insert(kDebugAcceleratorData[i].action);
+  std::set<AcceleratorAction> all_dev_actions;
+  for (size_t i = 0; i < kDeveloperAcceleratorDataLength; ++i)
+    all_dev_actions.insert(kDeveloperAcceleratorData[i].action);
 
   std::set<AcceleratorAction> actionsAllowedAtModalWindow;
   for (size_t k = 0; k < kActionsAllowedAtModalWindowLength; ++k)
     actionsAllowedAtModalWindow.insert(kActionsAllowedAtModalWindow[k]);
-  for (std::set<AcceleratorAction>::const_iterator it =
-           actionsAllowedAtModalWindow.begin();
-       it != actionsAllowedAtModalWindow.end(); ++it) {
-    EXPECT_TRUE(all_actions.find(*it) != all_actions.end() ||
-                all_debug_actions.find(*it) != all_debug_actions.end())
+  for (const auto& action : actionsAllowedAtModalWindow) {
+    EXPECT_TRUE(all_actions.find(action) != all_actions.end() ||
+                all_debug_actions.find(action) != all_debug_actions.end() ||
+                all_dev_actions.find(action) != all_dev_actions.end())
         << " action from kActionsAllowedAtModalWindow"
-        << " not found in kAcceleratorData or kDebugAcceleratorData. "
-        << "action: " << *it;
+        << " not found in kAcceleratorData, kDebugAcceleratorData or"
+        << " kDeveloperAcceleratorData action: " << action;
   }
   std::unique_ptr<aura::Window> window(
       CreateTestWindowInShellWithBounds(gfx::Rect(5, 5, 20, 20)));
   wm::ActivateWindow(window.get());
   WmShell::Get()->SimulateModalWindowOpenForTesting(true);
-  for (std::set<AcceleratorAction>::const_iterator it = all_actions.begin();
-       it != all_actions.end(); ++it) {
-    if (actionsAllowedAtModalWindow.find(*it) ==
+  for (const auto& action : all_actions) {
+    if (actionsAllowedAtModalWindow.find(action) ==
         actionsAllowedAtModalWindow.end()) {
-      EXPECT_TRUE(GetController()->PerformActionIfEnabled(*it))
-          << " for action (disallowed at modal window): " << *it;
+      EXPECT_TRUE(GetController()->PerformActionIfEnabled(action))
+          << " for action (disallowed at modal window): " << action;
     }
   }
   //  Testing of top row (F5-F10) accelerators that should still work
@@ -1237,7 +1273,7 @@ TEST_F(AcceleratorControllerTest, DisallowedAtModalWindow) {
   {
     DummyBrightnessControlDelegate* delegate =
         new DummyBrightnessControlDelegate;
-    GetController()->SetBrightnessControlDelegate(
+    SetBrightnessControlDelegate(
         std::unique_ptr<BrightnessControlDelegate>(delegate));
     EXPECT_EQ(0, delegate->handle_brightness_down_count());
     EXPECT_TRUE(ProcessInController(brightness_down));
@@ -1253,30 +1289,28 @@ TEST_F(AcceleratorControllerTest, DisallowedAtModalWindow) {
   const ui::Accelerator volume_down(ui::VKEY_VOLUME_DOWN, ui::EF_NONE);
   const ui::Accelerator volume_up(ui::VKEY_VOLUME_UP, ui::EF_NONE);
   {
+    base::UserActionTester user_action_tester;
+    ui::AcceleratorHistory* history = GetController()->accelerator_history();
+
+    EXPECT_EQ(0, user_action_tester.GetActionCount("Accel_VolumeMute_F8"));
     EXPECT_TRUE(ProcessInController(volume_mute));
+    EXPECT_EQ(1, user_action_tester.GetActionCount("Accel_VolumeMute_F8"));
+    EXPECT_EQ(volume_mute, history->current_accelerator());
+
+    EXPECT_EQ(0, user_action_tester.GetActionCount("Accel_VolumeDown_F9"));
     EXPECT_TRUE(ProcessInController(volume_down));
+    EXPECT_EQ(1, user_action_tester.GetActionCount("Accel_VolumeDown_F9"));
+    EXPECT_EQ(volume_down, history->current_accelerator());
+
+    EXPECT_EQ(0, user_action_tester.GetActionCount("Accel_VolumeUp_F10"));
     EXPECT_TRUE(ProcessInController(volume_up));
-    TestVolumeControlDelegate* delegate = new TestVolumeControlDelegate;
-    WmShell::Get()->system_tray_delegate()->SetVolumeControlDelegate(
-        std::unique_ptr<VolumeControlDelegate>(delegate));
-    EXPECT_EQ(0, delegate->handle_volume_mute_count());
-    EXPECT_TRUE(ProcessInController(volume_mute));
-    EXPECT_EQ(1, delegate->handle_volume_mute_count());
-    EXPECT_EQ(volume_mute, delegate->last_accelerator());
-    EXPECT_EQ(0, delegate->handle_volume_down_count());
-    EXPECT_TRUE(ProcessInController(volume_down));
-    EXPECT_EQ(1, delegate->handle_volume_down_count());
-    EXPECT_EQ(volume_down, delegate->last_accelerator());
-    EXPECT_EQ(0, delegate->handle_volume_up_count());
-    EXPECT_TRUE(ProcessInController(volume_up));
-    EXPECT_EQ(1, delegate->handle_volume_up_count());
-    EXPECT_EQ(volume_up, delegate->last_accelerator());
+    EXPECT_EQ(volume_up, history->current_accelerator());
+    EXPECT_EQ(1, user_action_tester.GetActionCount("Accel_VolumeUp_F10"));
   }
 }
-#endif
 
 TEST_F(AcceleratorControllerTest, DisallowedWithNoWindow) {
-  AccessibilityDelegate* delegate = WmShell::Get()->GetAccessibilityDelegate();
+  AccessibilityDelegate* delegate = WmShell::Get()->accessibility_delegate();
 
   for (size_t i = 0; i < kActionsNeedingWindowLength; ++i) {
     delegate->TriggerAccessibilityAlert(A11Y_ALERT_NONE);
@@ -1306,7 +1340,6 @@ TEST_F(AcceleratorControllerTest, DisallowedWithNoWindow) {
   }
 }
 
-#if defined(OS_CHROMEOS)
 namespace {
 
 // defines a class to test the behavior of deprecated accelerators.
@@ -1408,6 +1441,5 @@ TEST_F(DeprecatedAcceleratorTester, TestNewAccelerators) {
     ResetStateIfNeeded();
   }
 }
-#endif  // defined(OS_CHROMEOS)
 
 }  // namespace ash
