@@ -5,6 +5,8 @@
 #ifndef ASH_ACCELERATORS_ACCELERATOR_DELEGATE_H_
 #define ASH_ACCELERATORS_ACCELERATOR_DELEGATE_H_
 
+#include <memory>
+
 #include "ash/ash_export.h"
 #include "base/compiler_specific.h"
 #include "base/macros.h"
@@ -12,6 +14,8 @@
 
 namespace ash {
 
+class AcceleratorRouter;
+
 class ASH_EXPORT AcceleratorDelegate
     : NON_EXPORTED_BASE(public ::wm::AcceleratorDelegate) {
  public:
@@ -23,12 +27,7 @@ class ASH_EXPORT AcceleratorDelegate
                           const ui::Accelerator& accelerator) override;
 
  private:
-  // Returns true if the window should be allowed a chance to handle
-  // system keys.
-  bool CanConsumeSystemKeys(const ui::KeyEvent& event);
-
-  bool ShouldProcessAcceleratorNow(const ui::KeyEvent& event,
-                                   const ui::Accelerator& accelerator);
+  std::unique_ptr<AcceleratorRouter> router_;
 
   DISALLOW_COPY_AND_ASSIGN(AcceleratorDelegate);
 };
