@@ -17,7 +17,7 @@ class WmLookupAura : public WmLookup {
   ~WmLookupAura() override;
 
   // WmLookup:
-  WmRootWindowController* GetRootWindowControllerWithDisplayId(
+  RootWindowController* GetRootWindowControllerWithDisplayId(
       int64_t id) override;
   WmWindow* GetWindowForWidget(views::Widget* widget) override;
 
