@@ -5,12 +5,9 @@
 #ifndef ASH_AUTOCLICK_AUTOCLICK_CONTROLLER_H
 #define ASH_AUTOCLICK_AUTOCLICK_CONTROLLER_H
 
-#include <memory>
-
 #include "ash/ash_export.h"
 #include "base/macros.h"
 #include "base/time/time.h"
-#include "ui/gfx/geometry/point.h"
 
 namespace ash {
 
@@ -19,31 +16,8 @@ namespace ash {
 // the mouse had been at rest.
 class ASH_EXPORT AutoclickController {
  public:
-  class Delegate {
-   public:
-    Delegate() {}
-    virtual ~Delegate() {}
-
-    // Called when an autoclick gesture begins.
-    virtual void StartGesture(base::TimeDelta duration,
-                              const gfx::Point& center_point_in_screen) = 0;
-
-    // Called when the gesture has ended, either because the mouse moved or
-    // because the autoclick completed.
-    virtual void StopGesture() = 0;
-
-    // Called when the autoclick target has changed.
-    virtual void SetGestureCenter(const gfx::Point& center_point_in_screen) = 0;
-
-   private:
-    DISALLOW_COPY_AND_ASSIGN(Delegate);
-  };
-
   virtual ~AutoclickController() {}
 
-  // Set the delegate to work with.
-  virtual void SetDelegate(std::unique_ptr<Delegate> delegate) = 0;
-
   // Set whether autoclicking is enabled.
   virtual void SetEnabled(bool enabled) = 0;
 
@@ -54,9 +28,6 @@ class ASH_EXPORT AutoclickController {
   // to when the autoclick event is sent.
   virtual void SetAutoclickDelay(base::TimeDelta delay) = 0;
 
-  // Returns the autoclick delay in milliseconds.
-  virtual base::TimeDelta GetAutoclickDelay() const = 0;
-
   static AutoclickController* CreateInstance();
 
   // The default wait time between last mouse movement and sending
@@ -68,6 +39,9 @@ class ASH_EXPORT AutoclickController {
 
  protected:
   AutoclickController() {}
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(AutoclickController);
 };
 
 }  // namespace ash
