@@ -14,6 +14,10 @@ enum AccessibilityNotificationVisibility {
 
 enum AccessibilityAlert {
   A11Y_ALERT_NONE,
+  A11Y_ALERT_CAPS_ON,
+  A11Y_ALERT_CAPS_OFF,
+  A11Y_ALERT_SCREEN_ON,
+  A11Y_ALERT_SCREEN_OFF,
   A11Y_ALERT_WINDOW_NEEDED,
   A11Y_ALERT_WINDOW_OVERVIEW_MODE_ENTERED
 };
