@@ -5,6 +5,7 @@
 #include "ash/common/ash_constants.h"
 
 #include "third_party/skia/include/core/SkColor.h"
+#include "ui/gfx/color_palette.h"
 
 namespace ash {
 
@@ -13,12 +14,9 @@ const int kResizeOutsideBoundsSize = 6;
 const int kResizeOutsideBoundsScaleForTouch = 5;
 const int kResizeInsideBoundsSize = 1;
 
-#if defined(OS_CHROMEOS)
 const SkColor kChromeOsBootColor = SkColorSetRGB(0xfe, 0xfe, 0xfe);
-#endif
 
-const SkColor kFocusBorderColor = SkColorSetRGB(64, 128, 250);
-
-const int kImmersiveFullscreenTopEdgeInset = 8;
+const SkColor kFocusBorderColor = SkColorSetA(gfx::kGoogleBlue500, 0x99);
+const float kFocusBorderThickness = 2.f;
 
 }  // namespace ash
