@@ -11,6 +11,7 @@
 namespace ash {
 
 struct ShelfItem;
+class ShelfItemDelegate;
 
 class ASH_EXPORT ShelfModelObserver {
  public:
@@ -29,6 +30,13 @@ class ASH_EXPORT ShelfModelObserver {
   // before the change.
   virtual void ShelfItemChanged(int index, const ShelfItem& old_item) = 0;
 
+  // Gets called when a ShelfItemDelegate gets changed. Note that
+  // |item_delegate| can be null.
+  // NOTE: This is added a temporary fix for M39 to fix crbug.com/429870.
+  // TODO(skuhne): Find the real reason for this problem and remove this fix.
+  virtual void OnSetShelfItemDelegate(ShelfID id,
+                                      ShelfItemDelegate* item_delegate) = 0;
+
  protected:
   virtual ~ShelfModelObserver() {}
 };
