@@ -7,28 +7,37 @@
 
 #include <string>
 
-#include "ash/common/system/locale/locale_observer.h"
+#include "ash/public/interfaces/locale.mojom.h"
 #include "base/macros.h"
+#include "mojo/public/cpp/bindings/binding_set.h"
 
 namespace ash {
 
 // Observes the locale change and creates rich notification for the change.
-class LocaleNotificationController : public LocaleObserver {
+class LocaleNotificationController
+    : public mojom::LocaleNotificationController {
  public:
   LocaleNotificationController();
   ~LocaleNotificationController() override;
 
+  // Binds the mojom::LocaleNotificationController interface request to this
+  // object.
+  void BindRequest(mojom::LocaleNotificationControllerRequest request);
+
  private:
-  // Overridden from LocaleObserver.
-  void OnLocaleChanged(LocaleObserver::Delegate* delegate,
-                       const std::string& cur_locale,
+  // Overridden from mojom::LocaleNotificationController:
+  void OnLocaleChanged(const std::string& cur_locale,
                        const std::string& from_locale,
-                       const std::string& to_locale) override;
+                       const std::string& to_locale,
+                       const OnLocaleChangedCallback& callback) override;
 
   std::string cur_locale_;
   std::string from_locale_;
   std::string to_locale_;
 
+  // Bindings for the LocaleNotificationController interface.
+  mojo::BindingSet<mojom::LocaleNotificationController> bindings_;
+
   DISALLOW_COPY_AND_ASSIGN(LocaleNotificationController);
 };
 
