@@ -17,6 +17,9 @@ class ASH_EXPORT TrayPopupHeaderButton : public views::ToggleImageButton {
  public:
   static const char kViewClassName[];
 
+  TrayPopupHeaderButton(views::ButtonListener* listener,
+                        const gfx::ImageSkia& icon,
+                        int accessible_name_id);
   TrayPopupHeaderButton(views::ButtonListener* listener,
                         int enabled_resource_id,
                         int disabled_resource_id,
@@ -33,6 +36,8 @@ class ASH_EXPORT TrayPopupHeaderButton : public views::ToggleImageButton {
   // Overridden from views::CustomButton:
   void StateChanged() override;
 
+  void Initialize(const gfx::ImageSkia& icon, int accessible_name_id);
+
   DISALLOW_COPY_AND_ASSIGN(TrayPopupHeaderButton);
 };
 
