@@ -4,23 +4,24 @@
 
 #include "ash/common/system/web_notification/web_notification_tray.h"
 
-#include "ash/common/ash_switches.h"
 #include "ash/common/material_design/material_design_controller.h"
 #include "ash/common/session/session_state_delegate.h"
 #include "ash/common/shelf/shelf_constants.h"
 #include "ash/common/shelf/wm_shelf.h"
 #include "ash/common/shelf/wm_shelf_util.h"
-#include "ash/common/shell_window_ids.h"
+#include "ash/common/system/status_area_widget.h"
+#include "ash/common/system/tray/system_tray.h"
 #include "ash/common/system/tray/system_tray_delegate.h"
 #include "ash/common/system/tray/tray_bubble_wrapper.h"
 #include "ash/common/system/tray/tray_constants.h"
 #include "ash/common/system/tray/tray_utils.h"
 #include "ash/common/system/web_notification/ash_popup_alignment_delegate.h"
 #include "ash/common/wm_lookup.h"
-#include "ash/common/wm_root_window_controller.h"
 #include "ash/common/wm_shell.h"
 #include "ash/common/wm_window.h"
-#include "ash/system/tray/system_tray.h"
+#include "ash/public/cpp/shell_window_ids.h"
+#include "ash/resources/vector_icons/vector_icons.h"
+#include "ash/root_window_controller.h"
 #include "base/auto_reset.h"
 #include "base/i18n/number_formatting.h"
 #include "base/i18n/rtl.h"
@@ -39,26 +40,21 @@
 #include "ui/message_center/views/message_popup_collection.h"
 #include "ui/strings/grit/ui_strings.h"
 #include "ui/views/bubble/tray_bubble_view.h"
-#include "ui/views/controls/button/custom_button.h"
 #include "ui/views/controls/image_view.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/controls/menu/menu_runner.h"
 #include "ui/views/layout/fill_layout.h"
 
-#if defined(OS_CHROMEOS)
-
 namespace message_center {
 
 MessageCenterTrayDelegate* CreateMessageCenterTray() {
-  // On Windows+Ash the Tray will not be hosted in ash::Shell.
+  // On non-CrOS, the Tray will not be hosted in ash::Shell.
   NOTREACHED();
-  return NULL;
+  return nullptr;
 }
 
 }  // namespace message_center
 
-#endif  // defined(OS_CHROMEOS)
-
 namespace ash {
 namespace {
 
@@ -69,9 +65,10 @@ constexpr int kEnableQuietModeDay = 2;
 constexpr int kMaximumSmallIconCount = 3;
 
 constexpr gfx::Size kTrayItemInnerIconSize(16, 16);
-constexpr gfx::Size kTrayItemInnerBellIconSize(18, 18);
+constexpr gfx::Size kTrayItemInnerBellIconSizeNonMd(18, 18);
 constexpr gfx::Size kTrayItemOuterSize(26, 26);
-constexpr gfx::Insets kTrayItemInsets(3, 3);
+constexpr int kTrayMainAxisInset = 3;
+constexpr int kTrayCrossAxisInset = 0;
 
 constexpr int kTrayItemAnimationDurationMS = 200;
 
@@ -87,7 +84,8 @@ const SkColor kWebNotificationColorNoUnread =
     SkColorSetARGB(128, 255, 255, 255);
 const SkColor kWebNotificationColorWithUnread = SK_ColorWHITE;
 const int kNoUnreadIconSize = 18;
-}
+
+}  // namespace
 
 // Class to initialize and manage the WebNotificationBubble and
 // TrayBubbleWrapper instances for a bubble.
@@ -95,25 +93,17 @@ class WebNotificationBubbleWrapper {
  public:
   // Takes ownership of |bubble| and creates |bubble_wrapper_|.
   WebNotificationBubbleWrapper(WebNotificationTray* tray,
+                               TrayBackgroundView* anchor_tray,
                                message_center::MessageBubbleBase* bubble) {
     bubble_.reset(bubble);
     views::TrayBubbleView::AnchorAlignment anchor_alignment =
         tray->GetAnchorAlignment();
     views::TrayBubbleView::InitParams init_params =
         bubble->GetInitParams(anchor_alignment);
-    views::View* anchor = tray->tray_container();
-    if (anchor_alignment == views::TrayBubbleView::ANCHOR_ALIGNMENT_BOTTOM) {
-      gfx::Point bounds(anchor->width() / 2, 0);
-      views::View::ConvertPointToWidget(anchor, &bounds);
-      init_params.arrow_offset = bounds.x();
-    }
-    DCHECK(anchor);
-    // TrayBubbleView uses |anchor| and |tray| to determine the parent
-    // container. See WebNotificationTray::OnBeforeBubbleWidgetInit().
-    views::TrayBubbleView* bubble_view =
-        views::TrayBubbleView::Create(anchor, tray, &init_params);
+    views::TrayBubbleView* bubble_view = views::TrayBubbleView::Create(
+        anchor_tray->GetBubbleAnchor(), tray, &init_params);
+    bubble_view->set_anchor_view_insets(anchor_tray->GetBubbleAnchorInsets());
     bubble_wrapper_.reset(new TrayBubbleWrapper(tray, bubble_view));
-    bubble_view->SetArrowPaintType(views::BubbleBorder::PAINT_NONE);
     bubble->InitializeContents(bubble_view);
   }
 
@@ -176,7 +166,7 @@ class WebNotificationItem : public views::View, public gfx::AnimationDelegate {
   }
 
  protected:
-  // Overridden from views::ImageButton:
+  // Overridden from views::View:
   gfx::Size GetPreferredSize() const override {
     if (!animation_.get() || !animation_->is_animating())
       return kTrayItemOuterSize;
@@ -243,7 +233,7 @@ class WebNotificationItem : public views::View, public gfx::AnimationDelegate {
 class WebNotificationImage : public WebNotificationItem {
  public:
   WebNotificationImage(const gfx::ImageSkia& image,
-                       gfx::Size size,
+                       const gfx::Size& size,
                        gfx::AnimationContainer* container,
                        WebNotificationTray* tray)
       : WebNotificationItem(container, tray) {
@@ -307,24 +297,28 @@ WebNotificationTray::WebNotificationTray(WmShelf* shelf,
   DCHECK(status_area_window_);
   DCHECK(system_tray_);
 
-  gfx::ImageSkia bell_image;
   if (MaterialDesignController::IsShelfMaterial()) {
-    bell_image = CreateVectorIcon(gfx::VectorIconId::SHELF_NOTIFICATIONS,
-                                  kShelfIconColor);
+    SetInkDropMode(InkDropMode::ON);
+    SetContentsBackground(false);
+    gfx::ImageSkia bell_image =
+        CreateVectorIcon(kShelfNotificationsIcon, kShelfIconColor);
+    const gfx::Size bell_icon_size = kTrayItemInnerIconSize;
+    bell_icon_.reset(new WebNotificationImage(
+        bell_image, bell_icon_size, animation_container_.get(), this));
   } else {
-    bell_image =
+    SetContentsBackground(true);
+    gfx::ImageSkia bell_image =
         CreateVectorIcon(gfx::VectorIconId::NOTIFICATIONS, kNoUnreadIconSize,
                          kWebNotificationColorNoUnread);
+    const gfx::Size bell_icon_size = kTrayItemInnerBellIconSizeNonMd;
+    bell_icon_.reset(new WebNotificationImage(
+        bell_image, bell_icon_size, animation_container_.get(), this));
   }
-  bell_icon_.reset(new WebNotificationImage(bell_image,
-                                            kTrayItemInnerBellIconSize,
-                                            animation_container_.get(), this));
   tray_container()->AddChildView(bell_icon_.get());
 
   counter_.reset(new WebNotificationLabel(animation_container_.get(), this));
   tray_container()->AddChildView(counter_.get());
 
-  SetContentsBackground();
   message_center_tray_.reset(new message_center::MessageCenterTray(
       this, message_center::MessageCenter::Get()));
   popup_alignment_delegate_.reset(new AshPopupAlignmentDelegate(shelf));
@@ -337,7 +331,7 @@ WebNotificationTray::WebNotificationTray(WmShelf* shelf,
                                             display);
   OnMessageCenterTrayChanged();
 
-  tray_container()->SetMargin(kTrayItemInsets);
+  tray_container()->SetMargin(kTrayMainAxisInset, kTrayCrossAxisInset);
 }
 
 WebNotificationTray::~WebNotificationTray() {
@@ -361,7 +355,7 @@ bool WebNotificationTray::ShowMessageCenterInternal(bool show_settings) {
   should_block_shelf_auto_hide_ = true;
   message_center::MessageCenterBubble* message_center_bubble =
       new message_center::MessageCenterBubble(message_center(),
-                                              message_center_tray_.get(), true);
+                                              message_center_tray_.get());
 
   int max_height;
   if (IsHorizontalAlignment(shelf_alignment())) {
@@ -377,12 +371,22 @@ bool WebNotificationTray::ShowMessageCenterInternal(bool show_settings) {
       std::max(0, max_height - GetTrayConstant(TRAY_SPACING)));
   if (show_settings)
     message_center_bubble->SetSettingsVisible();
-  message_center_bubble_.reset(
-      new WebNotificationBubbleWrapper(this, message_center_bubble));
+
+  // For vertical shelf alignments, anchor to the WebNotificationTray, but for
+  // horizontal (i.e. bottom) shelves, anchor to the system tray.
+  TrayBackgroundView* anchor_tray = this;
+  if (IsHorizontalAlignment(shelf_alignment())) {
+    anchor_tray = WmShelf::ForWindow(status_area_window_)
+                      ->GetStatusAreaWidget()
+                      ->system_tray();
+  }
+
+  message_center_bubble_.reset(new WebNotificationBubbleWrapper(
+      this, anchor_tray, message_center_bubble));
 
   system_tray_->SetHideNotifications(true);
   shelf()->UpdateAutoHideState();
-  SetDrawBackgroundAsActive(true);
+  SetIsActive(true);
   return true;
 }
 
@@ -393,7 +397,7 @@ bool WebNotificationTray::ShowMessageCenter() {
 void WebNotificationTray::HideMessageCenter() {
   if (!message_center_bubble())
     return;
-  SetDrawBackgroundAsActive(false);
+  SetIsActive(false);
   message_center_bubble_.reset();
   should_block_shelf_auto_hide_ = false;
   show_message_center_on_unlock_ = false;
@@ -425,7 +429,8 @@ void WebNotificationTray::HidePopups() {
 // Private methods.
 
 bool WebNotificationTray::ShouldShowMessageCenter() {
-  return !system_tray_->HasNotificationBubble();
+  return WmShell::Get()->system_tray_delegate()->ShouldShowNotificationTray() &&
+         !system_tray_->HasNotificationBubble();
 }
 
 bool WebNotificationTray::ShouldBlockShelfAutoHide() const {
@@ -503,13 +508,6 @@ base::string16 WebNotificationTray::GetAccessibleNameForBubble() {
   return GetAccessibleNameForTray();
 }
 
-gfx::Rect WebNotificationTray::GetAnchorRect(
-    views::Widget* anchor_widget,
-    views::TrayBubbleView::AnchorType anchor_type,
-    views::TrayBubbleView::AnchorAlignment anchor_alignment) const {
-  return GetBubbleAnchorRect(anchor_widget, anchor_type, anchor_alignment);
-}
-
 void WebNotificationTray::OnBeforeBubbleWidgetInit(
     views::Widget* anchor_widget,
     views::Widget* bubble_widget,
@@ -554,12 +552,6 @@ bool WebNotificationTray::IsCommandIdEnabled(int command_id) const {
   return true;
 }
 
-bool WebNotificationTray::GetAcceleratorForCommandId(
-    int command_id,
-    ui::Accelerator* accelerator) {
-  return false;
-}
-
 void WebNotificationTray::ExecuteCommand(int command_id, int event_flags) {
   if (command_id == kToggleQuietMode) {
     bool in_quiet_mode = message_center()->IsQuietMode();
@@ -640,7 +632,7 @@ void WebNotificationTray::UpdateTrayContent() {
     counter_->SetVisible(false);
   }
 
-  SetVisible(IsLoggedIn());
+  SetVisible(IsLoggedIn() && ShouldShowMessageCenter());
   PreferredSizeChanged();
   Layout();
   SchedulePaint();
@@ -662,7 +654,6 @@ message_center::MessageCenter* WebNotificationTray::message_center() const {
 
 bool WebNotificationTray::IsLoggedIn() const {
   WmShell* shell = WmShell::Get();
-  // TODO(jamescook): Should this also check LoginState::LOCKED?
   return shell->system_tray_delegate()->GetUserLoginStatus() !=
              LoginStatus::NOT_LOGGED_IN &&
          !shell->GetSessionStateDelegate()->IsInSecondaryLoginScreen();
@@ -677,7 +668,7 @@ bool WebNotificationTray::IsPopupVisible() const {
 message_center::MessageCenterBubble*
 WebNotificationTray::GetMessageCenterBubbleForTest() {
   if (!message_center_bubble())
-    return NULL;
+    return nullptr;
   return static_cast<message_center::MessageCenterBubble*>(
       message_center_bubble()->bubble());
 }
