@@ -5,6 +5,8 @@
 #ifndef ASH_COMMON_WM_CONTAINER_FINDER_H_
 #define ASH_COMMON_WM_CONTAINER_FINDER_H_
 
+#include <vector>
+
 #include "ash/ash_export.h"
 
 namespace gfx {
@@ -17,7 +19,7 @@ class WmWindow;
 
 namespace wm {
 
-// Returns the first ancestor of |window| that has a known type.
+// Returns the first ancestor of |window| that has a known container ID.
 ASH_EXPORT WmWindow* GetContainerForWindow(WmWindow* window);
 
 // Returns the parent to add |window| to in |context|. This is generally
@@ -27,6 +29,13 @@ ASH_EXPORT WmWindow* GetDefaultParent(WmWindow* context,
                                       WmWindow* window,
                                       const gfx::Rect& bounds);
 
+// Returns the list of containers that match |container_id| in all root windows.
+// If |priority_root| is non-null, the container in |priority_root| is placed at
+// the front of the list.
+ASH_EXPORT std::vector<WmWindow*> GetContainersFromAllRootWindows(
+    int container_id,
+    WmWindow* priority_root = nullptr);
+
 }  // namespace wm
 }  // namespace ash
 
