@@ -14,11 +14,11 @@ enum OverviewAnimationType {
   OVERVIEW_ANIMATION_NONE,
   // Used to fade in the close button and label.
   OVERVIEW_ANIMATION_ENTER_OVERVIEW_MODE_FADE_IN,
+  // Used to fade out the label.
+  OVERVIEW_ANIMATION_EXIT_OVERVIEW_MODE_FADE_OUT,
   // Used to position windows when entering/exiting overview mode and when a
   // window is closed while overview mode is active.
   OVERVIEW_ANIMATION_LAY_OUT_SELECTOR_ITEMS,
-  // Used to hide non-overview mode windows.
-  OVERVIEW_ANIMATION_HIDE_WINDOW,
   // Used to restore windows to their original position when exiting overview
   // mode.
   OVERVIEW_ANIMATION_RESTORE_WINDOW,
