@@ -7,25 +7,24 @@
 #include <algorithm>
 #include <vector>
 
-#include "ash/common/material_design/material_design_controller.h"
 #include "ash/common/wm/overview/scoped_overview_animation_settings.h"
 #include "ash/common/wm/overview/scoped_overview_animation_settings_factory.h"
 #include "ash/common/wm/overview/window_selector_item.h"
 #include "ash/common/wm/window_state.h"
+#include "ash/common/wm_lookup.h"
+#include "ash/common/wm_shell.h"
 #include "ash/common/wm_window.h"
 #include "ash/common/wm_window_property.h"
+#include "ash/root_window_controller.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/single_thread_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
-#include "third_party/skia/include/core/SkPaint.h"
-#include "third_party/skia/include/core/SkPath.h"
-#include "third_party/skia/include/core/SkRect.h"
 #include "ui/compositor/layer.h"
-#include "ui/compositor/layer_delegate.h"
-#include "ui/compositor/paint_recorder.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/safe_integer_conversions.h"
 #include "ui/gfx/transform_util.h"
+#include "ui/views/widget/widget.h"
 
 using WmWindows = std::vector<ash::WmWindow*>;
 
@@ -36,13 +35,7 @@ namespace {
 // When set to true by tests makes closing the widget synchronous.
 bool immediate_close_for_tests = false;
 
-// The opacity level that windows will be set to when they are restored.
-const float kRestoreWindowOpacity = 1.0f;
-
-// Alpha value used to paint mask layer that masks the original window header.
-const int kOverviewContentMaskAlpha = 255;
-
-// Delay closing window with Material Design to allow it to shrink and fade out.
+// Delay closing window to allow it to shrink and fade out.
 const int kCloseWindowDelayInMilliseconds = 150;
 
 WmWindow* GetTransientRoot(WmWindow* window) {
@@ -175,83 +168,9 @@ TransientDescendantIteratorRange GetTransientTreeIterator(WmWindow* window) {
 
 }  // namespace
 
-// Mask layer that clips the window's original header in overview mode.
-// Only used with Material Design.
-class ScopedTransformOverviewWindow::OverviewContentMask
-    : public ui::LayerDelegate {
- public:
-  explicit OverviewContentMask();
-  ~OverviewContentMask() override;
-
-  void set_radius(float radius) { radius_ = radius; }
-  void set_inset(int inset) { inset_ = inset; }
-  ui::Layer* layer() { return &layer_; }
-
-  // Overridden from LayerDelegate.
-  void OnPaintLayer(const ui::PaintContext& context) override;
-  void OnDelegatedFrameDamage(const gfx::Rect& damage_rect_in_dip) override {}
-  void OnDeviceScaleFactorChanged(float device_scale_factor) override;
-  base::Closure PrepareForLayerBoundsChange() override;
-
- private:
-  ui::Layer layer_;
-  float radius_;
-  int inset_;
-
-  DISALLOW_COPY_AND_ASSIGN(OverviewContentMask);
-};
-
-ScopedTransformOverviewWindow::OverviewContentMask::OverviewContentMask()
-    : layer_(ui::LAYER_TEXTURED), radius_(0), inset_(0) {
-  layer_.set_delegate(this);
-}
-
-ScopedTransformOverviewWindow::OverviewContentMask::~OverviewContentMask() {
-  layer_.set_delegate(nullptr);
-}
-
-void ScopedTransformOverviewWindow::OverviewContentMask::OnPaintLayer(
-    const ui::PaintContext& context) {
-  ui::PaintRecorder recorder(context, layer()->size());
-  gfx::Rect bounds(layer()->bounds().size());
-  bounds.Inset(0, inset_, 0, 0);
-
-  // Tile a window into an area, rounding the bottom corners.
-  const SkRect rect = gfx::RectToSkRect(bounds);
-  const SkScalar corner_radius_scalar = SkIntToScalar(radius_);
-  SkScalar radii[8] = {0,
-                       0,  // top-left
-                       0,
-                       0,  // top-right
-                       corner_radius_scalar,
-                       corner_radius_scalar,  // bottom-right
-                       corner_radius_scalar,
-                       corner_radius_scalar};  // bottom-left
-  SkPath path;
-  path.addRoundRect(rect, radii, SkPath::kCW_Direction);
-
-  // Set a mask.
-  SkPaint paint;
-  paint.setAlpha(kOverviewContentMaskAlpha);
-  paint.setStyle(SkPaint::kFill_Style);
-  paint.setAntiAlias(true);
-  recorder.canvas()->DrawPath(path, paint);
-}
-
-void ScopedTransformOverviewWindow::OverviewContentMask::
-    OnDeviceScaleFactorChanged(float device_scale_factor) {
-  // Redrawing will take care of scale factor change.
-}
-
-base::Closure ScopedTransformOverviewWindow::OverviewContentMask::
-    PrepareForLayerBoundsChange() {
-  return base::Closure();
-}
-
 ScopedTransformOverviewWindow::ScopedTransformOverviewWindow(WmWindow* window)
     : window_(window),
       determined_original_window_shape_(false),
-      minimized_(window->GetShowState() == ui::SHOW_STATE_MINIMIZED),
       ignored_by_shelf_(window->GetWindowState()->ignored_by_shelf()),
       overview_started_(false),
       original_transform_(window->GetTargetTransform()),
@@ -261,50 +180,29 @@ ScopedTransformOverviewWindow::ScopedTransformOverviewWindow(WmWindow* window)
 ScopedTransformOverviewWindow::~ScopedTransformOverviewWindow() {}
 
 void ScopedTransformOverviewWindow::RestoreWindow() {
+  ShowHeader();
+  if (minimized_widget_) {
+    // TODO(oshima): Use unminimize animation instead of hiding animation.
+    minimized_widget_->CloseNow();
+    minimized_widget_.reset();
+    return;
+  }
   ScopedAnimationSettings animation_settings_list;
   BeginScopedAnimation(OverviewAnimationType::OVERVIEW_ANIMATION_RESTORE_WINDOW,
                        &animation_settings_list);
-  SetTransform(window()->GetRootWindow(), original_transform_,
-               false /* use_mask */, false /* use_shape */, 0);
-
+  SetTransform(window()->GetRootWindow(), original_transform_);
   std::unique_ptr<ScopedOverviewAnimationSettings> animation_settings =
       CreateScopedOverviewAnimationSettings(
           OverviewAnimationType::OVERVIEW_ANIMATION_LAY_OUT_SELECTOR_ITEMS,
           window_);
-  gfx::Transform transform;
-  if (minimized_ && window_->GetShowState() != ui::SHOW_STATE_MINIMIZED) {
-    // Setting opacity 0 and visible false ensures that the property change
-    // to SHOW_STATE_MINIMIZED will not animate the window from its original
-    // bounds to the minimized position.
-    // Hiding the window needs to be done before the target opacity is 0,
-    // otherwise the layer's visibility will not be updated
-    // (See VisibilityController::UpdateLayerVisibility).
-    window_->Hide();
-    window_->SetOpacity(0);
-    window_->SetShowState(ui::SHOW_STATE_MINIMIZED);
-  }
   window_->GetWindowState()->set_ignored_by_shelf(ignored_by_shelf_);
   SetOpacity(original_opacity_);
-
-  if (ash::MaterialDesignController::IsOverviewMaterial()) {
-    ui::Layer* layer = window()->GetLayer();
-    layer->SetMaskLayer(nullptr);
-    mask_.reset();
-
-    if (original_window_shape_) {
-      layer->SetAlphaShape(
-          base::WrapUnique(new SkRegion(*original_window_shape_.get())));
-    } else {
-      layer->SetAlphaShape(nullptr);
-    }
-    window()->SetMasksToBounds(false);
-  }
 }
 
 void ScopedTransformOverviewWindow::BeginScopedAnimation(
     OverviewAnimationType animation_type,
     ScopedAnimationSettings* animation_settings) {
-  for (auto* window : GetTransientTreeIterator(window_)) {
+  for (auto* window : GetTransientTreeIterator(GetOverviewWindow())) {
     animation_settings->push_back(
         CreateScopedOverviewAnimationSettings(animation_type, window));
   }
@@ -315,15 +213,18 @@ bool ScopedTransformOverviewWindow::Contains(const WmWindow* target) const {
     if (window->Contains(target))
       return true;
   }
-  return false;
+  WmWindow* mirror = GetOverviewWindowForMinimizedState();
+  return mirror && mirror->Contains(target);
 }
 
 gfx::Rect ScopedTransformOverviewWindow::GetTargetBoundsInScreen() const {
   gfx::Rect bounds;
-  for (auto* window : GetTransientTreeIterator(window_)) {
+  WmWindow* overview_window = GetOverviewWindow();
+  for (auto* window : GetTransientTreeIterator(overview_window)) {
     // Ignore other window types when computing bounding box of window
     // selector target item.
-    if (window != window_ && window->GetType() != ui::wm::WINDOW_TYPE_NORMAL &&
+    if (window != overview_window &&
+        window->GetType() != ui::wm::WINDOW_TYPE_NORMAL &&
         window->GetType() != ui::wm::WINDOW_TYPE_PANEL) {
       continue;
     }
@@ -333,17 +234,16 @@ gfx::Rect ScopedTransformOverviewWindow::GetTargetBoundsInScreen() const {
   return bounds;
 }
 
-gfx::Rect ScopedTransformOverviewWindow::GetTransformedBounds(
-    bool hide_header) const {
-  const bool material = ash::MaterialDesignController::IsOverviewMaterial();
-  const int top_inset = hide_header ? GetTopInset() : 0;
+gfx::Rect ScopedTransformOverviewWindow::GetTransformedBounds() const {
+  const int top_inset = GetTopInset();
   gfx::Rect bounds;
-  for (auto* window : GetTransientTreeIterator(window_)) {
+  WmWindow* overview_window = GetOverviewWindow();
+  for (auto* window : GetTransientTreeIterator(overview_window)) {
     // Ignore other window types when computing bounding box of window
     // selector target item.
-    if (window != window_ &&
-        (!material || (window->GetType() != ui::wm::WINDOW_TYPE_NORMAL &&
-                       window->GetType() != ui::wm::WINDOW_TYPE_PANEL))) {
+    if (window != overview_window &&
+        (window->GetType() != ui::wm::WINDOW_TYPE_NORMAL &&
+         window->GetType() != ui::wm::WINDOW_TYPE_PANEL)) {
       continue;
     }
     gfx::RectF window_bounds(window->GetTargetBounds());
@@ -352,10 +252,10 @@ gfx::Rect ScopedTransformOverviewWindow::GetTransformedBounds(
                             window->GetTargetTransform());
     new_transform.TransformRect(&window_bounds);
 
-    // With Material Design the preview title is shown above the preview window.
-    // Hide the window header for apps or browser windows with no tabs (web
-    // apps) to avoid showing both the window header and the preview title.
-    if (material && top_inset > 0) {
+    // The preview title is shown above the preview window. Hide the window
+    // header for apps or browser windows with no tabs (web apps) to avoid
+    // showing both the window header and the preview title.
+    if (top_inset > 0) {
       gfx::RectF header_bounds(window_bounds);
       header_bounds.set_height(top_inset);
       new_transform.TransformRect(&header_bounds);
@@ -367,7 +267,22 @@ gfx::Rect ScopedTransformOverviewWindow::GetTransformedBounds(
   return bounds;
 }
 
+SkColor ScopedTransformOverviewWindow::GetTopColor() const {
+  for (auto* window : GetTransientTreeIterator(window_)) {
+    // If there are regular windows in the transient ancestor tree, all those
+    // windows are shown in the same overview item and the header is not masked.
+    if (window != window_ && (window->GetType() == ui::wm::WINDOW_TYPE_NORMAL ||
+                              window->GetType() == ui::wm::WINDOW_TYPE_PANEL)) {
+      return SK_ColorTRANSPARENT;
+    }
+  }
+  return window_->GetColorProperty(WmWindowProperty::TOP_VIEW_COLOR);
+}
+
 int ScopedTransformOverviewWindow::GetTopInset() const {
+  // Mirror window doesn't have insets.
+  if (minimized_widget_)
+    return 0;
   for (auto* window : GetTransientTreeIterator(window_)) {
     // If there are regular windows in the transient ancestor tree, all those
     // windows are shown in the same overview item and the header is not masked.
@@ -379,19 +294,6 @@ int ScopedTransformOverviewWindow::GetTopInset() const {
   return window_->GetIntProperty(WmWindowProperty::TOP_VIEW_INSET);
 }
 
-void ScopedTransformOverviewWindow::ShowWindowIfMinimized() {
-  if (minimized_ && window_->GetShowState() == ui::SHOW_STATE_MINIMIZED)
-    window_->Show();
-}
-
-void ScopedTransformOverviewWindow::ShowWindowOnExit() {
-  if (minimized_) {
-    minimized_ = false;
-    original_transform_ = gfx::Transform();
-    original_opacity_ = kRestoreWindowOpacity;
-  }
-}
-
 void ScopedTransformOverviewWindow::OnWindowDestroyed() {
   window_ = nullptr;
 }
@@ -400,13 +302,8 @@ float ScopedTransformOverviewWindow::GetItemScale(const gfx::Size& source,
                                                   const gfx::Size& target,
                                                   int top_view_inset,
                                                   int title_height) {
-  if (ash::MaterialDesignController::IsOverviewMaterial()) {
-    return std::min(2.0f, static_cast<float>((target.height() - title_height)) /
-                              (source.height() - top_view_inset));
-  }
-  return std::min(
-      1.0f, std::min(static_cast<float>(target.width()) / source.width(),
-                     static_cast<float>(target.height()) / source.height()));
+  return std::min(2.0f, static_cast<float>((target.height() - title_height)) /
+                            (source.height() - top_view_inset));
 }
 
 gfx::Rect ScopedTransformOverviewWindow::ShrinkRectToFitPreservingAspectRatio(
@@ -418,14 +315,6 @@ gfx::Rect ScopedTransformOverviewWindow::ShrinkRectToFitPreservingAspectRatio(
   DCHECK_LE(top_view_inset, rect.height());
   const float scale =
       GetItemScale(rect.size(), bounds.size(), top_view_inset, title_height);
-  if (!ash::MaterialDesignController::IsOverviewMaterial()) {
-    return gfx::Rect(
-        bounds.x() + 0.5 * (bounds.width() - scale * rect.width()),
-        bounds.y() + title_height - scale * top_view_inset +
-            0.5 * (bounds.height() -
-                   (title_height + scale * (rect.height() - top_view_inset))),
-        rect.width() * scale, rect.height() * scale);
-  }
   const int horizontal_offset = gfx::ToFlooredInt(
       0.5 * (bounds.width() - gfx::ToFlooredInt(scale * rect.width())));
   const int width = bounds.width() - 2 * horizontal_offset;
@@ -450,50 +339,19 @@ gfx::Transform ScopedTransformOverviewWindow::GetTransformForRect(
 
 void ScopedTransformOverviewWindow::SetTransform(
     WmWindow* root_window,
-    const gfx::Transform& transform,
-    bool use_mask,
-    bool use_shape,
-    float radius) {
+    const gfx::Transform& transform) {
   DCHECK(overview_started_);
 
-  if (ash::MaterialDesignController::IsOverviewMaterial() &&
-      &transform != &original_transform_) {
-    if (use_mask && !mask_) {
-      mask_.reset(new OverviewContentMask());
-      mask_->layer()->SetFillsBoundsOpaquely(false);
-      window()->GetLayer()->SetMaskLayer(mask_->layer());
-    }
-    if (!determined_original_window_shape_) {
-      determined_original_window_shape_ = true;
-      SkRegion* window_shape = window()->GetLayer()->alpha_shape();
-      if (!original_window_shape_ && window_shape)
-        original_window_shape_.reset(new SkRegion(*window_shape));
-    }
-    gfx::Rect bounds(GetTargetBoundsInScreen().size());
-    const int inset = (use_mask || use_shape) ? GetTopInset() : 0;
-    if (mask_) {
-      // Mask layer is used both to hide the window header and to use rounded
-      // corners. Its layout needs to be update when setting a transform.
-      mask_->layer()->SetBounds(bounds);
-      mask_->set_inset(inset);
-      mask_->set_radius(radius);
-      window()->GetLayer()->SchedulePaint(bounds);
-    } else if (inset > 0) {
-      // Alpha shape is only used to to hide the window header and only when
-      // not using a mask layer.
-      bounds.Inset(0, inset, 0, 0);
-      SkRegion* region = new SkRegion;
-      region->setRect(RectToSkIRect(bounds));
-      if (original_window_shape_)
-        region->op(*original_window_shape_, SkRegion::kIntersect_Op);
-      window()->GetLayer()->SetAlphaShape(base::WrapUnique(region));
-      window()->SetMasksToBounds(true);
-    }
+  if (&transform != &original_transform_ &&
+      !determined_original_window_shape_) {
+    determined_original_window_shape_ = true;
+    SkRegion* window_shape = window()->GetLayer()->alpha_shape();
+    if (!original_window_shape_ && window_shape)
+      original_window_shape_.reset(new SkRegion(*window_shape));
   }
 
   gfx::Point target_origin(GetTargetBoundsInScreen().origin());
-
-  for (auto* window : GetTransientTreeIterator(window_)) {
+  for (auto* window : GetTransientTreeIterator(GetOverviewWindow())) {
     WmWindow* parent_window = window->GetParent();
     gfx::Point original_origin =
         parent_window->ConvertRectToScreen(window->GetTargetBounds()).origin();
@@ -506,14 +364,54 @@ void ScopedTransformOverviewWindow::SetTransform(
 }
 
 void ScopedTransformOverviewWindow::SetOpacity(float opacity) {
-  for (auto* window : GetTransientTreeIterator(window_)) {
+  for (auto* window : GetTransientTreeIterator(GetOverviewWindow())) {
     window->SetOpacity(opacity);
   }
 }
 
+void ScopedTransformOverviewWindow::HideHeader() {
+  // Mirrored Window does not have a header.
+  if (minimized_widget_)
+    return;
+  gfx::Rect bounds(GetTargetBoundsInScreen().size());
+  const int inset = GetTopInset();
+  if (inset > 0) {
+    // Use alpha shape to hide the window header.
+    bounds.Inset(0, inset, 0, 0);
+    std::unique_ptr<SkRegion> region(new SkRegion);
+    region->setRect(RectToSkIRect(bounds));
+    if (original_window_shape_)
+      region->op(*original_window_shape_, SkRegion::kIntersect_Op);
+    WmWindow* window = GetOverviewWindow();
+    window->GetLayer()->SetAlphaShape(std::move(region));
+    window->SetMasksToBounds(true);
+  }
+}
+
+void ScopedTransformOverviewWindow::ShowHeader() {
+  ui::Layer* layer = window()->GetLayer();
+  if (original_window_shape_) {
+    layer->SetAlphaShape(
+        base::MakeUnique<SkRegion>(*original_window_shape_.get()));
+  } else {
+    layer->SetAlphaShape(nullptr);
+  }
+  window()->SetMasksToBounds(false);
+}
+
+void ScopedTransformOverviewWindow::UpdateMirrorWindowForMinimizedState() {
+  // TODO(oshima): Disable animation.
+  if (window_->GetShowState() == ui::SHOW_STATE_MINIMIZED) {
+    if (!minimized_widget_)
+      CreateMirrorWindowForMinimizedState();
+  } else {
+    minimized_widget_->CloseNow();
+    minimized_widget_.reset();
+  }
+}
+
 void ScopedTransformOverviewWindow::Close() {
-  if (immediate_close_for_tests ||
-      !ash::MaterialDesignController::IsOverviewMaterial()) {
+  if (immediate_close_for_tests) {
     CloseWidget();
     return;
   }
@@ -527,7 +425,8 @@ void ScopedTransformOverviewWindow::PrepareForOverview() {
   DCHECK(!overview_started_);
   overview_started_ = true;
   window_->GetWindowState()->set_ignored_by_shelf(true);
-  ShowWindowIfMinimized();
+  if (window_->GetShowState() == ui::SHOW_STATE_MINIMIZED)
+    CreateMirrorWindowForMinimizedState();
 }
 
 void ScopedTransformOverviewWindow::CloseWidget() {
@@ -541,4 +440,70 @@ void ScopedTransformOverviewWindow::SetImmediateCloseForTests() {
   immediate_close_for_tests = true;
 }
 
+WmWindow* ScopedTransformOverviewWindow::GetOverviewWindow() const {
+  if (minimized_widget_)
+    return GetOverviewWindowForMinimizedState();
+  return window_;
+}
+
+void ScopedTransformOverviewWindow::EnsureVisible() {
+  original_opacity_ = 1.f;
+}
+
+void ScopedTransformOverviewWindow::OnGestureEvent(ui::GestureEvent* event) {
+  if (event->type() == ui::ET_GESTURE_TAP) {
+    EnsureVisible();
+    window_->Show();
+    window_->Activate();
+  }
+}
+
+void ScopedTransformOverviewWindow::OnMouseEvent(ui::MouseEvent* event) {
+  if (event->type() == ui::ET_MOUSE_PRESSED && event->IsOnlyLeftMouseButton()) {
+    EnsureVisible();
+    window_->Show();
+    window_->Activate();
+  }
+}
+
+WmWindow* ScopedTransformOverviewWindow::GetOverviewWindowForMinimizedState()
+    const {
+  return minimized_widget_
+             ? WmLookup::Get()->GetWindowForWidget(minimized_widget_.get())
+             : nullptr;
+}
+
+void ScopedTransformOverviewWindow::CreateMirrorWindowForMinimizedState() {
+  DCHECK(!minimized_widget_.get());
+  views::Widget::InitParams params;
+  params.type = views::Widget::InitParams::TYPE_WINDOW_FRAMELESS;
+  params.ownership = views::Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
+  params.visible_on_all_workspaces = true;
+  params.name = "OverviewModeMinimized";
+  params.activatable = views::Widget::InitParams::Activatable::ACTIVATABLE_NO;
+  params.accept_events = true;
+  minimized_widget_.reset(new views::Widget);
+  window_->GetRootWindow()
+      ->GetRootWindowController()
+      ->ConfigureWidgetInitParamsForContainer(
+          minimized_widget_.get(), window_->GetParent()->GetShellWindowId(),
+          &params);
+  minimized_widget_->set_focus_on_creation(false);
+  minimized_widget_->Init(params);
+
+  views::View* mirror_view = window_->CreateViewWithRecreatedLayers().release();
+  mirror_view->SetVisible(true);
+  mirror_view->SetTargetHandler(this);
+  minimized_widget_->SetContentsView(mirror_view);
+  gfx::Rect bounds(window_->GetBoundsInScreen());
+  gfx::Size preferred = mirror_view->GetPreferredSize();
+  // In unit tests, the content view can have empty size.
+  if (!preferred.IsEmpty()) {
+    int inset = bounds.height() - preferred.height();
+    bounds.Inset(0, 0, 0, inset);
+  }
+  minimized_widget_->SetBounds(bounds);
+  minimized_widget_->Show();
+}
+
 }  // namespace ash
