@@ -10,16 +10,16 @@
 
 #include "ash/common/shelf/wm_shelf.h"
 #include "ash/common/shelf/wm_shelf_util.h"
-#include "ash/common/shell_window_ids.h"
 #include "ash/common/wm/overview/window_selector_controller.h"
 #include "ash/common/wm/window_animation_types.h"
 #include "ash/common/wm/window_parenting_utils.h"
 #include "ash/common/wm/window_state.h"
 #include "ash/common/wm_lookup.h"
-#include "ash/common/wm_root_window_controller.h"
 #include "ash/common/wm_shell.h"
 #include "ash/common/wm_window.h"
 #include "ash/common/wm_window_property.h"
+#include "ash/public/cpp/shell_window_ids.h"
+#include "ash/root_window_controller.h"
 #include "base/auto_reset.h"
 #include "third_party/skia/include/core/SkColor.h"
 #include "third_party/skia/include/core/SkPaint.h"
@@ -181,7 +181,8 @@ class PanelCalloutWidget : public views::Widget {
   }
 
   void SetAlignment(ShelfAlignment alignment) {
-    gfx::Rect callout_bounds = GetWindowBoundsInScreen();
+    WmWindow* window = WmLookup::Get()->GetWindowForWidget(this);
+    gfx::Rect callout_bounds = window->GetBounds();
     if (IsHorizontalAlignment(alignment)) {
       callout_bounds.set_width(kArrowWidth);
       callout_bounds.set_height(kArrowHeight);
@@ -189,7 +190,12 @@ class PanelCalloutWidget : public views::Widget {
       callout_bounds.set_width(kArrowHeight);
       callout_bounds.set_height(kArrowWidth);
     }
-    SetBounds(callout_bounds);
+    WmWindow* parent = window->GetParent();
+    // It's important this go through WmWindow and not Widget. Going through
+    // Widget means it may move do a different screen, we don't want that.
+    window->SetBounds(callout_bounds);
+    // Setting the bounds should not trigger changing the parent.
+    DCHECK_EQ(parent, window->GetParent());
     if (background_->alignment() != alignment) {
       background_->set_alignment(alignment);
       SchedulePaintInRect(gfx::Rect(callout_bounds.size()));
@@ -334,7 +340,7 @@ void PanelLayoutManager::OnWindowAddedToLayout(WmWindow* child) {
   if (in_add_window_)
     return;
   base::AutoReset<bool> auto_reset_in_add_window(&in_add_window_, true);
-  if (!child->GetWindowState()->panel_attached()) {
+  if (!child->GetBoolProperty(WmWindowProperty::PANEL_ATTACHED)) {
     // This should only happen when a window is added to panel container as a
     // result of bounds change from within the application during a drag.
     // If so we have already stopped the drag and should reparent the panel
@@ -364,6 +370,7 @@ void PanelLayoutManager::OnWillRemoveWindowFromLayout(WmWindow* child) {}
 void PanelLayoutManager::OnWindowRemovedFromLayout(WmWindow* child) {
   if (child->GetType() == ui::wm::WINDOW_TYPE_POPUP)
     return;
+
   PanelList::iterator found =
       std::find(panel_windows_.begin(), panel_windows_.end(), child);
   if (found != panel_windows_.end()) {
@@ -432,7 +439,7 @@ void PanelLayoutManager::SetChildBounds(WmWindow* child,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// PanelLayoutManager, WmShellObserver implementation:
+// PanelLayoutManager, ShellObserver implementation:
 
 void PanelLayoutManager::OnOverviewModeEnded() {
   Relayout();
@@ -545,6 +552,11 @@ void PanelLayoutManager::OnShelfIconPositionsChanged() {
 // PanelLayoutManager private implementation:
 
 void PanelLayoutManager::MinimizePanel(WmWindow* panel) {
+  // Clusterfuzz can trigger panel accelerators before the shelf is created.
+  // TODO(jamescook): Revert this after http://crbug.com/648964 is fixed.
+  if (!shelf_)
+    return;
+
   panel->SetVisibilityAnimationType(
       wm::WINDOW_VISIBILITY_ANIMATION_TYPE_MINIMIZE);
   ui::Layer* layer = panel->GetLayer();
@@ -732,6 +744,11 @@ void PanelLayoutManager::Relayout() {
 }
 
 void PanelLayoutManager::UpdateStacking(WmWindow* active_panel) {
+  // Clusterfuzz can trigger panel accelerators before the shelf is created.
+  // TODO(jamescook): Revert this after http://crbug.com/648964 is fixed.
+  if (!shelf_)
+    return;
+
   if (!active_panel) {
     if (!last_active_panel_)
       return;
@@ -782,6 +799,11 @@ void PanelLayoutManager::UpdateStacking(WmWindow* active_panel) {
 }
 
 void PanelLayoutManager::UpdateCallouts() {
+  // Clusterfuzz can trigger panel accelerators before the shelf is created.
+  // TODO(jamescook): Revert this after http://crbug.com/648964 is fixed.
+  if (!shelf_)
+    return;
+
   const bool horizontal = IsHorizontalAlignment(shelf_->GetAlignment());
   for (PanelList::iterator iter = panel_windows_.begin();
        iter != panel_windows_.end(); ++iter) {
@@ -829,6 +851,8 @@ void PanelLayoutManager::UpdateCallouts() {
         callout_bounds);
 
     callout_widget_window->SetBoundsDirect(callout_bounds);
+    DCHECK_EQ(panel_container_, callout_widget_window->GetParent());
+    DCHECK_EQ(panel_container_, panel->GetParent());
     panel_container_->StackChildAbove(callout_widget_window, panel);
 
     ui::Layer* layer = callout_widget_window->GetLayer();
@@ -897,4 +921,6 @@ void PanelLayoutManager::OnKeyboardBoundsChanging(
   OnWindowResized();
 }
 
+void PanelLayoutManager::OnKeyboardClosed() {}
+
 }  // namespace ash
