@@ -5,8 +5,8 @@
 #include "ash/common/wm/root_window_finder.h"
 
 #include "ash/common/wm_lookup.h"
-#include "ash/common/wm_root_window_controller.h"
 #include "ash/common/wm_window.h"
+#include "ash/root_window_controller.h"
 #include "ui/display/display.h"
 #include "ui/display/screen.h"
 #include "ui/gfx/geometry/point.h"
@@ -19,7 +19,7 @@ WmWindow* GetRootWindowAt(const gfx::Point& point) {
   const display::Display& display =
       display::Screen::GetScreen()->GetDisplayNearestPoint(point);
   DCHECK(display.is_valid());
-  WmRootWindowController* root_window_controller =
+  RootWindowController* root_window_controller =
       WmLookup::Get()->GetRootWindowControllerWithDisplayId(display.id());
   return root_window_controller ? root_window_controller->GetWindow() : nullptr;
 }
@@ -27,7 +27,7 @@ WmWindow* GetRootWindowAt(const gfx::Point& point) {
 WmWindow* GetRootWindowMatching(const gfx::Rect& rect) {
   const display::Display& display =
       display::Screen::GetScreen()->GetDisplayMatching(rect);
-  WmRootWindowController* root_window_controller =
+  RootWindowController* root_window_controller =
       WmLookup::Get()->GetRootWindowControllerWithDisplayId(display.id());
   return root_window_controller ? root_window_controller->GetWindow() : nullptr;
 }
