@@ -4,8 +4,8 @@
 
 #include "ash/common/wm/switchable_windows.h"
 
-#include "ash/common/shell_window_ids.h"
 #include "ash/common/wm_window.h"
+#include "ash/public/cpp/shell_window_ids.h"
 
 namespace ash {
 namespace wm {
