@@ -4,9 +4,12 @@
 
 #include "ash/common/wm/wm_screen_util.h"
 
-#include "ash/common/wm_root_window_controller.h"
+#include "ash/common/wm_shell.h"
 #include "ash/common/wm_window.h"
+#include "ash/root_window_controller.h"
 #include "ui/display/display.h"
+#include "ui/display/screen.h"
+#include "ui/gfx/geometry/size_conversions.h"
 
 namespace ash {
 namespace wm {
@@ -32,5 +35,17 @@ gfx::Rect GetMaximizedWindowBoundsInParent(WmWindow* window) {
   return GetDisplayBoundsInParent(window);
 }
 
+gfx::Rect GetDisplayBoundsWithShelf(WmWindow* window) {
+  if (WmShell::Get()->IsInUnifiedMode()) {
+    // In unified desktop mode, there is only one shelf in the first display.
+    gfx::SizeF size(WmShell::Get()->GetFirstDisplay().size());
+    float scale = window->GetRootWindow()->GetBounds().height() / size.height();
+    size.Scale(scale, scale);
+    return gfx::Rect(gfx::ToCeiledSize(size));
+  }
+
+  return window->GetRootWindow()->GetBounds();
+}
+
 }  // namespace wm
 }  // namespace ash
