@@ -5,6 +5,9 @@
 #include "ash/common/wm/wm_snap_to_pixel_layout_manager.h"
 
 #include "ash/common/wm_window.h"
+#include "ash/common/wm_window_property.h"
+#include "ash/public/cpp/shell_window_ids.h"
+#include "base/memory/ptr_util.h"
 
 namespace ash {
 namespace wm {
@@ -13,6 +16,22 @@ WmSnapToPixelLayoutManager::WmSnapToPixelLayoutManager() {}
 
 WmSnapToPixelLayoutManager::~WmSnapToPixelLayoutManager() {}
 
+// static
+void WmSnapToPixelLayoutManager::InstallOnContainers(WmWindow* window) {
+  for (WmWindow* child : window->GetChildren()) {
+    if (child->GetShellWindowId() < kShellWindowId_Min ||
+        child->GetShellWindowId() > kShellWindowId_Max)  // not a container
+      continue;
+    if (child->GetBoolProperty(
+            WmWindowProperty::SNAP_CHILDREN_TO_PIXEL_BOUNDARY)) {
+      if (!child->GetLayoutManager())
+        child->SetLayoutManager(base::MakeUnique<WmSnapToPixelLayoutManager>());
+    } else {
+      InstallOnContainers(child);
+    }
+  }
+}
+
 void WmSnapToPixelLayoutManager::OnWindowResized() {}
 
 void WmSnapToPixelLayoutManager::OnWindowAddedToLayout(WmWindow* child) {}
