@@ -5,8 +5,8 @@
 #ifndef ASH_CONTENT_GPU_SUPPORT_IMPL_H_
 #define ASH_CONTENT_GPU_SUPPORT_IMPL_H_
 
+#include "ash/common/gpu_support.h"
 #include "ash/content/ash_with_content_export.h"
-#include "ash/gpu_support.h"
 #include "base/macros.h"
 
 namespace ash {
@@ -20,7 +20,6 @@ class ASH_WITH_CONTENT_EXPORT GPUSupportImpl : public GPUSupport {
  private:
   // Overridden from GPUSupport:
   bool IsPanelFittingDisabled() const override;
-  void DisableGpuWatchdog() override;
   void GetGpuProcessHandles(
       const GetGpuProcessHandlesCallback& callback) const override;
 
