@@ -10,7 +10,6 @@
 
 #include "ash/ash_export.h"
 #include "base/macros.h"
-#include "base/memory/linked_ptr.h"
 
 namespace ash {
 class ContainerHider;
@@ -30,7 +29,7 @@ class ASH_EXPORT DesktopCleaner {
   // Returns the list of containers that DesctopCleaner hides.
   static std::vector<int> GetContainersToHideForTest();
 
-  std::vector<linked_ptr<ContainerHider>> container_hiders_;
+  std::vector<std::unique_ptr<ContainerHider>> container_hiders_;
   std::unique_ptr<NotificationBlocker> notification_blocker_;
 
   friend class ash::test::FirstRunHelperTest;
