@@ -5,28 +5,14 @@
 #ifndef ASH_HOST_WINDOW_TREE_HOST_INIT_PARAMS_H_
 #define ASH_HOST_WINDOW_TREE_HOST_INIT_PARAMS_H_
 
-#include "build/build_config.h"
-
-#if defined(OS_WIN)
-#include <windows.h>
-#endif
-
 #include "ash/ash_export.h"
 #include "ui/gfx/geometry/rect.h"
 
 namespace ash {
 
 struct ASH_EXPORT AshWindowTreeHostInitParams {
-  AshWindowTreeHostInitParams();
-  ~AshWindowTreeHostInitParams();
-
   gfx::Rect initial_bounds;
-
-  bool offscreen;
-
-#if defined(OS_WIN)
-  HWND remote_hwnd;
-#endif
+  bool offscreen = false;
 };
 
 }  // namespace ash
