@@ -5,8 +5,10 @@
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/run_loop.h"
-#include "components/mus/public/interfaces/window_server_test.mojom.h"
-#include "services/shell/public/cpp/shell_test.h"
+#include "mash/quick_launch/public/interfaces/constants.mojom.h"
+#include "services/service_manager/public/cpp/service_test.h"
+#include "services/ui/public/interfaces/constants.mojom.h"
+#include "services/ui/public/interfaces/window_server_test.mojom.h"
 
 namespace ash {
 namespace mus {
@@ -16,31 +18,31 @@ void RunCallback(bool* success, const base::Closure& callback, bool result) {
   callback.Run();
 }
 
-class AppLaunchTest : public shell::test::ShellTest {
+class AppLaunchTest : public service_manager::test::ServiceTest {
  public:
-  AppLaunchTest() : ShellTest("exe:mash_unittests") {}
+  AppLaunchTest() : ServiceTest("mash_unittests") {}
   ~AppLaunchTest() override {}
 
  private:
   void SetUp() override {
     base::CommandLine::ForCurrentProcess()->AppendSwitch("use-test-config");
-    ShellTest::SetUp();
+    ServiceTest::SetUp();
   }
 
   DISALLOW_COPY_AND_ASSIGN(AppLaunchTest);
 };
 
 TEST_F(AppLaunchTest, TestQuickLaunch) {
-  connector()->Connect("mojo:ash");
-  connector()->Connect("mojo:quick_launch");
+  connector()->Connect("ash");
+  connector()->Connect(mash::quick_launch::mojom::kServiceName);
 
-  ::mus::mojom::WindowServerTestPtr test_interface;
-  connector()->ConnectToInterface("mojo:mus", &test_interface);
+  ui::mojom::WindowServerTestPtr test_interface;
+  connector()->BindInterface(ui::mojom::kServiceName, &test_interface);
 
   base::RunLoop run_loop;
   bool success = false;
   test_interface->EnsureClientHasDrawnWindow(
-      "mojo:quick_launch",
+      mash::quick_launch::mojom::kServiceName,
       base::Bind(&RunCallback, &success, run_loop.QuitClosure()));
   run_loop.Run();
   EXPECT_TRUE(success);
