@@ -4,9 +4,8 @@
 
 #include "ash/mus/bridge/wm_lookup_mus.h"
 
-#include "ash/mus/bridge/wm_root_window_controller_mus.h"
+#include "ash/common/wm_window.h"
 #include "ash/mus/bridge/wm_shell_mus.h"
-#include "ash/mus/bridge/wm_window_mus.h"
 #include "ui/views/widget/widget.h"
 
 namespace ash {
@@ -21,13 +20,13 @@ WmLookupMus::~WmLookupMus() {
     WmLookup::Set(nullptr);
 }
 
-WmRootWindowController* WmLookupMus::GetRootWindowControllerWithDisplayId(
+RootWindowController* WmLookupMus::GetRootWindowControllerWithDisplayId(
     int64_t id) {
   return WmShellMus::Get()->GetRootWindowControllerWithDisplayId(id);
 }
 
 WmWindow* WmLookupMus::GetWindowForWidget(views::Widget* widget) {
-  return WmWindowMus::Get(widget);
+  return WmWindow::Get(widget->GetNativeWindow());
 }
 
 }  // namespace mus
