@@ -10,60 +10,51 @@
 #include <vector>
 
 #include "ash/ash_export.h"
-#include "ash/common/shelf/shelf_types.h"
 #include "ash/metrics/user_metrics_recorder.h"
+#include "ash/public/cpp/shelf_types.h"
 #include "ash/wm/cursor_manager_chromeos.h"
 #include "ash/wm/system_modal_container_event_filter_delegate.h"
-#include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
 #include "ui/aura/window.h"
-#include "ui/base/ui_base_types.h"
 #include "ui/display/screen.h"
 #include "ui/events/event_target.h"
-#include "ui/gfx/geometry/insets.h"
-#include "ui/gfx/geometry/size.h"
 #include "ui/wm/core/cursor_manager.h"
-#include "ui/wm/public/activation_change_observer.h"
 
-namespace app_list {
-class AppListView;
-}
 namespace aura {
-class EventFilter;
 class RootWindow;
 class Window;
+class WindowManagerClient;
+class WindowTreeClient;
 namespace client {
 class ActivationClient;
 class FocusClient;
 }
 }
 
-namespace base {
-class SequencedWorkerPool;
-}
-
 namespace chromeos {
 class AudioA11yController;
 }
 
+namespace display {
+class DisplayChangeObserver;
+class DisplayConfigurator;
+class DisplayManager;
+}
+
 namespace gfx {
-class ImageSkia;
-class Point;
-class Rect;
+class Insets;
 }
 
 namespace ui {
-class DisplayConfigurator;
-class Layer;
 class UserActivityDetector;
 class UserActivityPowerManagerNotifier;
 }
+
 namespace views {
 class NonClientFrameView;
-class PointerWatcher;
 class Widget;
 namespace corewm {
 class TooltipController;
@@ -80,40 +71,28 @@ class WindowModalityController;
 
 namespace ash {
 
-class AcceleratorController;
-class AccessibilityDelegate;
-class AppListController;
+class AcceleratorControllerDelegateAura;
 class AshNativeCursorManager;
 class AutoclickController;
 class BluetoothNotificationController;
-class CaptureController;
-class DesktopBackgroundController;
-class DisplayChangeObserver;
 class DisplayColorManager;
 class DisplayConfigurationController;
-class WindowTreeHostManager;
 class DisplayErrorObserver;
-class DisplayManager;
 class DragDropController;
 class EventClientImpl;
-class EventRewriterEventFilter;
 class EventTransformationHandler;
 class FirstRunHelper;
 class GPUSupport;
 class HighContrastController;
-class KeyboardUMAEventFilter;
+class ImmersiveHandlerFactoryAsh;
+class LaserPointerController;
 class LinkHandlerModelFactory;
-class LocaleNotificationController;
 class LockStateController;
 enum class LoginStatus;
 class MagnificationController;
-class MaximizeModeController;
-class MaximizeModeWindowManager;
 class MouseCursorEventFilter;
-class NewWindowDelegate;
 class OverlayEventFilter;
 class PartialMagnificationController;
-class PointerWatcherDelegate;
 class PowerButtonController;
 class PowerEventObserver;
 class ProjectingObserver;
@@ -121,38 +100,27 @@ class ResizeShadowController;
 class ResolutionNotificationController;
 class RootWindowController;
 class ScopedOverviewAnimationSettingsFactoryAura;
-class ScopedTargetRootWindow;
-class ScreenAsh;
 class ScreenOrientationController;
 class ScreenshotController;
 class ScreenPinningController;
 class ScreenPositionController;
 class SessionStateDelegate;
-class Shelf;
-class ShelfDelegate;
-class ShelfItemDelegateManager;
-class ShelfModel;
-class ShelfWindowWatcher;
-class ShellDelegate;
 struct ShellInitParams;
-class SlowAnimationEventFilter;
-class StatusAreaWidget;
+class ShutdownObserver;
 class StickyKeysController;
 class SystemGestureEventFilter;
 class SystemModalContainerEventFilter;
 class SystemTray;
-class ToastManager;
 class ToplevelWindowEventHandler;
-class TouchTransformerController;
-class TouchObserverHUD;
-class UserWallpaperDelegate;
+class AshTouchTransformController;
+class ScreenLayoutObserver;
 class VirtualKeyboardController;
 class VideoActivityNotifier;
 class VideoDetector;
 class WebNotificationTray;
-class WindowCycleController;
 class WindowPositioner;
-class WmShellAura;
+class WindowTreeHostManager;
+class WmShell;
 class WmWindow;
 
 namespace shell {
@@ -169,8 +137,7 @@ class ShellTestApi;
 // Upon creation, the Shell sets itself as the RootWindow's delegate, which
 // takes ownership of the Shell.
 class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
-                         public ui::EventTarget,
-                         public aura::client::ActivationChangeObserver {
+                         public ui::EventTarget {
  public:
   typedef std::vector<RootWindowController*> RootWindowControllerList;
 
@@ -201,7 +168,7 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
 
   // Returns a root Window when used as a target when creating a new window.
   // The root window of the active window is used in most cases, but can
-  // be overridden by using ScopedTargetRootWindow().
+  // be overridden by using ScopedRootWindowForNewWindows.
   // If you want to get the root Window of the active window, just use
   // |wm::GetActiveWindow()->GetRootWindow()|.
   static aura::Window* GetTargetRootWindow();
@@ -218,58 +185,35 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   static const aura::Window* GetContainer(const aura::Window* root_window,
                                           int container_id);
 
-  // Returns the list of containers that match |container_id| in
-  // all root windows. If |priority_root| is given, the container
-  // in the |priority_root| will be inserted at the top of the list.
-  static std::vector<aura::Window*> GetContainersFromAllRootWindows(
-      int container_id,
-      aura::Window* priority_root);
-
-  void set_target_root_window(aura::Window* target_root_window) {
-    target_root_window_ = target_root_window;
+  // TODO(sky): move this and WindowManagerClient into ShellMash that is owned
+  // by Shell. Doing the move is gated on having mash create Shell.
+  static void set_window_tree_client(aura::WindowTreeClient* client) {
+    window_tree_client_ = client;
   }
 
-  // Shows the context menu for the background and launcher at
-  // |location_in_screen| (in screen coordinates).
-  void ShowContextMenu(const gfx::Point& location_in_screen,
-                       ui::MenuSourceType source_type);
+  static aura::WindowTreeClient* window_tree_client() {
+    return window_tree_client_;
+  }
 
-  // Shows the app list. |window| specifies in which display the app
-  // list should be shown. If this is NULL, the active root window
-  // will be used.
-  void ShowAppList(aura::Window* anchor);
-
-  // Dismisses the app list.
-  void DismissAppList();
-
-  // Shows the app list if it's not visible. Dismisses it otherwise.
-  void ToggleAppList(aura::Window* anchor);
-
-  // Returns app list target visibility.
-  bool GetAppListTargetVisibility() const;
+  static void set_window_manager_client(aura::WindowManagerClient* client) {
+    window_manager_client_ = client;
+  }
+  static aura::WindowManagerClient* window_manager_client() {
+    return window_manager_client_;
+  }
 
   // Creates a default views::NonClientFrameView for use by windows in the
   // Ash environment.
   views::NonClientFrameView* CreateDefaultNonClientFrameView(
       views::Widget* widget);
 
-  // Sets the work area insets of the display that contains |window|,
-  // this notifies observers too.
-  // TODO(sky): this no longer really replicates what happens and is unreliable.
-  // Remove this.
+  // Sets work area insets of the display containing |window|, pings observers.
   void SetDisplayWorkAreaInsets(aura::Window* window,
                                 const gfx::Insets& insets);
 
   // Called when the user logs in.
   void OnLoginStateChanged(LoginStatus status);
 
-  // Called after the logged-in user's profile is ready.
-  void OnLoginUserProfilePrepared();
-
-  // Called when the login status changes.
-  // TODO(oshima): Investigate if we can merge this and |OnLoginStateChanged|.
-  void UpdateAfterLoginStatusChange(LoginStatus status);
-
   // Called when the application is exiting.
   void OnAppTerminating();
 
@@ -280,22 +224,9 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // Called when a casting session is started or stopped.
   void OnCastingSessionStartedOrStopped(bool started);
 
-  // Called after maximize mode has started, windows might still animate though.
-  void OnMaximizeModeStarted();
-
-  // Called after maximize mode has ended, windows might still be returning to
-  // their original position.
-  void OnMaximizeModeEnded();
-
   // Called when a root window is created.
   void OnRootWindowAdded(WmWindow* root_window);
 
-  // Initializes |shelf_|.  Does nothing if it's already initialized.
-  void CreateShelf();
-
-  // Called when the shelf is created for |root_window|.
-  void OnShelfCreatedForRootWindow(WmWindow* root_window);
-
   // Creates a virtual keyboard. Deletes the old virtual keyboard if it already
   // exists.
   void CreateKeyboard();
@@ -303,25 +234,17 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // Deactivates the virtual keyboard.
   void DeactivateKeyboard();
 
-  // Show shelf view if it was created hidden (before session has started).
-  void ShowShelf();
-
-  void AddPointerWatcher(views::PointerWatcher* watcher);
-  void RemovePointerWatcher(views::PointerWatcher* watcher);
-
-#if defined(OS_CHROMEOS)
   // Test if MaximizeModeWindowManager is not enabled, and if
   // MaximizeModeController is not currently setting a display rotation. Or if
   // the |resolution_notification_controller_| is not showing its confirmation
   // dialog. If true then changes to display settings can be saved.
   bool ShouldSaveDisplaySettings();
-#endif
 
-  AcceleratorController* accelerator_controller() {
-    return accelerator_controller_.get();
+  AcceleratorControllerDelegateAura* accelerator_controller_delegate() {
+    return accelerator_controller_delegate_.get();
   }
 
-  DisplayManager* display_manager() { return display_manager_.get(); }
+  display::DisplayManager* display_manager() { return display_manager_.get(); }
   DisplayConfigurationController* display_configuration_controller() {
     return display_configuration_controller_.get();
   }
@@ -330,9 +253,6 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
     return tooltip_controller_.get();
   }
   OverlayEventFilter* overlay_filter() { return overlay_filter_.get(); }
-  DesktopBackgroundController* desktop_background_controller() {
-    return desktop_background_controller_.get();
-  }
   LinkHandlerModelFactory* link_handler_model_factory() {
     return link_handler_model_factory_;
   }
@@ -347,20 +267,21 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
     return lock_state_controller_.get();
   }
   VideoDetector* video_detector() { return video_detector_.get(); }
-  WindowCycleController* window_cycle_controller() {
-    return window_cycle_controller_.get();
-  }
   WindowTreeHostManager* window_tree_host_manager() {
     return window_tree_host_manager_.get();
   }
-#if defined(OS_CHROMEOS)
   PowerEventObserver* power_event_observer() {
     return power_event_observer_.get();
   }
-  TouchTransformerController* touch_transformer_controller() {
+  AshTouchTransformController* touch_transformer_controller() {
     return touch_transformer_controller_.get();
   }
-#endif  // defined(OS_CHROMEOS)
+  LaserPointerController* laser_pointer_controller() {
+    return laser_pointer_controller_.get();
+  }
+  PartialMagnificationController* partial_magnification_controller() {
+    return partial_magnification_controller_.get();
+  }
   ScreenshotController* screenshot_controller() {
     return screenshot_controller_.get();
   }
@@ -372,24 +293,10 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   }
   ::wm::CursorManager* cursor_manager() { return cursor_manager_.get(); }
 
-  ShellDelegate* delegate() { return delegate_.get(); }
-
-  UserWallpaperDelegate* user_wallpaper_delegate() {
-    return user_wallpaper_delegate_.get();
-  }
-
   SessionStateDelegate* session_state_delegate() {
     return session_state_delegate_.get();
   }
 
-  AccessibilityDelegate* accessibility_delegate() {
-    return accessibility_delegate_.get();
-  }
-
-  NewWindowDelegate* new_window_delegate() {
-    return new_window_delegate_.get();
-  }
-
   HighContrastController* high_contrast_controller() {
     return high_contrast_controller_.get();
   }
@@ -398,10 +305,6 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
     return magnification_controller_.get();
   }
 
-  PartialMagnificationController* partial_magnification_controller() {
-    return partial_magnification_controller_.get();
-  }
-
   AutoclickController* autoclick_controller() {
     return autoclick_controller_.get();
   }
@@ -410,38 +313,10 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
     return activation_client_;
   }
 
-  ShelfItemDelegateManager* shelf_item_delegate_manager() {
-    return shelf_item_delegate_manager_.get();
-  }
-
-  base::SequencedWorkerPool* blocking_pool() { return blocking_pool_; }
-
   // Force the shelf to query for it's current visibility state.
   // TODO(jamescook): Move to Shelf.
   void UpdateShelfVisibility();
 
-  // Called when the alignment for a shelf changes.
-  // TODO(jamescook): Move to Shelf.
-  void OnShelfAlignmentChanged(WmWindow* root_window);
-
-  // Called when the auto-hide behavior for a shelf changes.
-  // TODO(jamescook): Move to Shelf.
-  void OnShelfAutoHideBehaviorChanged(WmWindow* root_window);
-
-  // Notifies |observers_| when entering or exiting fullscreen mode in
-  // |root_window|.
-  // TODO(hidehiko): Rename this to NotifyFullscreenStateChanged.
-  void NotifyFullscreenStateChange(bool is_fullscreen, WmWindow* root_window);
-
-  // Creates a modal background (a partially-opaque fullscreen window)
-  // on all displays for |window|.
-  void CreateModalBackground(aura::Window* window);
-
-  // Called when a modal window is removed. It will activate
-  // another modal window if any, or remove modal screens
-  // on all displays.
-  void OnModalWindowRemoved(aura::Window* removed);
-
   // Returns WebNotificationTray on the primary root window.
   WebNotificationTray* GetWebNotificationTray();
 
@@ -467,19 +342,18 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // Starts the animation that occurs on first login.
   void DoInitialWorkspaceAnimation();
 
-  MaximizeModeController* maximize_mode_controller() {
-    return maximize_mode_controller_.get();
-  }
-
-#if defined(OS_CHROMEOS)
   // TODO(oshima): Move these objects to WindowTreeHostManager.
-  ui::DisplayConfigurator* display_configurator() {
+  display::DisplayConfigurator* display_configurator() {
     return display_configurator_.get();
   }
   DisplayErrorObserver* display_error_observer() {
     return display_error_observer_.get();
   }
 
+  ScreenLayoutObserver* screen_layout_observer() {
+    return screen_layout_observer_.get();
+  }
+
   ResolutionNotificationController* resolution_notification_controller() {
     return resolution_notification_controller_.get();
   }
@@ -495,15 +369,9 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   chromeos::AudioA11yController* audio_a11y_controller() {
     return audio_a11y_controller_.get();
   }
-#endif  // defined(OS_CHROMEOS)
-
-  ShelfModel* shelf_model() { return shelf_model_.get(); }
 
   WindowPositioner* window_positioner() { return window_positioner_.get(); }
 
-  // Returns the launcher delegate, creating if necesary.
-  ShelfDelegate* GetShelfDelegate();
-
   UserMetricsRecorder* metrics() { return user_metrics_recorder_.get(); }
 
   void SetTouchHudProjectionEnabled(bool enabled);
@@ -512,11 +380,6 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
     return is_touch_hud_projection_enabled_;
   }
 
-  // TODO(sky): remove this. This was needed by sysui, but as sysui is going
-  // away it should no longer be needed.
-  bool in_mus() const { return in_mus_; }
-
-#if defined(OS_CHROMEOS)
   // Creates instance of FirstRunHelper. Caller is responsible for deleting
   // returned object.
   ash::FirstRunHelper* CreateFirstRunHelper();
@@ -529,14 +392,10 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
     return sticky_keys_controller_.get();
   }
 
-#endif  // defined(OS_CHROMEOS)
-
   ScreenPinningController* screen_pinning_controller() {
     return screen_pinning_controller_.get();
   }
 
-  ToastManager* toast_manager() { return toast_manager_.get(); }
-
   GPUSupport* gpu_support() { return gpu_support_.get(); }
 
  private:
@@ -544,14 +403,10 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   FRIEND_TEST_ALL_PREFIXES(WindowManagerTest, MouseEventCursors);
   FRIEND_TEST_ALL_PREFIXES(WindowManagerTest, TransformActivate);
   friend class RootWindowController;
-  friend class ScopedTargetRootWindow;
   friend class test::ShellTestApi;
   friend class shell::WindowWatcher;
 
-  typedef std::pair<aura::Window*, gfx::Rect> WindowAndBoundsPair;
-
-  // Takes ownership of |delegate|.
-  Shell(ShellDelegate* delegate, base::SequencedWorkerPool* blocking_pool);
+  explicit Shell(std::unique_ptr<WmShell> wm_shell);
   ~Shell() override;
 
   void Init(const ShellInitParams& init_params);
@@ -562,10 +417,10 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // Initializes the root window so that it can host browser windows.
   void InitRootWindow(aura::Window* root_window);
 
-  // Prepares the shelf to be deleted.
-  void ShutdownShelf();
+  // Destroys all child windows including widgets across all roots.
+  void CloseAllRootWindowChildWindows();
 
-  // ash::SystemModalContainerEventFilterDelegate overrides:
+  // SystemModalContainerEventFilterDelegate:
   bool CanWindowReceiveEvents(aura::Window* window) override;
 
   // Overridden from ui::EventTarget:
@@ -574,47 +429,27 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   std::unique_ptr<ui::EventTargetIterator> GetChildIterator() const override;
   ui::EventTargeter* GetEventTargeter() override;
 
-  // Overridden from aura::client::ActivationChangeObserver:
-  void OnWindowActivated(
-      aura::client::ActivationChangeObserver::ActivationReason reason,
-      aura::Window* gained_active,
-      aura::Window* lost_active) override;
-
   static Shell* instance_;
 
+  // Only valid in mash, for classic ash this is null.
+  static aura::WindowTreeClient* window_tree_client_;
+  static aura::WindowManagerClient* window_manager_client_;
+
   // If set before the Shell is initialized, the mouse cursor will be hidden
   // when the screen is initially created.
   static bool initially_hide_cursor_;
 
   std::unique_ptr<ScopedOverviewAnimationSettingsFactoryAura>
       scoped_overview_animation_settings_factory_;
-  std::unique_ptr<WmShellAura> wm_shell_;
-
-  // When no explicit target display/RootWindow is given, new windows are
-  // created on |scoped_target_root_window_| , unless NULL in
-  // which case they are created on |target_root_window_|.
-  // |target_root_window_| never becomes NULL during the session.
-  aura::Window* target_root_window_;
-  aura::Window* scoped_target_root_window_;
+  std::unique_ptr<WmShell> wm_shell_;
 
   // The CompoundEventFilter owned by aura::Env object.
   std::unique_ptr<::wm::CompoundEventFilter> env_filter_;
 
-  std::vector<WindowAndBoundsPair> to_restore_;
-
   std::unique_ptr<UserMetricsRecorder> user_metrics_recorder_;
-  std::unique_ptr<AcceleratorController> accelerator_controller_;
-  std::unique_ptr<ShellDelegate> delegate_;
-  std::unique_ptr<UserWallpaperDelegate> user_wallpaper_delegate_;
+  std::unique_ptr<AcceleratorControllerDelegateAura>
+      accelerator_controller_delegate_;
   std::unique_ptr<SessionStateDelegate> session_state_delegate_;
-  std::unique_ptr<AccessibilityDelegate> accessibility_delegate_;
-  std::unique_ptr<NewWindowDelegate> new_window_delegate_;
-  std::unique_ptr<PointerWatcherDelegate> pointer_watcher_delegate_;
-  std::unique_ptr<ShelfDelegate> shelf_delegate_;
-  std::unique_ptr<ShelfItemDelegateManager> shelf_item_delegate_manager_;
-  std::unique_ptr<ShelfWindowWatcher> shelf_window_watcher_;
-
-  std::unique_ptr<ShelfModel> shelf_model_;
   std::unique_ptr<WindowPositioner> window_positioner_;
 
   std::unique_ptr<DragDropController> drag_drop_controller_;
@@ -623,18 +458,14 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   std::unique_ptr<::wm::VisibilityController> visibility_controller_;
   std::unique_ptr<::wm::WindowModalityController> window_modality_controller_;
   std::unique_ptr<views::corewm::TooltipController> tooltip_controller_;
-  std::unique_ptr<DesktopBackgroundController> desktop_background_controller_;
   LinkHandlerModelFactory* link_handler_model_factory_;
   std::unique_ptr<PowerButtonController> power_button_controller_;
   std::unique_ptr<LockStateController> lock_state_controller_;
   std::unique_ptr<ui::UserActivityDetector> user_activity_detector_;
   std::unique_ptr<VideoDetector> video_detector_;
-  std::unique_ptr<WindowCycleController> window_cycle_controller_;
   std::unique_ptr<WindowTreeHostManager> window_tree_host_manager_;
   std::unique_ptr<HighContrastController> high_contrast_controller_;
   std::unique_ptr<MagnificationController> magnification_controller_;
-  std::unique_ptr<PartialMagnificationController>
-      partial_magnification_controller_;
   std::unique_ptr<AutoclickController> autoclick_controller_;
   std::unique_ptr<aura::client::FocusClient> focus_client_;
 
@@ -652,9 +483,6 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // screenshot UI or the keyboard overlay is active.
   std::unique_ptr<OverlayEventFilter> overlay_filter_;
 
-  // An event filter for logging keyboard-related metrics.
-  std::unique_ptr<KeyboardUMAEventFilter> keyboard_metrics_filter_;
-
   // An event filter which handles moving and resizing windows.
   std::unique_ptr<ToplevelWindowEventHandler> toplevel_window_event_handler_;
 
@@ -664,14 +492,12 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // An event filter that pre-handles global accelerators.
   std::unique_ptr<::wm::AcceleratorFilter> accelerator_filter_;
 
-  std::unique_ptr<DisplayManager> display_manager_;
+  std::unique_ptr<display::DisplayManager> display_manager_;
   std::unique_ptr<DisplayConfigurationController>
       display_configuration_controller_;
 
-  std::unique_ptr<LocaleNotificationController> locale_notification_controller_;
   std::unique_ptr<ScreenPinningController> screen_pinning_controller_;
 
-#if defined(OS_CHROMEOS)
   std::unique_ptr<PowerEventObserver> power_event_observer_;
   std::unique_ptr<ui::UserActivityPowerManagerNotifier> user_activity_notifier_;
   std::unique_ptr<VideoActivityNotifier> video_activity_notifier_;
@@ -683,25 +509,28 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   std::unique_ptr<VirtualKeyboardController> virtual_keyboard_controller_;
   std::unique_ptr<chromeos::AudioA11yController> audio_a11y_controller_;
   // Controls video output device state.
-  std::unique_ptr<ui::DisplayConfigurator> display_configurator_;
+  std::unique_ptr<display::DisplayConfigurator> display_configurator_;
   std::unique_ptr<DisplayColorManager> display_color_manager_;
   std::unique_ptr<DisplayErrorObserver> display_error_observer_;
   std::unique_ptr<ProjectingObserver> projecting_observer_;
 
   // Listens for output changes and updates the display manager.
-  std::unique_ptr<DisplayChangeObserver> display_change_observer_;
+  std::unique_ptr<display::DisplayChangeObserver> display_change_observer_;
+
+  // Listens for shutdown and updates DisplayConfigurator.
+  std::unique_ptr<ShutdownObserver> shutdown_observer_;
 
   // Implements content::ScreenOrientationController for ChromeOS
   std::unique_ptr<ScreenOrientationController> screen_orientation_controller_;
+  std::unique_ptr<ScreenLayoutObserver> screen_layout_observer_;
 
-  std::unique_ptr<TouchTransformerController> touch_transformer_controller_;
+  std::unique_ptr<AshTouchTransformController> touch_transformer_controller_;
 
   std::unique_ptr<ui::EventHandler> magnifier_key_scroll_handler_;
   std::unique_ptr<ui::EventHandler> speech_feedback_handler_;
-#endif  // defined(OS_CHROMEOS)
-
-  std::unique_ptr<ToastManager> toast_manager_;
-  std::unique_ptr<MaximizeModeController> maximize_mode_controller_;
+  std::unique_ptr<LaserPointerController> laser_pointer_controller_;
+  std::unique_ptr<PartialMagnificationController>
+      partial_magnification_controller_;
 
   // |native_cursor_manager_| is owned by |cursor_manager_|, but we keep a
   // pointer to vend to test code.
@@ -719,9 +548,7 @@ class ASH_EXPORT Shell : public SystemModalContainerEventFilterDelegate,
   // Injected content::GPUDataManager support.
   std::unique_ptr<GPUSupport> gpu_support_;
 
-  base::SequencedWorkerPool* blocking_pool_;
-
-  bool in_mus_ = false;
+  std::unique_ptr<ImmersiveHandlerFactoryAsh> immersive_handler_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(Shell);
 };
