@@ -7,18 +7,17 @@
 #include <algorithm>
 #include <vector>
 
-#include "ash/common/ash_switches.h"
 #include "ash/common/session/session_state_delegate.h"
-#include "ash/common/shell_window_ids.h"
+#include "ash/common/shelf/shelf_layout_manager.h"
+#include "ash/common/shelf/shelf_widget.h"
+#include "ash/common/shelf/wm_shelf.h"
+#include "ash/common/wallpaper/wallpaper_widget_controller.h"
 #include "ash/common/wm_shell.h"
-#include "ash/desktop_background/desktop_background_widget_controller.h"
+#include "ash/common/wm_window.h"
 #include "ash/display/mouse_cursor_event_filter.h"
 #include "ash/drag_drop/drag_drop_controller.h"
+#include "ash/public/cpp/shell_window_ids.h"
 #include "ash/root_window_controller.h"
-#include "ash/shelf/shelf.h"
-#include "ash/shelf/shelf_layout_manager.h"
-#include "ash/shelf/shelf_widget.h"
-#include "ash/shell_delegate.h"
 #include "ash/test/ash_test_base.h"
 #include "ash/test/shell_test_api.h"
 #include "ash/wm/window_util.h"
@@ -58,8 +57,8 @@ aura::Window* GetAlwaysOnTopContainer() {
 // Expect ALL the containers!
 void ExpectAllContainers() {
   aura::Window* root_window = Shell::GetPrimaryRootWindow();
-  EXPECT_TRUE(Shell::GetContainer(root_window,
-                                  kShellWindowId_DesktopBackgroundContainer));
+  EXPECT_TRUE(
+      Shell::GetContainer(root_window, kShellWindowId_WallpaperContainer));
   EXPECT_TRUE(
       Shell::GetContainer(root_window, kShellWindowId_DefaultContainer));
   EXPECT_TRUE(
@@ -68,8 +67,8 @@ void ExpectAllContainers() {
   EXPECT_TRUE(Shell::GetContainer(root_window, kShellWindowId_ShelfContainer));
   EXPECT_TRUE(
       Shell::GetContainer(root_window, kShellWindowId_SystemModalContainer));
-  EXPECT_TRUE(Shell::GetContainer(
-      root_window, kShellWindowId_LockScreenBackgroundContainer));
+  EXPECT_TRUE(Shell::GetContainer(root_window,
+                                  kShellWindowId_LockScreenWallpaperContainer));
   EXPECT_TRUE(
       Shell::GetContainer(root_window, kShellWindowId_LockScreenContainer));
   EXPECT_TRUE(Shell::GetContainer(root_window,
@@ -84,10 +83,8 @@ void ExpectAllContainers() {
       Shell::GetContainer(root_window, kShellWindowId_OverlayContainer));
   EXPECT_TRUE(Shell::GetContainer(root_window,
                                   kShellWindowId_ImeWindowParentContainer));
-#if defined(OS_CHROMEOS)
   EXPECT_TRUE(
       Shell::GetContainer(root_window, kShellWindowId_MouseCursorContainer));
-#endif
 }
 
 class ModalWindow : public views::WidgetDelegateView {
@@ -96,7 +93,6 @@ class ModalWindow : public views::WidgetDelegateView {
   ~ModalWindow() override {}
 
   // Overridden from views::WidgetDelegate:
-  views::View* GetContentsView() override { return this; }
   bool CanResize() const override { return true; }
   base::string16 GetWindowTitle() const override {
     return base::ASCIIToUTF16("Modal Window");
@@ -116,11 +112,6 @@ class SimpleMenuDelegate : public ui::SimpleMenuModel::Delegate {
 
   bool IsCommandIdEnabled(int command_id) const override { return true; }
 
-  bool GetAcceleratorForCommandId(int command_id,
-                                  ui::Accelerator* accelerator) override {
-    return false;
-  }
-
   void ExecuteCommand(int command_id, int event_flags) override {}
 
  private:
@@ -352,8 +343,11 @@ TEST_F(ShellTest, LockScreenClosesActiveMenu) {
   std::unique_ptr<ui::SimpleMenuModel> menu_model(
       new ui::SimpleMenuModel(&menu_delegate));
   menu_model->AddItem(0, base::ASCIIToUTF16("Menu item"));
-  views::Widget* widget =
-      Shell::GetPrimaryRootWindowController()->wallpaper_controller()->widget();
+  views::Widget* widget = WmShell::Get()
+                              ->GetPrimaryRootWindow()
+                              ->GetRootWindowController()
+                              ->wallpaper_widget_controller()
+                              ->widget();
   std::unique_ptr<views::MenuRunner> menu_runner(
       new views::MenuRunner(menu_model.get(), views::MenuRunner::CONTEXT_MENU));
 
@@ -371,26 +365,24 @@ TEST_F(ShellTest, LockScreenClosesActiveMenu) {
 }
 
 TEST_F(ShellTest, ManagedWindowModeBasics) {
-  if (!SupportsHostWindowResize())
-    return;
-
   // We start with the usual window containers.
   ExpectAllContainers();
   // Shelf is visible.
-  ShelfWidget* shelf_widget = Shelf::ForPrimaryDisplay()->shelf_widget();
+  ShelfWidget* shelf_widget = GetPrimaryShelf()->shelf_widget();
   EXPECT_TRUE(shelf_widget->IsVisible());
   // Shelf is at bottom-left of screen.
   EXPECT_EQ(0, shelf_widget->GetWindowBoundsInScreen().x());
-  EXPECT_EQ(Shell::GetPrimaryRootWindow()->GetHost()->GetBounds().height(),
-            shelf_widget->GetWindowBoundsInScreen().bottom());
-  // We have a desktop background but not a bare layer.
+  EXPECT_EQ(
+      Shell::GetPrimaryRootWindow()->GetHost()->GetBoundsInPixels().height(),
+      shelf_widget->GetWindowBoundsInScreen().bottom());
+  // We have a wallpaper but not a bare layer.
   // TODO (antrim): enable once we find out why it fails component build.
-  //  DesktopBackgroundWidgetController* background =
+  //  WallpaperWidgetController* wallpaper =
   //      Shell::GetPrimaryRootWindow()->
   //          GetProperty(kWindowDesktopComponent);
-  //  EXPECT_TRUE(background);
-  //  EXPECT_TRUE(background->widget());
-  //  EXPECT_FALSE(background->layer());
+  //  EXPECT_TRUE(wallpaper);
+  //  EXPECT_TRUE(wallpaper->widget());
+  //  EXPECT_FALSE(wallpaper->layer());
 
   // Create a normal window.  It is not maximized.
   views::Widget::InitParams widget_params(
@@ -447,7 +439,7 @@ TEST_F(ShellTest, ToggleAutoHide) {
   window->Show();
   wm::ActivateWindow(window.get());
 
-  Shelf* shelf = Shelf::ForPrimaryDisplay();
+  WmShelf* shelf = GetPrimaryShelf();
   shelf->SetAutoHideBehavior(SHELF_AUTO_HIDE_BEHAVIOR_ALWAYS);
   EXPECT_EQ(SHELF_AUTO_HIDE_BEHAVIOR_ALWAYS, shelf->auto_hide_behavior());
 
