@@ -2,10 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "ash/test/ash_test_helper.h"
 #include "ash/test/ash_test_views_delegate.h"
 
 #include "ash/shell.h"
-#include "content/public/test/web_contents_tester.h"
 
 namespace ash {
 namespace test {
@@ -14,13 +14,6 @@ AshTestViewsDelegate::AshTestViewsDelegate() {}
 
 AshTestViewsDelegate::~AshTestViewsDelegate() {}
 
-content::WebContents* AshTestViewsDelegate::CreateWebContents(
-    content::BrowserContext* browser_context,
-    content::SiteInstance* site_instance) {
-  return content::WebContentsTester::CreateTestWebContents(browser_context,
-                                                           site_instance);
-}
-
 void AshTestViewsDelegate::OnBeforeWidgetInit(
     views::Widget::InitParams* params,
     views::internal::NativeWidgetDelegate* delegate) {
@@ -32,5 +25,15 @@ void AshTestViewsDelegate::OnBeforeWidgetInit(
   }
 }
 
+void AshTestViewsDelegate::NotifyAccessibilityEvent(views::View* view,
+                                                    ui::AXEvent event_type) {
+  TestViewsDelegate::NotifyAccessibilityEvent(view, event_type);
+
+  if (test_accessibility_event_delegate_) {
+    test_accessibility_event_delegate_->NotifyAccessibilityEvent(view,
+                                                                 event_type);
+  }
+}
+
 }  // namespace test
 }  // namespace ash
