@@ -5,15 +5,20 @@
 #ifndef ASH_WM_OVERVIEW_SCOPED_OVERVIEW_ANIMATION_SETTINGS_AURA_H_
 #define ASH_WM_OVERVIEW_SCOPED_OVERVIEW_ANIMATION_SETTINGS_AURA_H_
 
+#include <memory>
+
 #include "ash/common/wm/overview/overview_animation_type.h"
 #include "ash/common/wm/overview/scoped_overview_animation_settings.h"
 #include "base/macros.h"
-#include "ui/compositor/scoped_layer_animation_settings.h"
 
 namespace aura {
 class Window;
 }  // namespace aura
 
+namespace ui {
+class ScopedLayerAnimationSettings;
+}  // namespace ui
+
 namespace ash {
 
 // ScopedOverviewAnimationSettingsfor aura.
@@ -23,10 +28,11 @@ class ScopedOverviewAnimationSettingsAura
   ScopedOverviewAnimationSettingsAura(OverviewAnimationType animation_type,
                                       aura::Window* window);
   ~ScopedOverviewAnimationSettingsAura() override;
+  void AddObserver(ui::ImplicitAnimationObserver* observer) override;
 
  private:
   // The managed animation settings.
-  ui::ScopedLayerAnimationSettings animation_settings_;
+  std::unique_ptr<ui::ScopedLayerAnimationSettings> animation_settings_;
 
   DISALLOW_COPY_AND_ASSIGN(ScopedOverviewAnimationSettingsAura);
 };
