@@ -3,7 +3,7 @@
 // found in the LICENSE file.
 
 #include "ash/wm/system_modal_container_event_filter.h"
-
+#include "ash/common/wm_shell.h"
 #include "ash/wm/system_modal_container_event_filter_delegate.h"
 #include "ui/aura/window.h"
 #include "ui/events/event.h"
@@ -17,6 +17,9 @@ SystemModalContainerEventFilter::SystemModalContainerEventFilter(
 SystemModalContainerEventFilter::~SystemModalContainerEventFilter() {}
 
 void SystemModalContainerEventFilter::OnEvent(ui::Event* event) {
+  // Only filter modal events if a modal window is open.
+  if (!WmShell::Get()->IsSystemModalWindowOpen())
+    return;
   aura::Window* target = static_cast<aura::Window*>(event->target());
   if (!delegate_->CanWindowReceiveEvents(target))
     event->StopPropagation();
