@@ -2,27 +2,28 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "ash/wm/window_cycle_controller.h"
+#include "ash/common/wm/window_cycle_controller.h"
 
 #include <algorithm>
 #include <memory>
 
-#include "ash/aura/wm_window_aura.h"
 #include "ash/common/focus_cycler.h"
+#include "ash/common/scoped_root_window_for_new_windows.h"
 #include "ash/common/session/session_state_delegate.h"
-#include "ash/common/shell_window_ids.h"
+#include "ash/common/shelf/shelf_widget.h"
+#include "ash/common/shelf/wm_shelf.h"
+#include "ash/common/test/test_shelf_delegate.h"
+#include "ash/common/wm/window_cycle_list.h"
 #include "ash/common/wm/window_state.h"
 #include "ash/common/wm/wm_event.h"
 #include "ash/common/wm_shell.h"
-#include "ash/shelf/shelf.h"
-#include "ash/shelf/shelf_widget.h"
+#include "ash/common/wm_window.h"
+#include "ash/public/cpp/shell_window_ids.h"
 #include "ash/shell.h"
 #include "ash/test/ash_test_base.h"
-#include "ash/test/shelf_test_api.h"
 #include "ash/test/shelf_view_test_api.h"
-#include "ash/test/test_shelf_delegate.h"
+#include "ash/test/test_app_list_view_presenter_impl.h"
 #include "ash/test/test_shell_delegate.h"
-#include "ash/wm/window_cycle_list.h"
 #include "ash/wm/window_state_aura.h"
 #include "ash/wm/window_util.h"
 #include "ui/aura/client/aura_constants.h"
@@ -31,6 +32,10 @@
 #include "ui/aura/test/test_windows.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_event_dispatcher.h"
+#include "ui/display/display_layout_builder.h"
+#include "ui/display/manager/display_layout_store.h"
+#include "ui/display/manager/display_manager.h"
+#include "ui/display/test/display_manager_test_api.h"
 #include "ui/events/event_handler.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/gfx/geometry/rect.h"
@@ -39,28 +44,36 @@ namespace ash {
 
 namespace {
 
-class KeyEventCounter : public ui::EventHandler {
+class EventCounter : public ui::EventHandler {
  public:
-  KeyEventCounter() : key_events_(0) {}
-  ~KeyEventCounter() override {}
+  EventCounter() : key_events_(0), mouse_events_(0) {}
+  ~EventCounter() override {}
 
-  size_t GetCountAndReset() {
-    size_t count = key_events_;
+  int GetKeyEventCountAndReset() {
+    int count = key_events_;
     key_events_ = 0;
     return count;
   }
 
+  int GetMouseEventCountAndReset() {
+    int count = mouse_events_;
+    mouse_events_ = 0;
+    return count;
+  }
+
   // ui::EventHandler:
   void OnKeyEvent(ui::KeyEvent* event) override { key_events_++; }
+  void OnMouseEvent(ui::MouseEvent* event) override { mouse_events_++; }
 
  private:
-  size_t key_events_;
+  int key_events_;
+  int mouse_events_;
 
-  DISALLOW_COPY_AND_ASSIGN(KeyEventCounter);
+  DISALLOW_COPY_AND_ASSIGN(EventCounter);
 };
 
 bool IsWindowMinimized(aura::Window* window) {
-  return WmWindowAura::Get(window)->GetWindowState()->IsMinimized();
+  return WmWindow::Get(window)->GetWindowState()->IsMinimized();
 }
 
 }  // namespace
@@ -76,10 +89,11 @@ class WindowCycleControllerTest : public test::AshTestBase {
 
   void SetUp() override {
     test::AshTestBase::SetUp();
-    ASSERT_TRUE(test::TestShelfDelegate::instance());
+
+    WindowCycleList::DisableInitialDelayForTesting();
 
     shelf_view_test_.reset(new test::ShelfViewTestAPI(
-        test::ShelfTestAPI(Shelf::ForPrimaryDisplay()).shelf_view()));
+        GetPrimaryShelf()->GetShelfViewForTesting()));
     shelf_view_test_->SetAnimationDuration(1);
   }
 
@@ -87,14 +101,20 @@ class WindowCycleControllerTest : public test::AshTestBase {
     gfx::Rect rect(100, 100);
     aura::Window* window = CreateTestWindowInShellWithDelegateAndType(
         NULL, ui::wm::WINDOW_TYPE_PANEL, 0, rect);
-    test::TestShelfDelegate::instance()->AddShelfItem(window);
+    test::TestShelfDelegate::instance()->AddShelfItem(WmWindow::Get(window));
     shelf_view_test_->RunMessageLoopUntilAnimationsDone();
     return window;
   }
 
   const aura::Window::Windows GetWindows(WindowCycleController* controller) {
-    return WmWindowAura::ToAuraWindows(
-        controller->window_cycle_list()->windows());
+    return WmWindow::ToAuraWindows(controller->window_cycle_list()->windows());
+  }
+
+  const views::Widget* GetWindowCycleListWidget() const {
+    return WmShell::Get()
+        ->window_cycle_controller()
+        ->window_cycle_list()
+        ->widget();
   }
 
  private:
@@ -104,8 +124,7 @@ class WindowCycleControllerTest : public test::AshTestBase {
 };
 
 TEST_F(WindowCycleControllerTest, HandleCycleWindowBaseCases) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Cycling doesn't crash if there are no windows.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
@@ -123,8 +142,7 @@ TEST_F(WindowCycleControllerTest, HandleCycleWindowBaseCases) {
 // Verifies if there is only one window and it isn't active that cycling
 // activates it.
 TEST_F(WindowCycleControllerTest, SingleWindowNotActive) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Create a single test window.
   std::unique_ptr<Window> window0(CreateTestWindowInShellWithId(0));
@@ -142,8 +160,7 @@ TEST_F(WindowCycleControllerTest, SingleWindowNotActive) {
 }
 
 TEST_F(WindowCycleControllerTest, HandleCycleWindow) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Set up several windows to use to test cycling.  Create them in reverse
   // order so they are stacked 0 over 1 over 2.
@@ -163,30 +180,32 @@ TEST_F(WindowCycleControllerTest, HandleCycleWindow) {
   ASSERT_EQ(window1.get(), GetWindows(controller)[1]);
   ASSERT_EQ(window2.get(), GetWindows(controller)[2]);
 
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
 
   // Pressing and releasing Alt-tab again should cycle back to the most-
   // recently-used window in the current child order.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
+  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+
+  // Cancelled cycling shouldn't move the active window.
+  controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CancelCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
 
   // Pressing Alt-tab multiple times without releasing Alt should cycle through
   // all the windows and wrap around.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
   EXPECT_TRUE(controller->IsCycling());
-  EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
 
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
   EXPECT_TRUE(controller->IsCycling());
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
 
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
   EXPECT_TRUE(controller->IsCycling());
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
 
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_FALSE(controller->IsCycling());
   EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
 
@@ -195,29 +214,29 @@ TEST_F(WindowCycleControllerTest, HandleCycleWindow) {
   wm::ActivateWindow(window1.get());
   wm::ActivateWindow(window0.get());
 
-  // Likewise we can cycle backwards through all the windows.
+  // Likewise we can cycle backwards through the windows.
   controller->HandleCycleWindow(WindowCycleController::BACKWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
   controller->HandleCycleWindow(WindowCycleController::BACKWARD);
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
-  controller->HandleCycleWindow(WindowCycleController::BACKWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-  controller->StopCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+
+  // Reset our stacking order.
+  wm::ActivateWindow(window2.get());
+  wm::ActivateWindow(window1.get());
+  wm::ActivateWindow(window0.get());
 
   // When the screen is locked, cycling window does not take effect.
   WmShell::Get()->GetSessionStateDelegate()->LockScreen();
   EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-  controller->HandleCycleWindow(WindowCycleController::BACKWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+  EXPECT_FALSE(controller->IsCycling());
 
+  // Unlock, it works again.
   WmShell::Get()->GetSessionStateDelegate()->UnlockScreen();
   EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
 
   // When a modal window is active, cycling window does not take effect.
@@ -230,11 +249,13 @@ TEST_F(WindowCycleControllerTest, HandleCycleWindow) {
   EXPECT_TRUE(wm::IsActiveWindow(modal_window.get()));
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
   EXPECT_TRUE(wm::IsActiveWindow(modal_window.get()));
+  EXPECT_FALSE(controller->IsCycling());
   EXPECT_FALSE(wm::IsActiveWindow(window0.get()));
   EXPECT_FALSE(wm::IsActiveWindow(window1.get()));
   EXPECT_FALSE(wm::IsActiveWindow(window2.get()));
   controller->HandleCycleWindow(WindowCycleController::BACKWARD);
   EXPECT_TRUE(wm::IsActiveWindow(modal_window.get()));
+  EXPECT_FALSE(controller->IsCycling());
   EXPECT_FALSE(wm::IsActiveWindow(window0.get()));
   EXPECT_FALSE(wm::IsActiveWindow(window1.get()));
   EXPECT_FALSE(wm::IsActiveWindow(window2.get()));
@@ -251,13 +272,15 @@ TEST_F(WindowCycleControllerTest, MaximizedWindow) {
   EXPECT_TRUE(window1_state->IsActive());
 
   // Rotate focus, this should move focus to window0.
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::GetWindowState(window0.get())->IsActive());
+  EXPECT_FALSE(window1_state->IsActive());
 
   // One more time.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CompleteCycling();
   EXPECT_TRUE(window1_state->IsActive());
 }
 
@@ -274,20 +297,50 @@ TEST_F(WindowCycleControllerTest, Minimized) {
   EXPECT_TRUE(window0_state->IsActive());
 
   // Rotate focus, this should move focus to window1 and unminimize it.
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CompleteCycling();
+  EXPECT_FALSE(window0_state->IsActive());
   EXPECT_FALSE(window1_state->IsMinimized());
   EXPECT_TRUE(window1_state->IsActive());
 
   // One more time back to w0.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CompleteCycling();
   EXPECT_TRUE(window0_state->IsActive());
 }
 
+// Tests that when all windows are minimized, cycling starts with the first one
+// rather than the second.
+TEST_F(WindowCycleControllerTest, AllAreMinimized) {
+  // Create a couple of test windows.
+  std::unique_ptr<Window> window0(CreateTestWindowInShellWithId(0));
+  std::unique_ptr<Window> window1(CreateTestWindowInShellWithId(1));
+  wm::WindowState* window0_state = wm::GetWindowState(window0.get());
+  wm::WindowState* window1_state = wm::GetWindowState(window1.get());
+
+  window0_state->Minimize();
+  window1_state->Minimize();
+
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
+  controller->HandleCycleWindow(WindowCycleController::FORWARD);
+  controller->CompleteCycling();
+  EXPECT_TRUE(window0_state->IsActive());
+  EXPECT_FALSE(window0_state->IsMinimized());
+  EXPECT_TRUE(window1_state->IsMinimized());
+
+  // But it's business as usual when cycling backwards.
+  window0_state->Minimize();
+  window1_state->Minimize();
+  controller->HandleCycleWindow(WindowCycleController::BACKWARD);
+  controller->CompleteCycling();
+  EXPECT_TRUE(window0_state->IsMinimized());
+  EXPECT_TRUE(window1_state->IsActive());
+  EXPECT_FALSE(window1_state->IsMinimized());
+}
+
 TEST_F(WindowCycleControllerTest, AlwaysOnTopWindow) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Set up several windows to use to test cycling.
   std::unique_ptr<Window> window0(CreateTestWindowInShellWithId(0));
@@ -309,27 +362,11 @@ TEST_F(WindowCycleControllerTest, AlwaysOnTopWindow) {
   EXPECT_EQ(window2.get(), GetWindows(controller)[1]);
   EXPECT_EQ(window1.get(), GetWindows(controller)[2]);
 
-  controller->StopCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-
-  controller->StopCycling();
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+  controller->CompleteCycling();
 }
 
 TEST_F(WindowCycleControllerTest, AlwaysOnTopMultiWindow) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Set up several windows to use to test cycling.
   std::unique_ptr<Window> window0(CreateTestWindowInShellWithId(0));
@@ -353,25 +390,7 @@ TEST_F(WindowCycleControllerTest, AlwaysOnTopMultiWindow) {
   EXPECT_EQ(window2.get(), GetWindows(controller)[2]);
   EXPECT_EQ(window1.get(), GetWindows(controller)[3]);
 
-  controller->StopCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(window3.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-
-  controller->StopCycling();
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window3.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+  controller->CompleteCycling();
 }
 
 TEST_F(WindowCycleControllerTest, AlwaysOnTopMultipleRootWindows) {
@@ -383,10 +402,7 @@ TEST_F(WindowCycleControllerTest, AlwaysOnTopMultipleRootWindows) {
   aura::Window::Windows root_windows = Shell::GetAllRootWindows();
   ASSERT_EQ(2U, root_windows.size());
 
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
-
-  Shell::GetInstance()->set_target_root_window(root_windows[0]);
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Create two windows in the primary root.
   std::unique_ptr<Window> window0(CreateTestWindowInShellWithId(0));
@@ -396,8 +412,9 @@ TEST_F(WindowCycleControllerTest, AlwaysOnTopMultipleRootWindows) {
   std::unique_ptr<Window> window1(CreateTestWindowWithId(1, top_container0));
   EXPECT_EQ(root_windows[0], window1->GetRootWindow());
 
-  // And two on the secondary root.
-  Shell::GetInstance()->set_target_root_window(root_windows[1]);
+  // Move the active root window to the secondary root and create two windows.
+  ScopedRootWindowForNewWindows root_for_new_windows(
+      WmWindow::Get(root_windows[1]));
   std::unique_ptr<Window> window2(CreateTestWindowInShellWithId(2));
   EXPECT_EQ(root_windows[1], window2->GetRootWindow());
 
@@ -406,9 +423,6 @@ TEST_F(WindowCycleControllerTest, AlwaysOnTopMultipleRootWindows) {
   std::unique_ptr<Window> window3(CreateTestWindowWithId(3, top_container1));
   EXPECT_EQ(root_windows[1], window3->GetRootWindow());
 
-  // Move the active root window to the secondary.
-  Shell::GetInstance()->set_target_root_window(root_windows[1]);
-
   wm::ActivateWindow(window2.get());
 
   EXPECT_EQ(root_windows[0], window0->GetRootWindow());
@@ -428,30 +442,11 @@ TEST_F(WindowCycleControllerTest, AlwaysOnTopMultipleRootWindows) {
   EXPECT_EQ(window1.get(), GetWindows(controller)[2]);
   EXPECT_EQ(window0.get(), GetWindows(controller)[3]);
 
-  controller->StopCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(window3.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
-
-  controller->StopCycling();
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window3.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
+  controller->CompleteCycling();
 }
 
 TEST_F(WindowCycleControllerTest, MostRecentlyUsed) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   // Set up several windows to use to test cycling.
   std::unique_ptr<Window> window0(CreateTestWindowInShellWithId(0));
@@ -471,43 +466,46 @@ TEST_F(WindowCycleControllerTest, MostRecentlyUsed) {
   EXPECT_EQ(window2.get(), GetWindows(controller)[1]);
   EXPECT_EQ(window1.get(), GetWindows(controller)[2]);
 
+  // Cycling through then stopping the cycling will activate a window.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
 
+  // Cycling alone (without CompleteCycling()) doesn't activate.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+  EXPECT_FALSE(wm::IsActiveWindow(window0.get()));
 
-  controller->StopCycling();
+  // Showing the Alt+Tab UI does however deactivate the erstwhile active window.
+  EXPECT_FALSE(wm::IsActiveWindow(window1.get()));
 
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window1.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
+  controller->CompleteCycling();
 }
 
 // Tests that beginning window selection hides the app list.
 TEST_F(WindowCycleControllerTest, SelectingHidesAppList) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  // The tested behavior relies on the app list presenter implementation.
+  test::TestAppListViewPresenterImpl app_list_presenter_impl;
+
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
   std::unique_ptr<aura::Window> window1(CreateTestWindowInShellWithId(1));
-  Shell::GetInstance()->ShowAppList(NULL);
-  EXPECT_TRUE(Shell::GetInstance()->GetAppListTargetVisibility());
+  app_list_presenter_impl.Show(display_manager()->first_display_id());
+  EXPECT_TRUE(app_list_presenter_impl.IsVisible());
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_FALSE(Shell::GetInstance()->GetAppListTargetVisibility());
+  EXPECT_FALSE(app_list_presenter_impl.IsVisible());
+
+  // Make sure that dismissing the app list this way doesn't pass activation
+  // to a different window.
+  EXPECT_FALSE(wm::IsActiveWindow(window0.get()));
+  EXPECT_FALSE(wm::IsActiveWindow(window1.get()));
+
+  controller->CompleteCycling();
 }
 
-// Tests that cycling through windows shows and minimizes windows as they
-// are passed.
+// Tests that cycling through windows doesn't change their minimized state.
 TEST_F(WindowCycleControllerTest, CyclePreservesMinimization) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
   std::unique_ptr<aura::Window> window1(CreateTestWindowInShellWithId(1));
@@ -518,21 +516,20 @@ TEST_F(WindowCycleControllerTest, CyclePreservesMinimization) {
 
   // On window 2.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  EXPECT_FALSE(IsWindowMinimized(window1.get()));
+  EXPECT_TRUE(IsWindowMinimized(window1.get()));
 
   // Back on window 1.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
   EXPECT_TRUE(IsWindowMinimized(window1.get()));
 
-  controller->StopCycling();
+  controller->CompleteCycling();
 
   EXPECT_TRUE(IsWindowMinimized(window1.get()));
 }
 
 // Tests cycles between panel and normal windows.
 TEST_F(WindowCycleControllerTest, CyclePanels) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
   std::unique_ptr<aura::Window> panel0(CreatePanelWindow());
@@ -543,25 +540,24 @@ TEST_F(WindowCycleControllerTest, CyclePanels) {
   EXPECT_TRUE(wm::IsActiveWindow(panel0.get()));
 
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(panel1.get()));
 
   // Cycling again should select the most recently used panel.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(panel0.get()));
 
   // Cycling twice again should select the first window.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
 }
 
 // Tests cycles between panel and normal windows.
 TEST_F(WindowCycleControllerTest, CyclePanelsDestroyed) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
   std::unique_ptr<aura::Window> window1(CreateTestWindowInShellWithId(1));
@@ -582,14 +578,13 @@ TEST_F(WindowCycleControllerTest, CyclePanelsDestroyed) {
   panel1.reset();
   // Cycling again should now select window2.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
 }
 
 // Tests cycles between panel and normal windows.
 TEST_F(WindowCycleControllerTest, CycleMruPanelDestroyed) {
-  WindowCycleController* controller =
-      Shell::GetInstance()->window_cycle_controller();
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
 
   std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
   std::unique_ptr<aura::Window> window1(CreateTestWindowInShellWithId(1));
@@ -609,7 +604,7 @@ TEST_F(WindowCycleControllerTest, CycleMruPanelDestroyed) {
   panel0.reset();
   // Cycling again should now select panel1.
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->StopCycling();
+  controller->CompleteCycling();
   EXPECT_TRUE(wm::IsActiveWindow(panel1.get()));
 }
 
@@ -617,20 +612,71 @@ TEST_F(WindowCycleControllerTest, CycleMruPanelDestroyed) {
 TEST_F(WindowCycleControllerTest, TabKeyNotLeaked) {
   std::unique_ptr<Window> w0(CreateTestWindowInShellWithId(0));
   std::unique_ptr<Window> w1(CreateTestWindowInShellWithId(1));
-  KeyEventCounter key_count;
-  w0->AddPreTargetHandler(&key_count);
-  w1->AddPreTargetHandler(&key_count);
+  EventCounter event_count;
+  w0->AddPreTargetHandler(&event_count);
+  w1->AddPreTargetHandler(&event_count);
   ui::test::EventGenerator& generator = GetEventGenerator();
   wm::GetWindowState(w0.get())->Activate();
   generator.PressKey(ui::VKEY_MENU, ui::EF_NONE);
-  EXPECT_EQ(1u, key_count.GetCountAndReset());
+  EXPECT_EQ(1, event_count.GetKeyEventCountAndReset());
   generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
-  EXPECT_EQ(0u, key_count.GetCountAndReset());
+  EXPECT_EQ(0, event_count.GetKeyEventCountAndReset());
   generator.ReleaseKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
-  EXPECT_EQ(0u, key_count.GetCountAndReset());
+  EXPECT_EQ(0, event_count.GetKeyEventCountAndReset());
   generator.ReleaseKey(ui::VKEY_MENU, ui::EF_NONE);
   EXPECT_TRUE(wm::GetWindowState(w1.get())->IsActive());
-  EXPECT_EQ(0u, key_count.GetCountAndReset());
+  EXPECT_EQ(0, event_count.GetKeyEventCountAndReset());
+}
+
+// While the UI is active, mouse events are captured.
+TEST_F(WindowCycleControllerTest, MouseEventsCaptured) {
+  // This delegate allows the window to receive mouse events.
+  aura::test::TestWindowDelegate delegate;
+  std::unique_ptr<Window> w0(CreateTestWindowInShellWithDelegate(
+      &delegate, 0, gfx::Rect(0, 0, 100, 100)));
+  std::unique_ptr<Window> w1(CreateTestWindowInShellWithId(1));
+  EventCounter event_count;
+  w0->AddPreTargetHandler(&event_count);
+  w1->SetTargetHandler(&event_count);
+  ui::test::EventGenerator& generator = GetEventGenerator();
+  wm::ActivateWindow(w0.get());
+
+  // Events get through.
+  generator.MoveMouseToCenterOf(w0.get());
+  generator.ClickLeftButton();
+  EXPECT_LT(0, event_count.GetMouseEventCountAndReset());
+
+  // Start cycling.
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
+  controller->HandleCycleWindow(WindowCycleController::FORWARD);
+
+  // Events don't get through.
+  generator.ClickLeftButton();
+  EXPECT_EQ(0, event_count.GetMouseEventCountAndReset());
+
+  // Stop cycling: once again, events get through.
+  controller->CompleteCycling();
+  generator.ClickLeftButton();
+  EXPECT_LT(0, event_count.GetMouseEventCountAndReset());
+}
+
+// If mouse capture is lost, the UI closes.
+TEST_F(WindowCycleControllerTest, MouseCaptureLost) {
+  // This delegate allows the window to receive mouse events.
+  aura::test::TestWindowDelegate delegate;
+  std::unique_ptr<Window> w0(CreateTestWindowInShellWithDelegate(
+      &delegate, 0, gfx::Rect(0, 0, 100, 100)));
+  std::unique_ptr<Window> w1(CreateTestWindowInShellWithId(1));
+
+  // Start cycling.
+  WindowCycleController* controller = WmShell::Get()->window_cycle_controller();
+  controller->HandleCycleWindow(WindowCycleController::FORWARD);
+
+  // Some other widget grabs capture and this causes Alt+Tab to cease.
+  std::unique_ptr<views::Widget> widget = CreateTestWidget(
+      nullptr, kShellWindowId_DefaultContainer, gfx::Rect(1, 2, 3, 4));
+  widget->SetCapture(nullptr);
+  EXPECT_FALSE(controller->IsCycling());
 }
 
 // Tests that we can cycle past fullscreen windows: https://crbug.com/622396.
@@ -663,11 +709,65 @@ TEST_F(WindowCycleControllerTest, TabPastFullscreenWindow) {
   // pass on the alt+tab to continue cycling). To make this test work with or
   // without the new alt+tab selector we check for the event on either
   // fullscreen window.
-  KeyEventCounter key_count;
-  w0->AddPreTargetHandler(&key_count);
-  w1->AddPreTargetHandler(&key_count);
+  EventCounter event_count;
+  w0->AddPreTargetHandler(&event_count);
+  w1->AddPreTargetHandler(&event_count);
   generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
-  EXPECT_EQ(1u, key_count.GetCountAndReset());
+  EXPECT_EQ(1, event_count.GetKeyEventCountAndReset());
+}
+
+// Tests that the Alt+Tab UI's position isn't affected by the origin of the
+// display it's on. See crbug.com/675718
+TEST_F(WindowCycleControllerTest, MultiDisplayPositioning) {
+  int64_t primary_id = display::Screen::GetScreen()->GetPrimaryDisplay().id();
+  display::DisplayIdList list =
+      display::test::CreateDisplayIdListN(2, primary_id, primary_id + 1);
+
+  auto placements = {
+      display::DisplayPlacement::BOTTOM, display::DisplayPlacement::TOP,
+      display::DisplayPlacement::LEFT, display::DisplayPlacement::RIGHT,
+  };
+
+  gfx::Rect expected_bounds;
+  for (auto placement : placements) {
+    SCOPED_TRACE(placement);
+
+    display::DisplayLayoutBuilder builder(primary_id);
+    builder.AddDisplayPlacement(list[1], primary_id, placement, 0);
+    display_manager()->layout_store()->RegisterLayoutForDisplayIdList(
+        list, builder.Build());
+
+    // Use two displays.
+    UpdateDisplay("500x500,600x600");
+
+    gfx::Rect second_display_bounds =
+        display_manager()->GetDisplayAt(1).bounds();
+    std::unique_ptr<Window> window0(
+        CreateTestWindowInShellWithBounds(second_display_bounds));
+    // Activate this window so that the secondary display becomes the one where
+    // the Alt+Tab UI is shown.
+    wm::ActivateWindow(window0.get());
+    std::unique_ptr<Window> window1(
+        CreateTestWindowInShellWithBounds(second_display_bounds));
+
+    WindowCycleController* controller =
+        WmShell::Get()->window_cycle_controller();
+    controller->HandleCycleWindow(WindowCycleController::FORWARD);
+
+    const gfx::Rect bounds =
+        GetWindowCycleListWidget()->GetWindowBoundsInScreen();
+    EXPECT_TRUE(second_display_bounds.Contains(bounds));
+    EXPECT_FALSE(
+        display_manager()->GetDisplayAt(0).bounds().Intersects(bounds));
+    const gfx::Rect display_relative_bounds =
+        bounds - second_display_bounds.OffsetFromOrigin();
+    // Base case sets the expectation for other cases.
+    if (expected_bounds.IsEmpty())
+      expected_bounds = display_relative_bounds;
+    else
+      EXPECT_EQ(expected_bounds, display_relative_bounds);
+    controller->CompleteCycling();
+  }
 }
 
 }  // namespace ash
