@@ -5,7 +5,7 @@
 #ifndef ASH_WM_WORKSPACE_CONTROLLER_TEST_HELPER_H_
 #define ASH_WM_WORKSPACE_CONTROLLER_TEST_HELPER_H_
 
-#include "ash/wm/workspace_controller.h"
+#include "ash/common/wm/workspace_controller.h"
 #include "base/macros.h"
 
 namespace ash {
