@@ -4,6 +4,8 @@
 
 package org.chromium.base;
 
+import android.annotation.SuppressLint;
+
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.security.SecureRandom;
@@ -13,30 +15,25 @@ import java.security.SecureRandom;
  * <= 4.3. See
  * {@link http://android-developers.blogspot.com/2013/08/some-securerandom-thoughts.html}.
  */
+// TODO(crbug.com/635567): Fix this properly.
+@SuppressLint("SecureRandom")
 public class SecureRandomInitializer {
     private static final int NUM_RANDOM_BYTES = 16;
 
-    private static byte[] sSeedBytes = new byte[NUM_RANDOM_BYTES];
-
     /**
      * Safely initializes the random number generator, by seeding it with data from /dev/urandom.
      */
     public static void initialize(SecureRandom generator) throws IOException {
         FileInputStream fis = null;
         try {
+            byte[] seedBytes = new byte[NUM_RANDOM_BYTES];
             fis = new FileInputStream("/dev/urandom");
-            if (fis.read(sSeedBytes) != sSeedBytes.length) {
+            if (fis.read(seedBytes) != seedBytes.length) {
                 throw new IOException("Failed to get enough random data.");
             }
-            generator.setSeed(sSeedBytes);
+            generator.setSeed(seedBytes);
         } finally {
-            try {
-                if (fis != null) {
-                    fis.close();
-                }
-            } catch (IOException e) {
-                // Ignore exception closing the device.
-            }
+            StreamUtil.closeQuietly(fis);
         }
     }
 }
