@@ -40,8 +40,8 @@ class TestOptions(object):
     self.ptr_type = 'long'
     self.cpp = 'cpp'
     self.javap = 'javap'
-    self.native_exports = False
-    self.native_exports_optional = False
+    self.native_exports_optional = True
+    self.enable_profiling = False
 
 class TestGenerator(unittest.TestCase):
   def assertObjEquals(self, first, second):
@@ -403,6 +403,10 @@ class TestGenerator(unittest.TestCase):
           return
       }
     }
+    @CalledByNative
+    public static @Status int updateStatus(@Status int status) {
+        return getAndUpdateStatus(status);
+    }
     @CalledByNativeUnchecked
     private void uncheckedCall(int iParam);
 
@@ -524,6 +528,17 @@ class TestGenerator(unittest.TestCase):
             env_call=('Void', ''),
             unchecked=False,
         ),
+        CalledByNative(
+          return_type='int',
+          system_class=False,
+          static=True,
+          name='updateStatus',
+          method_id_var_name='updateStatus',
+          java_class_name='',
+          params=[Param(datatype='int', name='status')],
+          env_call=('Integer', ''),
+          unchecked=False,
+        ),
         CalledByNative(
             return_type='void',
             system_class=False,
@@ -819,7 +834,7 @@ public class java.util.HashSet {
     content = file(os.path.join(script_dir,
         'java/src/org/chromium/example/jni_generator/SampleForTests.java')
         ).read()
-    golden_file = os.path.join(script_dir, 'golden_sample_for_tests_jni.h')
+    golden_file = os.path.join(script_dir, 'SampleForTests_jni.golden')
     golden_content = file(golden_file).read()
     jni_from_java = jni_generator.JNIFromJavaSource(
         content, 'org/chromium/example/jni_generator/SampleForTests',
@@ -935,7 +950,7 @@ class Foo {
                                              natives, [], [], test_options)
     self.assertGoldenTextEquals(h.GetContent())
 
-  def runNativeExportsOption(self, optional):
+  def testNativeExportsOnlyOption(self):
     test_data = """
     package org.chromium.example.jni_generator;
 
@@ -967,19 +982,10 @@ class Foo {
     }
     """
     options = TestOptions()
-    options.native_exports = True
-    options.native_exports_optional = optional
+    options.native_exports_optional = False
     jni_from_java = jni_generator.JNIFromJavaSource(
         test_data, 'org/chromium/example/jni_generator/SampleForTests', options)
-    return jni_from_java.GetContent()
-
-  def testNativeExportsOption(self):
-    content = self.runNativeExportsOption(False)
-    self.assertGoldenTextEquals(content)
-
-  def testNativeExportsOptionalOption(self):
-    content = self.runNativeExportsOption(True)
-    self.assertGoldenTextEquals(content)
+    self.assertGoldenTextEquals(jni_from_java.GetContent())
 
   def testOuterInnerRaises(self):
     test_data = """
@@ -1041,7 +1047,7 @@ class Foo {
 def TouchStamp(stamp_path):
   dir_name = os.path.dirname(stamp_path)
   if not os.path.isdir(dir_name):
-    os.makedirs()
+    os.makedirs(dir_name)
 
   with open(stamp_path, 'a'):
     os.utime(stamp_path, None)
