@@ -4,6 +4,9 @@
 
 #include "base/test/icu_test_util.h"
 
+#include "base/base_switches.h"
+#include "base/command_line.h"
+#include "base/i18n/icu_util.h"
 #include "base/i18n/rtl.h"
 #include "third_party/icu/source/common/unicode/uloc.h"
 
@@ -17,5 +20,13 @@ ScopedRestoreICUDefaultLocale::~ScopedRestoreICUDefaultLocale() {
   i18n::SetICUDefaultLocale(default_locale_.data());
 }
 
+void InitializeICUForTesting() {
+  if (!base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kTestDoNotInitializeIcu)) {
+    base::i18n::AllowMultipleInitializeCallsForTesting();
+    i18n::InitializeICU();
+  }
+}
+
 }  // namespace test
 }  // namespace base
