@@ -43,31 +43,31 @@ AnimationHost::AnimationHost(ThreadInstance thread_instance)
     : mutator_host_client_(nullptr),
       thread_instance_(thread_instance),
       supports_scroll_animations_(false),
-      animation_waiting_for_deletion_(false) {
+      needs_push_properties_(false) {
   if (thread_instance_ == ThreadInstance::IMPL) {
     scroll_offset_animations_impl_ =
-        base::WrapUnique(new ScrollOffsetAnimationsImpl(this));
+        base::MakeUnique<ScrollOffsetAnimationsImpl>(this);
   } else {
-    scroll_offset_animations_ =
-        base::WrapUnique(new ScrollOffsetAnimations(this));
+    scroll_offset_animations_ = base::MakeUnique<ScrollOffsetAnimations>(this);
   }
 }
 
 AnimationHost::~AnimationHost() {
   scroll_offset_animations_impl_ = nullptr;
 
-  ClearTimelines();
+  ClearMutators();
   DCHECK(!mutator_host_client());
   DCHECK(element_to_animations_map_.empty());
 }
 
-std::unique_ptr<AnimationHost> AnimationHost::CreateImplInstance(
+std::unique_ptr<MutatorHost> AnimationHost::CreateImplInstance(
     bool supports_impl_scrolling) const {
   DCHECK_EQ(thread_instance_, ThreadInstance::MAIN);
-  auto animation_host_impl =
-      base::WrapUnique(new AnimationHost(ThreadInstance::IMPL));
-  animation_host_impl->SetSupportsScrollAnimations(supports_impl_scrolling);
-  return animation_host_impl;
+
+  auto mutator_host_impl =
+      base::WrapUnique<MutatorHost>(new AnimationHost(ThreadInstance::IMPL));
+  mutator_host_impl->SetSupportsScrollAnimations(supports_impl_scrolling);
+  return mutator_host_impl;
 }
 
 AnimationTimeline* AnimationHost::GetTimelineById(int timeline_id) const {
@@ -75,7 +75,7 @@ AnimationTimeline* AnimationHost::GetTimelineById(int timeline_id) const {
   return f == id_to_timeline_map_.end() ? nullptr : f->second.get();
 }
 
-void AnimationHost::ClearTimelines() {
+void AnimationHost::ClearMutators() {
   for (auto& kv : id_to_timeline_map_)
     EraseTimeline(kv.second);
   id_to_timeline_map_.clear();
@@ -92,6 +92,7 @@ void AnimationHost::AddAnimationTimeline(
   timeline->SetAnimationHost(this);
   id_to_timeline_map_.insert(
       std::make_pair(timeline->id(), std::move(timeline)));
+  SetNeedsPushProperties();
 }
 
 void AnimationHost::RemoveAnimationTimeline(
@@ -99,6 +100,7 @@ void AnimationHost::RemoveAnimationTimeline(
   DCHECK(timeline->id());
   EraseTimeline(timeline);
   id_to_timeline_map_.erase(timeline->id());
+  SetNeedsPushProperties();
 }
 
 void AnimationHost::RegisterElement(ElementId element_id,
@@ -127,7 +129,8 @@ void AnimationHost::RegisterPlayerForElement(ElementId element_id,
   if (!element_animations) {
     element_animations = ElementAnimations::Create();
     element_animations->SetElementId(element_id);
-    RegisterElementAnimations(element_animations.get());
+    element_to_animations_map_[element_animations->element_id()] =
+        element_animations;
   }
 
   if (element_animations->animation_host() != this) {
@@ -150,9 +153,11 @@ void AnimationHost::UnregisterPlayerForElement(ElementId element_id,
 
   if (element_animations->IsEmpty()) {
     element_animations->ClearAffectedElementTypes();
-    UnregisterElementAnimations(element_animations.get());
+    element_to_animations_map_.erase(element_animations->element_id());
     element_animations->SetAnimationHost(nullptr);
   }
+
+  RemoveFromTicking(player);
 }
 
 void AnimationHost::SetMutatorHostClient(MutatorHostClient* client) {
@@ -160,23 +165,33 @@ void AnimationHost::SetMutatorHostClient(MutatorHostClient* client) {
     return;
 
   mutator_host_client_ = client;
+
+  if (needs_push_properties() && mutator_host_client())
+    SetNeedsPushProperties();
 }
 
 void AnimationHost::SetNeedsCommit() {
   DCHECK(mutator_host_client_);
   mutator_host_client_->SetMutatorsNeedCommit();
-}
-
-void AnimationHost::SetNeedsRebuildPropertyTrees() {
-  DCHECK(mutator_host_client_);
+  // TODO(loyso): Invalidate property trees only if really needed.
   mutator_host_client_->SetMutatorsNeedRebuildPropertyTrees();
 }
 
-void AnimationHost::PushPropertiesTo(AnimationHost* host_impl) {
-  PushTimelinesToImplThread(host_impl);
-  RemoveTimelinesFromImplThread(host_impl);
-  PushPropertiesToImplThread(host_impl);
-  animation_waiting_for_deletion_ = false;
+void AnimationHost::SetNeedsPushProperties() {
+  needs_push_properties_ = true;
+}
+
+void AnimationHost::PushPropertiesTo(MutatorHost* mutator_host_impl) {
+  auto host_impl = static_cast<AnimationHost*>(mutator_host_impl);
+
+  if (needs_push_properties_) {
+    needs_push_properties_ = false;
+    PushTimelinesToImplThread(host_impl);
+    RemoveTimelinesFromImplThread(host_impl);
+    PushPropertiesToImplThread(host_impl);
+    // This is redundant but used in tests.
+    host_impl->needs_push_properties_ = false;
+  }
 }
 
 void AnimationHost::PushTimelinesToImplThread(AnimationHost* host_impl) const {
@@ -213,19 +228,24 @@ void AnimationHost::PushPropertiesToImplThread(AnimationHost* host_impl) {
   // to happen before the element animations are synced below.
   for (auto& kv : id_to_timeline_map_) {
     AnimationTimeline* timeline = kv.second.get();
-    AnimationTimeline* timeline_impl =
-        host_impl->GetTimelineById(timeline->id());
-    if (timeline_impl)
-      timeline->PushPropertiesTo(timeline_impl);
+    if (timeline->needs_push_properties()) {
+      AnimationTimeline* timeline_impl =
+          host_impl->GetTimelineById(timeline->id());
+      if (timeline_impl)
+        timeline->PushPropertiesTo(timeline_impl);
+    }
   }
 
   // Sync properties for created ElementAnimations.
   for (auto& kv : element_to_animations_map_) {
     const auto& element_animations = kv.second;
-    auto element_animations_impl =
-        host_impl->GetElementAnimationsForElementId(kv.first);
-    if (element_animations_impl)
-      element_animations->PushPropertiesTo(std::move(element_animations_impl));
+    if (element_animations->needs_push_properties()) {
+      auto element_animations_impl =
+          host_impl->GetElementAnimationsForElementId(kv.first);
+      if (element_animations_impl)
+        element_animations->PushPropertiesTo(
+            std::move(element_animations_impl));
+    }
   }
 
   // Update the impl-only scroll offset animations.
@@ -250,63 +270,64 @@ bool AnimationHost::SupportsScrollAnimations() const {
   return supports_scroll_animations_;
 }
 
-bool AnimationHost::NeedsAnimateLayers() const {
-  return !active_element_to_animations_map_.empty();
+bool AnimationHost::NeedsTickAnimations() const {
+  return !ticking_players_.empty();
 }
 
 bool AnimationHost::ActivateAnimations() {
-  if (!NeedsAnimateLayers())
+  if (!NeedsTickAnimations())
     return false;
 
   TRACE_EVENT0("cc", "AnimationHost::ActivateAnimations");
-  ElementToAnimationsMap active_element_animations_map_copy =
-      active_element_to_animations_map_;
-  for (auto& it : active_element_animations_map_copy)
-    it.second->ActivateAnimations();
+  PlayersList ticking_players_copy = ticking_players_;
+  for (auto& it : ticking_players_copy)
+    it->ActivateAnimations();
 
   return true;
 }
 
-bool AnimationHost::AnimateLayers(base::TimeTicks monotonic_time) {
-  if (!NeedsAnimateLayers())
+bool AnimationHost::TickAnimations(base::TimeTicks monotonic_time) {
+  if (!NeedsTickAnimations())
     return false;
 
-  TRACE_EVENT0("cc", "AnimationHost::AnimateLayers");
-  ElementToAnimationsMap active_element_animations_map_copy =
-      active_element_to_animations_map_;
-  for (auto& it : active_element_animations_map_copy)
-    it.second->Animate(monotonic_time);
+  TRACE_EVENT0("cc", "AnimationHost::TickAnimations");
+  PlayersList ticking_players_copy = ticking_players_;
+  for (auto& it : ticking_players_copy)
+    it->Tick(monotonic_time);
 
   return true;
 }
 
 bool AnimationHost::UpdateAnimationState(bool start_ready_animations,
-                                         AnimationEvents* events) {
-  if (!NeedsAnimateLayers())
+                                         MutatorEvents* mutator_events) {
+  if (!NeedsTickAnimations())
     return false;
 
+  auto animation_events = static_cast<AnimationEvents*>(mutator_events);
+
   TRACE_EVENT0("cc", "AnimationHost::UpdateAnimationState");
-  ElementToAnimationsMap active_element_animations_map_copy =
-      active_element_to_animations_map_;
-  for (auto& it : active_element_animations_map_copy)
-    it.second->UpdateState(start_ready_animations, events);
+  PlayersList ticking_players_copy = ticking_players_;
+  for (auto& it : ticking_players_copy)
+    it->UpdateState(start_ready_animations, animation_events);
 
   return true;
 }
 
-std::unique_ptr<AnimationEvents> AnimationHost::CreateEvents() {
-  return base::WrapUnique(new AnimationEvents());
+std::unique_ptr<MutatorEvents> AnimationHost::CreateEvents() {
+  return base::MakeUnique<AnimationEvents>();
 }
 
 void AnimationHost::SetAnimationEvents(
-    std::unique_ptr<AnimationEvents> events) {
+    std::unique_ptr<MutatorEvents> mutator_events) {
+  auto events =
+      base::WrapUnique(static_cast<AnimationEvents*>(mutator_events.release()));
+
   for (size_t event_index = 0; event_index < events->events_.size();
        ++event_index) {
     ElementId element_id = events->events_[event_index].element_id;
 
-    // Use the map of all ElementAnimations, not just active ones, since
-    // non-active ElementAnimations may still receive events for impl-only
-    // animations.
+    // Use the map of all ElementAnimations, not just ticking players, since
+    // non-ticking Players may still receive events for impl-only animations.
     const ElementToAnimationsMap& all_element_animations =
         element_to_animations_map_;
     auto iter = all_element_animations.find(element_id);
@@ -341,7 +362,7 @@ bool AnimationHost::ScrollOffsetAnimationWasInterrupted(
     ElementId element_id) const {
   auto element_animations = GetElementAnimationsForElementId(element_id);
   return element_animations
-             ? element_animations->scroll_offset_animation_was_interrupted()
+             ? element_animations->ScrollOffsetAnimationWasInterrupted()
              : false;
 }
 
@@ -411,7 +432,7 @@ bool AnimationHost::HasAnyAnimationTargetingProperty(
   if (!element_animations)
     return false;
 
-  return !!element_animations->GetAnimation(property);
+  return element_animations->HasAnyAnimationTargetingProperty(property);
 }
 
 bool AnimationHost::HasFilterAnimationThatInflatesBounds(
@@ -495,31 +516,34 @@ bool AnimationHost::AnimationStartScale(ElementId element_id,
 
 bool AnimationHost::HasAnyAnimation(ElementId element_id) const {
   auto element_animations = GetElementAnimationsForElementId(element_id);
-  return element_animations ? element_animations->has_any_animation() : false;
+  return element_animations ? element_animations->HasAnyAnimation() : false;
 }
 
-bool AnimationHost::HasActiveAnimationForTesting(ElementId element_id) const {
+bool AnimationHost::HasTickingAnimationForTesting(ElementId element_id) const {
   auto element_animations = GetElementAnimationsForElementId(element_id);
-  return element_animations ? element_animations->HasActiveAnimation() : false;
+  return element_animations ? element_animations->HasTickingAnimation() : false;
 }
 
 void AnimationHost::ImplOnlyScrollAnimationCreate(
     ElementId element_id,
     const gfx::ScrollOffset& target_offset,
-    const gfx::ScrollOffset& current_offset) {
+    const gfx::ScrollOffset& current_offset,
+    base::TimeDelta delayed_by) {
   DCHECK(scroll_offset_animations_impl_);
   scroll_offset_animations_impl_->ScrollAnimationCreate(
-      element_id, target_offset, current_offset);
+      element_id, target_offset, current_offset, delayed_by);
 }
 
 bool AnimationHost::ImplOnlyScrollAnimationUpdateTarget(
     ElementId element_id,
     const gfx::Vector2dF& scroll_delta,
     const gfx::ScrollOffset& max_scroll_offset,
-    base::TimeTicks frame_monotonic_time) {
+    base::TimeTicks frame_monotonic_time,
+    base::TimeDelta delayed_by) {
   DCHECK(scroll_offset_animations_impl_);
   return scroll_offset_animations_impl_->ScrollAnimationUpdateTarget(
-      element_id, scroll_delta, max_scroll_offset, frame_monotonic_time);
+      element_id, scroll_delta, max_scroll_offset, frame_monotonic_time,
+      delayed_by);
 }
 
 ScrollOffsetAnimations& AnimationHost::scroll_offset_animations() const {
@@ -532,45 +556,27 @@ void AnimationHost::ScrollAnimationAbort(bool needs_completion) {
   return scroll_offset_animations_impl_->ScrollAnimationAbort(needs_completion);
 }
 
-void AnimationHost::DidActivateElementAnimations(
-    ElementAnimations* element_animations) {
-  DCHECK(element_animations->element_id());
-  active_element_to_animations_map_[element_animations->element_id()] =
-      element_animations;
+void AnimationHost::AddToTicking(scoped_refptr<AnimationPlayer> player) {
+  DCHECK(std::find(ticking_players_.begin(), ticking_players_.end(), player) ==
+         ticking_players_.end());
+  ticking_players_.push_back(player);
 }
 
-void AnimationHost::DidDeactivateElementAnimations(
-    ElementAnimations* element_animations) {
-  DCHECK(element_animations->element_id());
-  active_element_to_animations_map_.erase(element_animations->element_id());
+void AnimationHost::RemoveFromTicking(scoped_refptr<AnimationPlayer> player) {
+  auto to_erase =
+      std::find(ticking_players_.begin(), ticking_players_.end(), player);
+  if (to_erase != ticking_players_.end())
+    ticking_players_.erase(to_erase);
 }
 
-void AnimationHost::RegisterElementAnimations(
-    ElementAnimations* element_animations) {
-  DCHECK(element_animations->element_id());
-  element_to_animations_map_[element_animations->element_id()] =
-      element_animations;
-}
-
-void AnimationHost::UnregisterElementAnimations(
-    ElementAnimations* element_animations) {
-  DCHECK(element_animations->element_id());
-  element_to_animations_map_.erase(element_animations->element_id());
-  DidDeactivateElementAnimations(element_animations);
+const AnimationHost::PlayersList& AnimationHost::ticking_players_for_testing()
+    const {
+  return ticking_players_;
 }
 
 const AnimationHost::ElementToAnimationsMap&
-AnimationHost::active_element_animations_for_testing() const {
-  return active_element_to_animations_map_;
-}
-
-const AnimationHost::ElementToAnimationsMap&
-AnimationHost::all_element_animations_for_testing() const {
+AnimationHost::element_animations_for_testing() const {
   return element_to_animations_map_;
 }
 
-void AnimationHost::OnAnimationWaitingForDeletion() {
-  animation_waiting_for_deletion_ = true;
-}
-
 }  // namespace cc
