@@ -8,7 +8,6 @@
 #include "cc/quads/debug_border_draw_quad.h"
 #include "cc/quads/render_pass.h"
 #include "cc/quads/render_pass_draw_quad.h"
-#include "cc/quads/render_pass_id.h"
 #include "cc/quads/solid_color_draw_quad.h"
 #include "cc/quads/stream_video_draw_quad.h"
 #include "cc/quads/surface_draw_quad.h"
@@ -63,19 +62,9 @@ class StructTraitsTest : public testing::Test, public mojom::TraitsTestService {
     callback.Run(f);
   }
 
-  void EchoQuadList(const QuadList& q,
-                    const EchoQuadListCallback& callback) override {
-    callback.Run(q);
-  }
-
-  void EchoRenderPass(const std::unique_ptr<RenderPass>& r,
+  void EchoRenderPass(std::unique_ptr<RenderPass> r,
                       const EchoRenderPassCallback& callback) override {
-    callback.Run(r);
-  }
-
-  void EchoRenderPassId(const RenderPassId& r,
-                        const EchoRenderPassIdCallback& callback) override {
-    callback.Run(r);
+    callback.Run(std::move(r));
   }
 
   void EchoReturnedResource(
@@ -100,6 +89,12 @@ class StructTraitsTest : public testing::Test, public mojom::TraitsTestService {
     callback.Run(s);
   }
 
+  void EchoSurfaceReference(
+      const SurfaceReference& s,
+      const EchoSurfaceReferenceCallback& callback) override {
+    callback.Run(s);
+  }
+
   void EchoSurfaceSequence(
       const SurfaceSequence& s,
       const EchoSurfaceSequenceCallback& callback) override {
@@ -145,6 +140,7 @@ TEST_F(StructTraitsTest, BeginFrameArgs) {
 // RenderPass, and QuadListBasic unit tests.
 TEST_F(StructTraitsTest, CompositorFrame) {
   std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+  render_pass->SetNew(1, gfx::Rect(5, 6), gfx::Rect(2, 3), gfx::Transform());
 
   // SharedQuadState.
   const gfx::Transform sqs_quad_to_target_transform(
@@ -155,7 +151,7 @@ TEST_F(StructTraitsTest, CompositorFrame) {
   const gfx::Rect sqs_clip_rect(123, 456, 789, 101112);
   const bool sqs_is_clipped = true;
   const float sqs_opacity = 0.9f;
-  const SkXfermode::Mode sqs_blend_mode = SkXfermode::kSrcOver_Mode;
+  const SkBlendMode sqs_blend_mode = SkBlendMode::kSrcOver;
   const int sqs_sorting_context_id = 1337;
   SharedQuadState* sqs = render_pass->CreateAndAppendSharedQuadState();
   sqs->SetAll(sqs_quad_to_target_transform, sqs_layer_bounds,
@@ -200,10 +196,8 @@ TEST_F(StructTraitsTest, CompositorFrame) {
   input.metadata.root_scroll_offset = root_scroll_offset;
   input.metadata.page_scale_factor = page_scale_factor;
   input.metadata.scrollable_viewport_size = scrollable_viewport_size;
-  input.delegated_frame_data.reset(new DelegatedFrameData);
-  input.delegated_frame_data->render_pass_list.push_back(
-      std::move(render_pass));
-  input.delegated_frame_data->resource_list.push_back(resource);
+  input.render_pass_list.push_back(std::move(render_pass));
+  input.resource_list.push_back(resource);
 
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
   CompositorFrame output;
@@ -214,18 +208,15 @@ TEST_F(StructTraitsTest, CompositorFrame) {
   EXPECT_EQ(page_scale_factor, output.metadata.page_scale_factor);
   EXPECT_EQ(scrollable_viewport_size, output.metadata.scrollable_viewport_size);
 
-  EXPECT_NE(nullptr, output.delegated_frame_data);
-  ASSERT_EQ(1u, output.delegated_frame_data->resource_list.size());
-  TransferableResource out_resource =
-      output.delegated_frame_data->resource_list[0];
+  ASSERT_EQ(1u, output.resource_list.size());
+  TransferableResource out_resource = output.resource_list[0];
   EXPECT_EQ(tr_id, out_resource.id);
   EXPECT_EQ(tr_format, out_resource.format);
   EXPECT_EQ(tr_filter, out_resource.filter);
   EXPECT_EQ(tr_size, out_resource.size);
 
-  EXPECT_EQ(1u, output.delegated_frame_data->render_pass_list.size());
-  const RenderPass* out_render_pass =
-      output.delegated_frame_data->render_pass_list[0].get();
+  EXPECT_EQ(1u, output.render_pass_list.size());
+  const RenderPass* out_render_pass = output.render_pass_list[0].get();
   ASSERT_EQ(2u, out_render_pass->quad_list.size());
   ASSERT_EQ(1u, out_render_pass->shared_quad_state_list.size());
 
@@ -267,8 +258,12 @@ TEST_F(StructTraitsTest, CompositorFrameMetadata) {
   const float max_page_scale_factor = 4.6f;
   const bool root_overflow_x_hidden = true;
   const bool root_overflow_y_hidden = true;
-  const gfx::Vector2dF location_bar_offset(1234.5f, 5432.1f);
-  const gfx::Vector2dF location_bar_content_translation(1234.5f, 5432.1f);
+  const bool may_contain_video = true;
+  const bool is_resourceless_software_draw_with_scroll_or_animation = true;
+  const float top_bar_height(1234.5f);
+  const float top_bar_shown_ratio(1.0f);
+  const float bottom_bar_height(1234.5f);
+  const float bottom_bar_shown_ratio(1.0f);
   const uint32_t root_background_color = 1337;
   Selection<gfx::SelectionBound> selection;
   selection.start.SetEdge(gfx::PointF(1234.5f, 67891.f),
@@ -285,9 +280,9 @@ TEST_F(StructTraitsTest, CompositorFrameMetadata) {
   latency_info.AddLatencyNumber(
       ui::LATENCY_BEGIN_SCROLL_LISTENER_UPDATE_MAIN_COMPONENT, 1337, 7331);
   std::vector<ui::LatencyInfo> latency_infos = {latency_info};
-  std::vector<uint32_t> satisfies_sequences = {1234, 1337};
   std::vector<SurfaceId> referenced_surfaces;
-  SurfaceId id(1234, 5678, 9101112);
+  SurfaceId id(FrameSinkId(1234, 4321),
+               LocalFrameId(5678, base::UnguessableToken::Create()));
   referenced_surfaces.push_back(id);
 
   CompositorFrameMetadata input;
@@ -300,12 +295,16 @@ TEST_F(StructTraitsTest, CompositorFrameMetadata) {
   input.max_page_scale_factor = max_page_scale_factor;
   input.root_overflow_x_hidden = root_overflow_x_hidden;
   input.root_overflow_y_hidden = root_overflow_y_hidden;
-  input.location_bar_offset = location_bar_offset;
-  input.location_bar_content_translation = location_bar_content_translation;
+  input.may_contain_video = may_contain_video;
+  input.is_resourceless_software_draw_with_scroll_or_animation =
+      is_resourceless_software_draw_with_scroll_or_animation;
+  input.top_controls_height = top_bar_height;
+  input.top_controls_shown_ratio = top_bar_shown_ratio;
+  input.bottom_controls_height = bottom_bar_height;
+  input.bottom_controls_shown_ratio = bottom_bar_shown_ratio;
   input.root_background_color = root_background_color;
   input.selection = selection;
   input.latency_info = latency_infos;
-  input.satisfies_sequences = satisfies_sequences;
   input.referenced_surfaces = referenced_surfaces;
 
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
@@ -320,9 +319,13 @@ TEST_F(StructTraitsTest, CompositorFrameMetadata) {
   EXPECT_EQ(max_page_scale_factor, output.max_page_scale_factor);
   EXPECT_EQ(root_overflow_x_hidden, output.root_overflow_x_hidden);
   EXPECT_EQ(root_overflow_y_hidden, output.root_overflow_y_hidden);
-  EXPECT_EQ(location_bar_offset, output.location_bar_offset);
-  EXPECT_EQ(location_bar_content_translation,
-            output.location_bar_content_translation);
+  EXPECT_EQ(may_contain_video, output.may_contain_video);
+  EXPECT_EQ(is_resourceless_software_draw_with_scroll_or_animation,
+            output.is_resourceless_software_draw_with_scroll_or_animation);
+  EXPECT_EQ(top_bar_height, output.top_controls_height);
+  EXPECT_EQ(top_bar_shown_ratio, output.top_controls_shown_ratio);
+  EXPECT_EQ(bottom_bar_height, output.bottom_controls_height);
+  EXPECT_EQ(bottom_bar_shown_ratio, output.bottom_controls_shown_ratio);
   EXPECT_EQ(root_background_color, output.root_background_color);
   EXPECT_EQ(selection, output.selection);
   EXPECT_EQ(latency_infos.size(), output.latency_info.size());
@@ -331,9 +334,6 @@ TEST_F(StructTraitsTest, CompositorFrameMetadata) {
       ui::LATENCY_BEGIN_SCROLL_LISTENER_UPDATE_MAIN_COMPONENT, 1337,
       &component));
   EXPECT_EQ(7331, component.sequence_number);
-  EXPECT_EQ(satisfies_sequences.size(), output.satisfies_sequences.size());
-  for (uint32_t i = 0; i < satisfies_sequences.size(); ++i)
-    EXPECT_EQ(satisfies_sequences[i], output.satisfies_sequences[i]);
   EXPECT_EQ(referenced_surfaces.size(), output.referenced_surfaces.size());
   for (uint32_t i = 0; i < referenced_surfaces.size(); ++i)
     EXPECT_EQ(referenced_surfaces[i], output.referenced_surfaces[i]);
@@ -409,47 +409,46 @@ TEST_F(StructTraitsTest, FilterOperations) {
 }
 
 TEST_F(StructTraitsTest, QuadListBasic) {
-  SharedQuadState sqs;
-  QuadList input;
+  std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+  render_pass->SetNew(1, gfx::Rect(), gfx::Rect(), gfx::Transform());
+
+  SharedQuadState* sqs = render_pass->CreateAndAppendSharedQuadState();
 
   const gfx::Rect rect1(1234, 4321, 1357, 7531);
   const SkColor color1 = SK_ColorRED;
   const int32_t width1 = 1337;
   DebugBorderDrawQuad* debug_quad =
-      input.AllocateAndConstruct<DebugBorderDrawQuad>();
-  debug_quad->SetNew(&sqs, rect1, rect1, color1, width1);
+      render_pass->CreateAndAppendDrawQuad<DebugBorderDrawQuad>();
+  debug_quad->SetNew(sqs, rect1, rect1, color1, width1);
 
   const gfx::Rect rect2(2468, 8642, 4321, 1234);
   const uint32_t color2 = 0xffffffff;
   const bool force_anti_aliasing_off = true;
   SolidColorDrawQuad* solid_quad =
-      input.AllocateAndConstruct<SolidColorDrawQuad>();
-  solid_quad->SetNew(&sqs, rect2, rect2, color2, force_anti_aliasing_off);
+      render_pass->CreateAndAppendDrawQuad<SolidColorDrawQuad>();
+  solid_quad->SetNew(sqs, rect2, rect2, color2, force_anti_aliasing_off);
 
   const gfx::Rect rect3(1029, 3847, 5610, 2938);
-  const SurfaceId surface_id(1234, 5678, 2468);
-  SurfaceDrawQuad* surface_quad = input.AllocateAndConstruct<SurfaceDrawQuad>();
-  surface_quad->SetNew(&sqs, rect3, rect3, surface_id);
+  const SurfaceId surface_id(
+      FrameSinkId(1234, 4321),
+      LocalFrameId(5678, base::UnguessableToken::Create()));
+  SurfaceDrawQuad* surface_quad =
+      render_pass->CreateAndAppendDrawQuad<SurfaceDrawQuad>();
+  surface_quad->SetNew(sqs, rect3, rect3, surface_id);
 
   const gfx::Rect rect4(1234, 5678, 9101112, 13141516);
   const ResourceId resource_id4(1337);
-  const RenderPassId render_pass_id(1234, 5678);
+  const int render_pass_id = 1234;
   const gfx::Vector2dF mask_uv_scale(1337.1f, 1234.2f);
   const gfx::Size mask_texture_size(1234, 5678);
-  FilterOperations filters;
-  filters.Append(FilterOperation::CreateBlurFilter(0.f));
-  filters.Append(FilterOperation::CreateZoomFilter(2.0f, 1));
   gfx::Vector2dF filters_scale(1234.1f, 4321.2f);
-  FilterOperations background_filters;
-  background_filters.Append(FilterOperation::CreateSaturateFilter(4.f));
-  background_filters.Append(FilterOperation::CreateZoomFilter(2.0f, 1));
-  background_filters.Append(FilterOperation::CreateSaturateFilter(2.f));
+  gfx::PointF filters_origin(8765.4f, 4567.8f);
 
   RenderPassDrawQuad* render_pass_quad =
-      input.AllocateAndConstruct<RenderPassDrawQuad>();
-  render_pass_quad->SetNew(&sqs, rect4, rect4, render_pass_id, resource_id4,
-                           mask_uv_scale, mask_texture_size, filters,
-                           filters_scale, background_filters);
+      render_pass->CreateAndAppendDrawQuad<RenderPassDrawQuad>();
+  render_pass_quad->SetNew(sqs, rect4, rect4, render_pass_id, resource_id4,
+                           mask_uv_scale, mask_texture_size, filters_scale,
+                           filters_origin);
 
   const gfx::Rect rect5(123, 567, 91011, 131415);
   const ResourceId resource_id5(1337);
@@ -461,9 +460,12 @@ TEST_F(StructTraitsTest, QuadListBasic) {
   const bool y_flipped = true;
   const bool nearest_neighbor = true;
   const bool secure_output_only = true;
+  const bool needs_blending = true;
+  const gfx::Size resource_size_in_pixels5(1234, 5678);
   TextureDrawQuad* texture_draw_quad =
-      input.AllocateAndConstruct<TextureDrawQuad>();
-  texture_draw_quad->SetNew(&sqs, rect5, rect5, rect5, resource_id5,
+      render_pass->CreateAndAppendDrawQuad<TextureDrawQuad>();
+  texture_draw_quad->SetAll(sqs, rect5, rect5, rect5, needs_blending,
+                            resource_id5, resource_size_in_pixels5,
                             premultiplied_alpha, uv_top_left, uv_bottom_right,
                             background_color, vertex_opacity, y_flipped,
                             nearest_neighbor, secure_output_only);
@@ -475,25 +477,25 @@ TEST_F(StructTraitsTest, QuadListBasic) {
                               9.8f, 8.1f, 7.3f, 6.3f, 5.7f, 4.8f, 3.4f, 2.4f,
                               1.2f);
   StreamVideoDrawQuad* stream_video_draw_quad =
-      input.AllocateAndConstruct<StreamVideoDrawQuad>();
-  stream_video_draw_quad->SetNew(&sqs, rect6, rect6, rect6, resource_id6,
+      render_pass->CreateAndAppendDrawQuad<StreamVideoDrawQuad>();
+  stream_video_draw_quad->SetNew(sqs, rect6, rect6, rect6, resource_id6,
                                  resource_size_in_pixels, matrix);
 
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
-  QuadList output;
-  proxy->EchoQuadList(input, &output);
+  std::unique_ptr<RenderPass> output;
+  proxy->EchoRenderPass(render_pass->DeepCopy(), &output);
 
-  ASSERT_EQ(input.size(), output.size());
+  ASSERT_EQ(render_pass->quad_list.size(), output->quad_list.size());
 
   const DebugBorderDrawQuad* out_debug_border_draw_quad =
-      DebugBorderDrawQuad::MaterialCast(output.ElementAt(0));
+      DebugBorderDrawQuad::MaterialCast(output->quad_list.ElementAt(0));
   EXPECT_EQ(rect1, out_debug_border_draw_quad->rect);
   EXPECT_EQ(rect1, out_debug_border_draw_quad->visible_rect);
   EXPECT_EQ(color1, out_debug_border_draw_quad->color);
   EXPECT_EQ(width1, out_debug_border_draw_quad->width);
 
   const SolidColorDrawQuad* out_solid_color_draw_quad =
-      SolidColorDrawQuad::MaterialCast(output.ElementAt(1));
+      SolidColorDrawQuad::MaterialCast(output->quad_list.ElementAt(1));
   EXPECT_EQ(rect2, out_solid_color_draw_quad->rect);
   EXPECT_EQ(rect2, out_solid_color_draw_quad->visible_rect);
   EXPECT_EQ(color2, out_solid_color_draw_quad->color);
@@ -501,34 +503,29 @@ TEST_F(StructTraitsTest, QuadListBasic) {
             out_solid_color_draw_quad->force_anti_aliasing_off);
 
   const SurfaceDrawQuad* out_surface_draw_quad =
-      SurfaceDrawQuad::MaterialCast(output.ElementAt(2));
+      SurfaceDrawQuad::MaterialCast(output->quad_list.ElementAt(2));
   EXPECT_EQ(rect3, out_surface_draw_quad->rect);
   EXPECT_EQ(rect3, out_surface_draw_quad->visible_rect);
   EXPECT_EQ(surface_id, out_surface_draw_quad->surface_id);
 
   const RenderPassDrawQuad* out_render_pass_draw_quad =
-      RenderPassDrawQuad::MaterialCast(output.ElementAt(3));
+      RenderPassDrawQuad::MaterialCast(output->quad_list.ElementAt(3));
   EXPECT_EQ(rect4, out_render_pass_draw_quad->rect);
   EXPECT_EQ(rect4, out_render_pass_draw_quad->visible_rect);
   EXPECT_EQ(render_pass_id, out_render_pass_draw_quad->render_pass_id);
   EXPECT_EQ(resource_id4, out_render_pass_draw_quad->mask_resource_id());
   EXPECT_EQ(mask_texture_size, out_render_pass_draw_quad->mask_texture_size);
-  EXPECT_EQ(filters.size(), out_render_pass_draw_quad->filters.size());
-  for (size_t i = 0; i < filters.size(); ++i)
-    EXPECT_EQ(filters.at(i), out_render_pass_draw_quad->filters.at(i));
   EXPECT_EQ(filters_scale, out_render_pass_draw_quad->filters_scale);
-  EXPECT_EQ(background_filters.size(),
-            out_render_pass_draw_quad->background_filters.size());
-  for (size_t i = 0; i < background_filters.size(); ++i)
-    EXPECT_EQ(background_filters.at(i),
-              out_render_pass_draw_quad->background_filters.at(i));
 
   const TextureDrawQuad* out_texture_draw_quad =
-      TextureDrawQuad::MaterialCast(output.ElementAt(4));
+      TextureDrawQuad::MaterialCast(output->quad_list.ElementAt(4));
   EXPECT_EQ(rect5, out_texture_draw_quad->rect);
   EXPECT_EQ(rect5, out_texture_draw_quad->opaque_rect);
   EXPECT_EQ(rect5, out_texture_draw_quad->visible_rect);
+  EXPECT_EQ(needs_blending, out_texture_draw_quad->needs_blending);
   EXPECT_EQ(resource_id5, out_texture_draw_quad->resource_id());
+  EXPECT_EQ(resource_size_in_pixels5,
+            out_texture_draw_quad->resource_size_in_pixels());
   EXPECT_EQ(premultiplied_alpha, out_texture_draw_quad->premultiplied_alpha);
   EXPECT_EQ(uv_top_left, out_texture_draw_quad->uv_top_left);
   EXPECT_EQ(uv_bottom_right, out_texture_draw_quad->uv_bottom_right);
@@ -542,7 +539,7 @@ TEST_F(StructTraitsTest, QuadListBasic) {
   EXPECT_EQ(secure_output_only, out_texture_draw_quad->secure_output_only);
 
   const StreamVideoDrawQuad* out_stream_video_draw_quad =
-      StreamVideoDrawQuad::MaterialCast(output.ElementAt(5));
+      StreamVideoDrawQuad::MaterialCast(output->quad_list.ElementAt(5));
   EXPECT_EQ(rect6, out_stream_video_draw_quad->rect);
   EXPECT_EQ(rect6, out_stream_video_draw_quad->opaque_rect);
   EXPECT_EQ(rect6, out_stream_video_draw_quad->visible_rect);
@@ -553,31 +550,36 @@ TEST_F(StructTraitsTest, QuadListBasic) {
 }
 
 TEST_F(StructTraitsTest, RenderPass) {
-  const RenderPassId id(3, 2);
+  const int id = 3;
   const gfx::Rect output_rect(45, 22, 120, 13);
   const gfx::Transform transform_to_root =
       gfx::Transform(1.0, 0.5, 0.5, -0.5, -1.0, 0.0);
   const gfx::Rect damage_rect(56, 123, 19, 43);
+  FilterOperations filters;
+  filters.Append(FilterOperation::CreateBlurFilter(0.f));
+  filters.Append(FilterOperation::CreateZoomFilter(2.0f, 1));
+  FilterOperations background_filters;
+  background_filters.Append(FilterOperation::CreateSaturateFilter(4.f));
+  background_filters.Append(FilterOperation::CreateZoomFilter(2.0f, 1));
+  background_filters.Append(FilterOperation::CreateSaturateFilter(2.f));
   const bool has_transparent_background = true;
   std::unique_ptr<RenderPass> input = RenderPass::Create();
-  input->SetAll(id, output_rect, damage_rect, transform_to_root,
-                has_transparent_background);
+  input->SetAll(id, output_rect, damage_rect, transform_to_root, filters,
+                background_filters, has_transparent_background);
 
   SharedQuadState* shared_state_1 = input->CreateAndAppendSharedQuadState();
   shared_state_1->SetAll(
       gfx::Transform(16.1f, 15.3f, 14.3f, 13.7f, 12.2f, 11.4f, 10.4f, 9.8f,
                      8.1f, 7.3f, 6.3f, 5.7f, 4.8f, 3.4f, 2.4f, 1.2f),
       gfx::Size(1, 2), gfx::Rect(1337, 5679, 9101112, 131415),
-      gfx::Rect(1357, 2468, 121314, 1337), true, 2, SkXfermode::kSrcOver_Mode,
-      1);
+      gfx::Rect(1357, 2468, 121314, 1337), true, 2, SkBlendMode::kSrcOver, 1);
 
   SharedQuadState* shared_state_2 = input->CreateAndAppendSharedQuadState();
   shared_state_2->SetAll(
       gfx::Transform(1.1f, 2.3f, 3.3f, 4.7f, 5.2f, 6.4f, 7.4f, 8.8f, 9.1f,
                      10.3f, 11.3f, 12.7f, 13.8f, 14.4f, 15.4f, 16.2f),
       gfx::Size(1337, 1234), gfx::Rect(1234, 5678, 9101112, 13141516),
-      gfx::Rect(1357, 2468, 121314, 1337), true, 2, SkXfermode::kSrcOver_Mode,
-      1);
+      gfx::Rect(1357, 2468, 121314, 1337), true, 2, SkBlendMode::kSrcOver, 1);
 
   // This quad uses the first shared quad state. The next two quads use the
   // second shared quad state.
@@ -596,12 +598,14 @@ TEST_F(StructTraitsTest, RenderPass) {
   SurfaceDrawQuad* surface_quad =
       input->CreateAndAppendDrawQuad<SurfaceDrawQuad>();
   const gfx::Rect surface_quad_rect(1337, 2448, 1234, 5678);
-  surface_quad->SetNew(shared_state_2, surface_quad_rect, surface_quad_rect,
-                       SurfaceId(1337, 1234, 2468));
+  surface_quad->SetNew(
+      shared_state_2, surface_quad_rect, surface_quad_rect,
+      SurfaceId(FrameSinkId(1337, 1234),
+                LocalFrameId(1234, base::UnguessableToken::Create())));
 
   std::unique_ptr<RenderPass> output;
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
-  proxy->EchoRenderPass(input, &output);
+  proxy->EchoRenderPass(input->DeepCopy(), &output);
 
   EXPECT_EQ(input->quad_list.size(), output->quad_list.size());
   EXPECT_EQ(input->shared_quad_state_list.size(),
@@ -611,6 +615,8 @@ TEST_F(StructTraitsTest, RenderPass) {
   EXPECT_EQ(damage_rect, output->damage_rect);
   EXPECT_EQ(transform_to_root, output->transform_to_root_target);
   EXPECT_EQ(has_transparent_background, output->has_transparent_background);
+  EXPECT_EQ(filters, output->filters);
+  EXPECT_EQ(background_filters, output->background_filters);
 
   SharedQuadState* out_sqs1 = output->shared_quad_state_list.ElementAt(0);
   EXPECT_EQ(shared_state_1->quad_to_target_transform,
@@ -661,15 +667,32 @@ TEST_F(StructTraitsTest, RenderPass) {
   EXPECT_EQ(surface_quad->surface_id, out_surface_quad->surface_id);
 }
 
-TEST_F(StructTraitsTest, RenderPassId) {
-  const int layer_id = 1337;
-  const uint32_t index = 0xdeadbeef;
-  RenderPassId input(layer_id, index);
+TEST_F(StructTraitsTest, RenderPassWithEmptySharedQuadStateList) {
+  const int id = 3;
+  const gfx::Rect output_rect(45, 22, 120, 13);
+  const gfx::Transform transform_to_root =
+      gfx::Transform(1.0, 0.5, 0.5, -0.5, -1.0, 0.0);
+  const gfx::Rect damage_rect(56, 123, 19, 43);
+  const bool has_transparent_background = true;
+  std::unique_ptr<RenderPass> input = RenderPass::Create();
+  input->SetAll(id, output_rect, damage_rect, transform_to_root,
+                FilterOperations(), FilterOperations(),
+                has_transparent_background);
+
+  // Unlike the previous test, don't add any quads to the list; we need to
+  // verify that the serialization code can deal with that.
+  std::unique_ptr<RenderPass> output;
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
-  RenderPassId output;
-  proxy->EchoRenderPassId(input, &output);
-  EXPECT_EQ(layer_id, output.layer_id);
-  EXPECT_EQ(index, output.index);
+  proxy->EchoRenderPass(input->DeepCopy(), &output);
+
+  EXPECT_EQ(input->quad_list.size(), output->quad_list.size());
+  EXPECT_EQ(input->shared_quad_state_list.size(),
+            output->shared_quad_state_list.size());
+  EXPECT_EQ(id, output->id);
+  EXPECT_EQ(output_rect, output->output_rect);
+  EXPECT_EQ(damage_rect, output->damage_rect);
+  EXPECT_EQ(transform_to_root, output->transform_to_root_target);
+  EXPECT_EQ(has_transparent_background, output->has_transparent_background);
 }
 
 TEST_F(StructTraitsTest, ReturnedResource) {
@@ -724,26 +747,41 @@ TEST_F(StructTraitsTest, Selection) {
 }
 
 TEST_F(StructTraitsTest, SurfaceId) {
-  const uint32_t id_namespace = 1337;
-  const uint32_t local_id = 0xfbadbeef;
-  const uint64_t nonce = 0xdeadbeef;
-  SurfaceId input(id_namespace, local_id, nonce);
+  static constexpr FrameSinkId frame_sink_id(1337, 1234);
+  static LocalFrameId local_frame_id(0xfbadbeef,
+                                     base::UnguessableToken::Create());
+  SurfaceId input(frame_sink_id, local_frame_id);
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
   SurfaceId output;
   proxy->EchoSurfaceId(input, &output);
-  EXPECT_EQ(id_namespace, output.id_namespace());
-  EXPECT_EQ(local_id, output.local_id());
-  EXPECT_EQ(nonce, output.nonce());
+  EXPECT_EQ(frame_sink_id, output.frame_sink_id());
+  EXPECT_EQ(local_frame_id, output.local_frame_id());
+}
+
+TEST_F(StructTraitsTest, SurfaceReference) {
+  const SurfaceId parent_id(
+      FrameSinkId(2016, 1234),
+      LocalFrameId(0xfbadbeef, base::UnguessableToken::Deserialize(123, 456)));
+  const SurfaceId child_id(
+      FrameSinkId(1111, 9999),
+      LocalFrameId(0xabcdabcd, base::UnguessableToken::Deserialize(333, 333)));
+  const SurfaceReference input(parent_id, child_id);
+
+  mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
+  SurfaceReference output;
+  proxy->EchoSurfaceReference(input, &output);
+  EXPECT_EQ(parent_id, output.parent_id());
+  EXPECT_EQ(child_id, output.child_id());
 }
 
 TEST_F(StructTraitsTest, SurfaceSequence) {
-  const uint32_t id_namespace = 2016;
+  const FrameSinkId frame_sink_id(2016, 1234);
   const uint32_t sequence = 0xfbadbeef;
-  SurfaceSequence input(id_namespace, sequence);
+  SurfaceSequence input(frame_sink_id, sequence);
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
   SurfaceSequence output;
   proxy->EchoSurfaceSequence(input, &output);
-  EXPECT_EQ(id_namespace, output.id_namespace);
+  EXPECT_EQ(frame_sink_id, output.frame_sink_id);
   EXPECT_EQ(sequence, output.sequence);
 }
 
@@ -756,7 +794,7 @@ TEST_F(StructTraitsTest, SharedQuadState) {
   const gfx::Rect clip_rect(123, 456, 789, 101112);
   const bool is_clipped = true;
   const float opacity = 0.9f;
-  const SkXfermode::Mode blend_mode = SkXfermode::kSrcOver_Mode;
+  const SkBlendMode blend_mode = SkBlendMode::kSrcOver;
   const int sorting_context_id = 1337;
   SharedQuadState input_sqs;
   input_sqs.SetAll(quad_to_target_transform, layer_bounds, visible_layer_rect,
@@ -781,9 +819,7 @@ TEST_F(StructTraitsTest, TransferableResource) {
   const uint32_t filter = 1234;
   const gfx::Size size(1234, 5678);
   const int8_t mailbox_name[GL_MAILBOX_SIZE_CHROMIUM] = {
-      0, 9, 8, 7, 6, 5, 4, 3, 2, 1, 9, 7, 5, 3, 1, 2, 4, 6, 8, 0, 0, 9,
-      8, 7, 6, 5, 4, 3, 2, 1, 9, 7, 5, 3, 1, 2, 4, 6, 8, 0, 0, 9, 8, 7,
-      6, 5, 4, 3, 2, 1, 9, 7, 5, 3, 1, 2, 4, 6, 8, 0, 0, 9, 8, 7};
+      0, 9, 8, 7, 6, 5, 4, 3, 2, 1, 9, 7, 5, 3, 1, 2};
   const gpu::CommandBufferNamespace command_buffer_namespace = gpu::IN_PROCESS;
   const int32_t extra_data_field = 0xbeefbeef;
   const gpu::CommandBufferId command_buffer_id(
@@ -826,6 +862,9 @@ TEST_F(StructTraitsTest, TransferableResource) {
 }
 
 TEST_F(StructTraitsTest, YUVDrawQuad) {
+  std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+  render_pass->SetNew(1, gfx::Rect(), gfx::Rect(), gfx::Transform());
+
   const DrawQuad::Material material = DrawQuad::YUV_VIDEO_CONTENT;
   const gfx::Rect rect(1234, 4321, 1357, 7531);
   const gfx::Rect opaque_rect(1357, 8642, 432, 123);
@@ -840,27 +879,29 @@ TEST_F(StructTraitsTest, YUVDrawQuad) {
   const uint32_t v_plane_resource_id = 2468;
   const uint32_t a_plane_resource_id = 7890;
   const YUVVideoDrawQuad::ColorSpace color_space = YUVVideoDrawQuad::JPEG;
+  const gfx::ColorSpace video_color_space = gfx::ColorSpace::CreateJpeg();
   const float resource_offset = 1337.5f;
   const float resource_multiplier = 1234.6f;
+  const uint32_t bits_per_channel = 13;
 
-  SharedQuadState sqs;
-  QuadList input;
-  YUVVideoDrawQuad* quad = input.AllocateAndConstruct<YUVVideoDrawQuad>();
-  quad->SetAll(&sqs, rect, opaque_rect, visible_rect, needs_blending,
+  SharedQuadState* sqs = render_pass->CreateAndAppendSharedQuadState();
+  YUVVideoDrawQuad* quad =
+      render_pass->CreateAndAppendDrawQuad<YUVVideoDrawQuad>();
+  quad->SetAll(sqs, rect, opaque_rect, visible_rect, needs_blending,
                ya_tex_coord_rect, uv_tex_coord_rect, ya_tex_size, uv_tex_size,
                y_plane_resource_id, u_plane_resource_id, v_plane_resource_id,
-               a_plane_resource_id, color_space, resource_offset,
-               resource_multiplier);
+               a_plane_resource_id, color_space, video_color_space,
+               resource_offset, resource_multiplier, bits_per_channel);
 
   mojom::TraitsTestServicePtr proxy = GetTraitsTestProxy();
-  QuadList output;
-  proxy->EchoQuadList(input, &output);
+  std::unique_ptr<RenderPass> output;
+  proxy->EchoRenderPass(render_pass->DeepCopy(), &output);
 
-  ASSERT_EQ(input.size(), output.size());
+  ASSERT_EQ(render_pass->quad_list.size(), output->quad_list.size());
 
-  ASSERT_EQ(material, output.ElementAt(0)->material);
+  ASSERT_EQ(material, output->quad_list.ElementAt(0)->material);
   const YUVVideoDrawQuad* out_quad =
-      YUVVideoDrawQuad::MaterialCast(output.ElementAt(0));
+      YUVVideoDrawQuad::MaterialCast(output->quad_list.ElementAt(0));
   EXPECT_EQ(rect, out_quad->rect);
   EXPECT_EQ(opaque_rect, out_quad->opaque_rect);
   EXPECT_EQ(visible_rect, out_quad->visible_rect);
@@ -876,6 +917,7 @@ TEST_F(StructTraitsTest, YUVDrawQuad) {
   EXPECT_EQ(color_space, out_quad->color_space);
   EXPECT_EQ(resource_offset, out_quad->resource_offset);
   EXPECT_EQ(resource_multiplier, out_quad->resource_multiplier);
+  EXPECT_EQ(bits_per_channel, out_quad->bits_per_channel);
 }
 
 }  // namespace cc
