@@ -11,7 +11,8 @@
 namespace mojo {
 
 // static
-bool StructTraits<cc::mojom::TransferableResource, cc::TransferableResource>::
+bool StructTraits<cc::mojom::TransferableResourceDataView,
+                  cc::TransferableResource>::
     Read(cc::mojom::TransferableResourceDataView data,
          cc::TransferableResource* out) {
   if (!data.ReadSize(&out->size) ||
@@ -23,6 +24,10 @@ bool StructTraits<cc::mojom::TransferableResource, cc::TransferableResource>::
   out->read_lock_fences_enabled = data.read_lock_fences_enabled();
   out->is_software = data.is_software();
   out->is_overlay_candidate = data.is_overlay_candidate();
+#if defined(OS_ANDROID)
+  out->is_backed_by_surface_texture = data.is_backed_by_surface_texture();
+  out->wants_promotion_hint = data.wants_promotion_hint();
+#endif
   return true;
 }
 
