@@ -17,26 +17,33 @@ ScopedResource::~ScopedResource() {
 
 void ScopedResource::Allocate(const gfx::Size& size,
                               ResourceProvider::TextureHint hint,
-                              ResourceFormat format) {
+                              ResourceFormat format,
+                              const gfx::ColorSpace& color_space) {
   DCHECK(!id());
   DCHECK(!size.IsEmpty());
 
   set_dimensions(size, format);
-  set_id(resource_provider_->CreateResource(size, hint, format));
+  set_id(resource_provider_->CreateResource(size, hint, format, color_space));
+  set_color_space(color_space);
 
 #if DCHECK_IS_ON()
   allocate_thread_id_ = base::PlatformThread::CurrentId();
 #endif
 }
 
-void ScopedResource::AllocateWithGpuMemoryBuffer(const gfx::Size& size,
-                                                 ResourceFormat format) {
+void ScopedResource::AllocateWithGpuMemoryBuffer(
+    const gfx::Size& size,
+    ResourceFormat format,
+    gfx::BufferUsage usage,
+    const gfx::ColorSpace& color_space) {
   DCHECK(!id());
   DCHECK(!size.IsEmpty());
 
   set_dimensions(size, format);
   set_id(resource_provider_->CreateGpuMemoryBufferResource(
-      size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, format));
+      size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, format, usage,
+      color_space));
+  set_color_space(color_space);
 
 #if DCHECK_IS_ON()
   allocate_thread_id_ = base::PlatformThread::CurrentId();
