@@ -4,6 +4,7 @@
 
 #include "cc/test/fake_picture_layer.h"
 
+#include "cc/proto/layer.pb.h"
 #include "cc/test/fake_picture_layer_impl.h"
 
 namespace cc {
@@ -11,8 +12,8 @@ namespace cc {
 FakePictureLayer::FakePictureLayer(ContentLayerClient* client)
     : PictureLayer(client),
       update_count_(0),
-      push_properties_count_(0),
-      always_update_resources_(false) {
+      always_update_resources_(false),
+      force_unsuitable_for_gpu_rasterization_(false) {
   SetBounds(gfx::Size(1, 1));
   SetIsDrawable(true);
 }
@@ -21,8 +22,8 @@ FakePictureLayer::FakePictureLayer(ContentLayerClient* client,
                                    std::unique_ptr<RecordingSource> source)
     : PictureLayer(client, std::move(source)),
       update_count_(0),
-      push_properties_count_(0),
-      always_update_resources_(false) {
+      always_update_resources_(false),
+      force_unsuitable_for_gpu_rasterization_(false) {
   SetBounds(gfx::Size(1, 1));
   SetIsDrawable(true);
 }
@@ -32,8 +33,8 @@ FakePictureLayer::~FakePictureLayer() {}
 std::unique_ptr<LayerImpl> FakePictureLayer::CreateLayerImpl(
     LayerTreeImpl* tree_impl) {
   if (is_mask())
-    return FakePictureLayerImpl::CreateMask(tree_impl, layer_id_);
-  return FakePictureLayerImpl::Create(tree_impl, layer_id_);
+    return FakePictureLayerImpl::CreateMask(tree_impl, id());
+  return FakePictureLayerImpl::Create(tree_impl, id());
 }
 
 bool FakePictureLayer::Update() {
@@ -42,9 +43,15 @@ bool FakePictureLayer::Update() {
   return updated || always_update_resources_;
 }
 
-void FakePictureLayer::PushPropertiesTo(LayerImpl* layer) {
-  PictureLayer::PushPropertiesTo(layer);
-  push_properties_count_++;
+bool FakePictureLayer::IsSuitableForGpuRasterization() const {
+  if (force_unsuitable_for_gpu_rasterization_)
+    return false;
+  return PictureLayer::IsSuitableForGpuRasterization();
+}
+
+void FakePictureLayer::SetTypeForProtoSerialization(
+    proto::LayerNode* proto) const {
+  proto->set_type(proto::LayerNode::FAKE_PICTURE_LAYER);
 }
 
 }  // namespace cc
