@@ -4,7 +4,7 @@
 
 #include "cc/test/fake_proxy.h"
 
-#include "cc/animation/layer_tree_mutator.h"
+#include "cc/trees/layer_tree_mutator.h"
 
 namespace cc {
 
@@ -18,15 +18,7 @@ bool FakeProxy::CommitToActiveTree() const {
   return false;
 }
 
-const RendererCapabilities& FakeProxy::GetRendererCapabilities() const {
-  return capabilities_;
-}
-
-RendererCapabilities& FakeProxy::GetRendererCapabilities() {
-  return capabilities_;
-}
-
-void FakeProxy::ReleaseOutputSurface() {}
+void FakeProxy::ReleaseCompositorFrameSink() {}
 
 bool FakeProxy::BeginMainFrameRequested() const { return false; }
 
