@@ -28,11 +28,16 @@ ResourceFormat FakeRasterBufferProviderImpl::GetResourceFormat(
   return ResourceFormat::RGBA_8888;
 }
 
-bool FakeRasterBufferProviderImpl::GetResourceRequiresSwizzle(
+bool FakeRasterBufferProviderImpl::IsResourceSwizzleRequired(
     bool must_support_alpha) const {
   return ResourceFormatRequiresSwizzle(GetResourceFormat(must_support_alpha));
 }
 
+bool FakeRasterBufferProviderImpl::CanPartialRasterIntoProvidedResource()
+    const {
+  return true;
+}
+
 void FakeRasterBufferProviderImpl::Shutdown() {}
 
 }  // namespace cc
