@@ -73,7 +73,7 @@ TestWebGraphicsContext3D::TestWebGraphicsContext3D()
       bound_buffer_(0),
       weak_ptr_factory_(this) {
   CreateNamespace();
-  set_support_image(true);
+  set_have_extension_egl_image(true);  // For stream textures.
 }
 
 TestWebGraphicsContext3D::~TestWebGraphicsContext3D() {
@@ -608,7 +608,7 @@ GLuint TestWebGraphicsContext3D::createImageCHROMIUM(ClientBuffer buffer,
                                                      GLsizei width,
                                                      GLsizei height,
                                                      GLenum internalformat) {
-  DCHECK_EQ(GL_RGBA, static_cast<int>(internalformat));
+  DCHECK(internalformat == GL_RGB || internalformat == GL_RGBA);
   GLuint image_id = NextImageId();
   base::AutoLock lock(namespace_->lock);
   std::unordered_set<unsigned>& images = namespace_->images;
@@ -631,7 +631,7 @@ GLuint TestWebGraphicsContext3D::createGpuMemoryBufferImageCHROMIUM(
     GLsizei height,
     GLenum internalformat,
     GLenum usage) {
-  DCHECK_EQ(GL_RGBA, static_cast<int>(internalformat));
+  DCHECK(internalformat == GL_RGB || internalformat == GL_RGBA);
   GLuint image_id = NextImageId();
   base::AutoLock lock(namespace_->lock);
   std::unordered_set<unsigned>& images = namespace_->images;
@@ -639,21 +639,16 @@ GLuint TestWebGraphicsContext3D::createGpuMemoryBufferImageCHROMIUM(
   return image_id;
 }
 
-void TestWebGraphicsContext3D::getImageivCHROMIUM(GLuint image_id,
-                                                  GLenum param,
-                                                  GLint* data) {
-  DCHECK_EQ(GL_GPU_MEMORY_BUFFER_ID, static_cast<int>(param));
-  base::AutoLock lock(namespace_->lock);
-  std::unordered_set<unsigned>& images = namespace_->images;
-  *data = images.find(image_id) == images.end() ? -1 : 1;
-}
-
 GLuint64 TestWebGraphicsContext3D::insertFenceSync() {
   return next_insert_fence_sync_++;
 }
 
 void TestWebGraphicsContext3D::genSyncToken(GLuint64 fence_sync,
                                             GLbyte* sync_token) {
+  // Don't return a valid sync token if context is lost. This matches behavior
+  // of CommandBufferProxyImpl.
+  if (context_lost_)
+    return;
   gpu::SyncToken sync_token_data(gpu::CommandBufferNamespace::GPU_IO, 0,
                                  gpu::CommandBufferId(), fence_sync);
   sync_token_data.SetVerifyFlush();
