@@ -4,7 +4,7 @@
 
 #include "cc/tiles/tile_draw_info.h"
 
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "cc/base/math_util.h"
 
 namespace cc {
