@@ -18,6 +18,7 @@
 #include "base/memory/ptr_util.h"
 #include "base/metrics/histogram.h"
 #include "base/numerics/safe_conversions.h"
+#include "base/optional.h"
 #include "base/threading/thread_checker.h"
 #include "base/trace_event/trace_event_argument.h"
 #include "cc/base/histograms.h"
@@ -37,10 +38,45 @@ namespace {
 // a tile is of solid color.
 const bool kUseColorEstimator = true;
 
+// TODO(enne): remove this histogram and its monitoring in M58 once there is
+// enough new data from the other two raster task timers.
 DEFINE_SCOPED_UMA_HISTOGRAM_AREA_TIMER(
-    ScopedRasterTaskTimer,
+    ScopedGeneralRasterTaskTimer,
     "Compositing.%s.RasterTask.RasterUs",
-    "Compositing.%s.RasterTask.RasterPixelsPerMs");
+    "Compositing.%s.RasterTask.RasterPixelsPerMs2");
+
+DEFINE_SCOPED_UMA_HISTOGRAM_AREA_TIMER(
+    ScopedSoftwareRasterTaskTimer,
+    "Compositing.%s.RasterTask.RasterUs.Software",
+    "Compositing.%s.RasterTask.RasterPixelsPerMs2.Software");
+
+DEFINE_SCOPED_UMA_HISTOGRAM_AREA_TIMER(
+    ScopedGpuRasterTaskTimer,
+    "Compositing.%s.RasterTask.RasterUs.Gpu",
+    "Compositing.%s.RasterTask.RasterPixelsPerMs2.Gpu");
+
+class ScopedRasterTaskTimer {
+ public:
+  explicit ScopedRasterTaskTimer(bool use_gpu_rasterization) {
+    if (use_gpu_rasterization)
+      gpu_timer_.emplace();
+    else
+      software_timer_.emplace();
+  }
+
+  void SetArea(int area) {
+    general_timer_.SetArea(area);
+    if (software_timer_)
+      software_timer_->SetArea(area);
+    if (gpu_timer_)
+      gpu_timer_->SetArea(area);
+  }
+
+ private:
+  ScopedGeneralRasterTaskTimer general_timer_;
+  base::Optional<ScopedSoftwareRasterTaskTimer> software_timer_;
+  base::Optional<ScopedGpuRasterTaskTimer> gpu_timer_;
+};
 
 class RasterTaskImpl : public TileTask {
  public:
@@ -50,18 +86,19 @@ class RasterTaskImpl : public TileTask {
                  scoped_refptr<RasterSource> raster_source,
                  const RasterSource::PlaybackSettings& playback_settings,
                  TileResolution tile_resolution,
+                 gfx::Rect invalidated_rect,
                  uint64_t source_prepare_tiles_id,
                  std::unique_ptr<RasterBuffer> raster_buffer,
                  TileTask::Vector* dependencies,
-                 bool supports_concurrent_execution)
-      : TileTask(supports_concurrent_execution, dependencies),
+                 bool is_gpu_rasterization)
+      : TileTask(!is_gpu_rasterization, dependencies),
         tile_manager_(tile_manager),
-        tile_(tile),
+        tile_id_(tile->id()),
         resource_(resource),
         raster_source_(std::move(raster_source)),
         content_rect_(tile->content_rect()),
-        invalid_content_rect_(tile->invalidated_content_rect()),
-        contents_scale_(tile->contents_scale()),
+        invalid_content_rect_(invalidated_rect),
+        raster_scale_(tile->contents_scale()),
         playback_settings_(playback_settings),
         tile_resolution_(tile_resolution),
         layer_id_(tile->layer_id()),
@@ -69,6 +106,7 @@ class RasterTaskImpl : public TileTask {
         tile_tracing_id_(static_cast<void*>(tile)),
         new_content_id_(tile->id()),
         source_frame_number_(tile->source_frame_number()),
+        is_gpu_rasterization_(is_gpu_rasterization),
         raster_buffer_(std::move(raster_buffer)) {
     DCHECK(origin_thread_checker_.CalledOnValidThread());
   }
@@ -83,14 +121,14 @@ class RasterTaskImpl : public TileTask {
 
     frame_viewer_instrumentation::ScopedRasterTask raster_task(
         tile_tracing_id_, tile_resolution_, source_frame_number_, layer_id_);
-    ScopedRasterTaskTimer timer;
+    ScopedRasterTaskTimer timer(is_gpu_rasterization_);
     timer.SetArea(content_rect_.size().GetArea());
 
     DCHECK(raster_source_);
 
     raster_buffer_->Playback(raster_source_.get(), content_rect_,
                              invalid_content_rect_, new_content_id_,
-                             contents_scale_, playback_settings_);
+                             raster_scale_, playback_settings_);
   }
 
   // Overridden from TileTask:
@@ -100,7 +138,7 @@ class RasterTaskImpl : public TileTask {
     // Here calling state().IsCanceled() is thread-safe, because this task is
     // already concluded as FINISHED or CANCELLED and no longer will be worked
     // upon by task graph runner.
-    tile_manager_->OnRasterTaskCompleted(std::move(raster_buffer_), tile_,
+    tile_manager_->OnRasterTaskCompleted(std::move(raster_buffer_), tile_id_,
                                          resource_, state().IsCanceled());
   }
 
@@ -117,14 +155,14 @@ class RasterTaskImpl : public TileTask {
   // origin thread. These are not thread-safe and should be accessed only in
   // origin thread. Ensure their access by checking CalledOnValidThread().
   TileManager* tile_manager_;
-  Tile* tile_;
+  Tile::Id tile_id_;
   Resource* resource_;
 
   // The following members should be used for running the task.
   scoped_refptr<RasterSource> raster_source_;
   gfx::Rect content_rect_;
   gfx::Rect invalid_content_rect_;
-  float contents_scale_;
+  float raster_scale_;
   RasterSource::PlaybackSettings playback_settings_;
   TileResolution tile_resolution_;
   int layer_id_;
@@ -132,6 +170,7 @@ class RasterTaskImpl : public TileTask {
   void* tile_tracing_id_;
   uint64_t new_content_id_;
   int source_frame_number_;
+  bool is_gpu_rasterization_;
   std::unique_ptr<RasterBuffer> raster_buffer_;
 
   DISALLOW_COPY_AND_ASSIGN(RasterTaskImpl);
@@ -178,11 +217,10 @@ void InsertNodeForTask(TaskGraph* graph,
                        uint16_t priority,
                        size_t dependencies) {
   DCHECK(std::find_if(graph->nodes.begin(), graph->nodes.end(),
-                      [task](const TaskGraph::Node& node) {
+                      [&task](const TaskGraph::Node& node) {
                         return node.task == task;
                       }) == graph->nodes.end());
-  graph->nodes.push_back(
-      TaskGraph::Node(task, category, priority, dependencies));
+  graph->nodes.emplace_back(task, category, priority, dependencies);
 }
 
 void InsertNodeForDecodeTask(TaskGraph* graph,
@@ -304,13 +342,15 @@ RasterTaskCompletionStatsAsValue(const RasterTaskCompletionStats& stats) {
   return std::move(state);
 }
 
-TileManager::TileManager(TileManagerClient* client,
-                         base::SequencedTaskRunner* task_runner,
-                         size_t scheduled_raster_task_limit,
-                         bool use_partial_raster,
-                         int max_preraster_distance_in_screen_pixels)
+TileManager::TileManager(
+    TileManagerClient* client,
+    base::SequencedTaskRunner* origin_task_runner,
+    scoped_refptr<base::SequencedTaskRunner> image_worker_task_runner,
+    size_t scheduled_raster_task_limit,
+    bool use_partial_raster,
+    bool check_tile_priority_inversion)
     : client_(client),
-      task_runner_(task_runner),
+      task_runner_(origin_task_runner),
       resource_pool_(nullptr),
       tile_task_manager_(nullptr),
       scheduled_raster_task_limit_(scheduled_raster_task_limit),
@@ -319,6 +359,8 @@ TileManager::TileManager(TileManagerClient* client,
       all_tiles_that_need_to_be_rasterized_are_scheduled_(true),
       did_check_for_completed_tasks_since_last_schedule_tasks_(true),
       did_oom_on_last_assign_(false),
+      image_controller_(origin_task_runner,
+                        std::move(image_worker_task_runner)),
       more_tiles_need_prepare_check_notifier_(
           task_runner_,
           base::Bind(&TileManager::CheckIfMoreTilesNeedToBePrepared,
@@ -329,8 +371,7 @@ TileManager::TileManager(TileManagerClient* client,
       has_scheduled_tile_tasks_(false),
       prepare_tiles_count_(0u),
       next_tile_id_(0u),
-      max_preraster_distance_in_screen_pixels_(
-          max_preraster_distance_in_screen_pixels),
+      check_tile_priority_inversion_(check_tile_priority_inversion),
       task_set_finished_weak_ptr_factory_(this) {}
 
 TileManager::~TileManager() {
@@ -349,66 +390,39 @@ void TileManager::FinishTasksAndCleanUp() {
 
   raster_buffer_provider_->Shutdown();
 
-  // Now that all tasks have been finished, we can clear any |orphan_tasks_|.
-  orphan_tasks_.clear();
-
   tile_task_manager_->CheckForCompletedTasks();
 
-  FreeResourcesForReleasedTiles();
-  CleanUpReleasedTiles();
-
   tile_task_manager_ = nullptr;
   resource_pool_ = nullptr;
   more_tiles_need_prepare_check_notifier_.Cancel();
   signals_check_notifier_.Cancel();
   task_set_finished_weak_ptr_factory_.InvalidateWeakPtrs();
+  raster_buffer_provider_ = nullptr;
 
-  for (auto& draw_image_pair : locked_images_)
-    image_decode_controller_->UnrefImage(draw_image_pair.first);
-  locked_images_.clear();
+  image_controller_.SetImageDecodeCache(nullptr);
+  locked_image_tasks_.clear();
 }
 
 void TileManager::SetResources(ResourcePool* resource_pool,
-                               ImageDecodeController* image_decode_controller,
-                               TileTaskManager* tile_task_manager,
+                               ImageDecodeCache* image_decode_cache,
+                               TaskGraphRunner* task_graph_runner,
                                RasterBufferProvider* raster_buffer_provider,
                                size_t scheduled_raster_task_limit,
                                bool use_gpu_rasterization) {
   DCHECK(!tile_task_manager_);
-  DCHECK(tile_task_manager);
+  DCHECK(task_graph_runner);
 
   use_gpu_rasterization_ = use_gpu_rasterization;
   scheduled_raster_task_limit_ = scheduled_raster_task_limit;
   resource_pool_ = resource_pool;
-  image_decode_controller_ = image_decode_controller;
-  tile_task_manager_ = tile_task_manager;
+  image_controller_.SetImageDecodeCache(image_decode_cache);
+  tile_task_manager_ = TileTaskManagerImpl::Create(task_graph_runner);
   raster_buffer_provider_ = raster_buffer_provider;
 }
 
 void TileManager::Release(Tile* tile) {
-  released_tiles_.push_back(tile);
-}
-
-void TileManager::FreeResourcesForReleasedTiles() {
-  for (auto* tile : released_tiles_)
-    FreeResourcesForTile(tile);
-}
-
-void TileManager::CleanUpReleasedTiles() {
-  std::vector<Tile*> tiles_to_retain;
-  for (auto* tile : released_tiles_) {
-    if (tile->HasRasterTask()) {
-      tiles_to_retain.push_back(tile);
-      continue;
-    }
-
-    DCHECK(!tile->draw_info().has_resource());
-    DCHECK(tiles_.find(tile->id()) != tiles_.end());
-    tiles_.erase(tile->id());
-
-    delete tile;
-  }
-  released_tiles_.swap(tiles_to_retain);
+  FreeResourcesForTile(tile);
+  tiles_.erase(tile->id());
 }
 
 void TileManager::DidFinishRunningTileTasksRequiredForActivation() {
@@ -416,6 +430,8 @@ void TileManager::DidFinishRunningTileTasksRequiredForActivation() {
                "TileManager::DidFinishRunningTileTasksRequiredForActivation");
   TRACE_EVENT_ASYNC_STEP_INTO1("cc", "ScheduledTasks", this, "running", "state",
                                ScheduledTasksStateAsValue());
+  // TODO(vmpstr): Temporary check to debug crbug.com/642927.
+  CHECK(tile_task_manager_);
   signals_.ready_to_activate = true;
   signals_check_notifier_.Schedule();
 }
@@ -424,6 +440,8 @@ void TileManager::DidFinishRunningTileTasksRequiredForDraw() {
   TRACE_EVENT0("cc", "TileManager::DidFinishRunningTileTasksRequiredForDraw");
   TRACE_EVENT_ASYNC_STEP_INTO1("cc", "ScheduledTasks", this, "running", "state",
                                ScheduledTasksStateAsValue());
+  // TODO(vmpstr): Temporary check to debug crbug.com/642927.
+  CHECK(tile_task_manager_);
   signals_.ready_to_draw = true;
   signals_check_notifier_.Schedule();
 }
@@ -436,14 +454,13 @@ void TileManager::DidFinishRunningAllTileTasks() {
 
   has_scheduled_tile_tasks_ = false;
 
-  bool memory_usage_above_limit = resource_pool_->memory_usage_bytes() >
-                                  global_state_.soft_memory_limit_in_bytes;
-
   if (all_tiles_that_need_to_be_rasterized_are_scheduled_ &&
-      !memory_usage_above_limit) {
+      !resource_pool_->ResourceUsageTooHigh()) {
     // TODO(ericrk): We should find a better way to safely handle re-entrant
     // notifications than always having to schedule a new task.
     // http://crbug.com/498439
+    // TODO(vmpstr): Temporary check to debug crbug.com/642927.
+    CHECK(tile_task_manager_);
     signals_.all_tile_tasks_completed = true;
     signals_check_notifier_.Schedule();
     return;
@@ -475,9 +492,6 @@ bool TileManager::PrepareTiles(
     did_check_for_completed_tasks_since_last_schedule_tasks_ = true;
   }
 
-  FreeResourcesForReleasedTiles();
-  CleanUpReleasedTiles();
-
   PrioritizedWorkToSchedule prioritized_work = AssignGpuMemoryToTiles();
 
   // Inform the client that will likely require a draw if the highest priority
@@ -634,11 +648,13 @@ TileManager::PrioritizedWorkToSchedule TileManager::AssignGpuMemoryToTiles() {
     }
 
     bool tile_is_needed_now = priority.priority_bin == TilePriority::NOW;
-    if (tile->use_picture_analysis() && kUseColorEstimator) {
+    if (!tile->is_solid_color_analysis_performed() &&
+        tile->use_picture_analysis() && kUseColorEstimator) {
       // We analyze for solid color here, to decide to continue
       // or drop the tile for scheduling and raster.
       // TODO(sohanjg): Check if we could use a shared analysis
       // canvas which is reset between tiles.
+      tile->set_solid_color_analysis_performed(true);
       SkColor color = SK_ColorTRANSPARENT;
       bool is_solid_color =
           prioritized_tile.raster_source()->PerformSolidColorAnalysis(
@@ -655,8 +671,7 @@ TileManager::PrioritizedWorkToSchedule TileManager::AssignGpuMemoryToTiles() {
 
     // Prepaint tiles that are far away are only processed for images.
     if (!tile->required_for_activation() && !tile->required_for_draw() &&
-        priority.distance_to_visible >
-            max_preraster_distance_in_screen_pixels_) {
+        prioritized_tile.is_process_for_images_only()) {
       work_to_schedule.tiles_to_process_for_images.push_back(prioritized_tile);
       continue;
     }
@@ -717,6 +732,41 @@ TileManager::PrioritizedWorkToSchedule TileManager::AssignGpuMemoryToTiles() {
       tile->draw_info().set_was_a_prepaint_tile();
   }
 
+  // Debugging to check that remaining tiles in the priority queue are not in
+  // the NOW bin and are required for neither activation nor draw.
+  // This runs if the following conditions hold:
+  //  - check_tile_priority_inversion has been enabled.
+  //  - the loop above has processed all tiles that would be needed for any
+  //    signals to fire (that is,
+  //    all_tiles_that_need_to_be_rasterized_are_scheduled_ is true)
+  //  - Memory limit policy allows for any tiles to be scheduled at all (ie it's
+  //    not ALLOW_NOTHING).
+  if (check_tile_priority_inversion_ &&
+      all_tiles_that_need_to_be_rasterized_are_scheduled_ &&
+      global_state_.memory_limit_policy != ALLOW_NOTHING) {
+    TilePriority::PriorityBin highest_bin_found = TilePriority::NOW;
+    for (; !raster_priority_queue->IsEmpty(); raster_priority_queue->Pop()) {
+      const PrioritizedTile& prioritized_tile = raster_priority_queue->Top();
+      Tile* tile = prioritized_tile.tile();
+      TilePriority priority = prioritized_tile.priority();
+
+      if (priority.priority_bin > highest_bin_found)
+        highest_bin_found = priority.priority_bin;
+
+      CHECK_NE(TilePriority::NOW, priority.priority_bin)
+          << "mode: " << global_state_.tree_priority
+          << " highest bin: " << highest_bin_found;
+      CHECK(!tile->required_for_activation())
+          << "mode: " << global_state_.tree_priority
+          << " bin: " << priority.priority_bin
+          << " highest bin: " << highest_bin_found;
+      CHECK(!tile->required_for_draw())
+          << "mode: " << global_state_.tree_priority
+          << " bin: " << priority.priority_bin
+          << " highest bin: " << highest_bin_found;
+    }
+  }
+
   // Note that we should try and further reduce memory in case the above loop
   // didn't reduce memory. This ensures that we always release as many resources
   // as possible to stay within the memory limit.
@@ -745,7 +795,7 @@ TileManager::PrioritizedWorkToSchedule TileManager::AssignGpuMemoryToTiles() {
 void TileManager::FreeResourcesForTile(Tile* tile) {
   TileDrawInfo& draw_info = tile->draw_info();
   if (draw_info.resource_) {
-    resource_pool_->ReleaseResource(draw_info.resource_, tile->id());
+    resource_pool_->ReleaseResource(draw_info.resource_);
     draw_info.resource_ = nullptr;
   }
 }
@@ -788,6 +838,8 @@ void TileManager::ScheduleTasks(
 
   graph_.Reset();
 
+  gfx::ColorSpace target_color_space = client_->GetTileColorSpace();
+
   scoped_refptr<TileTask> required_for_activation_done_task =
       CreateTaskSetFinishedTask(
           &TileManager::DidFinishRunningTileTasksRequiredForActivation);
@@ -806,7 +858,11 @@ void TileManager::ScheduleTasks(
     DCHECK(!tile->draw_info().resource_);
 
     if (!tile->raster_task_)
-      tile->raster_task_ = CreateRasterTask(prioritized_tile);
+      tile->raster_task_ = CreateRasterTask(
+          prioritized_tile,
+          prioritized_tile.raster_source()->HasImpliedColorSpace()
+              ? prioritized_tile.raster_source()->GetImpliedColorSpace()
+              : target_color_space);
 
     TileTask* task = tile->raster_task_.get();
 
@@ -837,54 +893,51 @@ void TileManager::ScheduleTasks(
 
   const std::vector<PrioritizedTile>& tiles_to_process_for_images =
       work_to_schedule.tiles_to_process_for_images;
-  std::vector<std::pair<DrawImage, scoped_refptr<TileTask>>> new_locked_images;
+  std::vector<DrawImage> new_locked_images;
   for (const PrioritizedTile& prioritized_tile : tiles_to_process_for_images) {
     Tile* tile = prioritized_tile.tile();
 
     std::vector<DrawImage> images;
     prioritized_tile.raster_source()->GetDiscardableImagesInRect(
         tile->enclosing_layer_rect(), tile->contents_scale(), &images);
-    ImageDecodeController::TracingInfo tracing_info(
-        prepare_tiles_count_, prioritized_tile.priority().priority_bin);
-    for (DrawImage& draw_image : images) {
-      scoped_refptr<TileTask> task;
-      bool need_to_unref_when_finished =
-          image_decode_controller_->GetTaskForImageAndRef(draw_image,
-                                                          tracing_info, &task);
-      // We only care about images that need to be locked (ie they need to be
-      // unreffed later).
-      if (!need_to_unref_when_finished)
-        continue;
-      new_locked_images.emplace_back(draw_image, task);
-
-      // If there's no actual task associated with this image, then we're done.
-      if (!task)
-        continue;
-
-      auto decode_it = std::find_if(graph_.nodes.begin(), graph_.nodes.end(),
-                                    [&task](const TaskGraph::Node& node) {
-                                      return node.task == task.get();
-                                    });
-      // If this task is already in the graph, then we don't have to insert it.
-      if (decode_it != graph_.nodes.end())
-        continue;
-
-      InsertNodeForDecodeTask(&graph_, task.get(), false, priority++);
-      all_count++;
-      graph_.edges.push_back(TaskGraph::Edge(task.get(), all_done_task.get()));
-    }
+    new_locked_images.insert(new_locked_images.end(), images.begin(),
+                             images.end());
   }
 
-  for (auto& draw_image_pair : locked_images_)
-    image_decode_controller_->UnrefImage(draw_image_pair.first);
-  // The old locked images have to stay around until past the ScheduleTasks call
-  // below, so we do a swap instead of a move.
-  locked_images_.swap(new_locked_images);
+  // TODO(vmpstr): SOON is misleading here, but these images can come from
+  // several diffent tiles. Rethink what we actually want to trace here. Note
+  // that I'm using SOON, since it can't be NOW (these are prepaint).
+  ImageDecodeCache::TracingInfo tracing_info(prepare_tiles_count_,
+                                             TilePriority::SOON);
+  std::vector<scoped_refptr<TileTask>> new_locked_image_tasks =
+      image_controller_.SetPredecodeImages(std::move(new_locked_images),
+                                           tracing_info);
+
+  for (auto& task : new_locked_image_tasks) {
+    auto decode_it = std::find_if(graph_.nodes.begin(), graph_.nodes.end(),
+                                  [&task](const TaskGraph::Node& node) {
+                                    return node.task == task.get();
+                                  });
+    // If this task is already in the graph, then we don't have to insert it.
+    if (decode_it != graph_.nodes.end())
+      continue;
+
+    InsertNodeForDecodeTask(&graph_, task.get(), false, priority++);
+    all_count++;
+    graph_.edges.push_back(TaskGraph::Edge(task.get(), all_done_task.get()));
+  }
+
+  // The old locked images tasks have to stay around until past the
+  // ScheduleTasks call below, so we do a swap instead of a move.
+  // TODO(crbug.com/647402): Have the tile_task_manager keep a ref on the tasks,
+  // since it makes it awkward for the callers to keep refs on tasks that only
+  // exist within the task graph runner.
+  locked_image_tasks_.swap(new_locked_image_tasks);
 
   // We must reduce the amount of unused resources before calling
   // ScheduleTasks to prevent usage from rising above limits.
   resource_pool_->ReduceResourceUsage();
-  image_decode_controller_->ReduceCacheUsage();
+  image_controller_.ReduceMemoryUsage();
 
   // Insert nodes for our task completion tasks. We enqueue these using
   // NONCONCURRENT_FOREGROUND category this is the highest prioirty category and
@@ -908,17 +961,6 @@ void TileManager::ScheduleTasks(
   // in |raster_queue_|.
   tile_task_manager_->ScheduleTasks(&graph_);
 
-  // It's now safe to clean up orphan tasks as raster worker pool is not
-  // allowed to keep around unreferenced raster tasks after ScheduleTasks() has
-  // been called.
-  orphan_tasks_.clear();
-
-  // It's also now safe to replace our *_done_task_ tasks.
-  required_for_activation_done_task_ =
-      std::move(required_for_activation_done_task);
-  required_for_draw_done_task_ = std::move(required_for_draw_done_task);
-  all_done_task_ = std::move(all_done_task);
-
   did_check_for_completed_tasks_since_last_schedule_tasks_ = false;
 
   TRACE_EVENT_ASYNC_STEP_INTO1("cc", "ScheduledTasks", this, "running", "state",
@@ -926,24 +968,27 @@ void TileManager::ScheduleTasks(
 }
 
 scoped_refptr<TileTask> TileManager::CreateRasterTask(
-    const PrioritizedTile& prioritized_tile) {
+    const PrioritizedTile& prioritized_tile,
+    const gfx::ColorSpace& color_space) {
   Tile* tile = prioritized_tile.tile();
 
   // Get the resource.
   uint64_t resource_content_id = 0;
   Resource* resource = nullptr;
-  if (use_partial_raster_ && tile->invalidated_id()) {
-    // TODO(danakj): For resources that are in use, we should still grab them
-    // and copy from them instead of rastering everything. crbug.com/492754
-    resource =
-        resource_pool_->TryAcquireResourceWithContentId(tile->invalidated_id());
+  gfx::Rect invalidated_rect = tile->invalidated_content_rect();
+  if (UsePartialRaster() && tile->invalidated_id()) {
+    resource = resource_pool_->TryAcquireResourceForPartialRaster(
+        tile->id(), tile->invalidated_content_rect(), tile->invalidated_id(),
+        &invalidated_rect);
   }
+
   if (resource) {
     resource_content_id = tile->invalidated_id();
     DCHECK_EQ(DetermineResourceFormat(tile), resource->format());
   } else {
     resource = resource_pool_->AcquireResource(tile->desired_texture_size(),
-                                               DetermineResourceFormat(tile));
+                                               DetermineResourceFormat(tile),
+                                               color_space);
   }
 
   // For LOW_RESOLUTION tiles, we don't draw or predecode images.
@@ -962,83 +1007,86 @@ scoped_refptr<TileTask> TileManager::CreateRasterTask(
 
   // We can skip the image hijack canvas if we have no images.
   playback_settings.use_image_hijack_canvas = !images.empty();
-  ImageDecodeController::TracingInfo tracing_info(
+
+  // Get the tasks for the required images.
+  ImageDecodeCache::TracingInfo tracing_info(
       prepare_tiles_count_, prioritized_tile.priority().priority_bin);
-  for (auto it = images.begin(); it != images.end();) {
-    scoped_refptr<TileTask> task;
-    bool need_to_unref_when_finished =
-        image_decode_controller_->GetTaskForImageAndRef(*it, tracing_info,
-                                                        &task);
-    if (task)
-      decode_tasks.push_back(task);
+  image_controller_.GetTasksForImagesAndRef(&images, &decode_tasks,
+                                            tracing_info);
 
-    if (need_to_unref_when_finished)
-      ++it;
-    else
-      it = images.erase(it);
-  }
-
-  bool supports_concurrent_execution = !use_gpu_rasterization_;
   std::unique_ptr<RasterBuffer> raster_buffer =
       raster_buffer_provider_->AcquireBufferForRaster(
           resource, resource_content_id, tile->invalidated_id());
   return make_scoped_refptr(new RasterTaskImpl(
       this, tile, resource, prioritized_tile.raster_source(), playback_settings,
-      prioritized_tile.priority().resolution, prepare_tiles_count_,
-      std::move(raster_buffer), &decode_tasks, supports_concurrent_execution));
+      prioritized_tile.priority().resolution, invalidated_rect,
+      prepare_tiles_count_, std::move(raster_buffer), &decode_tasks,
+      use_gpu_rasterization_));
+}
+
+void TileManager::ResetSignalsForTesting() {
+  signals_.reset();
 }
 
 void TileManager::OnRasterTaskCompleted(
     std::unique_ptr<RasterBuffer> raster_buffer,
-    Tile* tile,
+    Tile::Id tile_id,
     Resource* resource,
     bool was_canceled) {
-  DCHECK(tile);
-  DCHECK(tiles_.find(tile->id()) != tiles_.end());
   raster_buffer_provider_->ReleaseBufferForRaster(std::move(raster_buffer));
 
-  TileDrawInfo& draw_info = tile->draw_info();
-  DCHECK(tile->raster_task_.get());
-  orphan_tasks_.push_back(tile->raster_task_);
-  tile->raster_task_ = nullptr;
+  auto found = tiles_.find(tile_id);
+  Tile* tile = nullptr;
+  if (found != tiles_.end()) {
+    tile = found->second;
+    DCHECK(tile->raster_task_.get());
+    tile->raster_task_ = nullptr;
+  }
 
   // Unref all the images.
-  auto images_it = scheduled_draw_images_.find(tile->id());
-  const std::vector<DrawImage>& images = images_it->second;
-  for (const auto& image : images)
-    image_decode_controller_->UnrefImage(image);
+  auto images_it = scheduled_draw_images_.find(tile_id);
+  image_controller_.UnrefImages(images_it->second);
   scheduled_draw_images_.erase(images_it);
 
   if (was_canceled) {
     ++flush_stats_.canceled_count;
-    // TODO(ericrk): If more partial raster work is done in the future, it may
-    // be worth returning the resource to the pool with its previous ID (not
-    // currently tracked). crrev.com/1370333002/#ps40001 has a possible method
-    // of achieving this.
-    resource_pool_->ReleaseResource(resource, 0 /* content_id */);
+    resource_pool_->ReleaseResource(resource);
     return;
   }
 
+  resource_pool_->OnContentReplaced(resource->id(), tile_id);
   ++flush_stats_.completed_count;
 
+  if (!tile) {
+    resource_pool_->ReleaseResource(resource);
+    return;
+  }
+
+  TileDrawInfo& draw_info = tile->draw_info();
   draw_info.set_use_resource();
   draw_info.resource_ = resource;
   draw_info.contents_swizzled_ = DetermineResourceRequiresSwizzle(tile);
 
   DCHECK(draw_info.IsReadyToDraw());
   draw_info.set_was_ever_ready_to_draw();
-
   client_->NotifyTileStateChanged(tile);
 }
 
-ScopedTilePtr TileManager::CreateTile(const Tile::CreateInfo& info,
-                                      int layer_id,
-                                      int source_frame_number,
-                                      int flags) {
+void TileManager::SetDecodedImageTracker(
+    DecodedImageTracker* decoded_image_tracker) {
+  // TODO(vmpstr): If the tile manager needs to request out-of-raster decodes,
+  // it should retain and use |decoded_image_tracker| here.
+  decoded_image_tracker->set_image_controller(&image_controller_);
+}
+
+std::unique_ptr<Tile> TileManager::CreateTile(const Tile::CreateInfo& info,
+                                              int layer_id,
+                                              int source_frame_number,
+                                              int flags) {
   // We need to have a tile task worker pool to do anything meaningful with
   // tiles.
   DCHECK(tile_task_manager_);
-  ScopedTilePtr tile(
+  std::unique_ptr<Tile> tile(
       new Tile(this, info, layer_id, source_frame_number, flags));
   DCHECK(tiles_.find(tile->id()) == tiles_.end());
 
@@ -1046,16 +1094,6 @@ ScopedTilePtr TileManager::CreateTile(const Tile::CreateInfo& info,
   return tile;
 }
 
-void TileManager::SetTileTaskManagerForTesting(
-    TileTaskManager* tile_task_manager) {
-  tile_task_manager_ = tile_task_manager;
-}
-
-void TileManager::SetRasterBufferProviderForTesting(
-    RasterBufferProvider* raster_buffer_provider) {
-  raster_buffer_provider_ = raster_buffer_provider;
-}
-
 bool TileManager::AreRequiredTilesReadyToDraw(
     RasterTilePriorityQueue::Type type) const {
   std::unique_ptr<RasterTilePriorityQueue> raster_priority_queue(
@@ -1066,7 +1104,10 @@ bool TileManager::AreRequiredTilesReadyToDraw(
   // have to iterate the queue to check whether the required tiles are ready to
   // draw.
   for (; !raster_priority_queue->IsEmpty(); raster_priority_queue->Pop()) {
-    if (!raster_priority_queue->Top().tile()->draw_info().IsReadyToDraw())
+    const auto& prioritized_tile = raster_priority_queue->Top();
+    // TODO(vmpstr): Check to debug crbug.com/622080. Remove when fixed.
+    CHECK_EQ(prioritized_tile.priority().priority_bin, TilePriority::NOW);
+    if (!prioritized_tile.tile()->draw_info().IsReadyToDraw())
       return false;
   }
 
@@ -1103,7 +1144,7 @@ void TileManager::CheckAndIssueSignals() {
   if (signals_.ready_to_activate && !signals_.did_notify_ready_to_activate) {
     signals_.ready_to_activate = false;
     if (IsReadyToActivate()) {
-      TRACE_EVENT0("disabled-by-default-cc.debug",
+      TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                    "TileManager::CheckAndIssueSignals - ready to activate");
       signals_.did_notify_ready_to_activate = true;
       client_->NotifyReadyToActivate();
@@ -1114,7 +1155,7 @@ void TileManager::CheckAndIssueSignals() {
   if (signals_.ready_to_draw && !signals_.did_notify_ready_to_draw) {
     signals_.ready_to_draw = false;
     if (IsReadyToDraw()) {
-      TRACE_EVENT0("disabled-by-default-cc.debug",
+      TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                    "TileManager::CheckAndIssueSignals - ready to draw");
       signals_.did_notify_ready_to_draw = true;
       client_->NotifyReadyToDraw();
@@ -1127,7 +1168,7 @@ void TileManager::CheckAndIssueSignals() {
     signals_.all_tile_tasks_completed = false;
     if (!has_scheduled_tile_tasks_) {
       TRACE_EVENT0(
-          "disabled-by-default-cc.debug",
+          TRACE_DISABLED_BY_DEFAULT("cc.debug"),
           "TileManager::CheckAndIssueSignals - all tile tasks completed");
       signals_.did_notify_all_tile_tasks_completed = true;
       client_->NotifyAllTileTasksCompleted();
@@ -1159,16 +1200,20 @@ void TileManager::CheckIfMoreTilesNeedToBePrepared() {
   // If we're not in SMOOTHNESS_TAKES_PRIORITY  mode, we should unlock all
   // images since we're technically going idle here at least for this frame.
   if (global_state_.tree_priority != SMOOTHNESS_TAKES_PRIORITY) {
-    for (auto& draw_image_pair : locked_images_)
-      image_decode_controller_->UnrefImage(draw_image_pair.first);
-    locked_images_.clear();
+    image_controller_.SetPredecodeImages(std::vector<DrawImage>(),
+                                         ImageDecodeCache::TracingInfo());
+    locked_image_tasks_.clear();
   }
 
-  FreeResourcesForReleasedTiles();
-
   resource_pool_->ReduceResourceUsage();
-  image_decode_controller_->ReduceCacheUsage();
+  image_controller_.ReduceMemoryUsage();
 
+  // TODO(vmpstr): Temporary check to debug crbug.com/642927.
+  CHECK(tile_task_manager_);
+
+  // Schedule all checks in case we're left with solid color tiles only.
+  signals_.ready_to_activate = true;
+  signals_.ready_to_draw = true;
   signals_.all_tile_tasks_completed = true;
   signals_check_notifier_.Schedule();
 
@@ -1192,28 +1237,23 @@ void TileManager::CheckIfMoreTilesNeedToBePrepared() {
   // even when OOM. Note that we can't reuse the queue we used for
   // AssignGpuMemoryToTiles, since the AssignGpuMemoryToTiles call could have
   // evicted some tiles that would not be picked up by the old raster queue.
-  bool need_to_signal_activate = MarkTilesOutOfMemory(client_->BuildRasterQueue(
+  MarkTilesOutOfMemory(client_->BuildRasterQueue(
       global_state_.tree_priority,
       RasterTilePriorityQueue::Type::REQUIRED_FOR_ACTIVATION));
-  bool need_to_signal_draw = MarkTilesOutOfMemory(client_->BuildRasterQueue(
+  MarkTilesOutOfMemory(client_->BuildRasterQueue(
       global_state_.tree_priority,
       RasterTilePriorityQueue::Type::REQUIRED_FOR_DRAW));
 
+  // TODO(vmpstr): Temporary check to debug crbug.com/642927.
+  CHECK(tile_task_manager_);
+
   DCHECK(IsReadyToActivate());
   DCHECK(IsReadyToDraw());
-  signals_.ready_to_activate = need_to_signal_activate;
-  signals_.ready_to_draw = need_to_signal_draw;
-  // TODO(ericrk): Investigate why we need to schedule this (not just call it
-  // inline). http://crbug.com/498439
-  signals_check_notifier_.Schedule();
 }
 
-bool TileManager::MarkTilesOutOfMemory(
+void TileManager::MarkTilesOutOfMemory(
     std::unique_ptr<RasterTilePriorityQueue> queue) const {
   // Mark required tiles as OOM so that we can activate/draw without them.
-  if (queue->IsEmpty())
-    return false;
-
   for (; !queue->IsEmpty(); queue->Pop()) {
     Tile* tile = queue->Top().tile();
     if (tile->draw_info().IsReadyToDraw())
@@ -1221,7 +1261,6 @@ bool TileManager::MarkTilesOutOfMemory(
     tile->draw_info().set_oom();
     client_->NotifyTileStateChanged(tile);
   }
-  return true;
 }
 
 ResourceFormat TileManager::DetermineResourceFormat(const Tile* tile) const {
@@ -1229,8 +1268,7 @@ ResourceFormat TileManager::DetermineResourceFormat(const Tile* tile) const {
 }
 
 bool TileManager::DetermineResourceRequiresSwizzle(const Tile* tile) const {
-  return raster_buffer_provider_->GetResourceRequiresSwizzle(
-      !tile->is_opaque());
+  return raster_buffer_provider_->IsResourceSwizzleRequired(!tile->is_opaque());
 }
 
 std::unique_ptr<base::trace_event::ConvertableToTraceFormat>
@@ -1246,6 +1284,11 @@ TileManager::ScheduledTasksStateAsValue() const {
   return std::move(state);
 }
 
+bool TileManager::UsePartialRaster() const {
+  return use_partial_raster_ &&
+         raster_buffer_provider_->CanPartialRasterIntoProvidedResource();
+}
+
 // Utility function that can be used to create a "Task set finished" task that
 // posts |callback| to |task_runner| when run.
 scoped_refptr<TileTask> TileManager::CreateTaskSetFinishedTask(
