@@ -14,19 +14,18 @@
 #include "cc/raster/synchronous_task_graph_runner.h"
 #include "cc/resources/resource_pool.h"
 #include "cc/test/begin_frame_args_test.h"
+#include "cc/test/fake_compositor_frame_sink.h"
+#include "cc/test/fake_compositor_frame_sink_client.h"
 #include "cc/test/fake_impl_task_runner_provider.h"
 #include "cc/test/fake_layer_tree_host_impl.h"
-#include "cc/test/fake_output_surface.h"
-#include "cc/test/fake_output_surface_client.h"
 #include "cc/test/fake_picture_layer_impl.h"
 #include "cc/test/fake_picture_layer_tiling_client.h"
 #include "cc/test/fake_raster_source.h"
 #include "cc/test/fake_recording_source.h"
 #include "cc/test/fake_tile_manager.h"
 #include "cc/test/fake_tile_task_manager.h"
-#include "cc/test/test_gpu_memory_buffer_manager.h"
+#include "cc/test/layer_tree_settings_for_testing.h"
 #include "cc/test/test_layer_tree_host_base.h"
-#include "cc/test/test_shared_bitmap_manager.h"
 #include "cc/test/test_task_graph_runner.h"
 #include "cc/test/test_tile_priorities.h"
 #include "cc/tiles/eviction_tile_priority_queue.h"
@@ -46,9 +45,10 @@ namespace {
 class TileManagerTilePriorityQueueTest : public TestLayerTreeHostBase {
  public:
   LayerTreeSettings CreateSettings() override {
-    LayerTreeSettings settings;
+    LayerTreeSettingsForTesting settings;
     settings.create_low_res_tiling = true;
-    settings.verify_clip_tree_calculations = true;
+    settings.renderer_settings.buffer_to_texture_target_map =
+        DefaultBufferToTextureTargetMapForTesting();
     return settings;
   }
 
@@ -550,7 +550,7 @@ TEST_F(TileManagerTilePriorityQueueTest, RasterTilePriorityQueueInvalidation) {
             actual_required_for_activation_tiles);
 }
 
-TEST_F(TileManagerTilePriorityQueueTest, ActivationComesBeforeEventually) {
+TEST_F(TileManagerTilePriorityQueueTest, ActivationComesBeforeSoon) {
   host_impl()->AdvanceToNextFrame(base::TimeDelta::FromMilliseconds(1));
 
   gfx::Size layer_bounds(1000, 1000);
@@ -579,12 +579,11 @@ TEST_F(TileManagerTilePriorityQueueTest, ActivationComesBeforeEventually) {
       SMOOTHNESS_TAKES_PRIORITY, RasterTilePriorityQueue::Type::ALL));
   EXPECT_FALSE(queue->IsEmpty());
 
-  // Get all the tiles that are NOW or SOON and make sure they are ready to
-  // draw.
+  // Get all the tiles that are NOW and make sure they are ready to draw.
   std::vector<Tile*> all_tiles;
   while (!queue->IsEmpty()) {
     PrioritizedTile prioritized_tile = queue->Top();
-    if (prioritized_tile.priority().priority_bin >= TilePriority::EVENTUALLY)
+    if (prioritized_tile.priority().priority_bin >= TilePriority::SOON)
       break;
 
     all_tiles.push_back(prioritized_tile.tile());
@@ -913,7 +912,9 @@ TEST_F(TileManagerTilePriorityQueueTest,
   host_impl()->pending_tree()->BuildLayerListAndPropertyTreesForTesting();
   host_impl()->pending_tree()->UpdateDrawProperties(update_lcd_text);
 
-  pending_child_layer->OnOpacityAnimated(0.0);
+  host_impl()->pending_tree()->property_trees()->effect_tree.OnOpacityAnimated(
+      0.0f, pending_child_layer->effect_tree_index(),
+      host_impl()->pending_tree());
 
   host_impl()->AdvanceToNextFrame(base::TimeDelta::FromMilliseconds(1));
   host_impl()->pending_tree()->UpdateDrawProperties(update_lcd_text);
@@ -1092,7 +1093,8 @@ TEST_F(TileManagerTilePriorityQueueTest,
       PictureLayerTilingSet::Create(
           ACTIVE_TREE, &client, settings.tiling_interest_area_padding,
           settings.skewport_target_time_in_seconds,
-          settings.skewport_extrapolation_limit_in_screen_pixels);
+          settings.skewport_extrapolation_limit_in_screen_pixels,
+          settings.max_preraster_distance_in_screen_pixels);
 
   scoped_refptr<FakeRasterSource> raster_source =
       FakeRasterSource::CreateFilled(layer_bounds);
@@ -1203,7 +1205,8 @@ TEST_F(TileManagerTilePriorityQueueTest,
       PictureLayerTilingSet::Create(
           ACTIVE_TREE, &client, settings.tiling_interest_area_padding,
           settings.skewport_target_time_in_seconds,
-          settings.skewport_extrapolation_limit_in_screen_pixels);
+          settings.skewport_extrapolation_limit_in_screen_pixels,
+          settings.max_preraster_distance_in_screen_pixels);
 
   scoped_refptr<FakeRasterSource> raster_source =
       FakeRasterSource::CreateFilled(layer_bounds);
@@ -1323,12 +1326,12 @@ TEST_F(TileManagerTilePriorityQueueTest,
   EXPECT_TRUE(host_impl()->is_likely_to_require_a_draw());
 
   Resource* resource = host_impl()->resource_pool()->AcquireResource(
-      gfx::Size(256, 256), RGBA_8888);
+      gfx::Size(256, 256), RGBA_8888, gfx::ColorSpace());
 
   host_impl()->tile_manager()->CheckIfMoreTilesNeedToBePreparedForTesting();
   EXPECT_FALSE(host_impl()->is_likely_to_require_a_draw());
 
-  host_impl()->resource_pool()->ReleaseResource(resource, 0);
+  host_impl()->resource_pool()->ReleaseResource(resource);
 }
 
 TEST_F(TileManagerTilePriorityQueueTest, DefaultMemoryPolicy) {
@@ -1360,7 +1363,7 @@ TEST_F(TileManagerTilePriorityQueueTest, RasterQueueAllUsesCorrectTileBounds) {
 
   std::unique_ptr<PictureLayerTilingSet> tiling_set =
       PictureLayerTilingSet::Create(WhichTree::ACTIVE_TREE, &pending_client,
-                                    1.0f, 1.0f, 1000);
+                                    1.0f, 1.0f, 1000, 1000.f);
   pending_client.set_twin_tiling_set(tiling_set.get());
 
   auto* tiling = tiling_set->AddTiling(1.0f, raster_source);
@@ -1470,37 +1473,29 @@ class TileManagerTest : public TestLayerTreeHostBase {
   // MockLayerTreeHostImpl allows us to intercept tile manager callbacks.
   class MockLayerTreeHostImpl : public FakeLayerTreeHostImpl {
    public:
-    MockLayerTreeHostImpl(
-        const LayerTreeSettings& settings,
-        TaskRunnerProvider* task_runner_provider,
-        SharedBitmapManager* manager,
-        TaskGraphRunner* task_graph_runner,
-        gpu::GpuMemoryBufferManager* gpu_memory_buffer_manager)
+    MockLayerTreeHostImpl(const LayerTreeSettings& settings,
+                          TaskRunnerProvider* task_runner_provider,
+                          TaskGraphRunner* task_graph_runner)
         : FakeLayerTreeHostImpl(settings,
                                 task_runner_provider,
-                                manager,
-                                task_graph_runner,
-                                gpu_memory_buffer_manager) {}
+                                task_graph_runner) {}
 
-    MOCK_METHOD0(NotifyAllTileTasksCompleted, void());
+    MOCK_METHOD0(NotifyReadyToActivate, void());
     MOCK_METHOD0(NotifyReadyToDraw, void());
+    MOCK_METHOD0(NotifyAllTileTasksCompleted, void());
   };
 
   std::unique_ptr<FakeLayerTreeHostImpl> CreateHostImpl(
       const LayerTreeSettings& settings,
       TaskRunnerProvider* task_runner_provider,
-      SharedBitmapManager* shared_bitmap_manager,
-      TaskGraphRunner* task_graph_runner,
-      gpu::GpuMemoryBufferManager* gpu_memory_buffer_manager) override {
-    return base::WrapUnique(new MockLayerTreeHostImpl(
-        settings, task_runner_provider, shared_bitmap_manager,
-        task_graph_runner, gpu_memory_buffer_manager));
+      TaskGraphRunner* task_graph_runner) override {
+    return base::MakeUnique<MockLayerTreeHostImpl>(
+        settings, task_runner_provider, task_graph_runner);
   }
 
-  // By default use SoftwareOutputSurface.
-  std::unique_ptr<OutputSurface> CreateOutputSurface() override {
-    return FakeOutputSurface::CreateSoftware(
-        base::WrapUnique(new SoftwareOutputDevice));
+  // By default use software compositing (no context provider).
+  std::unique_ptr<CompositorFrameSink> CreateCompositorFrameSink() override {
+    return FakeCompositorFrameSink::CreateSoftware();
   }
 
   MockLayerTreeHostImpl& MockHostImpl() {
@@ -1510,12 +1505,14 @@ class TileManagerTest : public TestLayerTreeHostBase {
 
 // Test to ensure that we call NotifyAllTileTasksCompleted when PrepareTiles is
 // called.
-TEST_F(TileManagerTest, AllWorkFinishedTest) {
+TEST_F(TileManagerTest, AllWorkFinished) {
   // Check with no tile work enqueued.
   {
     base::RunLoop run_loop;
     EXPECT_FALSE(
         host_impl()->tile_manager()->HasScheduledTileTasksForTesting());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToActivate());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToDraw());
     EXPECT_CALL(MockHostImpl(), NotifyAllTileTasksCompleted())
         .WillOnce(testing::Invoke([&run_loop]() { run_loop.Quit(); }));
     host_impl()->tile_manager()->PrepareTiles(host_impl()->global_tile_state());
@@ -1529,6 +1526,8 @@ TEST_F(TileManagerTest, AllWorkFinishedTest) {
     base::RunLoop run_loop;
     EXPECT_FALSE(
         host_impl()->tile_manager()->HasScheduledTileTasksForTesting());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToActivate());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToDraw());
     EXPECT_CALL(MockHostImpl(), NotifyAllTileTasksCompleted())
         .WillOnce(testing::Invoke([&run_loop]() { run_loop.Quit(); }));
     host_impl()->tile_manager()->PrepareTiles(host_impl()->global_tile_state());
@@ -1536,6 +1535,40 @@ TEST_F(TileManagerTest, AllWorkFinishedTest) {
     EXPECT_TRUE(host_impl()->tile_manager()->HasScheduledTileTasksForTesting());
     run_loop.Run();
   }
+
+  // Check that if callbacks are called by CheckIfMoreTilesNeedToBePrepared if
+  // they haven't been called already.
+  {
+    base::RunLoop run_loop;
+    EXPECT_FALSE(
+        host_impl()->tile_manager()->HasScheduledTileTasksForTesting());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToActivate());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToDraw());
+    EXPECT_CALL(MockHostImpl(), NotifyAllTileTasksCompleted())
+        .WillOnce(testing::Invoke([&run_loop]() { run_loop.Quit(); }));
+    host_impl()->tile_manager()->ResetSignalsForTesting();
+    host_impl()->tile_manager()->SetMoreTilesNeedToBeRasterizedForTesting();
+    host_impl()->tile_manager()->CheckIfMoreTilesNeedToBePreparedForTesting();
+    run_loop.Run();
+  }
+
+  // Same test as above but with SMOOTHNESS_TAKES_PRIORITY.
+  {
+    base::RunLoop run_loop;
+    EXPECT_FALSE(
+        host_impl()->tile_manager()->HasScheduledTileTasksForTesting());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToActivate());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToDraw());
+    EXPECT_CALL(MockHostImpl(), NotifyAllTileTasksCompleted())
+        .WillOnce(testing::Invoke([&run_loop]() { run_loop.Quit(); }));
+    host_impl()->tile_manager()->ResetSignalsForTesting();
+    auto global_state = host_impl()->global_tile_state();
+    global_state.tree_priority = SMOOTHNESS_TAKES_PRIORITY;
+    host_impl()->tile_manager()->SetGlobalStateForTesting(global_state);
+    host_impl()->tile_manager()->SetMoreTilesNeedToBeRasterizedForTesting();
+    host_impl()->tile_manager()->CheckIfMoreTilesNeedToBePreparedForTesting();
+    run_loop.Run();
+  }
 }
 
 TEST_F(TileManagerTest, ActivateAndDrawWhenOOM) {
@@ -1549,6 +1582,8 @@ TEST_F(TileManagerTest, ActivateAndDrawWhenOOM) {
     base::RunLoop run_loop;
     EXPECT_FALSE(
         host_impl()->tile_manager()->HasScheduledTileTasksForTesting());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToActivate());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToDraw());
     EXPECT_CALL(MockHostImpl(), NotifyAllTileTasksCompleted())
         .WillOnce(testing::Invoke([&run_loop]() { run_loop.Quit(); }));
     host_impl()->tile_manager()->PrepareTiles(global_state);
@@ -1565,6 +1600,8 @@ TEST_F(TileManagerTest, ActivateAndDrawWhenOOM) {
   {
     base::RunLoop run_loop;
     host_impl()->set_notify_tile_state_changed_called(false);
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToActivate());
+    EXPECT_CALL(MockHostImpl(), NotifyReadyToDraw());
     EXPECT_CALL(MockHostImpl(), NotifyAllTileTasksCompleted())
         .WillOnce(testing::Invoke([&run_loop]() { run_loop.Quit(); }));
     host_impl()->tile_manager()->PrepareTiles(global_state);
@@ -1660,11 +1697,11 @@ TEST_F(TileManagerTest, LowResHasNoImage) {
 class ActivationTasksDoNotBlockReadyToDrawTest : public TileManagerTest {
  protected:
   std::unique_ptr<TaskGraphRunner> CreateTaskGraphRunner() override {
-    return base::WrapUnique(new SynchronousTaskGraphRunner());
+    return base::MakeUnique<SynchronousTaskGraphRunner>();
   }
 
-  std::unique_ptr<OutputSurface> CreateOutputSurface() override {
-    return FakeOutputSurface::Create3d();
+  std::unique_ptr<CompositorFrameSink> CreateCompositorFrameSink() override {
+    return FakeCompositorFrameSink::Create3d();
   }
 
   LayerTreeSettings CreateSettings() override {
@@ -1738,8 +1775,8 @@ TEST_F(PartialRasterTileManagerTest, CancelledTasksHaveNoContentId) {
   // Create a FakeTileTaskManagerImpl and set it on the tile manager so that all
   // scheduled work is immediately cancelled.
 
-  FakeTileTaskManagerImpl tile_task_manager;
-  host_impl()->tile_manager()->SetTileTaskManagerForTesting(&tile_task_manager);
+  host_impl()->tile_manager()->SetTileTaskManagerForTesting(
+      base::MakeUnique<FakeTileTaskManagerImpl>());
 
   // Pick arbitrary IDs - they don't really matter as long as they're constant.
   const int kLayerId = 7;
@@ -1780,8 +1817,11 @@ TEST_F(PartialRasterTileManagerTest, CancelledTasksHaveNoContentId) {
   // Make sure that the tile we invalidated above was not returned to the pool
   // with its invalidated resource ID.
   host_impl()->resource_pool()->CheckBusyResources();
-  EXPECT_FALSE(host_impl()->resource_pool()->TryAcquireResourceWithContentId(
-      kInvalidatedId));
+  gfx::Rect total_invalidated_rect;
+  EXPECT_FALSE(host_impl()->resource_pool()->TryAcquireResourceForPartialRaster(
+      kInvalidatedId + 1, gfx::Rect(), kInvalidatedId,
+      &total_invalidated_rect));
+  EXPECT_EQ(gfx::Rect(), total_invalidated_rect);
 
   // Free our host_impl_ before the tile_task_manager we passed it, as it
   // will use that class in clean up.
@@ -1824,8 +1864,8 @@ void RunPartialRasterCheck(std::unique_ptr<LayerTreeHostImpl> host_impl,
 
   // Create a VerifyResourceContentIdTileTaskManager to ensure that the
   // raster task we see is created with |kExpectedId|.
-  FakeTileTaskManagerImpl tile_task_manager;
-  host_impl->tile_manager()->SetTileTaskManagerForTesting(&tile_task_manager);
+  host_impl->tile_manager()->SetTileTaskManagerForTesting(
+      base::MakeUnique<FakeTileTaskManagerImpl>());
 
   VerifyResourceContentIdRasterBufferProvider raster_buffer_provider(
       kExpectedId);
@@ -1833,9 +1873,10 @@ void RunPartialRasterCheck(std::unique_ptr<LayerTreeHostImpl> host_impl,
       &raster_buffer_provider);
 
   // Ensure there's a resource with our |kInvalidatedId| in the resource pool.
-  host_impl->resource_pool()->ReleaseResource(
-      host_impl->resource_pool()->AcquireResource(kTileSize, RGBA_8888),
-      kInvalidatedId);
+  auto* resource = host_impl->resource_pool()->AcquireResource(
+      kTileSize, RGBA_8888, gfx::ColorSpace());
+  host_impl->resource_pool()->OnContentReplaced(resource->id(), kInvalidatedId);
+  host_impl->resource_pool()->ReleaseResource(resource);
   host_impl->resource_pool()->CheckBusyResources();
 
   scoped_refptr<FakeRasterSource> pending_raster_source =
