@@ -12,16 +12,17 @@
 #include "base/trace_event/trace_event.h"
 #include "base/trace_event/trace_event_argument.h"
 #include "base/trace_event/trace_event_synthetic_delay.h"
-#include "cc/animation/animation_events.h"
 #include "cc/debug/benchmark_instrumentation.h"
 #include "cc/debug/devtools_instrumentation.h"
-#include "cc/input/top_controls_manager.h"
+#include "cc/input/browser_controls_offset_manager.h"
+#include "cc/output/compositor_frame_sink.h"
 #include "cc/output/context_provider.h"
-#include "cc/output/output_surface.h"
 #include "cc/scheduler/compositor_timing_history.h"
 #include "cc/scheduler/delay_based_time_source.h"
-#include "cc/trees/layer_tree_host.h"
+#include "cc/trees/layer_tree_host_in_process.h"
 #include "cc/trees/layer_tree_impl.h"
+#include "cc/trees/mutator_host.h"
+#include "cc/trees/proxy_main.h"
 #include "cc/trees/task_runner_provider.h"
 #include "gpu/command_buffer/client/gles2_interface.h"
 
@@ -36,22 +37,10 @@ unsigned int nextBeginFrameId = 0;
 
 }  // namespace
 
-std::unique_ptr<ProxyImpl> ProxyImpl::Create(
-    ChannelImpl* channel_impl,
-    LayerTreeHost* layer_tree_host,
-    TaskRunnerProvider* task_runner_provider,
-    std::unique_ptr<BeginFrameSource> external_begin_frame_source) {
-  return base::WrapUnique(
-      new ProxyImpl(channel_impl, layer_tree_host, task_runner_provider,
-                    std::move(external_begin_frame_source)));
-}
-
-ProxyImpl::ProxyImpl(
-    ChannelImpl* channel_impl,
-    LayerTreeHost* layer_tree_host,
-    TaskRunnerProvider* task_runner_provider,
-    std::unique_ptr<BeginFrameSource> external_begin_frame_source)
-    : layer_tree_host_id_(layer_tree_host->id()),
+ProxyImpl::ProxyImpl(base::WeakPtr<ProxyMain> proxy_main_weak_ptr,
+                     LayerTreeHostInProcess* layer_tree_host,
+                     TaskRunnerProvider* task_runner_provider)
+    : layer_tree_host_id_(layer_tree_host->GetId()),
       commit_completion_waits_for_activation_(false),
       commit_completion_event_(nullptr),
       activation_completion_event_(nullptr),
@@ -64,43 +53,30 @@ ProxyImpl::ProxyImpl(
           base::Bind(&ProxyImpl::RenewTreePriority, base::Unretained(this)),
           base::TimeDelta::FromSecondsD(
               kSmoothnessTakesPriorityExpirationDelay)),
-      external_begin_frame_source_(std::move(external_begin_frame_source)),
       rendering_stats_instrumentation_(
           layer_tree_host->rendering_stats_instrumentation()),
-      channel_impl_(channel_impl) {
+      proxy_main_weak_ptr_(proxy_main_weak_ptr) {
   TRACE_EVENT0("cc", "ProxyImpl::ProxyImpl");
   DCHECK(IsImplThread());
   DCHECK(IsMainThreadBlocked());
 
-  layer_tree_host_impl_ = layer_tree_host->CreateLayerTreeHostImpl(this);
+  // Double checking we set this correctly since double->int truncations are
+  // silent and have been done mistakenly: crbug.com/568120.
+  DCHECK(!smoothness_priority_expiration_notifier_.delay().is_zero());
 
-  SchedulerSettings scheduler_settings(
-      layer_tree_host->settings().ToSchedulerSettings());
+  layer_tree_host_impl_ = layer_tree_host->CreateLayerTreeHostImpl(this);
+  const LayerTreeSettings& settings = layer_tree_host->GetSettings();
+
+  SchedulerSettings scheduler_settings(settings.ToSchedulerSettings());
 
   std::unique_ptr<CompositorTimingHistory> compositor_timing_history(
       new CompositorTimingHistory(
           scheduler_settings.using_synchronous_renderer_compositor,
           CompositorTimingHistory::RENDERER_UMA,
           rendering_stats_instrumentation_));
-
-  BeginFrameSource* frame_source = external_begin_frame_source_.get();
-  if (!scheduler_settings.throttle_frame_production) {
-    // Unthrottled source takes precedence over external sources.
-    unthrottled_begin_frame_source_.reset(
-        new BackToBackBeginFrameSource(base::MakeUnique<DelayBasedTimeSource>(
-            task_runner_provider_->ImplThreadTaskRunner())));
-    frame_source = unthrottled_begin_frame_source_.get();
-  }
-  if (!frame_source) {
-    synthetic_begin_frame_source_.reset(
-        new DelayBasedBeginFrameSource(base::MakeUnique<DelayBasedTimeSource>(
-            task_runner_provider_->ImplThreadTaskRunner())));
-    frame_source = synthetic_begin_frame_source_.get();
-  }
-  scheduler_ =
-      Scheduler::Create(this, scheduler_settings, layer_tree_host_id_,
-                        task_runner_provider_->ImplThreadTaskRunner(),
-                        frame_source, std::move(compositor_timing_history));
+  scheduler_.reset(new Scheduler(this, scheduler_settings, layer_tree_host_id_,
+                                 task_runner_provider_->ImplThreadTaskRunner(),
+                                 std::move(compositor_timing_history)));
 
   DCHECK_EQ(scheduler_->visible(), layer_tree_host_impl_->visible());
 }
@@ -115,15 +91,13 @@ ProxyImpl::~ProxyImpl() {
   DCHECK(IsImplThread());
   DCHECK(IsMainThreadBlocked());
 
-  // Take away the OutputSurface before destroying things so it doesn't try
-  // to call into its client mid-shutdown.
-  scheduler_->DidLoseOutputSurface();
-  layer_tree_host_impl_->ReleaseOutputSurface();
-
+  // Prevent the scheduler from performing actions while we're in an
+  // inconsistent state.
+  scheduler_->Stop();
+  // Take away the CompositorFrameSink before destroying things so it doesn't
+  // try to call into its client mid-shutdown.
+  layer_tree_host_impl_->ReleaseCompositorFrameSink();
   scheduler_ = nullptr;
-  external_begin_frame_source_ = nullptr;
-  unthrottled_begin_frame_source_ = nullptr;
-  synthetic_begin_frame_source_ = nullptr;
   layer_tree_host_impl_ = nullptr;
   // We need to explicitly shutdown the notifier to destroy any weakptrs it is
   // holding while still on the compositor thread. This also ensures any
@@ -138,30 +112,27 @@ void ProxyImpl::InitializeMutatorOnImpl(
   layer_tree_host_impl_->SetLayerTreeMutator(std::move(mutator));
 }
 
-void ProxyImpl::UpdateTopControlsStateOnImpl(TopControlsState constraints,
-                                             TopControlsState current,
-                                             bool animate) {
+void ProxyImpl::UpdateBrowserControlsStateOnImpl(
+    BrowserControlsState constraints,
+    BrowserControlsState current,
+    bool animate) {
   DCHECK(IsImplThread());
-  layer_tree_host_impl_->top_controls_manager()->UpdateTopControlsState(
+  layer_tree_host_impl_->browser_controls_manager()->UpdateBrowserControlsState(
       constraints, current, animate);
 }
 
-void ProxyImpl::InitializeOutputSurfaceOnImpl(OutputSurface* output_surface) {
-  TRACE_EVENT0("cc", "ProxyImpl::InitializeOutputSurfaceOnImplThread");
+void ProxyImpl::InitializeCompositorFrameSinkOnImpl(
+    CompositorFrameSink* compositor_frame_sink) {
+  TRACE_EVENT0("cc", "ProxyImpl::InitializeCompositorFrameSinkOnImplThread");
   DCHECK(IsImplThread());
 
   LayerTreeHostImpl* host_impl = layer_tree_host_impl_.get();
-  bool success = host_impl->InitializeRenderer(output_surface);
-  RendererCapabilities capabilities;
-  if (success) {
-    capabilities =
-        host_impl->GetRendererCapabilities().MainThreadCapabilities();
-  }
-
-  channel_impl_->DidInitializeOutputSurface(success, capabilities);
-
+  bool success = host_impl->InitializeRenderer(compositor_frame_sink);
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::DidInitializeCompositorFrameSink,
+                            proxy_main_weak_ptr_, success));
   if (success)
-    scheduler_->DidCreateAndInitializeOutputSurface();
+    scheduler_->DidCreateAndInitializeCompositorFrameSink();
 }
 
 void ProxyImpl::MainThreadHasStoppedFlingingOnImpl() {
@@ -184,7 +155,8 @@ void ProxyImpl::SetDeferCommitsOnImpl(bool defer_commits) const {
 
 void ProxyImpl::SetNeedsRedrawOnImpl(const gfx::Rect& damage_rect) {
   DCHECK(IsImplThread());
-  SetNeedsRedrawRectOnImplThread(damage_rect);
+  layer_tree_host_impl_->SetViewportDamage(damage_rect);
+  SetNeedsRedrawOnImplThread();
 }
 
 void ProxyImpl::SetNeedsCommitOnImpl() {
@@ -194,7 +166,8 @@ void ProxyImpl::SetNeedsCommitOnImpl() {
 
 void ProxyImpl::BeginMainFrameAbortedOnImpl(
     CommitEarlyOutReason reason,
-    base::TimeTicks main_thread_start_time) {
+    base::TimeTicks main_thread_start_time,
+    std::vector<std::unique_ptr<SwapPromise>> swap_promises) {
   TRACE_EVENT1("cc", "ProxyImpl::BeginMainFrameAbortedOnImplThread", "reason",
                CommitEarlyOutReasonToString(reason));
   DCHECK(IsImplThread());
@@ -203,18 +176,12 @@ void ProxyImpl::BeginMainFrameAbortedOnImpl(
   if (CommitEarlyOutHandledCommit(reason)) {
     SetInputThrottledUntilCommitOnImpl(false);
   }
-  layer_tree_host_impl_->BeginMainFrameAborted(reason);
+  layer_tree_host_impl_->BeginMainFrameAborted(reason,
+                                               std::move(swap_promises));
   scheduler_->NotifyBeginMainFrameStarted(main_thread_start_time);
   scheduler_->BeginMainFrameAborted(reason);
 }
 
-void ProxyImpl::FinishAllRenderingOnImpl(CompletionEvent* completion) {
-  TRACE_EVENT0("cc", "ProxyImpl::FinishAllRenderingOnImplThread");
-  DCHECK(IsImplThread());
-  layer_tree_host_impl_->FinishAllRendering();
-  completion->Signal();
-}
-
 void ProxyImpl::SetVisibleOnImpl(bool visible) {
   TRACE_EVENT1("cc", "ProxyImpl::SetVisibleOnImplThread", "visible", visible);
   DCHECK(IsImplThread());
@@ -222,22 +189,22 @@ void ProxyImpl::SetVisibleOnImpl(bool visible) {
   scheduler_->SetVisible(visible);
 }
 
-void ProxyImpl::ReleaseOutputSurfaceOnImpl(CompletionEvent* completion) {
+void ProxyImpl::ReleaseCompositorFrameSinkOnImpl(CompletionEvent* completion) {
   DCHECK(IsImplThread());
 
-  // Unlike DidLoseOutputSurfaceOnImplThread, we don't need to call
-  // LayerTreeHost::DidLoseOutputSurface since it already knows.
-  scheduler_->DidLoseOutputSurface();
-  layer_tree_host_impl_->ReleaseOutputSurface();
+  // Unlike DidLoseCompositorFrameSinkOnImplThread, we don't need to call
+  // LayerTreeHost::DidLoseCompositorFrameSink since it already knows.
+  scheduler_->DidLoseCompositorFrameSink();
+  layer_tree_host_impl_->ReleaseCompositorFrameSink();
   completion->Signal();
 }
 
 void ProxyImpl::FinishGLOnImpl(CompletionEvent* completion) {
   TRACE_EVENT0("cc", "ProxyImpl::FinishGLOnImplThread");
   DCHECK(IsImplThread());
-  if (layer_tree_host_impl_->output_surface()) {
+  if (layer_tree_host_impl_->compositor_frame_sink()) {
     ContextProvider* context_provider =
-        layer_tree_host_impl_->output_surface()->context_provider();
+        layer_tree_host_impl_->compositor_frame_sink()->context_provider();
     if (context_provider)
       context_provider->ContextGL()->Finish();
   }
@@ -248,7 +215,7 @@ void ProxyImpl::MainFrameWillHappenOnImplForTesting(
     CompletionEvent* completion,
     bool* main_frame_will_happen) {
   DCHECK(IsImplThread());
-  if (layer_tree_host_impl_->output_surface()) {
+  if (layer_tree_host_impl_->compositor_frame_sink()) {
     *main_frame_will_happen = scheduler_->MainFrameForTestingWillHappen();
   } else {
     *main_frame_will_happen = false;
@@ -256,21 +223,12 @@ void ProxyImpl::MainFrameWillHappenOnImplForTesting(
   completion->Signal();
 }
 
-void ProxyImpl::BlockNotifyReadyToActivateForTesting(bool block) {
-  DCHECK(IsImplThread());
-  layer_tree_host_impl_->BlockNotifyReadyToActivateForTesting(block);
-}
-
-CompletionEvent* ProxyImpl::ActivationCompletionEventForTesting() {
-  DCHECK(IsImplThread());
-  return activation_completion_event_;
-}
-
-void ProxyImpl::StartCommitOnImpl(CompletionEvent* completion,
-                                  LayerTreeHost* layer_tree_host,
-                                  base::TimeTicks main_thread_start_time,
-                                  bool hold_commit_for_activation) {
-  TRACE_EVENT0("cc", "ProxyImpl::StartCommitOnImplThread");
+void ProxyImpl::NotifyReadyToCommitOnImpl(
+    CompletionEvent* completion,
+    LayerTreeHostInProcess* layer_tree_host,
+    base::TimeTicks main_thread_start_time,
+    bool hold_commit_for_activation) {
+  TRACE_EVENT0("cc", "ProxyImpl::NotifyReadyToCommitOnImpl");
   DCHECK(!commit_completion_event_);
   DCHECK(IsImplThread() && IsMainThreadBlocked());
   DCHECK(scheduler_);
@@ -287,6 +245,8 @@ void ProxyImpl::StartCommitOnImpl(CompletionEvent* completion,
   // But, we can avoid a PostTask in here.
   scheduler_->NotifyBeginMainFrameStarted(main_thread_start_time);
 
+  layer_tree_host_impl_->ReadyToCommit();
+
   commit_completion_event_ = completion;
   commit_completion_waits_for_activation_ = hold_commit_for_activation;
 
@@ -295,29 +255,17 @@ void ProxyImpl::StartCommitOnImpl(CompletionEvent* completion,
   scheduler_->NotifyReadyToCommit();
 }
 
-void ProxyImpl::UpdateRendererCapabilitiesOnImplThread() {
+void ProxyImpl::DidLoseCompositorFrameSinkOnImplThread() {
+  TRACE_EVENT0("cc", "ProxyImpl::DidLoseCompositorFrameSinkOnImplThread");
   DCHECK(IsImplThread());
-  channel_impl_->SetRendererCapabilitiesMainCopy(
-      layer_tree_host_impl_->GetRendererCapabilities()
-          .MainThreadCapabilities());
-}
-
-void ProxyImpl::DidLoseOutputSurfaceOnImplThread() {
-  TRACE_EVENT0("cc", "ProxyImpl::DidLoseOutputSurfaceOnImplThread");
-  DCHECK(IsImplThread());
-  channel_impl_->DidLoseOutputSurface();
-  scheduler_->DidLoseOutputSurface();
-}
-
-void ProxyImpl::CommitVSyncParameters(base::TimeTicks timebase,
-                                      base::TimeDelta interval) {
-  DCHECK(IsImplThread());
-  if (synthetic_begin_frame_source_)
-    synthetic_begin_frame_source_->OnUpdateVSyncParameters(timebase, interval);
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE,
+      base::Bind(&ProxyMain::DidLoseCompositorFrameSink, proxy_main_weak_ptr_));
+  scheduler_->DidLoseCompositorFrameSink();
 }
 
 void ProxyImpl::SetBeginFrameSource(BeginFrameSource* source) {
-  // During shutdown, destroying the OutputSurface may unset the
+  // During shutdown, destroying the CompositorFrameSink may unset the
   // BeginFrameSource.
   if (scheduler_) {
     // TODO(enne): this overrides any preexisting begin frame source.  Those
@@ -326,21 +274,14 @@ void ProxyImpl::SetBeginFrameSource(BeginFrameSource* source) {
   }
 }
 
-void ProxyImpl::SetEstimatedParentDrawTime(base::TimeDelta draw_time) {
+void ProxyImpl::DidReceiveCompositorFrameAckOnImplThread() {
+  TRACE_EVENT0("cc,benchmark",
+               "ProxyImpl::DidReceiveCompositorFrameAckOnImplThread");
   DCHECK(IsImplThread());
-  scheduler_->SetEstimatedParentDrawTime(draw_time);
-}
-
-void ProxyImpl::DidSwapBuffersOnImplThread() {
-  DCHECK(IsImplThread());
-  scheduler_->DidSwapBuffers();
-}
-
-void ProxyImpl::DidSwapBuffersCompleteOnImplThread() {
-  TRACE_EVENT0("cc,benchmark", "ProxyImpl::DidSwapBuffersCompleteOnImplThread");
-  DCHECK(IsImplThread());
-  scheduler_->DidSwapBuffersComplete();
-  channel_impl_->DidCompleteSwapBuffers();
+  scheduler_->DidReceiveCompositorFrameAck();
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::DidReceiveCompositorFrameAck,
+                            proxy_main_weak_ptr_));
 }
 
 void ProxyImpl::OnCanDrawStateChanged(bool can_draw) {
@@ -367,12 +308,6 @@ void ProxyImpl::SetNeedsRedrawOnImplThread() {
   scheduler_->SetNeedsRedraw();
 }
 
-void ProxyImpl::SetNeedsRedrawRectOnImplThread(const gfx::Rect& damage_rect) {
-  DCHECK(IsImplThread());
-  layer_tree_host_impl_->SetViewportDamage(damage_rect);
-  SetNeedsRedrawOnImplThread();
-}
-
 void ProxyImpl::SetNeedsOneBeginImplFrameOnImplThread() {
   TRACE_EVENT0("cc", "ProxyImpl::SetNeedsOneBeginImplFrameOnImplThread");
   DCHECK(IsImplThread());
@@ -400,10 +335,12 @@ void ProxyImpl::SetVideoNeedsBeginFrames(bool needs_begin_frames) {
 }
 
 void ProxyImpl::PostAnimationEventsToMainThreadOnImplThread(
-    std::unique_ptr<AnimationEvents> events) {
+    std::unique_ptr<MutatorEvents> events) {
   TRACE_EVENT0("cc", "ProxyImpl::PostAnimationEventsToMainThreadOnImplThread");
   DCHECK(IsImplThread());
-  channel_impl_->SetAnimationEvents(std::move(events));
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::SetAnimationEvents,
+                            proxy_main_weak_ptr_, base::Passed(&events)));
 }
 
 bool ProxyImpl::IsInsideDraw() {
@@ -485,12 +422,14 @@ void ProxyImpl::DidPrepareTiles() {
 
 void ProxyImpl::DidCompletePageScaleAnimationOnImplThread() {
   DCHECK(IsImplThread());
-  channel_impl_->DidCompletePageScaleAnimation();
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::DidCompletePageScaleAnimation,
+                            proxy_main_weak_ptr_));
 }
 
-void ProxyImpl::OnDrawForOutputSurface(bool resourceless_software_draw) {
+void ProxyImpl::OnDrawForCompositorFrameSink(bool resourceless_software_draw) {
   DCHECK(IsImplThread());
-  scheduler_->OnDrawForOutputSurface(resourceless_software_draw);
+  scheduler_->OnDrawForCompositorFrameSink(resourceless_software_draw);
 }
 
 void ProxyImpl::WillBeginImplFrame(const BeginFrameArgs& args) {
@@ -516,32 +455,30 @@ void ProxyImpl::ScheduledActionSendBeginMainFrame(const BeginFrameArgs& args) {
       layer_tree_host_impl_->ProcessLayerTreeMutations();
   begin_main_frame_state->scroll_info =
       layer_tree_host_impl_->ProcessScrollDeltas();
-  begin_main_frame_state->memory_allocation_limit_bytes =
-      layer_tree_host_impl_->memory_allocation_limit_bytes();
   begin_main_frame_state->evicted_ui_resources =
       layer_tree_host_impl_->EvictedUIResourcesExist();
-  channel_impl_->BeginMainFrame(std::move(begin_main_frame_state));
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::BeginMainFrame, proxy_main_weak_ptr_,
+                            base::Passed(&begin_main_frame_state)));
   devtools_instrumentation::DidRequestMainThreadFrame(layer_tree_host_id_);
 }
 
-DrawResult ProxyImpl::ScheduledActionDrawAndSwapIfPossible() {
-  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionDrawAndSwap");
+DrawResult ProxyImpl::ScheduledActionDrawIfPossible() {
+  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionDraw");
   DCHECK(IsImplThread());
 
-  // SchedulerStateMachine::DidDrawIfPossibleCompleted isn't set up to
-  // handle DRAW_ABORTED_CANT_DRAW.  Moreover, the scheduler should
-  // never generate this call when it can't draw.
+  // The scheduler should never generate this call when it can't draw.
   DCHECK(layer_tree_host_impl_->CanDraw());
 
   bool forced_draw = false;
-  return DrawAndSwapInternal(forced_draw);
+  return DrawInternal(forced_draw);
 }
 
-DrawResult ProxyImpl::ScheduledActionDrawAndSwapForced() {
-  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionDrawAndSwapForced");
+DrawResult ProxyImpl::ScheduledActionDrawForced() {
+  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionDrawForced");
   DCHECK(IsImplThread());
   bool forced_draw = true;
-  return DrawAndSwapInternal(forced_draw);
+  return DrawInternal(forced_draw);
 }
 
 void ProxyImpl::ScheduledActionCommit() {
@@ -589,10 +526,13 @@ void ProxyImpl::ScheduledActionActivateSyncTree() {
   layer_tree_host_impl_->ActivateSyncTree();
 }
 
-void ProxyImpl::ScheduledActionBeginOutputSurfaceCreation() {
-  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionBeginOutputSurfaceCreation");
+void ProxyImpl::ScheduledActionBeginCompositorFrameSinkCreation() {
+  TRACE_EVENT0("cc",
+               "ProxyImpl::ScheduledActionBeginCompositorFrameSinkCreation");
   DCHECK(IsImplThread());
-  channel_impl_->RequestNewOutputSurface();
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::RequestNewCompositorFrameSink,
+                            proxy_main_weak_ptr_));
 }
 
 void ProxyImpl::ScheduledActionPrepareTiles() {
@@ -601,20 +541,22 @@ void ProxyImpl::ScheduledActionPrepareTiles() {
   layer_tree_host_impl_->PrepareTiles();
 }
 
-void ProxyImpl::ScheduledActionInvalidateOutputSurface() {
-  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionInvalidateOutputSurface");
+void ProxyImpl::ScheduledActionInvalidateCompositorFrameSink() {
+  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionInvalidateCompositorFrameSink");
   DCHECK(IsImplThread());
-  DCHECK(layer_tree_host_impl_->output_surface());
-  layer_tree_host_impl_->output_surface()->Invalidate();
+  DCHECK(layer_tree_host_impl_->compositor_frame_sink());
+  layer_tree_host_impl_->compositor_frame_sink()->Invalidate();
 }
 
 void ProxyImpl::SendBeginMainFrameNotExpectedSoon() {
   DCHECK(IsImplThread());
-  channel_impl_->BeginMainFrameNotExpectedSoon();
+  MainThreadTaskRunner()->PostTask(
+      FROM_HERE, base::Bind(&ProxyMain::BeginMainFrameNotExpectedSoon,
+                            proxy_main_weak_ptr_));
 }
 
-DrawResult ProxyImpl::DrawAndSwapInternal(bool forced_draw) {
-  TRACE_EVENT_SYNTHETIC_DELAY("cc.DrawAndSwap");
+DrawResult ProxyImpl::DrawInternal(bool forced_draw) {
+  TRACE_EVENT_SYNTHETIC_DELAY("cc.Draw");
 
   DCHECK(IsImplThread());
   DCHECK(layer_tree_host_impl_.get());
@@ -650,23 +592,25 @@ DrawResult ProxyImpl::DrawAndSwapInternal(bool forced_draw) {
   }
 
   if (draw_frame) {
-    layer_tree_host_impl_->DrawLayers(&frame);
+    if (layer_tree_host_impl_->DrawLayers(&frame))
+      // Drawing implies we submitted a frame to the CompositorFrameSink.
+      scheduler_->DidSubmitCompositorFrame();
     result = DRAW_SUCCESS;
   } else {
     DCHECK_NE(DRAW_SUCCESS, result);
   }
+
   layer_tree_host_impl_->DidDrawAllLayers(frame);
 
   bool start_ready_animations = draw_frame;
   layer_tree_host_impl_->UpdateAnimationState(start_ready_animations);
 
-  if (draw_frame)
-    layer_tree_host_impl_->SwapBuffers(frame);
-
   // Tell the main thread that the the newly-commited frame was drawn.
   if (next_frame_is_newly_committed_frame_) {
     next_frame_is_newly_committed_frame_ = false;
-    channel_impl_->DidCommitAndDrawFrame();
+    MainThreadTaskRunner()->PostTask(
+        FROM_HERE,
+        base::Bind(&ProxyMain::DidCommitAndDrawFrame, proxy_main_weak_ptr_));
   }
 
   DCHECK_NE(INVALID_RESULT, result);
@@ -686,4 +630,8 @@ ProxyImpl::BlockedMainCommitOnly& ProxyImpl::blocked_main_commit() {
   return main_thread_blocked_commit_vars_unsafe_;
 }
 
+base::SingleThreadTaskRunner* ProxyImpl::MainThreadTaskRunner() {
+  return task_runner_provider_->MainThreadTaskRunner();
+}
+
 }  // namespace cc
