@@ -13,7 +13,7 @@
 namespace cc {
 
 class LayerImpl;
-class LayerTreeHost;
+class LayerTree;
 class LayerTreeImpl;
 class Layer;
 
@@ -28,7 +28,7 @@ class CC_EXPORT TreeSynchronizer {
 
   static void PushLayerProperties(LayerTreeImpl* pending_tree,
                                   LayerTreeImpl* active_tree);
-  static void PushLayerProperties(LayerTreeHost* host_tree,
+  static void PushLayerProperties(LayerTree* host_tree,
                                   LayerTreeImpl* impl_tree);
 
  private:
