@@ -22,6 +22,8 @@ enum {
   SOUND_PASSTHROUGH,
   SOUND_EXIT_SCREEN,
   SOUND_ENTER_SCREEN,
+  SOUND_SPOKEN_FEEDBACK_TOGGLE_COUNTDOWN_HIGH,
+  SOUND_SPOKEN_FEEDBACK_TOGGLE_COUNTDOWN_LOW,
   SOUND_COUNT,
 };
 
