@@ -279,7 +279,8 @@ class CrasAudioClientImpl : public CrasAudioClient {
 
   void NameOwnerChangedReceived(const std::string& old_owner,
                                 const std::string& new_owner) {
-    FOR_EACH_OBSERVER(Observer, observers_, AudioClientRestarted());
+    for (auto& observer : observers_)
+      observer.AudioClientRestarted();
   }
 
   // Called when a OutputMuteChanged signal is received.
@@ -292,7 +293,8 @@ class CrasAudioClientImpl : public CrasAudioClient {
       LOG(ERROR) << "Error reading signal from cras:"
                  << signal->ToString();
     }
-    FOR_EACH_OBSERVER(Observer, observers_, OutputMuteChanged(user_mute));
+    for (auto& observer : observers_)
+      observer.OutputMuteChanged(user_mute);
   }
 
   // Called when a InputMuteChanged signal is received.
@@ -303,11 +305,13 @@ class CrasAudioClientImpl : public CrasAudioClient {
       LOG(ERROR) << "Error reading signal from cras:"
                  << signal->ToString();
     }
-    FOR_EACH_OBSERVER(Observer, observers_, InputMuteChanged(mute));
+    for (auto& observer : observers_)
+      observer.InputMuteChanged(mute);
   }
 
   void NodesChangedReceived(dbus::Signal* signal) {
-    FOR_EACH_OBSERVER(Observer, observers_, NodesChanged());
+    for (auto& observer : observers_)
+      observer.NodesChanged();
   }
 
   void ActiveOutputNodeChangedReceived(dbus::Signal* signal) {
@@ -317,7 +321,8 @@ class CrasAudioClientImpl : public CrasAudioClient {
       LOG(ERROR) << "Error reading signal from cras:"
                  << signal->ToString();
     }
-    FOR_EACH_OBSERVER(Observer, observers_, ActiveOutputNodeChanged(node_id));
+    for (auto& observer : observers_)
+      observer.ActiveOutputNodeChanged(node_id);
   }
 
   void ActiveInputNodeChangedReceived(dbus::Signal* signal) {
@@ -327,7 +332,8 @@ class CrasAudioClientImpl : public CrasAudioClient {
       LOG(ERROR) << "Error reading signal from cras:"
                  << signal->ToString();
     }
-    FOR_EACH_OBSERVER(Observer, observers_, ActiveInputNodeChanged(node_id));
+    for (auto& observer : observers_)
+      observer.ActiveInputNodeChanged(node_id);
   }
 
   void OutputNodeVolumeChangedReceived(dbus::Signal* signal) {
@@ -341,8 +347,8 @@ class CrasAudioClientImpl : public CrasAudioClient {
     if (!reader.PopInt32(&volume)) {
       LOG(ERROR) << "Error eading signal from cras:" << signal->ToString();
     }
-    FOR_EACH_OBSERVER(Observer, observers_,
-                      OutputNodeVolumeChanged(node_id, volume));
+    for (auto& observer : observers_)
+      observer.OutputNodeVolumeChanged(node_id, volume);
   }
 
   void OnGetVolumeState(const GetVolumeStateCallback& callback,
@@ -420,7 +426,7 @@ class CrasAudioClientImpl : public CrasAudioClient {
 
   bool GetAudioNode(dbus::Response* response,
                     dbus::MessageReader* array_reader,
-                    AudioNode *node) {
+                    AudioNode* node) {
     while (array_reader->HasMoreData()) {
       dbus::MessageReader dict_entry_reader(response);
       dbus::MessageReader value_reader(response);
@@ -456,8 +462,12 @@ class CrasAudioClientImpl : public CrasAudioClient {
         if (!value_reader.PopString(&node->mic_positions))
           return false;
       } else if (key == cras::kStableDeviceIdProperty) {
-        if (!value_reader.PopUint64(&node->stable_device_id))
+        if (!value_reader.PopUint64(&node->stable_device_id_v1))
           return false;
+      } else if (key == cras::kStableDeviceIdNewProperty) {
+        if (!value_reader.PopUint64(&node->stable_device_id_v2))
+          return false;
+        node->has_v2_stable_device_id = true;
       }
     }
 
