@@ -2,10 +2,17 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "chromeos/dbus/shill_manager_client.h"
+
+#include <memory>
+#include <utility>
+
 #include "base/bind.h"
+#include "base/memory/ptr_util.h"
+#include "base/run_loop.h"
+#include "base/test/mock_callback.h"
 #include "base/values.h"
 #include "chromeos/dbus/shill_client_unittest_base.h"
-#include "chromeos/dbus/shill_manager_client.h"
 #include "dbus/message.h"
 #include "dbus/object_path.h"
 #include "dbus/values_util.h"
@@ -46,6 +53,21 @@ void ExpectStringArgumentsFollowedByObjectPath(
   EXPECT_FALSE(reader->HasMoreData());
 }
 
+void ExpectThrottlingArguments(bool throttling_enabled_expected,
+                               uint32_t upload_rate_kbits_expected,
+                               uint32_t download_rate_kbits_expected,
+                               dbus::MessageReader* reader) {
+  bool throttling_enabled_actual;
+  uint32_t upload_rate_kbits_actual;
+  uint32_t download_rate_kbits_actual;
+  ASSERT_TRUE(reader->PopBool(&throttling_enabled_actual));
+  EXPECT_EQ(throttling_enabled_actual, throttling_enabled_expected);
+  ASSERT_TRUE(reader->PopUint32(&upload_rate_kbits_actual));
+  EXPECT_EQ(upload_rate_kbits_expected, upload_rate_kbits_actual);
+  ASSERT_TRUE(reader->PopUint32(&download_rate_kbits_actual));
+  EXPECT_EQ(download_rate_kbits_expected, download_rate_kbits_actual);
+  EXPECT_FALSE(reader->HasMoreData());
+}
 
 }  // namespace
 
@@ -62,7 +84,7 @@ class ShillManagerClientTest : public ShillClientUnittestBase {
     client_.reset(ShillManagerClient::Create());
     client_->Init(mock_bus_.get());
     // Run the message loop to run the signal connection result callback.
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void TearDown() override { ShillClientUnittestBase::TearDown(); }
@@ -126,7 +148,7 @@ TEST_F(ShillManagerClientTest, GetProperties) {
   // Call method.
   client_->GetProperties(base::Bind(&ExpectDictionaryValueResult, &value));
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, GetNetworksForGeolocation) {
@@ -160,11 +182,11 @@ TEST_F(ShillManagerClientTest, GetNetworksForGeolocation) {
   // Create the expected value.
   base::DictionaryValue type_dict_value;
   base::ListValue* type_entry_value = new base::ListValue;
-  base::DictionaryValue* property_dict_value = new base::DictionaryValue;
+  auto property_dict_value = base::MakeUnique<base::DictionaryValue>();
   property_dict_value->SetWithoutPathExpansion(
       shill::kGeoMacAddressProperty,
       new base::StringValue("01:23:45:67:89:AB"));
-  type_entry_value->Append(property_dict_value);
+  type_entry_value->Append(std::move(property_dict_value));
   type_dict_value.SetWithoutPathExpansion("wifi", type_entry_value);
 
   // Set expectations.
@@ -176,7 +198,7 @@ TEST_F(ShillManagerClientTest, GetNetworksForGeolocation) {
                                                 &type_dict_value));
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, SetProperty) {
@@ -190,17 +212,15 @@ TEST_F(ShillManagerClientTest, SetProperty) {
                                   &value),
                        response.get());
   // Call method.
-  MockClosure mock_closure;
-  MockErrorCallback mock_error_callback;
-  client_->SetProperty(shill::kCheckPortalListProperty,
-                       value,
-                       mock_closure.GetCallback(),
-                       mock_error_callback.GetCallback());
+  base::MockCallback<base::Closure> mock_closure;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->SetProperty(shill::kCheckPortalListProperty, value,
+                       mock_closure.Get(), mock_error_callback.Get());
   EXPECT_CALL(mock_closure, Run()).Times(1);
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, RequestScan) {
@@ -211,16 +231,15 @@ TEST_F(ShillManagerClientTest, RequestScan) {
                        base::Bind(&ExpectStringArgument, shill::kTypeWifi),
                        response.get());
   // Call method.
-  MockClosure mock_closure;
-  MockErrorCallback mock_error_callback;
-  client_->RequestScan(shill::kTypeWifi,
-                       mock_closure.GetCallback(),
-                       mock_error_callback.GetCallback());
+  base::MockCallback<base::Closure> mock_closure;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->RequestScan(shill::kTypeWifi, mock_closure.Get(),
+                       mock_error_callback.Get());
   EXPECT_CALL(mock_closure, Run()).Times(1);
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, EnableTechnology) {
@@ -231,16 +250,39 @@ TEST_F(ShillManagerClientTest, EnableTechnology) {
                        base::Bind(&ExpectStringArgument, shill::kTypeWifi),
                        response.get());
   // Call method.
-  MockClosure mock_closure;
-  MockErrorCallback mock_error_callback;
-  client_->EnableTechnology(shill::kTypeWifi,
-                            mock_closure.GetCallback(),
-                            mock_error_callback.GetCallback());
+  base::MockCallback<base::Closure> mock_closure;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->EnableTechnology(shill::kTypeWifi, mock_closure.Get(),
+                            mock_error_callback.Get());
   EXPECT_CALL(mock_closure, Run()).Times(1);
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
+}
+
+TEST_F(ShillManagerClientTest, NetworkThrottling) {
+  // Create response.
+  std::unique_ptr<dbus::Response> response(dbus::Response::CreateEmpty());
+  // Set expectations.
+  const bool enabled = true;
+  const uint32_t upload_rate = 1200;
+  const uint32_t download_rate = 2000;
+  PrepareForMethodCall(shill::kSetNetworkThrottlingFunction,
+                       base::Bind(&ExpectThrottlingArguments, enabled,
+                                  upload_rate, download_rate),
+                       response.get());
+  // Call method.
+  base::MockCallback<base::Closure> mock_closure;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->SetNetworkThrottlingStatus(enabled, upload_rate, download_rate,
+                                      mock_closure.Get(),
+                                      mock_error_callback.Get());
+  EXPECT_CALL(mock_closure, Run()).Times(1);
+  EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
+
+  // Run the message loop.
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, DisableTechnology) {
@@ -251,16 +293,15 @@ TEST_F(ShillManagerClientTest, DisableTechnology) {
                        base::Bind(&ExpectStringArgument, shill::kTypeWifi),
                        response.get());
   // Call method.
-  MockClosure mock_closure;
-  MockErrorCallback mock_error_callback;
-  client_->DisableTechnology(shill::kTypeWifi,
-                             mock_closure.GetCallback(),
-                             mock_error_callback.GetCallback());
+  base::MockCallback<base::Closure> mock_closure;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->DisableTechnology(shill::kTypeWifi, mock_closure.Get(),
+                             mock_error_callback.Get());
   EXPECT_CALL(mock_closure, Run()).Times(1);
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, ConfigureService) {
@@ -279,15 +320,14 @@ TEST_F(ShillManagerClientTest, ConfigureService) {
       base::Bind(&ExpectDictionaryValueArgument, arg.get(), string_valued),
       response.get());
   // Call method.
-  MockErrorCallback mock_error_callback;
-  client_->ConfigureService(*arg,
-                            base::Bind(&ExpectObjectPathResultWithoutStatus,
-                                       object_path),
-                            mock_error_callback.GetCallback());
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->ConfigureService(
+      *arg, base::Bind(&ExpectObjectPathResultWithoutStatus, object_path),
+      mock_error_callback.Get());
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, GetService) {
@@ -306,15 +346,14 @@ TEST_F(ShillManagerClientTest, GetService) {
       base::Bind(&ExpectDictionaryValueArgument, arg.get(), string_valued),
       response.get());
   // Call method.
-  MockErrorCallback mock_error_callback;
-  client_->GetService(*arg,
-                      base::Bind(&ExpectObjectPathResultWithoutStatus,
-                                 object_path),
-                      mock_error_callback.GetCallback());
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
+  client_->GetService(
+      *arg, base::Bind(&ExpectObjectPathResultWithoutStatus, object_path),
+      mock_error_callback.Get());
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, VerifyDestination) {
@@ -337,7 +376,7 @@ TEST_F(ShillManagerClientTest, VerifyDestination) {
                        response.get());
 
   // Call method.
-  MockErrorCallback mock_error_callback;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
   ShillManagerClient::VerificationProperties properties;
   properties.certificate = arguments[0];
   properties.public_key = arguments[1];
@@ -347,13 +386,12 @@ TEST_F(ShillManagerClientTest, VerifyDestination) {
   properties.device_ssid = arguments[5];
   properties.device_bssid = arguments[6];
   client_->VerifyDestination(
-      properties,
-      base::Bind(&ExpectBoolResultWithoutStatus, expected),
-      mock_error_callback.GetCallback());
+      properties, base::Bind(&ExpectBoolResultWithoutStatus, expected),
+      mock_error_callback.Get());
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, VerifyAndEncryptCredentials) {
@@ -380,7 +418,7 @@ TEST_F(ShillManagerClientTest, VerifyAndEncryptCredentials) {
                        response.get());
 
   // Call method.
-  MockErrorCallback mock_error_callback;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
   ShillManagerClient::VerificationProperties properties;
   properties.certificate = arguments[0];
   properties.public_key = arguments[1];
@@ -390,14 +428,13 @@ TEST_F(ShillManagerClientTest, VerifyAndEncryptCredentials) {
   properties.device_ssid = arguments[5];
   properties.device_bssid = arguments[6];
   client_->VerifyAndEncryptCredentials(
-      properties,
-      service_path,
+      properties, service_path,
       base::Bind(&ExpectStringResultWithoutStatus, expected),
-      mock_error_callback.GetCallback());
+      mock_error_callback.Get());
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(ShillManagerClientTest, VerifyAndEncryptData) {
@@ -421,7 +458,7 @@ TEST_F(ShillManagerClientTest, VerifyAndEncryptData) {
                        response.get());
 
   // Call method.
-  MockErrorCallback mock_error_callback;
+  base::MockCallback<ShillManagerClient::ErrorCallback> mock_error_callback;
   ShillManagerClient::VerificationProperties properties;
   properties.certificate = arguments[0];
   properties.public_key = arguments[1];
@@ -431,14 +468,13 @@ TEST_F(ShillManagerClientTest, VerifyAndEncryptData) {
   properties.device_ssid = arguments[5];
   properties.device_bssid = arguments[6];
   client_->VerifyAndEncryptData(
-      properties,
-      arguments[7],
+      properties, arguments[7],
       base::Bind(&ExpectStringResultWithoutStatus, expected),
-      mock_error_callback.GetCallback());
+      mock_error_callback.Get());
   EXPECT_CALL(mock_error_callback, Run(_, _)).Times(0);
 
   // Run the message loop.
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 }  // namespace chromeos
