@@ -223,10 +223,6 @@ class UpdateEngineClientImpl : public UpdateEngineClient {
     update_engine_proxy_ = bus->GetObjectProxy(
         update_engine::kUpdateEngineServiceName,
         dbus::ObjectPath(update_engine::kUpdateEngineServicePath));
-
-    // Monitor the D-Bus signal for brightness changes. Only the power
-    // manager knows the actual brightness level. We don't cache the
-    // brightness level in Chrome as it will make things less reliable.
     update_engine_proxy_->ConnectToSignal(
         update_engine::kUpdateEngineInterface,
         update_engine::kStatusUpdate,
@@ -234,15 +230,24 @@ class UpdateEngineClientImpl : public UpdateEngineClient {
                    weak_ptr_factory_.GetWeakPtr()),
         base::Bind(&UpdateEngineClientImpl::StatusUpdateConnected,
                    weak_ptr_factory_.GetWeakPtr()));
-
-    // Get update engine status for the initial status. Update engine won't
-    // send StatusUpdate signal unless there is a status change. If chrome
-    // crashes after UPDATE_STATUS_UPDATED_NEED_REBOOT status is set,
-    // restarted chrome would not get this status. See crbug.com/154104.
-    GetUpdateEngineStatus();
+    update_engine_proxy_->WaitForServiceToBeAvailable(
+        base::Bind(&UpdateEngineClientImpl::OnServiceInitiallyAvailable,
+                   weak_ptr_factory_.GetWeakPtr()));
   }
 
  private:
+  void OnServiceInitiallyAvailable(bool service_is_available) {
+    if (service_is_available) {
+      // Get update engine status for the initial status. Update engine won't
+      // send StatusUpdate signal unless there is a status change. If chrome
+      // crashes after UPDATE_STATUS_UPDATED_NEED_REBOOT status is set,
+      // restarted chrome would not get this status. See crbug.com/154104.
+      GetUpdateEngineStatus();
+    } else {
+      LOG(ERROR) << "Failed to wait for D-Bus service to become available";
+    }
+  }
+
   void GetUpdateEngineStatus() {
     dbus::MethodCall method_call(
         update_engine::kUpdateEngineInterface,
@@ -323,7 +328,8 @@ class UpdateEngineClientImpl : public UpdateEngineClient {
     }
     status.status = UpdateStatusFromString(current_operation);
     last_status_ = status;
-    FOR_EACH_OBSERVER(Observer, observers_, UpdateStatusChanged(status));
+    for (auto& observer : observers_)
+      observer.UpdateStatusChanged(status);
   }
 
   // Called when GetStatus call failed.
@@ -414,7 +420,8 @@ class UpdateEngineClientImpl : public UpdateEngineClient {
     status.new_size = new_size;
 
     last_status_ = status;
-    FOR_EACH_OBSERVER(Observer, observers_, UpdateStatusChanged(status));
+    for (auto& observer : observers_)
+      observer.UpdateStatusChanged(status);
   }
 
   // Called when the status update signal is initially connected.
@@ -561,7 +568,8 @@ class UpdateEngineClientFakeImpl : public UpdateEngineClientStubImpl {
         break;
     }
     last_status_.status = next_status;
-    FOR_EACH_OBSERVER(Observer, observers_, UpdateStatusChanged(last_status_));
+    for (auto& observer : observers_)
+      observer.UpdateStatusChanged(last_status_);
     if (last_status_.status != UPDATE_STATUS_IDLE) {
       base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
           FROM_HERE, base::Bind(&UpdateEngineClientFakeImpl::StateTransition,
@@ -595,7 +603,7 @@ UpdateEngineClient* UpdateEngineClient::Create(
     DBusClientImplementationType type) {
   if (type == REAL_DBUS_CLIENT_IMPLEMENTATION)
     return new UpdateEngineClientImpl();
-  DCHECK_EQ(STUB_DBUS_CLIENT_IMPLEMENTATION, type);
+  DCHECK_EQ(FAKE_DBUS_CLIENT_IMPLEMENTATION, type);
   if (base::CommandLine::ForCurrentProcess()->HasSwitch(
           switches::kTestAutoUpdateUI))
     return new UpdateEngineClientFakeImpl();
@@ -607,12 +615,12 @@ UpdateEngineClient* UpdateEngineClient::Create(
 bool UpdateEngineClient::IsTargetChannelMoreStable(
     const std::string& current_channel,
     const std::string& target_channel) {
-  auto cix = std::find(kReleaseChannelsList,
-                       kReleaseChannelsList + arraysize(kReleaseChannelsList),
-                       current_channel);
-  auto tix = std::find(kReleaseChannelsList,
-                       kReleaseChannelsList + arraysize(kReleaseChannelsList),
-                       target_channel);
+  const char** cix = std::find(
+      kReleaseChannelsList,
+      kReleaseChannelsList + arraysize(kReleaseChannelsList), current_channel);
+  const char** tix = std::find(
+      kReleaseChannelsList,
+      kReleaseChannelsList + arraysize(kReleaseChannelsList), target_channel);
   return tix > cix;
 }
 
