@@ -8,10 +8,12 @@
 
 #include <algorithm>
 #include <string>
+#include <utility>
 
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
-#include "base/metrics/histogram.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/metrics/sparse_histogram.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
@@ -197,7 +199,8 @@ bool ParseServerResponse(const GURL& server_url,
     PrintGeolocationError(
         server_url,
         "Unexpected response type : " +
-            base::StringPrintf("%u", response_value->GetType()),
+            base::StringPrintf(
+                "%u", static_cast<unsigned int>(response_value->GetType())),
         position);
     RecordUmaEvent(SIMPLE_GEOLOCATION_REQUEST_EVENT_RESPONSE_MALFORMED);
     return false;
@@ -332,8 +335,7 @@ std::string SimpleGeolocationRequest::FormatRequestBody() const {
   request->SetWithoutPathExpansion(kWifiAccessPoints, wifi_access_points);
 
   for (const WifiAccessPoint& access_point : *wifi_data_) {
-    base::DictionaryValue* access_point_dictionary = new base::DictionaryValue;
-    wifi_access_points->Append(access_point_dictionary);
+    auto access_point_dictionary = base::MakeUnique<base::DictionaryValue>();
 
     access_point_dictionary->SetStringWithoutPathExpansion(
         kMacAddress, access_point.mac_address);
@@ -350,6 +352,8 @@ std::string SimpleGeolocationRequest::FormatRequestBody() const {
         kChannel, access_point.channel);
     access_point_dictionary->SetIntegerWithoutPathExpansion(
         kSignalToNoiseRatio, access_point.signal_to_noise);
+
+    wifi_access_points->Append(std::move(access_point_dictionary));
   }
   std::string result;
   if (!base::JSONWriter::Write(*request, &result)) {
