@@ -387,7 +387,7 @@ void ClientCertResolver::NetworkListChanged() {
   for (NetworkStateHandler::NetworkStateList::const_iterator it =
            networks.begin(); it != networks.end(); ++it) {
     const std::string& service_path = (*it)->path();
-    if (ContainsKey(old_resolved_networks, service_path)) {
+    if (base::ContainsKey(old_resolved_networks, service_path)) {
       resolved_networks_.insert(service_path);
       continue;
     }
@@ -569,7 +569,8 @@ void ClientCertResolver::NotifyResolveRequestCompleted() {
   resolve_task_running_ = false;
   const bool changed = network_properties_changed_;
   network_properties_changed_ = false;
-  FOR_EACH_OBSERVER(Observer, observers_, ResolveRequestCompleted(changed));
+  for (auto& observer : observers_)
+    observer.ResolveRequestCompleted(changed);
 }
 
 base::Time ClientCertResolver::Now() const {
