@@ -134,7 +134,7 @@ class NetworkConnectionHandlerTest : public testing::Test {
 
     base::RunLoop().RunUntilIdle();
     LoginState::Initialize();
-    network_state_handler_.reset(NetworkStateHandler::InitializeForTest());
+    network_state_handler_ = NetworkStateHandler::InitializeForTest();
     network_config_handler_.reset(
         NetworkConfigurationHandler::InitializeForTest(
             network_state_handler_.get(),
@@ -161,6 +161,7 @@ class NetworkConnectionHandlerTest : public testing::Test {
   }
 
   void TearDown() override {
+    network_state_handler_->Shutdown();
     managed_config_handler_.reset();
     network_profile_handler_.reset();
     network_connection_handler_->RemoveObserver(
