@@ -5,34 +5,32 @@
 #ifndef COMPONENTS_ARC_TEST_FAKE_NOTIFICATIONS_INSTANCE_H_
 #define COMPONENTS_ARC_TEST_FAKE_NOTIFICATIONS_INSTANCE_H_
 
+#include <string>
+#include <utility>
+#include <vector>
+
 #include "components/arc/common/notifications.mojom.h"
-#include "components/arc/test/fake_arc_bridge_instance.h"
 
 namespace arc {
 
 class FakeNotificationsInstance : public mojom::NotificationsInstance {
  public:
-  FakeNotificationsInstance(
-      mojo::InterfaceRequest<mojom::NotificationsInstance> request);
+  FakeNotificationsInstance();
   ~FakeNotificationsInstance() override;
 
   void Init(mojom::NotificationsHostPtr host_ptr) override;
 
   void SendNotificationEventToAndroid(
-      const mojo::String& key,
+      const std::string& key,
       mojom::ArcNotificationEvent event) override;
-  void CreateNotificationWindow(const mojo::String& key) override;
-  void CloseNotificationWindow(const mojo::String& key) override;
+  void CreateNotificationWindow(const std::string& key) override;
+  void CloseNotificationWindow(const std::string& key) override;
 
-  const std::vector<std::pair<mojo::String, mojom::ArcNotificationEvent>>&
+  const std::vector<std::pair<std::string, mojom::ArcNotificationEvent>>&
   events() const;
 
-  void WaitForIncomingMethodCall();
-
  private:
-  std::vector<std::pair<mojo::String, mojom::ArcNotificationEvent>> events_;
-
-  mojo::Binding<mojom::NotificationsInstance> binding_;
+  std::vector<std::pair<std::string, mojom::ArcNotificationEvent>> events_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeNotificationsInstance);
 };
