@@ -13,6 +13,7 @@
 #include "base/command_line.h"
 #include "base/i18n/case_conversion.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
 #include "base/sha1.h"
 #include "base/strings/string_number_conversions.h"
@@ -27,13 +28,14 @@
 #include "components/autofill/core/browser/field_types.h"
 #include "components/autofill/core/browser/form_field.h"
 #include "components/autofill/core/common/autofill_constants.h"
+#include "components/autofill/core/common/autofill_util.h"
 #include "components/autofill/core/common/form_data.h"
 #include "components/autofill/core/common/form_data_predictions.h"
 #include "components/autofill/core/common/form_field_data.h"
 #include "components/autofill/core/common/form_field_data_predictions.h"
-#include "components/rappor/rappor_service.h"
-#include "components/rappor/rappor_utils.h"
-#include "third_party/re2/src/re2/re2.h"
+#include "components/autofill/core/common/signatures_util.h"
+#include "components/rappor/public/rappor_utils.h"
+#include "components/rappor/rappor_service_impl.h"
 
 namespace autofill {
 namespace {
@@ -42,9 +44,6 @@ const char kClientVersion[] = "6.1.1715.1442/en (GGLL)";
 const char kBillingMode[] = "billing";
 const char kShippingMode[] = "shipping";
 
-// Strip away >= 5 consecutive digits.
-const char kIgnorePatternInFieldName[] = "\\d{5,}";
-
 // A form is considered to have a high prediction mismatch rate if the number of
 // mismatches exceeds this threshold.
 const int kNumberOfMismatchesThreshold = 3;
@@ -57,9 +56,6 @@ const int kNumberOfMismatchesThreshold = 3;
 const int kCommonNamePrefixRemovalFieldThreshold = 3;
 const int kMinCommonNamePrefixLength = 16;
 
-// Maximum number of characters in the field label to be encoded in a proto.
-const int kMaxFieldLabelNumChars = 200;
-
 // Helper for |EncodeUploadRequest()| that creates a bit field corresponding to
 // |available_field_types| and returns the hex representation as a string.
 std::string EncodeFieldTypes(const ServerFieldTypeSet& available_field_types) {
@@ -275,14 +271,6 @@ HtmlFieldType FieldTypeFromAutocompleteAttributeValue(
   return HTML_TYPE_UNRECOGNIZED;
 }
 
-std::string StripDigitsIfRequired(const base::string16& input) {
-  std::string return_string = base::UTF16ToUTF8(input);
-
-  re2::RE2::GlobalReplace(&return_string, re2::RE2(kIgnorePatternInFieldName),
-                          std::string());
-  return return_string;
-}
-
 std::ostream& operator<<(
     std::ostream& out,
     const autofill::AutofillQueryResponseContents& response) {
@@ -293,6 +281,14 @@ std::ostream& operator<<(
   return out;
 }
 
+bool IsCreditCardExpirationType(ServerFieldType type) {
+  return type == CREDIT_CARD_EXP_MONTH ||
+         type == CREDIT_CARD_EXP_2_DIGIT_YEAR ||
+         type == CREDIT_CARD_EXP_4_DIGIT_YEAR ||
+         type == CREDIT_CARD_EXP_DATE_2_DIGIT_YEAR ||
+         type == CREDIT_CARD_EXP_DATE_4_DIGIT_YEAR;
+}
+
 }  // namespace
 
 FormStructure::FormStructure(const FormData& form)
@@ -312,14 +308,8 @@ FormStructure::FormStructure(const FormData& form)
   // Copy the form fields.
   std::map<base::string16, size_t> unique_names;
   for (const FormFieldData& field : form.fields) {
-    if (!ShouldSkipField(field)) {
-      // Add all supported form fields (including with empty names) to the
-      // signature.  This is a requirement for Autofill servers.
-      form_signature_field_names_.append("&");
-      form_signature_field_names_.append(StripDigitsIfRequired(field.name));
-
+    if (!ShouldSkipField(field))
       ++active_field_count_;
-    }
 
     if (field.form_control_type == "password")
       has_password_field_ = true;
@@ -332,9 +322,10 @@ FormStructure::FormStructure(const FormData& form)
     base::string16 unique_name =
         field.name + base::ASCIIToUTF16("_") +
         base::SizeTToString16(++unique_names[field.name]);
-    fields_.push_back(new AutofillField(field, unique_name));
+    fields_.push_back(base::MakeUnique<AutofillField>(field, unique_name));
   }
 
+  form_signature_ = autofill::CalculateFormSignature(form);
   // Do further processing on the fields, as needed.
   ProcessExtractedFields();
 }
@@ -355,8 +346,8 @@ void FormStructure::DetermineHeuristicTypes() {
   if (active_field_count() >= kRequiredFieldsForPredictionRoutines &&
       (is_form_tag_ || is_formless_checkout_)) {
     const FieldCandidatesMap field_type_map =
-        FormField::ParseFormFields(fields_.get(), is_form_tag_);
-    for (AutofillField* field : fields_) {
+        FormField::ParseFormFields(fields_, is_form_tag_);
+    for (const auto& field : fields_) {
       const auto iter = field_type_map.find(field->unique_name());
       if (iter != field_type_map.end())
         field->set_heuristic_type(iter->second.BestHeuristicType());
@@ -389,7 +380,7 @@ bool FormStructure::EncodeUploadRequest(
 
   // Verify that |available_field_types| agrees with the possible field types we
   // are uploading.
-  for (const AutofillField* field : *this) {
+  for (const auto& field : *this) {
     for (const auto& type : field->possible_types()) {
       DCHECK(type == UNKNOWN_TYPE || type == EMPTY_TYPE ||
              available_field_types.count(type));
@@ -398,12 +389,12 @@ bool FormStructure::EncodeUploadRequest(
 
   upload->set_submission(observed_submission);
   upload->set_client_version(kClientVersion);
-  upload->set_form_signature(FormSignature64Bit());
+  upload->set_form_signature(form_signature());
   upload->set_autofill_used(form_was_autofilled);
   upload->set_data_present(EncodeFieldTypes(available_field_types));
 
   if (IsAutofillFieldMetadataEnabled()) {
-    upload->set_action_signature(Hash64Bit(target_url_.host()));
+    upload->set_action_signature(StrToHash64Bit(target_url_.host()));
     if (!form_name().empty())
       upload->set_form_name(base::UTF16ToUTF8(form_name()));
   }
@@ -435,8 +426,8 @@ bool FormStructure::EncodeQueryRequest(
   // Some badly formatted web sites repeat forms - detect that and encode only
   // one form as returned data would be the same for all the repeated forms.
   std::set<std::string> processed_forms;
-  for (const auto& form : forms) {
-    std::string signature(form->FormSignature());
+  for (const auto* form : forms) {
+    std::string signature(form->FormSignatureAsStr());
     if (processed_forms.find(signature) != processed_forms.end())
       continue;
     processed_forms.insert(signature);
@@ -448,16 +439,14 @@ bool FormStructure::EncodeQueryRequest(
     encoded_signatures->push_back(signature);
   }
 
-  if (!encoded_signatures->size())
-    return false;
-
-  return true;
+  return !encoded_signatures->empty();
 }
 
 // static
-void FormStructure::ParseQueryResponse(std::string payload,
-                                       const std::vector<FormStructure*>& forms,
-                                       rappor::RapporService* rappor_service) {
+void FormStructure::ParseQueryResponse(
+    std::string payload,
+    const std::vector<FormStructure*>& forms,
+    rappor::RapporServiceImpl* rappor_service) {
   AutofillMetrics::LogServerQueryMetric(
       AutofillMetrics::QUERY_RESPONSE_RECEIVED);
 
@@ -480,7 +469,7 @@ void FormStructure::ParseQueryResponse(std::string payload,
         response.upload_required() ? UPLOAD_REQUIRED : UPLOAD_NOT_REQUIRED;
 
     bool query_response_has_no_server_data = true;
-    for (AutofillField* field : form->fields_) {
+    for (auto& field : form->fields_) {
       if (form->ShouldSkipField(*field))
         continue;
 
@@ -543,13 +532,13 @@ std::vector<FormDataPredictions> FormStructure::GetFieldTypePredictions(
     form.data.origin = form_structure->source_url_;
     form.data.action = form_structure->target_url_;
     form.data.is_form_tag = form_structure->is_form_tag_;
-    form.signature = form_structure->FormSignature();
+    form.signature = form_structure->FormSignatureAsStr();
 
-    for (const AutofillField* field : form_structure->fields_) {
+    for (const auto& field : form_structure->fields_) {
       form.data.fields.push_back(FormFieldData(*field));
 
       FormFieldDataPredictions annotated_field;
-      annotated_field.signature = field->FieldSignature();
+      annotated_field.signature = field->FieldSignatureAsStr();
       annotated_field.heuristic_type =
           AutofillType(field->heuristic_type()).ToString();
       annotated_field.server_type =
@@ -572,8 +561,8 @@ bool FormStructure::IsAutofillFieldMetadataEnabled() {
   return base::StartsWith(group_name, "Enabled", base::CompareCase::SENSITIVE);
 }
 
-std::string FormStructure::FormSignature() const {
-  return base::Uint64ToString(FormSignature64Bit());
+std::string FormStructure::FormSignatureAsStr() const {
+  return base::Uint64ToString(form_signature());
 }
 
 bool FormStructure::IsAutofillable() const {
@@ -583,9 +572,25 @@ bool FormStructure::IsAutofillable() const {
   return ShouldBeParsed();
 }
 
+bool FormStructure::IsCompleteCreditCardForm() const {
+  bool found_cc_number = false;
+  bool found_cc_expiration = false;
+  for (const auto& field : fields_) {
+    ServerFieldType type = field->Type().GetStorableType();
+    if (!found_cc_expiration && IsCreditCardExpirationType(type)) {
+      found_cc_expiration = true;
+    } else if (!found_cc_number && type == CREDIT_CARD_NUMBER) {
+      found_cc_number = true;
+    }
+    if (found_cc_expiration && found_cc_number)
+      return true;
+  }
+  return false;
+}
+
 void FormStructure::UpdateAutofillCount() {
   autofill_count_ = 0;
-  for (const AutofillField* field : *this) {
+  for (const auto& field : *this) {
     if (field && field->IsFieldFillable())
       ++autofill_count_;
   }
@@ -602,11 +607,11 @@ bool FormStructure::ShouldBeParsed() const {
   // Rule out http(s)://*/search?...
   //  e.g. http://www.google.com/search?q=...
   //       http://search.yahoo.com/search?p=...
-  if (target_url_.path() == "/search")
+  if (target_url_.path_piece() == "/search")
     return false;
 
   bool has_text_field = false;
-  for (const AutofillField* it : *this) {
+  for (const auto& it : *this) {
     has_text_field |= it->form_control_type != "select-one";
   }
 
@@ -623,13 +628,13 @@ void FormStructure::UpdateFromCache(const FormStructure& cached_form) {
   // Map from field signatures to cached fields.
   std::map<std::string, const AutofillField*> cached_fields;
   for (size_t i = 0; i < cached_form.field_count(); ++i) {
-    const AutofillField* field = cached_form.field(i);
-    cached_fields[field->FieldSignature()] = field;
+    const auto& field = cached_form.field(i);
+    cached_fields[field->FieldSignatureAsStr()] = field;
   }
 
-  for (AutofillField* field : *this) {
-    std::map<std::string, const AutofillField*>::const_iterator
-        cached_field = cached_fields.find(field->FieldSignature());
+  for (auto& field : *this) {
+    std::map<std::string, const AutofillField*>::const_iterator cached_field =
+        cached_fields.find(field->FieldSignatureAsStr());
     if (cached_field != cached_fields.end()) {
       if (field->form_control_type != "select-one" &&
           field->value == cached_field->second->value) {
@@ -660,13 +665,13 @@ void FormStructure::UpdateFromCache(const FormStructure& cached_form) {
   DCHECK_EQ(cached_form.form_name_, form_name_);
   DCHECK_EQ(cached_form.source_url_, source_url_);
   DCHECK_EQ(cached_form.target_url_, target_url_);
-  form_signature_field_names_ = cached_form.form_signature_field_names_;
+  form_signature_ = cached_form.form_signature_;
 }
 
 void FormStructure::LogQualityMetrics(const base::TimeTicks& load_time,
                                       const base::TimeTicks& interaction_time,
                                       const base::TimeTicks& submission_time,
-                                      rappor::RapporService* rappor_service,
+                                      rappor::RapporServiceImpl* rappor_service,
                                       bool did_show_suggestions,
                                       bool observed_submission) const {
   size_t num_detected_field_types = 0;
@@ -676,7 +681,7 @@ void FormStructure::LogQualityMetrics(const base::TimeTicks& load_time,
   bool did_autofill_all_possible_fields = true;
   bool did_autofill_some_possible_fields = false;
   for (size_t i = 0; i < field_count(); ++i) {
-    const AutofillField* field = this->field(i);
+    const auto& field = this->field(i);
 
     // No further logging for password fields.  Those are primarily related to a
     // different feature code path, and so make more sense to track outside of
@@ -842,7 +847,7 @@ void FormStructure::LogQualityMetrics(const base::TimeTicks& load_time,
 }
 
 void FormStructure::LogQualityMetricsBasedOnAutocomplete() const {
-  for (const AutofillField* field : fields_) {
+  for (const auto& field : fields_) {
     if (field->html_type() != HTML_TYPE_UNSPECIFIED &&
         field->html_type() != HTML_TYPE_UNRECOGNIZED) {
       // The type inferred by the autocomplete attribute.
@@ -883,7 +888,7 @@ void FormStructure::ParseFieldTypesFromAutocompleteAttributes() {
 
   has_author_specified_types_ = false;
   has_author_specified_sections_ = false;
-  for (AutofillField* field : fields_) {
+  for (const auto& field : fields_) {
     // To prevent potential section name collisions, add a default suffix for
     // other fields.  Without this, 'autocomplete' attribute values
     // "section--shipping street-address" and "shipping street-address" would be
@@ -894,17 +899,14 @@ void FormStructure::ParseFieldTypesFromAutocompleteAttributes() {
     // section names.
     field->set_section(kDefaultSection);
 
-    // Canonicalize the attribute value by trimming whitespace, collapsing
-    // non-space characters (e.g. tab) to spaces, and converting to lowercase.
-    std::string autocomplete_attribute =
-        base::CollapseWhitespaceASCII(field->autocomplete_attribute, false);
-    autocomplete_attribute = base::ToLowerASCII(autocomplete_attribute);
+    std::vector<std::string> tokens =
+        LowercaseAndTokenizeAttributeString(field->autocomplete_attribute);
 
     // The autocomplete attribute is overloaded: it can specify either a field
     // type hint or whether autocomplete should be enabled at all.  Ignore the
     // latter type of attribute value.
-    if (autocomplete_attribute.empty() || autocomplete_attribute == "on" ||
-        autocomplete_attribute == "off") {
+    if (tokens.empty() ||
+        (tokens.size() == 1 && (tokens[0] == "on" || tokens[0] == "off"))) {
       continue;
     }
 
@@ -914,15 +916,11 @@ void FormStructure::ParseFieldTypesFromAutocompleteAttributes() {
     // the form.
     has_author_specified_types_ = true;
 
-    // Tokenize the attribute value.  Per the spec, the tokens are parsed in
-    // reverse order.
-    std::vector<std::string> tokens =
-        base::SplitString(autocomplete_attribute, " ", base::KEEP_WHITESPACE,
-                          base::SPLIT_WANT_NONEMPTY);
-
     // The final token must be the field type.
     // If it is not one of the known types, abort.
     DCHECK(!tokens.empty());
+
+    // Per the spec, the tokens are parsed in reverse order.
     std::string field_type_token = tokens.back();
     tokens.pop_back();
     HtmlFieldType field_type =
@@ -1010,7 +1008,7 @@ bool FormStructure::FillFields(
 std::set<base::string16> FormStructure::PossibleValues(ServerFieldType type) {
   std::set<base::string16> values;
   AutofillType target_type(type);
-  for (const AutofillField* field : fields_) {
+  for (const auto& field : fields_) {
     if (field->Type().GetStorableType() != target_type.GetStorableType() ||
         field->Type().group() != target_type.group()) {
       continue;
@@ -1038,7 +1036,7 @@ std::set<base::string16> FormStructure::PossibleValues(ServerFieldType type) {
 
 base::string16 FormStructure::GetUniqueValue(HtmlFieldType type) const {
   base::string16 value;
-  for (const AutofillField* field : fields_) {
+  for (const auto& field : fields_) {
     if (field->html_type() != type)
       continue;
 
@@ -1060,7 +1058,7 @@ const AutofillField* FormStructure::field(size_t index) const {
     return NULL;
   }
 
-  return fields_[index];
+  return fields_[index].get();
 }
 
 AutofillField* FormStructure::field(size_t index) {
@@ -1110,19 +1108,14 @@ void FormStructure::EncodeFormForQuery(
     AutofillQueryContents::Form* query_form) const {
   DCHECK(!IsMalformed());
 
-  query_form->set_signature(FormSignature64Bit());
-  for (const AutofillField* field : fields_) {
+  query_form->set_signature(form_signature());
+  for (const auto& field : fields_) {
     if (ShouldSkipField(*field))
       continue;
 
     AutofillQueryContents::Form::Field* added_field = query_form->add_field();
-    unsigned sig = 0;
 
-    // The signature is a required field. If it can't be parsed, the proto would
-    // not serialize.
-    if (!base::StringToUint(field->FieldSignature(), &sig))
-      continue;
-    added_field->set_signature(sig);
+    added_field->set_signature(field->GetFieldSignature());
 
     if (IsAutofillFieldMetadataEnabled()) {
       added_field->set_type(field->form_control_type);
@@ -1130,12 +1123,6 @@ void FormStructure::EncodeFormForQuery(
       if (!field->name.empty())
         added_field->set_name(base::UTF16ToUTF8(field->name));
 
-      if (!field->label.empty()) {
-        std::string truncated;
-        base::TruncateUTF8ToByteSize(base::UTF16ToUTF8(field->label),
-                                     kMaxFieldLabelNumChars, &truncated);
-        added_field->set_label(truncated);
-      }
     }
   }
 }
@@ -1143,7 +1130,7 @@ void FormStructure::EncodeFormForQuery(
 void FormStructure::EncodeFormForUpload(AutofillUploadContents* upload) const {
   DCHECK(!IsMalformed());
 
-  for (const AutofillField* field : fields_) {
+  for (const auto& field : fields_) {
     // Don't upload checkable fields.
     if (IsCheckable(field->check_status))
       continue;
@@ -1159,12 +1146,12 @@ void FormStructure::EncodeFormForUpload(AutofillUploadContents* upload) const {
       if (field->generation_type())
         added_field->set_generation_type(field->generation_type());
 
-      unsigned sig = 0;
-      // The signature is a required field. If it can't be parsed, the proto
-      // would not serialize.
-      if (!base::StringToUint(field->FieldSignature(), &sig))
-        continue;
-      added_field->set_signature(sig);
+      if (field->form_classifier_outcome()) {
+        added_field->set_form_classifier_outcome(
+            field->form_classifier_outcome());
+      }
+
+      added_field->set_signature(field->GetFieldSignature());
 
       if (IsAutofillFieldMetadataEnabled()) {
         added_field->set_type(field->form_control_type);
@@ -1172,57 +1159,19 @@ void FormStructure::EncodeFormForUpload(AutofillUploadContents* upload) const {
         if (!field->name.empty())
           added_field->set_name(base::UTF16ToUTF8(field->name));
 
-        if (!field->label.empty()) {
-          std::string truncated;
-          base::TruncateUTF8ToByteSize(base::UTF16ToUTF8(field->label),
-                                       kMaxFieldLabelNumChars, &truncated);
-          added_field->set_label(truncated);
-        }
-
         if (!field->autocomplete_attribute.empty())
           added_field->set_autocomplete(field->autocomplete_attribute);
 
         if (!field->css_classes.empty())
           added_field->set_css_classes(base::UTF16ToUTF8(field->css_classes));
+
+        if (field->properties_mask)
+          added_field->set_properties_mask(field->properties_mask);
       }
     }
   }
 }
 
-uint64_t FormStructure::Hash64Bit(const std::string& str) {
-  std::string hash_bin = base::SHA1HashString(str);
-  DCHECK_EQ(base::kSHA1Length, hash_bin.length());
-
-  uint64_t hash64 = (((static_cast<uint64_t>(hash_bin[0])) & 0xFF) << 56) |
-                    (((static_cast<uint64_t>(hash_bin[1])) & 0xFF) << 48) |
-                    (((static_cast<uint64_t>(hash_bin[2])) & 0xFF) << 40) |
-                    (((static_cast<uint64_t>(hash_bin[3])) & 0xFF) << 32) |
-                    (((static_cast<uint64_t>(hash_bin[4])) & 0xFF) << 24) |
-                    (((static_cast<uint64_t>(hash_bin[5])) & 0xFF) << 16) |
-                    (((static_cast<uint64_t>(hash_bin[6])) & 0xFF) << 8) |
-                    ((static_cast<uint64_t>(hash_bin[7])) & 0xFF);
-
-  return hash64;
-}
-
-uint64_t FormStructure::FormSignature64Bit() const {
-  std::string scheme(target_url_.scheme());
-  std::string host(target_url_.host());
-
-  // If target host or scheme is empty, set scheme and host of source url.
-  // This is done to match the Toolbar's behavior.
-  if (scheme.empty() || host.empty()) {
-    scheme = source_url_.scheme();
-    host = source_url_.host();
-  }
-
-  std::string form_string = scheme + "://" + host + "&" +
-                            base::UTF16ToUTF8(form_name_) +
-                            form_signature_field_names_;
-
-  return Hash64Bit(form_string);
-}
-
 bool FormStructure::IsMalformed() const {
   if (!field_count())  // Nothing to add.
     return true;
@@ -1249,7 +1198,7 @@ void FormStructure::IdentifySections(bool has_author_specified_sections) {
     std::set<ServerFieldType> seen_types;
     ServerFieldType previous_type = UNKNOWN_TYPE;
 
-    for (AutofillField* field : fields_) {
+    for (const auto& field : fields_) {
       const ServerFieldType current_type = field->Type().GetStorableType();
 
       bool already_saw_current_type = seen_types.count(current_type) > 0;
@@ -1303,7 +1252,7 @@ void FormStructure::IdentifySections(bool has_author_specified_sections) {
 
   // Ensure that credit card and address fields are in separate sections.
   // This simplifies the section-aware logic in autofill_manager.cc.
-  for (AutofillField* field : fields_) {
+  for (const auto& field : fields_) {
     FieldTypeGroup field_type_group = field->Type().group();
     if (field_type_group == CREDIT_CARD)
       field->set_section(field->section() + "-cc");
@@ -1325,7 +1274,7 @@ void FormStructure::ProcessExtractedFields() {
   // Find the longest common prefix within all the field names.
   std::vector<base::string16> names;
   names.reserve(field_count());
-  for (const AutofillField* field : *this)
+  for (const auto& field : *this)
     names.push_back(field->name);
 
   const base::string16 longest_prefix = FindLongestCommonPrefix(names);
@@ -1333,7 +1282,7 @@ void FormStructure::ProcessExtractedFields() {
     return;
 
   // The name without the prefix will be used for heuristics parsing.
-  for (AutofillField* field : *this) {
+  for (auto& field : *this) {
     if (field->name.size() > longest_prefix.size()) {
       field->set_parseable_name(
           field->name.substr(longest_prefix.size(), field->name.size()));
