@@ -22,9 +22,10 @@ public class MockCertVerifier {
      *         net::GetTestCertsDirectory() to accept in testing.
      * @return a pointer to the newly created net::MockCertVerifier.
      */
-    public static long createMockCertVerifier(String[] certs) {
-        return nativeCreateMockCertVerifier(certs, UrlUtils.getIsolatedTestRoot());
+    public static long createMockCertVerifier(String[] certs, boolean knownRoot) {
+        return nativeCreateMockCertVerifier(certs, knownRoot, UrlUtils.getIsolatedTestRoot());
     }
 
-    private static native long nativeCreateMockCertVerifier(String[] certs, String testDataDir);
+    private static native long nativeCreateMockCertVerifier(
+            String[] certs, boolean knownRoot, String testDataDir);
 }
