@@ -27,14 +27,15 @@
 #include "base/synchronization/waitable_event.h"
 #include "base/test/launcher/unit_test_launcher.h"
 #include "base/test/test_suite.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "base/trace_event/trace_event.h"
 #include "components/display_compositor/gl_helper.h"
 #include "components/display_compositor/gl_helper_readback_support.h"
 #include "components/display_compositor/gl_helper_scaling.h"
-#include "gpu/command_buffer/client/gl_in_process_context.h"
 #include "gpu/command_buffer/client/gles2_implementation.h"
 #include "gpu/command_buffer/client/shared_memory_limits.h"
+#include "gpu/ipc/gl_in_process_context.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkTypes.h"
@@ -66,15 +67,16 @@ class GLHelperTest : public testing::Test {
     attributes.sample_buffers = 1;
     attributes.bind_generates_resource = false;
 
-    context_.reset(gpu::GLInProcessContext::Create(
-        nullptr,                     /* service */
-        nullptr,                     /* surface */
-        true,                        /* offscreen */
-        gfx::kNullAcceleratedWidget, /* window */
-        nullptr,                     /* share_context */
-        attributes, gpu::SharedMemoryLimits(),
-        nullptr, /* gpu_memory_buffer_manager */
-        nullptr /* image_factory */));
+    context_.reset(
+        gpu::GLInProcessContext::Create(nullptr,                 /* service */
+                                        nullptr,                 /* surface */
+                                        true,                    /* offscreen */
+                                        gpu::kNullSurfaceHandle, /* window */
+                                        nullptr, /* share_context */
+                                        attributes, gpu::SharedMemoryLimits(),
+                                        nullptr, /* gpu_memory_buffer_manager */
+                                        nullptr, /* image_factory */
+                                        base::ThreadTaskRunnerHandle::Get()));
     gl_ = context_->GetImplementation();
     gpu::ContextSupport* support = context_->GetImplementation();
 
@@ -258,10 +260,8 @@ class GLHelperTest : public testing::Test {
     }
 
     // Check the output size matches the destination of the last stage
-    EXPECT_EQ(scaler_stages[scaler_stages.size() - 1].dst_size.width(),
-              dst_size.width());
-    EXPECT_EQ(scaler_stages[scaler_stages.size() - 1].dst_size.height(),
-              dst_size.height());
+    EXPECT_EQ(scaler_stages.back().dst_size.width(), dst_size.width());
+    EXPECT_EQ(scaler_stages.back().dst_size.height(), dst_size.height());
 
     // Used to verify that up-scales are not attempted after some
     // other scale.
