@@ -31,7 +31,9 @@ struct GoogleConfigParams {
 };
 
 const GoogleConfigParams kGoogleConfigs[] = {
-  // Origins with subdomains and same-origin collectors.
+  // Origins with subdomains and same-origin collectors. Currently, all
+  // origins with same-origin collectors also run collectors on their www
+  // subdomain. (e.g., both foo.com and www.foo.com.)
   { "google.ac", true, true, true },
   { "google.ad", true, true, true },
   { "google.ae", true, true, true },
@@ -88,7 +90,6 @@ const GoogleConfigParams kGoogleConfigs[] = {
   { "google.co.za", true, true, true },
   { "google.co.zm", true, true, true },
   { "google.co.zw", true, true, true },
-  { "google.com", true, true, false },
   { "google.com.af", true, true, true },
   { "google.com.ag", true, true, true },
   { "google.com.ai", true, true, true },
@@ -254,6 +255,10 @@ const GoogleConfigParams kGoogleConfigs[] = {
   { "google.ws", true, true, true },
   { "l.google.com", true, true, true },
 
+  // google.com is a special case. We have a custom config for www.google.com,
+  // so set generate_config_for_www_subdomain = false.
+  { "google.com", true, true, false },
+
   // Origins with subdomains and without same-origin collectors.
   { "2mdn.net", true, false, false },
   { "adgoogle.net", true, false, false },
@@ -293,8 +298,8 @@ const GoogleConfigParams kGoogleConfigs[] = {
   { "adwhirl.com", true, false, false },
   { "android.com", true, false, false },
   { "anycast-edge.metric.gstatic.com", true, false, false },
-  { "anycast1.metric.gstatic.com", true, false, false },
-  { "anycast1-stb.metric.gstatic.com", true, false, false },
+  { "anycast-stb.metric.gstatic.com", true, false, false },
+  { "anycast.metric.gstatic.com", true, false, false },
   { "chromecast.com", true, false, false },
   { "chromeexperiments.com", true, false, false },
   { "chromestatus.com", true, false, false },
@@ -343,22 +348,24 @@ const GoogleConfigParams kGoogleConfigs[] = {
   { "googleusercontent.com", true, false, false },
   { "gstatic.cn", true, false, false },
   { "gstatic.com", true, false, false },
+  { "gvt3.com", true, false, false },
+  { "gvt9.com", true, false, false },
   { "picasa.com", true, false, false },
   { "recaptcha.net", true, false, false },
   { "stackdriver.com", true, false, false },
-  { "stbcast.metric.gstatic.com", true, false, false },
-  { "stbcast2.metric.gstatic.com", true, false, false },
-  { "stbcast3.metric.gstatic.com", true, false, false },
-  { "stbcast4.metric.gstatic.com", true, false, false },
   { "stbcast-stb.metric.gstatic.com", true, false, false },
+  { "stbcast.metric.gstatic.com", true, false, false },
   { "stbcast2-stb.metric.gstatic.com", true, false, false },
+  { "stbcast2.metric.gstatic.com", true, false, false },
   { "stbcast3-stb.metric.gstatic.com", true, false, false },
+  { "stbcast3.metric.gstatic.com", true, false, false },
   { "stbcast4-stb.metric.gstatic.com", true, false, false },
-  { "unicast.metric.gstatic.com", true, false, false },
+  { "stbcast4.metric.gstatic.com", true, false, false },
   { "unicast-edge.metric.gstatic.com", true, false, false },
   { "unicast-stb.metric.gstatic.com", true, false, false },
-  { "unicast2.metric.gstatic.com", true, false, false },
+  { "unicast.metric.gstatic.com", true, false, false },
   { "unicast2-stb.metric.gstatic.com", true, false, false },
+  { "unicast2.metric.gstatic.com", true, false, false },
   { "waze.com", true, false, false },
   { "withgoogle.com", true, false, false },
   { "youtu.be", true, false, false },
@@ -503,8 +510,8 @@ const GoogleConfigParams kGoogleConfigs[] = {
   { "ddm.google.com", false, true, false },
   { "gmail.com", false, true, false },
   { "gmail.google.com", false, true, false },
-  { "mail.google.com", false, true, false },
   { "mail-attachment.googleusercontent.com", false, true, false },
+  { "mail.google.com", false, true, false },
   { "www.gmail.com", false, true, false },
 
   // Origins without subdomains or same-origin collectors.
@@ -514,6 +521,7 @@ const GoogleConfigParams kGoogleConfigs[] = {
 };
 
 const char* kGoogleStandardCollectors[] = {
+  "https://beacons.gcp.gvt2.com/domainreliability/upload",
   "https://beacons.gvt2.com/domainreliability/upload",
   "https://beacons2.gvt2.com/domainreliability/upload",
   "https://beacons3.gvt2.com/domainreliability/upload",
@@ -544,10 +552,11 @@ static std::unique_ptr<DomainReliabilityConfig> CreateGoogleConfig(
     GURL::Replacements replacements;
     replacements.SetPathStr(kGoogleOriginSpecificCollectorPathString);
     config->collectors.push_back(
-        new GURL(config->origin.ReplaceComponents(replacements)));
+        base::MakeUnique<GURL>(config->origin.ReplaceComponents(replacements)));
   }
   for (size_t i = 0; i < arraysize(kGoogleStandardCollectors); i++)
-    config->collectors.push_back(new GURL(kGoogleStandardCollectors[i]));
+    config->collectors.push_back(
+        base::MakeUnique<GURL>(kGoogleStandardCollectors[i]));
   config->success_sample_rate = 0.05;
   config->failure_sample_rate = 1.00;
   config->path_prefixes.clear();
@@ -558,13 +567,13 @@ static std::unique_ptr<DomainReliabilityConfig> CreateGoogleConfig(
 
 // static
 void GetAllGoogleConfigs(
-    std::vector<DomainReliabilityConfig*>* configs_out) {
+    std::vector<std::unique_ptr<DomainReliabilityConfig>>* configs_out) {
   configs_out->clear();
 
   for (auto& params : kGoogleConfigs) {
-    configs_out->push_back(CreateGoogleConfig(params, false).release());
+    configs_out->push_back(CreateGoogleConfig(params, false));
     if (params.duplicate_for_www)
-      configs_out->push_back(CreateGoogleConfig(params, true).release());
+      configs_out->push_back(CreateGoogleConfig(params, true));
   }
 }
 
