@@ -128,6 +128,9 @@ class CancelTestableFakeDriveService : public FakeDriveService {
   bool upload_new_file_cancelable_;
 };
 
+const char kHello[] = "Hello";
+const size_t kHelloLen = sizeof(kHello) - 1;
+
 }  // namespace
 
 class JobSchedulerTest : public testing::Test {
@@ -584,7 +587,7 @@ TEST_F(JobSchedulerTest, DownloadFileCellularDisabled) {
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
 
   const base::FilePath kOutputFilePath =
-      temp_dir.path().AppendASCII("whatever.txt");
+      temp_dir.GetPath().AppendASCII("whatever.txt");
   google_apis::DriveApiErrorCode download_error =
       google_apis::DRIVE_OTHER_ERROR;
   base::FilePath output_file_path;
@@ -639,7 +642,7 @@ TEST_F(JobSchedulerTest, DownloadFileWimaxDisabled) {
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
 
   const base::FilePath kOutputFilePath =
-      temp_dir.path().AppendASCII("whatever.txt");
+      temp_dir.GetPath().AppendASCII("whatever.txt");
   google_apis::DriveApiErrorCode download_error =
       google_apis::DRIVE_OTHER_ERROR;
   base::FilePath output_file_path;
@@ -694,7 +697,7 @@ TEST_F(JobSchedulerTest, DownloadFileCellularEnabled) {
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
 
   const base::FilePath kOutputFilePath =
-      temp_dir.path().AppendASCII("whatever.txt");
+      temp_dir.GetPath().AppendASCII("whatever.txt");
   google_apis::DriveApiErrorCode download_error =
       google_apis::DRIVE_OTHER_ERROR;
   base::FilePath output_file_path;
@@ -741,7 +744,7 @@ TEST_F(JobSchedulerTest, DownloadFileWimaxEnabled) {
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
 
   const base::FilePath kOutputFilePath =
-      temp_dir.path().AppendASCII("whatever.txt");
+      temp_dir.GetPath().AppendASCII("whatever.txt");
   google_apis::DriveApiErrorCode download_error =
       google_apis::DRIVE_OTHER_ERROR;
   base::FilePath output_file_path;
@@ -814,10 +817,8 @@ TEST_F(JobSchedulerTest, JobInfo) {
   expected_types.insert(TYPE_DOWNLOAD_FILE);
   scheduler_->DownloadFile(
       base::FilePath::FromUTF8Unsafe("drive/whatever.txt"),  // virtual path
-      kDummyDownloadFileSize,
-      temp_dir.path().AppendASCII("whatever.txt"),
-      "2_file_resource_id",
-      ClientContext(BACKGROUND),
+      kDummyDownloadFileSize, temp_dir.GetPath().AppendASCII("whatever.txt"),
+      "2_file_resource_id", ClientContext(BACKGROUND),
       google_apis::test_util::CreateCopyResultCallback(&error, &path),
       google_apis::GetContentCallback());
 
@@ -911,10 +912,8 @@ TEST_F(JobSchedulerTest, JobInfoProgress) {
   // Download job.
   scheduler_->DownloadFile(
       base::FilePath::FromUTF8Unsafe("drive/whatever.txt"),  // virtual path
-      kDummyDownloadFileSize,
-      temp_dir.path().AppendASCII("whatever.txt"),
-      "2_file_resource_id",
-      ClientContext(BACKGROUND),
+      kDummyDownloadFileSize, temp_dir.GetPath().AppendASCII("whatever.txt"),
+      "2_file_resource_id", ClientContext(BACKGROUND),
       google_apis::test_util::CreateCopyResultCallback(&error, &path),
       google_apis::GetContentCallback());
   base::RunLoop().RunUntilIdle();
@@ -927,14 +926,14 @@ TEST_F(JobSchedulerTest, JobInfoProgress) {
   EXPECT_LE(download_progress.back(), 26);
 
   // Upload job.
-  path = temp_dir.path().AppendASCII("new_file.txt");
-  ASSERT_TRUE(google_apis::test_util::WriteStringToFile(path, "Hello"));
+  path = temp_dir.GetPath().AppendASCII("new_file.txt");
+  ASSERT_TRUE(google_apis::test_util::WriteStringToFile(path, kHello));
   google_apis::DriveApiErrorCode upload_error =
       google_apis::DRIVE_OTHER_ERROR;
   std::unique_ptr<google_apis::FileResource> entry;
 
   scheduler_->UploadNewFile(
-      fake_drive_service_->GetRootResourceId(), std::string("Hello").size(),
+      fake_drive_service_->GetRootResourceId(), kHelloLen,
       base::FilePath::FromUTF8Unsafe("drive/new_file.txt"), path, "dummy title",
       "plain/plain", UploadNewFileOptions(), ClientContext(BACKGROUND),
       google_apis::test_util::CreateCopyResultCallback(&upload_error, &entry));
@@ -951,8 +950,8 @@ TEST_F(JobSchedulerTest, JobInfoProgress) {
 TEST_F(JobSchedulerTest, CancelPendingJob) {
   base::ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
-  base::FilePath upload_path = temp_dir.path().AppendASCII("new_file.txt");
-  ASSERT_TRUE(google_apis::test_util::WriteStringToFile(upload_path, "Hello"));
+  base::FilePath upload_path = temp_dir.GetPath().AppendASCII("new_file.txt");
+  ASSERT_TRUE(google_apis::test_util::WriteStringToFile(upload_path, kHello));
 
   // To create a pending job for testing, set the mode to cellular connection
   // and issue BACKGROUND jobs.
@@ -963,7 +962,7 @@ TEST_F(JobSchedulerTest, CancelPendingJob) {
   google_apis::DriveApiErrorCode error1 = google_apis::DRIVE_OTHER_ERROR;
   std::unique_ptr<google_apis::FileResource> entry;
   scheduler_->UploadNewFile(
-      fake_drive_service_->GetRootResourceId(), std::string("Hello").size(),
+      fake_drive_service_->GetRootResourceId(), kHelloLen,
       base::FilePath::FromUTF8Unsafe("dummy/path"), upload_path,
       "dummy title 1", "text/plain", UploadNewFileOptions(),
       ClientContext(BACKGROUND),
@@ -977,7 +976,7 @@ TEST_F(JobSchedulerTest, CancelPendingJob) {
   // Start the second job.
   google_apis::DriveApiErrorCode error2 = google_apis::DRIVE_OTHER_ERROR;
   scheduler_->UploadNewFile(
-      fake_drive_service_->GetRootResourceId(), std::string("Hello").size(),
+      fake_drive_service_->GetRootResourceId(), kHelloLen,
       base::FilePath::FromUTF8Unsafe("dummy/path"), upload_path,
       "dummy title 2", "text/plain", UploadNewFileOptions(),
       ClientContext(BACKGROUND),
@@ -999,15 +998,15 @@ TEST_F(JobSchedulerTest, CancelRunningJob) {
 
   base::ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
-  base::FilePath upload_path = temp_dir.path().AppendASCII("new_file.txt");
-  ASSERT_TRUE(google_apis::test_util::WriteStringToFile(upload_path, "Hello"));
+  base::FilePath upload_path = temp_dir.GetPath().AppendASCII("new_file.txt");
+  ASSERT_TRUE(google_apis::test_util::WriteStringToFile(upload_path, kHello));
 
   // Run as a cancelable task.
   fake_drive_service_->set_upload_new_file_cancelable(true);
   google_apis::DriveApiErrorCode error1 = google_apis::DRIVE_OTHER_ERROR;
   std::unique_ptr<google_apis::FileResource> entry;
   scheduler_->UploadNewFile(
-      fake_drive_service_->GetRootResourceId(), std::string("Hello").size(),
+      fake_drive_service_->GetRootResourceId(), kHelloLen,
       base::FilePath::FromUTF8Unsafe("dummy/path"), upload_path,
       "dummy title 1", "text/plain", UploadNewFileOptions(),
       ClientContext(USER_INITIATED),
@@ -1022,7 +1021,7 @@ TEST_F(JobSchedulerTest, CancelRunningJob) {
   fake_drive_service_->set_upload_new_file_cancelable(false);
   google_apis::DriveApiErrorCode error2 = google_apis::DRIVE_OTHER_ERROR;
   scheduler_->UploadNewFile(
-      fake_drive_service_->GetRootResourceId(), std::string("Hello").size(),
+      fake_drive_service_->GetRootResourceId(), kHelloLen,
       base::FilePath::FromUTF8Unsafe("dummy/path"), upload_path,
       "dummy title 2", "text/plain", UploadNewFileOptions(),
       ClientContext(USER_INITIATED),
