@@ -18,7 +18,7 @@
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/metrics/histogram.h"
-#include "base/metrics/sparse_histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/sequenced_task_runner.h"
 #include "base/threading/thread_restrictions.h"
 #include "components/drive/drive.pb.h"
@@ -630,6 +630,24 @@ bool ResourceMetadataStorage::Initialize() {
     }
   }
 
+  // Update local resouces if 'starred' property has not been initialized.
+  if (resource_map_) {
+    ResourceMetadataHeader header;
+    if (GetHeader(&header) != FILE_ERROR_OK)
+      return false;
+
+    if (!header.starred_property_initialized()) {
+      // largest changestamp == 0 means data in DB is obsolete.
+      // So data for all entries will be reloaded.
+      header.set_largest_changestamp(0);
+      header.set_starred_property_initialized(true);
+      FileError error = PutHeader(header);
+
+      if (error != FILE_ERROR_OK)
+        return false;
+    }
+  }
+
   UMA_HISTOGRAM_ENUMERATION("Drive.MetadataDBInitResult",
                             init_result,
                             DB_INIT_MAX_VALUE);
@@ -826,7 +844,7 @@ ResourceMetadataStorage::GetIterator() {
 
   std::unique_ptr<leveldb::Iterator> it(
       resource_map_->NewIterator(leveldb::ReadOptions()));
-  return base::WrapUnique(new Iterator(std::move(it)));
+  return base::MakeUnique<Iterator>(std::move(it));
 }
 
 FileError ResourceMetadataStorage::GetChild(const std::string& parent_id,
