@@ -6,8 +6,10 @@
 
 #include <stddef.h>
 
+#include <memory>
 #include <string>
 #include <utility>
+#include <vector>
 
 #include "base/files/file_util.h"
 #include "base/json/json_string_value_serializer.h"
@@ -132,7 +134,8 @@ void FileListCallbackAdapter(const FileListCallback& callback,
   for (size_t i = 0; i < change_list->items().size(); ++i) {
     const ChangeResource& entry = *change_list->items()[i];
     if (entry.file())
-      file_list->mutable_items()->push_back(new FileResource(*entry.file()));
+      file_list->mutable_items()->push_back(
+          base::MakeUnique<FileResource>(*entry.file()));
   }
   callback.Run(error, std::move(file_list));
 }
@@ -252,7 +255,6 @@ FakeDriveService::FakeDriveService()
 
 FakeDriveService::~FakeDriveService() {
   DCHECK(thread_checker_.CalledOnValidThread());
-  STLDeleteValues(&entries_);
 }
 
 bool FakeDriveService::LoadAppListForDriveApi(
@@ -261,7 +263,7 @@ bool FakeDriveService::LoadAppListForDriveApi(
 
   // Load JSON data, which must be a dictionary.
   std::unique_ptr<base::Value> value = test_util::LoadJSONFile(relative_path);
-  CHECK_EQ(base::Value::TYPE_DICTIONARY, value->GetType());
+  CHECK_EQ(base::Value::Type::DICTIONARY, value->GetType());
   app_info_value_.reset(
       static_cast<base::DictionaryValue*>(value.release()));
   return !!app_info_value_;
@@ -289,7 +291,7 @@ void FakeDriveService::AddApp(const std::string& app_id,
   JSONStringValueDeserializer json(app_json);
   std::string error_message;
   std::unique_ptr<base::Value> value(json.Deserialize(NULL, &error_message));
-  CHECK_EQ(base::Value::TYPE_DICTIONARY, value->GetType());
+  CHECK_EQ(base::Value::Type::DICTIONARY, value->GetType());
 
   base::ListValue* item_list;
   CHECK(app_info_value_->GetListWithoutPathExpansion("items", &item_list));
@@ -336,10 +338,6 @@ void FakeDriveService::SetQuotaValue(int64_t used, int64_t total) {
   about_resource_->set_quota_bytes_total(total);
 }
 
-GURL FakeDriveService::GetFakeLinkUrl(const std::string& resource_id) {
-  return GURL("https://fake_server/" + net::EscapePath(resource_id));
-}
-
 void FakeDriveService::Initialize(const std::string& account_id) {
   DCHECK(thread_checker_.CalledOnValidThread());
 }
@@ -552,8 +550,8 @@ CancelCallback FakeDriveService::GetFileResource(
   if (entry && entry->change_resource.file()) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE, base::Bind(callback, HTTP_SUCCESS,
-                              base::Passed(base::WrapUnique(new FileResource(
-                                  *entry->change_resource.file())))));
+                              base::Passed(base::MakeUnique<FileResource>(
+                                  *entry->change_resource.file()))));
     return CancelCallback();
   }
 
@@ -840,7 +838,7 @@ CancelCallback FakeDriveService::CopyResource(
   copied_entry->content_data = entry->content_data;
   copied_entry->share_url = entry->share_url;
   copied_entry->change_resource.set_file(
-      base::WrapUnique(new FileResource(*entry->change_resource.file())));
+      base::MakeUnique<FileResource>(*entry->change_resource.file()));
 
   ChangeResource* new_change = &copied_entry->change_resource;
   FileResource* new_file = new_change->mutable_file();
@@ -851,7 +849,6 @@ CancelCallback FakeDriveService::CopyResource(
 
   ParentReference parent;
   parent.set_file_id(parent_resource_id);
-  parent.set_parent_link(GetFakeLinkUrl(parent_resource_id));
   std::vector<ParentReference> parents;
   parents.push_back(parent);
   *new_file->mutable_parents() = parents;
@@ -863,12 +860,12 @@ CancelCallback FakeDriveService::CopyResource(
   UpdateETag(new_file);
 
   // Add the new entry to the map.
-  entries_[new_resource_id] = copied_entry.release();
+  entries_[new_resource_id] = std::move(copied_entry);
 
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(callback, HTTP_SUCCESS,
-                 base::Passed(base::WrapUnique(new FileResource(*new_file)))));
+                 base::Passed(base::MakeUnique<FileResource>(*new_file))));
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&FakeDriveService::NotifyObservers,
@@ -919,7 +916,6 @@ CancelCallback FakeDriveService::UpdateResource(
   if (!parent_resource_id.empty()) {
     ParentReference parent;
     parent.set_file_id(parent_resource_id);
-    parent.set_parent_link(GetFakeLinkUrl(parent_resource_id));
 
     std::vector<ParentReference> parents;
     parents.push_back(parent);
@@ -938,7 +934,7 @@ CancelCallback FakeDriveService::UpdateResource(
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(callback, HTTP_SUCCESS,
-                 base::Passed(base::WrapUnique(new FileResource(*file)))));
+                 base::Passed(base::MakeUnique<FileResource>(*file))));
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&FakeDriveService::NotifyObservers,
@@ -973,7 +969,6 @@ CancelCallback FakeDriveService::AddResourceToDirectory(
   // one more parent, not overwriting old ones.
   ParentReference parent;
   parent.set_file_id(parent_resource_id);
-  parent.set_parent_link(GetFakeLinkUrl(parent_resource_id));
   change->mutable_file()->mutable_parents()->push_back(parent);
 
   AddNewChangestamp(change);
@@ -1226,7 +1221,7 @@ CancelCallback FakeDriveService::ResumeUpload(
 
     completion_callback.Run(
         HTTP_CREATED,
-        base::WrapUnique(new FileResource(*new_entry->change_resource.file())));
+        base::MakeUnique<FileResource>(*new_entry->change_resource.file()));
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
         base::Bind(&FakeDriveService::NotifyObservers,
@@ -1253,8 +1248,7 @@ CancelCallback FakeDriveService::ResumeUpload(
   AddNewChangestamp(change);
   UpdateETag(file);
 
-  completion_callback.Run(HTTP_SUCCESS,
-                          base::WrapUnique(new FileResource(*file)));
+  completion_callback.Run(HTTP_SUCCESS, base::MakeUnique<FileResource>(*file));
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&FakeDriveService::NotifyObservers,
@@ -1416,8 +1410,8 @@ void FakeDriveService::AddNewFileWithResourceId(
 
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE, base::Bind(callback, HTTP_CREATED,
-                            base::Passed(base::WrapUnique(new FileResource(
-                                *new_entry->change_resource.file())))));
+                            base::Passed(base::MakeUnique<FileResource>(
+                                *new_entry->change_resource.file()))));
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&FakeDriveService::NotifyObservers,
@@ -1460,8 +1454,8 @@ CancelCallback FakeDriveService::AddNewDirectoryWithResourceId(
 
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE, base::Bind(callback, HTTP_CREATED,
-                            base::Passed(base::WrapUnique(new FileResource(
-                                *new_entry->change_resource.file())))));
+                            base::Passed(base::MakeUnique<FileResource>(
+                                *new_entry->change_resource.file()))));
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&FakeDriveService::NotifyObservers,
@@ -1498,7 +1492,7 @@ void FakeDriveService::SetLastModifiedTime(
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(callback, HTTP_SUCCESS,
-                 base::Passed(base::WrapUnique(new FileResource(*file)))));
+                 base::Passed(base::MakeUnique<FileResource>(*file))));
 }
 
 google_apis::DriveApiErrorCode FakeDriveService::SetUserPermission(
@@ -1553,10 +1547,11 @@ FakeDriveService::EntryInfo* FakeDriveService::FindEntryByResourceId(
     const std::string& resource_id) {
   DCHECK(thread_checker_.CalledOnValidThread());
 
-  EntryInfoMap::iterator it = entries_.find(resource_id);
+  auto it = entries_.find(resource_id);
   // Deleted entries don't have FileResource.
-  return it != entries_.end() && it->second->change_resource.file() ?
-      it->second : NULL;
+  return it != entries_.end() && it->second->change_resource.file()
+             ? it->second.get()
+             : NULL;
 }
 
 std::string FakeDriveService::GetNewResourceId() {
@@ -1633,7 +1628,6 @@ const FakeDriveService::EntryInfo* FakeDriveService::AddNewEntry(
   if (!parent_resource_id.empty()) {
     ParentReference parent;
     parent.set_file_id(parent_resource_id);
-    parent.set_parent_link(GetFakeLinkUrl(parent.file_id()));
     std::vector<ParentReference> parents;
     parents.push_back(parent);
     *new_file->mutable_parents() = parents;
@@ -1649,8 +1643,8 @@ const FakeDriveService::EntryInfo* FakeDriveService::AddNewEntry(
       base::Time() + base::TimeDelta::FromMilliseconds(++published_date_seq_);
   new_file->set_created_date(published_date);
 
-  EntryInfo* raw_new_entry = new_entry.release();
-  entries_[resource_id] = raw_new_entry;
+  EntryInfo* raw_new_entry = new_entry.get();
+  entries_[resource_id] = std::move(new_entry);
   return raw_new_entry;
 }
 
@@ -1671,10 +1665,9 @@ void FakeDriveService::GetChangeListInternal(
 
   // Filter out entries per parameters like |directory_resource_id| and
   // |search_query|.
-  ScopedVector<ChangeResource> entries;
+  std::vector<std::unique_ptr<ChangeResource>> entries;
   int num_entries_matched = 0;
-  for (EntryInfoMap::iterator it = entries_.begin(); it != entries_.end();
-       ++it) {
+  for (auto it = entries_.begin(); it != entries_.end(); ++it) {
     const ChangeResource& entry = it->second->change_resource;
     bool should_exclude = false;
 
@@ -1727,11 +1720,10 @@ void FakeDriveService::GetChangeListInternal(
       entry_copied->set_file_id(entry.file_id());
       entry_copied->set_deleted(entry.is_deleted());
       if (entry.file()) {
-        entry_copied->set_file(
-            base::WrapUnique(new FileResource(*entry.file())));
+        entry_copied->set_file(base::MakeUnique<FileResource>(*entry.file()));
       }
       entry_copied->set_modification_date(entry.modification_date());
-      entries.push_back(entry_copied.release());
+      entries.push_back(std::move(entry_copied));
     }
   }
 
@@ -1803,7 +1795,8 @@ FakeDriveService::StartBatchRequest() {
 }
 
 void FakeDriveService::NotifyObservers() {
-  FOR_EACH_OBSERVER(ChangeObserver, change_observers_, OnNewChangeAvailable());
+  for (auto& observer : change_observers_)
+    observer.OnNewChangeAvailable();
 }
 
 }  // namespace drive
