@@ -12,6 +12,7 @@
 #include "base/i18n/rtl.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
@@ -23,20 +24,20 @@
 #include "components/error_page/common/error_page_switches.h"
 #include "components/error_page/common/net_error_info.h"
 #include "components/strings/grit/components_chromium_strings.h"
-#include "components/strings/grit/components_google_chrome_strings.h"
 #include "components/strings/grit/components_strings.h"
 #include "components/url_formatter/url_formatter.h"
 #include "net/base/escape.h"
 #include "net/base/net_errors.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/webui/web_ui_util.h"
+#include "url/origin.h"
 
 #if defined(OS_WIN)
 #include "base/win/windows_version.h"
 #endif
 
 #if defined(OS_ANDROID)
-#include "components/offline_pages/offline_page_feature.h"
+#include "components/offline_pages/core/offline_page_feature.h"
 #endif
 
 namespace error_page {
@@ -65,6 +66,7 @@ enum NAV_SUGGESTIONS {
   SUGGEST_COMPLETE_SETUP                    = 1 << 11,
   // Reload page suggestion for pages created by a post.
   SUGGEST_REPOST_RELOAD                     = 1 << 12,
+  SUGGEST_NAVIGATE_TO_ORIGIN                = 1 << 13,
 };
 
 enum SHOW_BUTTONS {
@@ -74,7 +76,6 @@ enum SHOW_BUTTONS {
 
 struct LocalizedErrorMap {
   int error_code;
-  unsigned int title_resource_id;
   unsigned int heading_resource_id;
   // Detailed summary used when the error is in the main frame and shown on
   // mouse over when the error is in a frame.
@@ -83,9 +84,9 @@ struct LocalizedErrorMap {
   int buttons; // Which buttons if any to show.
 };
 
+// clang-format off
 const LocalizedErrorMap net_error_options[] = {
   {net::ERR_TIMED_OUT,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_TIMED_OUT,
    SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG |
@@ -93,7 +94,6 @@ const LocalizedErrorMap net_error_options[] = {
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CONNECTION_TIMED_OUT,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_TIMED_OUT,
    SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG |
@@ -101,7 +101,6 @@ const LocalizedErrorMap net_error_options[] = {
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CONNECTION_CLOSED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_CONNECTION_CLOSED,
    SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG |
@@ -109,7 +108,6 @@ const LocalizedErrorMap net_error_options[] = {
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CONNECTION_RESET,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_CONNECTION_RESET,
    SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG |
@@ -117,21 +115,18 @@ const LocalizedErrorMap net_error_options[] = {
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CONNECTION_REFUSED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_CONNECTION_REFUSED,
    SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CONNECTION_FAILED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_CONNECTION_FAILED,
    SUGGEST_DIAGNOSE_TOOL,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_NAME_NOT_RESOLVED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_NAME_NOT_RESOLVED,
    SUGGEST_CHECK_CONNECTION | SUGGEST_DNS_CONFIG | SUGGEST_FIREWALL_CONFIG |
@@ -139,21 +134,18 @@ const LocalizedErrorMap net_error_options[] = {
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_ICANN_NAME_COLLISION,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_ICANN_NAME_COLLISION,
    SUGGEST_NONE,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_ADDRESS_UNREACHABLE,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_ADDRESS_UNREACHABLE,
    SUGGEST_DIAGNOSE_TOOL,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_NETWORK_ACCESS_DENIED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NETWORK_ACCESS_DENIED,
    IDS_ERRORPAGES_SUMMARY_NETWORK_ACCESS_DENIED,
    SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG |
@@ -161,202 +153,181 @@ const LocalizedErrorMap net_error_options[] = {
    SHOW_NO_BUTTONS,
   },
   {net::ERR_PROXY_CONNECTION_FAILED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED,
    IDS_ERRORPAGES_SUMMARY_PROXY_CONNECTION_FAILED,
    SUGGEST_PROXY_CONFIG | SUGGEST_CONTACT_ADMINISTRATOR | SUGGEST_DIAGNOSE_TOOL,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_INTERNET_DISCONNECTED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED,
    IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED,
    SUGGEST_OFFLINE_CHECKS | SUGGEST_DIAGNOSE_TOOL,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_FILE_NOT_FOUND,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_FILE_NOT_FOUND,
    IDS_ERRORPAGES_SUMMARY_FILE_NOT_FOUND,
    SUGGEST_NONE,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_CACHE_MISS,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_CACHE_READ_FAILURE,
    IDS_ERRORPAGES_SUMMARY_CACHE_READ_FAILURE,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CACHE_READ_FAILURE,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_CACHE_READ_FAILURE,
    IDS_ERRORPAGES_SUMMARY_CACHE_READ_FAILURE,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_NETWORK_IO_SUSPENDED,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_CONNECTION_INTERRUPTED,
    IDS_ERRORPAGES_SUMMARY_NETWORK_IO_SUSPENDED,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_TOO_MANY_REDIRECTS,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_TOO_MANY_REDIRECTS,
    SUGGEST_LEARNMORE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_EMPTY_RESPONSE,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_EMPTY_RESPONSE,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_LENGTH,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_DISPOSITION,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_RESPONSE_HEADERS_MULTIPLE_LOCATION,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+   IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE,
+   SUGGEST_NONE,
+   SHOW_BUTTON_RELOAD,
+  },
+  {net::ERR_INVALID_HTTP_RESPONSE,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_CONTENT_LENGTH_MISMATCH,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_CONNECTION_CLOSED,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_INCOMPLETE_CHUNKED_ENCODING,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
    IDS_ERRORPAGES_SUMMARY_CONNECTION_CLOSED,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_SSL_PROTOCOL_ERROR,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE,
    SUGGEST_DIAGNOSE_TOOL,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_BAD_SSL_CLIENT_AUTH_CERT,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_ERRORPAGES_SUMMARY_BAD_SSL_CLIENT_AUTH_CERT,
    SUGGEST_CONTACT_ADMINISTRATOR,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_SSL_WEAK_SERVER_EPHEMERAL_DH_KEY,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_ERRORPAGES_SUMMARY_SSL_SECURITY_ERROR,
    SUGGEST_LEARNMORE,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_SSL_PINNED_KEY_NOT_IN_CERT_CHAIN,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_CERT_ERROR_SUMMARY_PINNING_FAILURE_DETAILS,
    SUGGEST_NONE,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_TEMPORARILY_THROTTLED,
-   IDS_ERRORPAGES_TITLE_ACCESS_DENIED,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
    SUGGEST_DISABLE_EXTENSION,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_BLOCKED_BY_CLIENT,
-   IDS_ERRORPAGES_TITLE_BLOCKED,
    IDS_ERRORPAGES_HEADING_BLOCKED,
    IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_EXTENSION,
    SUGGEST_DISABLE_EXTENSION,
    SHOW_BUTTON_RELOAD,
   },
+  {net::ERR_BLOCKED_BY_XSS_AUDITOR,
+   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+   IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_XSS_AUDITOR,
+   SUGGEST_NAVIGATE_TO_ORIGIN,
+   SHOW_NO_BUTTONS,
+  },
   {net::ERR_NETWORK_CHANGED,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_CONNECTION_INTERRUPTED,
    IDS_ERRORPAGES_SUMMARY_NETWORK_CHANGED,
    SUGGEST_NONE,
    SHOW_BUTTON_RELOAD,
   },
   {net::ERR_BLOCKED_BY_ADMINISTRATOR,
-   IDS_ERRORPAGES_TITLE_BLOCKED,
    IDS_ERRORPAGES_HEADING_BLOCKED,
    IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_ADMINISTRATOR,
    SUGGEST_CONTACT_ADMINISTRATOR,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_BLOCKED_ENROLLMENT_CHECK_PENDING,
-   IDS_ERRORPAGES_TITLE_BLOCKED,
    IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED,
    IDS_ERRORPAGES_SUMMARY_BLOCKED_ENROLLMENT_CHECK_PENDING,
    SUGGEST_COMPLETE_SETUP,
    SHOW_NO_BUTTONS,
   },
-  {net::ERR_SSL_FALLBACK_BEYOND_MINIMUM_VERSION,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
-   IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
-   IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE,
-   SUGGEST_NONE,
-   SHOW_NO_BUTTONS,
-  },
   {net::ERR_SSL_VERSION_OR_CIPHER_MISMATCH,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_ERRORPAGES_SUMMARY_SSL_VERSION_OR_CIPHER_MISMATCH,
    SUGGEST_UNSUPPORTED_CIPHER,
    SHOW_NO_BUTTONS,
   },
   {net::ERR_SSL_OBSOLETE_CIPHER,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_ERRORPAGES_SUMMARY_SSL_VERSION_OR_CIPHER_MISMATCH,
    SUGGEST_UNSUPPORTED_CIPHER,
    SHOW_NO_BUTTONS,
   },
-  {net::ERR_TEMPORARY_BACKOFF,
-   IDS_ERRORPAGES_TITLE_ACCESS_DENIED,
-   IDS_ERRORPAGES_HEADING_ACCESS_DENIED,
-   IDS_ERRORPAGES_SUMMARY_TEMPORARY_BACKOFF,
-   SUGGEST_NONE,
-   SHOW_NO_BUTTONS,
-  },
   {net::ERR_SSL_SERVER_CERT_BAD_FORMAT,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION,
    IDS_ERRORPAGES_SUMMARY_SSL_SECURITY_ERROR,
    SUGGEST_NONE,
    SHOW_NO_BUTTONS,
   },
+  {net::ERR_BLOCKED_BY_RESPONSE,
+   IDS_ERRORPAGES_HEADING_BLOCKED,
+   IDS_ERRORPAGES_SUMMARY_CONNECTION_REFUSED,
+   SUGGEST_NONE,
+   SHOW_NO_BUTTONS
+  },
 };
+// clang-format on
 
 // Special error page to be used in the case of navigating back to a page
 // generated by a POST.  LocalizedError::HasStrings expects this net error code
 // to also appear in the array above.
 const LocalizedErrorMap repost_error = {
   net::ERR_CACHE_MISS,
-  IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
   IDS_HTTP_POST_WARNING_TITLE,
   IDS_ERRORPAGES_HTTP_POST_WARNING,
   SUGGEST_REPOST_RELOAD,
@@ -364,61 +335,48 @@ const LocalizedErrorMap repost_error = {
 };
 
 const LocalizedErrorMap http_error_options[] = {
-  {403,
-   IDS_ERRORPAGES_TITLE_ACCESS_DENIED,
-   IDS_ERRORPAGES_HEADING_ACCESS_DENIED,
-   IDS_ERRORPAGES_SUMMARY_FORBIDDEN,
-   SUGGEST_NONE,
-   SHOW_BUTTON_RELOAD,
-  },
-  {410,
-   IDS_ERRORPAGES_TITLE_NOT_FOUND,
-   IDS_ERRORPAGES_HEADING_NOT_FOUND,
-   IDS_ERRORPAGES_SUMMARY_GONE,
-   SUGGEST_NONE,
-   SHOW_NO_BUTTONS,
-  },
+    {
+        403, IDS_ERRORPAGES_HEADING_ACCESS_DENIED,
+        IDS_ERRORPAGES_SUMMARY_FORBIDDEN, SUGGEST_NONE, SHOW_BUTTON_RELOAD,
+    },
+    {
+        404, IDS_ERRORPAGES_HEADING_NOT_FOUND, IDS_ERRORPAGES_SUMMARY_NOT_FOUND,
+        SUGGEST_NONE, SHOW_BUTTON_RELOAD,
+    },
+    {
+        410, IDS_ERRORPAGES_HEADING_NOT_FOUND, IDS_ERRORPAGES_SUMMARY_GONE,
+        SUGGEST_NONE, SHOW_NO_BUTTONS,
+    },
 
-  {500,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
-   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
-   IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST,
-   SUGGEST_NONE,
-   SHOW_BUTTON_RELOAD,
-  },
-  {501,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
-   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
-   IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST,
-   SUGGEST_NONE,
-   SHOW_NO_BUTTONS,
-  },
-  {502,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
-   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
-   IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST,
-   SUGGEST_NONE,
-   SHOW_BUTTON_RELOAD,
-  },
-  {503,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
-   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
-   IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST,
-   SUGGEST_NONE,
-   SHOW_BUTTON_RELOAD,
-  },
-  {504,
-   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
-   IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
-   IDS_ERRORPAGES_SUMMARY_GATEWAY_TIMEOUT,
-   SUGGEST_NONE,
-   SHOW_BUTTON_RELOAD,
-  },
+    {
+        500, IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+        IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST, SUGGEST_NONE,
+        SHOW_BUTTON_RELOAD,
+    },
+    {
+        501, IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+        IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST, SUGGEST_NONE,
+        SHOW_NO_BUTTONS,
+    },
+    {
+        502, IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+        IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST, SUGGEST_NONE,
+        SHOW_BUTTON_RELOAD,
+    },
+    {
+        503, IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+        IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST, SUGGEST_NONE,
+        SHOW_BUTTON_RELOAD,
+    },
+    {
+        504, IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING,
+        IDS_ERRORPAGES_SUMMARY_GATEWAY_TIMEOUT, SUGGEST_NONE,
+        SHOW_BUTTON_RELOAD,
+    },
 };
 
 const LocalizedErrorMap dns_probe_error_options[] = {
   {error_page::DNS_PROBE_POSSIBLE,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_DNS_PROBE_RUNNING,
    SUGGEST_DIAGNOSE_TOOL,
@@ -429,7 +387,6 @@ const LocalizedErrorMap dns_probe_error_options[] = {
   // error, which might be one of several DNS-related errors.
 
   {error_page::DNS_PROBE_STARTED,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_DNS_PROBE_RUNNING,
    // Include SUGGEST_RELOAD so the More button doesn't jump when we update.
@@ -441,14 +398,12 @@ const LocalizedErrorMap dns_probe_error_options[] = {
   // original error, which might be one of several DNS-related errors.
 
   {error_page::DNS_PROBE_FINISHED_NO_INTERNET,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED,
    IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED,
    SUGGEST_OFFLINE_CHECKS | SUGGEST_DIAGNOSE_TOOL,
    SHOW_NO_BUTTONS,
   },
   {error_page::DNS_PROBE_FINISHED_BAD_CONFIG,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_NAME_NOT_RESOLVED,
    SUGGEST_DNS_CONFIG | SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG |
@@ -456,7 +411,6 @@ const LocalizedErrorMap dns_probe_error_options[] = {
    SHOW_BUTTON_RELOAD,
   },
   {error_page::DNS_PROBE_FINISHED_NXDOMAIN,
-   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_NAME_NOT_RESOLVED,
    SUGGEST_DIAGNOSE_TOOL,
@@ -561,13 +515,13 @@ void AddSingleEntryDictionaryToList(base::ListValue* list,
                                     const char* path,
                                     int message_id,
                                     bool insert_as_first_item) {
-  base::DictionaryValue* suggestion_list_item = new base::DictionaryValue;
+  auto suggestion_list_item = base::MakeUnique<base::DictionaryValue>();
   suggestion_list_item->SetString(path, l10n_util::GetStringUTF16(message_id));
 
   if (insert_as_first_item) {
-    list->Insert(0, suggestion_list_item);
+    list->Insert(0, std::move(suggestion_list_item));
   } else {
-    list->Append(suggestion_list_item);
+    list->Append(std::move(suggestion_list_item));
   }
 }
 
@@ -676,6 +630,23 @@ void GetSuggestionsSummaryList(int error_code,
   }
   DCHECK(!IsSuggested(suggestions, SUGGEST_REPOST_RELOAD));
 
+  if (IsOnlySuggestion(suggestions, SUGGEST_NAVIGATE_TO_ORIGIN)) {
+    DCHECK(suggestions_summary_list->empty());
+    DCHECK(!(suggestions & ~SUGGEST_NAVIGATE_TO_ORIGIN));
+    url::Origin failed_origin(failed_url);
+    if (failed_origin.unique())
+      return;
+
+    auto suggestion = base::MakeUnique<base::DictionaryValue>();
+    suggestion->SetString("summary",
+                          l10n_util::GetStringUTF16(
+                              IDS_ERRORPAGES_SUGGESTION_NAVIGATE_TO_ORIGIN));
+    suggestion->SetString("originURL", failed_origin.Serialize());
+    suggestions_summary_list->Append(std::move(suggestion));
+    return;
+  }
+  DCHECK(!IsSuggested(suggestions, SUGGEST_NAVIGATE_TO_ORIGIN));
+
   if (IsOnlySuggestion(suggestions, SUGGEST_LEARNMORE)) {
     DCHECK(suggestions_summary_list->empty());
     AddLinkedSuggestionToList(error_code, locale, suggestions_summary_list,
@@ -785,7 +756,7 @@ base::DictionaryValue* AddSuggestionDetailDictionaryToList(
     suggestion_list_item->SetString("body",
         l10n_util::GetStringUTF16(body_message_id));
   }
-  list->Append(suggestion_list_item);
+  list->Append(base::WrapUnique(suggestion_list_item));
   return suggestion_list_item;
 }
 
@@ -870,7 +841,7 @@ void LocalizedError::GetStrings(
     bool is_post,
     bool stale_copy_in_cache,
     bool can_show_network_diagnostics_dialog,
-    bool has_offline_pages,
+    bool is_incognito,
     const std::string& locale,
     std::unique_ptr<error_page::ErrorPageParams> params,
     base::DictionaryValue* error_strings) {
@@ -880,7 +851,6 @@ void LocalizedError::GetStrings(
   // options with default values.
   LocalizedErrorMap options = {
     0,
-    IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
     IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
     IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
     SUGGEST_NONE,
@@ -899,7 +869,6 @@ void LocalizedError::GetStrings(
   if (error_domain == net::kErrorDomain &&
       error_code == net::ERR_ACCESS_DENIED &&
       failed_url.scheme() == "file") {
-    options.title_resource_id = IDS_ERRORPAGES_TITLE_ACCESS_DENIED;
     options.heading_resource_id = IDS_ERRORPAGES_HEADING_FILE_ACCESS_DENIED;
     options.summary_resource_id = IDS_ERRORPAGES_SUMMARY_FILE_ACCESS_DENIED;
     options.suggestions = SUGGEST_NONE;
@@ -912,12 +881,15 @@ void LocalizedError::GetStrings(
   // URLs are always LTR.
   if (base::i18n::IsRTL())
     base::i18n::WrapStringWithLTRFormatting(&failed_url_string);
-  error_strings->SetString("title",
-      l10n_util::GetStringFUTF16(options.title_resource_id, failed_url_string));
-  std::string icon_class = GetIconClassForError(error_domain, error_code);
-  error_strings->SetString("iconClass", icon_class);
 
   base::string16 host_name(url_formatter::IDNToUnicode(failed_url.host()));
+  if (failed_url.SchemeIsHTTPOrHTTPS())
+    error_strings->SetString("title", host_name);
+  else
+    error_strings->SetString("title", failed_url_string);
+
+  std::string icon_class = GetIconClassForError(error_domain, error_code);
+  error_strings->SetString("iconClass", icon_class);
 
   base::DictionaryValue* heading = new base::DictionaryValue;
   heading->SetString("msg",
@@ -1006,7 +978,13 @@ void LocalizedError::GetStrings(
 
   // Add the reload suggestion, if needed for pages that didn't come
   // from a post.
+#if defined(OS_ANDROID)
+  bool reload_visible = false;
+#endif  // defined(OS_ANDROID)
   if (params->suggest_reload && !is_post) {
+#if defined(OS_ANDROID)
+    reload_visible = true;
+#endif  // defined(OS_ANDROID)
     base::DictionaryValue* reload_button = new base::DictionaryValue;
     reload_button->SetString(
         "msg", l10n_util::GetStringUTF16(IDS_ERRORPAGES_BUTTON_RELOAD));
@@ -1057,6 +1035,23 @@ void LocalizedError::GetStrings(
       show_saved_copy_button->SetString("primary", "true");
     error_strings->Set("showSavedCopyButton", show_saved_copy_button);
   }
+
+#if defined(OS_ANDROID)
+  if (!is_post && !reload_visible && !show_saved_copy_visible &&
+      !is_incognito && failed_url.is_valid() &&
+      failed_url.SchemeIsHTTPOrHTTPS() &&
+      offline_pages::IsOfflinePagesAsyncDownloadEnabled()) {
+    std::unique_ptr<base::DictionaryValue> download_button =
+        base::MakeUnique<base::DictionaryValue>();
+    download_button->SetString(
+        "msg",
+        l10n_util::GetStringUTF16(IDS_ERRORPAGES_BUTTON_DOWNLOAD));
+    download_button->SetString(
+        "disabledMsg",
+        l10n_util::GetStringUTF16(IDS_ERRORPAGES_BUTTON_DOWNLOADING));
+    error_strings->Set("downloadButton", std::move(download_button));
+  }
+#endif  // defined(OS_ANDROID)
 }
 
 base::string16 LocalizedError::GetErrorDetails(const std::string& error_domain,
