@@ -5,7 +5,9 @@
 #include "components/exo/test/exo_test_base.h"
 
 #include "base/command_line.h"
+#include "base/memory/ptr_util.h"
 #include "components/exo/test/exo_test_helper.h"
+#include "components/exo/wm_helper_ash.h"
 #include "ui/wm/core/wm_core_switches.h"
 
 namespace exo {
@@ -23,9 +25,13 @@ void ExoTestBase::SetUp() {
   // Disable window animation when running tests.
   command_line->AppendSwitch(wm::switches::kWindowAnimationsDisabled);
   AshTestBase::SetUp();
+  wm_helper_ = base::MakeUnique<WMHelperAsh>();
+  WMHelper::SetInstance(wm_helper_.get());
 }
 
 void ExoTestBase::TearDown() {
+  WMHelper::SetInstance(nullptr);
+  wm_helper_.reset();
   AshTestBase::TearDown();
 }
 
