@@ -19,6 +19,7 @@
 #include "base/stl_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/test/scoped_feature_list.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "components/flags_ui/feature_entry.h"
@@ -62,23 +63,18 @@ const char kDisableFeatures[] = "dummy-disable-features";
 
 const char kTestTrial[] = "TestTrial";
 const char kTestParam[] = "param";
-const char kTestParamValue1[] = "value1";
-const char kTestParamValue2[] = "value2";
+const char kTestParamValue[] = "value";
 
 const base::Feature kTestFeature1{"FeatureName1",
                                   base::FEATURE_ENABLED_BY_DEFAULT};
 const base::Feature kTestFeature2{"FeatureName2",
                                   base::FEATURE_ENABLED_BY_DEFAULT};
 
-const FeatureEntry::FeatureParam kTestVariationDefault[] = {
-    {kTestParam, kTestParamValue1}};
-
 const FeatureEntry::FeatureParam kTestVariationOther[] = {
-    {kTestParam, kTestParamValue2}};
+    {kTestParam, kTestParamValue}};
 
 const FeatureEntry::FeatureVariation kTestVariations[] = {
-    {"", kTestVariationDefault, 1},
-    {"dummy description", kTestVariationOther, 1}};
+    {"dummy description", kTestVariationOther, 1, nullptr}};
 
 // Those have to be valid ids for the translation system but the value are
 // never used, so pick one at random from the current component.
@@ -148,6 +144,10 @@ class FlagsStateTest : public ::testing::Test {
     flags_state_.reset(new FlagsState(kEntries, arraysize(kEntries)));
   }
 
+  ~FlagsStateTest() override {
+    variations::testing::ClearAllVariationParams();
+  }
+
   TestingPrefServiceSimple prefs_;
   PrefServiceFlagsStorage flags_storage_;
   std::unique_ptr<FlagsState> flags_state_;
@@ -268,23 +268,27 @@ TEST_F(FlagsStateTest, ConvertFlagsToSwitches) {
 
 TEST_F(FlagsStateTest, RegisterAllFeatureVariationParameters) {
   const FeatureEntry& entry = kEntries[7];
-  // Select the "Disabled" variation.
+  std::unique_ptr<base::FeatureList> feature_list(new base::FeatureList);
+
+  // Select the "Default" variation.
   flags_state_->SetFeatureEntryEnabled(&flags_storage_, entry.NameForOption(0),
                                        true);
-  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_);
+  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_,
+                                                      feature_list.get());
   // No value should be associated.
   EXPECT_EQ("", variations::GetVariationParamValue(kTestTrial, kTestParam));
   // The trial should not be created.
   base::FieldTrial* trial = base::FieldTrialList::Find(kTestTrial);
   EXPECT_EQ(nullptr, trial);
 
-  // Select the first "Enabled" variation.
+  // Select the default "Enabled" variation.
   flags_state_->SetFeatureEntryEnabled(&flags_storage_, entry.NameForOption(1),
                                        true);
 
-  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_);
-  // The value should be associated.
-  EXPECT_EQ(kTestParamValue1,
+  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_,
+                                                      feature_list.get());
+  // No value should be associated as this is the default option.
+  EXPECT_EQ("",
             variations::GetVariationParamValue(kTestTrial, kTestParam));
 
   // The trial should be created.
@@ -293,15 +297,40 @@ TEST_F(FlagsStateTest, RegisterAllFeatureVariationParameters) {
   // The about:flags group should be selected for the trial.
   EXPECT_EQ(internal::kTrialGroupAboutFlags, trial->group_name());
 
-  // Select the second "Enabled" variation.
+  // Select the only one variation.
   flags_state_->SetFeatureEntryEnabled(&flags_storage_, entry.NameForOption(2),
                                        true);
-  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_);
+  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_,
+                                                      feature_list.get());
   // Associating for the second time should not change the value.
-  EXPECT_EQ(kTestParamValue1,
+  EXPECT_EQ("",
             variations::GetVariationParamValue(kTestTrial, kTestParam));
 }
 
+TEST_F(FlagsStateTest, RegisterAllFeatureVariationParametersNonDefault) {
+  const FeatureEntry& entry = kEntries[7];
+  std::unique_ptr<base::FeatureList> feature_list(new base::FeatureList);
+
+  // Select the only one variation.
+  flags_state_->SetFeatureEntryEnabled(&flags_storage_, entry.NameForOption(2),
+                                       true);
+  flags_state_->RegisterAllFeatureVariationParameters(&flags_storage_,
+                                                      feature_list.get());
+
+  // Set the feature_list as the main instance so that
+  // variations::GetVariationParamValueByFeature below works.
+  base::test::ScopedFeatureList scoped_feature_list;
+  scoped_feature_list.InitWithFeatureList(std::move(feature_list));
+
+  // The param should have the value predefined in this variation.
+  EXPECT_EQ(kTestParamValue,
+            variations::GetVariationParamValue(kTestTrial, kTestParam));
+
+  // The value should be associated also via the name of the feature.
+  EXPECT_EQ(kTestParamValue, variations::GetVariationParamValueByFeature(
+                                  kTestFeature2, kTestParam));
+}
+
 base::CommandLine::StringType CreateSwitch(const std::string& value) {
 #if defined(OS_WIN)
   return base::ASCIIToUTF16(value);
@@ -380,10 +409,10 @@ TEST_F(FlagsStateTest, RemoveFlagSwitches) {
   // This shouldn't do anything before ConvertFlagsToSwitches() wasn't called.
   flags_state_->RemoveFlagsSwitches(&switch_list);
   ASSERT_EQ(4u, switch_list.size());
-  EXPECT_TRUE(ContainsKey(switch_list, kSwitch1));
-  EXPECT_TRUE(ContainsKey(switch_list, switches::kFlagSwitchesBegin));
-  EXPECT_TRUE(ContainsKey(switch_list, switches::kFlagSwitchesEnd));
-  EXPECT_TRUE(ContainsKey(switch_list, "foo"));
+  EXPECT_TRUE(base::ContainsKey(switch_list, kSwitch1));
+  EXPECT_TRUE(base::ContainsKey(switch_list, switches::kFlagSwitchesBegin));
+  EXPECT_TRUE(base::ContainsKey(switch_list, switches::kFlagSwitchesEnd));
+  EXPECT_TRUE(base::ContainsKey(switch_list, "foo"));
 
   // Call ConvertFlagsToSwitches(), then RemoveFlagsSwitches() again.
   base::CommandLine command_line(base::CommandLine::NO_PROGRAM);
@@ -395,7 +424,7 @@ TEST_F(FlagsStateTest, RemoveFlagSwitches) {
 
   // Now the about:flags-related switch should have been removed.
   ASSERT_EQ(1u, switch_list.size());
-  EXPECT_TRUE(ContainsKey(switch_list, "foo"));
+  EXPECT_TRUE(base::ContainsKey(switch_list, "foo"));
 }
 
 TEST_F(FlagsStateTest, RemoveFlagSwitches_Features) {
@@ -446,13 +475,13 @@ TEST_F(FlagsStateTest, RemoveFlagSwitches_Features) {
                                          kDisableFeatures);
     auto switch_list = command_line.GetSwitches();
     EXPECT_EQ(cases[i].expected_enable_features != nullptr,
-              ContainsKey(switch_list, kEnableFeatures));
+              base::ContainsKey(switch_list, kEnableFeatures));
     if (cases[i].expected_enable_features)
       EXPECT_EQ(CreateSwitch(cases[i].expected_enable_features),
                 switch_list[kEnableFeatures]);
 
     EXPECT_EQ(cases[i].expected_disable_features != nullptr,
-              ContainsKey(switch_list, kDisableFeatures));
+              base::ContainsKey(switch_list, kDisableFeatures));
     if (cases[i].expected_disable_features)
       EXPECT_EQ(CreateSwitch(cases[i].expected_disable_features),
                 switch_list[kDisableFeatures]);
@@ -462,12 +491,12 @@ TEST_F(FlagsStateTest, RemoveFlagSwitches_Features) {
     switch_list = command_line.GetSwitches();
     flags_state_->RemoveFlagsSwitches(&switch_list);
     EXPECT_EQ(cases[i].existing_enable_features != nullptr,
-              ContainsKey(switch_list, kEnableFeatures));
+              base::ContainsKey(switch_list, kEnableFeatures));
     if (cases[i].existing_enable_features)
       EXPECT_EQ(CreateSwitch(cases[i].existing_enable_features),
                 switch_list[kEnableFeatures]);
     EXPECT_EQ(cases[i].existing_disable_features != nullptr,
-              ContainsKey(switch_list, kEnableFeatures));
+              base::ContainsKey(switch_list, kEnableFeatures));
     if (cases[i].existing_disable_features)
       EXPECT_EQ(CreateSwitch(cases[i].existing_disable_features),
                 switch_list[kDisableFeatures]);
