@@ -5,6 +5,7 @@
 #include "components/history/core/browser/delete_directive_handler.h"
 
 #include <stddef.h>
+
 #include <utility>
 
 #include "base/json/json_writer.h"
@@ -14,10 +15,10 @@
 #include "components/history/core/browser/history_backend.h"
 #include "components/history/core/browser/history_db_task.h"
 #include "components/history/core/browser/history_service.h"
-#include "sync/api/sync_change.h"
-#include "sync/protocol/history_delete_directive_specifics.pb.h"
-#include "sync/protocol/proto_value_conversions.h"
-#include "sync/protocol/sync.pb.h"
+#include "components/sync/model/sync_change.h"
+#include "components/sync/protocol/history_delete_directive_specifics.pb.h"
+#include "components/sync/protocol/proto_value_conversions.h"
+#include "components/sync/protocol/sync.pb.h"
 
 namespace {
 
