@@ -38,7 +38,6 @@ class PrefService;
 class PrefRegistrySimple;
 
 namespace base {
-class DictionaryValue;
 class HistogramSamples;
 class PrefService;
 }
@@ -47,14 +46,12 @@ namespace variations {
 struct ActiveGroupId;
 }
 
-namespace net {
-class URLFetcher;
-}
-
 namespace metrics {
 
 class MetricsLogUploader;
 class MetricsReportingScheduler;
+class MetricsRotationScheduler;
+class MetricsUploadScheduler;
 class MetricsServiceAccessor;
 class MetricsServiceClient;
 class MetricsStateManager;
@@ -124,16 +121,6 @@ class MetricsService : public base::HistogramFlattener {
   // Returns true if the last session exited cleanly.
   bool WasLastShutdownClean() const;
 
-  // Returns the preferred entropy provider used to seed persistent activities
-  // based on whether or not metrics reporting will be permitted on this client.
-  //
-  // If metrics reporting is enabled, this method returns an entropy provider
-  // that has a high source of entropy, partially based on the client ID.
-  // Otherwise, it returns an entropy provider that is based on a low entropy
-  // source.
-  std::unique_ptr<const base::FieldTrial::EntropyProvider>
-  CreateEntropyProvider();
-
   // At startup, prefs needs to be called with a list of all the pref names and
   // types we'll be using.
   static void RegisterPrefs(PrefRegistrySimple* registry);
@@ -216,9 +203,13 @@ class MetricsService : public base::HistogramFlattener {
   // Pushes a log that has been generated by an external component.
   void PushExternalLog(const std::string& log);
 
-  // Returns a callback to data use pref updating function which can be called
-  // from any thread, but this function should be called on UI thread.
-  UpdateUsagePrefCallbackType GetDataUseForwardingCallback();
+  // Updates data usage tracking prefs with the specified values.
+  void UpdateMetricsUsagePrefs(const std::string& service_name,
+                               int message_size,
+                               bool is_cellular);
+
+  // Merge any data from metrics providers into the global StatisticsRecorder.
+  void MergeHistogramDeltas();
 
  protected:
   // Exposed for testing.
@@ -259,9 +250,18 @@ class MetricsService : public base::HistogramFlattener {
   // registered at a time for a given trial_name. Only the last group name that
   // is registered for a given trial name will be recorded. The values passed
   // in must not correspond to any real field trial in the code.
+  // Note: Should not be used to replace trials that were registered with
+  // RegisterMultiGroupSyntheticFieldTrial().
   void RegisterSyntheticFieldTrial(
       const variations::SyntheticTrialGroup& trial_group);
 
+  // Similar to RegisterSyntheticFieldTrial(), but registers a synthetic trial
+  // that has multiple active groups for a given trial name hash. Any previous
+  // groups registered for |trial_name_hash| will be replaced.
+  void RegisterSyntheticMultiGroupFieldTrial(
+      uint32_t trial_name_hash,
+      const std::vector<uint32_t>& group_name_hashes);
+
   // Calls into the client to initialize some system profile metrics.
   void StartInitTask();
 
@@ -275,7 +275,7 @@ class MetricsService : public base::HistogramFlattener {
   // call to this function.  Also updates the cumulative uptime metric (stored
   // as a pref) for uninstall.  Uptimes are measured using TimeTicks, which
   // guarantees that it is monotonic and does not jump if the user changes
-  // his/her clock.  The TimeTicks implementation also makes the clock not
+  // their clock.  The TimeTicks implementation also makes the clock not
   // count time the computer is suspended.
   void GetUptimes(PrefService* pref,
                   base::TimeDelta* incremental_uptime,
@@ -340,9 +340,10 @@ class MetricsService : public base::HistogramFlattener {
   // Prepares the initial stability log, which is only logged when the previous
   // run of Chrome crashed.  This log contains any stability metrics left over
   // from that previous run, and only these stability metrics.  It uses the
-  // system profile from the previous session.  Returns true if a log was
-  // created.
-  bool PrepareInitialStabilityLog();
+  // system profile from the previous session.  |prefs_previous_version| is used
+  // to validate the version number recovered from the system profile.  Returns
+  // true if a log was created.
+  bool PrepareInitialStabilityLog(const std::string& prefs_previous_version);
 
   // Prepares the initial metrics log, which includes startup histograms and
   // profiler data, as well as incremental stability-related metrics.
@@ -362,18 +363,12 @@ class MetricsService : public base::HistogramFlattener {
   void IncrementLongPrefsValue(const char* path);
 
   // Records that the browser was shut down cleanly.
-  void LogCleanShutdown();
+  void LogCleanShutdown(bool end_completed);
 
   // Records state that should be periodically saved, like uptime and
   // buffered plugin stability statistics.
   void RecordCurrentState(PrefService* pref);
 
-  // Checks whether events should currently be logged.
-  bool ShouldLogEvents();
-
-  // Sets the value of the specified path in prefs and schedules a save.
-  void RecordBooleanPrefValue(const char* path, bool value);
-
   // Notifies observers on a synthetic trial list change.
   void NotifySyntheticTrialObservers();
 
@@ -396,10 +391,6 @@ class MetricsService : public base::HistogramFlattener {
   // i.e., histograms with the |kUmaStabilityHistogramFlag| flag set.
   void RecordCurrentStabilityHistograms();
 
-  // Skips staged upload and discards the log. Used in case of unsuccessful
-  // upload or intentional sampling of logs.
-  void SkipAndDiscardUpload();
-
   // Manager for the various in-flight logs.
   MetricsLogManager log_manager_;
 
@@ -455,8 +446,13 @@ class MetricsService : public base::HistogramFlattener {
   // A number that identifies the how many times the app has been launched.
   int session_id_;
 
-  // The scheduler for determining when uploads should happen.
+  // The scheduler for determining when log rotations+uploads should happen.
+  // TODO(holte): Remove this once we've switched to split schedulers.
   std::unique_ptr<MetricsReportingScheduler> scheduler_;
+  // The scheduler for determining when log rotations should happen.
+  std::unique_ptr<MetricsRotationScheduler> rotation_scheduler_;
+  // The scheduler for determining when uploads should happen.
+  std::unique_ptr<MetricsUploadScheduler> upload_scheduler_;
 
   // Stores the time of the first call to |GetUptimes()|.
   base::TimeTicks first_updated_time_;
@@ -482,6 +478,8 @@ class MetricsService : public base::HistogramFlattener {
   FRIEND_TEST_ALL_PREFIXES(MetricsServiceTest,
                            PermutedEntropyCacheClearedWhenLowEntropyReset);
   FRIEND_TEST_ALL_PREFIXES(MetricsServiceTest, RegisterSyntheticTrial);
+  FRIEND_TEST_ALL_PREFIXES(MetricsServiceTest,
+                           RegisterSyntheticMultiGroupFieldTrial);
 
   // Pointer used for obtaining data use pref updater callback on above layers.
   std::unique_ptr<DataUseTracker> data_use_tracker_;
