@@ -7,7 +7,6 @@
 
 #include <map>
 #include <memory>
-#include <utility>
 
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
@@ -46,7 +45,7 @@ class NonSfiListener : public IPC::Listener {
   base::Thread io_thread_;
   base::WaitableEvent shutdown_event_;
   std::unique_ptr<IPC::SyncChannel> channel_;
-  scoped_refptr<NaClTrustedListener> trusted_listener_;
+  std::unique_ptr<NaClTrustedListener> trusted_listener_;
 
   std::unique_ptr<std::map<std::string, int>> key_fd_map_;
 
