@@ -14,6 +14,7 @@
 #include "base/files/scoped_temp_dir.h"
 #include "base/i18n/case_conversion.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/path_service.h"
 #include "base/run_loop.h"
@@ -32,6 +33,7 @@
 #include "components/omnibox/browser/in_memory_url_index_test_util.h"
 #include "components/omnibox/browser/in_memory_url_index_types.h"
 #include "components/omnibox/browser/url_index_private_data.h"
+#include "components/search_engines/template_url_service.h"
 #include "sql/transaction.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -53,6 +55,18 @@ const size_t kInvalid = base::string16::npos;
 const size_t kMaxMatches = 3;
 const char kClientWhitelistedScheme[] = "xyz";
 
+// TemplateURLs used to test filtering of search engine URLs.
+const char kDefaultTemplateURLKeyword[] = "default-engine.com";
+const char kNonDefaultTemplateURLKeyword[] = "non-default-engine.com";
+const TemplateURLService::Initializer kTemplateURLData[] = {
+    {kDefaultTemplateURLKeyword,
+     "http://default-engine.com?q={searchTerms}",
+     "Default"},
+    {kNonDefaultTemplateURLKeyword,
+     "http://non-default-engine.com?q={searchTerms}",
+     "Not Default"},
+};
+
 // Helper function to set lower case |lower_string| and |lower_terms| (words
 // list) based on supplied |search_string| and |cursor_position|. If
 // |cursor_position| is set and useful (not at either end of the string), allow
@@ -159,6 +173,7 @@ class InMemoryURLIndexTest : public testing::Test {
   base::ScopedTempDir history_dir_;
   std::unique_ptr<history::HistoryService> history_service_;
   history::HistoryDatabase* history_database_;
+  std::unique_ptr<TemplateURLService> template_url_service_;
   std::unique_ptr<InMemoryURLIndex> url_index_;
 };
 
@@ -218,7 +233,8 @@ bool InMemoryURLIndexTest::DeleteURL(const GURL& url) {
 void InMemoryURLIndexTest::SetUp() {
   // We cannot access the database until the backend has been loaded.
   if (history_dir_.CreateUniqueTempDir())
-    history_service_ = history::CreateHistoryService(history_dir_.path(), true);
+    history_service_ =
+        history::CreateHistoryService(history_dir_.GetPath(), true);
   ASSERT_TRUE(history_service_);
   BlockUntilInMemoryURLIndexIsRefreshed(url_index_.get());
 
@@ -300,6 +316,13 @@ void InMemoryURLIndexTest::SetUp() {
     transaction.Commit();
   }
 
+  // Set up a simple template URL service with a default search engine.
+  template_url_service_ = base::MakeUnique<TemplateURLService>(
+      kTemplateURLData, arraysize(kTemplateURLData));
+  TemplateURL* template_url = template_url_service_->GetTemplateURLForKeyword(
+      base::ASCIIToUTF16(kDefaultTemplateURLKeyword));
+  template_url_service_->SetUserSelectedDefaultSearchProvider(template_url);
+
   if (InitializeInMemoryURLIndexInSetUp())
     InitializeInMemoryURLIndex();
 }
@@ -325,8 +348,8 @@ void InMemoryURLIndexTest::InitializeInMemoryURLIndex() {
   SchemeSet client_schemes_to_whitelist;
   client_schemes_to_whitelist.insert(kClientWhitelistedScheme);
   url_index_.reset(new InMemoryURLIndex(
-      nullptr, history_service_.get(), nullptr, pool_owner_.pool().get(),
-      base::FilePath(), client_schemes_to_whitelist));
+      nullptr, history_service_.get(), template_url_service_.get(),
+      pool_owner_.pool().get(), base::FilePath(), client_schemes_to_whitelist));
   url_index_->Init();
   url_index_->RebuildFromHistory(history_database_);
 }
@@ -537,7 +560,20 @@ TEST_F(InMemoryURLIndexTest, Retrieval) {
   // No results since it will be suppressed by default scoring.
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("qui c"),
                                              base::string16::npos, kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
+
+  // A URL that comes from the default search engine should not be returned.
+  matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("query"),
+                                             base::string16::npos, kMaxMatches);
+  EXPECT_EQ(0U, matches.size());
+
+  // But if it's not from the default search engine, it should be returned.
+  TemplateURL* template_url = template_url_service_->GetTemplateURLForKeyword(
+      base::ASCIIToUTF16(kNonDefaultTemplateURLKeyword));
+  template_url_service_->SetUserSelectedDefaultSearchProvider(template_url);
+  matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("query"),
+                                             base::string16::npos, kMaxMatches);
+  EXPECT_EQ(1U, matches.size());
 
   // Search which will match at the end of an URL with encoded characters.
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("Mice"),
@@ -545,7 +581,7 @@ TEST_F(InMemoryURLIndexTest, Retrieval) {
   ASSERT_EQ(1U, matches.size());
   EXPECT_EQ(30, matches[0].url_info.id());
 
-  // Check that URLs are not escaped an escape time.
+  // Check that URLs are not escaped an extra time.
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("1% wikipedia"),
                                              base::string16::npos, kMaxMatches);
   ASSERT_EQ(1U, matches.size());
@@ -567,12 +603,12 @@ TEST_F(InMemoryURLIndexTest, CursorPositionRetrieval) {
   // See if a very specific term with no cursor gives an empty result.
   ScoredHistoryMatches matches = url_index_->HistoryItemsForTerms(
       ASCIIToUTF16("DrudReport"), base::string16::npos, kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
 
   // The same test with the cursor at the end should give an empty result.
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("DrudReport"), 10u,
                                              kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
 
   // If the cursor is between Drud and Report, we should find the desired
   // result.
@@ -586,12 +622,12 @@ TEST_F(InMemoryURLIndexTest, CursorPositionRetrieval) {
   // result on this input.
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("MORTGAGERATE DROPS"),
                                              base::string16::npos, kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
 
   // Ditto with cursor at end.
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("MORTGAGERATE DROPS"),
                                              18u, kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
 
   // If the cursor is between MORTAGE And RATE, we should find the
   // desired result.
@@ -624,7 +660,7 @@ TEST_F(InMemoryURLIndexTest, URLPrefixMatching) {
   // "view.atdmt" - found
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("view.atdmt"),
                                              base::string16::npos, kMaxMatches);
-  ASSERT_EQ(1U, matches.size());
+  EXPECT_EQ(1U, matches.size());
 
   // "view.atdmt" - found
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("view.atdmt"),
@@ -665,13 +701,13 @@ TEST_F(InMemoryURLIndexTest, ProperStringMatching) {
   // "view.atdmt" - found
   ScoredHistoryMatches matches = url_index_->HistoryItemsForTerms(
       ASCIIToUTF16("atdmt view"), base::string16::npos, kMaxMatches);
-  ASSERT_EQ(1U, matches.size());
+  EXPECT_EQ(1U, matches.size());
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("atdmt.view"),
                                              base::string16::npos, kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
   matches = url_index_->HistoryItemsForTerms(ASCIIToUTF16("view.atdmt"),
                                              base::string16::npos, kMaxMatches);
-  ASSERT_EQ(1U, matches.size());
+  EXPECT_EQ(1U, matches.size());
 }
 
 TEST_F(InMemoryURLIndexTest, HugeResultSet) {
@@ -686,16 +722,16 @@ TEST_F(InMemoryURLIndexTest, HugeResultSet) {
   ScoredHistoryMatches matches = url_index_->HistoryItemsForTerms(
       ASCIIToUTF16("b"), base::string16::npos, kMaxMatches);
   URLIndexPrivateData& private_data(*GetPrivateData());
-  ASSERT_EQ(kMaxMatches, matches.size());
+  EXPECT_EQ(kMaxMatches, matches.size());
   // There are 7 matches already in the database.
-  ASSERT_EQ(1008U, private_data.pre_filter_item_count_);
-  ASSERT_EQ(500U, private_data.post_filter_item_count_);
-  ASSERT_EQ(kMaxMatches, private_data.post_scoring_item_count_);
+  EXPECT_EQ(1008U, private_data.pre_filter_item_count_);
+  EXPECT_EQ(500U, private_data.post_filter_item_count_);
+  EXPECT_EQ(kMaxMatches, private_data.post_scoring_item_count_);
 }
 
 TEST_F(InMemoryURLIndexTest, TitleSearch) {
   // Signal if someone has changed the test DB.
-  EXPECT_EQ(29U, GetPrivateData()->history_info_map_.size());
+  EXPECT_EQ(30U, GetPrivateData()->history_info_map_.size());
 
   // Ensure title is being searched.
   ScoredHistoryMatches matches = url_index_->HistoryItemsForTerms(
@@ -733,7 +769,7 @@ TEST_F(InMemoryURLIndexTest, TitleChange) {
   base::string16 new_terms = ASCIIToUTF16("does eat oats little lambs ivy");
   matches = url_index_->HistoryItemsForTerms(new_terms, base::string16::npos,
                                              kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
 
   // Update the row.
   old_row.set_title(ASCIIToUTF16("Does eat oats and little lambs eat ivy"));
@@ -746,7 +782,7 @@ TEST_F(InMemoryURLIndexTest, TitleChange) {
   EXPECT_EQ(expected_id, matches[0].url_info.id());
   matches = url_index_->HistoryItemsForTerms(original_terms,
                                              base::string16::npos, kMaxMatches);
-  ASSERT_EQ(0U, matches.size());
+  EXPECT_EQ(0U, matches.size());
 }
 
 TEST_F(InMemoryURLIndexTest, NonUniqueTermCharacterSets) {
@@ -960,7 +996,7 @@ TEST_F(InMemoryURLIndexTest, WhitelistedURLs) {
     { "prospero://host.dom//pros/name", false },
     { "rsync://syler@lost.com/Source", false },
     { "rtsp://media.example.com:554/twister/audiotrack", false },
-    { "service:acap://some.where.net;authentication=KERBEROSV4", false },
+    { "acap://some.where.net;authentication=KERBEROSV4", false },
     { "shttp://www.terces.com/secret", false },
     { "sieve://example.com//script", false },
     { "sip:+1-212-555-1212:1234@gateway.com;user=phone", false },
@@ -1035,7 +1071,7 @@ TEST_F(InMemoryURLIndexTest, ReadVisitsFromHistory) {
 TEST_F(InMemoryURLIndexTest, CacheSaveRestore) {
   base::ScopedTempDir temp_directory;
   ASSERT_TRUE(temp_directory.CreateUniqueTempDir());
-  set_history_dir(temp_directory.path());
+  set_history_dir(temp_directory.GetPath());
 
   URLIndexPrivateData& private_data(*GetPrivateData());
 
@@ -1104,7 +1140,7 @@ TEST_F(InMemoryURLIndexTest, CacheSaveRestore) {
 TEST_F(InMemoryURLIndexTest, RebuildFromHistoryIfCacheOld) {
   base::ScopedTempDir temp_directory;
   ASSERT_TRUE(temp_directory.CreateUniqueTempDir());
-  set_history_dir(temp_directory.path());
+  set_history_dir(temp_directory.GetPath());
 
   URLIndexPrivateData& private_data(*GetPrivateData());
 
@@ -1176,7 +1212,7 @@ TEST_F(InMemoryURLIndexTest, RebuildFromHistoryIfCacheOld) {
   ExpectPrivateDataEqual(*old_data.get(), new_data);
 }
 
-TEST_F(InMemoryURLIndexTest, AddHistoryMatch) {
+TEST_F(InMemoryURLIndexTest, CalculateWordStartsOffsets) {
   const struct {
     const char* search_string;
     size_t cursor_position;
@@ -1240,18 +1276,17 @@ TEST_F(InMemoryURLIndexTest, AddHistoryMatch) {
     String16Vector lower_terms;
     StringToTerms(test_cases[i].search_string, test_cases[i].cursor_position,
                   &lower_string, &lower_terms);
-    URLIndexPrivateData::AddHistoryMatch match(nullptr, nullptr,
-                                               *GetPrivateData(),
-                                               lower_string, lower_terms,
-                                               base::Time::Now());
+    WordStarts lower_terms_to_word_starts_offsets;
+    URLIndexPrivateData::CalculateWordStartsOffsets(
+        lower_terms, &lower_terms_to_word_starts_offsets);
 
     // Verify against expectations.
     EXPECT_EQ(test_cases[i].expected_word_starts_offsets_size,
-              match.lower_terms_to_word_starts_offsets_.size());
+              lower_terms_to_word_starts_offsets.size());
     for (size_t j = 0; j < test_cases[i].expected_word_starts_offsets_size;
          ++j) {
       EXPECT_EQ(test_cases[i].expected_word_starts_offsets[j],
-                match.lower_terms_to_word_starts_offsets_[j]);
+                lower_terms_to_word_starts_offsets[j]);
     }
   }
 }
@@ -1279,7 +1314,7 @@ InMemoryURLIndexCacheTest::InMemoryURLIndexCacheTest()
 
 void InMemoryURLIndexCacheTest::SetUp() {
   ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
-  base::FilePath path(temp_dir_.path());
+  base::FilePath path(temp_dir_.GetPath());
   url_index_.reset(new InMemoryURLIndex(nullptr, nullptr, nullptr,
                                         pool_owner_.pool().get(), path,
                                         SchemeSet()));
@@ -1303,7 +1338,7 @@ bool InMemoryURLIndexCacheTest::GetCacheFilePath(
 
 TEST_F(InMemoryURLIndexCacheTest, CacheFilePath) {
   base::FilePath expectedPath =
-      temp_dir_.path().Append(FILE_PATH_LITERAL("History Provider Cache"));
+      temp_dir_.GetPath().Append(FILE_PATH_LITERAL("History Provider Cache"));
   std::vector<base::FilePath::StringType> expected_parts;
   expectedPath.GetComponents(&expected_parts);
   base::FilePath full_file_path;
