@@ -8,6 +8,7 @@
 
 #include "base/command_line.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
 #include "base/strings/string16.h"
 #include "build/build_config.h"
@@ -30,7 +31,7 @@ class OmniboxFieldTrialTest : public testing::Test {
     // a DCHECK.
     field_trial_list_.reset();
     field_trial_list_.reset(new base::FieldTrialList(
-        new metrics::SHA1EntropyProvider("foo")));
+        base::MakeUnique<metrics::SHA1EntropyProvider>("foo")));
     variations::testing::ClearAllVariationParams();
   }
 
@@ -363,10 +364,6 @@ TEST_F(OmniboxFieldTrialTest, GetValueForRuleInContext) {
                     OmniboxEventProto::HOME_PAGE);         // exact match
     ExpectRuleValue("rule1-4-*-value",
                     "rule1", OmniboxEventProto::OTHER);    // partial fallback
-    ExpectRuleValue("rule1-*-*-value",
-                    "rule1",
-                    OmniboxEventProto::                    // fallback to global
-                    SEARCH_RESULT_PAGE_DOING_SEARCH_TERM_REPLACEMENT);
     // Tests for rule 2.
     ExpectRuleValue("rule2-*-0-value",
                     "rule2",
