@@ -7,14 +7,14 @@
 #include "base/values.h"
 #include "components/autofill/core/common/autofill_pref_names.h"
 #include "components/policy/core/common/policy_map.h"
+#include "components/policy/policy_constants.h"
 #include "components/prefs/pref_value_map.h"
-#include "policy/policy_constants.h"
 
 namespace policy {
 
 AutofillPolicyHandler::AutofillPolicyHandler()
     : TypeCheckingPolicyHandler(key::kAutoFillEnabled,
-                                base::Value::TYPE_BOOLEAN) {}
+                                base::Value::Type::BOOLEAN) {}
 
 AutofillPolicyHandler::~AutofillPolicyHandler() {
 }
