@@ -10,8 +10,10 @@
 #include "base/bind_helpers.h"
 #include "base/guid.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/stl_util.h"
 #include "components/policy/core/common/cloud/device_management_service.h"
+#include "components/policy/core/common/cloud/signing_service.h"
 #include "google_apis/gaia/gaia_constants.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "net/url_request/url_request_context_getter.h"
@@ -31,6 +33,8 @@ DeviceMode TranslateProtobufDeviceMode(
       return DEVICE_MODE_ENTERPRISE;
     case em::DeviceRegisterResponse::RETAIL:
       return DEVICE_MODE_LEGACY_RETAIL_MODE;
+    case em::DeviceRegisterResponse::CHROME_AD:
+      return DEVICE_MODE_ENTERPRISE_AD;
   }
   LOG(ERROR) << "Unknown enrollment mode in registration response: " << mode;
   return DEVICE_MODE_NOT_SET;
@@ -51,25 +55,18 @@ void CloudPolicyClient::Observer::OnRobotAuthCodesFetched(
 CloudPolicyClient::CloudPolicyClient(
     const std::string& machine_id,
     const std::string& machine_model,
-    const std::string& verification_key_hash,
     DeviceManagementService* service,
-    scoped_refptr<net::URLRequestContextGetter> request_context)
+    scoped_refptr<net::URLRequestContextGetter> request_context,
+    SigningService* signing_service)
     : machine_id_(machine_id),
       machine_model_(machine_model),
-      verification_key_hash_(verification_key_hash),
-      device_mode_(DEVICE_MODE_NOT_SET),
-      submit_machine_id_(false),
-      public_key_version_(-1),
-      public_key_version_valid_(false),
-      invalidation_version_(0),
-      fetched_invalidation_version_(0),
       service_(service),                  // Can be null for unit tests.
-      status_(DM_STATUS_SUCCESS),
-      request_context_(request_context) {
+      signing_service_(signing_service),
+      request_context_(request_context),
+      weak_ptr_factory_(this) {
 }
 
 CloudPolicyClient::~CloudPolicyClient() {
-  STLDeleteValues(&responses_);
 }
 
 void CloudPolicyClient::SetupRegistration(const std::string& dm_token,
@@ -82,11 +79,19 @@ void CloudPolicyClient::SetupRegistration(const std::string& dm_token,
   client_id_ = client_id;
   request_jobs_.clear();
   policy_fetch_request_job_.reset();
-  STLDeleteValues(&responses_);
+  responses_.clear();
 
   NotifyRegistrationStateChanged();
 }
 
+// Sets the client ID or generate a new one. A new one is intentionally
+// generated on each new registration request in order to preserve privacy.
+// Reusing IDs would mean the server could track clients by their registration
+// attempts.
+void CloudPolicyClient::SetClientId(const std::string& client_id) {
+  client_id_ = client_id.empty() ?  base::GenerateGUID() : client_id;
+}
+
 void CloudPolicyClient::Register(em::DeviceRegisterRequest::Type type,
                                  em::DeviceRegisterRequest::Flavor flavor,
                                  const std::string& auth_token,
@@ -97,14 +102,7 @@ void CloudPolicyClient::Register(em::DeviceRegisterRequest::Type type,
   DCHECK(!auth_token.empty());
   DCHECK(!is_registered());
 
-  if (client_id.empty()) {
-    // Generate a new client ID. This is intentionally done on each new
-    // registration request in order to preserve privacy. Reusing IDs would mean
-    // the server could track clients by their registration attempts.
-    client_id_ = base::GenerateGUID();
-  } else {
-    client_id_ = client_id;
-  }
+  SetClientId(client_id);
 
   policy_fetch_request_job_.reset(
       service_->CreateJob(DeviceManagementRequestJob::TYPE_REGISTRATION,
@@ -128,11 +126,75 @@ void CloudPolicyClient::Register(em::DeviceRegisterRequest::Type type,
   request->set_flavor(flavor);
 
   policy_fetch_request_job_->SetRetryCallback(
-      base::Bind(&CloudPolicyClient::OnRetryRegister, base::Unretained(this)));
+      base::Bind(&CloudPolicyClient::OnRetryRegister,
+                 weak_ptr_factory_.GetWeakPtr()));
 
   policy_fetch_request_job_->Start(
       base::Bind(&CloudPolicyClient::OnRegisterCompleted,
-                 base::Unretained(this)));
+                 weak_ptr_factory_.GetWeakPtr()));
+}
+
+void CloudPolicyClient::RegisterWithCertificate(
+    em::DeviceRegisterRequest::Type type,
+    em::DeviceRegisterRequest::Flavor flavor,
+    const std::string& pem_certificate_chain,
+    const std::string& client_id,
+    const std::string& requisition,
+    const std::string& current_state_key) {
+  DCHECK(signing_service_);
+  DCHECK(service_);
+  DCHECK(!is_registered());
+
+  SetClientId(client_id);
+
+  em::CertificateBasedDeviceRegistrationData data;
+  data.set_certificate_type(em::CertificateBasedDeviceRegistrationData::
+      ENTERPRISE_ENROLLMENT_CERTIFICATE);
+  data.set_device_certificate(pem_certificate_chain);
+
+  em::DeviceRegisterRequest* request = data.mutable_device_register_request();
+  if (!client_id.empty())
+    request->set_reregister(true);
+  request->set_type(type);
+  if (!machine_id_.empty())
+    request->set_machine_id(machine_id_);
+  if (!machine_model_.empty())
+    request->set_machine_model(machine_model_);
+  if (!requisition.empty())
+    request->set_requisition(requisition);
+  if (!current_state_key.empty())
+    request->set_server_backed_state_key(current_state_key);
+  request->set_flavor(flavor);
+
+  signing_service_->SignData(data.SerializeAsString(),
+      base::Bind(&CloudPolicyClient::OnRegisterWithCertificateRequestSigned,
+                 weak_ptr_factory_.GetWeakPtr()));
+}
+
+void CloudPolicyClient::OnRegisterWithCertificateRequestSigned(bool success,
+    em::SignedData signed_data) {
+  if (!success) {
+    const em::DeviceManagementResponse response;
+    OnRegisterCompleted(DM_STATUS_CANNOT_SIGN_REQUEST, 0, response);
+    return;
+  }
+
+  policy_fetch_request_job_.reset(
+      service_->CreateJob(
+          DeviceManagementRequestJob::TYPE_CERT_BASED_REGISTRATION,
+          GetRequestContext()));
+  policy_fetch_request_job_->SetClientID(client_id_);
+  em::SignedData* signed_request = policy_fetch_request_job_->GetRequest()->
+      mutable_certificate_based_register_request()->mutable_signed_request();
+  signed_request->set_data(signed_data.data());
+  signed_request->set_signature(signed_data.signature());
+  signed_request->set_extra_data_bytes(signed_data.extra_data_bytes());
+  policy_fetch_request_job_->SetRetryCallback(
+      base::Bind(&CloudPolicyClient::OnRetryRegister,
+                 weak_ptr_factory_.GetWeakPtr()));
+  policy_fetch_request_job_->Start(
+      base::Bind(&CloudPolicyClient::OnRegisterCompleted,
+                 weak_ptr_factory_.GetWeakPtr()));
 }
 
 void CloudPolicyClient::SetInvalidationInfo(int64_t version,
@@ -150,6 +212,8 @@ void CloudPolicyClient::FetchPolicy() {
                           GetRequestContext()));
   policy_fetch_request_job_->SetDMToken(dm_token_);
   policy_fetch_request_job_->SetClientID(client_id_);
+  if (!public_key_version_valid_)
+    policy_fetch_request_job_->SetCritical(true);
 
   em::DeviceManagementRequest* request =
       policy_fetch_request_job_->GetRequest();
@@ -167,8 +231,7 @@ void CloudPolicyClient::FetchPolicy() {
     if (public_key_version_valid_)
       fetch_request->set_public_key_version(public_key_version_);
 
-    if (!verification_key_hash_.empty())
-      fetch_request->set_verification_key_hash(verification_key_hash_);
+    fetch_request->set_verification_key_hash(kPolicyVerificationKeyHash);
 
     // These fields are included only in requests for chrome policy.
     if (IsChromePolicy(type_to_fetch.first)) {
@@ -205,18 +268,15 @@ void CloudPolicyClient::FetchPolicy() {
   // Fire the job.
   policy_fetch_request_job_->Start(
       base::Bind(&CloudPolicyClient::OnPolicyFetchCompleted,
-                 base::Unretained(this)));
+                 weak_ptr_factory_.GetWeakPtr()));
 }
 
 void CloudPolicyClient::FetchRobotAuthCodes(const std::string& auth_token) {
   CHECK(is_registered());
-  DCHECK(!auth_token.empty());
 
   policy_fetch_request_job_.reset(service_->CreateJob(
       DeviceManagementRequestJob::TYPE_API_AUTH_CODE_FETCH,
       GetRequestContext()));
-  // The credentials of a domain user are needed in order to mint a new OAuth2
-  // authorization token for the robot account.
   policy_fetch_request_job_->SetOAuthToken(auth_token);
   policy_fetch_request_job_->SetDMToken(dm_token_);
   policy_fetch_request_job_->SetClientID(client_id_);
@@ -227,10 +287,11 @@ void CloudPolicyClient::FetchRobotAuthCodes(const std::string& auth_token) {
   request->set_oauth2_client_id(
       GaiaUrls::GetInstance()->oauth2_chrome_client_id());
   request->add_auth_scope(GaiaConstants::kAnyApiOAuth2Scope);
+  request->set_device_type(em::DeviceServiceApiAccessRequest::CHROME);
 
   policy_fetch_request_job_->Start(
       base::Bind(&CloudPolicyClient::OnFetchRobotAuthCodesCompleted,
-                 base::Unretained(this)));
+                 weak_ptr_factory_.GetWeakPtr()));
 }
 
 void CloudPolicyClient::Unregister() {
@@ -243,7 +304,7 @@ void CloudPolicyClient::Unregister() {
   policy_fetch_request_job_->GetRequest()->mutable_unregister_request();
   policy_fetch_request_job_->Start(
       base::Bind(&CloudPolicyClient::OnUnregisterCompleted,
-                 base::Unretained(this)));
+                 weak_ptr_factory_.GetWeakPtr()));
 }
 
 void CloudPolicyClient::UploadCertificate(
@@ -262,7 +323,7 @@ void CloudPolicyClient::UploadCertificate(
 
   const DeviceManagementRequestJob::Callback job_callback =
       base::Bind(&CloudPolicyClient::OnCertificateUploadCompleted,
-                 base::Unretained(this), request_job.get(), callback);
+                 weak_ptr_factory_.GetWeakPtr(), request_job.get(), callback);
 
   request_jobs_.push_back(std::move(request_job));
   request_jobs_.back()->Start(job_callback);
@@ -288,7 +349,7 @@ void CloudPolicyClient::UploadDeviceStatus(
 
   const DeviceManagementRequestJob::Callback job_callback =
       base::Bind(&CloudPolicyClient::OnStatusUploadCompleted,
-                 base::Unretained(this), request_job.get(), callback);
+                 weak_ptr_factory_.GetWeakPtr(), request_job.get(), callback);
 
   request_jobs_.push_back(std::move(request_job));
   request_jobs_.back()->Start(job_callback);
@@ -316,7 +377,7 @@ void CloudPolicyClient::FetchRemoteCommands(
 
   const DeviceManagementRequestJob::Callback job_callback =
       base::Bind(&CloudPolicyClient::OnRemoteCommandsFetched,
-                 base::Unretained(this), request_job.get(), callback);
+                 weak_ptr_factory_.GetWeakPtr(), request_job.get(), callback);
 
   request_jobs_.push_back(std::move(request_job));
   request_jobs_.back()->Start(job_callback);
@@ -342,7 +403,7 @@ void CloudPolicyClient::GetDeviceAttributeUpdatePermission(
 
   const DeviceManagementRequestJob::Callback job_callback =
       base::Bind(&CloudPolicyClient::OnDeviceAttributeUpdatePermissionCompleted,
-      base::Unretained(this), request_job.get(), callback);
+      weak_ptr_factory_.GetWeakPtr(), request_job.get(), callback);
 
   request_jobs_.push_back(std::move(request_job));
   request_jobs_.back()->Start(job_callback);
@@ -370,7 +431,7 @@ void CloudPolicyClient::UpdateDeviceAttributes(
 
   const DeviceManagementRequestJob::Callback job_callback =
       base::Bind(&CloudPolicyClient::OnDeviceAttributeUpdated,
-      base::Unretained(this), request_job.get(), callback);
+      weak_ptr_factory_.GetWeakPtr(), request_job.get(), callback);
 
   request_jobs_.push_back(std::move(request_job));
   request_jobs_.back()->Start(job_callback);
@@ -393,8 +454,8 @@ void CloudPolicyClient::UpdateGcmId(
   request->set_gcm_id(gcm_id);
 
   const DeviceManagementRequestJob::Callback job_callback =
-      base::Bind(&CloudPolicyClient::OnGcmIdUpdated, base::Unretained(this),
-                 request_job.get(), callback);
+      base::Bind(&CloudPolicyClient::OnGcmIdUpdated,
+                 weak_ptr_factory_.GetWeakPtr(), request_job.get(), callback);
 
   request_jobs_.push_back(std::move(request_job));
   request_jobs_.back()->Start(job_callback);
@@ -428,9 +489,8 @@ void CloudPolicyClient::SetStateKeysToUpload(
 const em::PolicyFetchResponse* CloudPolicyClient::GetPolicyFor(
     const std::string& policy_type,
     const std::string& settings_entity_id) const {
-  ResponseMap::const_iterator it =
-      responses_.find(std::make_pair(policy_type, settings_entity_id));
-  return it == responses_.end() ? nullptr : it->second;
+  auto it = responses_.find(std::make_pair(policy_type, settings_entity_id));
+  return it == responses_.end() ? nullptr : it->second.get();
 }
 
 scoped_refptr<net::URLRequestContextGetter>
@@ -520,7 +580,7 @@ void CloudPolicyClient::OnPolicyFetchCompleted(
   if (status == DM_STATUS_SUCCESS) {
     const em::DevicePolicyResponse& policy_response =
         response.policy_response();
-    STLDeleteValues(&responses_);
+    responses_.clear();
     for (int i = 0; i < policy_response.response_size(); ++i) {
       const em::PolicyFetchResponse& response = policy_response.response(i);
       em::PolicyData policy_data;
@@ -535,12 +595,12 @@ void CloudPolicyClient::OnPolicyFetchCompleted(
       if (policy_data.has_settings_entity_id())
         entity_id = policy_data.settings_entity_id();
       std::pair<std::string, std::string> key(type, entity_id);
-      if (ContainsKey(responses_, key)) {
+      if (base::ContainsKey(responses_, key)) {
         LOG(WARNING) << "Duplicate PolicyFetchResponse for type: "
             << type << ", entity: " << entity_id << ", ignoring";
         continue;
       }
-      responses_[key] = new em::PolicyFetchResponse(response);
+      responses_[key] = base::MakeUnique<em::PolicyFetchResponse>(response);
     }
     state_keys_to_upload_.clear();
     NotifyPolicyFetched();
@@ -596,7 +656,7 @@ void CloudPolicyClient::OnDeviceAttributeUpdatePermissionCompleted(
     int net_error,
     const em::DeviceManagementResponse& response) {
   // TODO(pbond): remove this LOG once http://crbug.com/586961 is fixed.
-  LOG(WARNING) << "Recieve DeviceAttributeUpdatePermissionResponse status="
+  LOG(WARNING) << "Receive DeviceAttributeUpdatePermissionResponse status="
                << status << " net_error=" << net_error;
   bool success = false;
 
@@ -648,7 +708,7 @@ void CloudPolicyClient::OnDeviceAttributeUpdated(
 
 void CloudPolicyClient::RemoveJob(const DeviceManagementRequestJob* job) {
   for (auto it = request_jobs_.begin(); it != request_jobs_.end(); ++it) {
-    if (*it == job) {
+    if (it->get() == job) {
       request_jobs_.erase(it);
       return;
     }
@@ -708,19 +768,23 @@ void CloudPolicyClient::OnGcmIdUpdated(
 }
 
 void CloudPolicyClient::NotifyPolicyFetched() {
-  FOR_EACH_OBSERVER(Observer, observers_, OnPolicyFetched(this));
+  for (auto& observer : observers_)
+    observer.OnPolicyFetched(this);
 }
 
 void CloudPolicyClient::NotifyRegistrationStateChanged() {
-  FOR_EACH_OBSERVER(Observer, observers_, OnRegistrationStateChanged(this));
+  for (auto& observer : observers_)
+    observer.OnRegistrationStateChanged(this);
 }
 
 void CloudPolicyClient::NotifyRobotAuthCodesFetched() {
-  FOR_EACH_OBSERVER(Observer, observers_, OnRobotAuthCodesFetched(this));
+  for (auto& observer : observers_)
+    observer.OnRobotAuthCodesFetched(this);
 }
 
 void CloudPolicyClient::NotifyClientError() {
-  FOR_EACH_OBSERVER(Observer, observers_, OnClientError(this));
+  for (auto& observer : observers_)
+    observer.OnClientError(this);
 }
 
 }  // namespace policy
