@@ -17,11 +17,13 @@
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
+#include "base/message_loop/message_loop.h"
 #include "components/policy/core/common/cloud/mock_cloud_policy_client.h"
 #include "components/policy/core/common/cloud/mock_device_management_service.h"
+#include "components/policy/core/common/cloud/mock_signing_service.h"
+#include "components/policy/proto/device_management_backend.pb.h"
 #include "net/url_request/url_request_context_getter.h"
 #include "net/url_request/url_request_test_util.h"
-#include "policy/proto/device_management_backend.pb.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -51,6 +53,7 @@ const char kResultPayload[] = "output_payload";
 const char kAssetId[] = "fake-asset-id";
 const char kLocation[] = "fake-location";
 const char kGcmID[] = "fake-gcm-id";
+const char kEnrollmentCertificate[] = "fake-certificate";
 
 const int64_t kAgeOfCommand = 123123123;
 const int64_t kLastCommandId = 123456789;
@@ -93,8 +96,24 @@ class CloudPolicyClientTest : public testing::Test {
     register_request->set_machine_model(kMachineModel);
     register_request->set_flavor(
         em::DeviceRegisterRequest::FLAVOR_USER_REGISTRATION);
-    registration_response_.mutable_register_response()->
-        set_device_management_token(kDMToken);
+
+    em::CertificateBasedDeviceRegistrationData data;
+    data.set_certificate_type(em::CertificateBasedDeviceRegistrationData::
+        ENTERPRISE_ENROLLMENT_CERTIFICATE);
+    data.set_device_certificate(kEnrollmentCertificate);
+
+    em::DeviceRegisterRequest* request = data.mutable_device_register_request();
+    request->set_type(em::DeviceRegisterRequest::DEVICE);
+    request->set_machine_id(kMachineID);
+    request->set_machine_model(kMachineModel);
+    request->set_flavor(
+        em::DeviceRegisterRequest::FLAVOR_ENROLLMENT_ATTESTATION);
+
+    em::CertificateBasedDeviceRegisterRequest* cert_based_register_request =
+        cert_based_registration_request_
+        .mutable_certificate_based_register_request();
+    fake_signing_service_.SignDataSynchronously(data.SerializeAsString(),
+        cert_based_register_request->mutable_signed_request());
 
     em::PolicyFetchRequest* policy_fetch_request =
         policy_request_.mutable_policy_request()->add_request();
@@ -104,6 +123,9 @@ class CloudPolicyClientTest : public testing::Test {
     policy_response_.mutable_policy_response()->add_response()->set_policy_data(
         CreatePolicyData("fake-policy-data"));
 
+    registration_response_.mutable_register_response()->
+        set_device_management_token(kDMToken);
+
     unregistration_request_.mutable_unregister_request();
     unregistration_response_.mutable_unregister_response();
     upload_certificate_request_.mutable_cert_upload_request()->
@@ -118,7 +140,7 @@ class CloudPolicyClientTest : public testing::Test {
     em::RemoteCommandResult* command_result =
         remote_command_request_.mutable_remote_command_request()
             ->add_command_results();
-    command_result->set_unique_id(kLastCommandId);
+    command_result->set_command_id(kLastCommandId);
     command_result->set_result(
         em::RemoteCommandResult_ResultType_RESULT_SUCCESS);
     command_result->set_payload(kResultPayload);
@@ -129,7 +151,7 @@ class CloudPolicyClientTest : public testing::Test {
             ->add_commands();
     command->set_age_of_command(kAgeOfCommand);
     command->set_payload(kPayload);
-    command->set_unique_id(kLastCommandId + 1);
+    command->set_command_id(kLastCommandId + 1);
     command->set_type(em::RemoteCommand_Type_COMMAND_ECHO_TEST);
 
     attribute_update_permission_request_.
@@ -169,10 +191,9 @@ class CloudPolicyClientTest : public testing::Test {
 
     request_context_ =
         new net::TestURLRequestContextGetter(loop_.task_runner());
-    client_.reset(new CloudPolicyClient(kMachineID, kMachineModel,
-                                        kPolicyVerificationKeyHash,
-                                        &service_,
-                                        request_context_));
+    client_ = base::MakeUnique<CloudPolicyClient>(kMachineID, kMachineModel,
+                                                  &service_, request_context_,
+                                                  &fake_signing_service_);
     client_->AddPolicyTypeToFetch(policy_type_, std::string());
     client_->AddObserver(&observer_);
   }
@@ -189,6 +210,18 @@ class CloudPolicyClientTest : public testing::Test {
         .WillOnce(SaveArg<4>(&client_id_));
   }
 
+  void ExpectCertBasedRegistration() {
+    EXPECT_CALL(service_, CreateJob(
+        DeviceManagementRequestJob::TYPE_CERT_BASED_REGISTRATION,
+        request_context_))
+        .WillOnce(service_.SucceedJob(registration_response_));
+    EXPECT_CALL(service_,
+                StartJob(dm_protocol::kValueRequestCertBasedRegister,
+                         std::string(), _, std::string(), _,
+                         MatchProto(cert_based_registration_request_)))
+        .WillOnce(SaveArg<4>(&client_id_));
+  }
+
   void ExpectPolicyFetch(const std::string& dm_token) {
     EXPECT_CALL(service_,
                 CreateJob(DeviceManagementRequestJob::TYPE_POLICY_FETCH,
@@ -295,6 +328,7 @@ class CloudPolicyClientTest : public testing::Test {
 
   // Request protobufs used as expectations for the client requests.
   em::DeviceManagementRequest registration_request_;
+  em::DeviceManagementRequest cert_based_registration_request_;
   em::DeviceManagementRequest policy_request_;
   em::DeviceManagementRequest unregistration_request_;
   em::DeviceManagementRequest upload_certificate_request_;
@@ -321,6 +355,7 @@ class CloudPolicyClientTest : public testing::Test {
   MockDeviceManagementService service_;
   StrictMock<MockCloudPolicyClientObserver> observer_;
   StrictMock<MockStatusCallbackObserver> callback_observer_;
+  FakeSigningService fake_signing_service_;
   std::unique_ptr<CloudPolicyClient> client_;
   // Pointer to the client's request context.
   scoped_refptr<net::URLRequestContextGetter> request_context_;
@@ -364,6 +399,34 @@ TEST_F(CloudPolicyClientTest, RegistrationAndPolicyFetch) {
   CheckPolicyResponse();
 }
 
+TEST_F(CloudPolicyClientTest, RegistrationWithCertificateAndPolicyFetch) {
+  ExpectCertBasedRegistration();
+  fake_signing_service_.set_success(true);
+  EXPECT_CALL(observer_, OnRegistrationStateChanged(_));
+  client_->RegisterWithCertificate(em::DeviceRegisterRequest::DEVICE,
+      em::DeviceRegisterRequest::FLAVOR_ENROLLMENT_ATTESTATION,
+      kEnrollmentCertificate, std::string(), std::string(), std::string());
+  EXPECT_TRUE(client_->is_registered());
+  EXPECT_FALSE(client_->GetPolicyFor(policy_type_, std::string()));
+  EXPECT_EQ(DM_STATUS_SUCCESS, client_->status());
+
+  ExpectPolicyFetch(kDMToken);
+  EXPECT_CALL(observer_, OnPolicyFetched(_));
+  client_->FetchPolicy();
+  EXPECT_EQ(DM_STATUS_SUCCESS, client_->status());
+  CheckPolicyResponse();
+}
+
+TEST_F(CloudPolicyClientTest, RegistrationWithCertificateFailToSignRequest) {
+  fake_signing_service_.set_success(false);
+  EXPECT_CALL(observer_, OnClientError(_));
+  client_->RegisterWithCertificate(em::DeviceRegisterRequest::DEVICE,
+      em::DeviceRegisterRequest::FLAVOR_ENROLLMENT_ATTESTATION,
+      kEnrollmentCertificate, std::string(), std::string(), std::string());
+  EXPECT_FALSE(client_->is_registered());
+  EXPECT_EQ(DM_STATUS_CANNOT_SIGN_REQUEST, client_->status());
+}
+
 TEST_F(CloudPolicyClientTest, RegistrationParametersPassedThrough) {
   registration_request_.mutable_register_request()->set_reregister(true);
   registration_request_.mutable_register_request()->set_requisition(
@@ -850,7 +913,7 @@ TEST_F(CloudPolicyClientTest, FetchRemoteCommands) {
   const std::vector<em::RemoteCommandResult> command_results(
       1, remote_command_request_.remote_command_request().command_results(0));
   client_->FetchRemoteCommands(
-      base::WrapUnique(new RemoteCommandJob::UniqueIDType(kLastCommandId)),
+      base::MakeUnique<RemoteCommandJob::UniqueIDType>(kLastCommandId),
       command_results, callback);
 
   EXPECT_EQ(DM_STATUS_SUCCESS, client_->status());
