@@ -13,8 +13,8 @@
 #include "components/policy/core/common/cloud/mock_cloud_policy_store.h"
 #include "components/policy/core/common/cloud/policy_builder.h"
 #include "components/policy/core/common/policy_switches.h"
+#include "components/policy/policy_constants.h"
 #include "net/url_request/url_request_context_getter.h"
-#include "policy/policy_constants.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -57,11 +57,11 @@ class UserCloudPolicyStoreTest : public testing::Test {
   void SetUp() override {
     ASSERT_TRUE(tmp_dir_.CreateUniqueTempDir());
     store_.reset(new UserCloudPolicyStore(policy_file(), key_file(),
-                                          GetPolicyVerificationKey(),
                                           loop_.task_runner()));
     external_data_manager_.reset(new MockCloudExternalDataManager);
     external_data_manager_->SetPolicyStore(store_.get());
     store_->SetSigninUsername(PolicyBuilder::kFakeUsername);
+    EXPECT_EQ(PolicyBuilder::kFakeUsername, store_->signin_username());
     store_->AddObserver(&observer_);
 
     // Install an initial public key, so that by default the validation of
@@ -88,11 +88,11 @@ class UserCloudPolicyStoreTest : public testing::Test {
   }
 
   base::FilePath policy_file() {
-    return tmp_dir_.path().AppendASCII("policy");
+    return tmp_dir_.GetPath().AppendASCII("policy");
   }
 
   base::FilePath key_file() {
-    return tmp_dir_.path().AppendASCII("policy_key");
+    return tmp_dir_.GetPath().AppendASCII("policy_key");
   }
 
   // Verifies that store_->policy_map() has the appropriate entries.
@@ -229,7 +229,8 @@ TEST_F(UserCloudPolicyStoreTest, ShouldFailToLoadUnsignedPolicy) {
   // Now mimic a new policy coming down - this should result in a new key
   // being installed.
   StorePolicyAndEnsureLoaded(policy_.policy());
-  EXPECT_EQ(policy_.policy().new_public_key(), store_->policy_key());
+  EXPECT_EQ(policy_.policy().new_public_key(),
+            store_->policy_signature_public_key());
   EXPECT_TRUE(store_->policy()->has_public_key_version());
   EXPECT_TRUE(base::PathExists(key_file()));
 }
@@ -285,7 +286,7 @@ TEST_F(UserCloudPolicyStoreTest, StoreRotatedKey) {
   StorePolicyAndEnsureLoaded(policy_.policy());
   EXPECT_FALSE(policy_.policy().has_new_public_key_signature());
   std::string original_policy_key = policy_.policy().new_public_key();
-  EXPECT_EQ(original_policy_key, store_->policy_key());
+  EXPECT_EQ(original_policy_key, store_->policy_signature_public_key());
 
   // Now do key rotation.
   policy_.SetDefaultSigningKey();
@@ -294,7 +295,8 @@ TEST_F(UserCloudPolicyStoreTest, StoreRotatedKey) {
   EXPECT_TRUE(policy_.policy().has_new_public_key_signature());
   EXPECT_NE(original_policy_key, policy_.policy().new_public_key());
   StorePolicyAndEnsureLoaded(policy_.policy());
-  EXPECT_EQ(policy_.policy().new_public_key(), store_->policy_key());
+  EXPECT_EQ(policy_.policy().new_public_key(),
+            store_->policy_signature_public_key());
 }
 
 TEST_F(UserCloudPolicyStoreTest, ProvisionKeyTwice) {
@@ -348,12 +350,11 @@ TEST_F(UserCloudPolicyStoreTest, StoreThenLoad) {
   // Store a simple policy and make sure it can be read back in.
   // policy.
   StorePolicyAndEnsureLoaded(policy_.policy());
-  EXPECT_FALSE(store_->policy_key().empty());
+  EXPECT_FALSE(store_->policy_signature_public_key().empty());
 
   // Now, make sure the policy can be read back in from a second store.
-  std::unique_ptr<UserCloudPolicyStore> store2(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store2(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store2->SetSigninUsername(PolicyBuilder::kFakeUsername);
   store2->AddObserver(&observer_);
   EXPECT_CALL(observer_, OnStoreLoaded(store2.get()));
@@ -367,7 +368,8 @@ TEST_F(UserCloudPolicyStoreTest, StoreThenLoad) {
   EXPECT_EQ(CloudPolicyStore::STATUS_OK, store2->status());
   store2->RemoveObserver(&observer_);
   // Make sure that we properly resurrected the keys.
-  EXPECT_EQ(store2->policy_key(), store_->policy_key());
+  EXPECT_EQ(store2->policy_signature_public_key(),
+            store_->policy_signature_public_key());
 }
 
 TEST_F(UserCloudPolicyStoreTest, StoreThenLoadImmediately) {
@@ -376,9 +378,8 @@ TEST_F(UserCloudPolicyStoreTest, StoreThenLoadImmediately) {
   StorePolicyAndEnsureLoaded(policy_.policy());
 
   // Now, make sure the policy can be read back in from a second store.
-  std::unique_ptr<UserCloudPolicyStore> store2(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store2(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store2->SetSigninUsername(PolicyBuilder::kFakeUsername);
   store2->AddObserver(&observer_);
   EXPECT_CALL(observer_, OnStoreLoaded(store2.get()));
@@ -421,9 +422,8 @@ TEST_F(UserCloudPolicyStoreTest, LoadValidationError) {
 
   // Sign out, and sign back in as a different user, and try to load the profile
   // data (should fail due to mismatched username).
-  std::unique_ptr<UserCloudPolicyStore> store2(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store2(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store2->SetSigninUsername("foobar@foobar.com");
   store2->AddObserver(&observer_);
   ExpectError(store2.get(), CloudPolicyStore::STATUS_VALIDATION_ERROR);
@@ -435,9 +435,8 @@ TEST_F(UserCloudPolicyStoreTest, LoadValidationError) {
 
   // Sign out - we should be able to load the policy (don't check usernames
   // when signed out).
-  std::unique_ptr<UserCloudPolicyStore> store3(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store3(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store3->AddObserver(&observer_);
   EXPECT_CALL(observer_, OnStoreLoaded(store3.get()));
   store3->Load();
@@ -447,9 +446,8 @@ TEST_F(UserCloudPolicyStoreTest, LoadValidationError) {
   store3->RemoveObserver(&observer_);
 
   // Now start a signin as a different user - this should fail validation.
-  std::unique_ptr<UserCloudPolicyStore> store4(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store4(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store4->SetSigninUsername("foobar@foobar.com");
   store4->AddObserver(&observer_);
   ExpectError(store4.get(), CloudPolicyStore::STATUS_VALIDATION_ERROR);
@@ -476,9 +474,8 @@ TEST_F(UserCloudPolicyStoreTest, KeyRotation) {
 
   // Now load this in a new store - this should trigger key rotation. The keys
   // will still verify using the existing verification key.
-  std::unique_ptr<UserCloudPolicyStore> store2(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store2(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store2->SetSigninUsername(PolicyBuilder::kFakeUsername);
   store2->AddObserver(&observer_);
   EXPECT_CALL(observer_, OnStoreLoaded(store2.get()));
@@ -503,9 +500,8 @@ TEST_F(UserCloudPolicyStoreTest, InvalidCachedVerificationSignature) {
 
   // Now load this in a new store - this should cause a validation error because
   // the key won't verify.
-  std::unique_ptr<UserCloudPolicyStore> store2(new UserCloudPolicyStore(
-      policy_file(), key_file(), GetPolicyVerificationKey(),
-      loop_.task_runner()));
+  std::unique_ptr<UserCloudPolicyStore> store2(
+      new UserCloudPolicyStore(policy_file(), key_file(), loop_.task_runner()));
   store2->SetSigninUsername(PolicyBuilder::kFakeUsername);
   store2->AddObserver(&observer_);
   ExpectError(store2.get(), CloudPolicyStore::STATUS_VALIDATION_ERROR);
