@@ -26,9 +26,13 @@
 
 namespace {
 
+// String used for source parameter in GAIA cookie manager calls.
+const char kSource[] = "ChromiumAccountReconcilor";
+
 class AccountEqualToFunc {
  public:
-  AccountEqualToFunc(const gaia::ListedAccount& account) : account_(account) {}
+  explicit AccountEqualToFunc(const gaia::ListedAccount& account)
+      : account_(account) {}
   bool operator()(const gaia::ListedAccount& other) const;
 
  private:
@@ -242,14 +246,14 @@ void AccountReconcilor::PerformMergeAction(const std::string& account_id) {
     return;
   }
   VLOG(1) << "AccountReconcilor::PerformMergeAction: " << account_id;
-  cookie_manager_service_->AddAccountToCookie(account_id);
+  cookie_manager_service_->AddAccountToCookie(account_id, kSource);
 }
 
 void AccountReconcilor::PerformLogoutAllAccountsAction() {
   if (!switches::IsEnableAccountConsistency())
     return;
   VLOG(1) << "AccountReconcilor::PerformLogoutAllAccountsAction";
-  cookie_manager_service_->LogOutAllAccounts();
+  cookie_manager_service_->LogOutAllAccounts(kSource);
 }
 
 void AccountReconcilor::StartReconcile() {
@@ -286,7 +290,8 @@ void AccountReconcilor::StartReconcile() {
 
   // Rely on the GCMS to manage calls to and responses from ListAccounts.
   if (cookie_manager_service_->ListAccounts(&gaia_accounts_,
-                                            &signed_out_accounts)) {
+                                            &signed_out_accounts,
+                                            kSource)) {
     OnGaiaAccountsInCookieUpdated(
         gaia_accounts_,
         signed_out_accounts,
@@ -366,7 +371,7 @@ void AccountReconcilor::OnNewProfileManagementFlagChanged(
 void AccountReconcilor::OnReceivedManageAccountsResponse(
     signin::GAIAServiceType service_type) {
   if (service_type == signin::GAIA_SERVICE_TYPE_ADDSESSION) {
-    cookie_manager_service_->TriggerListAccounts();
+    cookie_manager_service_->TriggerListAccounts(kSource);
   }
 }
 
