@@ -18,19 +18,24 @@
 #include "components/test_runner/layout_test_runtime_flags.h"
 #include "components/test_runner/test_runner_export.h"
 #include "components/test_runner/web_test_runner.h"
+#include "media/midi/midi_service.mojom.h"
+#include "third_party/WebKit/public/platform/WebEffectiveConnectionType.h"
 #include "third_party/WebKit/public/platform/WebImage.h"
 #include "v8/include/v8.h"
 
 class GURL;
 class SkBitmap;
 
+namespace base {
+class NullableString16;
+}
+
 namespace blink {
 class WebContentSettingsClient;
 class WebFrame;
 class WebLocalFrame;
 class WebString;
 class WebView;
-class WebWidget;
 }
 
 namespace gin {
@@ -54,7 +59,7 @@ class WebTestDelegate;
 // 1. It implements |testRunner| javascript bindings for "global" / "ambient".
 //    Examples:
 //    - testRunner.dumpAsText (test flag affecting test behavior)
-//    - testRunner.setAllowDisplayOfInsecureContent (test flag affecting product
+//    - testRunner.setAllowRunningOfInsecureContent (test flag affecting product
 //      behavior)
 //    - testRunner.setTextSubpixelPositioning (directly interacts with product).
 //    Note that "per-view" (non-"global") bindings are handled by
@@ -99,8 +104,8 @@ class TestRunner : public WebTestRunner {
   void SetFocus(blink::WebView* web_view, bool focus) override;
 
   // Methods used by WebViewTestClient and WebFrameTestClient.
-  void OnAnimationScheduled(blink::WebWidget* widget);
-  void OnAnimationBegun(blink::WebWidget* widget);
+  void OnNavigationBegin(blink::WebFrame* frame);
+  void OnNavigationEnd() { will_navigate_ = false; }
   std::string GetAcceptLanguages() const;
   bool shouldStayOnPageAfterHandlingBeforeUnload() const;
   MockScreenOrientationClient* getMockScreenOrientationClient();
@@ -112,6 +117,7 @@ class TestRunner : public WebTestRunner {
   void ShowDevTools(const std::string& settings,
                     const std::string& frontend_url);
   void ClearDevToolsLocalStorage();
+  void SetV8CacheDisabled(bool);
   void setShouldDumpAsText(bool);
   void setShouldDumpAsMarkup(bool);
   void setCustomTextOutput(const std::string& text);
@@ -149,23 +155,28 @@ class TestRunner : public WebTestRunner {
   // pending load requests in WorkQueue).
   bool tryToClearTopLoadingFrame(blink::WebFrame*);
 
+  blink::WebFrame* mainFrame() const;
   blink::WebFrame* topLoadingFrame() const;
   void policyDelegateDone();
   bool policyDelegateEnabled() const;
   bool policyDelegateIsPermissive() const;
   bool policyDelegateShouldNotifyDone() const;
-  bool shouldDumpResourcePriorities() const;
   void setToolTipText(const blink::WebString&);
   void setDragImage(const blink::WebImage& drag_image);
   bool shouldDumpNavigationPolicy() const;
 
-  bool midiAccessorResult();
+  midi::mojom::Result midiAccessorResult();
 
   // Methods used by MockColorChooser:
   void DidOpenChooser();
   void DidCloseChooser();
 
   bool ShouldDumpConsoleMessages() const;
+  bool ShouldDumpJavaScriptDialogs() const;
+
+  blink::WebEffectiveConnectionType effective_connection_type() const {
+    return effective_connection_type_;
+  }
 
   // A single item in the work queue.
   class WorkItem {
@@ -317,10 +328,6 @@ class TestRunner : public WebTestRunner {
   // Enable or disable plugins.
   void SetPluginsEnabled(bool enabled);
 
-  // Returns |true| if an animation has been scheduled in one or more WebViews
-  // participating in the layout test.
-  bool GetAnimationScheduled() const;
-
   ///////////////////////////////////////////////////////////////////////////
   // Methods that modify the state of TestRunner
 
@@ -404,7 +411,6 @@ class TestRunner : public WebTestRunner {
   void SetScriptsAllowed(bool allowed);
   void SetStorageAllowed(bool allowed);
   void SetPluginsAllowed(bool allowed);
-  void SetAllowDisplayOfInsecureContent(bool allowed);
   void SetAllowRunningOfInsecureContent(bool allowed);
   void SetAutoplayAllowed(bool allowed);
   void DumpPermissionClientCallbacks();
@@ -442,11 +448,6 @@ class TestRunner : public WebTestRunner {
   // Causes WillSendRequest to clear certain headers.
   void SetWillSendRequestClearHeader(const std::string& header);
 
-  // This function sets a flag that tells the test_shell to dump a descriptive
-  // line for each resource load's priority and any time that priority
-  // changes. It takes no arguments, and ignores any that may be present.
-  void DumpResourceRequestPriorities();
-
   // Sets a flag to enable the mock theme.
   void SetUseMockTheme(bool use);
 
@@ -459,7 +460,7 @@ class TestRunner : public WebTestRunner {
   // results will be the drag image instead of a snapshot of the page.
   void DumpDragImage();
 
-  // Sets a flag that tells the WebTestProxy to dump the default navigation
+  // Sets a flag that tells the WebViewTestProxy to dump the default navigation
   // policy passed to the decidePolicyForNavigation callback.
   void DumpNavigationPolicy();
 
@@ -467,8 +468,21 @@ class TestRunner : public WebTestRunner {
   // to test output.
   void SetDumpConsoleMessages(bool value);
 
+  // Controls whether JavaScript dialogs such as alert() are dumped to test
+  // output.
+  void SetDumpJavaScriptDialogs(bool value);
+
+  // Overrides the NetworkQualityEstimator's estimated network type. If |type|
+  // is TypeUnknown the NQE's value is used. Be sure to call this with
+  // TypeUnknown at the end of your test if you use this.
+  void SetEffectiveConnectionType(
+      blink::WebEffectiveConnectionType connection_type);
+
+  // Controls whether the mock spell checker is enabled.
+  void SetMockSpellCheckerEnabled(bool enabled);
+
   ///////////////////////////////////////////////////////////////////////////
-  // Methods interacting with the WebTestProxy
+  // Methods interacting with the WebViewTestProxy
 
   ///////////////////////////////////////////////////////////////////////////
   // Methods forwarding to the WebTestDelegate
@@ -493,8 +507,10 @@ class TestRunner : public WebTestRunner {
   // Sets the default quota for all origins
   void SetDatabaseQuota(int quota);
 
-  // Changes the cookie policy from the default to allow all cookies.
-  void SetAlwaysAcceptCookies(bool accept);
+  // Sets the cookie policy to:
+  // - allow all cookies when |block| is false
+  // - block only third-party cookies when |block| is true
+  void SetBlockThirdPartyCookies(bool block);
 
   // Converts a URL starting with file:///tmp/ to the local mapping.
   std::string PathToLocalResource(const std::string& path);
@@ -506,19 +522,20 @@ class TestRunner : public WebTestRunner {
                      const GURL& origin,
                      const GURL& embedding_origin);
 
-  // Resolve the beforeinstallprompt event with the matching request id.
-  void ResolveBeforeInstallPromptPromise(int request_id,
-                                         const std::string& platform);
+  // Resolve the in-flight beforeinstallprompt event.
+  void ResolveBeforeInstallPromptPromise(const std::string& platform);
 
   // Calls setlocale(LC_ALL, ...) for a specified locale.
   // Resets between tests.
   void SetPOSIXLocale(const std::string& locale);
 
   // MIDI function to control permission handling.
-  void SetMIDIAccessorResult(bool result);
+  void SetMIDIAccessorResult(midi::mojom::Result result);
 
   // Simulates a click on a Web Notification.
-  void SimulateWebNotificationClick(const std::string& title, int action_index);
+  void SimulateWebNotificationClick(const std::string& title,
+                                    int action_index,
+                                    const base::NullableString16& reply);
 
   // Simulates closing a Web Notification.
   void SimulateWebNotificationClose(const std::string& title, bool by_user);
@@ -531,11 +548,12 @@ class TestRunner : public WebTestRunner {
 
   // Credential Manager mock functions
   // TODO(mkwst): Support FederatedCredential.
-  void AddMockCredentialManagerResponse(const std::string& id,
+  void SetMockCredentialManagerResponse(const std::string& id,
                                         const std::string& name,
                                         const std::string& avatar,
                                         const std::string& password);
-  void AddMockCredentialManagerError(const std::string& error);
+  void ClearMockCredentialManagerResponse();
+  void SetMockCredentialManagerError(const std::string& error);
 
   // Takes care of notifying the delegate after a change to layout test runtime
   // flags.
@@ -589,8 +607,8 @@ class TestRunner : public WebTestRunner {
   // a series of 1px-wide, view-tall paints across the width of the view.
   bool sweep_horizontally_;
 
-  // If false, MockWebMIDIAccessor fails on startSession() for testing.
-  bool midi_accessor_result_;
+  // startSession() result of MockWebMIDIAccessor for testing.
+  midi::mojom::Result midi_accessor_result_;
 
   bool has_custom_text_output_;
   std::string custom_text_output_;
@@ -612,6 +630,12 @@ class TestRunner : public WebTestRunner {
 
   bool use_mock_theme_;
 
+  // This is true in the period between the start of a navigation and when the
+  // provisional load for that navigation is started. Note that when
+  // browser-side navigation is enabled there is an arbitrary gap between these
+  // two events.
+  bool will_navigate_;
+
   std::unique_ptr<MockCredentialManagerClient> credential_manager_client_;
   std::unique_ptr<MockScreenOrientationClient> mock_screen_orientation_client_;
   std::unique_ptr<MockWebSpeechRecognizer> speech_recognizer_;
@@ -629,11 +653,15 @@ class TestRunner : public WebTestRunner {
   // is ok, because this is taken care of in WebTestDelegate::SetFocus).
   blink::WebView* previously_focused_view_;
 
-  std::set<blink::WebWidget*> widgets_with_scheduled_animations_;
-
   // True if we run a test in LayoutTests/imported/{csswg-test,wpt}/.
   bool is_web_platform_tests_mode_;
 
+  // An effective connection type settable by layout tests.
+  blink::WebEffectiveConnectionType effective_connection_type_;
+
+  // Forces v8 compilation cache to be disabled (used for inspector tests).
+  bool disable_v8_cache_ = false;
+
   base::WeakPtrFactory<TestRunner> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(TestRunner);
