@@ -14,7 +14,6 @@
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "build/build_config.h"
-#include "components/test_runner/app_banner_client.h"
 #include "components/test_runner/layout_and_paint_async_then.h"
 #include "components/test_runner/layout_dump.h"
 #include "components/test_runner/mock_content_settings_client.h"
@@ -28,9 +27,8 @@
 #include "components/test_runner/test_interfaces.h"
 #include "components/test_runner/test_preferences.h"
 #include "components/test_runner/test_runner.h"
-#include "components/test_runner/web_task.h"
 #include "components/test_runner/web_test_delegate.h"
-#include "components/test_runner/web_test_proxy.h"
+#include "components/test_runner/web_view_test_proxy.h"
 #include "gin/arguments.h"
 #include "gin/array_buffer.h"
 #include "gin/handle.h"
@@ -50,6 +48,7 @@
 #include "third_party/WebKit/public/web/WebDocument.h"
 #include "third_party/WebKit/public/web/WebFindOptions.h"
 #include "third_party/WebKit/public/web/WebFrame.h"
+#include "third_party/WebKit/public/web/WebFrameWidget.h"
 #include "third_party/WebKit/public/web/WebInputElement.h"
 #include "third_party/WebKit/public/web/WebKit.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
@@ -77,15 +76,15 @@ using namespace blink;
 namespace test_runner {
 
 TestRunnerForSpecificView::TestRunnerForSpecificView(
-    WebTestProxyBase* web_test_proxy_base)
-    : web_test_proxy_base_(web_test_proxy_base), weak_factory_(this) {
+    WebViewTestProxyBase* web_view_test_proxy_base)
+    : web_view_test_proxy_base_(web_view_test_proxy_base), weak_factory_(this) {
   Reset();
 }
 
 TestRunnerForSpecificView::~TestRunnerForSpecificView() {}
 
 void TestRunnerForSpecificView::Install(blink::WebLocalFrame* frame) {
-  web_test_proxy_base_->test_interfaces()->GetTestRunner()->Install(
+  web_view_test_proxy_base_->test_interfaces()->GetTestRunner()->Install(
       frame, weak_factory_.GetWeakPtr());
 }
 
@@ -143,12 +142,12 @@ bool TestRunnerForSpecificView::isPointerLocked() {
 }
 
 void TestRunnerForSpecificView::PostTask(const base::Closure& callback) {
-  delegate()->PostTask(new WebCallbackTask(callback));
+  delegate()->PostTask(callback);
 }
 
 void TestRunnerForSpecificView::PostDelayedTask(long long delay,
                                                 const base::Closure& callback) {
-  delegate()->PostDelayedTask(new WebCallbackTask(callback), delay);
+  delegate()->PostDelayedTask(callback, delay);
 }
 
 void TestRunnerForSpecificView::PostV8Callback(
@@ -234,10 +233,13 @@ void TestRunnerForSpecificView::CapturePixelsAsyncThen(
   v8::UniquePersistent<v8::Function> persistent_callback(
       blink::mainThreadIsolate(), callback);
 
-  web_test_proxy_base_->test_interfaces()->GetTestRunner()->DumpPixelsAsync(
-      web_view(), base::Bind(&TestRunnerForSpecificView::CapturePixelsCallback,
-                             weak_factory_.GetWeakPtr(),
-                             base::Passed(std::move(persistent_callback))));
+  web_view_test_proxy_base_->test_interfaces()
+      ->GetTestRunner()
+      ->DumpPixelsAsync(
+          web_view(),
+          base::Bind(&TestRunnerForSpecificView::CapturePixelsCallback,
+                     weak_factory_.GetWeakPtr(),
+                     base::Passed(std::move(persistent_callback))));
 }
 
 void TestRunnerForSpecificView::CapturePixelsCallback(
@@ -392,11 +394,10 @@ void TestRunnerForSpecificView::SetColorProfile(
 }
 
 void TestRunnerForSpecificView::DispatchBeforeInstallPromptEvent(
-    int request_id,
     const std::vector<std::string>& event_platforms,
     v8::Local<v8::Function> callback) {
   delegate()->DispatchBeforeInstallPromptEvent(
-      request_id, event_platforms,
+      event_platforms,
       base::Bind(
           &TestRunnerForSpecificView::DispatchBeforeInstallPromptCallback,
           weak_factory_.GetWeakPtr(),
@@ -454,10 +455,12 @@ bool TestRunnerForSpecificView::IsCommandEnabled(const std::string& command) {
 }
 
 bool TestRunnerForSpecificView::HasCustomPageSizeStyle(int page_index) {
+  // TODO(dcheng): This class has many implicit assumptions that the frames it
+  // operates on are always local.
   WebFrame* frame = web_view()->mainFrame();
-  if (!frame)
+  if (!frame || frame->isWebRemoteFrame())
     return false;
-  return frame->hasCustomPageSizeStyle(page_index);
+  return frame->toWebLocalFrame()->hasCustomPageSizeStyle(page_index);
 }
 
 void TestRunnerForSpecificView::ForceRedSelectionColors() {
@@ -524,8 +527,8 @@ void TestRunnerForSpecificView::ForceNextDrawingBufferCreationToFail() {
 }
 
 void TestRunnerForSpecificView::SetWindowIsKey(bool value) {
-  web_test_proxy_base_->test_interfaces()->GetTestRunner()->SetFocus(web_view(),
-                                                                     value);
+  web_view_test_proxy_base_->test_interfaces()->GetTestRunner()->SetFocus(
+      web_view(), value);
 }
 
 void TestRunnerForSpecificView::DidAcquirePointerLock() {
@@ -598,7 +601,7 @@ TestRunnerForSpecificView::EvaluateScriptInIsolatedWorldAndReturnValue(
   // This relies on the iframe focusing itself when it loads. This is a bit
   // sketchy, but it seems to be what other tests do.
   web_view()->focusedFrame()->executeScriptInIsolatedWorld(world_id, &source, 1,
-                                                           1, &values);
+                                                           &values);
   // Since only one script was added, only one result is expected
   if (values.size() == 1 && !values[0].IsEmpty())
     return values[0];
@@ -609,7 +612,7 @@ void TestRunnerForSpecificView::EvaluateScriptInIsolatedWorld(
     int world_id,
     const std::string& script) {
   WebScriptSource source(WebString::fromUTF8(script));
-  web_view()->focusedFrame()->executeScriptInIsolatedWorld(world_id, &source, 1,
+  web_view()->focusedFrame()->executeScriptInIsolatedWorld(world_id, &source,
                                                            1);
 }
 
@@ -671,7 +674,11 @@ bool TestRunnerForSpecificView::FindString(
 }
 
 std::string TestRunnerForSpecificView::SelectionAsMarkup() {
-  return web_view()->mainFrame()->selectionAsMarkup().utf8();
+  if (!web_view()->mainFrame()->toWebLocalFrame()) {
+    CHECK(false) << "This function cannot be called if the main frame is not a "
+                    "local frame.";
+  }
+  return web_view()->mainFrame()->toWebLocalFrame()->selectionAsMarkup().utf8();
 }
 
 void TestRunnerForSpecificView::SetViewSourceForFrame(const std::string& name,
@@ -683,11 +690,11 @@ void TestRunnerForSpecificView::SetViewSourceForFrame(const std::string& name,
 }
 
 blink::WebView* TestRunnerForSpecificView::web_view() {
-  return web_test_proxy_base_->web_view();
+  return web_view_test_proxy_base_->web_view();
 }
 
 WebTestDelegate* TestRunnerForSpecificView::delegate() {
-  return web_test_proxy_base_->delegate();
+  return web_view_test_proxy_base_->delegate();
 }
 
 }  // namespace test_runner
