@@ -4,10 +4,17 @@
 
 #include "components/variations/variations_url_constants.h"
 
+#include "build/build_config.h"
+
 namespace variations {
 
 // Default server of Variations seed info.
+#if defined(OS_ANDROID)
+const char kDefaultServerUrl[] =
+    "https://clientservices.googleapis.com/chrome-variations/seed";
+#else
 const char kDefaultServerUrl[] =
     "https://clients4.google.com/chrome-variations/seed";
+#endif
 
 }  // namespace variations
