@@ -9,6 +9,7 @@
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
 #include "content/browser/child_process_security_policy_impl.h"
 #include "content/browser/loader/resource_dispatcher_host_impl.h"
@@ -17,6 +18,7 @@
 #include "content/public/browser/resource_dispatcher_host_delegate.h"
 #include "content/public/browser/resource_throttle.h"
 #include "content/public/browser/web_contents.h"
+#include "content/public/common/content_switches.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/content_browser_test.h"
 #include "content/public/test/content_browser_test_utils.h"
@@ -43,11 +45,12 @@ class TrackingResourceDispatcherHostDelegate
   TrackingResourceDispatcherHostDelegate() : throttle_created_(false) {
   }
 
-  void RequestBeginning(net::URLRequest* request,
-                        ResourceContext* resource_context,
-                        AppCacheService* appcache_service,
-                        ResourceType resource_type,
-                        ScopedVector<ResourceThrottle>* throttles) override {
+  void RequestBeginning(
+      net::URLRequest* request,
+      ResourceContext* resource_context,
+      AppCacheService* appcache_service,
+      ResourceType resource_type,
+      std::vector<std::unique_ptr<ResourceThrottle>>* throttles) override {
     CHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
     ShellResourceDispatcherHostDelegate::RequestBeginning(
         request, resource_context, appcache_service, resource_type, throttles);
@@ -55,7 +58,7 @@ class TrackingResourceDispatcherHostDelegate
     ASSERT_FALSE(throttle_created_);
     // If this is a request for the tracked URL, add a throttle to track it.
     if (request->url() == tracked_url_)
-      throttles->push_back(new TrackingThrottle(request, this));
+      throttles->push_back(base::MakeUnique<TrackingThrottle>(request, this));
   }
 
   // Starts tracking a URL.  The request for previously tracked URL, if any,
@@ -100,6 +103,8 @@ class TrackingResourceDispatcherHostDelegate
       // If the request is deleted without being cancelled, its status will
       // indicate it succeeded, so have to check if the request is still pending
       // as well.
+      // TODO(maksims): Stop using request_->status() here, because it is
+      // going to be deprecated.
       tracker_->OnTrackedRequestDestroyed(
           !request_->is_pending() && request_->status().is_success());
     }
@@ -154,7 +159,7 @@ class NoTransferRequestDelegate : public WebContentsDelegate {
  public:
   NoTransferRequestDelegate() {}
 
-  bool ShouldTransferNavigation() override {
+  bool ShouldTransferNavigation(bool is_main_frame_navigation) override {
     // Intentionally cancel the transfer.
     return false;
   }
@@ -163,7 +168,14 @@ class NoTransferRequestDelegate : public WebContentsDelegate {
   DISALLOW_COPY_AND_ASSIGN(NoTransferRequestDelegate);
 };
 
-class CrossSiteTransferTest : public ContentBrowserTest {
+enum class TestParameter {
+  LOADING_WITHOUT_MOJO,
+  LOADING_WITH_MOJO,
+};
+
+class CrossSiteTransferTest
+    : public ContentBrowserTest,
+      public ::testing::WithParamInterface<TestParameter> {
  public:
   CrossSiteTransferTest() : old_delegate_(nullptr) {}
 
@@ -174,8 +186,8 @@ class CrossSiteTransferTest : public ContentBrowserTest {
         base::Bind(&CrossSiteTransferTest::InjectResourceDispatcherHostDelegate,
                    base::Unretained(this)));
     host_resolver()->AddRule("*", "127.0.0.1");
-    ASSERT_TRUE(embedded_test_server()->Start());
     content::SetupCrossSiteRedirector(embedded_test_server());
+    ASSERT_TRUE(embedded_test_server()->Start());
   }
 
   void TearDownOnMainThread() override {
@@ -205,6 +217,10 @@ class CrossSiteTransferTest : public ContentBrowserTest {
 
   void SetUpCommandLine(base::CommandLine* command_line) override {
     IsolateAllSitesForTesting(command_line);
+    if (GetParam() == TestParameter::LOADING_WITH_MOJO) {
+      command_line->AppendSwitchASCII(switches::kEnableBlinkFeatures,
+                                      "LoadingWithMojo");
+    }
   }
 
   void InjectResourceDispatcherHostDelegate() {
@@ -242,7 +258,7 @@ class CrossSiteTransferTest : public ContentBrowserTest {
 #endif
 // Tests that the |should_replace_current_entry| flag persists correctly across
 // request transfers that began with a cross-process navigation.
-IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest,
+IN_PROC_BROWSER_TEST_P(CrossSiteTransferTest,
                        MAYBE_ReplaceEntryCrossProcessThenTransfer) {
   const NavigationController& controller =
       shell()->web_contents()->GetController();
@@ -301,7 +317,7 @@ IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest,
 // request transfers that began with a content-initiated in-process
 // navigation. This test is the same as the test above, except transfering from
 // in-process.
-IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest,
+IN_PROC_BROWSER_TEST_P(CrossSiteTransferTest,
                        ReplaceEntryInProcessThenTransfer) {
   const NavigationController& controller =
       shell()->web_contents()->GetController();
@@ -342,7 +358,7 @@ IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest,
 
 // Tests that the |should_replace_current_entry| flag persists correctly across
 // request transfers that cross processes twice from renderer policy.
-IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest,
+IN_PROC_BROWSER_TEST_P(CrossSiteTransferTest,
                        MAYBE_ReplaceEntryCrossProcessTwice) {
   const NavigationController& controller =
       shell()->web_contents()->GetController();
@@ -387,7 +403,7 @@ IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest,
 
 // Tests that the request is destroyed when a cross process navigation is
 // cancelled.
-IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest, NoLeakOnCrossSiteCancel) {
+IN_PROC_BROWSER_TEST_P(CrossSiteTransferTest, NoLeakOnCrossSiteCancel) {
   const NavigationController& controller =
       shell()->web_contents()->GetController();
 
@@ -431,7 +447,7 @@ IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest, NoLeakOnCrossSiteCancel) {
 // files encapsulated by HTTP POST body that is forwarded to the new renderer.
 // Invalid handling of this scenario has been suspected as the cause of at least
 // some of the renderer kills tracked in https://crbug.com/613260.
-IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest, PostWithFileData) {
+IN_PROC_BROWSER_TEST_P(CrossSiteTransferTest, PostWithFileData) {
   // Navigate to the page with form that posts via 307 redirection to
   // |redirect_target_url| (cross-site from |form_url|).  Using 307 (rather than
   // 302) redirection is important to preserve the HTTP method and POST body.
@@ -441,11 +457,12 @@ IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest, PostWithFileData) {
   EXPECT_TRUE(NavigateToURL(shell(), form_url));
 
   // Prepare a file to upload.
+  base::ThreadRestrictions::ScopedAllowIO allow_io_for_temp_dir;
   base::ScopedTempDir temp_dir;
   base::FilePath file_path;
   std::string file_content("test-file-content");
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
-  ASSERT_TRUE(base::CreateTemporaryFileInDir(temp_dir.path(), &file_path));
+  ASSERT_TRUE(base::CreateTemporaryFileInDir(temp_dir.GetPath(), &file_path));
   ASSERT_LT(
       0, base::WriteFile(file_path, file_content.data(), file_content.size()));
 
@@ -494,4 +511,9 @@ IN_PROC_BROWSER_TEST_F(CrossSiteTransferTest, PostWithFileData) {
               ::testing::HasSubstr("form-data; name=\"file\""));
 }
 
+INSTANTIATE_TEST_CASE_P(CrossSiteTransferTest,
+                        CrossSiteTransferTest,
+                        ::testing::Values(TestParameter::LOADING_WITHOUT_MOJO,
+                                          TestParameter::LOADING_WITH_MOJO));
+
 }  // namespace content
