@@ -20,10 +20,11 @@ class SharedWorkerDevToolsAgentHost : public WorkerDevToolsAgentHost {
                                 const SharedWorkerInstance& shared_worker);
 
   // DevToolsAgentHost override.
-  Type GetType() override;
+  std::string GetType() override;
   std::string GetTitle() override;
   GURL GetURL() override;
   bool Activate() override;
+  void Reload() override;
   bool Close() override;
 
   bool Matches(const SharedWorkerInstance& other);
