@@ -11,7 +11,7 @@
 
 #include "base/files/file_util.h"
 #include "base/logging.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
@@ -33,7 +33,12 @@ const char session_storage_uma_name[] = "SessionStorageDatabase.Open";
 enum SessionStorageUMA {
   SESSION_STORAGE_UMA_SUCCESS,
   SESSION_STORAGE_UMA_RECREATED,
-  SESSION_STORAGE_UMA_FAIL,
+  SESSION_STORAGE_UMA_RECREATE_FAIL,  // Deprecated in M56 (issue 183679)
+  SESSION_STORAGE_UMA_RECREATE_NOT_FOUND,
+  SESSION_STORAGE_UMA_RECREATE_NOT_SUPPORTED,
+  SESSION_STORAGE_UMA_RECREATE_CORRUPTION,
+  SESSION_STORAGE_UMA_RECREATE_INVALID_ARGUMENT,
+  SESSION_STORAGE_UMA_RECREATE_IO_ERROR,
   SESSION_STORAGE_UMA_MAX
 };
 
@@ -298,7 +303,8 @@ bool SessionStorageDatabase::ReadNamespacesAndOrigins(
   std::string current_namespace_id;
   for (it->Next(); it->Valid(); it->Next()) {
     std::string key = it->key().ToString();
-    if (key.find(namespace_prefix) != 0) {
+    if (!base::StartsWith(key, namespace_prefix,
+                          base::CompareCase::SENSITIVE)) {
       // Iterated past the "namespace-" keys.
       break;
     }
@@ -345,7 +351,7 @@ void SessionStorageDatabase::OnMemoryDump(
   bool res = base::StringToUint64(db_memory_usage, &size);
   DCHECK(res);
 
-  auto mad = pmd->CreateAllocatorDump(
+  auto* mad = pmd->CreateAllocatorDump(
       base::StringPrintf("dom_storage/session_storage_0x%" PRIXPTR,
                          reinterpret_cast<uintptr_t>(this)));
   mad->AddScalar(base::trace_event::MemoryAllocatorDump::kNameSize,
@@ -390,9 +396,30 @@ bool SessionStorageDatabase::LazyOpen(bool create_if_needed) {
     if (!s.ok()) {
       LOG(WARNING) << "Failed to open leveldb in " << file_path_.value()
                    << ", error: " << s.ToString();
-      UMA_HISTOGRAM_ENUMERATION(session_storage_uma_name,
-                                SESSION_STORAGE_UMA_FAIL,
-                                SESSION_STORAGE_UMA_MAX);
+      if (s.IsNotFound()) {
+        UMA_HISTOGRAM_ENUMERATION(session_storage_uma_name,
+                                  SESSION_STORAGE_UMA_RECREATE_NOT_FOUND,
+                                  SESSION_STORAGE_UMA_MAX);
+      } else if (s.IsNotSupportedError()) {
+        UMA_HISTOGRAM_ENUMERATION(session_storage_uma_name,
+                                  SESSION_STORAGE_UMA_RECREATE_NOT_SUPPORTED,
+                                  SESSION_STORAGE_UMA_MAX);
+      } else if (s.IsCorruption()) {
+        UMA_HISTOGRAM_ENUMERATION(session_storage_uma_name,
+                                  SESSION_STORAGE_UMA_RECREATE_CORRUPTION,
+                                  SESSION_STORAGE_UMA_MAX);
+      } else if (s.IsInvalidArgument()) {
+        UMA_HISTOGRAM_ENUMERATION(session_storage_uma_name,
+                                  SESSION_STORAGE_UMA_RECREATE_INVALID_ARGUMENT,
+                                  SESSION_STORAGE_UMA_MAX);
+      } else if (s.IsIOError()) {
+        UMA_HISTOGRAM_ENUMERATION(session_storage_uma_name,
+                                  SESSION_STORAGE_UMA_RECREATE_IO_ERROR,
+                                  SESSION_STORAGE_UMA_MAX);
+      } else {
+        NOTREACHED();
+      }
+
       DCHECK(db == NULL);
       db_error_ = true;
       return false;
@@ -497,7 +524,8 @@ bool SessionStorageDatabase::GetAreasInNamespace(
   // Skip the dummy entry "namespace-<namespaceid>-" and iterate the origins.
   for (it->Next(); it->Valid(); it->Next()) {
     std::string key = it->key().ToString();
-    if (key.find(namespace_start_key) != 0) {
+    if (!base::StartsWith(key, namespace_start_key,
+                          base::CompareCase::SENSITIVE)) {
       // Iterated past the origins for this namespace.
       break;
     }
@@ -549,7 +577,7 @@ bool SessionStorageDatabase::DeleteAreaHelper(
   if (!it->Valid())
     return true;
   std::string key = it->key().ToString();
-  if (key.find(namespace_start_key) != 0)
+  if (!base::StartsWith(key, namespace_start_key, base::CompareCase::SENSITIVE))
     batch->Delete(namespace_start_key);
   return true;
 }
@@ -608,7 +636,7 @@ bool SessionStorageDatabase::ReadMap(const std::string& map_id,
   // Skip the dummy entry "map-<mapid>-".
   for (it->Next(); it->Valid(); it->Next()) {
     std::string key = it->key().ToString();
-    if (key.find(map_start_key) != 0) {
+    if (!base::StartsWith(key, map_start_key, base::CompareCase::SENSITIVE)) {
       // Iterated past the keys in this map.
       break;
     }
