@@ -14,6 +14,7 @@
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
+#include "content/browser/download/save_file_manager.h"
 #include "content/browser/download/save_package.h"
 #include "content/browser/loader/resource_dispatcher_host_impl.h"
 #include "content/public/common/url_constants.h"
@@ -100,16 +101,16 @@ class SavePackageTest : public RenderViewHostImplTestHarness {
     // RenderViewHostImplTestHarness::SetUp.
     ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
 
-    save_package_success_ =
-        new SavePackage(contents(), SAVE_PAGE_TYPE_AS_COMPLETE_HTML,
-                        temp_dir_.path().AppendASCII("testfile" HTML_EXTENSION),
-                        temp_dir_.path().AppendASCII("testfile_files"));
+    save_package_success_ = new SavePackage(
+        contents(), SAVE_PAGE_TYPE_AS_COMPLETE_HTML,
+        temp_dir_.GetPath().AppendASCII("testfile" HTML_EXTENSION),
+        temp_dir_.GetPath().AppendASCII("testfile_files"));
 
     base::FilePath::StringType long_file_name = GetLongFileName();
     save_package_fail_ = new SavePackage(
         contents(), SAVE_PAGE_TYPE_AS_COMPLETE_HTML,
-        temp_dir_.path().Append(long_file_name + FPL_HTML_EXTENSION),
-        temp_dir_.path().Append(long_file_name + FPL("_files")));
+        temp_dir_.GetPath().Append(long_file_name + FPL_HTML_EXTENSION),
+        temp_dir_.GetPath().Append(long_file_name + FPL("_files")));
   }
 
   BrowserContext* CreateBrowserContext() override {
@@ -118,6 +119,8 @@ class SavePackageTest : public RenderViewHostImplTestHarness {
     // to create a ResourceDispatcherHostImpl so that our SavePackage objects
     // can initialize correctly.
     rdh_.reset(new ResourceDispatcherHostImpl);
+    // Initialize the SaveFileManager instance which we will use for the tests.
+    save_file_manager_ = new SaveFileManager();
     return RenderViewHostImplTestHarness::CreateBrowserContext();
   }
 
@@ -135,7 +138,7 @@ class SavePackageTest : public RenderViewHostImplTestHarness {
   // Returns a path that is *almost* kMaxFilePathLength long
   base::FilePath::StringType GetLongFileName() const {
     size_t target_length =
-        kMaxFilePathLength - 9 - temp_dir_.path().value().length();
+        kMaxFilePathLength - 9 - temp_dir_.GetPath().value().length();
     return base::FilePath::StringType(target_length, FPL('a'));
   }
 
@@ -148,6 +151,7 @@ class SavePackageTest : public RenderViewHostImplTestHarness {
   base::ScopedTempDir temp_dir_;
 
   std::unique_ptr<ResourceDispatcherHostImpl> rdh_;
+  scoped_refptr<SaveFileManager> save_file_manager_;
 };
 
 static const struct {
@@ -388,6 +392,24 @@ static const struct SuggestedSaveNameTestCase {
     FPL("bar"),
     false
   },
+  // A URL with escaped special characters, when title matches the URL.
+  { "http://foo.com/%40.txt",
+    base::ASCIIToUTF16("foo.com/%40.txt"),
+    FPL("@.txt"),
+    false
+  },
+  // A URL with unescaped special characters, when title matches the URL.
+  { "http://foo.com/@.txt",
+    base::ASCIIToUTF16("foo.com/@.txt"),
+    FPL("@.txt"),
+    false
+  },
+  // A URL with punycode in the host name, when title matches the URL.
+  { "http://xn--bcher-kva.com",
+    base::UTF8ToUTF16("bücher.com"),
+    FPL("bücher.com"),
+    false
+  },
   // If the title matches the URL, but there is no "filename" component,
   // use the domain.
   { "http://foo.com",
@@ -419,11 +441,9 @@ TEST_F(SavePackageTest, MAYBE_TestSuggestedSaveNames) {
   GURL url = net::URLRequestMockHTTPJob::GetMockUrl("save_page/a.htm");
   NavigateAndCommit(url);
   for (size_t i = 0; i < arraysize(kSuggestedSaveNames); ++i) {
-    scoped_refptr<SavePackage> save_package(new SavePackage(contents()));
-    save_package->page_url_ = GURL(kSuggestedSaveNames[i].page_url);
-    save_package->title_ = kSuggestedSaveNames[i].page_title;
-
-    base::FilePath save_name = save_package->GetSuggestedNameForSaveAs(
+    base::FilePath save_name = SavePackage::GetSuggestedNameForSaveAs(
+        kSuggestedSaveNames[i].page_title,
+        GURL(kSuggestedSaveNames[i].page_url),
         kSuggestedSaveNames[i].ensure_html_extension, std::string());
     EXPECT_EQ(kSuggestedSaveNames[i].expected_name, save_name.value()) <<
         "Test case " << i;
