@@ -5,7 +5,10 @@
 #include "base/macros.h"
 #include "content/browser/frame_host/navigation_handle_impl.h"
 #include "content/public/browser/navigation_throttle.h"
+#include "content/public/browser/ssl_status.h"
+#include "content/public/common/request_context_type.h"
 #include "content/test/test_render_frame_host.h"
+#include "content/test/test_web_contents.h"
 
 namespace content {
 
@@ -65,8 +68,14 @@ class NavigationHandleImplTest : public RenderViewHostImplTestHarness {
   void SetUp() override {
     RenderViewHostImplTestHarness::SetUp();
     test_handle_ = NavigationHandleImpl::Create(
-        GURL(), main_test_rfh()->frame_tree_node(), true, false, false,
-        base::TimeTicks::Now(), 0);
+        GURL(), main_test_rfh()->frame_tree_node(),
+        true,   // is_renderer_initiated
+        false,  // is_same_page
+        base::TimeTicks::Now(), 0,
+        false);  // started_from_context_menu
+    EXPECT_EQ(REQUEST_CONTEXT_TYPE_UNSPECIFIED,
+              test_handle_->request_context_type_);
+    contents()->GetMainFrame()->InitializeRenderFrameIfNeeded();
   }
 
   void TearDown() override {
@@ -103,6 +112,8 @@ class NavigationHandleImplTest : public RenderViewHostImplTestHarness {
     // the NavigationHandleImplTest.
     test_handle_->WillStartRequest(
         "GET", nullptr, Referrer(), false, ui::PAGE_TRANSITION_LINK, false,
+        REQUEST_CONTEXT_TYPE_LOCATION,
+        blink::WebMixedContentContextType::Blockable,
         base::Bind(&NavigationHandleImplTest::UpdateThrottleCheckResult,
                    base::Unretained(this)));
   }
@@ -120,6 +131,7 @@ class NavigationHandleImplTest : public RenderViewHostImplTestHarness {
     // the NavigationHandleImplTest.
     test_handle_->WillRedirectRequest(
         GURL(), "GET", GURL(), false, scoped_refptr<net::HttpResponseHeaders>(),
+        net::HttpResponseInfo::CONNECTION_INFO_HTTP1_1,
         base::Bind(&NavigationHandleImplTest::UpdateThrottleCheckResult,
                    base::Unretained(this)));
   }
@@ -133,11 +145,14 @@ class NavigationHandleImplTest : public RenderViewHostImplTestHarness {
     was_callback_called_ = false;
     callback_result_ = NavigationThrottle::DEFER;
 
-    // It's safe to use base::Unretained since the NavigationHandle is owned by
-    // the NavigationHandleImplTest.
+    // It's safe to use base::Unretained since the NavigationHandle is owned
+    // by the NavigationHandleImplTest. The ConnectionInfo is different from
+    // that sent to WillRedirectRequest to verify that it's correctly plumbed
+    // in both cases.
     test_handle_->WillProcessResponse(
-        main_test_rfh(),
-        scoped_refptr<net::HttpResponseHeaders>(),
+        main_test_rfh(), scoped_refptr<net::HttpResponseHeaders>(),
+        net::HttpResponseInfo::CONNECTION_INFO_QUIC_35, SSLStatus(),
+        GlobalRequestID(), false, false, false, base::Closure(),
         base::Bind(&NavigationHandleImplTest::UpdateThrottleCheckResult,
                    base::Unretained(this)));
   }
@@ -178,6 +193,41 @@ class NavigationHandleImplTest : public RenderViewHostImplTestHarness {
   NavigationThrottle::ThrottleCheckResult callback_result_;
 };
 
+// Checks that the request_context_type is properly set.
+// Note: can be extended to cover more internal members.
+TEST_F(NavigationHandleImplTest, SimpleDataChecks) {
+  SimulateWillStartRequest();
+  EXPECT_EQ(REQUEST_CONTEXT_TYPE_LOCATION,
+            test_handle()->request_context_type());
+  EXPECT_EQ(net::HttpResponseInfo::CONNECTION_INFO_UNKNOWN,
+            test_handle()->GetConnectionInfo());
+
+  test_handle()->Resume();
+  SimulateWillRedirectRequest();
+  EXPECT_EQ(REQUEST_CONTEXT_TYPE_LOCATION,
+            test_handle()->request_context_type());
+  EXPECT_EQ(net::HttpResponseInfo::CONNECTION_INFO_HTTP1_1,
+            test_handle()->GetConnectionInfo());
+
+  test_handle()->Resume();
+  SimulateWillProcessResponse();
+  EXPECT_EQ(REQUEST_CONTEXT_TYPE_LOCATION,
+            test_handle()->request_context_type());
+  EXPECT_EQ(net::HttpResponseInfo::CONNECTION_INFO_QUIC_35,
+            test_handle()->GetConnectionInfo());
+}
+
+TEST_F(NavigationHandleImplTest, SimpleDataCheckNoRedirect) {
+  SimulateWillStartRequest();
+  EXPECT_EQ(net::HttpResponseInfo::CONNECTION_INFO_UNKNOWN,
+            test_handle()->GetConnectionInfo());
+
+  test_handle()->Resume();
+  SimulateWillProcessResponse();
+  EXPECT_EQ(net::HttpResponseInfo::CONNECTION_INFO_QUIC_35,
+            test_handle()->GetConnectionInfo());
+}
+
 // Checks that a deferred navigation can be properly resumed.
 TEST_F(NavigationHandleImplTest, ResumeDeferred) {
   TestNavigationThrottle* test_throttle =
@@ -629,13 +679,15 @@ TEST_F(NavigationHandleImplTest, CancelThenProceedWillProcessResponse) {
   EXPECT_EQ(0, proceed_throttle->will_start_calls());
   EXPECT_EQ(0, proceed_throttle->will_redirect_calls());
 
-  // Simulate WillRedirectRequest. The request should not be deferred. The
+  // Simulate WillProcessResponse. The request should not be deferred. The
   // callback should have been called. The second throttle should not have
   // been notified.
   SimulateWillProcessResponse();
   EXPECT_FALSE(IsDeferringStart());
   EXPECT_FALSE(IsDeferringRedirect());
+  EXPECT_FALSE(IsDeferringResponse());
   EXPECT_TRUE(was_callback_called());
+  EXPECT_TRUE(IsCanceling());
   EXPECT_EQ(NavigationThrottle::CANCEL_AND_IGNORE, callback_result());
   EXPECT_EQ(0, cancel_throttle->will_start_calls());
   EXPECT_EQ(0, cancel_throttle->will_redirect_calls());
@@ -645,4 +697,39 @@ TEST_F(NavigationHandleImplTest, CancelThenProceedWillProcessResponse) {
   EXPECT_EQ(0, proceed_throttle->will_process_response_calls());
 }
 
+// Checks that a NavigationThrottle asking to block the response followed by a
+// NavigationThrottle asking to proceed behave correctly in WillProcessResponse.
+// The navigation will be canceled directly, and the second throttle will not
+// be called.
+TEST_F(NavigationHandleImplTest, BlockResponseThenProceedWillProcessResponse) {
+  TestNavigationThrottle* cancel_throttle =
+      CreateTestNavigationThrottle(NavigationThrottle::BLOCK_RESPONSE);
+  TestNavigationThrottle* proceed_throttle =
+      CreateTestNavigationThrottle(NavigationThrottle::PROCEED);
+  EXPECT_FALSE(IsDeferringStart());
+  EXPECT_FALSE(IsDeferringRedirect());
+  EXPECT_EQ(0, cancel_throttle->will_start_calls());
+  EXPECT_EQ(0, cancel_throttle->will_redirect_calls());
+  EXPECT_EQ(0, cancel_throttle->will_process_response_calls());
+  EXPECT_EQ(0, proceed_throttle->will_start_calls());
+  EXPECT_EQ(0, proceed_throttle->will_redirect_calls());
+
+  // Simulate WillRedirectRequest. The request should not be deferred. The
+  // callback should have been called. The second throttle should not have
+  // been notified.
+  SimulateWillProcessResponse();
+  EXPECT_FALSE(IsDeferringStart());
+  EXPECT_FALSE(IsDeferringRedirect());
+  EXPECT_FALSE(IsDeferringResponse());
+  EXPECT_TRUE(was_callback_called());
+  EXPECT_TRUE(IsCanceling());
+  EXPECT_EQ(NavigationThrottle::BLOCK_RESPONSE, callback_result());
+  EXPECT_EQ(0, cancel_throttle->will_start_calls());
+  EXPECT_EQ(0, cancel_throttle->will_redirect_calls());
+  EXPECT_EQ(1, cancel_throttle->will_process_response_calls());
+  EXPECT_EQ(0, proceed_throttle->will_start_calls());
+  EXPECT_EQ(0, proceed_throttle->will_redirect_calls());
+  EXPECT_EQ(0, proceed_throttle->will_process_response_calls());
+}
+
 }  // namespace content
