@@ -9,6 +9,7 @@
 #include "content/browser/frame_host/navigation_request_info.h"
 #include "content/browser/loader/navigation_url_loader_factory.h"
 #include "content/browser/loader/navigation_url_loader_impl.h"
+#include "content/public/browser/navigation_ui_data.h"
 
 namespace content {
 
@@ -17,15 +18,18 @@ static NavigationURLLoaderFactory* g_factory = nullptr;
 std::unique_ptr<NavigationURLLoader> NavigationURLLoader::Create(
     BrowserContext* browser_context,
     std::unique_ptr<NavigationRequestInfo> request_info,
-    ServiceWorkerContextWrapper* service_worker_context_wrapper,
+    std::unique_ptr<NavigationUIData> navigation_ui_data,
+    ServiceWorkerNavigationHandle* service_worker_handle,
+    AppCacheNavigationHandle* appcache_handle,
     NavigationURLLoaderDelegate* delegate) {
   if (g_factory) {
     return g_factory->CreateLoader(browser_context, std::move(request_info),
-                                   service_worker_context_wrapper, delegate);
+                                   std::move(navigation_ui_data),
+                                   service_worker_handle, delegate);
   }
-  return std::unique_ptr<NavigationURLLoader>(
-      new NavigationURLLoaderImpl(browser_context, std::move(request_info),
-                                  service_worker_context_wrapper, delegate));
+  return std::unique_ptr<NavigationURLLoader>(new NavigationURLLoaderImpl(
+      browser_context, std::move(request_info), std::move(navigation_ui_data),
+      service_worker_handle, appcache_handle, delegate));
 }
 
 void NavigationURLLoader::SetFactoryForTesting(
