@@ -19,35 +19,36 @@
 #include <string>
 #include <vector>
 
+#include "base/callback_forward.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
+#include "base/memory/ref_counted.h"
 #include "base/observer_list.h"
 #include "base/time/time.h"
-#include "base/timer/timer.h"
-#include "content/browser/download/download_resource_handler.h"
-#include "content/browser/download/save_types.h"
 #include "content/browser/loader/global_routing_id.h"
-#include "content/browser/loader/resource_loader.h"
 #include "content/browser/loader/resource_loader_delegate.h"
-#include "content/browser/loader/resource_scheduler.h"
 #include "content/common/content_export.h"
+#include "content/common/url_loader.mojom.h"
 #include "content/public/browser/global_request_id.h"
 #include "content/public/browser/resource_dispatcher_host.h"
-#include "content/public/browser/web_contents_observer.h"
+#include "content/public/browser/resource_request_info.h"
+#include "content/public/common/previews_state.h"
+#include "content/public/common/request_context_type.h"
 #include "content/public/common/resource_type.h"
 #include "ipc/ipc_message.h"
+#include "net/base/load_states.h"
 #include "net/base/request_priority.h"
-#include "net/cookies/canonical_cookie.h"
-#include "net/url_request/url_request.h"
-
-class ResourceHandler;
+#include "third_party/WebKit/public/platform/WebMixedContentContextType.h"
+#include "url/gurl.h"
 
 namespace base {
 class FilePath;
+class RepeatingTimer;
 }
 
 namespace net {
-class URLRequestJobFactory;
+class URLRequest;
+class HttpRequestHeaders;
 }
 
 namespace storage {
@@ -55,46 +56,44 @@ class ShareableFileReference;
 }
 
 namespace content {
+class AppCacheNavigationHandleCore;
 class AppCacheService;
 class AsyncRevalidationManager;
-class CertStore;
-class FrameTree;
 class LoaderDelegate;
 class NavigationURLLoaderImplCore;
+class NavigationUIData;
 class RenderFrameHostImpl;
 class ResourceContext;
 class ResourceDispatcherHostDelegate;
+class ResourceLoader;
+class ResourceHandler;
 class ResourceMessageDelegate;
-class ResourceMessageFilter;
+class ResourceRequesterInfo;
 class ResourceRequestInfoImpl;
-class SaveFileManager;
+class ResourceScheduler;
 class ServiceWorkerNavigationHandleCore;
-struct CommonNavigationParams;
-struct DownloadSaveInfo;
 struct NavigationRequestInfo;
 struct Referrer;
 struct ResourceRequest;
 
-// This class is responsible for notifying the IO thread (specifically, the
-// ResourceDispatcherHostImpl) of frame events. It has an interace for callers
-// to use and also sends notifications on WebContentsObserver events. All
-// methods (static or class) will be called from the UI thread and post to the
-// IO thread.
-// TODO(csharrison): Add methods tracking visibility and audio changes, to
-// propogate to the ResourceScheduler.
-class LoaderIOThreadNotifier : public WebContentsObserver {
- public:
-  explicit LoaderIOThreadNotifier(WebContents* web_contents);
-  ~LoaderIOThreadNotifier() override;
-
-  // content::WebContentsObserver:
-  void RenderFrameDeleted(RenderFrameHost* render_frame_host) override;
-};
+using CreateDownloadHandlerIntercept =
+    base::Callback<std::unique_ptr<ResourceHandler>(net::URLRequest*)>;
 
 class CONTENT_EXPORT ResourceDispatcherHostImpl
     : public ResourceDispatcherHost,
       public ResourceLoaderDelegate {
  public:
+  // Used to handle the result of SyncLoad IPC. |result| is null if it's
+  // unavailable due to an error.
+  using SyncLoadResultCallback =
+      base::Callback<void(const SyncLoadResult* result)>;
+
+  // This constructor should be used if we want downloads to work correctly.
+  // TODO(ananta)
+  // Work on moving creation of download handlers out of
+  // ResourceDispatcherHostImpl.
+  ResourceDispatcherHostImpl(
+      CreateDownloadHandlerIntercept download_handler_intercept);
   ResourceDispatcherHostImpl();
   ~ResourceDispatcherHostImpl() override;
 
@@ -124,6 +123,9 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   void SetDelegate(ResourceDispatcherHostDelegate* delegate) override;
   void SetAllowCrossOriginAuthPrompt(bool value) override;
   void ClearLoginDelegateForRequest(net::URLRequest* request) override;
+  void RegisterInterceptor(const std::string& http_header,
+                           const std::string& starts_with,
+                           const InterceptorCallback& interceptor) override;
 
   // Puts the resource dispatcher host in an inactive state (unable to begin
   // new requests).  Cancels all pending requests.
@@ -136,29 +138,7 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
 
   // Returns true if the message was a resource message that was processed.
   bool OnMessageReceived(const IPC::Message& message,
-                         ResourceMessageFilter* filter);
-
-  DownloadInterruptReason BeginDownload(
-      std::unique_ptr<net::URLRequest> request,
-      const Referrer& referrer,
-      bool is_content_initiated,
-      ResourceContext* context,
-      int render_process_id,
-      int render_view_route_id,
-      int render_frame_route_id,
-      bool do_not_prompt_for_login,
-      const std::string& suggested_name);
-
-  // Initiates a save file from the browser process (as opposed to a resource
-  // request from the renderer or another child process).
-  void BeginSaveFile(const GURL& url,
-                     const Referrer& referrer,
-                     SaveItemId save_item_id,
-                     SavePackageId save_package_id,
-                     int child_id,
-                     int render_view_route_id,
-                     int render_frame_route_id,
-                     ResourceContext* context);
+                         ResourceRequesterInfo* requester_info);
 
   // Cancels the given request if it still exists.
   void CancelRequest(int child_id, int request_id);
@@ -168,7 +148,7 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // resumed by a new process.
   void MarkAsTransferredNavigation(
       const GlobalRequestID& id,
-      const scoped_refptr<ResourceResponse>& response);
+      const base::Closure& on_transfer_complete_callback);
 
   // Cancels a request previously marked as being transferred, for use when a
   // navigation was cancelled.
@@ -197,10 +177,6 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // request. Experimentally obtained.
   static const int kAvgBytesPerOutstandingRequest = 4400;
 
-  SaveFileManager* save_file_manager() const {
-    return save_file_manager_.get();
-  }
-
   // Called when a RenderViewHost is created.
   void OnRenderViewHostCreated(int child_id,
                                int route_id);
@@ -258,11 +234,13 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // Must be called after the ResourceRequestInfo has been created
   // and associated with the request.
   // This is marked virtual so it can be overriden in testing.
+  // TODO(ananta)
+  // This method should be removed or moved outside this class.
   virtual std::unique_ptr<ResourceHandler> CreateResourceHandlerForDownload(
       net::URLRequest* request,
       bool is_content_initiated,
       bool must_download,
-      const std::string& suggested_name);
+      bool is_new_request);
 
   // Called to determine whether the response to |request| should be intercepted
   // and handled as a stream. Streams are used to pass direct access to a
@@ -297,9 +275,17 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
 
   // PlzNavigate: Begins a request for NavigationURLLoader. |loader| is the
   // loader to attach to the leaf resource handler.
-  void BeginNavigationRequest(ResourceContext* resource_context,
-                              const NavigationRequestInfo& info,
-                              NavigationURLLoaderImplCore* loader);
+  void BeginNavigationRequest(
+      ResourceContext* resource_context,
+      const NavigationRequestInfo& info,
+      std::unique_ptr<NavigationUIData> navigation_ui_data,
+      NavigationURLLoaderImplCore* loader,
+      ServiceWorkerNavigationHandleCore* service_worker_handle_core,
+      AppCacheNavigationHandleCore* appcache_handle_core);
+
+  int num_in_flight_requests_for_testing() const {
+    return num_in_flight_requests_;
+  }
 
   // Turns on stale-while-revalidate support, regardless of command-line flags
   // or experiment status. For unit tests only.
@@ -309,8 +295,64 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // transferred. The LoaderDelegate should be interacted with on the IO thread.
   void SetLoaderDelegate(LoaderDelegate* loader_delegate);
 
+  void OnRenderFrameDeleted(const GlobalFrameRoutingId& global_routing_id);
+
+  // Called when loading a request with mojo.
+  void OnRequestResourceWithMojo(
+      ResourceRequesterInfo* requester_info,
+      int routing_id,
+      int request_id,
+      const ResourceRequest& request,
+      mojom::URLLoaderAssociatedRequest mojo_request,
+      mojom::URLLoaderClientAssociatedPtr url_loader_client);
+
+  void OnSyncLoadWithMojo(ResourceRequesterInfo* requester_info,
+                          int routing_id,
+                          int request_id,
+                          const ResourceRequest& request_data,
+                          const SyncLoadResultCallback& result_handler);
+
+  // Helper function for initializing the |request| passed in. By initializing
+  // we mean setting the |referrer| on the |request|, associating the
+  // ResourceRequestInfoImpl structure with the |request|, etc.
+  // This function should be called for invoking the BeginURLRequest() function
+  // to initiate a URL request.
+  void InitializeURLRequest(net::URLRequest* request,
+                            const Referrer& referrer,
+                            bool is_download,
+                            int render_process_host_id,
+                            int render_view_routing_id,
+                            int render_frame_routing_id,
+                            PreviewsState previews_state,
+                            ResourceContext* context);
+
+  // Helper function for initiating a URL request. The |is_download| and
+  // |is_content_initiated and |do_not_prompt_for_login| parameters are
+  // specific to download requests.
+  // TODO(ananta)
+  // Look into a better way of passing these parameters in.
+  // Please note that the InitializeURLRequest() function needs to be called
+  // called to initialize the request before calling this function.
+  void BeginURLRequest(std::unique_ptr<net::URLRequest> request,
+                       std::unique_ptr<ResourceHandler> handler,
+                       bool is_download,
+                       bool is_content_initiated,
+                       bool do_not_prompt_for_login,
+                       ResourceContext* context);
+
+  bool is_shutdown() const { return is_shutdown_; }
+
+  // Creates a new request ID for browser initiated requests. See the comments
+  // of |request_id_| for the details. Must be called on the IO thread.
+  int MakeRequestID();
+
+  // Cancels a request as requested by a renderer. This function is called when
+  // a mojo connection is lost.
+  // Note that this cancel is subtly different from the other CancelRequest
+  // methods in this file, which also tear down the loader.
+  void CancelRequestFromRenderer(GlobalRequestID request_id);
+
  private:
-  friend class LoaderIOThreadNotifier;
   friend class ResourceDispatcherHostTest;
 
   FRIEND_TEST_ALL_PREFIXES(ResourceDispatcherHostTest,
@@ -323,6 +365,10 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
                            TestProcessCancelDetachableTimesOut);
   FRIEND_TEST_ALL_PREFIXES(SitePerProcessIgnoreCertErrorsBrowserTest,
                            CrossSiteRedirectCertificateStore);
+  FRIEND_TEST_ALL_PREFIXES(ResourceDispatcherHostTest, LoadInfo);
+  FRIEND_TEST_ALL_PREFIXES(ResourceDispatcherHostTest, LoadInfoSamePriority);
+  FRIEND_TEST_ALL_PREFIXES(ResourceDispatcherHostTest, LoadInfoUploadProgress);
+  FRIEND_TEST_ALL_PREFIXES(ResourceDispatcherHostTest, LoadInfoTwoRenderViews);
 
   struct OustandingRequestsStats {
     int memory_cost;
@@ -333,15 +379,38 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   friend class ResourceMessageDelegate;
 
   // Information about status of a ResourceLoader.
-  struct LoadInfo {
+  struct CONTENT_EXPORT LoadInfo {
+    LoadInfo();
+    LoadInfo(const LoadInfo& other);
+    ~LoadInfo();
+
+    ResourceRequestInfo::WebContentsGetter web_contents_getter;
     GURL url;
     net::LoadStateWithParam load_state;
     uint64_t upload_position;
     uint64_t upload_size;
   };
 
-  // Map from ProcessID+RouteID pair to the "most interesting" LoadState.
-  typedef std::map<GlobalRoutingID, LoadInfo> LoadInfoMap;
+  // Map from WebContents* to the "most interesting" LoadState.
+  typedef std::map<WebContents*, LoadInfo> LoadInfoMap;
+  typedef std::vector<LoadInfo> LoadInfoList;
+
+  // Information about a HTTP header interceptor.
+  struct HeaderInterceptorInfo {
+    // Structure is sufficiently complicated to require the constructor
+    // destructor definitions in the cc file.
+    HeaderInterceptorInfo();
+    ~HeaderInterceptorInfo();
+    HeaderInterceptorInfo(const HeaderInterceptorInfo& other);
+
+    // Used to prefix match the value of the http header.
+    std::string starts_with;
+    // The interceptor.
+    InterceptorCallback interceptor;
+  };
+
+  // Map from HTTP header to its information HeaderInterceptorInfo.
+  typedef std::map<std::string, HeaderInterceptorInfo> HeaderInterceptorMap;
 
   // ResourceLoaderDelegate implementation:
   ResourceDispatcherHostLoginDelegate* CreateLoginDelegate(
@@ -349,7 +418,8 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
       net::AuthChallengeInfo* auth_info) override;
   bool HandleExternalProtocol(ResourceLoader* loader, const GURL& url) override;
   void DidStartRequest(ResourceLoader* loader) override;
-  void DidReceiveRedirect(ResourceLoader* loader, const GURL& new_url) override;
+  void DidReceiveRedirect(ResourceLoader* loader, const GURL& new_url,
+                          ResourceResponse* response) override;
   void DidReceiveResponse(ResourceLoader* loader) override;
   void DidFinishLoading(ResourceLoader* loader) override;
   std::unique_ptr<net::ClientCertStore> CreateClientCertStore(
@@ -361,9 +431,7 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // A shutdown helper that runs on the IO thread.
   void OnShutdown();
 
-  void OnRenderFrameDeleted(const GlobalFrameRoutingId& global_routing_id);
-
-  // Helper function for regular and download requests.
+  // Helper function for URL requests.
   void BeginRequestInternal(std::unique_ptr<net::URLRequest> request,
                             std::unique_ptr<ResourceHandler> handler);
 
@@ -441,8 +509,18 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // waiting for a host name to resolve implies being stuck.
   static bool LoadInfoIsMoreInteresting(const LoadInfo& a, const LoadInfo& b);
 
-  // Gets the most interesting LoadInfo for each GlobalRoutingID.
-  std::unique_ptr<LoadInfoMap> GetLoadInfoForAllRoutes();
+  // Picks the most interesting LoadInfos from the given array per WebContents
+  // and calls the delegate with them.
+  static void UpdateLoadStateOnUI(LoaderDelegate* loader_delegate,
+                                  std::unique_ptr<LoadInfoList> infos);
+
+  // Returns the most interesting LoadInfos from the given array per
+  // WebContents.
+  static std::unique_ptr<LoadInfoMap> PickMoreInterestingLoadInfos(
+      std::unique_ptr<LoadInfoList> infos);
+
+  // Gets all the LoadInfos for each pending request.
+  std::unique_ptr<LoadInfoList> GetLoadInfoForAllRoutes();
 
   // Checks all pending requests and updates the load info if necessary.
   void UpdateLoadInfo();
@@ -452,10 +530,21 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
       const GlobalFrameRoutingId& global_routing_id,
       bool cancel_requests);
 
-  void OnRequestResource(int routing_id,
+  void OnRequestResource(ResourceRequesterInfo* requester_info,
+                         int routing_id,
                          int request_id,
                          const ResourceRequest& request_data);
-  void OnSyncLoad(int request_id,
+
+  void OnRequestResourceInternal(
+      ResourceRequesterInfo* requester_info,
+      int routing_id,
+      int request_id,
+      const ResourceRequest& request_data,
+      mojom::URLLoaderAssociatedRequest mojo_request,
+      mojom::URLLoaderClientAssociatedPtr url_loader_client);
+
+  void OnSyncLoad(ResourceRequesterInfo* requester_info,
+                  int request_id,
                   const ResourceRequest& request_data,
                   IPC::Message* sync_result);
 
@@ -463,27 +552,66 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
 
   // Update the ResourceRequestInfo and internal maps when a request is
   // transferred from one process to another.
-  void UpdateRequestForTransfer(int child_id,
-                                int route_id,
-                                int request_id,
-                                const ResourceRequest& request_data,
-                                LoaderMap::iterator iter);
+  void UpdateRequestForTransfer(
+      ResourceRequesterInfo* requester_info,
+      int route_id,
+      int request_id,
+      const ResourceRequest& request_data,
+      LoaderMap::iterator iter,
+      mojom::URLLoaderAssociatedRequest mojo_request,
+      mojom::URLLoaderClientAssociatedPtr url_loader_client);
 
-  void BeginRequest(int request_id,
-                    const ResourceRequest& request_data,
-                    IPC::Message* sync_result,  // only valid for sync
-                    int route_id);              // only valid for async
+  // If |request_data| is for a request being transferred from another process,
+  // then CompleteTransfer method can be used to complete the transfer.
+  void CompleteTransfer(ResourceRequesterInfo* requester_info,
+                        int request_id,
+                        const ResourceRequest& request_data,
+                        int route_id,
+                        mojom::URLLoaderAssociatedRequest mojo_request,
+                        mojom::URLLoaderClientAssociatedPtr url_loader_client);
+
+  void BeginRequest(
+      ResourceRequesterInfo* requester_info,
+      int request_id,
+      const ResourceRequest& request_data,
+      const SyncLoadResultCallback& sync_result_handler,  // only valid for sync
+      int route_id,
+      mojom::URLLoaderAssociatedRequest mojo_request,
+      mojom::URLLoaderClientAssociatedPtr url_loader_client);
+
+  // There are requests which need decisions to be made like the following:
+  // Whether the presence of certain HTTP headers like the Origin header are
+  // valid, etc. These requests may need to be aborted based on these
+  // decisions which could be time consuming. We allow for these decisions
+  // to be made asynchronously. The request proceeds when we hear back from
+  // the interceptors about whether to continue or not.
+  // The |continue_request| parameter in the function indicates whether the
+  // request should be continued or aborted. The |error_code| parameter is set
+  // if |continue_request| is false.
+  void ContinuePendingBeginRequest(
+      scoped_refptr<ResourceRequesterInfo> requester_info,
+      int request_id,
+      const ResourceRequest& request_data,
+      const SyncLoadResultCallback& sync_result_handler,  // only valid for sync
+      int route_id,
+      const net::HttpRequestHeaders& headers,
+      mojom::URLLoaderAssociatedRequest mojo_request,
+      mojom::URLLoaderClientAssociatedPtr url_loader_client,
+      bool continue_request,
+      int error_code);
 
   // Creates a ResourceHandler to be used by BeginRequest() for normal resource
   // loading.
   std::unique_ptr<ResourceHandler> CreateResourceHandler(
+      ResourceRequesterInfo* requester_info,
       net::URLRequest* request,
       const ResourceRequest& request_data,
-      IPC::Message* sync_result,
+      const SyncLoadResultCallback& sync_result_handler,
       int route_id,
-      int process_type,
       int child_id,
-      ResourceContext* resource_context);
+      ResourceContext* resource_context,
+      mojom::URLLoaderAssociatedRequest mojo_request,
+      mojom::URLLoaderClientAssociatedPtr url_loader_client);
 
   // Wraps |handler| in the standard resource handlers for normal resource
   // loading and navigation requests. This adds MimeTypeResourceHandler and
@@ -492,15 +620,18 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
       net::URLRequest* request,
       ResourceType resource_type,
       ResourceContext* resource_context,
+      RequestContextType fetch_request_context_type,
+      blink::WebMixedContentContextType fetch_mixed_content_context_type,
       AppCacheService* appcache_service,
       int child_id,
       int route_id,
       std::unique_ptr<ResourceHandler> handler);
 
-  void OnDataDownloadedACK(int request_id);
-  void OnCancelRequest(int request_id);
-  void OnReleaseDownloadedFile(int request_id);
-  void OnDidChangePriority(int request_id,
+  void OnCancelRequest(ResourceRequesterInfo* requester_info, int request_id);
+  void OnReleaseDownloadedFile(ResourceRequesterInfo* requester_info,
+                               int request_id);
+  void OnDidChangePriority(ResourceRequesterInfo* requester_info,
+                           int request_id,
                            net::RequestPriority new_priority,
                            int intra_priority_value);
 
@@ -510,6 +641,7 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
       int child_id,
       int render_view_route_id,
       int render_frame_route_id,
+      PreviewsState previews_state,
       bool download,
       ResourceContext* context);
 
@@ -543,18 +675,29 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
                                          ResourceMessageDelegate* delegate);
 
   int BuildLoadFlagsForRequest(const ResourceRequest& request_data,
-                               int child_id,
                                bool is_sync_load);
 
-  // The certificate on a ResourceResponse is associated with a
-  // particular renderer process. As a transfer to a new process
-  // completes, the stored certificate has to be updated to reflect the
-  // new renderer process.
-  void UpdateResponseCertificateForTransfer(ResourceResponse* response,
-                                            const net::SSLInfo& ssl_info,
-                                            int child_id);
+  // Consults the RendererSecurity policy to determine whether the
+  // ResourceDispatcherHostImpl should service this request.  A request might
+  // be disallowed if the renderer is not authorized to retrieve the request
+  // URL or if the renderer is attempting to upload an unauthorized file.
+  bool ShouldServiceRequest(int child_id,
+                            const ResourceRequest& request_data,
+                            const net::HttpRequestHeaders& headers,
+                            ResourceRequesterInfo* requester_info,
+                            ResourceContext* resource_context);
 
-  CertStore* GetCertStore();
+  // Notifies the ResourceDispatcherHostDelegate about a download having
+  // started. The function returns the |handler| passed in, if the download
+  // is not throttled. If the download is to be throttled (Decided by the
+  // delegate) the function returns a ThrottlingResourceHandler to handle the
+  // download.
+  std::unique_ptr<ResourceHandler> HandleDownloadStarted(
+      net::URLRequest* request,
+      std::unique_ptr<ResourceHandler> handler,
+      bool is_content_initiated,
+      bool must_download,
+      bool is_new_request);
 
   LoaderMap pending_loaders_;
 
@@ -571,9 +714,6 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // not empty and at least one RenderViewHost is loading.
   std::unique_ptr<base::RepeatingTimer> update_load_states_timer_;
 
-  // We own the save file manager.
-  scoped_refptr<SaveFileManager> save_file_manager_;
-
   // Request ID for browser initiated requests. request_ids generated by
   // child processes are counted up from 0, while browser created requests
   // start at -2 and go down from there. (We need to start at -2 because -1 is
@@ -626,10 +766,6 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
   // may be because of explicit user action.
   base::TimeTicks last_user_gesture_time_;
 
-  // Used during IPC message dispatching so that the handlers can get a pointer
-  // to the source of the message.
-  ResourceMessageFilter* filter_;
-
   ResourceDispatcherHostDelegate* delegate_;
 
   LoaderDelegate* loader_delegate_;
@@ -646,9 +782,11 @@ class CONTENT_EXPORT ResourceDispatcherHostImpl
 
   std::unique_ptr<ResourceScheduler> scheduler_;
 
-  // Allows tests to use a mock CertStore. If set, the CertStore must
-  // outlive this ResourceDispatcherHostImpl.
-  CertStore* cert_store_for_testing_;
+  // Used to invoke an interceptor for the HTTP header.
+  HeaderInterceptorMap http_header_interceptor_map_;
+
+  // Points to the registered download handler intercept.
+  CreateDownloadHandlerIntercept create_download_handler_intercept_;
 
   DISALLOW_COPY_AND_ASSIGN(ResourceDispatcherHostImpl);
 };
