@@ -8,7 +8,9 @@
 #include <utility>
 #include <vector>
 
+#include "base/command_line.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
 #include "base/stl_util.h"
 #include "content/browser/service_worker/embedded_worker_registry.h"
@@ -16,9 +18,16 @@
 #include "content/browser/service_worker/embedded_worker_test_helper.h"
 #include "content/browser/service_worker/service_worker_context_core.h"
 #include "content/browser/service_worker/service_worker_context_wrapper.h"
+#include "content/browser/service_worker/service_worker_test_utils.h"
+#include "content/common/service_worker/embedded_worker.mojom.h"
 #include "content/common/service_worker/embedded_worker_messages.h"
+#include "content/common/service_worker/embedded_worker_start_params.h"
+#include "content/common/service_worker/service_worker_event_dispatcher.mojom.h"
+#include "content/common/service_worker/service_worker_utils.h"
 #include "content/public/common/child_process_host.h"
+#include "content/public/common/content_switches.h"
 #include "content/public/test/test_browser_thread_bundle.h"
+#include "mojo/public/cpp/bindings/strong_binding.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -33,17 +42,6 @@ void SaveStatusAndCall(ServiceWorkerStatusCode* out,
   callback.Run();
 }
 
-std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params>
-CreateStartParams(int version_id, const GURL& scope, const GURL& script_url) {
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      new EmbeddedWorkerMsg_StartWorker_Params);
-  params->service_worker_version_id = version_id;
-  params->scope = scope;
-  params->script_url = script_url;
-  params->pause_after_download = false;
-  return params;
-}
-
 }  // namespace
 
 class EmbeddedWorkerInstanceTest : public testing::Test,
@@ -97,14 +95,32 @@ class EmbeddedWorkerInstanceTest : public testing::Test,
                                       const GURL& url) {
     ServiceWorkerStatusCode status;
     base::RunLoop run_loop;
-    std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params =
+    std::unique_ptr<EmbeddedWorkerStartParams> params =
         CreateStartParams(id, pattern, url);
-    worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                                run_loop.QuitClosure()));
+    worker->Start(
+        std::move(params), CreateEventDispatcher(),
+        base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
     run_loop.Run();
     return status;
   }
 
+  std::unique_ptr<EmbeddedWorkerStartParams>
+  CreateStartParams(int version_id, const GURL& scope, const GURL& script_url) {
+    std::unique_ptr<EmbeddedWorkerStartParams> params =
+        base::MakeUnique<EmbeddedWorkerStartParams>();
+    params->service_worker_version_id = version_id;
+    params->scope = scope;
+    params->script_url = script_url;
+    params->pause_after_download = false;
+    params->is_installed = false;
+    return params;
+  }
+
+  mojom::ServiceWorkerEventDispatcherRequest CreateEventDispatcher() {
+    dispatchers_.emplace_back();
+    return mojo::MakeRequest(&dispatchers_.back());
+  }
+
   ServiceWorkerContextCore* context() { return helper_->context(); }
 
   EmbeddedWorkerRegistry* embedded_worker_registry() {
@@ -114,6 +130,14 @@ class EmbeddedWorkerInstanceTest : public testing::Test,
 
   IPC::TestSink* ipc_sink() { return helper_->ipc_sink(); }
 
+  std::vector<std::unique_ptr<
+      EmbeddedWorkerTestHelper::MockEmbeddedWorkerInstanceClient>>*
+  mock_instance_clients() {
+    return helper_->mock_instance_clients();
+  }
+
+  std::vector<mojom::ServiceWorkerEventDispatcherPtr> dispatchers_;
+
   TestBrowserThreadBundle thread_bundle_;
   std::unique_ptr<EmbeddedWorkerTestHelper> helper_;
   std::vector<EventLog> events_;
@@ -129,18 +153,20 @@ class StalledInStartWorkerHelper : public EmbeddedWorkerTestHelper {
   StalledInStartWorkerHelper() : EmbeddedWorkerTestHelper(base::FilePath()) {}
   ~StalledInStartWorkerHelper() override{};
 
-  void OnStartWorker(int embedded_worker_id,
-                     int64_t service_worker_version_id,
-                     const GURL& scope,
-                     const GURL& script_url,
-                     bool pause_after_download) override {
+  void OnStartWorker(
+      int embedded_worker_id,
+      int64_t service_worker_version_id,
+      const GURL& scope,
+      const GURL& script_url,
+      bool pause_after_download,
+      mojom::ServiceWorkerEventDispatcherRequest request) override {
     if (force_stall_in_start_) {
       // Do nothing to simulate a stall in the worker process.
       return;
     }
-    EmbeddedWorkerTestHelper::OnStartWorker(embedded_worker_id,
-                                            service_worker_version_id, scope,
-                                            script_url, pause_after_download);
+    EmbeddedWorkerTestHelper::OnStartWorker(
+        embedded_worker_id, service_worker_version_id, scope, script_url,
+        pause_after_download, std::move(request));
   }
 
   void set_force_stall_in_start(bool force_stall_in_start) {
@@ -151,21 +177,11 @@ class StalledInStartWorkerHelper : public EmbeddedWorkerTestHelper {
   bool force_stall_in_start_ = true;
 };
 
-class FailToSendIPCHelper : public EmbeddedWorkerTestHelper {
- public:
-  FailToSendIPCHelper() : EmbeddedWorkerTestHelper(base::FilePath()) {}
-  ~FailToSendIPCHelper() override {}
-
-  bool Send(IPC::Message* message) override {
-    delete message;
-    return false;
-  }
-};
-
 TEST_F(EmbeddedWorkerInstanceTest, StartAndStop) {
   std::unique_ptr<EmbeddedWorkerInstance> worker =
       embedded_worker_registry()->CreateWorker();
   EXPECT_EQ(EmbeddedWorkerStatus::STOPPED, worker->status());
+  worker->AddListener(this);
 
   const int64_t service_worker_version_id = 55L;
   const GURL pattern("http://example.com/");
@@ -178,10 +194,11 @@ TEST_F(EmbeddedWorkerInstanceTest, StartAndStop) {
   // Start should succeed.
   ServiceWorkerStatusCode status;
   base::RunLoop run_loop;
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params =
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
       CreateStartParams(service_worker_version_id, pattern, url);
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              run_loop.QuitClosure()));
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
   EXPECT_EQ(EmbeddedWorkerStatus::STARTING, worker->status());
   run_loop.Run();
   EXPECT_EQ(SERVICE_WORKER_OK, status);
@@ -192,7 +209,7 @@ TEST_F(EmbeddedWorkerInstanceTest, StartAndStop) {
   EXPECT_EQ(helper_->mock_render_process_id(), worker->process_id());
 
   // Stop the worker.
-  EXPECT_EQ(SERVICE_WORKER_OK, worker->Stop());
+  EXPECT_TRUE(worker->Stop());
   EXPECT_EQ(EmbeddedWorkerStatus::STOPPING, worker->status());
   base::RunLoop().RunUntilIdle();
 
@@ -200,11 +217,12 @@ TEST_F(EmbeddedWorkerInstanceTest, StartAndStop) {
   // EmbeddedWorkerTestHelper.
   EXPECT_EQ(EmbeddedWorkerStatus::STOPPED, worker->status());
 
-  // Verify that we've sent two messages to start and terminate the worker.
-  ASSERT_TRUE(
-      ipc_sink()->GetUniqueMessageMatching(EmbeddedWorkerMsg_StartWorker::ID));
-  ASSERT_TRUE(ipc_sink()->GetUniqueMessageMatching(
-      EmbeddedWorkerMsg_StopWorker::ID));
+  // Check if the IPCs are fired in expected order.
+  ASSERT_EQ(4u, events_.size());
+  EXPECT_EQ(PROCESS_ALLOCATED, events_[0].type);
+  EXPECT_EQ(START_WORKER_MESSAGE_SENT, events_[1].type);
+  EXPECT_EQ(STARTED, events_[2].type);
+  EXPECT_EQ(STOPPED, events_[3].type);
 }
 
 // Test that a worker that failed twice will use a new render process
@@ -233,17 +251,18 @@ TEST_F(EmbeddedWorkerInstanceTest, ForceNewProcess) {
     // Start once normally.
     ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_MAX_VALUE;
     base::RunLoop run_loop;
-    std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-        CreateStartParams(service_worker_version_id, pattern, url));
-    worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                                run_loop.QuitClosure()));
+    std::unique_ptr<EmbeddedWorkerStartParams> params =
+        CreateStartParams(service_worker_version_id, pattern, url);
+    worker->Start(
+        std::move(params), CreateEventDispatcher(),
+        base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
     run_loop.Run();
     EXPECT_EQ(SERVICE_WORKER_OK, status);
     EXPECT_EQ(EmbeddedWorkerStatus::RUNNING, worker->status());
     // The worker should be using the default render process.
     EXPECT_EQ(helper_->mock_render_process_id(), worker->process_id());
 
-    EXPECT_EQ(SERVICE_WORKER_OK, worker->Stop());
+    EXPECT_TRUE(worker->Stop());
     base::RunLoop().RunUntilIdle();
   }
 
@@ -257,10 +276,11 @@ TEST_F(EmbeddedWorkerInstanceTest, ForceNewProcess) {
     // Start again.
     ServiceWorkerStatusCode status;
     base::RunLoop run_loop;
-    std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-        CreateStartParams(service_worker_version_id, pattern, url));
-    worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                                run_loop.QuitClosure()));
+    std::unique_ptr<EmbeddedWorkerStartParams> params =
+        CreateStartParams(service_worker_version_id, pattern, url);
+    worker->Start(
+        std::move(params), CreateEventDispatcher(),
+        base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
     EXPECT_EQ(EmbeddedWorkerStatus::STARTING, worker->status());
     run_loop.Run();
     EXPECT_EQ(SERVICE_WORKER_OK, status);
@@ -268,7 +288,7 @@ TEST_F(EmbeddedWorkerInstanceTest, ForceNewProcess) {
     EXPECT_EQ(EmbeddedWorkerStatus::RUNNING, worker->status());
     // The worker should be using the new render process.
     EXPECT_EQ(helper_->new_render_process_id(), worker->process_id());
-    EXPECT_EQ(SERVICE_WORKER_OK, worker->Stop());
+    EXPECT_TRUE(worker->Stop());
     base::RunLoop().RunUntilIdle();
   }
 }
@@ -313,7 +333,7 @@ TEST_F(EmbeddedWorkerInstanceTest, StopWhenDevToolsAttached) {
 
   // Calling Stop() actually stops the worker regardless of whether devtools
   // is attached or not.
-  EXPECT_EQ(SERVICE_WORKER_OK, worker->Stop());
+  EXPECT_TRUE(worker->Stop());
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(EmbeddedWorkerStatus::STOPPED, worker->status());
 }
@@ -337,10 +357,11 @@ TEST_F(EmbeddedWorkerInstanceTest, RemoveWorkerInSharedProcess) {
     // Start worker1.
     ServiceWorkerStatusCode status;
     base::RunLoop run_loop;
-    std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-        CreateStartParams(version_id1, pattern, url));
-    worker1->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                                 run_loop.QuitClosure()));
+    std::unique_ptr<EmbeddedWorkerStartParams> params =
+        CreateStartParams(version_id1, pattern, url);
+    worker1->Start(
+        std::move(params), CreateEventDispatcher(),
+        base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
     run_loop.Run();
     EXPECT_EQ(SERVICE_WORKER_OK, status);
   }
@@ -349,10 +370,11 @@ TEST_F(EmbeddedWorkerInstanceTest, RemoveWorkerInSharedProcess) {
     // Start worker2.
     ServiceWorkerStatusCode status;
     base::RunLoop run_loop;
-    std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-        CreateStartParams(version_id2, pattern, url));
-    worker2->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                                 run_loop.QuitClosure()));
+    std::unique_ptr<EmbeddedWorkerStartParams> params =
+        CreateStartParams(version_id2, pattern, url);
+    worker2->Start(
+        std::move(params), CreateEventDispatcher(),
+        base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
     run_loop.Run();
     EXPECT_EQ(SERVICE_WORKER_OK, status);
   }
@@ -386,10 +408,11 @@ TEST_F(EmbeddedWorkerInstanceTest, DetachDuringProcessAllocation) {
 
   // Run the start worker sequence and detach during process allocation.
   ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_MAX_VALUE;
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, scope, url));
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              base::Bind(&base::DoNothing)));
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, scope, url);
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, base::Bind(&base::DoNothing)));
   worker->Detach();
   base::RunLoop().RunUntilIdle();
 
@@ -418,10 +441,11 @@ TEST_F(EmbeddedWorkerInstanceTest, DetachAfterSendingStartWorkerMessage) {
 
   // Run the start worker sequence until a start worker message is sent.
   ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_MAX_VALUE;
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, scope, url));
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              base::Bind(&base::DoNothing)));
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, scope, url);
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, base::Bind(&base::DoNothing)));
   base::RunLoop().RunUntilIdle();
 
   ASSERT_EQ(2u, events_.size());
@@ -457,10 +481,11 @@ TEST_F(EmbeddedWorkerInstanceTest, StopDuringProcessAllocation) {
   // Stop the start worker sequence before a process is allocated.
   ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_MAX_VALUE;
 
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, scope, url));
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              base::Bind(&base::DoNothing)));
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, scope, url);
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, base::Bind(&base::DoNothing)));
   worker->Stop();
   base::RunLoop().RunUntilIdle();
 
@@ -481,8 +506,9 @@ TEST_F(EmbeddedWorkerInstanceTest, StopDuringProcessAllocation) {
   status = SERVICE_WORKER_ERROR_MAX_VALUE;
   std::unique_ptr<base::RunLoop> run_loop(new base::RunLoop);
   params = CreateStartParams(version_id, scope, url);
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              run_loop->QuitClosure()));
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, run_loop->QuitClosure()));
   run_loop->Run();
 
   EXPECT_EQ(SERVICE_WORKER_OK, status);
@@ -495,11 +521,33 @@ TEST_F(EmbeddedWorkerInstanceTest, StopDuringProcessAllocation) {
   worker->Stop();
 }
 
+class DontReceiveResumeAfterDownloadInstanceClient
+    : public EmbeddedWorkerTestHelper::MockEmbeddedWorkerInstanceClient {
+ public:
+  explicit DontReceiveResumeAfterDownloadInstanceClient(
+      base::WeakPtr<EmbeddedWorkerTestHelper> helper,
+      bool* was_resume_after_download_called)
+      : EmbeddedWorkerTestHelper::MockEmbeddedWorkerInstanceClient(helper),
+        was_resume_after_download_called_(was_resume_after_download_called) {}
+
+ private:
+  void ResumeAfterDownload() override {
+    *was_resume_after_download_called_ = true;
+  }
+
+  bool* const was_resume_after_download_called_;
+};
+
 TEST_F(EmbeddedWorkerInstanceTest, StopDuringPausedAfterDownload) {
   const int64_t version_id = 55L;
   const GURL scope("http://example.com/");
   const GURL url("http://example.com/worker.js");
 
+  bool was_resume_after_download_called = false;
+  helper_->RegisterMockInstanceClient(
+      base::MakeUnique<DontReceiveResumeAfterDownloadInstanceClient>(
+          helper_->AsWeakPtr(), &was_resume_after_download_called));
+
   std::unique_ptr<EmbeddedWorkerInstance> worker =
       embedded_worker_registry()->CreateWorker();
   worker->AddListener(this);
@@ -507,11 +555,12 @@ TEST_F(EmbeddedWorkerInstanceTest, StopDuringPausedAfterDownload) {
   // Run the start worker sequence until pause after download.
   ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_MAX_VALUE;
 
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, scope, url));
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, scope, url);
   params->pause_after_download = true;
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              base::Bind(&base::DoNothing)));
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, base::Bind(&base::DoNothing)));
   base::RunLoop().RunUntilIdle();
 
   // Make the worker stopping and attempt to send a resume after download
@@ -522,8 +571,7 @@ TEST_F(EmbeddedWorkerInstanceTest, StopDuringPausedAfterDownload) {
 
   // The resume after download message should not have been sent.
   EXPECT_EQ(EmbeddedWorkerStatus::STOPPED, worker->status());
-  EXPECT_FALSE(ipc_sink()->GetFirstMessageMatching(
-      EmbeddedWorkerMsg_ResumeAfterDownload::ID));
+  EXPECT_FALSE(was_resume_after_download_called);
 }
 
 TEST_F(EmbeddedWorkerInstanceTest, StopAfterSendingStartWorkerMessage) {
@@ -538,10 +586,11 @@ TEST_F(EmbeddedWorkerInstanceTest, StopAfterSendingStartWorkerMessage) {
 
   // Run the start worker sequence until a start worker message is sent.
   ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_MAX_VALUE;
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, scope, url));
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              base::Bind(&base::DoNothing)));
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, scope, url);
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, base::Bind(&base::DoNothing)));
   base::RunLoop().RunUntilIdle();
 
   ASSERT_EQ(2u, events_.size());
@@ -572,8 +621,9 @@ TEST_F(EmbeddedWorkerInstanceTest, StopAfterSendingStartWorkerMessage) {
   std::unique_ptr<base::RunLoop> run_loop(new base::RunLoop);
 
   params = CreateStartParams(version_id, scope, url);
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              run_loop->QuitClosure()));
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, run_loop->QuitClosure()));
   run_loop->Run();
 
   // The worker should be started.
@@ -600,10 +650,11 @@ TEST_F(EmbeddedWorkerInstanceTest, Detach) {
 
   // Start the worker.
   base::RunLoop run_loop;
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, pattern, url));
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              run_loop.QuitClosure()));
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, pattern, url);
+  worker->Start(
+      std::move(params), CreateEventDispatcher(),
+      base::Bind(&SaveStatusAndCall, &status, run_loop.QuitClosure()));
   run_loop.Run();
 
   // Detach.
@@ -620,33 +671,79 @@ TEST_F(EmbeddedWorkerInstanceTest, Detach) {
 
 // Test for when sending the start IPC failed.
 TEST_F(EmbeddedWorkerInstanceTest, FailToSendStartIPC) {
-  helper_.reset(new FailToSendIPCHelper());
-
   const int64_t version_id = 55L;
   const GURL pattern("http://example.com/");
   const GURL url("http://example.com/worker.js");
 
+  // Let StartWorker fail; mojo IPC fails to connect to a remote interface.
+  helper_->RegisterMockInstanceClient(nullptr);
+
   std::unique_ptr<EmbeddedWorkerInstance> worker =
       embedded_worker_registry()->CreateWorker();
   helper_->SimulateAddProcessToPattern(pattern,
                                        helper_->mock_render_process_id());
-  ServiceWorkerStatusCode status = SERVICE_WORKER_ERROR_FAILED;
   worker->AddListener(this);
 
   // Attempt to start the worker.
-  base::RunLoop run_loop;
-  std::unique_ptr<EmbeddedWorkerMsg_StartWorker_Params> params(
-      CreateStartParams(version_id, pattern, url));
-  worker->Start(std::move(params), base::Bind(&SaveStatusAndCall, &status,
-                                              run_loop.QuitClosure()));
-  run_loop.Run();
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, pattern, url);
+  worker->Start(std::move(params), CreateEventDispatcher(),
+                base::Bind(&ServiceWorkerUtils::NoOpStatusCallback));
+  base::RunLoop().RunUntilIdle();
 
-  // The callback should have run, and we should have got an OnStopped message.
-  EXPECT_EQ(SERVICE_WORKER_ERROR_IPC_FAILED, status);
-  ASSERT_EQ(2u, events_.size());
+  // Worker should handle the failure of binding as detach.
+  ASSERT_EQ(3u, events_.size());
   EXPECT_EQ(PROCESS_ALLOCATED, events_[0].type);
-  EXPECT_EQ(STOPPED, events_[1].type);
-  EXPECT_EQ(EmbeddedWorkerStatus::STARTING, events_[1].status);
+  EXPECT_EQ(START_WORKER_MESSAGE_SENT, events_[1].type);
+  EXPECT_EQ(DETACHED, events_[2].type);
+  EXPECT_EQ(EmbeddedWorkerStatus::STARTING, events_[2].status);
+}
+
+class FailEmbeddedWorkerInstanceClientImpl
+    : public EmbeddedWorkerTestHelper::MockEmbeddedWorkerInstanceClient {
+ public:
+  explicit FailEmbeddedWorkerInstanceClientImpl(
+      base::WeakPtr<EmbeddedWorkerTestHelper> helper)
+      : EmbeddedWorkerTestHelper::MockEmbeddedWorkerInstanceClient(helper) {}
+
+ private:
+  void StartWorker(
+      const EmbeddedWorkerStartParams& /* unused */,
+      mojom::ServiceWorkerEventDispatcherRequest /* unused */) override {
+    helper_->mock_instance_clients()->clear();
+  }
+};
+
+TEST_F(EmbeddedWorkerInstanceTest, RemoveRemoteInterface) {
+  const int64_t version_id = 55L;
+  const GURL pattern("http://example.com/");
+  const GURL url("http://example.com/worker.js");
+
+  // Let StartWorker fail; binding is discarded in the middle of IPC
+  helper_->RegisterMockInstanceClient(
+      base::MakeUnique<FailEmbeddedWorkerInstanceClientImpl>(
+          helper_->AsWeakPtr()));
+  ASSERT_EQ(mock_instance_clients()->size(), 1UL);
+
+  std::unique_ptr<EmbeddedWorkerInstance> worker =
+      embedded_worker_registry()->CreateWorker();
+  helper_->SimulateAddProcessToPattern(pattern,
+                                       helper_->mock_render_process_id());
+  worker->AddListener(this);
+
+  // Attempt to start the worker.
+  std::unique_ptr<EmbeddedWorkerStartParams> params =
+      CreateStartParams(version_id, pattern, url);
+  worker->Start(std::move(params), CreateEventDispatcher(),
+                base::Bind(&ServiceWorkerUtils::NoOpStatusCallback));
+  base::RunLoop().RunUntilIdle();
+
+  // Worker should handle the sudden shutdown as detach.
+  ASSERT_EQ(3u, events_.size());
+  EXPECT_EQ(PROCESS_ALLOCATED, events_[0].type);
+  EXPECT_EQ(START_WORKER_MESSAGE_SENT, events_[1].type);
+  EXPECT_EQ(DETACHED, events_[2].type);
+  EXPECT_EQ(EmbeddedWorkerStatus::STARTING, events_[2].status);
 }
 
 }  // namespace content
