@@ -13,27 +13,34 @@
 #include <queue>
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/callback.h"
-#include "base/containers/scoped_ptr_hash_map.h"
 #include "base/gtest_prod_util.h"
 #include "base/id_map.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
 #include "base/observer_list.h"
+#include "base/optional.h"
 #include "base/threading/thread_task_runner_handle.h"
+#include "base/time/tick_clock.h"
+#include "base/time/time.h"
 #include "base/timer/timer.h"
 #include "content/browser/service_worker/embedded_worker_instance.h"
 #include "content/browser/service_worker/embedded_worker_status.h"
+#include "content/browser/service_worker/service_worker_context_request_handler.h"
 #include "content/browser/service_worker/service_worker_metrics.h"
 #include "content/browser/service_worker/service_worker_script_cache_map.h"
 #include "content/common/content_export.h"
+#include "content/common/origin_trials/trial_token_validator.h"
+#include "content/common/service_worker/service_worker_event_dispatcher.mojom.h"
 #include "content/common/service_worker/service_worker_status_code.h"
 #include "content/common/service_worker/service_worker_types.h"
 #include "ipc/ipc_message.h"
-#include "services/shell/public/cpp/interface_provider.h"
+#include "mojo/public/cpp/bindings/interface_ptr.h"
+#include "services/service_manager/public/cpp/interface_provider.h"
 #include "third_party/WebKit/public/platform/modules/serviceworker/WebServiceWorkerEventResult.h"
 #include "url/gurl.h"
 #include "url/origin.h"
@@ -49,7 +56,6 @@ class HttpResponseInfo;
 
 namespace content {
 
-class EmbeddedWorkerRegistry;
 class ServiceWorkerContextCore;
 class ServiceWorkerProviderHost;
 class ServiceWorkerRegistration;
@@ -87,10 +93,26 @@ class CONTENT_EXPORT ServiceWorkerVersion
                          // timed out.
   };
 
+  // Whether the version has fetch handlers or not.
+  enum class FetchHandlerExistence {
+    UNKNOWN,  // This version is a new version and not installed yet.
+    EXISTS,
+    DOES_NOT_EXIST,
+  };
+
+  // Navigation Preload support status of the service worker.
+  enum class NavigationPreloadSupportStatus {
+    SUPPORTED,
+    NOT_SUPPORTED_FIELD_TRIAL_STOPPED,
+    NOT_SUPPORTED_DISABLED_BY_COMMAND_LINE,
+    NOT_SUPPORTED_NO_VALID_ORIGIN_TRIAL_TOKEN,
+  };
+
   class Listener {
    public:
     virtual void OnRunningStateChanged(ServiceWorkerVersion* version) {}
     virtual void OnVersionStateChanged(ServiceWorkerVersion* version) {}
+    virtual void OnDevToolsRoutingIdChanged(ServiceWorkerVersion* version) {}
     virtual void OnMainScriptHttpResponseInfoSet(
         ServiceWorkerVersion* version) {}
     virtual void OnErrorReported(ServiceWorkerVersion* version,
@@ -131,10 +153,17 @@ class CONTENT_EXPORT ServiceWorkerVersion
   }
   ServiceWorkerVersionInfo GetInfo();
   Status status() const { return status_; }
-  bool has_fetch_handler() const { return has_fetch_handler_; }
-  void set_has_fetch_handler(bool has_fetch_handler) {
-    has_fetch_handler_ = has_fetch_handler;
+
+  // This status is set to EXISTS or DOES_NOT_EXIST when the install event has
+  // been executed in a new version or when an installed version is loaded from
+  // the storage. When a new version is not installed yet, it is  UNKNOW.
+  FetchHandlerExistence fetch_handler_existence() const {
+    return fetch_handler_existence_;
   }
+  // This also updates |site_for_uma_| when it was Site::OTHER.
+  void set_fetch_handler_existence(FetchHandlerExistence existence);
+
+  bool should_exclude_from_uma() const { return should_exclude_from_uma_; }
 
   const std::vector<GURL>& foreign_fetch_scopes() const {
     return foreign_fetch_scopes_;
@@ -150,6 +179,19 @@ class CONTENT_EXPORT ServiceWorkerVersion
     foreign_fetch_origins_ = origins;
   }
 
+  // Meaningful only if this version is active.
+  const NavigationPreloadState& navigation_preload_state() const {
+    DCHECK(status_ == ACTIVATING || status_ == ACTIVATED) << status_;
+    return navigation_preload_state_;
+  }
+  // Only intended for use by ServiceWorkerRegistration. Generally use
+  // ServiceWorkerRegistration::EnableNavigationPreload or
+  // ServiceWorkerRegistration::SetNavigationPreloadHeader instead of this
+  // function.
+  void SetNavigationPreloadState(const NavigationPreloadState& state);
+
+  ServiceWorkerMetrics::Site site_for_uma() const { return site_for_uma_; }
+
   // This sets the new status and also run status change callbacks
   // if there're any (see RegisterStatusChangeCallback).
   void SetStatus(Status status);
@@ -214,23 +256,32 @@ class CONTENT_EXPORT ServiceWorkerVersion
                                     const base::TimeDelta& timeout,
                                     TimeoutBehavior timeout_behavior);
 
+  // Starts a request of type EventType::EXTERNAL_REQUEST.
+  // Provides a mechanism to external clients to keep the worker running.
+  // |request_uuid| is a GUID for clients to identify the request.
+  // Returns true if the request was successfully scheduled to starrt.
+  bool StartExternalRequest(const std::string& request_uuid);
+
   // Informs ServiceWorkerVersion that an event has finished being dispatched.
   // Returns false if no pending requests with the provided id exist, for
   // example if the request has already timed out.
   // Pass the result of the event to |was_handled|, which is used to record
   // statistics based on the event status.
   // TODO(mek): Use something other than a bool for event status.
-  bool FinishRequest(int request_id, bool was_handled);
+  bool FinishRequest(int request_id,
+                     bool was_handled,
+                     base::Time dispatch_event_time);
 
-  // Connects to a specific mojo service exposed by the (running) service
-  // worker. If a connection to a service for the same Interface already exists
-  // this will return that existing connection. The |request_id| must be a value
-  // previously returned by StartRequest. If the connection to the service
-  // fails or closes before the request finished, the error callback associated
-  // with |request_id| is called.
-  // Only call GetMojoServiceForRequest once for a specific |request_id|.
-  template <typename Interface>
-  base::WeakPtr<Interface> GetMojoServiceForRequest(int request_id);
+  // Finishes an external request that was started by StartExternalRequest().
+  // Returns false if there was an error finishing the request: e.g. the request
+  // was not found or the worker already terminated.
+  bool FinishExternalRequest(const std::string& request_uuid);
+
+  // This must be called when the worker is running.
+  mojom::ServiceWorkerEventDispatcher* event_dispatcher() {
+    DCHECK(event_dispatcher_.is_bound());
+    return event_dispatcher_.get();
+  }
 
   // Dispatches an event. If dispatching the event fails, all of the error
   // callbacks that were associated with |request_ids| via StartRequest are
@@ -330,6 +381,19 @@ class CONTENT_EXPORT ServiceWorkerVersion
     pause_after_download_ = pause_after_download;
   }
 
+  // Returns nullptr if the main script is not loaded yet and:
+  //  1) The worker is a new one.
+  //  OR
+  //  2) The worker is an existing one but the entry in ServiceWorkerDatabase
+  //     was written by old version of Chrome (< M56), so |origin_trial_tokens|
+  //     wasn't set in the entry.
+  const TrialTokenValidator::FeatureToTokensMap* origin_trial_tokens() const {
+    return origin_trial_tokens_.get();
+  }
+  // Set valid tokens in |tokens|. Invalid tokens in |tokens| are ignored.
+  void SetValidOriginTrialTokens(
+      const TrialTokenValidator::FeatureToTokensMap& tokens);
+
   void SetDevToolsAttached(bool attached);
 
   // Sets the HttpResponseInfo used to load the main script.
@@ -342,10 +406,70 @@ class CONTENT_EXPORT ServiceWorkerVersion
   // Simulate ping timeout. Should be used for tests-only.
   void SimulatePingTimeoutForTesting();
 
+  // Used to allow tests to change time for testing.
+  void SetTickClockForTesting(std::unique_ptr<base::TickClock> tick_clock);
+
   // Returns true if the service worker has work to do: it has pending
   // requests, in-progress streaming URLRequestJobs, or pending start callbacks.
   bool HasWork() const;
 
+  // Returns the number of pending external request count of this worker.
+  size_t GetExternalRequestCountForTest() const {
+    return external_request_uuid_to_request_id_.size();
+  }
+
+  // Returns the amount of time left until the request with the latest
+  // expiration time expires.
+  base::TimeDelta remaining_timeout() const {
+    return max_request_expiration_time_ - tick_clock_->NowTicks();
+  }
+
+  // Callback function for simple events dispatched through mojo interface
+  // mojom::ServiceWorkerEventDispatcher, once all simple events got dispatched
+  // through mojo, OnSimpleEventResponse function could be removed.
+  void OnSimpleEventFinished(int request_id,
+                             ServiceWorkerStatusCode status,
+                             base::Time dispatch_event_time);
+
+  void NotifyMainScriptRequestHandlerCreated();
+  void NotifyMainScriptJobCreated(
+      ServiceWorkerContextRequestHandler::CreateJobStatus status);
+
+  // Returns the Navigation Preload support status of the service worker.
+  //  - Origin Trial: Have an effective token.
+  //                                 Command line
+  //                             Default  Enable  Disabled
+  //                   Default      A       A        B2
+  //      Field trial  Enabled      A       A        B2
+  //                   Disabled     B1      A        B2
+  //
+  //  - Origin Trial: No token.
+  //                                 Command line
+  //                             Default  Enable  Disabled
+  //                   Default      C       A        C
+  //      Field trial  Enabled      C       A        C
+  //                   Disabled     C       A        C
+  //
+  //   * A  = SUPPORTED
+  //     B1 = NOT_SUPPORTED_FIELD_TRIAL_STOPPED
+  //     B2 = NOT_SUPPORTED_DISABLED_BY_COMMAND_LINE
+  //     C  = NOT_SUPPORTED_NO_VALID_ORIGIN_TRIAL_TOKEN
+  //
+  // There are three types of behaviors:
+  //  - A: Navigation Preload related methods and attributes are available in JS
+  //       and work correctly.
+  //  - B: Navigation Preload related methods and attributes are available in
+  //       JS. But NavigationPreloadManager's enable, disable and setHeaderValue
+  //       methods always return a rejected promise. And FetchEvent's
+  //       preloadResponse attribute returns a promise which always resolve with
+  //       undefined.
+  //  - C: Navigation Preload related methods and attributes are not available
+  //       in JS.
+  // This method returns SUPPORTED only for A case.
+  // blink::OriginTrials::serviceWorkerNavigationPreloadEnabled() returns true
+  // for both A and B case. So the methods and attributes are available in JS.
+  NavigationPreloadSupportStatus GetNavigationPreloadSupportStatus() const;
+
  private:
   friend class base::RefCounted<ServiceWorkerVersion>;
   friend class ServiceWorkerMetrics;
@@ -357,6 +481,8 @@ class CONTENT_EXPORT ServiceWorkerVersion
 
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerControlleeRequestHandlerTest,
                            ActivateWaitingVersion);
+  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerControlleeRequestHandlerTest,
+                           FallbackWithNoFetchHandler);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, IdleTimeout);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, SetDevToolsAttached);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, StaleUpdate_FreshWorker);
@@ -366,7 +492,7 @@ class CONTENT_EXPORT ServiceWorkerVersion
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, StaleUpdate_RunningWorker);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest,
                            StaleUpdate_DoNotDeferTimer);
-  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, RequestTimeout);
+  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerRequestTimeoutTest, RequestTimeout);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerFailToStartTest, Timeout);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionBrowserTest,
                            TimeoutStartingWorker);
@@ -376,9 +502,9 @@ class CONTENT_EXPORT ServiceWorkerVersion
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerStallInStoppingTest, DetachThenRestart);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest,
                            RegisterForeignFetchScopes);
+  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, RequestNowTimeout);
+  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, RequestNowTimeoutKill);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, RequestCustomizedTimeout);
-  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest,
-                           RequestCustomizedTimeoutKill);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerVersionTest, MixedRequestTimeouts);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerURLRequestJobTest, EarlyResponse);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerURLRequestJobTest, CancelRequest);
@@ -402,60 +528,31 @@ class CONTENT_EXPORT ServiceWorkerVersion
     TimeoutBehavior timeout_behavior;
   };
 
-  template <typename CallbackType>
   struct PendingRequest {
-    PendingRequest(const CallbackType& callback,
-                   const base::TimeTicks& time,
+    PendingRequest(const StatusCallback& error_callback,
+                   base::Time time,
+                   const base::TimeTicks& time_ticks,
                    ServiceWorkerMetrics::EventType event_type);
-    ~PendingRequest() {}
+    ~PendingRequest();
 
-    CallbackType callback;
-    base::TimeTicks start_time;
+    // ------------------------------------------------------------------------
+    // For all requests. Set by StartRequest.
+    // ------------------------------------------------------------------------
+    StatusCallback error_callback;
+    base::Time start_time;
+    base::TimeTicks start_time_ticks;
     ServiceWorkerMetrics::EventType event_type;
-    // Name of the mojo service this request is associated with. Used to call
-    // the callback when a connection closes with outstanding requests.
-    // Compared as pointer, so should only contain static strings. Typically
-    // this would be Interface::Name_ for some mojo interface.
-    const char* mojo_service = nullptr;
+
+    // ------------------------------------------------------------------------
+    // For IPC message requests.
+    // ------------------------------------------------------------------------
+    // Set by RegisterRequestCallback. Receives IPC responses to the request via
+    // OnMessageReceived.
     std::unique_ptr<EmbeddedWorkerInstance::Listener> listener;
+    // True if an IPC message was sent to dispatch the event for this request.
     bool is_dispatched = false;
   };
 
-  // Base class to enable storing a list of mojo interface pointers for
-  // arbitrary interfaces. The destructor is also responsible for calling the
-  // error callbacks for any outstanding requests using this service.
-  class CONTENT_EXPORT BaseMojoServiceWrapper {
-   public:
-    BaseMojoServiceWrapper(ServiceWorkerVersion* worker,
-                           const char* service_name);
-    virtual ~BaseMojoServiceWrapper();
-
-   private:
-    ServiceWorkerVersion* worker_;
-    const char* service_name_;
-
-    DISALLOW_COPY_AND_ASSIGN(BaseMojoServiceWrapper);
-  };
-
-  // Wrapper around a mojo::InterfacePtr, which passes out WeakPtr's to the
-  // interface.
-  template <typename Interface>
-  class MojoServiceWrapper : public BaseMojoServiceWrapper {
-   public:
-    MojoServiceWrapper(ServiceWorkerVersion* worker,
-                       mojo::InterfacePtr<Interface> interface)
-        : BaseMojoServiceWrapper(worker, Interface::Name_),
-          interface_(std::move(interface)),
-          weak_ptr_factory_(interface_.get()) {}
-
-    base::WeakPtr<Interface> GetWeakPtr() {
-      return weak_ptr_factory_.GetWeakPtr();
-    }
-
-   private:
-    mojo::InterfacePtr<Interface> interface_;
-    base::WeakPtrFactory<Interface> weak_ptr_factory_;
-  };
 
   typedef ServiceWorkerVersion self;
   using ServiceWorkerClients = std::vector<ServiceWorkerClientInfo>;
@@ -506,6 +603,12 @@ class CONTENT_EXPORT ServiceWorkerVersion
 
   ~ServiceWorkerVersion() override;
 
+  // The following methods all rely on the internal |tick_clock_| for the
+  // current time.
+  void RestartTick(base::TimeTicks* time) const;
+  bool RequestExpired(const base::TimeTicks& expiration) const;
+  base::TimeDelta GetTickDuration(const base::TimeTicks& time) const;
+
   // EmbeddedWorkerInstance::Listener overrides:
   void OnThreadStarted() override;
   void OnStarting() override;
@@ -515,6 +618,7 @@ class CONTENT_EXPORT ServiceWorkerVersion
   void OnDetached(EmbeddedWorkerStatus old_status) override;
   void OnScriptLoaded() override;
   void OnScriptLoadFailed() override;
+  void OnRegisteredToDevToolsManager() override;
   void OnReportException(const base::string16& error_message,
                          int line_number,
                          int column_number,
@@ -537,8 +641,13 @@ class CONTENT_EXPORT ServiceWorkerVersion
   void OnGetClients(int request_id,
                     const ServiceWorkerClientQueryOptions& options);
 
+  // Receiver function of responses of simple events dispatched through chromium
+  // IPCs. This is internally the same with OnSimpleEventFinished and will be
+  // replaced with OnSimpleEventFinished after all of simple events are
+  // dispatched via mojo.
   void OnSimpleEventResponse(int request_id,
-                             blink::WebServiceWorkerEventResult result);
+                             blink::WebServiceWorkerEventResult result,
+                             base::Time dispatch_event_time);
   void OnOpenWindow(int request_id, GURL url);
   void OnOpenWindowFinished(int request_id,
                             ServiceWorkerStatusCode status,
@@ -575,7 +684,7 @@ class CONTENT_EXPORT ServiceWorkerVersion
       bool is_browser_startup_complete,
       const StatusCallback& callback,
       ServiceWorkerStatusCode status,
-      const scoped_refptr<ServiceWorkerRegistration>& registration);
+      scoped_refptr<ServiceWorkerRegistration> registration);
   void StartWorkerInternal();
 
   void DidSkipWaiting(int request_id);
@@ -623,25 +732,36 @@ class CONTENT_EXPORT ServiceWorkerVersion
 
   void FoundRegistrationForUpdate(
       ServiceWorkerStatusCode status,
-      const scoped_refptr<ServiceWorkerRegistration>& registration);
+      scoped_refptr<ServiceWorkerRegistration> registration);
 
   void OnStoppedInternal(EmbeddedWorkerStatus old_status);
 
-  // Called when the remote side of a connection to a mojo service is lost.
-  void OnMojoConnectionError(const char* service_name);
-
   // Called at the beginning of each Dispatch*Event function: records
   // the time elapsed since idle (generally the time since the previous
   // event ended).
   void OnBeginEvent();
 
+  // Resets |start_worker_first_purpose_| and fires and clears all start
+  // callbacks.
+  void FinishStartWorker(ServiceWorkerStatusCode status);
+
+  // Removes any pending external request that has GUID of |request_uuid|.
+  void CleanUpExternalRequest(const std::string& request_uuid,
+                              ServiceWorkerStatusCode status);
+
   const int64_t version_id_;
   const int64_t registration_id_;
   const GURL script_url_;
   const GURL scope_;
   std::vector<GURL> foreign_fetch_scopes_;
   std::vector<url::Origin> foreign_fetch_origins_;
-  bool has_fetch_handler_ = true;
+  FetchHandlerExistence fetch_handler_existence_;
+  // The source of truth for navigation preload state is the
+  // ServiceWorkerRegistration. |navigation_preload_state_| is essentially a
+  // cached value because it must be looked up quickly and a live registration
+  // doesn't necessarily exist whenever there is a live version.
+  NavigationPreloadState navigation_preload_state_;
+  ServiceWorkerMetrics::Site site_for_uma_;
 
   Status status_ = NEW;
   std::unique_ptr<EmbeddedWorkerInstance> embedded_worker_;
@@ -649,17 +769,17 @@ class CONTENT_EXPORT ServiceWorkerVersion
   std::vector<StatusCallback> stop_callbacks_;
   std::vector<base::Closure> status_change_callbacks_;
 
-  // Message callbacks. (Update HasInflightRequests() too when you update this
-  // list.)
-  IDMap<PendingRequest<StatusCallback>, IDMapOwnPointer> custom_requests_;
+  // Holds in-flight requests, including requests due to outstanding push,
+  // fetch, sync, etc. events.
+  IDMap<std::unique_ptr<PendingRequest>> pending_requests_;
 
-  // Stores all open connections to mojo services. Maps the service name to
-  // the actual interface pointer. When a connection is closed it is removed
-  // from this map.
-  // mojo_services_[Interface::Name_] is assumed to always contain a
-  // MojoServiceWrapper<Interface> instance.
-  base::ScopedPtrHashMap<const char*, std::unique_ptr<BaseMojoServiceWrapper>>
-      mojo_services_;
+  // Container for pending external requests for this service worker.
+  // (key, value): (request uuid, request id).
+  using RequestUUIDToRequestIDMap = std::map<std::string, int>;
+  RequestUUIDToRequestIDMap external_request_uuid_to_request_id_;
+
+  // Connected to ServiceWorkerContextClient while the worker is running.
+  mojom::ServiceWorkerEventDispatcherPtr event_dispatcher_;
 
   std::set<const ServiceWorkerURLRequestJob*> streaming_url_request_jobs_;
 
@@ -683,13 +803,18 @@ class CONTENT_EXPORT ServiceWorkerVersion
   // to update once the worker stops, but will also update if it stays alive too
   // long.
   base::TimeTicks stale_time_;
+  // The latest expiration time of all requests that have ever been started. In
+  // particular this is not just the maximum of the expiration times of all
+  // currently existing requests, but also takes into account the former
+  // expiration times of finished requests.
+  base::TimeTicks max_request_expiration_time_;
 
-  // New requests are added to |requests_| along with their entry in a callback
-  // map. Requests are sorted by their expiration time (soonest to expire on top
-  // of the priority queue). The timeout timer periodically checks |requests_|
-  // for entries that should time out or have already been fulfilled (i.e.,
-  // removed from the callback map).
-  RequestInfoPriorityQueue requests_;
+  // Keeps track of requests for timeout purposes. Requests are sorted by
+  // their expiration time (soonest to expire on top of the priority queue). The
+  // timeout timer periodically checks |timeout_queue_| for entries that should
+  // time out or have already been fulfilled (i.e., removed from
+  // |pending_requests_|).
+  RequestInfoPriorityQueue timeout_queue_;
 
   bool skip_waiting_ = false;
   bool skip_recording_startup_time_ = false;
@@ -701,46 +826,34 @@ class CONTENT_EXPORT ServiceWorkerVersion
   std::vector<int> pending_skip_waiting_requests_;
   std::unique_ptr<net::HttpResponseInfo> main_script_http_info_;
 
+  std::unique_ptr<TrialTokenValidator::FeatureToTokensMap> origin_trial_tokens_;
+
   // If not OK, the reason that StartWorker failed. Used for
   // running |start_callbacks_|.
   ServiceWorkerStatusCode start_worker_status_ = SERVICE_WORKER_OK;
 
+  // The clock used to vend tick time.
+  std::unique_ptr<base::TickClock> tick_clock_;
+
   std::unique_ptr<PingController> ping_controller_;
   std::unique_ptr<Metrics> metrics_;
   const bool should_exclude_from_uma_ = false;
 
   bool stop_when_devtools_detached_ = false;
 
+  // Keeps the first purpose of starting the worker for UMA. Cleared in
+  // FinishStartWorker().
+  base::Optional<ServiceWorkerMetrics::EventType> start_worker_first_purpose_;
+
+  bool main_script_request_handler_created_ = false;
+  ServiceWorkerContextRequestHandler::CreateJobStatus main_script_job_created_ =
+      ServiceWorkerContextRequestHandler::CreateJobStatus::UNINITIALIZED;
+
   base::WeakPtrFactory<ServiceWorkerVersion> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(ServiceWorkerVersion);
 };
 
-template <typename Interface>
-base::WeakPtr<Interface> ServiceWorkerVersion::GetMojoServiceForRequest(
-    int request_id) {
-  DCHECK_EQ(EmbeddedWorkerStatus::RUNNING, running_status());
-  PendingRequest<StatusCallback>* request = custom_requests_.Lookup(request_id);
-  DCHECK(request) << "Invalid request id";
-  DCHECK(!request->mojo_service)
-      << "Request is already associated with a mojo service";
-
-  MojoServiceWrapper<Interface>* service =
-      static_cast<MojoServiceWrapper<Interface>*>(
-          mojo_services_.get(Interface::Name_));
-  if (!service) {
-    mojo::InterfacePtr<Interface> interface;
-    embedded_worker_->GetRemoteInterfaces()->GetInterface(&interface);
-    interface.set_connection_error_handler(
-        base::Bind(&ServiceWorkerVersion::OnMojoConnectionError,
-                   weak_factory_.GetWeakPtr(), Interface::Name_));
-    service = new MojoServiceWrapper<Interface>(this, std::move(interface));
-    mojo_services_.add(Interface::Name_, base::WrapUnique(service));
-  }
-  request->mojo_service = Interface::Name_;
-  return service->GetWeakPtr();
-}
-
 template <typename ResponseMessage>
 void ServiceWorkerVersion::DispatchSimpleEvent(int request_id,
                                                const IPC::Message& message) {
@@ -752,7 +865,7 @@ template <typename ResponseMessage, typename ResponseCallbackType>
 void ServiceWorkerVersion::RegisterRequestCallback(
     int request_id,
     const ResponseCallbackType& callback) {
-  PendingRequest<StatusCallback>* request = custom_requests_.Lookup(request_id);
+  PendingRequest* request = pending_requests_.Lookup(request_id);
   DCHECK(request) << "Invalid request id";
   DCHECK(!request->listener) << "Callback was already registered";
   DCHECK(!request->is_dispatched) << "Request already dispatched an IPC event";
