@@ -4,9 +4,11 @@
 
 package org.chromium.content.browser.input;
 
+import android.annotation.SuppressLint;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.Looper;
+import android.text.TextUtils;
 import android.view.KeyCharacterMap;
 import android.view.KeyEvent;
 import android.view.View;
@@ -35,8 +37,7 @@ import java.util.concurrent.LinkedBlockingQueue;
  * so 'extends' here should have no functional effect at all. See crbug.com/616334 for more
  * details.
  */
-public class ThreadedInputConnection extends BaseInputConnection
-        implements ChromiumBaseInputConnection {
+class ThreadedInputConnection extends BaseInputConnection implements ChromiumBaseInputConnection {
     private static final String TAG = "cr_Ime";
     private static final boolean DEBUG_LOGS = false;
 
@@ -84,7 +85,7 @@ public class ThreadedInputConnection extends BaseInputConnection
     private final Runnable mFinishComposingTextRunnable = new Runnable() {
         @Override
         public void run() {
-            mImeAdapter.finishComposingText();
+            finishComposingTextOnUiThread();
         }
     };
 
@@ -96,6 +97,7 @@ public class ThreadedInputConnection extends BaseInputConnection
     // a bunch of new objects for each key stroke.
     private final BlockingQueue<TextInputState> mQueue = new LinkedBlockingQueue<>();
     private int mPendingAccent;
+    private TextInputState mCachedTextInputState;
 
     ThreadedInputConnection(View view, ImeAdapter imeAdapter, Handler handler) {
         super(view, true);
@@ -112,18 +114,21 @@ public class ThreadedInputConnection extends BaseInputConnection
     }
 
     @Override
-    public void updateStateOnUiThread(final String text, final int selectionStart,
-            final int selectionEnd, final int compositionStart, final int compositionEnd,
-            boolean singleLine, final boolean isNonImeChange) {
+    public void updateStateOnUiThread(String text, final int selectionStart, final int selectionEnd,
+            final int compositionStart, final int compositionEnd, boolean singleLine,
+            final boolean replyToRequest) {
         ImeUtils.checkOnUiThread();
 
-        final TextInputState newState =
-                new TextInputState(text, new Range(selectionStart, selectionEnd),
-                        new Range(compositionStart, compositionEnd), singleLine, !isNonImeChange);
-        if (DEBUG_LOGS) Log.w(TAG, "updateState: %s", newState);
+        // crbug.com/663880: Non-breaking spaces can cause the IME to get confused. Replace with
+        // normal spaces.
+        text = text.replace('\u00A0', ' ');
 
-        addToQueueOnUiThread(newState);
-        if (isNonImeChange) {
+        mCachedTextInputState = new TextInputState(text, new Range(selectionStart, selectionEnd),
+                new Range(compositionStart, compositionEnd), singleLine, replyToRequest);
+        if (DEBUG_LOGS) Log.w(TAG, "updateState: %s", mCachedTextInputState);
+
+        addToQueueOnUiThread(mCachedTextInputState);
+        if (!replyToRequest) {
             mHandler.post(mProcessPendingInputStatesRunnable);
         }
     }
@@ -206,6 +211,12 @@ public class ThreadedInputConnection extends BaseInputConnection
 
     private TextInputState requestAndWaitForTextInputState() {
         if (DEBUG_LOGS) Log.w(TAG, "requestAndWaitForTextInputState");
+        if (runningOnUiThread()) {
+            Log.w(TAG, "InputConnection API is not called on IME thread. Returning cached result.");
+            // Returning cached result is a workaround for existing webview apps. (crbug.com/643477)
+            return mCachedTextInputState;
+        }
+        assertOnImeThread();
         ThreadUtils.postOnUiThread(mRequestTextInputStateUpdate);
         return blockAndGetStateUpdate();
     }
@@ -227,6 +238,11 @@ public class ThreadedInputConnection extends BaseInputConnection
         return mQueue;
     }
 
+    @VisibleForTesting
+    protected boolean runningOnUiThread() {
+        return ThreadUtils.runningOnUiThread();
+    }
+
     private void assertOnImeThread() {
         ImeUtils.checkCondition(mHandler.getLooper() == Looper.myLooper());
     }
@@ -253,7 +269,7 @@ public class ThreadedInputConnection extends BaseInputConnection
             if (state.shouldUnblock()) {
                 if (DEBUG_LOGS) Log.w(TAG, "blockAndGetStateUpdate: unblocked");
                 return null;
-            } else if (state.fromIme()) {
+            } else if (state.replyToRequest()) {
                 if (shouldUpdateSelection) updateSelection(state);
                 if (DEBUG_LOGS) Log.w(TAG, "blockAndGetStateUpdate done: %d", mQueue.size());
                 return state;
@@ -274,6 +290,7 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean setComposingText(final CharSequence text, final int newCursorPosition) {
         if (DEBUG_LOGS) Log.w(TAG, "setComposingText [%s] [%d]", text, newCursorPosition);
+        if (text == null) return false;
         return updateComposingText(text, newCursorPosition, false);
     }
 
@@ -283,32 +300,36 @@ public class ThreadedInputConnection extends BaseInputConnection
     @VisibleForTesting
     public boolean updateComposingText(
             final CharSequence text, final int newCursorPosition, final boolean isPendingAccent) {
-        final int accentToSend =
-                isPendingAccent ? (mPendingAccent | KeyCharacterMap.COMBINING_ACCENT) : 0;
-        assertOnImeThread();
-        cancelCombiningAccent();
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
-                mImeAdapter.sendCompositionToNative(text, newCursorPosition, false, accentToSend);
+                updateComposingTextOnUiThread(text, newCursorPosition, isPendingAccent);
             }
         });
         notifyUserAction();
         return true;
     }
 
+    private void updateComposingTextOnUiThread(
+            CharSequence text, int newCursorPosition, boolean isPendingAccent) {
+        int accentToSend =
+                isPendingAccent ? (mPendingAccent | KeyCharacterMap.COMBINING_ACCENT) : 0;
+        cancelCombiningAccentOnUiThread();
+        mImeAdapter.sendCompositionToNative(text, newCursorPosition, false, accentToSend);
+    }
+
     /**
      * @see InputConnection#commitText(java.lang.CharSequence, int)
      */
     @Override
     public boolean commitText(final CharSequence text, final int newCursorPosition) {
         if (DEBUG_LOGS) Log.w(TAG, "commitText [%s] [%d]", text, newCursorPosition);
-        assertOnImeThread();
-        cancelCombiningAccent();
+        if (text == null) return false;
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
-                mImeAdapter.sendCompositionToNative(text, newCursorPosition, text.length() > 0, 0);
+                cancelCombiningAccentOnUiThread();
+                mImeAdapter.sendCompositionToNative(text, newCursorPosition, true, 0);
             }
         });
         notifyUserAction();
@@ -321,7 +342,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean performEditorAction(final int actionCode) {
         if (DEBUG_LOGS) Log.w(TAG, "performEditorAction [%d]", actionCode);
-        assertOnImeThread();
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
@@ -337,7 +357,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean performContextMenuAction(final int id) {
         if (DEBUG_LOGS) Log.w(TAG, "performContextMenuAction [%d]", id);
-        assertOnImeThread();
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
@@ -353,7 +372,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public ExtractedText getExtractedText(ExtractedTextRequest request, int flags) {
         if (DEBUG_LOGS) Log.w(TAG, "getExtractedText");
-        assertOnImeThread();
         TextInputState textInputState = requestAndWaitForTextInputState();
         if (textInputState == null) return null;
         ExtractedText extractedText = new ExtractedText();
@@ -370,6 +388,7 @@ public class ThreadedInputConnection extends BaseInputConnection
      */
     @Override
     public boolean beginBatchEdit() {
+        assertOnImeThread();
         if (DEBUG_LOGS) Log.w(TAG, "beginBatchEdit [%b]", (mNumNestedBatchEdits == 0));
         assertOnImeThread();
         mNumNestedBatchEdits++;
@@ -397,13 +416,12 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean deleteSurroundingText(final int beforeLength, final int afterLength) {
         if (DEBUG_LOGS) Log.w(TAG, "deleteSurroundingText [%d %d]", beforeLength, afterLength);
-        assertOnImeThread();
-        if (mPendingAccent != 0) {
-            finishComposingText();
-        }
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
+                if (mPendingAccent != 0) {
+                    finishComposingTextOnUiThread();
+                }
                 mImeAdapter.deleteSurroundingText(beforeLength, afterLength);
             }
         });
@@ -424,13 +442,10 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean sendKeyEvent(final KeyEvent event) {
         if (DEBUG_LOGS) Log.w(TAG, "sendKeyEvent [%d %d]", event.getAction(), event.getKeyCode());
-        assertOnImeThread();
-
-        if (handleCombiningAccent(event)) return true;
-
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
+                if (handleCombiningAccentOnUiThread(event)) return;
                 mImeAdapter.sendKeyEvent(event);
             }
         });
@@ -438,7 +453,7 @@ public class ThreadedInputConnection extends BaseInputConnection
         return true;
     }
 
-    private boolean handleCombiningAccent(final KeyEvent event) {
+    private boolean handleCombiningAccentOnUiThread(final KeyEvent event) {
         // TODO(changwan): this will break the current composition. check if we can
         // implement it in the renderer instead.
         int action = event.getAction();
@@ -449,30 +464,31 @@ public class ThreadedInputConnection extends BaseInputConnection
             int pendingAccent = unicodeChar & KeyCharacterMap.COMBINING_ACCENT_MASK;
             StringBuilder builder = new StringBuilder();
             builder.appendCodePoint(pendingAccent);
-            updateComposingText(builder.toString(), 1, true);
-            setCombiningAccent(pendingAccent);
+            updateComposingTextOnUiThread(builder.toString(), 1, true);
+            setCombiningAccentOnUiThread(pendingAccent);
             return true;
         } else if (mPendingAccent != 0 && unicodeChar != 0) {
             int combined = KeyEvent.getDeadChar(mPendingAccent, unicodeChar);
             if (combined != 0) {
                 StringBuilder builder = new StringBuilder();
                 builder.appendCodePoint(combined);
-                commitText(builder.toString(), 1);
+                String text = builder.toString();
+                mImeAdapter.sendCompositionToNative(text, 1, text.length() > 0, 0);
                 return true;
             }
             // Noncombinable character; commit the accent character and fall through to sending
             // the key event for the character afterwards.
-            finishComposingText();
+            finishComposingTextOnUiThread();
         }
         return false;
     }
 
     @VisibleForTesting
-    public void setCombiningAccent(int pendingAccent) {
+    public void setCombiningAccentOnUiThread(int pendingAccent) {
         mPendingAccent = pendingAccent;
     }
 
-    private void cancelCombiningAccent() {
+    private void cancelCombiningAccentOnUiThread() {
         mPendingAccent = 0;
     }
 
@@ -482,20 +498,23 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean finishComposingText() {
         if (DEBUG_LOGS) Log.w(TAG, "finishComposingText");
-        cancelCombiningAccent();
         // This is the only function that may be called on UI thread because
         // of direct calls from InputMethodManager.
         ThreadUtils.postOnUiThread(mFinishComposingTextRunnable);
         return true;
     }
 
+    private void finishComposingTextOnUiThread() {
+        cancelCombiningAccentOnUiThread();
+        mImeAdapter.finishComposingText();
+    }
+
     /**
      * @see InputConnection#setSelection(int, int)
      */
     @Override
     public boolean setSelection(final int start, final int end) {
         if (DEBUG_LOGS) Log.w(TAG, "setSelection [%d %d]", start, end);
-        assertOnImeThread();
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
@@ -511,7 +530,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean setComposingRegion(final int start, final int end) {
         if (DEBUG_LOGS) Log.w(TAG, "setComposingRegion [%d %d]", start, end);
-        assertOnImeThread();
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
@@ -527,7 +545,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public CharSequence getTextBeforeCursor(int maxChars, int flags) {
         if (DEBUG_LOGS) Log.w(TAG, "getTextBeforeCursor [%d %x]", maxChars, flags);
-        assertOnImeThread();
         TextInputState textInputState = requestAndWaitForTextInputState();
         if (textInputState == null) return null;
         return textInputState.getTextBeforeSelection(maxChars);
@@ -539,7 +556,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public CharSequence getTextAfterCursor(int maxChars, int flags) {
         if (DEBUG_LOGS) Log.w(TAG, "getTextAfterCursor [%d %x]", maxChars, flags);
-        assertOnImeThread();
         TextInputState textInputState = requestAndWaitForTextInputState();
         if (textInputState == null) return null;
         return textInputState.getTextAfterSelection(maxChars);
@@ -551,7 +567,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public CharSequence getSelectedText(int flags) {
         if (DEBUG_LOGS) Log.w(TAG, "getSelectedText [%x]", flags);
-        assertOnImeThread();
         TextInputState textInputState = requestAndWaitForTextInputState();
         if (textInputState == null) return null;
         return textInputState.getSelectedText();
@@ -562,10 +577,14 @@ public class ThreadedInputConnection extends BaseInputConnection
      */
     @Override
     public int getCursorCapsMode(int reqModes) {
-        if (DEBUG_LOGS) Log.w(TAG, "getCursorCapsMode [%x]", reqModes);
-        assertOnImeThread();
-        // TODO(changwan): implement this.
-        return 0;
+        TextInputState textInputState = requestAndWaitForTextInputState();
+        int result = 0;
+        if (textInputState != null) {
+            result = TextUtils.getCapsMode(
+                    textInputState.text(), textInputState.selection().start(), reqModes);
+        }
+        if (DEBUG_LOGS) Log.w(TAG, "getCursorCapsMode [%x]: %x", reqModes, result);
+        return result;
     }
 
     /**
@@ -574,7 +593,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean commitCompletion(CompletionInfo text) {
         if (DEBUG_LOGS) Log.w(TAG, "commitCompletion [%s]", text);
-        assertOnImeThread();
         return false;
     }
 
@@ -587,7 +605,6 @@ public class ThreadedInputConnection extends BaseInputConnection
             Log.w(TAG, "commitCorrection [%s]",
                     ImeUtils.getCorrectionInfoDebugString(correctionInfo));
         }
-        assertOnImeThread();
         return false;
     }
 
@@ -597,7 +614,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean clearMetaKeyStates(int states) {
         if (DEBUG_LOGS) Log.w(TAG, "clearMetaKeyStates [%x]", states);
-        assertOnImeThread();
         return false;
     }
 
@@ -619,7 +635,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean performPrivateCommand(String action, Bundle data) {
         if (DEBUG_LOGS) Log.w(TAG, "performPrivateCommand [%s]", action);
-        assertOnImeThread();
         return false;
     }
 
@@ -629,7 +644,6 @@ public class ThreadedInputConnection extends BaseInputConnection
     @Override
     public boolean requestCursorUpdates(final int cursorUpdateMode) {
         if (DEBUG_LOGS) Log.w(TAG, "requestCursorUpdates [%x]", cursorUpdateMode);
-        assertOnImeThread();
         ThreadUtils.postOnUiThread(new Runnable() {
             @Override
             public void run() {
@@ -642,7 +656,10 @@ public class ThreadedInputConnection extends BaseInputConnection
     /**
      * @see InputConnection#closeConnection()
      */
+    // TODO(crbug.com/635567): Fix this properly.
+    @SuppressLint("MissingSuperCall")
     public void closeConnection() {
+        if (DEBUG_LOGS) Log.w(TAG, "closeConnection");
         // TODO(changwan): Implement this. http://crbug.com/595525
     }
 }
