@@ -4,6 +4,7 @@
 
 package org.chromium.content.browser.webcontents;
 
+import android.annotation.SuppressLint;
 import android.graphics.Bitmap;
 import android.graphics.Rect;
 import android.os.Bundle;
@@ -11,19 +12,23 @@ import android.os.Parcel;
 import android.os.ParcelUuid;
 import android.os.Parcelable;
 
-import org.chromium.base.ObserverList;
 import org.chromium.base.ThreadUtils;
 import org.chromium.base.VisibleForTesting;
 import org.chromium.base.annotations.CalledByNative;
 import org.chromium.base.annotations.JNINamespace;
+import org.chromium.content.browser.AppWebMessagePort;
+import org.chromium.content.browser.AppWebMessagePortService;
+import org.chromium.content.browser.MediaSessionImpl;
 import org.chromium.content_public.browser.AccessibilitySnapshotCallback;
 import org.chromium.content_public.browser.AccessibilitySnapshotNode;
 import org.chromium.content_public.browser.ContentBitmapCallback;
 import org.chromium.content_public.browser.ImageDownloadCallback;
 import org.chromium.content_public.browser.JavaScriptCallback;
+import org.chromium.content_public.browser.MessagePortService;
 import org.chromium.content_public.browser.NavigationController;
 import org.chromium.content_public.browser.WebContents;
 import org.chromium.content_public.browser.WebContentsObserver;
+import org.chromium.ui.OverscrollRefreshHandler;
 import org.chromium.ui.accessibility.AXTextStyle;
 
 import java.util.ArrayList;
@@ -59,6 +64,8 @@ import java.util.UUID;
      * A {@link android.os.Parcelable.Creator} instance that is used to build
      * {@link WebContentsImpl} objects from a {@link Parcel}.
      */
+    // TODO(crbug.com/635567): Fix this properly.
+    @SuppressLint("ParcelClassLoader")
     public static final Parcelable.Creator<WebContents> CREATOR =
             new Parcelable.Creator<WebContents>() {
                 @Override
@@ -88,7 +95,9 @@ import java.util.UUID;
     // Lazily created proxy observer for handling all Java-based WebContentsObservers.
     private WebContentsObserverProxy mObserverProxy;
 
-    private boolean mContextMenuOpened;
+    // The media session for this WebContents. It is constructed by the native MediaSession and has
+    // the same life time as native MediaSession.
+    private MediaSessionImpl mMediaSession;
 
     private WebContentsImpl(
             long nativeWebContentsAndroid, NavigationController navigationController) {
@@ -211,12 +220,6 @@ import java.util.UUID;
         nativeUnselect(mNativeWebContentsAndroid);
     }
 
-    @Override
-    public void insertCSS(String css) {
-        if (isDestroyed()) return;
-        nativeInsertCSS(mNativeWebContentsAndroid, css);
-    }
-
     @Override
     public void onHide() {
         nativeOnHide(mNativeWebContentsAndroid);
@@ -269,18 +272,12 @@ import java.util.UUID;
     }
 
     @Override
-    public void updateTopControlsState(boolean enableHiding, boolean enableShowing,
-            boolean animate) {
-        nativeUpdateTopControlsState(mNativeWebContentsAndroid, enableHiding,
-                enableShowing, animate);
+    public void updateBrowserControlsState(
+            boolean enableHiding, boolean enableShowing, boolean animate) {
+        nativeUpdateBrowserControlsState(
+                mNativeWebContentsAndroid, enableHiding, enableShowing, animate);
     }
 
-    @Override
-    public void showImeIfNeeded() {
-        nativeShowImeIfNeeded(mNativeWebContentsAndroid);
-    }
-
-    @Override
     public void scrollFocusedEditableNodeIntoView() {
         // The native side keeps track of whether the zoom and scroll actually occurred. It is
         // more efficient to do it this way and sometimes fire an unnecessary message rather
@@ -339,8 +336,18 @@ import java.util.UUID;
     }
 
     @Override
-    public void sendMessageToFrame(String frameName, String message, String targetOrigin) {
-        nativeSendMessageToFrame(mNativeWebContentsAndroid, frameName, message, targetOrigin);
+    public void postMessageToFrame(String frameName, String message,
+            String sourceOrigin, String targetOrigin, int[] sentPortIds) {
+        nativePostMessageToFrame(mNativeWebContentsAndroid, frameName, message,
+                sourceOrigin, targetOrigin, sentPortIds);
+    }
+
+    @Override
+    public AppWebMessagePort[] createMessageChannel(MessagePortService service)
+            throws IllegalStateException {
+        AppWebMessagePort[] ports = ((AppWebMessagePortService) service).createMessageChannel();
+        nativeCreateMessageChannel(mNativeWebContentsAndroid, ports);
+        return ports;
     }
 
     @Override
@@ -365,23 +372,11 @@ import java.util.UUID;
     }
 
     @Override
-    public void resumeMediaSession() {
-        nativeResumeMediaSession(mNativeWebContentsAndroid);
-    }
-
-    @Override
-    public void suspendMediaSession() {
-        nativeSuspendMediaSession(mNativeWebContentsAndroid);
-    }
-
-    @Override
-    public void stopMediaSession() {
-        nativeStopMediaSession(mNativeWebContentsAndroid);
-    }
-
-    @Override
-    public String getEncoding() {
-        return nativeGetEncoding(mNativeWebContentsAndroid);
+    @VisibleForTesting
+    public void simulateRendererKilledForTesting(boolean wasOomProtected) {
+        if (mObserverProxy != null) {
+            mObserverProxy.renderProcessGone(wasOomProtected);
+        }
     }
 
     // root node can be null if parsing fails.
@@ -434,37 +429,18 @@ import java.util.UUID;
         mObserverProxy.removeObserver(observer);
     }
 
-    @VisibleForTesting
     @Override
-    public ObserverList.RewindableIterator<WebContentsObserver> getObserversForTesting() {
-        return mObserverProxy.getObserversForTesting();
+    public void setOverscrollRefreshHandler(OverscrollRefreshHandler handler) {
+        nativeSetOverscrollRefreshHandler(mNativeWebContentsAndroid, handler);
     }
 
     @Override
-    public void getContentBitmapAsync(Bitmap.Config config, float scale, Rect srcRect,
-            ContentBitmapCallback callback) {
+    public void getContentBitmapAsync(
+            Bitmap.Config config, float scale, Rect srcRect, ContentBitmapCallback callback) {
         nativeGetContentBitmap(mNativeWebContentsAndroid, callback, config, scale,
                 srcRect.left, srcRect.top, srcRect.width(), srcRect.height());
     }
 
-    @Override
-    public void onContextMenuOpened() {
-        mContextMenuOpened = true;
-    }
-
-    @Override
-    public void onContextMenuClosed() {
-        if (!mContextMenuOpened) {
-            return;
-        } else {
-            mContextMenuOpened = false;
-        }
-
-        if (mNativeWebContentsAndroid != 0) {
-            nativeOnContextMenuClosed(mNativeWebContentsAndroid);
-        }
-    }
-
     @CalledByNative
     private void onGetContentBitmapFinished(ContentBitmapCallback callback, Bitmap bitmap,
             int response) {
@@ -489,6 +465,16 @@ import java.util.UUID;
         callback.onFinishDownloadImage(id, httpStatusCode, imageUrl, bitmaps, sizes);
     }
 
+    @Override
+    public void dismissTextHandles() {
+        nativeDismissTextHandles(mNativeWebContentsAndroid);
+    }
+
+    @CalledByNative
+    private final void setMediaSession(MediaSessionImpl mediaSession) {
+        mMediaSession = mediaSession;
+    }
+
     @CalledByNative
     private static List<Bitmap> createBitmapList() {
         return new ArrayList<Bitmap>();
@@ -525,7 +511,6 @@ import java.util.UUID;
     private native void nativeReplace(long nativeWebContentsAndroid, String word);
     private native void nativeSelectAll(long nativeWebContentsAndroid);
     private native void nativeUnselect(long nativeWebContentsAndroid);
-    private native void nativeInsertCSS(long nativeWebContentsAndroid, String css);
     private native void nativeOnHide(long nativeWebContentsAndroid);
     private native void nativeOnShow(long nativeWebContentsAndroid);
     private native void nativeSuspendAllMediaPlayers(long nativeWebContentsAndroid);
@@ -537,9 +522,8 @@ import java.util.UUID;
     private native boolean nativeFocusLocationBarByDefault(long nativeWebContentsAndroid);
     private native boolean nativeIsRenderWidgetHostViewReady(long nativeWebContentsAndroid);
     private native void nativeExitFullscreen(long nativeWebContentsAndroid);
-    private native void nativeUpdateTopControlsState(long nativeWebContentsAndroid,
+    private native void nativeUpdateBrowserControlsState(long nativeWebContentsAndroid,
             boolean enableHiding, boolean enableShowing, boolean animate);
-    private native void nativeShowImeIfNeeded(long nativeWebContentsAndroid);
     private native void nativeScrollFocusedEditableNodeIntoView(long nativeWebContentsAndroid);
     private native void nativeSelectWordAroundCaret(long nativeWebContentsAndroid);
     private native void nativeAdjustSelectionByCharacterOffset(
@@ -554,23 +538,23 @@ import java.util.UUID;
             String script, JavaScriptCallback callback);
     private native void nativeAddMessageToDevToolsConsole(
             long nativeWebContentsAndroid, int level, String message);
-    private native void nativeSendMessageToFrame(long nativeWebContentsAndroid,
-            String frameName, String message, String targetOrigin);
+    private native void nativePostMessageToFrame(long nativeWebContentsAndroid, String frameName,
+            String message, String sourceOrigin, String targetOrigin, int[] sentPortIds);
+    private native void nativeCreateMessageChannel(
+            long nativeWebContentsAndroid, AppWebMessagePort[] ports);
     private native boolean nativeHasAccessedInitialDocument(
             long nativeWebContentsAndroid);
     private native int nativeGetThemeColor(long nativeWebContentsAndroid);
     private native void nativeRequestAccessibilitySnapshot(
             long nativeWebContentsAndroid, AccessibilitySnapshotCallback callback);
-    private native void nativeResumeMediaSession(long nativeWebContentsAndroid);
-    private native void nativeSuspendMediaSession(long nativeWebContentsAndroid);
-    private native void nativeStopMediaSession(long nativeWebContentsAndroid);
-    private native String nativeGetEncoding(long nativeWebContentsAndroid);
+    private native void nativeSetOverscrollRefreshHandler(
+            long nativeWebContentsAndroid, OverscrollRefreshHandler nativeOverscrollRefreshHandler);
     private native void nativeGetContentBitmap(long nativeWebContentsAndroid,
             ContentBitmapCallback callback, Bitmap.Config config, float scale,
             float x, float y, float width, float height);
-    private native void nativeOnContextMenuClosed(long nativeWebContentsAndroid);
     private native void nativeReloadLoFiImages(long nativeWebContentsAndroid);
     private native int nativeDownloadImage(long nativeWebContentsAndroid,
             String url, boolean isFavicon, int maxBitmapSize,
             boolean bypassCache, ImageDownloadCallback callback);
+    private native void nativeDismissTextHandles(long nativeWebContentsAndroid);
 }
