@@ -8,12 +8,12 @@ import android.content.ClipData;
 import android.content.ClipboardManager;
 import android.content.Context;
 import android.content.Intent;
-import android.test.suitebuilder.annotation.SmallTest;
+import android.support.test.filters.SmallTest;
 import android.text.TextUtils;
 
 import org.chromium.base.ThreadUtils;
+import org.chromium.base.test.util.DisabledTest;
 import org.chromium.base.test.util.Feature;
-import org.chromium.base.test.util.FlakyTest;
 import org.chromium.base.test.util.UrlUtils;
 import org.chromium.content.browser.input.ChromiumBaseInputConnection;
 import org.chromium.content.browser.input.ImeTestUtils;
@@ -41,6 +41,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
             + "<br/><input id=\"disabled_text\" type=\"text\" disabled value=\"Sample Text\" />"
             + "</form></body></html>");
     private ContentViewCore mContentViewCore;
+    private SelectionPopupController mSelectionPopupController;
 
     @Override
     public void setUp() throws Exception {
@@ -50,13 +51,14 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         waitForActiveShellToBeDoneLoading();
 
         mContentViewCore = getContentViewCore();
-        assertWaitForPageScaleFactorMatch(1.1f);
+        mSelectionPopupController = mContentViewCore.getSelectionPopupControllerForTesting();
         waitForSelectActionBarVisible(false);
         waitForPastePopupStatus(false);
     }
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectionClearedAfterLossOfFocus() throws Throwable {
         requestFocusOnUiThread(true);
 
@@ -65,74 +67,77 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
 
         requestFocusOnUiThread(false);
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
 
         requestFocusOnUiThread(true);
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
     }
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectionPreservedAfterLossOfFocusIfRequested() throws Throwable {
         requestFocusOnUiThread(true);
 
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         mContentViewCore.preserveSelectionOnNextLossOfFocus();
         requestFocusOnUiThread(false);
         waitForSelectActionBarVisible(false);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         requestFocusOnUiThread(true);
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         // Losing focus yet again should properly clear the selection.
         requestFocusOnUiThread(false);
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
     }
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectionPreservedAfterReshown() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         setVisibileOnUiThread(false);
         waitForSelectActionBarVisible(false);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         setVisibileOnUiThread(true);
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
     }
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectionPreservedAfterReattached() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         setAttachedOnUiThread(false);
         waitForSelectActionBarVisible(false);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         setAttachedOnUiThread(true);
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
     }
 
     /*
     @SmallTest
     @Feature({"TextInput"})
     */
-    @FlakyTest(message = "https://crbug.com/592428")
+    @DisabledTest(message = "https://crbug.com/592428")
     public void testPastePopupNotShownOnLongPressingNonEmptyInput() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
@@ -142,11 +147,8 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         waitForPastePopupStatus(false);
     }
 
-    /*
     @SmallTest
     @Feature({"TextInput"})
-    */
-    @FlakyTest(message = "https://crbug.com/592428")
     public void testPastePopupClearedOnTappingEmptyInput() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
@@ -155,11 +157,8 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         waitForPastePopupStatus(false);
     }
 
-    /*
     @SmallTest
     @Feature({"TextInput"})
-    */
-    @FlakyTest(message = "https://crbug.com/592428")
     public void testPastePopupClearedOnTappingNonEmptyInput() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
@@ -168,11 +167,8 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         waitForPastePopupStatus(false);
     }
 
-    /*
     @SmallTest
     @Feature({"TextInput"})
-    */
-    @FlakyTest(message = "https://crbug.com/592428")
     public void testPastePopupClearedOnTappingOutsideInput() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
@@ -181,11 +177,8 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         waitForPastePopupStatus(false);
     }
 
-    /*
     @SmallTest
     @Feature({"TextInput"})
-    */
-    @FlakyTest(message = "https://crbug.com/592428")
     public void testPastePopupClearedOnLongPressingOutsideInput() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
@@ -194,26 +187,23 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         waitForPastePopupStatus(false);
     }
 
-    /*
     @SmallTest
     @Feature({"TextInput"})
-    */
-    @FlakyTest(message = "https://crbug.com/592428")
     public void testPastePopupNotShownOnLongPressingDisabledInput() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
         waitForPastePopupStatus(true);
-        assertTrue(mContentViewCore.hasInsertion());
+        waitForInsertion(true);
         DOMUtils.longPressNode(this, mContentViewCore, "disabled_text");
         waitForPastePopupStatus(false);
-        assertFalse(mContentViewCore.hasInsertion());
+        waitForInsertion(false);
     }
 
     /*
     @SmallTest
     @Feature({"TextInput"})
     */
-    @FlakyTest(message = "https://crbug.com/592428")
+    @DisabledTest(message = "https://crbug.com/592428")
     public void testPastePopupDismissedOnDestroy() throws Throwable {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "empty_input_text");
@@ -229,83 +219,90 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testActionBarConfiguredCorrectlyForInput() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
-        assertTrue(mContentViewCore.getSelectActionHandler().isSelectionEditable());
-        assertFalse(mContentViewCore.getSelectActionHandler().isSelectionPassword());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
+        assertTrue(mSelectionPopupController.isSelectionEditable());
+        assertFalse(mSelectionPopupController.isSelectionPassword());
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testActionBarConfiguredCorrectlyForPassword() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "password");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
-        assertTrue(mContentViewCore.getSelectActionHandler().isSelectionEditable());
-        assertTrue(mContentViewCore.getSelectActionHandler().isSelectionPassword());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
+        assertTrue(mSelectionPopupController.isSelectionEditable());
+        assertTrue(mSelectionPopupController.isSelectionPassword());
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testActionBarConfiguredCorrectlyForPlainText() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
-        assertFalse(mContentViewCore.getSelectActionHandler().isSelectionEditable());
-        assertFalse(mContentViewCore.getSelectActionHandler().isSelectionPassword());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
+        assertFalse(mSelectionPopupController.isSelectionEditable());
+        assertFalse(mSelectionPopupController.isSelectionPassword());
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testActionBarConfiguredCorrectlyForTextArea() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
-        assertTrue(mContentViewCore.getSelectActionHandler().isSelectionEditable());
-        assertFalse(mContentViewCore.getSelectActionHandler().isSelectionPassword());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
+        assertTrue(mSelectionPopupController.isSelectionEditable());
+        assertFalse(mSelectionPopupController.isSelectionPassword());
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPlainTextCopy() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCopy();
         waitForClipboardContents(mContentViewCore.getContext(), "SamplePlainTextOne");
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarInputCopy() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCopy();
         waitForClipboardContents(mContentViewCore.getContext(), "SampleInputText");
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPasswordCopy() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCopy();
         waitForClipboardContents(mContentViewCore.getContext(), "SamplePlainTextOne");
         DOMUtils.longPressNode(this, mContentViewCore, "password");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCopy();
         // Copy option won't be there for Password, hence no change in Clipboard
         // Validating with previous Clipboard content
@@ -314,27 +311,29 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarTextAreaCopy() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCopy();
         waitForClipboardContents(mContentViewCore.getContext(), "SampleTextArea");
     }
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPlainTextCut() throws Exception {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals(mContentViewCore.getSelectedText(), "SamplePlainTextOne");
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals(mSelectionPopupController.getSelectedText(), "SamplePlainTextOne");
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCut();
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         // Cut option won't be available for plain text.
         // Hence validating previous Clipboard content.
         waitForClipboardContents(mContentViewCore.getContext(), "SampleTextToCopy");
@@ -342,30 +341,32 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarInputCut() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals(mContentViewCore.getSelectedText(), "SampleInputText");
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals(mSelectionPopupController.getSelectedText(), "SampleInputText");
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCut();
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
         waitForClipboardContents(mContentViewCore.getContext(), "SampleInputText");
-        assertEquals(mContentViewCore.getSelectedText(), "");
+        assertEquals(mSelectionPopupController.getSelectedText(), "");
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPasswordCut() throws Exception {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "password");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCut();
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         // Cut option won't be there for Password, hence no change in Clipboard
         // Validating with previous Clipboard content
         waitForClipboardContents(mContentViewCore.getContext(), "SampleTextToCopy");
@@ -373,67 +374,72 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarTextAreaCut() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals(mContentViewCore.getSelectedText(), "SampleTextArea");
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals(mSelectionPopupController.getSelectedText(), "SampleTextArea");
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarCut();
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
         waitForClipboardContents(mContentViewCore.getContext(), "SampleTextArea");
-        assertEquals(mContentViewCore.getSelectedText(), "");
+        assertEquals(mSelectionPopupController.getSelectedText(), "");
     }
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPlainTextSelectAll() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarSelectAll();
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         waitForSelectActionBarVisible(true);
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarInputSelectAll() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarSelectAll();
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         waitForSelectActionBarVisible(true);
-        assertEquals(mContentViewCore.getSelectedText(), "SampleInputText");
+        assertEquals(mSelectionPopupController.getSelectedText(), "SampleInputText");
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPasswordSelectAll() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "password");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarSelectAll();
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         waitForSelectActionBarVisible(true);
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarTextAreaSelectAll() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarSelectAll();
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         waitForSelectActionBarVisible(true);
-        assertEquals(mContentViewCore.getSelectedText(), "SampleTextArea");
+        assertEquals(mSelectionPopupController.getSelectedText(), "SampleTextArea");
     }
 
     private CharSequence getTextBeforeCursor(final int length, final int flags) {
@@ -452,16 +458,16 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
      * @SmallTest
      * @Feature({"TextSelection", "TextInput"})
      */
-    @FlakyTest(message = "http://crbug.com/606942")
+    @DisabledTest(message = "http://crbug.com/606942")
     public void testCursorPositionAfterHidingActionMode() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarSelectAll();
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         waitForSelectActionBarVisible(true);
-        assertEquals(mContentViewCore.getSelectedText(), "SampleTextArea");
+        assertEquals(mSelectionPopupController.getSelectedText(), "SampleTextArea");
         hideSelectActionMode();
         waitForSelectActionBarVisible(false);
         CriteriaHelper.pollInstrumentationThread(
@@ -475,97 +481,103 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
 
     @SmallTest
     @Feature({"TextSelection"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarPlainTextPaste() throws Exception {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarPaste();
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text_1");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
         // Paste option won't be available for plain text.
         // Hence content won't be changed.
-        assertNotSame(mContentViewCore.getSelectedText(), "SampleTextToCopy");
+        assertNotSame(mSelectionPopupController.getSelectedText(), "SampleTextToCopy");
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarInputPaste() throws Exception {
         copyStringToClipboard("SampleTextToCopy");
 
         // Select the input field.
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         // Paste into the input field.
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarPaste();
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
 
         // Ensure the new text matches the pasted text.
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals("SampleTextToCopy", mContentViewCore.getSelectedText());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals("SampleTextToCopy", mSelectionPopupController.getSelectedText());
     }
 
     /*
     @SmallTest
     @Feature({"TextInput"})
     */
-    @FlakyTest(message = "https://crbug.com/592428")
+    @DisabledTest(message = "https://crbug.com/592428")
     public void testSelectActionBarPasswordPaste() throws Exception {
         copyStringToClipboard("SamplePassword2");
 
         // Select the password field.
         DOMUtils.longPressNode(this, mContentViewCore, "password");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals(mContentViewCore.getSelectedText().length(), "SamplePassword".length());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals(mSelectionPopupController.getSelectedText().length(),
+                "SamplePassword".length());
 
         // Paste "SamplePassword2" into the password field, replacing
         // "SamplePassword".
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarPaste();
         waitForSelectActionBarVisible(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
 
         // Ensure the new text matches the pasted text. Note that we can't
         // actually compare strings as password field selections only provide
         // a placeholder with the correct length.
         DOMUtils.longPressNode(this, mContentViewCore, "password");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals(mContentViewCore.getSelectedText().length(), "SamplePassword2".length());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals(mSelectionPopupController.getSelectedText().length(),
+                "SamplePassword2".length());
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarTextAreaPaste() throws Exception {
         copyStringToClipboard("SampleTextToCopy");
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarPaste();
         DOMUtils.clickNode(this, mContentViewCore, "plain_text_1");
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertEquals(mContentViewCore.getSelectedText(), "SampleTextToCopy");
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertEquals(mSelectionPopupController.getSelectedText(), "SampleTextToCopy");
     }
 
     @SmallTest
     @Feature({"TextInput"})
+    @DisabledTest(message = "crbug.com/592428")
     public void testSelectActionBarSearchAndShareLaunchesNewTask() throws Exception {
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         waitForSelectActionBarVisible(true);
-        assertTrue(mContentViewCore.hasSelection());
-        assertNotNull(mContentViewCore.getSelectActionHandler());
+        assertTrue(mSelectionPopupController.hasSelection());
+        assertTrue(mSelectionPopupController.isActionModeValid());
         selectActionBarSearch();
         Intent i = getActivity().getLastSentIntent();
         int new_task_flag = Intent.FLAG_ACTIVITY_NEW_TASK;
@@ -580,7 +592,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.getSelectActionHandler().paste();
+                mSelectionPopupController.paste();
             }
         });
     }
@@ -589,7 +601,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.getSelectActionHandler().selectAll();
+                mSelectionPopupController.selectAll();
             }
         });
     }
@@ -598,7 +610,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.getSelectActionHandler().cut();
+                mSelectionPopupController.cut();
             }
         });
     }
@@ -607,7 +619,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.getSelectActionHandler().copy();
+                mSelectionPopupController.copy();
             }
         });
     }
@@ -616,7 +628,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.getSelectActionHandler().search();
+                mSelectionPopupController.search();
             }
         });
     }
@@ -625,7 +637,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.getSelectActionHandler().share();
+                mSelectionPopupController.share();
             }
         });
     }
@@ -634,13 +646,12 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                mContentViewCore.hideSelectActionMode();
+                mContentViewCore.destroySelectActionMode();
             }
         });
     }
 
-    private void waitForClipboardContents(final Context context, final String expectedContents)
-            throws InterruptedException {
+    private void waitForClipboardContents(final Context context, final String expectedContents) {
         CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
@@ -653,8 +664,7 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         });
     }
 
-    private void waitForSelectActionBarVisible(
-            final boolean visible) throws InterruptedException {
+    private void waitForSelectActionBarVisible(final boolean visible) {
         CriteriaHelper.pollUiThread(Criteria.equals(visible, new Callable<Boolean>() {
             @Override
             public Boolean call() {
@@ -709,11 +719,20 @@ public class ContentViewCoreSelectionTest extends ContentShellTestBase {
         clipboardManager.setPrimaryClip(clip);
     }
 
-    private void waitForPastePopupStatus(final boolean show) throws InterruptedException {
+    private void waitForPastePopupStatus(final boolean show) {
         CriteriaHelper.pollUiThread(Criteria.equals(show, new Callable<Boolean>() {
             @Override
             public Boolean call() {
-                return mContentViewCore.isPastePopupShowing();
+                return mSelectionPopupController.isPastePopupShowing();
+            }
+        }));
+    }
+
+    private void waitForInsertion(final boolean show) {
+        CriteriaHelper.pollUiThread(Criteria.equals(show, new Callable<Boolean>() {
+            @Override
+            public Boolean call() {
+                return mSelectionPopupController.isInsertion();
             }
         }));
     }
