@@ -6,23 +6,31 @@ package org.chromium.content.browser;
 
 import android.content.Context;
 import android.media.AudioManager;
-import android.test.suitebuilder.annotation.MediumTest;
-import android.test.suitebuilder.annotation.SmallTest;
+import android.support.test.filters.MediumTest;
+import android.support.test.filters.SmallTest;
 
+import org.chromium.base.ThreadUtils;
+import org.chromium.base.annotations.SuppressFBWarnings;
 import org.chromium.base.test.util.CommandLineFlags;
 import org.chromium.base.test.util.DisabledTest;
 import org.chromium.base.test.util.Feature;
+import org.chromium.base.test.util.Restriction;
+import org.chromium.base.test.util.RetryOnFailure;
 import org.chromium.content.browser.test.util.Criteria;
 import org.chromium.content.browser.test.util.CriteriaHelper;
 import org.chromium.content.browser.test.util.DOMUtils;
 import org.chromium.content.common.ContentSwitches;
+import org.chromium.content_public.browser.MediaSession;
+import org.chromium.content_public.browser.MediaSessionObserver;
 import org.chromium.content_shell_apk.ContentShellTestBase;
 
+import java.util.ArrayList;
 import java.util.concurrent.Callable;
 
 /**
  * Tests for MediaSession.
  */
+@RetryOnFailure
 @CommandLineFlags.Add(ContentSwitches.DISABLE_GESTURE_REQUIREMENT_FOR_MEDIA_PLAYBACK)
 public class MediaSessionTest extends ContentShellTestBase {
     private static final String MEDIA_SESSION_TEST_URL =
@@ -68,7 +76,7 @@ public class MediaSessionTest extends ContentShellTestBase {
             mAudioFocusState = AudioManager.AUDIOFOCUS_LOSS;
         }
 
-        public void waitForFocusStateChange(int focusType) throws InterruptedException {
+        public void waitForFocusStateChange(int focusType) {
             CriteriaHelper.pollInstrumentationThread(
                     Criteria.equals(focusType, new Callable<Integer>() {
                         @Override
@@ -81,6 +89,35 @@ public class MediaSessionTest extends ContentShellTestBase {
 
     private MockAudioFocusChangeListener mAudioFocusChangeListener;
 
+    @SuppressFBWarnings("URF_UNREAD_FIELD")
+    private MediaSessionObserver mObserver;
+
+    private ArrayList<StateRecord> mStateRecords = new ArrayList<StateRecord>();
+
+    private static class StateRecord {
+        public boolean isControllable;
+        public boolean isSuspended;
+
+        public StateRecord(boolean isControllable, boolean isSuspended) {
+            this.isControllable = isControllable;
+            this.isSuspended = isSuspended;
+        }
+
+        @Override
+        public boolean equals(Object obj) {
+            if (obj == this) return true;
+            if (!(obj instanceof StateRecord)) return false;
+
+            StateRecord other = (StateRecord) obj;
+            return isControllable == other.isControllable && isSuspended == other.isSuspended;
+        }
+
+        @Override
+        public int hashCode() {
+            return (isControllable ? 2 : 0) + (isSuspended ? 1 : 0);
+        }
+    }
+
     @Override
     public void setUp() throws Exception {
         super.setUp();
@@ -92,6 +129,19 @@ public class MediaSessionTest extends ContentShellTestBase {
         }
 
         mAudioFocusChangeListener = new MockAudioFocusChangeListener();
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                mObserver =
+                        new MediaSessionObserver(MediaSession.fromWebContents(getWebContents())) {
+                            @Override
+                            public void mediaSessionStateChanged(
+                                    boolean isControllable, boolean isSuspended) {
+                                mStateRecords.add(new StateRecord(isControllable, isSuspended));
+                            }
+                        };
+            }
+        });
     }
 
     @Override
@@ -143,6 +193,7 @@ public class MediaSessionTest extends ContentShellTestBase {
 
     @MediumTest
     @Feature({"MediaSession"})
+    @RetryOnFailure
     public void testShortVideoIsTransient() throws Exception {
         assertEquals(AudioManager.AUDIOFOCUS_LOSS, mAudioFocusChangeListener.getAudioFocusState());
         mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
@@ -313,6 +364,7 @@ public class MediaSessionTest extends ContentShellTestBase {
 
     @SmallTest
     @Feature({"MediaSession"})
+    @DisabledTest(message = "crbug.com/625584")
     public void testMediaDuck() throws Exception {
         assertEquals(AudioManager.AUDIOFOCUS_LOSS, mAudioFocusChangeListener.getAudioFocusState());
         mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
@@ -350,6 +402,8 @@ public class MediaSessionTest extends ContentShellTestBase {
 
     @MediumTest
     @Feature({"MediaSession"})
+    @Restriction(Restriction.RESTRICTION_TYPE_NON_LOW_END_DEVICE)  // crbug.com/589176
+    @RetryOnFailure
     public void testMediaResumeAfterTransientFocusLoss() throws Exception {
         assertEquals(AudioManager.AUDIOFOCUS_LOSS, mAudioFocusChangeListener.getAudioFocusState());
         mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
@@ -375,4 +429,62 @@ public class MediaSessionTest extends ContentShellTestBase {
         DOMUtils.waitForMediaPlay(getWebContents(), LONG_AUDIO);
         DOMUtils.waitForMediaPlay(getWebContents(), LONG_VIDEO);
     }
+
+    @MediumTest
+    @Feature({"MediaSession"})
+    @RetryOnFailure
+    public void testSessionSuspendedAfterFocusLossWhenPlaying() throws Exception {
+        ArrayList<StateRecord> expectedStates = new ArrayList<StateRecord>();
+        expectedStates.add(new StateRecord(true, false));
+        expectedStates.add(new StateRecord(true, true));
+
+        assertEquals(AudioManager.AUDIOFOCUS_LOSS, mAudioFocusChangeListener.getAudioFocusState());
+        mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
+        assertEquals(AudioManager.AUDIOFOCUS_GAIN, mAudioFocusChangeListener.getAudioFocusState());
+
+        DOMUtils.playMedia(getWebContents(), LONG_AUDIO);
+        DOMUtils.waitForMediaPlay(getWebContents(), LONG_AUDIO);
+
+        // Wait for the media to be really playing.
+        mAudioFocusChangeListener.waitForFocusStateChange(AudioManager.AUDIOFOCUS_LOSS);
+
+        mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
+        assertEquals(AudioManager.AUDIOFOCUS_GAIN, mAudioFocusChangeListener.getAudioFocusState());
+
+        DOMUtils.waitForMediaPauseBeforeEnd(getWebContents(), LONG_AUDIO);
+
+        assertEquals(expectedStates, mStateRecords);
+    }
+
+    @MediumTest
+    @Feature({"MediaSession"})
+    @RetryOnFailure
+    public void testSessionSuspendedAfterFocusLossWhenPaused() throws Exception {
+        ArrayList<StateRecord> expectedStates = new ArrayList<StateRecord>();
+        expectedStates.add(new StateRecord(true, false));
+        expectedStates.add(new StateRecord(true, true));
+
+        assertEquals(AudioManager.AUDIOFOCUS_LOSS, mAudioFocusChangeListener.getAudioFocusState());
+        mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
+        assertEquals(AudioManager.AUDIOFOCUS_GAIN, mAudioFocusChangeListener.getAudioFocusState());
+
+        DOMUtils.playMedia(getWebContents(), LONG_AUDIO);
+        DOMUtils.waitForMediaPlay(getWebContents(), LONG_AUDIO);
+
+        // Wait for the media to be really playing.
+        mAudioFocusChangeListener.waitForFocusStateChange(AudioManager.AUDIOFOCUS_LOSS);
+
+        DOMUtils.pauseMedia(getWebContents(), LONG_AUDIO);
+        DOMUtils.waitForMediaPauseBeforeEnd(getWebContents(), LONG_AUDIO);
+
+        assertEquals(expectedStates, mStateRecords);
+
+        mAudioFocusChangeListener.requestAudioFocus(AudioManager.AUDIOFOCUS_GAIN);
+        assertEquals(AudioManager.AUDIOFOCUS_GAIN, mAudioFocusChangeListener.getAudioFocusState());
+
+        // Wait for 1 second before observing MediaSession state change.
+        Thread.sleep(AUDIO_FOCUS_CHANGE_TIMEOUT);
+
+        assertEquals(expectedStates, mStateRecords);
+    }
 }
