@@ -10,9 +10,9 @@ import android.content.ClipboardManager;
 import android.content.Context;
 import android.content.res.Configuration;
 import android.os.Handler;
-import android.os.Looper;
-import android.test.suitebuilder.annotation.MediumTest;
-import android.test.suitebuilder.annotation.SmallTest;
+import android.support.test.filters.MediumTest;
+import android.support.test.filters.SmallTest;
+import android.text.InputType;
 import android.text.TextUtils;
 import android.util.Pair;
 import android.view.KeyEvent;
@@ -22,10 +22,11 @@ import android.view.inputmethod.EditorInfo;
 import android.view.inputmethod.InputConnection;
 
 import org.chromium.base.ThreadUtils;
-import org.chromium.base.test.util.CommandLineFlags;
+import org.chromium.base.test.util.DisabledTest;
 import org.chromium.base.test.util.Feature;
-import org.chromium.base.test.util.FlakyTest;
+import org.chromium.base.test.util.UrlUtils;
 import org.chromium.content.browser.ContentViewCore;
+import org.chromium.content.browser.SelectionPopupController;
 import org.chromium.content.browser.test.util.Criteria;
 import org.chromium.content.browser.test.util.CriteriaHelper;
 import org.chromium.content.browser.test.util.DOMUtils;
@@ -44,7 +45,7 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeoutException;
 
 /**
- * Integration tests for text input using cases based on fixed regressions.
+ * IME (input method editor) and text input tests.
  */
 public class ImeTest extends ContentShellTestBase {
     protected ChromiumBaseInputConnection mConnection;
@@ -56,6 +57,7 @@ public class ImeTest extends ContentShellTestBase {
 
     private ContentViewCore mContentViewCore;
     private WebContents mWebContents;
+    private SelectionPopupController mSelectionPopupController;
     private TestCallbackHelperContainer mCallbackContainer;
     protected TestInputMethodManagerWrapper mInputMethodManagerWrapper;
 
@@ -64,6 +66,7 @@ public class ImeTest extends ContentShellTestBase {
         super.setUp();
         startActivityWithTestUrl(INPUT_FORM_HTML);
         mContentViewCore = getContentViewCore();
+        mSelectionPopupController = mContentViewCore.getSelectionPopupControllerForTesting();
         mWebContents = getWebContents();
 
         mInputMethodManagerWrapper = new TestInputMethodManagerWrapper(mContentViewCore);
@@ -75,16 +78,14 @@ public class ImeTest extends ContentShellTestBase {
 
         mCallbackContainer = new TestCallbackHelperContainer(mContentViewCore);
         DOMUtils.waitForNonZeroNodeBounds(mWebContents, "input_text");
-        DOMUtils.clickNode(this, mContentViewCore, "input_text");
+        boolean result = DOMUtils.clickNode(this, mContentViewCore, "input_text");
+
+        assertEquals("Failed to dispatch touch event.", true, result);
         assertWaitForKeyboardStatus(true);
 
         mConnection = getInputConnection();
         mImeAdapter = getImeAdapter();
 
-        if (usingReplicaInputConnection()) {
-            // This is not needed if onCreateInputConnection() can return correct selection range.
-            waitAndVerifyUpdateSelection(0, 0, 0, -1, -1);
-        }
         waitForKeyboardStates(1, 0, 1, new Integer[] {TextInputType.TEXT});
         assertEquals(0, mConnectionFactory.getOutAttrs().initialSelStart);
         assertEquals(0, mConnectionFactory.getOutAttrs().initialSelEnd);
@@ -92,9 +93,39 @@ public class ImeTest extends ContentShellTestBase {
         waitForEventLogs("selectionchange");
         clearEventLogs();
 
+        waitAndVerifyUpdateSelection(0, 0, 0, -1, -1);
         resetAllStates();
     }
 
+    private void fullyLoadUrl(final String url) throws Throwable {
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                getActivity().getActiveShell().loadUrl(url);
+            }
+        });
+        waitForActiveShellToBeDoneLoading();
+    }
+
+    @MediumTest
+    @Feature({"TextInput", "Main"})
+    public void testKeyboardDismissedWhenNavigating() throws Throwable {
+        assertWaitForKeyboardStatus(true);
+
+        // Hide keyboard when loading a new Url.
+        fullyLoadUrl(UrlUtils.getIsolatedTestFileUrl(INPUT_FORM_HTML));
+        assertWaitForKeyboardStatus(false);
+
+        DOMUtils.clickNode(this, mContentViewCore, "input_text");
+        assertWaitForKeyboardStatus(true);
+
+        // Hide keyboard when navigating.
+        final String code = "document.getElementById(\"link\").click()";
+        JavaScriptUtils.executeJavaScriptAndWaitForResult(
+                getContentViewCore().getWebContents(), code);
+        assertWaitForKeyboardStatus(false);
+    }
+
     @MediumTest
     @Feature({"TextInput", "Main"})
     public void testKeyboardDismissedAfterClickingGo() throws Throwable {
@@ -123,29 +154,67 @@ public class ImeTest extends ContentShellTestBase {
         assertWaitForKeyboardStatus(false);
     }
 
+    // crbug.com/643519
     @SmallTest
     @Feature({"TextInput", "Main"})
-    public void testDeleteSurroundingTextWithZeroValue() throws Throwable {
-        commitText("hello", 1);
-        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
-        deleteSurroundingText(0, 0);
+    public void testCompositionWithNullTextNotCrash() throws Throwable {
+        commitText(null, 1);
+        assertTextsAroundCursor("", null, "");
 
-        setSelection(0, 0);
-        waitAndVerifyUpdateSelection(1, 0, 0, -1, -1);
-        deleteSurroundingText(0, 0);
-
-        setSelection(2, 2);
-        waitAndVerifyUpdateSelection(2, 2, 2, -1, -1);
-        deleteSurroundingText(0, 0);
-        assertTextsAroundCursor("he", null, "llo");
+        setComposingText(null, 1);
+        assertTextsAroundCursor("", null, "");
     }
 
-    // When newCursorPosition != 1, setComposingText doesn't work for ReplicaInputConnection
-    // because there is a bug in BaseInputConnection.
-    @CommandLineFlags.Add("enable-features=ImeThread")
     @SmallTest
     @Feature({"TextInput", "Main"})
-    public void testSetComposingTextForDifferentnewCursorPositions() throws Throwable {
+    public void testDeleteSurroundingTextWithRangeSelection() throws Throwable {
+        commitText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
+
+        setSelection(1, 4);
+        waitAndVerifyUpdateSelection(1, 1, 4, -1, -1);
+
+        deleteSurroundingText(0, 0);
+        assertTextsAroundCursor("h", "ell", "o");
+
+        deleteSurroundingText(1, 1);
+        assertTextsAroundCursor("", "ell", "");
+
+        deleteSurroundingText(1, 0);
+        assertTextsAroundCursor("", "ell", "");
+
+        deleteSurroundingText(0, 1);
+        assertTextsAroundCursor("", "ell", "");
+    }
+
+    @SmallTest
+    @Feature({"TextInput", "Main"})
+    public void testDeleteSurroundingTextWithCursorSelection() throws Throwable {
+        commitText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
+
+        setSelection(2, 2);
+        waitAndVerifyUpdateSelection(1, 2, 2, -1, -1);
+
+        deleteSurroundingText(0, 0);
+        assertTextsAroundCursor("he", null, "llo");
+
+        deleteSurroundingText(1, 1);
+        assertTextsAroundCursor("h", null, "lo");
+
+        deleteSurroundingText(1, 0);
+        assertTextsAroundCursor("", null, "lo");
+
+        deleteSurroundingText(0, 10);
+        assertTextsAroundCursor("", null, "");
+
+        deleteSurroundingText(10, 10);
+        assertTextsAroundCursor("", null, "");
+    }
+
+    @SmallTest
+    @Feature({"TextInput", "Main"})
+    public void testSetComposingTextForNewCursorPositions() throws Throwable {
         // Cursor is on the right of composing text when newCursorPosition > 0.
         setComposingText("ab", 1);
         waitAndVerifyUpdateSelection(0, 2, 2, 0, 2);
@@ -189,6 +258,81 @@ public class ImeTest extends ContentShellTestBase {
         waitAndVerifyUpdateSelection(10, 8, 8, 2, 6);
     }
 
+    @SmallTest
+    @Feature({"TextInput", "Main"})
+    public void testCommitTextForNewCursorPositions() throws Throwable {
+        // Cursor is on the left of committing text.
+        commitText("ab", 0);
+        waitAndVerifyUpdateSelection(0, 0, 0, -1, -1);
+
+        // Cursor is on the right of committing text.
+        commitText("cd", 1);
+        waitAndVerifyUpdateSelection(1, 2, 2, -1, -1);
+
+        // Cursor is between the committing text and the right boundary.
+        commitText("ef", 2);
+        waitAndVerifyUpdateSelection(2, 5, 5, -1, -1);
+
+        // Cursor is between the left boundary and the committing text.
+        commitText("gh", -3);
+        waitAndVerifyUpdateSelection(3, 2, 2, -1, -1);
+
+        // Cursor is on the right boundary.
+        commitText("ij", 7);
+        waitAndVerifyUpdateSelection(4, 10, 10, -1, -1);
+
+        // Cursor is on the left boundary.
+        commitText("kl", -10);
+        waitAndVerifyUpdateSelection(5, 0, 0, -1, -1);
+
+        // Cursor exceeds the right boundary.
+        commitText("mn", 100);
+        waitAndVerifyUpdateSelection(6, 14, 14, -1, -1);
+
+        // Cursor exceeds the left boundary.
+        commitText("op", -100);
+        waitAndVerifyUpdateSelection(7, 0, 0, -1, -1);
+    }
+
+    @SmallTest
+    @Feature({"TextInput", "Main"})
+    public void testSetComposingTextWithEmptyText() throws Throwable {
+        commitText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
+
+        setComposingText("AB", 1);
+        waitAndVerifyUpdateSelection(1, 7, 7, 5, 7);
+
+        // With previous composition.
+        setComposingText("", -3);
+        waitAndVerifyUpdateSelection(2, 2, 2, -1, -1);
+        assertTextsAroundCursor("he", null, "llo");
+
+        // Without previous composition.
+        setComposingText("", 3);
+        waitAndVerifyUpdateSelection(3, 4, 4, -1, -1);
+        assertTextsAroundCursor("hell", null, "o");
+    }
+
+    @SmallTest
+    @Feature({"TextInput", "Main"})
+    public void testCommitTextWithEmptyText() throws Throwable {
+        commitText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
+        setSelection(2, 2);
+        waitAndVerifyUpdateSelection(1, 2, 2, -1, -1);
+
+        setComposingText("world", 1);
+        waitAndVerifyUpdateSelection(2, 7, 7, 2, 7);
+        // With previous composition.
+        commitText("", 2);
+        waitAndVerifyUpdateSelection(3, 3, 3, -1, -1);
+
+        // Without previous composition.
+        commitText("", -1);
+        waitAndVerifyUpdateSelection(4, 2, 2, -1, -1);
+    }
+
     @SmallTest
     @Feature({"TextInput", "Main"})
     public void testCommitWhileComposingText() throws Throwable {
@@ -308,22 +452,11 @@ public class ImeTest extends ContentShellTestBase {
     private void assertTextsAroundCursor(
             CharSequence before, CharSequence selected, CharSequence after) throws Exception {
         assertEquals(before, getTextBeforeCursor(100, 0));
-
-        CharSequence actualSelected = getSelectedText(0);
-        assertEquals(selected, actualSelected);
-
-        if (usingReplicaInputConnection() && after.equals("\n")) {
-            // When the text ends with \n, we have a second new line that is not user
-            // visible/editable one, it is a side effect of using <br> internally.
-            // Replica model simply deviates from the blink editor in this case.
-            assertEquals("", getTextAfterCursor(100, 0));
-        } else {
-            assertEquals(after, getTextAfterCursor(100, 0));
-        }
+        assertEquals(selected, getSelectedText(0));
+        assertEquals(after, getTextAfterCursor(100, 0));
     }
 
-    private void waitForKeyboardStates(int show, int hide, int restart, Integer[] history)
-            throws InterruptedException {
+    private void waitForKeyboardStates(int show, int hide, int restart, Integer[] history) {
         final String expected = stringifyKeyboardStates(show, hide, restart, history);
         CriteriaHelper.pollUiThread(Criteria.equals(expected, new Callable<String>() {
             @Override
@@ -403,8 +536,9 @@ public class ImeTest extends ContentShellTestBase {
         assertWaitForSelectActionBarStatus(true);
     }
 
-    @SmallTest
-    @Feature({"TextInput"})
+    // @SmallTest
+    // @Feature({"TextInput"})
+    @DisabledTest
     public void testLongPressInputWhileComposingText() throws Exception {
         assertWaitForSelectActionBarStatus(false);
         setComposingText("Sample Text", 1);
@@ -475,20 +609,14 @@ public class ImeTest extends ContentShellTestBase {
         });
     }
 
-    private void reloadPage() {
+    private void reloadPage() throws Throwable {
         // Reload the page, then focus will be lost and keyboard should be hidden.
-        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
-            @Override
-            public void run() {
-                final String currentUrl = getContentViewCore().getWebContents().getUrl();
-                getActivity().getActiveShell().loadUrl(currentUrl);
-            }
-        });
+        fullyLoadUrl(getContentViewCore().getWebContents().getUrl());
     }
 
     @SmallTest
     @Feature({"TextInput"})
-    public void testPhysicalKeyboard_AttachDetach() throws Exception {
+    public void testPhysicalKeyboard_AttachDetach() throws Throwable {
         attachPhysicalKeyboard();
         // We still call showSoftKeyboard, which will be ignored by physical keyboard.
         waitForKeyboardStates(1, 0, 1, new Integer[] {TextInputType.TEXT});
@@ -559,12 +687,21 @@ public class ImeTest extends ContentShellTestBase {
     public void testImeStaysOnLongPressingDifferentNonEmptyInputs() throws Exception {
         DOMUtils.focusNode(mWebContents, "input_text");
         assertWaitForKeyboardStatus(true);
+
         commitText("Sample Text", 1);
+        // We should wait to avoid race condition.
+        waitAndVerifyUpdateSelection(0, 11, 11, -1, -1);
+
         DOMUtils.focusNode(mWebContents, "textarea");
+        waitAndVerifyUpdateSelection(1, 0, 0, -1, -1);
+
         commitText("Sample Text", 1);
+        waitAndVerifyUpdateSelection(2, 11, 11, -1, -1);
+
         DOMUtils.longPressNode(this, mContentViewCore, "input_text");
         assertWaitForKeyboardStatus(true);
         assertWaitForSelectActionBarStatus(true);
+
         DOMUtils.longPressNode(this, mContentViewCore, "textarea");
         assertWaitForKeyboardStatus(true);
     }
@@ -617,40 +754,6 @@ public class ImeTest extends ContentShellTestBase {
         assertTextsAroundCursor("blablargblarg", null, "");
     }
 
-    // Chrome can crash after pasting long text into textarea, becasue there is an overflow bug in
-    // SpannableStringBuilder#replace(). This can be avoided by enabling ImeThread.
-    // crbug.com/606059
-    @CommandLineFlags.Add("enable-features=ImeThread")
-    @MediumTest
-    @Feature({"TextInput"})
-    @FlakyTest
-    public void testPasteLongText() throws Exception {
-        int textLength = 25000;
-        String text = new String(new char[textLength]).replace("\0", "a");
-
-        commitText(text, 1);
-        waitAndVerifyUpdateSelection(0, textLength, textLength, -1, -1);
-        selectAll();
-        waitAndVerifyUpdateSelection(1, 0, textLength, -1, -1);
-        copy();
-
-        focusElement("textarea");
-        waitAndVerifyUpdateSelection(2, 0, 0, -1, -1);
-
-        // In order to reproduce the bug, we need some text after the pasting text.
-        commitText("hello", 1);
-        waitAndVerifyUpdateSelection(3, 5, 5, -1, -1);
-
-        setSelection(0, 0);
-        waitAndVerifyUpdateSelection(4, 0, 0, -1, -1);
-
-        // It will crash after the 3rd paste if ImeThread is not enabled.
-        for (int i = 0; i < 10; i++) {
-            paste();
-            waitAndVerifyUpdateSelection(5 + i, textLength * (i + 1), textLength * (i + 1), -1, -1);
-        }
-    }
-
     @SmallTest
     @Feature({"TextInput"})
     public void testImeSelectAndUnSelectAll() throws Exception {
@@ -914,10 +1017,6 @@ public class ImeTest extends ContentShellTestBase {
         dispatchKeyEvent(new KeyEvent(KeyEvent.ACTION_UP, KeyEvent.KEYCODE_B));
         assertEquals("hôˆb", getTextBeforeCursor(9, 0));
         int index = 4;
-        if (usingReplicaInputConnection()) {
-            // A transitional state due to finishComposingText.
-            waitAndVerifyUpdateSelection(index++, 3, 3, -1, -1);
-        }
         waitAndVerifyUpdateSelection(index++, 4, 4, -1, -1);
 
         // ALT-i
@@ -933,22 +1032,22 @@ public class ImeTest extends ContentShellTestBase {
         assertEquals("hôˆb", getTextBeforeCursor(9, 0));
         dispatchKeyEvent(new KeyEvent(KeyEvent.ACTION_UP, KeyEvent.KEYCODE_DEL));
         assertEquals("hôˆb", getTextBeforeCursor(9, 0));
-        if (usingReplicaInputConnection()) {
-            // A transitional state due to finishComposingText in deleteSurroundingTextImpl.
-            waitAndVerifyUpdateSelection(index++, 5, 5, -1, -1);
-        }
         waitAndVerifyUpdateSelection(index++, 4, 4, -1, -1);
     }
 
     @SmallTest
     @Feature({"TextInput", "Main"})
     public void testSetComposingRegionOutOfBounds() throws Throwable {
-        focusElement("textarea");
+        focusElementAndWaitForStateUpdate("textarea");
         setComposingText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, 0, 5);
 
         setComposingRegion(0, 0);
+        waitAndVerifyUpdateSelection(1, 5, 5, -1, -1);
         setComposingRegion(0, 9);
-        setComposingRegion(9, 0);
+        waitAndVerifyUpdateSelection(2, 5, 5, 0, 5);
+        setComposingRegion(9, 1);
+        waitAndVerifyUpdateSelection(3, 5, 5, 1, 5);
     }
 
     @SmallTest
@@ -1015,50 +1114,7 @@ public class ImeTest extends ContentShellTestBase {
         dispatchKeyEvent(new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_DPAD_LEFT));
         dispatchKeyEvent(new KeyEvent(KeyEvent.ACTION_UP, KeyEvent.KEYCODE_DPAD_LEFT));
 
-        if (usingReplicaInputConnection()) {
-            // Ideally getTextBeforeCursor immediately after dispatchKeyEvent should return a
-            // correct value, but we have a stop-gap solution in render_widget_input_handler and it
-            // make take some round trip time until we get the correct value.
-            waitUntilGetCharacterBeforeCursorBecomes("l");
-        } else {
-            assertTextsAroundCursor("hell", null, "o");
-        }
-    }
-
-    private void waitUntilGetCharacterBeforeCursorBecomes(final String expectedText)
-            throws InterruptedException {
-        pollForCriteriaOnThread(Criteria.equals(expectedText, new Callable<String>() {
-            @Override
-            public String call() {
-                return (String) mConnection.getTextBeforeCursor(1, 0);
-            }
-        }));
-    }
-
-    private void pollForCriteriaOnThread(final Criteria criteria) throws InterruptedException {
-        final Callable<Boolean> callable = new Callable<Boolean>() {
-            @Override
-            public Boolean call() throws Exception {
-                return criteria.isSatisfied();
-            }
-        };
-        CriteriaHelper.pollInstrumentationThread(new Criteria() {
-            @Override
-            public boolean isSatisfied() {
-                try {
-                    return runBlockingOnImeThread(callable);
-                } catch (Exception e) {
-                    e.printStackTrace();
-                    fail();
-                    return false;
-                }
-            }
-
-            @Override
-            public String getFailureReason() {
-                return criteria.getFailureReason();
-            }
-        });
+        assertTextsAroundCursor("hell", null, "o");
     }
 
     @SmallTest
@@ -1079,7 +1135,8 @@ public class ImeTest extends ContentShellTestBase {
         CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
-                return mContentViewCore.isPastePopupShowing();
+                return mSelectionPopupController.isPastePopupShowing()
+                       && mSelectionPopupController.isInsertion();
             }
         });
 
@@ -1087,10 +1144,10 @@ public class ImeTest extends ContentShellTestBase {
         CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
-                return !mContentViewCore.isPastePopupShowing();
+                return !mSelectionPopupController.isPastePopupShowing();
             }
         });
-        assertFalse(mContentViewCore.hasInsertion());
+        assertFalse(mSelectionPopupController.isInsertion());
     }
 
     @SmallTest
@@ -1104,7 +1161,7 @@ public class ImeTest extends ContentShellTestBase {
 
         setComposingText("h", 1);
         assertWaitForSelectActionBarStatus(false);
-        assertFalse(mContentViewCore.hasSelection());
+        assertFalse(mSelectionPopupController.hasSelection());
     }
 
     @SmallTest
@@ -1112,11 +1169,11 @@ public class ImeTest extends ContentShellTestBase {
     public void testTextHandlesPreservedWithDpadNavigation() throws Throwable {
         DOMUtils.longPressNode(this, mContentViewCore, "plain_text");
         assertWaitForSelectActionBarStatus(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
 
         dispatchKeyEvent(new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_DPAD_DOWN));
         assertWaitForSelectActionBarStatus(true);
-        assertTrue(mContentViewCore.hasSelection());
+        assertTrue(mSelectionPopupController.hasSelection());
     }
 
     @MediumTest
@@ -1142,36 +1199,37 @@ public class ImeTest extends ContentShellTestBase {
 
     @MediumTest
     @Feature({"TextInput"})
-    public void testContentEditableEvents_SetComposingText() throws Throwable {
+    public void testContentEditableEvents_ComposingText() throws Throwable {
         focusElementAndWaitForStateUpdate("contenteditable_event");
         waitForEventLogs("selectionchange,selectionchange");
         clearEventLogs();
 
-        beginBatchEdit();
         setComposingText("a", 1);
-        finishComposingText();
-        endBatchEdit();
-        waitAndVerifyUpdateSelection(0, 1, 1, -1, -1);
-
-        // TODO(changwan): reduce the number of selection changes
+        waitAndVerifyUpdateSelection(0, 1, 1, 0, 1);
+        // TODO(changwan): reduce the number of selection changes.
         waitForEventLogs("keydown(229),compositionstart(),compositionupdate(a),input,keyup(229),"
-                + "compositionupdate(a),input,compositionend(a),selectionchange,selectionchange,"
-                + "selectionchange,selectionchange,selectionchange");
+                + "selectionchange,selectionchange");
+        clearEventLogs();
+
+        finishComposingText();
+        waitAndVerifyUpdateSelection(1, 1, 1, -1, -1);
+        waitForEventLogs(
+                "compositionend(a)");
     }
 
     @MediumTest
     @Feature({"TextInput"})
-    public void testInputTextEvents_SetComposingText() throws Throwable {
-        beginBatchEdit();
+    public void testInputTextEvents_ComposingText() throws Throwable {
         setComposingText("a", 1);
-        finishComposingText();
-        endBatchEdit();
-        waitAndVerifyUpdateSelection(0, 1, 1, -1, -1);
-
-        // TODO(changwan): reduce the number of selection changes
+        waitAndVerifyUpdateSelection(0, 1, 1, 0, 1);
+        // TODO(changwan): reduce the number of selection changes.
         waitForEventLogs("keydown(229),compositionstart(),compositionupdate(a),"
-                + "input,keyup(229),compositionupdate(a),input,compositionend(a),selectionchange,"
-                + "selectionchange,selectionchange,selectionchange,selectionchange");
+                + "input,keyup(229),selectionchange,selectionchange");
+        clearEventLogs();
+
+        finishComposingText();
+        waitAndVerifyUpdateSelection(1, 1, 1, -1, -1);
+        waitForEventLogs("compositionend(a)");
     }
 
     @MediumTest
@@ -1203,29 +1261,58 @@ public class ImeTest extends ContentShellTestBase {
         waitForEventLogs("selectionchange,selectionchange");
         clearEventLogs();
 
-        commitText("a", 1);
-        waitAndVerifyUpdateSelection(0, 1, 1, -1, -1);
+        commitText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
         waitForEventLogs("keydown(229),input,keyup(229),selectionchange");
         clearEventLogs();
 
-        deleteSurroundingText(1, 0);
-        waitAndVerifyUpdateSelection(1, 0, 0, -1, -1);
+        setSelection(2, 2);
+        waitAndVerifyUpdateSelection(1, 2, 2, -1, -1);
+        waitForEventLogs("selectionchange");
+        clearEventLogs();
 
-        waitForEventLogs("keydown(229),input,keyup(229),selectionchange,selectionchange");
+        deleteSurroundingText(1, 1);
+        waitAndVerifyUpdateSelection(2, 1, 1, -1, -1);
+        // TODO(yabinh): It should only fire 1 input and 1 selectionchange events.
+        waitForEventLogs("keydown(229),input,input,keyup(229),selectionchange,selectionchange,"
+                + "selectionchange,selectionchange");
     }
 
     @MediumTest
     @Feature({"TextInput"})
     public void testInputTextEvents_DeleteSurroundingText() throws Throwable {
-        commitText("a", 1);
-        waitAndVerifyUpdateSelection(0, 1, 1, -1, -1);
+        commitText("hello", 1);
+        waitAndVerifyUpdateSelection(0, 5, 5, -1, -1);
         waitForEventLogs("keydown(229),input,keyup(229),selectionchange");
         clearEventLogs();
 
-        deleteSurroundingText(1, 0);
-        waitAndVerifyUpdateSelection(1, 0, 0, -1, -1);
+        setSelection(2, 2);
+        waitAndVerifyUpdateSelection(1, 2, 2, -1, -1);
+        waitForEventLogs("selectionchange");
+        clearEventLogs();
 
-        waitForEventLogs("keydown(229),input,keyup(229),selectionchange,selectionchange");
+        deleteSurroundingText(1, 1);
+        waitAndVerifyUpdateSelection(2, 1, 1, -1, -1);
+        // TODO(yabinh): It should only fire 1 input and 1 selectionchange events.
+        waitForEventLogs("keydown(229),input,input,keyup(229),selectionchange,selectionchange,"
+                + "selectionchange,selectionchange");
+    }
+
+    @MediumTest
+    @Feature({"TextInput"})
+    public void testGetCursorCapsMode() throws Throwable {
+        focusElementAndWaitForStateUpdate("contenteditable_event");
+        commitText("Hello World", 1);
+        waitAndVerifyUpdateSelection(0, 11, 11, -1, -1);
+        assertEquals(0,
+                getCursorCapsMode(InputType.TYPE_TEXT_FLAG_CAP_WORDS));
+        setSelection(6, 6);
+        waitAndVerifyUpdateSelection(1, 6, 6, -1, -1);
+        assertEquals(InputType.TYPE_TEXT_FLAG_CAP_WORDS,
+                getCursorCapsMode(InputType.TYPE_TEXT_FLAG_CAP_WORDS));
+        commitText("\n", 1);
+        assertEquals(InputType.TYPE_TEXT_FLAG_CAP_WORDS,
+                getCursorCapsMode(InputType.TYPE_TEXT_FLAG_CAP_WORDS));
     }
 
     private void clearEventLogs() throws Exception {
@@ -1237,32 +1324,13 @@ public class ImeTest extends ContentShellTestBase {
     private void waitForEventLogs(String expectedLogs) throws Exception {
         final String code = "getEventLogs()";
         final String sanitizedExpectedLogs = "\"" + expectedLogs + "\"";
-        if (usingReplicaInputConnection()) {
-            // When using replica input connection, JavaScript update will lands later.
-            CriteriaHelper.pollInstrumentationThread(new Criteria() {
-                @Override
-                public boolean isSatisfied() {
-                    try {
-                        String eventLogs = JavaScriptUtils.executeJavaScriptAndWaitForResult(
-                                getContentViewCore().getWebContents(), code);
-                        updateFailureReason(eventLogs);
-                        return sanitizedExpectedLogs.equals(eventLogs);
-                    } catch (InterruptedException | TimeoutException e) {
-                        updateFailureReason(e.getMessage());
-                        return false;
-                    }
-                }
-            });
-        } else {
-            assertEquals(sanitizedExpectedLogs, JavaScriptUtils.executeJavaScriptAndWaitForResult(
-                    getContentViewCore().getWebContents(), code));
-        }
+        assertEquals(sanitizedExpectedLogs, JavaScriptUtils.executeJavaScriptAndWaitForResult(
+                                                    getContentViewCore().getWebContents(), code));
     }
 
     // https://crbug.com/604675
     @MediumTest
     @Feature({"TextInput"})
-    @CommandLineFlags.Add("enable-features=ImeThread")
     public void testAlertInKeyUpListenerDoesNotCrash() throws Exception {
         // Call 'alert()' when 'keyup' event occurs. Since we are in contentshell,
         // this does not actually pops up the alert window.
@@ -1280,7 +1348,6 @@ public class ImeTest extends ContentShellTestBase {
     // https://crbug.com/616334
     @SmallTest
     @Feature({"TextInput"})
-    @CommandLineFlags.Add("enable-features=ImeThread")
     public void testCastToBaseInputConnection() throws Exception {
         commitText("a", 1);
         final BaseInputConnection baseInputConnection = (BaseInputConnection) mConnection;
@@ -1316,6 +1383,30 @@ public class ImeTest extends ContentShellTestBase {
         waitAndVerifyUpdateSelection(0, 7, 7, -1, -1);
     }
 
+    // crbug.com/643477
+    @MediumTest
+    @Feature({"TextInput"})
+    public void testUiThreadAccess() throws Exception {
+        final ChromiumBaseInputConnection connection = mConnection;
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                // We allow UI thread access for most functions, except for
+                // beginBatchEdit(), endBatchEdit(), and get* methods().
+                assertTrue(connection.commitText("a", 1));
+                assertTrue(connection.setComposingText("b", 1));
+                assertTrue(connection.setComposingText("bc", 1));
+                assertTrue(connection.finishComposingText());
+            }
+        });
+        assertEquals("abc", runBlockingOnImeThread(new Callable<CharSequence>() {
+            @Override
+            public CharSequence call() throws Exception {
+                return connection.getTextBeforeCursor(5, 0);
+            }
+        }));
+    }
+
     private void performGo(TestCallbackHelperContainer testCallbackHelperContainer)
             throws Throwable {
         final InputConnection inputConnection = mConnection;
@@ -1342,7 +1433,7 @@ public class ImeTest extends ContentShellTestBase {
                 });
     }
 
-    private void assertWaitForKeyboardStatus(final boolean show) throws InterruptedException {
+    private void assertWaitForKeyboardStatus(final boolean show) {
         CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
@@ -1358,8 +1449,7 @@ public class ImeTest extends ContentShellTestBase {
         });
     }
 
-    private void assertWaitForSelectActionBarStatus(
-            final boolean show) throws InterruptedException {
+    private void assertWaitForSelectActionBarStatus(final boolean show) {
         CriteriaHelper.pollUiThread(Criteria.equals(show, new Callable<Boolean>() {
             @Override
             public Boolean call() {
@@ -1369,8 +1459,7 @@ public class ImeTest extends ContentShellTestBase {
     }
 
     private void waitAndVerifyUpdateSelection(final int index, final int selectionStart,
-            final int selectionEnd, final int compositionStart, final int compositionEnd)
-            throws InterruptedException {
+            final int selectionEnd, final int compositionStart, final int compositionEnd) {
         final List<Pair<Range, Range>> states = mInputMethodManagerWrapper.getUpdateSelectionList();
         CriteriaHelper.pollUiThread(new Criteria() {
             @Override
@@ -1389,8 +1478,7 @@ public class ImeTest extends ContentShellTestBase {
         mInputMethodManagerWrapper.getUpdateSelectionList().clear();
     }
 
-    private void assertClipboardContents(final Activity activity, final String expectedContents)
-            throws InterruptedException {
+    private void assertClipboardContents(final Activity activity, final String expectedContents) {
         CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
@@ -1431,6 +1519,8 @@ public class ImeTest extends ContentShellTestBase {
         });
     }
 
+    // After calling this method, we should call assertClipboardContents() to wait for the clipboard
+    // to get updated. See cubug.com/621046
     private void copy() {
         final WebContents webContents = mWebContents;
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
@@ -1451,6 +1541,18 @@ public class ImeTest extends ContentShellTestBase {
         });
     }
 
+    private void setClip(final CharSequence text) {
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                final ClipboardManager clipboardManager =
+                        (ClipboardManager) getActivity().getSystemService(
+                                Context.CLIPBOARD_SERVICE);
+                clipboardManager.setPrimaryClip(ClipData.newPlainText(null, text));
+            }
+        });
+    }
+
     private void paste() {
         final WebContents webContents = mWebContents;
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
@@ -1481,7 +1583,12 @@ public class ImeTest extends ContentShellTestBase {
         });
     }
 
-    private <T> T runBlockingOnImeThread(Callable<T> c) throws Exception {
+    /**
+     * Run the {@Callable} on IME thread (or UI thread if not applicable).
+     * @param c The callable
+     * @return The result from running the callable.
+     */
+    protected <T> T runBlockingOnImeThread(Callable<T> c) throws Exception {
         return ImeTestUtils.runBlockingOnHandler(mConnectionFactory.getHandler(), c);
     }
 
@@ -1597,6 +1704,16 @@ public class ImeTest extends ContentShellTestBase {
         });
     }
 
+    private int getCursorCapsMode(final int reqModes) throws Throwable {
+        final ChromiumBaseInputConnection connection = mConnection;
+        return runBlockingOnImeThread(new Callable<Integer>() {
+            @Override
+            public Integer call() {
+                return connection.getCursorCapsMode(reqModes);
+            }
+        });
+    }
+
     private void dispatchKeyEvent(final KeyEvent event) {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
@@ -1635,10 +1752,6 @@ public class ImeTest extends ContentShellTestBase {
         mConnection = getInputConnection();
     }
 
-    private boolean usingReplicaInputConnection() {
-        return mConnectionFactory.getHandler().getLooper() == Looper.getMainLooper();
-    }
-
     private static class TestInputConnectionFactory implements ChromiumBaseInputConnection.Factory {
         private final ChromiumBaseInputConnection.Factory mFactory;
 
@@ -1651,11 +1764,11 @@ public class ImeTest extends ContentShellTestBase {
 
         @Override
         public ChromiumBaseInputConnection initializeAndGet(View view, ImeAdapter imeAdapter,
-                int inputType, int inputFlags, int selectionStart, int selectionEnd,
+                int inputType, int inputFlags, int inputMode, int selectionStart, int selectionEnd,
                 EditorInfo outAttrs) {
             mTextInputTypeList.add(inputType);
             mOutAttrs = outAttrs;
-            return mFactory.initializeAndGet(view, imeAdapter, inputType, inputFlags,
+            return mFactory.initializeAndGet(view, imeAdapter, inputType, inputMode, inputFlags,
                     selectionStart, selectionEnd, outAttrs);
         }
 
