@@ -14,23 +14,26 @@
 #include <vector>
 
 #include "base/callback_forward.h"
-#include "base/memory/scoped_vector.h"
+#include "base/task_scheduler/task_scheduler.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "content/public/browser/certificate_request_result_type.h"
 #include "content/public/browser/navigation_throttle.h"
 #include "content/public/common/content_client.h"
 #include "content/public/common/media_stream_request.h"
-#include "content/public/common/mojo_application_info.h"
 #include "content/public/common/resource_type.h"
+#include "content/public/common/service_info.h"
 #include "content/public/common/socket_permission_request.h"
 #include "content/public/common/window_container_type.h"
 #include "gpu/config/gpu_control_list.h"
 #include "media/audio/audio_manager.h"
+#include "media/media_features.h"
+#include "media/mojo/interfaces/remoting.mojom.h"
 #include "net/base/mime_util.h"
 #include "net/cookies/canonical_cookie.h"
 #include "storage/browser/fileapi/file_system_context.h"
 #include "third_party/WebKit/public/platform/WebPageVisibilityState.h"
+#include "ui/base/page_transition_types.h"
 #include "ui/base/window_open_disposition.h"
 
 #if defined(OS_POSIX) && !defined(OS_MACOSX)
@@ -45,8 +48,8 @@ class GURL;
 
 namespace base {
 class CommandLine;
-class DictionaryValue;
 class FilePath;
+class SchedulerWorkerPoolParams;
 }
 
 namespace blink {
@@ -57,13 +60,17 @@ namespace gfx {
 class ImageSkia;
 }
 
+namespace gpu {
+class GpuChannelEstablishFactory;
+}
+
 namespace media {
 class CdmFactory;
 }
 
-namespace shell {
+namespace service_manager {
 class InterfaceRegistry;
-class ShellClient;
+class Service;
 }
 
 namespace net {
@@ -73,8 +80,10 @@ class SSLCertRequestInfo;
 class SSLInfo;
 class URLRequest;
 class URLRequestContext;
-class URLRequestContextGetter;
-class X509Certificate;
+}
+
+namespace rappor {
+class RapporService;
 }
 
 namespace sandbox {
@@ -90,7 +99,6 @@ class Origin;
 }
 
 namespace storage {
-class ExternalMountPoints;
 class FileSystemBackend;
 class QuotaEvictionPolicy;
 }
@@ -98,23 +106,22 @@ class QuotaEvictionPolicy;
 namespace content {
 
 enum class PermissionType;
-class AccessTokenStore;
 class BrowserChildProcessHost;
 class BrowserContext;
 class BrowserMainParts;
-class BrowserPluginGuestDelegate;
 class BrowserPpapiHost;
 class BrowserURLHandler;
 class ClientCertificateDelegate;
+class ControllerPresentationServiceDelegate;
 class DevToolsManagerDelegate;
-class ExternalVideoSurfaceContainer;
-class GeolocationDelegate;
-class LocationProvider;
+class GpuProcessHost;
 class MediaObserver;
+class MemoryCoordinatorDelegate;
 class NavigationHandle;
+class NavigationUIData;
 class PlatformNotificationService;
-class PresentationServiceDelegate;
 class QuotaPermissionContext;
+class ReceiverPresentationServiceDelegate;
 class RenderFrameHost;
 class RenderProcessHost;
 class RenderViewHost;
@@ -188,17 +195,17 @@ class CONTENT_EXPORT ContentBrowserClient {
   virtual bool ShouldUseProcessPerSite(BrowserContext* browser_context,
                                        const GURL& effective_url);
 
-  // Returns true if site isolation should be enabled for |effective_url|. This
-  // call allows the embedder to supplement the site isolation policy enforced
-  // by the content layer.
+  // Returns true if site isolation should be enabled for |effective_site_url|.
+  // This call allows the embedder to supplement the site isolation policy
+  // enforced by the content layer.
   //
   // Will only be called if both of the following happen:
   //   1. The embedder asked to be consulted, by returning true from
   //      ContentClient::IsSupplementarySiteIsolationModeEnabled().
-  //   2. The content layer didn't decide to isolate |effective_url| according
-  //      to its internal policy (e.g. because of --site-per-process).
+  //   2. The content layer didn't decide to isolate |effective_site_url|
+  //      according to its internal policy (e.g. because of --site-per-process).
   virtual bool DoesSiteRequireDedicatedProcess(BrowserContext* browser_context,
-                                               const GURL& effective_url);
+                                               const GURL& effective_site_url);
 
   // Returns true unless the effective URL is part of a site that cannot live in
   // a process restricted to just that site.  This is only called if site
@@ -233,21 +240,24 @@ class CONTENT_EXPORT ContentBrowserClient {
   // This is called on the UI thread.
   virtual bool CanCommitURL(RenderProcessHost* process_host, const GURL& url);
 
-  // Returns true if no URL within |origin| is allowed to commit in the given
-  // process.  Must return false if there exists at least one URL in |origin|
-  // that is allowed to commit.
-  // This is called on the IO thread.
-  virtual bool IsIllegalOrigin(ResourceContext* resource_context,
-                               int child_process_id,
-                               const GURL& origin);
-
   // Returns whether a URL should be allowed to open from a specific context.
   // This also applies in cases where the new URL will open in another process.
   virtual bool ShouldAllowOpenURL(SiteInstance* site_instance, const GURL& url);
 
-  // Allows the embedder to override OpenURLParams.
-  virtual void OverrideOpenURLParams(SiteInstance* site_instance,
-                                     OpenURLParams* params) {}
+  // Allows the embedder to override parameters when navigating. Called for both
+  // opening new URLs and when transferring URLs across processes.
+  virtual void OverrideNavigationParams(SiteInstance* site_instance,
+                                        ui::PageTransition* transition,
+                                        bool* is_renderer_initiated,
+                                        content::Referrer* referrer) {}
+
+  // Allows the embedder to override top document isolation for specific frames.
+  // |url| is the URL being loaded in the subframe, and |parent_site_instance|
+  // is the SiteInstance of the parent frame. Called only for subframes and only
+  // when top document isolation mode is enabled.
+  virtual bool ShouldFrameShareParentSiteInstanceDespiteTopDocumentIsolation(
+      const GURL& url,
+      SiteInstance* parent_site_instance);
 
   // Returns whether a new view for a given |site_url| can be launched in a
   // given |process_host|.
@@ -282,8 +292,7 @@ class CONTENT_EXPORT ContentBrowserClient {
 
   // Returns true if the given navigation redirect should cause a renderer
   // process swap.
-  // This is called on the IO thread.
-  virtual bool ShouldSwapProcessesForRedirect(ResourceContext* resource_context,
+  virtual bool ShouldSwapProcessesForRedirect(BrowserContext* browser_context,
                                               const GURL& current_url,
                                               const GURL& new_url);
 
@@ -291,10 +300,6 @@ class CONTENT_EXPORT ContentBrowserClient {
   // current SiteInstance, if it does not yet have a site.
   virtual bool ShouldAssignSiteForURL(const GURL& url);
 
-  // See CharacterEncoding's comment.
-  virtual std::string GetCanonicalEncodingNameByAliasName(
-      const std::string& alias_name);
-
   // Allows the embedder to pass extra command line flags.
   // switches::kProcessType will already be set at this point.
   virtual void AppendExtraCommandLineSwitches(base::CommandLine* command_line,
@@ -319,14 +324,14 @@ class CONTENT_EXPORT ContentBrowserClient {
 
   // Allow the embedder to control if a Service Worker can be associated
   // with the given scope.
-  // Invalid (-1) process/frame ids indicate this is for starting a service
-  // worker, which is not necessarily associated with a particular frame.
+  // A null |wc_getter| callback indicates this is for starting a service
+  // worker, which is not necessarily associated with a particular tab.
   // This is called on the IO thread.
-  virtual bool AllowServiceWorker(const GURL& scope,
-                                  const GURL& first_party,
-                                  content::ResourceContext* context,
-                                  int render_process_id,
-                                  int render_frame_id);
+  virtual bool AllowServiceWorker(
+      const GURL& scope,
+      const GURL& first_party,
+      ResourceContext* context,
+      const base::Callback<WebContents*(void)>& wc_getter);
 
   virtual bool IsDataSaverEnabled(BrowserContext* context);
 
@@ -370,17 +375,14 @@ class CONTENT_EXPORT ContentBrowserClient {
       ResourceContext* context,
       const std::vector<std::pair<int, int> >& render_frames);
 
-#if defined(ENABLE_WEBRTC)
+#if BUILDFLAG(ENABLE_WEBRTC)
   // Allow the embedder to control if WebRTC identities are allowed to be cached
   // and potentially reused for future requests (within the same origin).
   // This is called on the IO thread.
   virtual bool AllowWebRTCIdentityCache(const GURL& url,
                                         const GURL& first_party_url,
                                         ResourceContext* context);
-#endif  // defined(ENABLE_WEBRTC)
-
-  // Allow the embedder to control whether we can use <keygen>.
-  virtual bool AllowKeygen(const GURL& url, content::ResourceContext* context);
+#endif  // BUILDFLAG(ENABLE_WEBRTC)
 
   // Allow the embedder to control whether we can use Web Bluetooth.
   // TODO(crbug.com/589228): Replace this with a use of the permission system.
@@ -394,10 +396,10 @@ class CONTENT_EXPORT ContentBrowserClient {
       const url::Origin& requesting_origin,
       const url::Origin& embedding_origin);
 
-  // Returns a blacklist of UUIDs that have restrictions when accessed
-  // via Web Bluetooth. Parsed by BluetoothBlacklist::Add().
+  // Returns a blocklist of UUIDs that have restrictions when accessed
+  // via Web Bluetooth. Parsed by BluetoothBlocklist::Add().
   //
-  // The blacklist string must be a comma-separated list of UUID:exclusion
+  // The blocklist string must be a comma-separated list of UUID:exclusion
   // pairs. The pairs may be separated by whitespace. Pair components are
   // colon-separated and must not have whitespace around the colon.
   //
@@ -407,7 +409,7 @@ class CONTENT_EXPORT ContentBrowserClient {
   //
   // Example:
   // "1812:e, 00001800-0000-1000-8000-00805f9b34fb:w, ignored:1, alsoignored."
-  virtual std::string GetWebBluetoothBlacklist();
+  virtual std::string GetWebBluetoothBlocklist();
 
   // Allow the embedder to override the request context based on the URL for
   // certain operations, like cookie access. Returns nullptr to indicate the
@@ -462,20 +464,18 @@ class CONTENT_EXPORT ContentBrowserClient {
   // Informs the embedder that a certificate error has occured.  If
   // |overridable| is true and if |strict_enforcement| is false, the user
   // can ignore the error and continue. The embedder can call the callback
-  // asynchronously. If |result| is not set to
-  // CERTIFICATE_REQUEST_RESULT_TYPE_CONTINUE, the request will be cancelled
-  // or denied immediately, and the callback won't be run.
-  virtual void AllowCertificateError(WebContents* web_contents,
-                                     bool is_main_frame,
-                                     int cert_error,
-                                     const net::SSLInfo& ssl_info,
-                                     const GURL& request_url,
-                                     ResourceType resource_type,
-                                     bool overridable,
-                                     bool strict_enforcement,
-                                     bool expired_previous_decision,
-                                     const base::Callback<void(bool)>& callback,
-                                     CertificateRequestResultType* result) {}
+  // asynchronously.
+  virtual void AllowCertificateError(
+      WebContents* web_contents,
+      bool is_main_frame,
+      int cert_error,
+      const net::SSLInfo& ssl_info,
+      const GURL& request_url,
+      ResourceType resource_type,
+      bool overridable,
+      bool strict_enforcement,
+      bool expired_previous_decision,
+      const base::Callback<void(CertificateRequestResultType)>& callback) {}
 
   // Selects a SSL client certificate and returns it to the |delegate|. Note:
   // |delegate| may be called synchronously or asynchronously.
@@ -486,16 +486,6 @@ class CONTENT_EXPORT ContentBrowserClient {
       net::SSLCertRequestInfo* cert_request_info,
       std::unique_ptr<ClientCertificateDelegate> delegate);
 
-  // Adds a new installable certificate or private key.
-  // Typically used to install an X.509 user certificate.
-  // Note that it's up to the embedder to verify that the data is
-  // well-formed. |cert_data| will be nullptr if |cert_size| is 0.
-  virtual void AddCertificate(net::CertificateMimeType cert_type,
-                              const void* cert_data,
-                              size_t cert_size,
-                              int render_process_id,
-                              int render_frame_id) {}
-
   // Returns a class to get notifications about media event. The embedder can
   // return nullptr if they're not interested.
   virtual MediaObserver* GetMediaObserver();
@@ -509,20 +499,20 @@ class CONTENT_EXPORT ContentBrowserClient {
   // type. If true is returned, |no_javascript_access| will indicate whether
   // the window that is created should be scriptable/in the same process.
   // This is called on the IO thread.
-  virtual bool CanCreateWindow(const GURL& opener_url,
+  virtual bool CanCreateWindow(int opener_render_process_id,
+                               int opener_render_frame_id,
+                               const GURL& opener_url,
                                const GURL& opener_top_level_frame_url,
                                const GURL& source_origin,
                                WindowContainerType container_type,
                                const GURL& target_url,
                                const Referrer& referrer,
+                               const std::string& frame_name,
                                WindowOpenDisposition disposition,
                                const blink::WebWindowFeatures& features,
                                bool user_gesture,
                                bool opener_suppressed,
                                ResourceContext* context,
-                               int render_process_id,
-                               int opener_render_view_id,
-                               int opener_render_frame_id,
                                bool* no_javascript_access);
 
   // Notifies the embedder that the ResourceDispatcherHost has been created.
@@ -537,13 +527,6 @@ class CONTENT_EXPORT ContentBrowserClient {
   // Getters for common objects.
   virtual net::NetLog* GetNetLog();
 
-  // Allows the embedder to provide a Delegate for Geolocation to override some
-  // functionality of the API (e.g. AccessTokenStore, LocationProvider).
-  virtual GeolocationDelegate* CreateGeolocationDelegate();
-
-  // Returns true if fast shutdown is possible.
-  virtual bool IsFastShutdownPossible();
-
   // Called by WebContents to override the WebKit preferences that are used by
   // the renderer. The content layer will add its own settings, and then it's up
   // to the embedder to update it if it wants.
@@ -560,6 +543,17 @@ class CONTENT_EXPORT ContentBrowserClient {
   // Clears browser cookies.
   virtual void ClearCookies(RenderFrameHost* rfh) {}
 
+  // Clears |browser_context|'s data stored for the given |origin|.
+  // The datatypes to be removed are specified by |remove_cookies|,
+  // |remove_storage|, and |remove_cache|. Note that cookies should be removed
+  // for the entire eTLD+1 of |origin|. Must call |callback| when finished.
+  virtual void ClearSiteData(content::BrowserContext* browser_context,
+                             const url::Origin& origin,
+                             bool remove_cookies,
+                             bool remove_storage,
+                             bool remove_cache,
+                             const base::Closure& callback) {}
+
   // Returns the default download directory.
   // This can be called on any thread.
   virtual base::FilePath GetDefaultDownloadDirectory();
@@ -580,6 +574,9 @@ class CONTENT_EXPORT ContentBrowserClient {
   virtual BrowserPpapiHost* GetExternalBrowserPpapiHost(
       int plugin_child_id);
 
+  // Gets the factory to use to establish a connection to the GPU process.
+  virtual gpu::GpuChannelEstablishFactory* GetGpuChannelEstablishFactory();
+
   // Returns true if the socket operation specified by |params| is allowed from
   // the given |browser_context| and |url|. If |params| is nullptr, this method
   // checks the basic "socket" permission, which is for those operations that
@@ -627,7 +624,8 @@ class CONTENT_EXPORT ContentBrowserClient {
   virtual void GetAdditionalFileSystemBackends(
       BrowserContext* browser_context,
       const base::FilePath& storage_partition_path,
-      ScopedVector<storage::FileSystemBackend>* additional_backends) {}
+      std::vector<std::unique_ptr<storage::FileSystemBackend>>*
+          additional_backends) {}
 
   // Creates a new DevToolsManagerDelegate. The caller owns the returned value.
   // It's valid to return nullptr.
@@ -648,64 +646,80 @@ class CONTENT_EXPORT ContentBrowserClient {
       BrowserContext* browser_context,
       const GURL& url);
 
-  // Generate a Shell user-id for the supplied browser context. Defaults to
+  // Generate a Service user-id for the supplied browser context. Defaults to
   // returning a random GUID.
-  virtual std::string GetShellUserIdForBrowserContext(
+  virtual std::string GetServiceUserIdForBrowserContext(
       BrowserContext* browser_context);
 
-  // Allows to register browser Mojo interfaces exposed through the
-  // RenderProcessHost.
+  // Allows to register browser interfaces exposed through the
+  // RenderProcessHost. Note that interface factory callbacks added to
+  // |registry| will by default be run immediately on the IO thread, unless a
+  // task runner is provided.
   virtual void ExposeInterfacesToRenderer(
-      shell::InterfaceRegistry* registry,
+      service_manager::InterfaceRegistry* registry,
       RenderProcessHost* render_process_host) {}
 
-  // Allows to register browser Mojo interfaces exposed through the
-  // FrameMojoShell.
-  virtual void RegisterFrameMojoShellInterfaces(
-      shell::InterfaceRegistry* registry,
+  // Called when RenderFrameHostImpl connects to the Media service. Expose
+  // interfaces to the service using |registry|.
+  virtual void ExposeInterfacesToMediaService(
+      service_manager::InterfaceRegistry* registry,
       RenderFrameHost* render_frame_host) {}
 
   // Allows to register browser Mojo interfaces exposed through the
   // RenderFrameHost.
   virtual void RegisterRenderFrameMojoInterfaces(
-      shell::InterfaceRegistry* registry,
+      service_manager::InterfaceRegistry* registry,
       RenderFrameHost* render_frame_host) {}
 
-  using StaticMojoApplicationMap = std::map<std::string, MojoApplicationInfo>;
+  // Allows to register browser Mojo interfaces exposed through the
+  // GpuProcessHost. Called on the IO thread. Note that interface factory
+  // callbacks added to |registry| will by default be run immediately on the IO
+  // thread, unless a task runner is provided.
+  virtual void ExposeInterfacesToGpuProcess(
+      service_manager::InterfaceRegistry* registry,
+      GpuProcessHost* render_process_host) {}
 
-  // Registers Mojo applications to be loaded in the browser process by the
-  // browser's global Mojo shell.
-  virtual void RegisterInProcessMojoApplications(
-      StaticMojoApplicationMap* apps) {}
+  using StaticServiceMap = std::map<std::string, ServiceInfo>;
 
-  using OutOfProcessMojoApplicationMap = std::map<std::string, base::string16>;
+  // Registers services to be loaded in the browser process by the Service
+  // Manager.
+  virtual void RegisterInProcessServices(StaticServiceMap* services) {}
 
-  // Registers Mojo applications to be loaded out of the browser process, in a
-  // sandboxed utility process. The value of each map entry should be the
-  // process name to use for the application's host process when launched.
-  virtual void RegisterOutOfProcessMojoApplications(
-      OutOfProcessMojoApplicationMap* apps) {}
+  using OutOfProcessServiceMap = std::map<std::string, base::string16>;
 
-  // Registers Mojo applications to be loaded out of the browser process (in
-  // a utility process) without the sandbox.
+  // Registers services to be loaded out of the browser process, in a sandboxed
+  // utility process. The value of each map entry should be the process name to
+  // use for the service's host process when launched.
+  virtual void RegisterOutOfProcessServices(OutOfProcessServiceMap* services) {}
+
+  // Registers services to be loaded out of the browser process (in a utility
+  // process) without the sandbox.
   //
-  // WARNING: This path is NOT recommended! If a Mojo application needs a
-  // service that is only available out of the sandbox, it could ask the browser
-  // process to provide it (e.g. through OverrideFrameMojoShellServices()). Only
-  // use this method when that approach does not work.
-  virtual void RegisterUnsandboxedOutOfProcessMojoApplications(
-      OutOfProcessMojoApplicationMap* apps) {}
+  // WARNING: This path is NOT recommended! If a service needs another service
+  // that is only available out of the sandbox, it could ask the browser
+  // process to provide it. Only use this method when that approach does not
+  // work.
+  virtual void RegisterUnsandboxedOutOfProcessServices(
+      OutOfProcessServiceMap* services) {}
 
-  // A map of Mojo application names to corresponding manifest contents.
-  using MojoApplicationManifestMap = std::map<std::string, std::string>;
+  // Allow the embedder to provide a dictionary loaded from a JSON file
+  // resembling a service manifest whose capabilities section will be merged
+  // with content's own for |name|. Additional entries will be appended to their
+  // respective sections.
+  virtual std::unique_ptr<base::Value> GetServiceManifestOverlay(
+      base::StringPiece name);
 
-  // Registers manifest contents for Mojo applications.
-  // See "services/shell/manifest.json" for an example Mojo app manifest.
-  //
-  // TODO(rockot): http://crbug.com/610426. Add more documentations about
-  //  Mojo app manifest.
-  virtual void RegisterMojoApplicationManifests(
-      MojoApplicationManifestMap* manifests) {}
+  struct ServiceManifestInfo {
+    // The name of the service.
+    std::string name;
+
+    // The resource ID of the manifest.
+    int resource_id;
+  };
+
+  // Allows the embedder to provide extra service manifests to be registered
+  // with the service manager context.
+  virtual std::vector<ServiceManifestInfo> GetExtraServiceManifests();
 
   // Allows to override the visibility state of a RenderFrameHost.
   // |visibility_state| should not be null. It will only be set if needed.
@@ -713,10 +727,17 @@ class CONTENT_EXPORT ContentBrowserClient {
       RenderFrameHost* render_frame_host,
       blink::WebPageVisibilityState* visibility_state) {}
 
-  // Allows an embedder to provide its own PresentationServiceDelegate
+  // Allows an embedder to provide its own ControllerPresentationServiceDelegate
   // implementation. Returns nullptr if unavailable.
-  virtual PresentationServiceDelegate* GetPresentationServiceDelegate(
-      WebContents* web_contents);
+  virtual ControllerPresentationServiceDelegate*
+  GetControllerPresentationServiceDelegate(WebContents* web_contents);
+
+  // Allows an embedder to provide its own ReceiverPresentationServiceDelegate
+  // implementation. Returns nullptr if unavailable. Only WebContents created
+  // for offscreen presentations should be passed to this API. The WebContents
+  // must belong to an incognito profile.
+  virtual ReceiverPresentationServiceDelegate*
+  GetReceiverPresentationServiceDelegate(WebContents* web_contents);
 
   // Allows programmatic opening of a new tab/window without going through
   // another WebContents. For example, from a Worker. |callback| will be
@@ -733,7 +754,13 @@ class CONTENT_EXPORT ContentBrowserClient {
   // to control the flow of a navigation on the UI thread. The embedder is
   // guaranteed that the throttles will be executed in the order they were
   // provided.
-  virtual ScopedVector<NavigationThrottle> CreateThrottlesForNavigation(
+  virtual std::vector<std::unique_ptr<NavigationThrottle>>
+  CreateThrottlesForNavigation(NavigationHandle* navigation_handle);
+
+  // PlzNavigate
+  // Called at the start of the navigation to get opaque data the embedder
+  // wants to see passed to the corresponding URLRequest on the IO thread.
+  virtual std::unique_ptr<NavigationUIData> GetNavigationUIData(
       NavigationHandle* navigation_handle);
 
   // Allows the embedder to provide its own AudioManager implementation.
@@ -747,23 +774,14 @@ class CONTENT_EXPORT ContentBrowserClient {
 
   // Populates |mappings| with all files that need to be mapped before launching
   // a child process.
-#if defined(OS_ANDROID)
-  virtual void GetAdditionalMappedFilesForChildProcess(
-      const base::CommandLine& command_line,
-      int child_process_id,
-      content::FileDescriptorInfo* mappings,
-      std::map<int, base::MemoryMappedFile::Region>* regions) {}
-#elif defined(OS_POSIX) && !defined(OS_MACOSX)
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
   virtual void GetAdditionalMappedFilesForChildProcess(
       const base::CommandLine& command_line,
       int child_process_id,
       content::FileDescriptorInfo* mappings) {}
-#endif  // defined(OS_ANDROID)
+#endif  // defined(OS_POSIX) && !defined(OS_MACOSX)
 
 #if defined(OS_WIN)
-  // Returns the name of the dll that contains cursors and other resources.
-  virtual const wchar_t* GetResourceDllName();
-
   // This is called on the PROCESS_LAUNCHER thread before the renderer process
   // is launched. It gives the embedder a chance to add loosen the sandbox
   // policy.
@@ -774,19 +792,40 @@ class CONTENT_EXPORT ContentBrowserClient {
   // an AppContainer.
   virtual base::string16 GetAppContainerSidForSandboxType(
       int sandbox_type) const;
-
-  // Returns whether the Win32k lockdown process mitigation should be applied to
-  // a process hosting a plugin with the specified |mime_type|.
-  virtual bool IsWin32kLockdownEnabledForMimeType(
-      const std::string& mime_type) const;
 #endif
 
-#if defined(VIDEO_HOLE)
-  // Allows an embedder to provide its own ExternalVideoSurfaceContainer
-  // implementation.  Return nullptr to disable external surface video.
-  virtual ExternalVideoSurfaceContainer*
-  OverrideCreateExternalVideoSurfaceContainer(WebContents* web_contents);
-#endif
+  // Returns an instance of MemoryCoordinatorDelegate.
+  virtual std::unique_ptr<MemoryCoordinatorDelegate>
+  GetMemoryCoordinatorDelegate();
+
+  // Binds a new media remoter service to |request|, if supported by the
+  // embedder, for the |source| that lives in the render frame represented
+  // by |render_frame_host|. This may be called multiple times if there is more
+  // than one source candidate in the same render frame.
+  virtual void CreateMediaRemoter(RenderFrameHost* render_frame_host,
+                                  media::mojom::RemotingSourcePtr source,
+                                  media::mojom::RemoterRequest request) {}
+
+  // Returns the RapporService from the browser process.
+  virtual ::rappor::RapporService* GetRapporService();
+
+  // Provides parameters for initializing the global task scheduler. If
+  // |params_vector| is empty, default parameters are used.
+  virtual void GetTaskSchedulerInitializationParams(
+      std::vector<base::SchedulerWorkerPoolParams>* params_vector,
+      base::TaskScheduler::WorkerPoolIndexForTraitsCallback*
+          index_to_traits_callback) {}
+
+  // Performs any necessary PostTask API redirection to the task scheduler.
+  virtual void PerformExperimentalTaskSchedulerRedirections() {}
+
+  // Returns true if the DOMStorageTaskRunner should be redirected to the task
+  // scheduler.
+  virtual bool ShouldRedirectDOMStorageTaskRunner();
+
+  // If this returns true, all BrowserThreads (but UI/IO) that support it on
+  // this platform will experimentally be redirected to TaskScheduler.
+  virtual bool RedirectNonUINonIOBrowserThreadsToTaskScheduler();
 
   virtual gpu::GpuControlList::OsType GetOsTypeOverrideForGpuDataManager(
       std::string* os_version);
