@@ -16,7 +16,7 @@ class ListValue;
 
 namespace content {
 
-class RenderViewHost;
+class RenderFrameHost;
 class WebUI;
 
 // A WebUI page is controller by the embedder's WebUIController object. It
@@ -32,9 +32,10 @@ class CONTENT_EXPORT WebUIController {
                                           const std::string& message,
                                           const base::ListValue& args);
 
-  // Called when RenderView is first created. This is *not* called for every
-  // page load because in some cases a RenderView will be reused.
-  virtual void RenderViewCreated(RenderViewHost* render_view_host) {}
+  // Called when a RenderFrame is created.  This is *not* called for every
+  // page load because in some cases a RenderFrame will be reused, for example
+  // when reloading or navigating to a same-site URL.
+  virtual void RenderFrameCreated(RenderFrameHost* render_frame_host) {}
 
   WebUI* web_ui() const { return web_ui_; }
 
