@@ -10,20 +10,23 @@
 
 #include "base/location.h"
 #include "base/memory/ptr_util.h"
+#include "base/metrics/field_trial.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "build/build_config.h"
-#include "components/scheduler/renderer/renderer_scheduler.h"
 #include "content/app/mojo/mojo_init.h"
 #include "content/common/dom_storage/dom_storage_types.h"
 #include "content/common/frame_messages.h"
 #include "content/common/input_messages.h"
+#include "content/common/renderer.mojom.h"
 #include "content/common/resize_params.h"
 #include "content/common/site_isolation_policy.h"
 #include "content/common/view_messages.h"
 #include "content/public/browser/content_browser_client.h"
 #include "content/public/browser/native_web_keyboard_event.h"
 #include "content/public/common/content_client.h"
+#include "content/public/common/content_switches.h"
+#include "content/public/common/previews_state.h"
 #include "content/public/common/renderer_preferences.h"
 #include "content/public/renderer/content_renderer_client.h"
 #include "content/public/test/frame_load_waiter.h"
@@ -37,17 +40,20 @@
 #include "content/test/mock_render_process.h"
 #include "content/test/test_content_client.h"
 #include "content/test/test_render_frame.h"
-#include "third_party/WebKit/public/platform/WebScreenInfo.h"
+#include "third_party/WebKit/public/platform/WebGestureEvent.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebMouseEvent.h"
 #include "third_party/WebKit/public/platform/WebURLRequest.h"
+#include "third_party/WebKit/public/platform/scheduler/renderer/renderer_scheduler.h"
 #include "third_party/WebKit/public/web/WebDocument.h"
 #include "third_party/WebKit/public/web/WebHistoryItem.h"
 #include "third_party/WebKit/public/web/WebInputElement.h"
-#include "third_party/WebKit/public/web/WebInputEvent.h"
 #include "third_party/WebKit/public/web/WebKit.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
 #include "third_party/WebKit/public/web/WebScriptSource.h"
 #include "third_party/WebKit/public/web/WebView.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/events/base_event_utils.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "v8/include/v8.h"
 
@@ -114,9 +120,8 @@ class RendererBlinkPlatformImplTestOverrideImpl
     : public RendererBlinkPlatformImpl {
  public:
   RendererBlinkPlatformImplTestOverrideImpl(
-      scheduler::RendererScheduler* scheduler)
-      : RendererBlinkPlatformImpl(scheduler, nullptr) {
-  }
+      blink::scheduler::RendererScheduler* scheduler)
+      : RendererBlinkPlatformImpl(scheduler, nullptr) {}
 
   // Get rid of the dependency to the sandbox, which is not available in
   // RenderViewTest.
@@ -125,7 +130,7 @@ class RendererBlinkPlatformImplTestOverrideImpl
 
 RenderViewTest::RendererBlinkPlatformImplTestOverride::
     RendererBlinkPlatformImplTestOverride() {
-  renderer_scheduler_ = scheduler::RendererScheduler::Create();
+  renderer_scheduler_ = blink::scheduler::RendererScheduler::Create();
   blink_platform_impl_.reset(
       new RendererBlinkPlatformImplTestOverrideImpl(renderer_scheduler_.get()));
 }
@@ -155,7 +160,7 @@ RenderViewTest::~RenderViewTest() {
 void RenderViewTest::ProcessPendingMessages() {
   msg_loop_.task_runner()->PostTask(FROM_HERE,
                                     base::MessageLoop::QuitWhenIdleClosure());
-  msg_loop_.Run();
+  base::RunLoop().Run();
 }
 
 WebLocalFrame* RenderViewTest::GetMainFrame() {
@@ -229,6 +234,13 @@ void RenderViewTest::GoForward(const GURL& url, const PageState& state) {
 }
 
 void RenderViewTest::SetUp() {
+  // Initialize mojo firstly to enable Blink initialization to use it.
+  InitializeMojo();
+  test_io_thread_.reset(new base::TestIOThread(base::TestIOThread::kAutoStart));
+  ipc_support_.reset(new mojo::edk::ScopedIPCSupport(
+      test_io_thread_->task_runner(),
+      mojo::edk::ScopedIPCSupport::ShutdownPolicy::FAST));
+
   // Blink needs to be initialized before calling CreateContentRendererClient()
   // because it uses blink internally.
   blink::initialize(blink_platform_impl_.Get());
@@ -263,6 +275,10 @@ void RenderViewTest::SetUp() {
   autorelease_pool_.reset(new base::mac::ScopedNSAutoreleasePool());
 #endif
   command_line_.reset(new base::CommandLine(base::CommandLine::NO_PROGRAM));
+  field_trial_list_.reset(new base::FieldTrialList(nullptr));
+  // We don't use the descriptor here anyways so it's ok to pass -1.
+  base::FieldTrialList::CreateTrialsFromCommandLine(
+      *command_line_, switches::kFieldTrialHandle, -1);
   params_.reset(new MainFunctionParams(*command_line_));
   platform_.reset(new RendererMainPlatformDelegate(*params_));
   platform_->PlatformInitialize();
@@ -287,7 +303,7 @@ void RenderViewTest::SetUp() {
   compositor_deps_.reset(new FakeCompositorDependencies);
   mock_process_.reset(new MockRenderProcess);
 
-  ViewMsg_New_Params view_params;
+  mojom::CreateViewParams view_params;
   view_params.opener_frame_route_id = MSG_ROUTING_NONE;
   view_params.window_was_created_with_opener = false;
   view_params.renderer_preferences = RendererPreferences();
@@ -301,20 +317,14 @@ void RenderViewTest::SetUp() {
   view_params.proxy_routing_id = MSG_ROUTING_NONE;
   view_params.hidden = false;
   view_params.never_visible = false;
-  view_params.next_page_id = 1;
   view_params.initial_size = *InitialSizeParams();
   view_params.enable_auto_resize = false;
   view_params.min_size = gfx::Size();
   view_params.max_size = gfx::Size();
 
-  InitializeMojo();
-  test_io_thread_.reset(new base::TestIOThread(base::TestIOThread::kAutoStart));
-  ipc_support_.reset(
-      new mojo::edk::test::ScopedIPCSupport(test_io_thread_->task_runner()));
-
   // This needs to pass the mock render thread to the view.
-  RenderViewImpl* view =
-      RenderViewImpl::Create(compositor_deps_.get(), view_params, false);
+  RenderViewImpl* view = RenderViewImpl::Create(
+      compositor_deps_.get(), view_params, RenderWidget::ShowCallback());
   view_ = view;
 }
 
@@ -338,17 +348,12 @@ void RenderViewTest::TearDown() {
   base::RunLoop().RunUntilIdle();
 
 #if defined(OS_MACOSX)
-  // Needs to run before blink::shutdown().
   autorelease_pool_.reset(NULL);
 #endif
 
   leak_detector->collectGarbageAndReport();
 
-  base::RunLoop().RunUntilIdle();
-
   blink_platform_impl_.Shutdown();
-  blink::shutdown();
-
   platform_->PlatformUninitialize();
   platform_.reset();
   params_.reset();
@@ -364,10 +369,11 @@ void RenderViewTest::onLeakDetectionComplete(const Result& result) {
   EXPECT_EQ(0u, result.numberOfLiveNodes);
   EXPECT_EQ(0u, result.numberOfLiveLayoutObjects);
   EXPECT_EQ(0u, result.numberOfLiveResources);
-  EXPECT_EQ(0u, result.numberOfLiveActiveDOMObjects);
+  EXPECT_EQ(0u, result.numberOfLiveSuspendableObjects);
   EXPECT_EQ(0u, result.numberOfLiveScriptPromises);
   EXPECT_EQ(0u, result.numberOfLiveFrames);
   EXPECT_EQ(0u, result.numberOfLiveV8PerContextData);
+  EXPECT_EQ(0u, result.numberOfWorkerGlobalScopes);
 }
 
 void RenderViewTest::SendNativeKeyEvent(
@@ -447,9 +453,10 @@ bool RenderViewTest::SimulateElementClick(const std::string& element_id) {
 }
 
 void RenderViewTest::SimulatePointClick(const gfx::Point& point) {
-  WebMouseEvent mouse_event;
-  mouse_event.type = WebInputEvent::MouseDown;
-  mouse_event.button = WebMouseEvent::ButtonLeft;
+  WebMouseEvent mouse_event(WebInputEvent::MouseDown,
+                            WebInputEvent::NoModifiers,
+                            ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  mouse_event.button = WebMouseEvent::Button::Left;
   mouse_event.x = point.x();
   mouse_event.y = point.y();
   mouse_event.clickCount = 1;
@@ -457,7 +464,7 @@ void RenderViewTest::SimulatePointClick(const gfx::Point& point) {
   impl->OnMessageReceived(InputMsg_HandleInputEvent(
       0, &mouse_event, ui::LatencyInfo(),
       InputEventDispatchType::DISPATCH_TYPE_BLOCKING));
-  mouse_event.type = WebInputEvent::MouseUp;
+  mouse_event.setType(WebInputEvent::MouseUp);
   impl->OnMessageReceived(InputMsg_HandleInputEvent(
       0, &mouse_event, ui::LatencyInfo(),
       InputEventDispatchType::DISPATCH_TYPE_BLOCKING));
@@ -473,9 +480,10 @@ bool RenderViewTest::SimulateElementRightClick(const std::string& element_id) {
 }
 
 void RenderViewTest::SimulatePointRightClick(const gfx::Point& point) {
-  WebMouseEvent mouse_event;
-  mouse_event.type = WebInputEvent::MouseDown;
-  mouse_event.button = WebMouseEvent::ButtonRight;
+  WebMouseEvent mouse_event(WebInputEvent::MouseDown,
+                            WebInputEvent::NoModifiers,
+                            ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  mouse_event.button = WebMouseEvent::Button::Right;
   mouse_event.x = point.x();
   mouse_event.y = point.y();
   mouse_event.clickCount = 1;
@@ -483,20 +491,21 @@ void RenderViewTest::SimulatePointRightClick(const gfx::Point& point) {
   impl->OnMessageReceived(InputMsg_HandleInputEvent(
       0, &mouse_event, ui::LatencyInfo(),
       InputEventDispatchType::DISPATCH_TYPE_BLOCKING));
-  mouse_event.type = WebInputEvent::MouseUp;
+  mouse_event.setType(WebInputEvent::MouseUp);
   impl->OnMessageReceived(InputMsg_HandleInputEvent(
       0, &mouse_event, ui::LatencyInfo(),
       InputEventDispatchType::DISPATCH_TYPE_BLOCKING));
 }
 
 void RenderViewTest::SimulateRectTap(const gfx::Rect& rect) {
-  WebGestureEvent gesture_event;
+  WebGestureEvent gesture_event(
+      WebInputEvent::GestureTap, WebInputEvent::NoModifiers,
+      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
   gesture_event.x = rect.CenterPoint().x();
   gesture_event.y = rect.CenterPoint().y();
   gesture_event.data.tap.tapCount = 1;
   gesture_event.data.tap.width = rect.width();
   gesture_event.data.tap.height = rect.height();
-  gesture_event.type = WebInputEvent::GestureTap;
   gesture_event.sourceDevice = blink::WebGestureDeviceTouchpad;
   RenderViewImpl* impl = static_cast<RenderViewImpl*>(view_);
   impl->OnMessageReceived(InputMsg_HandleInputEvent(
@@ -515,7 +524,7 @@ void RenderViewTest::Reload(const GURL& url) {
       url, Referrer(), ui::PAGE_TRANSITION_LINK, FrameMsg_Navigate_Type::RELOAD,
       true, false, base::TimeTicks(),
       FrameMsg_UILoadMetricsReportType::NO_REPORT, GURL(), GURL(),
-      LOFI_UNSPECIFIED, base::TimeTicks::Now(), "GET", nullptr);
+      PREVIEWS_UNSPECIFIED, base::TimeTicks::Now(), "GET", nullptr);
   RenderViewImpl* impl = static_cast<RenderViewImpl*>(view_);
   TestRenderFrame* frame =
       static_cast<TestRenderFrame*>(impl->GetMainRenderFrame());
@@ -530,15 +539,13 @@ uint32_t RenderViewTest::GetNavigationIPCType() {
 }
 
 void RenderViewTest::Resize(gfx::Size new_size,
-                            gfx::Rect resizer_rect,
                             bool is_fullscreen_granted) {
   ResizeParams params;
-  params.screen_info = blink::WebScreenInfo();
+  params.screen_info = ScreenInfo();
   params.new_size = new_size;
   params.physical_backing_size = new_size;
   params.top_controls_height = 0.f;
-  params.top_controls_shrink_blink_size = false;
-  params.resizer_rect = resizer_rect;
+  params.browser_controls_shrink_blink_size = false;
   params.is_fullscreen_granted = is_fullscreen_granted;
   params.display_mode = blink::WebDisplayModeBrowser;
   std::unique_ptr<IPC::Message> resize_message(new ViewMsg_Resize(0, params));
@@ -547,21 +554,23 @@ void RenderViewTest::Resize(gfx::Size new_size,
 
 void RenderViewTest::SimulateUserTypingASCIICharacter(char ascii_character,
                                                       bool flush_message_loop) {
-  blink::WebKeyboardEvent event;
-  event.text[0] = ascii_character;
-  ASSERT_TRUE(GetWindowsKeyCode(ascii_character, &event.windowsKeyCode));
+  int modifiers = blink::WebInputEvent::NoModifiers;
   if (isupper(ascii_character) || ascii_character == '@' ||
       ascii_character == '_') {
-    event.modifiers = blink::WebKeyboardEvent::ShiftKey;
+    modifiers = blink::WebKeyboardEvent::ShiftKey;
   }
 
-  event.type = blink::WebKeyboardEvent::RawKeyDown;
+  blink::WebKeyboardEvent event(
+      blink::WebKeyboardEvent::RawKeyDown, modifiers,
+      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  event.text[0] = ascii_character;
+  ASSERT_TRUE(GetWindowsKeyCode(ascii_character, &event.windowsKeyCode));
   SendWebKeyboardEvent(event);
 
-  event.type = blink::WebKeyboardEvent::Char;
+  event.setType(blink::WebKeyboardEvent::Char);
   SendWebKeyboardEvent(event);
 
-  event.type = blink::WebKeyboardEvent::KeyUp;
+  event.setType(blink::WebKeyboardEvent::KeyUp);
   SendWebKeyboardEvent(event);
 
   if (flush_message_loop) {
@@ -620,10 +629,9 @@ void RenderViewTest::DidNavigateWithinPage(blink::WebLocalFrame* frame,
 
 blink::WebWidget* RenderViewTest::GetWebWidget() {
   RenderViewImpl* impl = static_cast<RenderViewImpl*>(view_);
-  return impl->webwidget();
+  return impl->GetWebWidget();
 }
 
-
 ContentClient* RenderViewTest::CreateContentClient() {
   return new TestContentClient;
 }
@@ -637,7 +645,7 @@ ContentRendererClient* RenderViewTest::CreateContentRendererClient() {
 }
 
 std::unique_ptr<ResizeParams> RenderViewTest::InitialSizeParams() {
-  return base::WrapUnique(new ResizeParams());
+  return base::MakeUnique<ResizeParams>();
 }
 
 void RenderViewTest::GoToOffset(int offset,
@@ -653,10 +661,9 @@ void RenderViewTest::GoToOffset(int offset,
       url, Referrer(), ui::PAGE_TRANSITION_FORWARD_BACK,
       FrameMsg_Navigate_Type::NORMAL, true, false, base::TimeTicks(),
       FrameMsg_UILoadMetricsReportType::NO_REPORT, GURL(), GURL(),
-      LOFI_UNSPECIFIED, base::TimeTicks::Now(), "GET", nullptr);
+      PREVIEWS_UNSPECIFIED, base::TimeTicks::Now(), "GET", nullptr);
   RequestNavigationParams request_params;
   request_params.page_state = state;
-  request_params.page_id = impl->page_id_ + offset;
   request_params.nav_entry_id = pending_offset + 1;
   request_params.pending_history_list_offset = pending_offset;
   request_params.current_history_list_offset = impl->history_list_offset_;
