@@ -20,7 +20,6 @@
 #include "base/files/scoped_temp_dir.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/linked_ptr.h"
 #include "base/message_loop/message_loop.h"
 #include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
@@ -40,6 +39,10 @@
 #include "net/base/escape.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if defined(OS_POSIX)
+#include "base/files/file_descriptor_watcher_posix.h"
+#endif
+
 #if defined(OS_WIN)
 #include "base/base_switches.h"
 #include "content/common/sandbox_win.h"
@@ -236,13 +239,13 @@ size_t WrapperTestLauncherDelegate::RunTests(
     std::string full_name(test_names[i]);
     std::string pre_test_name(GetPreTestName(full_name));
 
-    while (ContainsKey(all_test_names_, pre_test_name)) {
+    while (base::ContainsKey(all_test_names_, pre_test_name)) {
       additional_tests_to_run_count++;
 
-      DCHECK(!ContainsKey(dependent_test_map_, pre_test_name));
+      DCHECK(!base::ContainsKey(dependent_test_map_, pre_test_name));
       dependent_test_map_[pre_test_name] = full_name;
 
-      DCHECK(!ContainsKey(reverse_dependent_test_map_, full_name));
+      DCHECK(!base::ContainsKey(reverse_dependent_test_map_, full_name));
       reverse_dependent_test_map_[full_name] = pre_test_name;
 
       full_name = pre_test_name;
@@ -256,15 +259,15 @@ size_t WrapperTestLauncherDelegate::RunTests(
     // Make sure no PRE_ tests were requested explicitly.
     DCHECK_EQ(full_name, RemoveAnyPrePrefixes(full_name));
 
-    if (!ContainsKey(user_data_dir_map_, full_name)) {
+    if (!base::ContainsKey(user_data_dir_map_, full_name)) {
       base::FilePath temp_dir;
-      CHECK(base::CreateTemporaryDirInDir(temp_dir_.path(),
+      CHECK(base::CreateTemporaryDirInDir(temp_dir_.GetPath(),
                                           FILE_PATH_LITERAL("d"), &temp_dir));
       user_data_dir_map_[full_name] = temp_dir;
     }
 
     // If the test has any dependencies, get to the root and start with that.
-    while (ContainsKey(reverse_dependent_test_map_, full_name))
+    while (base::ContainsKey(reverse_dependent_test_map_, full_name))
       full_name = GetPreTestName(full_name);
 
     std::vector<std::string> test_list;
@@ -288,7 +291,7 @@ size_t WrapperTestLauncherDelegate::RetryTests(
   // from the very first one.
   for (size_t i = 0; i < test_names.size(); i++) {
     std::string test_name(test_names[i]);
-    while (ContainsKey(reverse_dependent_test_map_, test_name)) {
+    while (base::ContainsKey(reverse_dependent_test_map_, test_name)) {
       test_name = reverse_dependent_test_map_[test_name];
       test_names_set.insert(test_name);
     }
@@ -314,9 +317,9 @@ size_t WrapperTestLauncherDelegate::RetryTests(
     // Make sure PRE_ tests and tests that depend on them share the same
     // data directory - based it on the test name without prefixes.
     std::string test_name_no_pre(RemoveAnyPrePrefixes(full_name));
-    if (!ContainsKey(user_data_dir_map_, test_name_no_pre)) {
+    if (!base::ContainsKey(user_data_dir_map_, test_name_no_pre)) {
       base::FilePath temp_dir;
-      CHECK(base::CreateTemporaryDirInDir(temp_dir_.path(),
+      CHECK(base::CreateTemporaryDirInDir(temp_dir_.GetPath(),
                                           FILE_PATH_LITERAL("d"), &temp_dir));
       user_data_dir_map_[test_name_no_pre] = temp_dir;
     }
@@ -327,7 +330,7 @@ size_t WrapperTestLauncherDelegate::RetryTests(
     std::string test_name = full_name.substr(dot_pos + 1);
     std::string pre_test_name(
         test_case_name + "." + kPreTestPrefix + test_name);
-    if (!ContainsKey(test_names_set, pre_test_name))
+    if (!base::ContainsKey(test_names_set, pre_test_name))
       tests_to_run_now.push_back(full_name);
   }
 
@@ -403,7 +406,7 @@ void WrapperTestLauncherDelegate::RunDependentTest(
     test_result.status = base::TestResult::TEST_SKIPPED;
     test_launcher->OnTestFinished(test_result);
 
-    if (ContainsKey(dependent_test_map_, test_name)) {
+    if (base::ContainsKey(dependent_test_map_, test_name)) {
       RunDependentTest(test_launcher,
                        dependent_test_map_[test_name],
                        test_result);
@@ -435,13 +438,13 @@ void WrapperTestLauncherDelegate::GTestCallback(
 
   result.output_snippet = GetTestOutputSnippet(result, output);
 
-  if (ContainsKey(dependent_test_map_, test_name)) {
+  if (base::ContainsKey(dependent_test_map_, test_name)) {
     RunDependentTest(test_launcher, dependent_test_map_[test_name], result);
   } else {
     // No other tests depend on this, we can delete the temporary directory now.
     // Do so to avoid too many temporary files using lots of disk space.
     std::string test_name_no_pre(RemoveAnyPrePrefixes(test_name));
-    if (ContainsKey(user_data_dir_map_, test_name_no_pre)) {
+    if (base::ContainsKey(user_data_dir_map_, test_name_no_pre)) {
       if (!base::DeleteFile(user_data_dir_map_[test_name_no_pre], true)) {
         LOG(WARNING) << "Failed to delete "
                      << user_data_dir_map_[test_name_no_pre].value();
@@ -537,6 +540,9 @@ int LaunchTests(TestLauncherDelegate* launcher_delegate,
           "process mode).\n");
 
   base::MessageLoopForIO message_loop;
+#if defined(OS_POSIX)
+  base::FileDescriptorWatcher file_descriptor_watcher(&message_loop);
+#endif
 
   // Allow the |launcher_delegate| to modify |default_jobs|.
   launcher_delegate->AdjustDefaultParallelJobs(&default_jobs);
