@@ -64,8 +64,8 @@ uint32_t AXStateFromBlink(const blink::WebAXObject& o) {
   if (o.isEditable())
     state |= (1 << ui::AX_STATE_EDITABLE);
 
-  if (o.isEnabled())
-    state |= (1 << ui::AX_STATE_ENABLED);
+  if (!o.isEnabled())
+    state |= (1 << ui::AX_STATE_DISABLED);
 
   if (o.isSelected())
     state |= (1 << ui::AX_STATE_SELECTED);
@@ -101,6 +101,8 @@ ui::AXRole AXRoleFromBlink(blink::WebAXRole role) {
       return ui::AX_ROLE_APPLICATION;
     case blink::WebAXRoleArticle:
       return ui::AX_ROLE_ARTICLE;
+    case blink::WebAXRoleAudio:
+      return ui::AX_ROLE_AUDIO;
     case blink::WebAXRoleBanner:
       return ui::AX_ROLE_BANNER;
     case blink::WebAXRoleBlockquote:
@@ -155,6 +157,8 @@ ui::AXRole AXRoleFromBlink(blink::WebAXRole role) {
       return ui::AX_ROLE_DOCUMENT;
     case blink::WebAXRoleEmbeddedObject:
       return ui::AX_ROLE_EMBEDDED_OBJECT;
+    case blink::WebAXRoleFeed:
+      return ui::AX_ROLE_FEED;
     case blink::WebAXRoleFigcaption:
       return ui::AX_ROLE_FIGCAPTION;
     case blink::WebAXRoleFigure:
@@ -303,6 +307,8 @@ ui::AXRole AXRoleFromBlink(blink::WebAXRole role) {
       return ui::AX_ROLE_TABLE;
     case blink::WebAXRoleTableHeaderContainer:
       return ui::AX_ROLE_TABLE_HEADER_CONTAINER;
+    case blink::WebAXRoleTerm:
+      return ui::AX_ROLE_TERM;
     case blink::WebAXRoleTextField:
       return ui::AX_ROLE_TEXT_FIELD;
     case blink::WebAXRoleTime:
@@ -323,6 +329,8 @@ ui::AXRole AXRoleFromBlink(blink::WebAXRole role) {
       return ui::AX_ROLE_UNKNOWN;
     case blink::WebAXRoleUserInterfaceTooltip:
       return ui::AX_ROLE_TOOLTIP;
+    case blink::WebAXRoleVideo:
+      return ui::AX_ROLE_VIDEO;
     case blink::WebAXRoleWebArea:
       return ui::AX_ROLE_ROOT_WEB_AREA;
     case blink::WebAXRoleLineBreak:
@@ -405,6 +413,32 @@ ui::AXEvent AXEventFromBlink(blink::WebAXEvent event) {
   }
 }
 
+ui::AXSupportedAction AXSupportedActionFromBlink(
+    blink::WebAXSupportedAction supported_action) {
+  switch (supported_action) {
+    case blink::WebAXSupportedAction::None:
+      return ui::AX_SUPPORTED_ACTION_NONE;
+    case blink::WebAXSupportedAction::Activate:
+      return ui::AX_SUPPORTED_ACTION_ACTIVATE;
+    case blink::WebAXSupportedAction::Check:
+      return ui::AX_SUPPORTED_ACTION_CHECK;
+    case blink::WebAXSupportedAction::Click:
+      return ui::AX_SUPPORTED_ACTION_CLICK;
+    case blink::WebAXSupportedAction::Jump:
+      return ui::AX_SUPPORTED_ACTION_JUMP;
+    case blink::WebAXSupportedAction::Open:
+      return ui::AX_SUPPORTED_ACTION_OPEN;
+    case blink::WebAXSupportedAction::Press:
+      return ui::AX_SUPPORTED_ACTION_PRESS;
+    case blink::WebAXSupportedAction::Select:
+      return ui::AX_SUPPORTED_ACTION_SELECT;
+    case blink::WebAXSupportedAction::Uncheck:
+      return ui::AX_SUPPORTED_ACTION_UNCHECK;
+  }
+  NOTREACHED();
+  return ui::AX_SUPPORTED_ACTION_NONE;
+}
+
 ui::AXMarkerType AXMarkerTypeFromBlink(blink::WebAXMarkerType marker_type) {
   switch (marker_type) {
     case blink::WebAXMarkerTypeSpelling:
@@ -542,8 +576,6 @@ ui::AXDescriptionFrom AXDescriptionFromFromBlink(
       return ui::AX_DESCRIPTION_FROM_ATTRIBUTE;
     case blink::WebAXDescriptionFromContents:
       return ui::AX_DESCRIPTION_FROM_CONTENTS;
-    case blink::WebAXDescriptionFromPlaceholder:
-      return ui::AX_DESCRIPTION_FROM_PLACEHOLDER;
     case blink::WebAXDescriptionFromRelatedElement:
       return ui::AX_DESCRIPTION_FROM_RELATED_ELEMENT;
   }
@@ -551,4 +583,15 @@ ui::AXDescriptionFrom AXDescriptionFromFromBlink(
   return ui::AX_DESCRIPTION_FROM_NONE;
 }
 
+ui::AXTextAffinity AXTextAffinityFromBlink(blink::WebAXTextAffinity affinity) {
+  switch (affinity) {
+    case blink::WebAXTextAffinityUpstream:
+      return ui::AX_TEXT_AFFINITY_UPSTREAM;
+    case blink::WebAXTextAffinityDownstream:
+      return ui::AX_TEXT_AFFINITY_DOWNSTREAM;
+  }
+  NOTREACHED();
+  return ui::AX_TEXT_AFFINITY_DOWNSTREAM;
+}
+
 }  // namespace content.
