@@ -14,22 +14,22 @@ namespace content {
 RTCVideoDecoderFactory::RTCVideoDecoderFactory(
     media::GpuVideoAcceleratorFactories* gpu_factories)
     : gpu_factories_(gpu_factories) {
-  DVLOG(2) << __FUNCTION__;
+  DVLOG(2) << __func__;
 }
 
 RTCVideoDecoderFactory::~RTCVideoDecoderFactory() {
-  DVLOG(2) << __FUNCTION__;
+  DVLOG(2) << __func__;
 }
 
 webrtc::VideoDecoder* RTCVideoDecoderFactory::CreateVideoDecoder(
     webrtc::VideoCodecType type) {
-  DVLOG(2) << __FUNCTION__;
+  DVLOG(2) << __func__;
   return RTCVideoDecoder::Create(type, gpu_factories_).release();
 }
 
 void RTCVideoDecoderFactory::DestroyVideoDecoder(
     webrtc::VideoDecoder* decoder) {
-  DVLOG(2) << __FUNCTION__;
+  DVLOG(2) << __func__;
   RTCVideoDecoder::Destroy(decoder, gpu_factories_);
 }
 
