@@ -11,11 +11,12 @@
 
 #include "base/bind.h"
 #include "base/callback.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "cc/blink/web_layer_impl.h"
 #include "cc/layers/video_frame_provider_client_impl.h"
 #include "cc/layers/video_layer.h"
-#include "content/common/gpu/client/context_provider_command_buffer.h"
+#include "content/child/child_process.h"
 #include "content/public/renderer/media_stream_audio_renderer.h"
 #include "content/public/renderer/media_stream_renderer_factory.h"
 #include "content/public/renderer/media_stream_video_renderer.h"
@@ -23,9 +24,14 @@
 #include "content/renderer/media/webmediaplayer_ms_compositor.h"
 #include "content/renderer/render_frame_impl.h"
 #include "content/renderer/render_thread_impl.h"
+#include "media/base/bind_to_current_loop.h"
+#include "media/base/media_content_type.h"
 #include "media/base/media_log.h"
 #include "media/base/video_frame.h"
+#include "media/base/video_rotation.h"
+#include "media/base/video_types.h"
 #include "media/blink/webmediaplayer_util.h"
+#include "services/ui/public/cpp/gpu/context_provider_command_buffer.h"
 #include "third_party/WebKit/public/platform/WebMediaPlayerClient.h"
 #include "third_party/WebKit/public/platform/WebMediaPlayerSource.h"
 #include "third_party/WebKit/public/platform/WebRect.h"
@@ -33,15 +39,117 @@
 
 namespace content {
 
+// FrameDeliverer is responsible for delivering frames received on
+// the IO thread by calling of EnqueueFrame() method of |compositor_|.
+//
+// It is created on the main thread, but methods should be called and class
+// should be destructed on the IO thread.
+class WebMediaPlayerMS::FrameDeliverer {
+ public:
+  FrameDeliverer(const base::WeakPtr<WebMediaPlayerMS>& player,
+                 const MediaStreamVideoRenderer::RepaintCB& enqueue_frame_cb)
+      : last_frame_opaque_(true),
+        last_frame_rotation_(media::VIDEO_ROTATION_0),
+        received_first_frame_(false),
+        main_task_runner_(base::ThreadTaskRunnerHandle::Get()),
+        player_(player),
+        enqueue_frame_cb_(enqueue_frame_cb),
+        weak_factory_(this) {
+    io_thread_checker_.DetachFromThread();
+  }
+
+  ~FrameDeliverer() { DCHECK(io_thread_checker_.CalledOnValidThread()); }
+
+  void OnVideoFrame(scoped_refptr<media::VideoFrame> frame) {
+    DCHECK(io_thread_checker_.CalledOnValidThread());
+
+#if defined(OS_ANDROID)
+    if (render_frame_suspended_)
+      return;
+#endif  // defined(OS_ANDROID)
+
+    base::TimeTicks render_time;
+    if (frame->metadata()->GetTimeTicks(
+            media::VideoFrameMetadata::REFERENCE_TIME, &render_time)) {
+      TRACE_EVENT1("webrtc", "WebMediaPlayerMS::OnVideoFrame",
+                   "Ideal Render Instant", render_time.ToInternalValue());
+    } else {
+      TRACE_EVENT0("webrtc", "WebMediaPlayerMS::OnVideoFrame");
+    }
+
+    const bool is_opaque = media::IsOpaque(frame->format());
+    media::VideoRotation video_rotation = media::VIDEO_ROTATION_0;
+    ignore_result(frame->metadata()->GetRotation(
+        media::VideoFrameMetadata::ROTATION, &video_rotation));
+
+    if (!received_first_frame_) {
+      received_first_frame_ = true;
+      last_frame_opaque_ = is_opaque;
+      last_frame_rotation_ = video_rotation;
+      main_task_runner_->PostTask(
+          FROM_HERE, base::Bind(&WebMediaPlayerMS::OnFirstFrameReceived,
+                                player_, video_rotation, is_opaque));
+    } else {
+      if (last_frame_opaque_ != is_opaque) {
+        last_frame_opaque_ = is_opaque;
+        main_task_runner_->PostTask(
+            FROM_HERE, base::Bind(&WebMediaPlayerMS::OnOpacityChanged, player_,
+                                  is_opaque));
+      }
+      if (last_frame_rotation_ != video_rotation) {
+        last_frame_rotation_ = video_rotation;
+        main_task_runner_->PostTask(
+            FROM_HERE, base::Bind(&WebMediaPlayerMS::OnRotationChanged, player_,
+                                  video_rotation, is_opaque));
+      }
+    }
+
+    enqueue_frame_cb_.Run(frame);
+  }
+
+#if defined(OS_ANDROID)
+  void SetRenderFrameSuspended(bool render_frame_suspended) {
+    DCHECK(io_thread_checker_.CalledOnValidThread());
+    render_frame_suspended_ = render_frame_suspended;
+  }
+#endif  // defined(OS_ANDROID)
+
+  MediaStreamVideoRenderer::RepaintCB GetRepaintCallback() {
+    return base::Bind(&FrameDeliverer::OnVideoFrame,
+                      weak_factory_.GetWeakPtr());
+  }
+
+ private:
+  bool last_frame_opaque_;
+  media::VideoRotation last_frame_rotation_;
+  bool received_first_frame_;
+
+#if defined(OS_ANDROID)
+  bool render_frame_suspended_ = false;
+#endif  // defined(OS_ANDROID)
+
+  const scoped_refptr<base::SingleThreadTaskRunner> main_task_runner_;
+  const base::WeakPtr<WebMediaPlayerMS> player_;
+  const MediaStreamVideoRenderer::RepaintCB enqueue_frame_cb_;
+
+  // Used for DCHECKs to ensure method calls are executed on the correct thread.
+  base::ThreadChecker io_thread_checker_;
+
+  base::WeakPtrFactory<FrameDeliverer> weak_factory_;
+
+  DISALLOW_COPY_AND_ASSIGN(FrameDeliverer);
+};
+
 WebMediaPlayerMS::WebMediaPlayerMS(
     blink::WebFrame* frame,
     blink::WebMediaPlayerClient* client,
     base::WeakPtr<media::WebMediaPlayerDelegate> delegate,
     media::MediaLog* media_log,
     std::unique_ptr<MediaStreamRendererFactory> factory,
-    const scoped_refptr<base::SingleThreadTaskRunner>& compositor_task_runner,
-    const scoped_refptr<base::SingleThreadTaskRunner>& media_task_runner,
-    const scoped_refptr<base::TaskRunner>& worker_task_runner,
+    scoped_refptr<base::SingleThreadTaskRunner> io_task_runner,
+    scoped_refptr<base::SingleThreadTaskRunner> compositor_task_runner,
+    scoped_refptr<base::SingleThreadTaskRunner> media_task_runner,
+    scoped_refptr<base::TaskRunner> worker_task_runner,
     media::GpuVideoAcceleratorFactories* gpu_factories,
     const blink::WebString& sink_id,
     const blink::WebSecurityOrigin& security_origin)
@@ -53,14 +161,14 @@ WebMediaPlayerMS::WebMediaPlayerMS(
       delegate_(delegate),
       delegate_id_(0),
       paused_(true),
-      render_frame_suspended_(false),
-      received_first_frame_(false),
+      video_rotation_(media::VIDEO_ROTATION_0),
       media_log_(media_log),
       renderer_factory_(std::move(factory)),
+      io_task_runner_(io_task_runner),
+      compositor_task_runner_(compositor_task_runner),
       media_task_runner_(media_task_runner),
       worker_task_runner_(worker_task_runner),
       gpu_factories_(gpu_factories),
-      compositor_task_runner_(compositor_task_runner),
       initial_audio_output_device_id_(sink_id.utf8()),
       initial_security_origin_(security_origin.isNull()
                                    ? url::Origin()
@@ -68,7 +176,7 @@ WebMediaPlayerMS::WebMediaPlayerMS(
       volume_(1.0),
       volume_multiplier_(1.0),
       should_play_upon_shown_(false) {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(client);
   if (delegate_)
     delegate_id_ = delegate_->AddObserver(this);
@@ -78,15 +186,19 @@ WebMediaPlayerMS::WebMediaPlayerMS(
 }
 
 WebMediaPlayerMS::~WebMediaPlayerMS() {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
 
   // Destruct compositor resources in the proper order.
   get_client()->setWebLayer(nullptr);
   if (video_weblayer_)
     static_cast<cc::VideoLayer*>(video_weblayer_->layer())->StopUsingProvider();
+
+  if (frame_deliverer_)
+    io_task_runner_->DeleteSoon(FROM_HERE, frame_deliverer_.release());
+
   if (compositor_)
-    compositor_task_runner_->DeleteSoon(FROM_HERE, compositor_.release());
+    compositor_->StopUsingProvider();
 
   if (video_frame_provider_)
     video_frame_provider_->Stop();
@@ -106,7 +218,7 @@ WebMediaPlayerMS::~WebMediaPlayerMS() {
 void WebMediaPlayerMS::load(LoadType load_type,
                             const blink::WebMediaPlayerSource& source,
                             CORSMode /*cors_mode*/) {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
 
   // TODO(acolwell): Change this to DCHECK_EQ(load_type, LoadTypeMediaStream)
@@ -115,8 +227,8 @@ void WebMediaPlayerMS::load(LoadType load_type,
   blink::WebMediaStream web_stream =
       GetWebMediaStreamFromWebMediaPlayerSource(source);
 
-  compositor_.reset(new WebMediaPlayerMSCompositor(compositor_task_runner_,
-                                                   web_stream, AsWeakPtr()));
+  compositor_ = new WebMediaPlayerMSCompositor(compositor_task_runner_,
+                                               web_stream, AsWeakPtr());
 
   SetNetworkState(WebMediaPlayer::NetworkStateLoading);
   SetReadyState(WebMediaPlayer::ReadyStateHaveNothing);
@@ -124,9 +236,13 @@ void WebMediaPlayerMS::load(LoadType load_type,
       web_stream.isNull() ? std::string() : web_stream.id().utf8();
   media_log_->AddEvent(media_log_->CreateLoadEvent(stream_id));
 
+  frame_deliverer_.reset(new WebMediaPlayerMS::FrameDeliverer(
+      AsWeakPtr(),
+      base::Bind(&WebMediaPlayerMSCompositor::EnqueueFrame, compositor_)));
   video_frame_provider_ = renderer_factory_->GetVideoRenderer(
-      web_stream, base::Bind(&WebMediaPlayerMS::OnSourceError, AsWeakPtr()),
-      base::Bind(&WebMediaPlayerMS::OnFrameAvailable, AsWeakPtr()),
+      web_stream, media::BindToCurrentLoop(base::Bind(
+                      &WebMediaPlayerMS::OnSourceError, AsWeakPtr())),
+      frame_deliverer_->GetRepaintCallback(), io_task_runner_,
       media_task_runner_, worker_task_runner_, gpu_factories_);
 
   RenderFrame* const frame = RenderFrame::FromWebFrame(frame_);
@@ -160,7 +276,7 @@ void WebMediaPlayerMS::load(LoadType load_type,
 }
 
 void WebMediaPlayerMS::play() {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
 
   media_log_->AddEvent(media_log_->CreateEvent(media::MediaLogEvent::PLAY));
@@ -176,19 +292,19 @@ void WebMediaPlayerMS::play() {
     audio_renderer_->Play();
 
   if (delegate_) {
-    // TODO(perkj, magjed): We send a duration of 1 second here to avoid
-    // creating an interactive media session on Android. We'd like to use zero
-    // here, but that is treated as an unknown duration and assumed to be
-    // interactive. See http://crbug.com/595297 for more details.
-    delegate_->DidPlay(delegate_id_, hasVideo(), hasAudio(), false,
-                       base::TimeDelta::FromSeconds(1));
+    // TODO(perkj, magjed): We use OneShot focus type here so that it takes
+    // audio focus once it starts, and then will not respond to further audio
+    // focus changes. See http://crbug.com/596516 for more details.
+    delegate_->DidPlay(delegate_id_, hasVideo(), hasAudio(),
+                       media::MediaContentType::OneShot);
+    delegate_->SetIdle(delegate_id_, false);
   }
 
   paused_ = false;
 }
 
 void WebMediaPlayerMS::pause() {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
 
   should_play_upon_shown_ = false;
@@ -205,8 +321,10 @@ void WebMediaPlayerMS::pause() {
   if (audio_renderer_)
     audio_renderer_->Pause();
 
-  if (delegate_)
-    delegate_->DidPause(delegate_id_, false);
+  if (delegate_) {
+    delegate_->DidPause(delegate_id_);
+    delegate_->SetIdle(delegate_id_, true);
+  }
 
   paused_ = true;
 }
@@ -225,7 +343,7 @@ void WebMediaPlayerMS::setRate(double rate) {
 }
 
 void WebMediaPlayerMS::setVolume(double volume) {
-  DVLOG(1) << __FUNCTION__ << "(volume=" << volume << ")";
+  DVLOG(1) << __func__ << "(volume=" << volume << ")";
   DCHECK(thread_checker_.CalledOnValidThread());
   volume_ = volume;
   if (audio_renderer_.get())
@@ -236,7 +354,7 @@ void WebMediaPlayerMS::setSinkId(
     const blink::WebString& sink_id,
     const blink::WebSecurityOrigin& security_origin,
     blink::WebSetSinkIdCallbacks* web_callback) {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
   const media::OutputDeviceStatusCB callback =
       media::ConvertToOutputDeviceStatusCB(web_callback);
@@ -264,6 +382,11 @@ bool WebMediaPlayerMS::hasAudio() const {
 
 blink::WebSize WebMediaPlayerMS::naturalSize() const {
   DCHECK(thread_checker_.CalledOnValidThread());
+  if (video_rotation_ == media::VIDEO_ROTATION_90 ||
+      video_rotation_ == media::VideoRotation::VIDEO_ROTATION_270) {
+    const gfx::Size& current_size = compositor_->GetCurrentSize();
+    return blink::WebSize(current_size.height(), current_size.width());
+  }
   return blink::WebSize(compositor_->GetCurrentSize());
 }
 
@@ -293,13 +416,13 @@ double WebMediaPlayerMS::currentTime() const {
 }
 
 blink::WebMediaPlayer::NetworkState WebMediaPlayerMS::getNetworkState() const {
-  DVLOG(1) << __FUNCTION__ << ", state:" << network_state_;
+  DVLOG(1) << __func__ << ", state:" << network_state_;
   DCHECK(thread_checker_.CalledOnValidThread());
   return network_state_;
 }
 
 blink::WebMediaPlayer::ReadyState WebMediaPlayerMS::getReadyState() const {
-  DVLOG(1) << __FUNCTION__ << ", state:" << ready_state_;
+  DVLOG(1) << __func__ << ", state:" << ready_state_;
   DCHECK(thread_checker_.CalledOnValidThread());
   return ready_state_;
 }
@@ -325,9 +448,8 @@ bool WebMediaPlayerMS::didLoadingProgress() {
 
 void WebMediaPlayerMS::paint(blink::WebCanvas* canvas,
                              const blink::WebRect& rect,
-                             unsigned char alpha,
-                             SkXfermode::Mode mode) {
-  DVLOG(3) << __FUNCTION__;
+                             SkPaint& paint) {
+  DVLOG(3) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
 
   const scoped_refptr<media::VideoFrame> frame =
@@ -344,8 +466,8 @@ void WebMediaPlayerMS::paint(blink::WebCanvas* canvas,
     DCHECK(context_3d.gl);
   }
   const gfx::RectF dest_rect(rect.x, rect.y, rect.width, rect.height);
-  video_renderer_.Paint(frame, canvas, dest_rect, alpha, mode,
-                        media::VIDEO_ROTATION_0, context_3d);
+  video_renderer_.Paint(frame, canvas, dest_rect, paint,
+                        video_rotation_, context_3d);
 }
 
 bool WebMediaPlayerMS::hasSingleSecurityOrigin() const {
@@ -384,7 +506,7 @@ size_t WebMediaPlayerMS::videoDecodedByteCount() const {
   return 0;
 }
 
-void WebMediaPlayerMS::OnHidden() {
+void WebMediaPlayerMS::OnFrameHidden() {
 #if defined(OS_ANDROID)
   DCHECK(thread_checker_.CalledOnValidThread());
 
@@ -394,30 +516,19 @@ void WebMediaPlayerMS::OnHidden() {
   //
   // During undoable tab closures OnHidden() may be called back to back, so we
   // can't rely on |render_frame_suspended_| being false here.
+  if (frame_deliverer_) {
+    io_task_runner_->PostTask(
+        FROM_HERE, base::Bind(&FrameDeliverer::SetRenderFrameSuspended,
+                              base::Unretained(frame_deliverer_.get()), true));
+  }
 
-  render_frame_suspended_ = true;
   if (!paused_)
     compositor_->ReplaceCurrentFrameWithACopy();
 #endif  // defined(OS_ANDROID)
 }
 
-void WebMediaPlayerMS::OnShown() {
+void WebMediaPlayerMS::OnFrameClosed() {
 #if defined(OS_ANDROID)
-  DCHECK(thread_checker_.CalledOnValidThread());
-
-  render_frame_suspended_ = false;
-
-  // Resume playback on visibility. play() clears |should_play_upon_shown_|.
-  if (should_play_upon_shown_)
-    play();
-#endif  // defined(OS_ANDROID)
-}
-
-void WebMediaPlayerMS::OnSuspendRequested(bool must_suspend) {
-#if defined(OS_ANDROID)
-  if (!must_suspend)
-    return;
-
   if (!paused_) {
     pause();
     should_play_upon_shown_ = true;
@@ -426,10 +537,32 @@ void WebMediaPlayerMS::OnSuspendRequested(bool must_suspend) {
   if (delegate_)
     delegate_->PlayerGone(delegate_id_);
 
-  render_frame_suspended_ = true;
-#endif
+  if (frame_deliverer_) {
+    io_task_runner_->PostTask(
+        FROM_HERE, base::Bind(&FrameDeliverer::SetRenderFrameSuspended,
+                              base::Unretained(frame_deliverer_.get()), true));
+  }
+#endif  // defined(OS_ANDROID)
 }
 
+void WebMediaPlayerMS::OnFrameShown() {
+#if defined(OS_ANDROID)
+  DCHECK(thread_checker_.CalledOnValidThread());
+
+  if (frame_deliverer_) {
+    io_task_runner_->PostTask(
+        FROM_HERE, base::Bind(&FrameDeliverer::SetRenderFrameSuspended,
+                              base::Unretained(frame_deliverer_.get()), false));
+  }
+
+  // Resume playback on visibility. play() clears |should_play_upon_shown_|.
+  if (should_play_upon_shown_)
+    play();
+#endif  // defined(OS_ANDROID)
+}
+
+void WebMediaPlayerMS::OnIdleTimeout() {}
+
 void WebMediaPlayerMS::OnPlay() {
   // TODO(perkj, magjed): It's not clear how WebRTC should work with an
   // MediaSession, until these issues are resolved, disable session controls.
@@ -437,19 +570,16 @@ void WebMediaPlayerMS::OnPlay() {
 }
 
 void WebMediaPlayerMS::OnPause() {
-  // TODO(perkj, magjed): See TODO in Onplay().
+  // TODO(perkj, magjed): See TODO in OnPlay().
 }
 
 void WebMediaPlayerMS::OnVolumeMultiplierUpdate(double multiplier) {
-  volume_multiplier_ = multiplier;
-  setVolume(volume_);
+  // TODO(perkj, magjed): See TODO in OnPlay().
 }
 
 bool WebMediaPlayerMS::copyVideoTextureToPlatformTexture(
     gpu::gles2::GLES2Interface* gl,
     unsigned int texture,
-    unsigned int internal_format,
-    unsigned int type,
     bool premultiply_alpha,
     bool flip_y) {
   TRACE_EVENT0("media", "WebMediaPlayerMS:copyVideoTextureToPlatformTexture");
@@ -470,46 +600,78 @@ bool WebMediaPlayerMS::copyVideoTextureToPlatformTexture(
   context_3d = media::Context3D(provider->ContextGL(), provider->GrContext());
   DCHECK(context_3d.gl);
   return video_renderer_.CopyVideoFrameTexturesToGLTexture(
-      context_3d, gl, video_frame.get(), texture, internal_format, type,
-      premultiply_alpha, flip_y);
+      context_3d, gl, video_frame.get(), texture, premultiply_alpha, flip_y);
 }
 
-void WebMediaPlayerMS::OnFrameAvailable(
-    const scoped_refptr<media::VideoFrame>& frame) {
-  DVLOG(3) << __FUNCTION__;
+bool WebMediaPlayerMS::texImageImpl(TexImageFunctionID functionID,
+                                    unsigned target,
+                                    gpu::gles2::GLES2Interface* gl,
+                                    int level,
+                                    int internalformat,
+                                    unsigned format,
+                                    unsigned type,
+                                    int xoffset,
+                                    int yoffset,
+                                    int zoffset,
+                                    bool flip_y,
+                                    bool premultiply_alpha) {
+  TRACE_EVENT0("media", "WebMediaPlayerMS:texImageImpl");
   DCHECK(thread_checker_.CalledOnValidThread());
 
-  if (render_frame_suspended_)
-    return;
-
-  base::TimeTicks render_time;
-  if (frame->metadata()->GetTimeTicks(media::VideoFrameMetadata::REFERENCE_TIME,
-                                      &render_time)) {
-    TRACE_EVENT1("webrtc", "WebMediaPlayerMS::OnFrameAvailable",
-                 "Ideal Render Instant", render_time.ToInternalValue());
-  } else {
-    TRACE_EVENT0("webrtc", "WebMediaPlayerMS::OnFrameAvailable");
+  const scoped_refptr<media::VideoFrame> video_frame =
+      compositor_->GetCurrentFrameWithoutUpdatingStatistics();
+  if (!video_frame || !video_frame->IsMappable() ||
+      video_frame->HasTextures() ||
+      video_frame->format() != media::PIXEL_FORMAT_Y16) {
+    return false;
   }
 
-  if (!received_first_frame_) {
-    received_first_frame_ = true;
-    SetReadyState(WebMediaPlayer::ReadyStateHaveMetadata);
-    SetReadyState(WebMediaPlayer::ReadyStateHaveEnoughData);
-
-    if (video_frame_provider_.get()) {
-      video_weblayer_.reset(new cc_blink::WebLayerImpl(
-          cc::VideoLayer::Create(compositor_.get(), media::VIDEO_ROTATION_0)));
-      video_weblayer_->layer()->SetContentsOpaque(false);
-      video_weblayer_->SetContentsOpaqueIsFixed(true);
-      get_client()->setWebLayer(video_weblayer_.get());
-    }
+  if (functionID == TexImage2D) {
+    return media::SkCanvasVideoRenderer::TexImage2D(
+        target, gl, video_frame.get(), level, internalformat, format, type,
+        flip_y, premultiply_alpha);
+  } else if (functionID == TexSubImage2D) {
+    return media::SkCanvasVideoRenderer::TexSubImage2D(
+        target, gl, video_frame.get(), level, format, type, xoffset, yoffset,
+        flip_y, premultiply_alpha);
   }
+  return false;
+}
 
-  compositor_->EnqueueFrame(frame);
+void WebMediaPlayerMS::OnFirstFrameReceived(media::VideoRotation video_rotation,
+                                            bool is_opaque) {
+  DVLOG(1) << __func__;
+  DCHECK(thread_checker_.CalledOnValidThread());
+  SetReadyState(WebMediaPlayer::ReadyStateHaveMetadata);
+  SetReadyState(WebMediaPlayer::ReadyStateHaveEnoughData);
+
+  OnRotationChanged(video_rotation, is_opaque);
+}
+
+void WebMediaPlayerMS::OnOpacityChanged(bool is_opaque) {
+  DVLOG(1) << __func__;
+  DCHECK(thread_checker_.CalledOnValidThread());
+
+  // Opacity can be changed during the session without resetting
+  // |video_weblayer_|.
+  video_weblayer_->layer()->SetContentsOpaque(is_opaque);
+}
+
+void WebMediaPlayerMS::OnRotationChanged(media::VideoRotation video_rotation,
+                                         bool is_opaque) {
+  DVLOG(1) << __func__;
+  DCHECK(thread_checker_.CalledOnValidThread());
+  video_rotation_ = video_rotation;
+
+  video_weblayer_.reset(new cc_blink::WebLayerImpl(
+      cc::VideoLayer::Create(compositor_.get(), video_rotation)));
+  video_weblayer_->layer()->SetContentsOpaque(is_opaque);
+  video_weblayer_->SetContentsOpaqueIsFixed(true);
+  get_client()->setWebLayer(video_weblayer_.get());
 }
 
 void WebMediaPlayerMS::RepaintInternal() {
-  DVLOG(1) << __FUNCTION__;
+  DVLOG(1) << __func__;
   DCHECK(thread_checker_.CalledOnValidThread());
   get_client()->repaint();
 }
