@@ -10,8 +10,6 @@
 #include "base/memory/ref_counted.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/time/time.h"
-#include "components/mus/public/cpp/tests/test_window.h"
-#include "content/common/input/did_overscroll_params.h"
 #include "content/common/input/input_event_ack.h"
 #include "content/common/input/input_event_ack_state.h"
 #include "content/public/test/mock_render_thread.h"
@@ -21,15 +19,19 @@
 #include "content/renderer/mus/render_widget_mus_connection.h"
 #include "content/renderer/render_widget.h"
 #include "content/test/fake_compositor_dependencies.h"
-#include "content/test/fake_renderer_scheduler.h"
 #include "mojo/public/cpp/bindings/interface_request.h"
+#include "services/ui/public/cpp/tests/test_window.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/WebKit/public/platform/scheduler/test/fake_renderer_scheduler.h"
+#include "ui/events/blink/did_overscroll_params.h"
 #include "ui/events/event_utils.h"
 #include "ui/events/mojo/event.mojom.h"
 #include "ui/events/mojo/event_constants.mojom.h"
 #include "ui/events/mojo/keyboard_codes.mojom.h"
 
-using mus::mojom::EventResult;
+using ui::mojom::EventResult;
+
+namespace content {
 
 namespace {
 
@@ -61,35 +63,35 @@ class TestCallback : public base::RefCounted<TestCallback> {
 
 // Allows for overriding the behaviour of HandleInputEvent, to simulate input
 // handlers which consume events before they are sent to the renderer.
-class TestInputHandlerManager : public content::InputHandlerManager {
+class TestInputHandlerManager : public InputHandlerManager {
  public:
   TestInputHandlerManager(
       const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
-      content::InputHandlerManagerClient* client,
-      scheduler::RendererScheduler* renderer_scheduler)
+      InputHandlerManagerClient* client,
+      blink::scheduler::RendererScheduler* renderer_scheduler)
       : InputHandlerManager(task_runner, client, nullptr, renderer_scheduler),
         override_result_(false),
-        result_(content::InputEventAckState::INPUT_EVENT_ACK_STATE_UNKNOWN) {}
+        result_(InputEventAckState::INPUT_EVENT_ACK_STATE_UNKNOWN) {}
   ~TestInputHandlerManager() override {}
 
   // Stops overriding the behaviour of HandleInputEvent
   void ClearHandleInputEventOverride();
 
   // Overrides the behaviour of HandleInputEvent, returing |result|.
-  void SetHandleInputEventResult(content::InputEventAckState result);
+  void SetHandleInputEventResult(InputEventAckState result);
 
-  // content::InputHandlerManager:
-  content::InputEventAckState HandleInputEvent(
-      int routing_id,
-      const blink::WebInputEvent* input_event,
-      ui::LatencyInfo* latency_info) override;
+  // InputHandlerManager:
+  void HandleInputEvent(int routing_id,
+                        blink::WebScopedInputEvent input_event,
+                        const ui::LatencyInfo& latency_info,
+                        const InputEventAckStateCallback& callback) override;
 
  private:
-  // If true content::InputHandlerManager::HandleInputEvent is not called.
+  // If true InputHandlerManager::HandleInputEvent is not called.
   bool override_result_;
 
   // The result to return in HandleInputEvent if |override_result_|.
-  content::InputEventAckState result_;
+  InputEventAckState result_;
 
   DISALLOW_COPY_AND_ASSIGN(TestInputHandlerManager);
 };
@@ -99,55 +101,64 @@ void TestInputHandlerManager::ClearHandleInputEventOverride() {
 }
 
 void TestInputHandlerManager::SetHandleInputEventResult(
-    content::InputEventAckState result) {
+    InputEventAckState result) {
   override_result_ = true;
   result_ = result;
 }
 
-content::InputEventAckState TestInputHandlerManager::HandleInputEvent(
+void TestInputHandlerManager::HandleInputEvent(
     int routing_id,
-    const blink::WebInputEvent* input_event,
-    ui::LatencyInfo* latency_info) {
-  if (override_result_)
-    return result_;
-  return content::InputHandlerManager::HandleInputEvent(routing_id, input_event,
-                                                        latency_info);
+    blink::WebScopedInputEvent input_event,
+    const ui::LatencyInfo& latency_info,
+    const InputEventAckStateCallback& callback) {
+  if (override_result_) {
+    callback.Run(result_, std::move(input_event), latency_info, nullptr);
+    return;
+  }
+  InputHandlerManager::HandleInputEvent(routing_id, std::move(input_event),
+                                        latency_info, callback);
 }
 
 // Empty implementation of InputHandlerManagerClient.
-class TestInputHandlerManagerClient
-    : public content::InputHandlerManagerClient {
+class TestInputHandlerManagerClient : public InputHandlerManagerClient {
  public:
   TestInputHandlerManagerClient() {}
   ~TestInputHandlerManagerClient() override{};
 
-  // content::InputHandlerManagerClient:
-  void SetBoundHandler(const Handler& handler) override {}
+  // InputHandlerManagerClient:
+  void SetInputHandlerManager(
+      InputHandlerManager* input_handler_manager) override {}
   void RegisterRoutingID(int routing_id) override {}
   void UnregisterRoutingID(int routing_id) override {}
   void DidOverscroll(int routing_id,
-                     const content::DidOverscrollParams& params) override {}
-  void DidStartFlinging(int routing_id) override {}
+                     const ui::DidOverscrollParams& params) override {}
   void DidStopFlinging(int routing_id) override {}
-  void NotifyInputEventHandled(
+  void DispatchNonBlockingEventToMainThread(
       int routing_id,
-      blink::WebInputEvent::Type type,
-      content::InputEventAckState ack_result) override {}
+      blink::WebScopedInputEvent event,
+      const ui::LatencyInfo& latency_info) override {}
+
+  void NotifyInputEventHandled(int routing_id,
+                               blink::WebInputEvent::Type type,
+                               blink::WebInputEventResult result,
+                               InputEventAckState ack_result) override {}
+  void ProcessRafAlignedInput(int routing_id) override {}
 
  private:
   DISALLOW_COPY_AND_ASSIGN(TestInputHandlerManagerClient);
 };
 
 // Implementation of RenderWidget for testing, performs no initialization.
-class TestRenderWidget : public content::RenderWidget {
+class TestRenderWidget : public RenderWidget {
  public:
-  explicit TestRenderWidget(content::CompositorDependencies* compositor_deps)
-      : content::RenderWidget(compositor_deps,
-                              blink::WebPopupTypeNone,
-                              blink::WebScreenInfo(),
-                              true,
-                              false,
-                              false) {}
+  explicit TestRenderWidget(CompositorDependencies* compositor_deps)
+      : RenderWidget(1,
+                     compositor_deps,
+                     blink::WebPopupTypeNone,
+                     ScreenInfo(),
+                     true,
+                     false,
+                     false) {}
 
  protected:
   ~TestRenderWidget() override {}
@@ -160,52 +171,50 @@ class TestRenderWidget : public content::RenderWidget {
 // HandleInputEvent. This will perform no actions on input until a
 // RenderWidgetInputHandlerDelegate is set. Once set this will always ack
 // received events.
-class TestRenderWidgetInputHandler : public content::RenderWidgetInputHandler {
+class TestRenderWidgetInputHandler : public RenderWidgetInputHandler {
  public:
-  TestRenderWidgetInputHandler(content::RenderWidget* render_widget);
+  TestRenderWidgetInputHandler(RenderWidget* render_widget);
   ~TestRenderWidgetInputHandler() override {}
 
-  void set_delegate(content::RenderWidgetInputHandlerDelegate* delegate) {
+  void set_delegate(RenderWidgetInputHandlerDelegate* delegate) {
     delegate_ = delegate;
   }
-  void set_state(content::InputEventAckState state) { state_ = state; }
+  void set_state(InputEventAckState state) { state_ = state; }
 
-  // content::RenderWidgetInputHandler:
+  // RenderWidgetInputHandler:
   void HandleInputEvent(const blink::WebInputEvent& input_event,
                         const ui::LatencyInfo& latency_info,
-                        content::InputEventDispatchType dispatch_type) override;
+                        InputEventDispatchType dispatch_type) override;
 
  private:
   // The input delegate which receives event acks.
-  content::RenderWidgetInputHandlerDelegate* delegate_;
+  RenderWidgetInputHandlerDelegate* delegate_;
 
   // The result of input handling to send to |delegate_| during the ack.
-  content::InputEventAckState state_;
+  InputEventAckState state_;
 
   DISALLOW_COPY_AND_ASSIGN(TestRenderWidgetInputHandler);
 };
 
 TestRenderWidgetInputHandler::TestRenderWidgetInputHandler(
-    content::RenderWidget* render_widget)
-    : content::RenderWidgetInputHandler(render_widget, render_widget),
+    RenderWidget* render_widget)
+    : RenderWidgetInputHandler(render_widget, render_widget),
       delegate_(nullptr),
-      state_(content::InputEventAckState::INPUT_EVENT_ACK_STATE_UNKNOWN) {}
+      state_(InputEventAckState::INPUT_EVENT_ACK_STATE_UNKNOWN) {}
 
 void TestRenderWidgetInputHandler::HandleInputEvent(
     const blink::WebInputEvent& input_event,
     const ui::LatencyInfo& latency_info,
-    content::InputEventDispatchType dispatch_type) {
+    InputEventDispatchType dispatch_type) {
   if (delegate_) {
-    std::unique_ptr<content::InputEventAck> ack(
-        new content::InputEventAck(input_event.type, state_));
+    std::unique_ptr<InputEventAck> ack(new InputEventAck(
+        InputEventAckSource::COMPOSITOR_THREAD, input_event.type(), state_));
     delegate_->OnInputEventAck(std::move(ack));
   }
 }
 
 }  // namespace
 
-namespace content {
-
 // Test suite for CompositorMusConnection, this does not setup a full renderer
 // environment. This does not establish a connection to a mus server, nor does
 // it initialize one.
@@ -220,7 +229,7 @@ class CompositorMusConnectionTest : public testing::Test {
 
   // Calls CompositorMusConnection::OnWindowInputEvent.
   void OnWindowInputEvent(
-      mus::Window* window,
+      ui::Window* window,
       const ui::Event& event,
       std::unique_ptr<base::Callback<void(EventResult)>>* ack_callback);
 
@@ -248,10 +257,10 @@ class CompositorMusConnectionTest : public testing::Test {
   // Mocks/Fakes of the testing environment.
   TestInputHandlerManagerClient input_handler_manager_client_;
   FakeCompositorDependencies compositor_dependencies_;
-  FakeRendererScheduler renderer_scheduler_;
+  blink::scheduler::FakeRendererScheduler renderer_scheduler_;
   MockRenderThread render_thread_;
   scoped_refptr<TestRenderWidget> render_widget_;
-  mojo::InterfaceRequest<mus::mojom::WindowTreeClient> request_;
+  mojo::InterfaceRequest<ui::mojom::WindowTreeClient> request_;
 
   // Not owned, RenderWidgetMusConnection tracks in static state. Cleared during
   // TearDown.
@@ -277,7 +286,7 @@ std::unique_ptr<ui::Event> CompositorMusConnectionTest::GenerateKeyEvent() {
 }
 
 void CompositorMusConnectionTest::OnWindowInputEvent(
-    mus::Window* window,
+    ui::Window* window,
     const ui::Event& event,
     std::unique_ptr<base::Callback<void(EventResult)>>* ack_callback) {
   compositor_connection_->OnWindowInputEvent(window, event, ack_callback);
@@ -336,12 +345,12 @@ TEST_F(CompositorMusConnectionTest, NotConsumed) {
   input_handler->set_state(
       InputEventAckState::INPUT_EVENT_ACK_STATE_NOT_CONSUMED);
 
-  mus::TestWindow test_window;
+  ui::TestWindow test_window;
   std::unique_ptr<ui::Event> event(GenerateKeyEvent());
   scoped_refptr<TestCallback> test_callback(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback)));
+          base::Bind(&TestCallback::ResultCallback, test_callback)));
 
   OnWindowInputEvent(&test_window, *event.get(), &ack_callback);
   // OnWindowInputEvent is expected to clear the callback if it plans on
@@ -363,12 +372,12 @@ TEST_F(CompositorMusConnectionTest, Consumed) {
   input_handler->set_delegate(connection());
   input_handler->set_state(InputEventAckState::INPUT_EVENT_ACK_STATE_CONSUMED);
 
-  mus::TestWindow test_window;
+  ui::TestWindow test_window;
   std::unique_ptr<ui::Event> event(GenerateKeyEvent());
   scoped_refptr<TestCallback> test_callback(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback)));
+          base::Bind(&TestCallback::ResultCallback, test_callback)));
 
   OnWindowInputEvent(&test_window, *event.get(), &ack_callback);
   // OnWindowInputEvent is expected to clear the callback if it plans on
@@ -385,12 +394,12 @@ TEST_F(CompositorMusConnectionTest, Consumed) {
 // Tests that when the RenderWidgetInputHandler does not ack before a new event
 // arrives, that only the most recent ack is fired.
 TEST_F(CompositorMusConnectionTest, LostAck) {
-  mus::TestWindow test_window;
+  ui::TestWindow test_window;
   std::unique_ptr<ui::Event> event1(GenerateKeyEvent());
   scoped_refptr<TestCallback> test_callback1(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback1(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback1)));
+          base::Bind(&TestCallback::ResultCallback, test_callback1)));
 
   OnWindowInputEvent(&test_window, *event1.get(), &ack_callback1);
   EXPECT_FALSE(ack_callback1.get());
@@ -407,7 +416,7 @@ TEST_F(CompositorMusConnectionTest, LostAck) {
   scoped_refptr<TestCallback> test_callback2(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback2(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback2)));
+          base::Bind(&TestCallback::ResultCallback, test_callback2)));
   OnWindowInputEvent(&test_window, *event2.get(), &ack_callback2);
   EXPECT_FALSE(ack_callback2.get());
 
@@ -420,43 +429,45 @@ TEST_F(CompositorMusConnectionTest, LostAck) {
 }
 
 // Tests that when an input handler consumes the event, that
-// CompositorMusConnection does not consume the ack, nor calls it.
+// CompositorMusConnection will consume the ack, but call as UNHANDLED.
 TEST_F(CompositorMusConnectionTest, InputHandlerConsumes) {
   input_handler_manager()->SetHandleInputEventResult(
       InputEventAckState::INPUT_EVENT_ACK_STATE_CONSUMED);
-  mus::TestWindow test_window;
+  ui::TestWindow test_window;
   std::unique_ptr<ui::Event> event(GenerateKeyEvent());
   scoped_refptr<TestCallback> test_callback(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback)));
+          base::Bind(&TestCallback::ResultCallback, test_callback)));
 
   OnWindowInputEvent(&test_window, *event.get(), &ack_callback);
 
-  EXPECT_TRUE(ack_callback.get());
+  EXPECT_FALSE(ack_callback.get());
   VerifyAndRunQueues(false, false);
-  EXPECT_FALSE(test_callback->called());
+  EXPECT_TRUE(test_callback->called());
+  EXPECT_EQ(EventResult::UNHANDLED, test_callback->result());
 }
 
 // Tests that when the renderer will not ack an event, that
-// CompositorMusConnection does not consume the ack, nor calls it.
+// CompositorMusConnection will consume the ack, but call as UNHANDLED.
 TEST_F(CompositorMusConnectionTest, RendererWillNotSendAck) {
-  mus::TestWindow test_window;
+  ui::TestWindow test_window;
   ui::PointerEvent event(
-      ui::ET_POINTER_DOWN, gfx::Point(), gfx::Point(), ui::EF_NONE, 0,
+      ui::ET_POINTER_DOWN, gfx::Point(), gfx::Point(), ui::EF_NONE, 0, 0,
       ui::PointerDetails(ui::EventPointerType::POINTER_TYPE_MOUSE),
       ui::EventTimeForNow());
 
   scoped_refptr<TestCallback> test_callback(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback)));
+          base::Bind(&TestCallback::ResultCallback, test_callback)));
 
   OnWindowInputEvent(&test_window, event, &ack_callback);
-  EXPECT_TRUE(ack_callback.get());
+  EXPECT_FALSE(ack_callback.get());
 
   VerifyAndRunQueues(true, false);
-  EXPECT_FALSE(test_callback->called());
+  EXPECT_TRUE(test_callback->called());
+  EXPECT_EQ(EventResult::UNHANDLED, test_callback->result());
 }
 
 // Tests that when a touch event id provided, that CompositorMusConnection
@@ -466,16 +477,16 @@ TEST_F(CompositorMusConnectionTest, TouchEventConsumed) {
   input_handler->set_delegate(connection());
   input_handler->set_state(InputEventAckState::INPUT_EVENT_ACK_STATE_CONSUMED);
 
-  mus::TestWindow test_window;
+  ui::TestWindow test_window;
   ui::PointerEvent event(
-      ui::ET_POINTER_DOWN, gfx::Point(), gfx::Point(), ui::EF_NONE, 0,
+      ui::ET_POINTER_DOWN, gfx::Point(), gfx::Point(), ui::EF_NONE, 0, 0,
       ui::PointerDetails(ui::EventPointerType::POINTER_TYPE_TOUCH),
       ui::EventTimeForNow());
 
   scoped_refptr<TestCallback> test_callback(new TestCallback);
   std::unique_ptr<base::Callback<void(EventResult)>> ack_callback(
       new base::Callback<void(EventResult)>(
-          base::Bind(&::TestCallback::ResultCallback, test_callback)));
+          base::Bind(&TestCallback::ResultCallback, test_callback)));
 
   OnWindowInputEvent(&test_window, event, &ack_callback);
   // OnWindowInputEvent is expected to clear the callback if it plans on
