@@ -24,20 +24,18 @@
 #include "base/strings/string16.h"
 #include "base/timer/timer.h"
 #include "build/build_config.h"
-#include "cc/input/top_controls_state.h"
+#include "cc/input/browser_controls_state.h"
 #include "cc/resources/shared_bitmap.h"
 #include "content/common/content_export.h"
-#include "content/common/drag_event_source_info.h"
-#include "content/common/edit_command.h"
 #include "content/common/frame_message_enums.h"
 #include "content/common/navigation_gesture.h"
 #include "content/common/page_message_enums.h"
 #include "content/common/view_message_enums.h"
+#include "content/public/common/browser_controls_state.h"
 #include "content/public/common/drop_data.h"
 #include "content/public/common/page_zoom.h"
 #include "content/public/common/referrer.h"
 #include "content/public/common/renderer_preferences.h"
-#include "content/public/common/top_controls_state.h"
 #include "content/public/common/web_preferences.h"
 #include "content/public/renderer/render_view.h"
 #include "content/renderer/render_frame_impl.h"
@@ -45,6 +43,7 @@
 #include "content/renderer/render_widget_owner_delegate.h"
 #include "content/renderer/stats_collection_observer.h"
 #include "ipc/ipc_platform_file.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
 #include "third_party/WebKit/public/platform/WebSecurityOrigin.h"
 #include "third_party/WebKit/public/web/WebAXObject.h"
 #include "third_party/WebKit/public/web/WebConsoleMessage.h"
@@ -53,7 +52,6 @@
 #include "third_party/WebKit/public/web/WebFrameWidget.h"
 #include "third_party/WebKit/public/web/WebHistoryItem.h"
 #include "third_party/WebKit/public/web/WebIconURL.h"
-#include "third_party/WebKit/public/web/WebInputEvent.h"
 #include "third_party/WebKit/public/web/WebNavigationType.h"
 #include "third_party/WebKit/public/web/WebNode.h"
 #include "third_party/WebKit/public/web/WebViewClient.h"
@@ -65,7 +63,6 @@
 #if defined(OS_ANDROID)
 #include "content/renderer/android/content_detector.h"
 #include "content/renderer/android/renderer_date_time_picker.h"
-#include "third_party/WebKit/public/web/WebContentDetectionResult.h"
 #endif
 
 #if defined(COMPILER_MSVC)
@@ -75,37 +72,17 @@
 #pragma warning(disable: 4250)
 #endif
 
-class SkBitmap;
-struct PP_NetAddress_Private;
-struct ViewMsg_New_Params;
-struct ViewMsg_StopFinding_Params;
-
-namespace base {
-class CommandLine;
-}
-
 namespace blink {
-class WebApplicationCacheHost;
-class WebApplicationCacheHostClient;
-class WebDOMMessageEvent;
 class WebDataSource;
 class WebDateTimeChooserCompletion;
-class WebDragData;
 class WebGestureEvent;
 class WebIconURL;
-class WebImage;
-class WebPeerConnection00Handler;
-class WebPeerConnection00HandlerClient;
 class WebMouseEvent;
-class WebPeerConnectionHandler;
-class WebPeerConnectionHandlerClient;
 class WebSpeechRecognizer;
 class WebStorageNamespace;
-class WebTouchEvent;
 class WebURLRequest;
 struct WebActiveWheelFlingParameters;
 struct WebDateTimeChooserParams;
-struct WebFileChooserParams;
 struct WebMediaPlayerAction;
 struct WebPluginAction;
 struct WebPoint;
@@ -119,24 +96,18 @@ class WebHitTestResult;
 namespace content {
 
 class HistoryController;
-class HistoryEntry;
-class PageState;
+class RendererDateTimePicker;
 class RenderViewImplTest;
 class RenderViewObserver;
 class RenderViewTest;
-class RendererDateTimePicker;
-class RendererWebColorChooserImpl;
 class SpeechRecognitionDispatcher;
-class WebPluginDelegateProxy;
 struct FaviconURL;
 struct FileChooserParams;
-struct FileChooserFileInfo;
-struct RenderViewImplParams;
 struct ResizeParams;
 
-#if defined(OS_ANDROID)
-class WebMediaPlayerProxyAndroid;
-#endif
+namespace mojom {
+class CreateViewParams;
+}
 
 //
 // RenderView is an object that manages a WebView object, and provides a
@@ -154,21 +125,23 @@ class CONTENT_EXPORT RenderViewImpl
       public RenderView,
       public base::SupportsWeakPtr<RenderViewImpl> {
  public:
-  // Creates a new RenderView. |opener_id| is the routing ID of the RenderView
-  // responsible for creating this RenderView. Note that if the original opener
-  // has been closed, |window_was_created_with_opener| will be true and
-  // |opener_id| will be MSG_ROUTING_NONE. When |swapped_out| is true, the
-  // |proxy_routing_id| is specified, so a RenderFrameProxy can be created for
-  // this RenderView's main RenderFrame.
-  static RenderViewImpl* Create(CompositorDependencies* compositor_deps,
-                                const ViewMsg_New_Params& params,
-                                bool was_created_by_renderer);
+  // Creates a new RenderView. Note that if the original opener has been closed,
+  // |params.window_was_created_with_opener| will be true and
+  // |params.opener_frame_route_id| will be MSG_ROUTING_NONE. When
+  // |params.swapped_out| is true, |params.proxy_routing_id| is specified, so a
+  // RenderFrameProxy can be created for this RenderView's main RenderFrame. The
+  // opener should provide a non-null value for |show_callback| if it needs to
+  // send an additional IPC to finish making this view visible.
+  static RenderViewImpl* Create(
+      CompositorDependencies* compositor_deps,
+      const mojom::CreateViewParams& params,
+      const RenderWidget::ShowCallback& show_callback);
 
   // Used by content_layouttest_support to hook into the creation of
   // RenderViewImpls.
   static void InstallCreateHook(RenderViewImpl* (*create_render_view_impl)(
       CompositorDependencies* compositor_deps,
-      const ViewMsg_New_Params&));
+      const mojom::CreateViewParams&));
 
   // Returns the RenderViewImpl containing the given WebView.
   static RenderViewImpl* FromWebView(blink::WebView* webview);
@@ -179,6 +152,9 @@ class CONTENT_EXPORT RenderViewImpl
   // May return NULL when the view is closing.
   blink::WebView* webview() const;
 
+  // Returns the RenderWidget for this RenderView.
+  RenderWidget* GetWidget() const;
+
   const WebPreferences& webkit_preferences() const {
     return webkit_preferences_;
   }
@@ -223,8 +199,7 @@ class CONTENT_EXPORT RenderViewImpl
   void FrameDidStartLoading(blink::WebFrame* frame);
   void FrameDidStopLoading(blink::WebFrame* frame);
 
-  // Sets the zoom level and notifies observers. Doesn't call zoomLevelChanged,
-  // as that is only for changes that aren't initiated by the client.
+  // Sets the zoom level and notifies observers.
   void SetZoomLevel(double zoom_level);
 
   double page_zoom_level() {
@@ -237,12 +212,6 @@ class CONTENT_EXPORT RenderViewImpl
 
   void AttachWebFrameWidget(blink::WebFrameWidget* frame_widget);
 
-  // Plugin-related functions --------------------------------------------------
-
-#if defined(ENABLE_PLUGINS)
-  PepperPluginInstanceImpl* GetFocusedPepperPlugin();
-#endif  // ENABLE_PLUGINS
-
   void TransferActiveWheelFlingAnimation(
       const blink::WebActiveWheelFlingParameters& params);
 
@@ -254,6 +223,15 @@ class CONTENT_EXPORT RenderViewImpl
   // Synchronously sends the current navigation state to the browser.
   void SendUpdateState();
 
+  // A popup widget opened by this view needs to be shown.
+  void ShowCreatedPopupWidget(RenderWidget* popup_widget,
+                              blink::WebNavigationPolicy policy,
+                              const gfx::Rect& initial_rect);
+  // A RenderWidgetFullscreen widget opened by this view needs to be shown.
+  void ShowCreatedFullscreenWidget(RenderWidget* fullscreen_widget,
+                                   blink::WebNavigationPolicy policy,
+                                   const gfx::Rect& initial_rect);
+
   // Returns the length of the session history of this RenderView. Note that
   // this only coincides with the actual length of the session history if this
   // RenderView is the currently active RenderView of a WebContents.
@@ -284,44 +262,32 @@ class CONTENT_EXPORT RenderViewImpl
   // blink::WebWidgetClient implementation ------------------------------------
 
   // Most methods are handled by RenderWidget.
-  void didFocus() override;
   void show(blink::WebNavigationPolicy policy) override;
   void didHandleGestureEvent(const blink::WebGestureEvent& event,
                              bool event_cancelled) override;
   void onMouseDown(const blink::WebNode& mouse_down_node) override;
-  void initializeLayerTreeView() override;
+  blink::WebLayerTreeView* initializeLayerTreeView() override;
 
   // TODO(lfg): Remove once WebViewClient no longer inherits from
   // WebWidgetClient.
-  bool allowsBrokenNullLayerTreeView() const override;
   void closeWidgetSoon() override;
   void convertViewportToWindow(blink::WebRect* rect) override;
   void convertWindowToViewport(blink::WebFloatRect* rect) override;
-  void didAutoResize(const blink::WebSize& newSize) override;
-  void didChangeCursor(const blink::WebCursorInfo& info) override;
-  void didInvalidateRect(const blink::WebRect& rect) override;
-  void didMeaningfulLayout(blink::WebMeaningfulLayout layout_type) override;
   void didOverscroll(const blink::WebFloatSize& overscrollDelta,
                      const blink::WebFloatSize& accumulatedOverscroll,
                      const blink::WebFloatPoint& positionInViewport,
                      const blink::WebFloatSize& velocityInViewport) override;
-  void didUpdateTextOfFocusedElementByNonUserInput() override;
   void hasTouchEventHandlers(bool has_handlers) override;
-  blink::WebLayerTreeView* layerTreeView() override;
   void resetInputMethod() override;
-  blink::WebRect rootWindowRect() override;
-  void scheduleAnimation() override;
   blink::WebScreenInfo screenInfo() override;
   void setToolTipText(const blink::WebString&,
                       blink::WebTextDirection hint) override;
   void setTouchAction(blink::WebTouchAction touchAction) override;
-  void setWindowRect(const blink::WebRect& rect) override;
-  void showImeIfNeeded() override;
+  void showVirtualKeyboard() override;
   void showUnhandledTapUIIfNeeded(const blink::WebPoint& tappedPosition,
                                   const blink::WebNode& tappedNode,
                                   bool pageChanged) override;
-  blink::WebRect windowRect() override;
-  blink::WebRect windowResizerRect() override;
+  blink::WebWidgetClient* widgetClient() override;
 
   // blink::WebViewClient implementation --------------------------------------
 
@@ -338,7 +304,6 @@ class CONTENT_EXPORT RenderViewImpl
       const blink::WebString& path,
       blink::WebFileChooserCompletion* chooser_completion) override;
   void didCancelCompositionOnSelectionChange() override;
-  bool handleCurrentKeyboardEvent() override;
   void SetValidationMessageDirection(base::string16* main_text,
                                      blink::WebTextDirection main_text_hint,
                                      base::string16* sub_text,
@@ -354,11 +319,6 @@ class CONTENT_EXPORT RenderViewImpl
   void setStatusText(const blink::WebString& text) override;
   void setMouseOverURL(const blink::WebURL& url) override;
   void setKeyboardFocusURL(const blink::WebURL& url) override;
-  void startDragging(blink::WebLocalFrame* frame,
-                     const blink::WebDragData& data,
-                     blink::WebDragOperationsMask mask,
-                     const blink::WebImage& image,
-                     const blink::WebPoint& imageOffset) override;
   bool acceptsLoadDrops() override;
   void focusNext() override;
   void focusPrevious() override;
@@ -377,18 +337,20 @@ class CONTENT_EXPORT RenderViewImpl
   int historyForwardListCount() override;
   blink::WebSpeechRecognizer* speechRecognizer() override;
   void zoomLimitsChanged(double minimum_level, double maximum_level) override;
-  virtual void zoomLevelChanged();
   void pageScaleFactorChanged() override;
   virtual double zoomLevelToZoomFactor(double zoom_level) const;
   virtual double zoomFactorToZoomLevel(double factor) const;
   void draggableRegionsChanged() override;
   void pageImportanceSignalsChanged() override;
+  void didAutoResize(const blink::WebSize& newSize) override;
+  blink::WebRect rootWindowRect() override;
+  void didFocus() override;
 
 #if defined(OS_ANDROID)
   void scheduleContentIntent(const blink::WebURL& intent,
                              bool is_main_frame) override;
   void cancelScheduledContentIntents() override;
-  blink::WebContentDetectionResult detectContentAround(
+  blink::WebURL detectContentIntentAt(
       const blink::WebHitTestResult& touch_hit) override;
 
   // Only used on Android since all other platforms implement
@@ -401,7 +363,6 @@ class CONTENT_EXPORT RenderViewImpl
   // RenderView implementation -------------------------------------------------
 
   bool Send(IPC::Message* message) override;
-  RenderWidget* GetWidget() const override;
   RenderFrameImpl* GetMainRenderFrame() override;
   int GetRoutingID() const override;
   gfx::Size GetSize() const override;
@@ -413,26 +374,20 @@ class CONTENT_EXPORT RenderViewImpl
   bool ShouldDisplayScrollbars(int width, int height) const override;
   int GetEnabledBindings() const override;
   bool GetContentStateImmediately() const override;
-  void DidStartLoading() override;
-  void DidStopLoading() override;
   void Repaint(const gfx::Size& size) override;
   void SetEditCommandForNextKeyEvent(const std::string& name,
                                      const std::string& value) override;
   void ClearEditCommands() override;
-  SSLStatus GetSSLStatusOfFrame(blink::WebFrame* frame) const override;
   const std::string& GetAcceptLanguages() const override;
 #if defined(OS_ANDROID)
-  void UpdateTopControlsState(TopControlsState constraints,
-                              TopControlsState current,
-                              bool animate) override;
+  void UpdateBrowserControlsState(BrowserControlsState constraints,
+                                  BrowserControlsState current,
+                                  bool animate) override;
 #endif
   void ConvertViewportToWindowViaWidget(blink::WebRect* rect) override;
   gfx::RectF ElementBoundsInWindow(const blink::WebElement& element) override;
-  float GetDeviceScaleFactorForTest() const override;
   bool HasAddedInputHandler() const override;
 
-  gfx::Point ConvertWindowPointToViewport(const gfx::Point& point);
-
   bool uses_temporary_zoom_level() const { return uses_temporary_zoom_level_; }
 
   // Please do not add your stuff randomly to the end here. If there is an
@@ -441,37 +396,23 @@ class CONTENT_EXPORT RenderViewImpl
 
  protected:
   // RenderWidget overrides:
+  blink::WebWidget* GetWebWidget() const override;
   void CloseForFrame() override;
   void Close() override;
   void OnResize(const ResizeParams& params) override;
   void OnSetFocus(bool enable) override;
   GURL GetURLForGraphicsContext3D() override;
-  void OnImeSetComposition(
-      const base::string16& text,
-      const std::vector<blink::WebCompositionUnderline>& underlines,
-      const gfx::Range& replacement_range,
-      int selection_start,
-      int selection_end) override;
-  void OnImeConfirmComposition(const base::string16& text,
-                               const gfx::Range& replacement_range,
-                               bool keep_selection) override;
   void OnOrientationChange() override;
-  ui::TextInputType GetTextInputType() override;
-  void GetSelectionBounds(gfx::Rect* start, gfx::Rect* end) override;
-  void GetCompositionCharacterBounds(
-      std::vector<gfx::Rect>* character_bounds_in_window) override;
-  void GetCompositionRange(gfx::Range* range) override;
-  bool CanComposeInline() override;
   void DidCommitCompositorFrame() override;
   void DidCompletePageScaleAnimation() override;
   void OnDeviceScaleFactorChanged() override;
   void ResizeWebWidget() override;
 
   RenderViewImpl(CompositorDependencies* compositor_deps,
-                 const ViewMsg_New_Params& params);
+                 const mojom::CreateViewParams& params);
 
-  void Initialize(const ViewMsg_New_Params& params,
-                  bool was_created_by_renderer);
+  void Initialize(const mojom::CreateViewParams& params,
+                  const RenderWidget::ShowCallback& show_callback);
   void SetScreenMetricsEmulationParameters(
       bool enabled,
       const blink::WebDeviceEmulationParams& params) override;
@@ -526,7 +467,6 @@ class CONTENT_EXPORT RenderViewImpl
   FRIEND_TEST_ALL_PREFIXES(RenderViewTest, MacTestCmdUp);
 #endif
   FRIEND_TEST_ALL_PREFIXES(RenderViewImplTest, SetHistoryLengthAndOffset);
-  FRIEND_TEST_ALL_PREFIXES(RenderViewImplTest, ZoomLimit);
   FRIEND_TEST_ALL_PREFIXES(RenderViewImplTest, NavigateFrame);
   FRIEND_TEST_ALL_PREFIXES(RenderViewImplTest, BasicRenderFrame);
   FRIEND_TEST_ALL_PREFIXES(RenderViewImplTest, TextInputTypeWithPepper);
@@ -542,8 +482,6 @@ class CONTENT_EXPORT RenderViewImpl
   FRIEND_TEST_ALL_PREFIXES(RenderViewImplScaleFactorTest,
                            GetCompositionCharacterBoundsTest);
 
-  typedef std::map<GURL, double> HostZoomLevels;
-
   enum ErrorPageType {
     DNS_ERROR,
     HTTP_404,
@@ -557,12 +495,8 @@ class CONTENT_EXPORT RenderViewImpl
   void RenderWidgetFocusChangeComplete() override;
   bool DoesRenderWidgetHaveTouchEventHandlersAt(
       const gfx::Point& point) const override;
-  void RenderWidgetDidHandleKeyEvent() override;
-  bool RenderWidgetWillHandleGestureEvent(
-      const blink::WebGestureEvent& event) override;
   bool RenderWidgetWillHandleMouseEvent(
       const blink::WebMouseEvent& event) override;
-  void RenderWidgetDidFlushPaint() override;
 
   // Old WebFrameClient implementations ----------------------------------------
 
@@ -591,11 +525,9 @@ class CONTENT_EXPORT RenderViewImpl
   void OnExecuteEditCommand(const std::string& name, const std::string& value);
   void OnMoveCaret(const gfx::Point& point);
   void OnScrollFocusedEditableNodeIntoRect(const gfx::Rect& rect);
-  void OnSetEditCommandsForNextKeyEvent(const EditCommands& edit_commands);
   void OnAllowBindings(int enabled_bindings_flags);
   void OnAllowScriptToClose(bool script_can_close);
   void OnCancelDownload(int32_t download_id);
-  void OnClearFocusedElement();
   void OnClosePage();
   void OnClose();
 
@@ -604,26 +536,6 @@ class CONTENT_EXPORT RenderViewImpl
   void OnDeterminePageLanguage();
   void OnDisableScrollbarsForSmallWindows(
       const gfx::Size& disable_scrollbars_size_limit);
-  void OnDragSourceEnded(const gfx::Point& client_point,
-                         const gfx::Point& screen_point,
-                         blink::WebDragOperation drag_operation);
-  void OnDragSourceSystemDragEnded();
-  void OnDragTargetDrop(const DropData& drop_data,
-                        const gfx::Point& client_pt,
-                        const gfx::Point& screen_pt,
-                        int key_modifiers);
-  // Real data that is dragged is not included at DragEnter time.
-  void OnDragTargetDragEnter(
-      const std::vector<DropData::Metadata>& drop_meta_data,
-      const gfx::Point& client_pt,
-      const gfx::Point& screen_pt,
-      blink::WebDragOperationsMask operations_allowed,
-      int key_modifiers);
-  void OnDragTargetDragLeave();
-  void OnDragTargetDragOver(const gfx::Point& client_pt,
-                            const gfx::Point& screen_pt,
-                            blink::WebDragOperationsMask operations_allowed,
-                            int key_modifiers);
   void OnEnablePreferredSizeChangedMode();
   void OnEnableAutoResize(const gfx::Size& min_size, const gfx::Size& max_size);
   void OnDisableAutoResize(const gfx::Size& new_size);
@@ -635,29 +547,25 @@ class CONTENT_EXPORT RenderViewImpl
                         const blink::WebPluginAction& action);
   void OnMoveOrResizeStarted();
   void OnReleaseDisambiguationPopupBitmap(const cc::SharedBitmapId& id);
-  void OnResetPageEncodingToDefault();
   void OnSetActive(bool active);
   void OnSetBackgroundOpaque(bool opaque);
   void OnExitFullscreen();
   void OnSetHistoryOffsetAndLength(int history_offset, int history_length);
   void OnSetInitialFocus(bool reverse);
-  void OnSetPageEncoding(const std::string& encoding_name);
   void OnSetRendererPrefs(const RendererPreferences& renderer_prefs);
   void OnSetWebUIProperty(const std::string& name, const std::string& value);
-  void OnSetZoomLevelForLoadingURL(const GURL& url, double zoom_level);
   void OnSuppressDialogsUntilSwapOut();
-  void OnThemeChanged();
   void OnUpdateTargetURLAck();
   void OnUpdateWebPreferences(const WebPreferences& prefs);
   void OnSetPageScale(float page_scale_factor);
-  void OnZoom(PageZoom zoom);
-  void OnForceRedraw(int request_id);
+  void OnForceRedraw(const ui::LatencyInfo& latency_info);
   void OnSelectWordAroundCaret();
+  void OnAudioStateChanged(bool is_audio_playing);
 #if defined(OS_ANDROID)
   void OnUndoScrollFocusedEditableNodeIntoRect();
-  void OnUpdateTopControlsState(bool enable_hiding,
-                                bool enable_showing,
-                                bool animate);
+  void OnUpdateBrowserControlsState(bool enable_hiding,
+                                    bool enable_showing,
+                                    bool animate);
   void OnExtractSmartClipData(const gfx::Rect& rect);
 #elif defined(OS_MACOSX)
   void OnGetRenderedText();
@@ -722,8 +630,20 @@ class CONTENT_EXPORT RenderViewImpl
     navigation_gesture_ = gesture;
   }
 
+// Platform specific theme preferences if any are updated here.
+#if defined(OS_WIN)
+  void UpdateThemePrefs();
+#else
+  void UpdateThemePrefs() {}
+#endif
+
   void UpdateWebViewWithDeviceScaleFactor();
 
+  // Send the appropriate ack to be able discard this input event message.
+  void OnDiscardInputEvent(const blink::WebInputEvent* input_event,
+                           const ui::LatencyInfo& latency_info,
+                           InputEventDispatchType dispatch_type);
+
   // ---------------------------------------------------------------------------
   // ADDING NEW FUNCTIONS? Please keep private functions alphabetized and put
   // it in the same order in the .cc file as it was in the header.
@@ -734,8 +654,6 @@ class CONTENT_EXPORT RenderViewImpl
   WebPreferences webkit_preferences_;
   RendererPreferences renderer_preferences_;
 
-  HostZoomLevels host_zoom_levels_;
-
   // Whether content state (such as form state, scroll position and page
   // contents) should be sent to the browser immediately. This is normally
   // false, but set to true by some tests.
@@ -765,9 +683,6 @@ class CONTENT_EXPORT RenderViewImpl
   // TODO(nasko): Move to RenderFrame, as this is per-frame state.
   NavigationGesture navigation_gesture_;
 
-  // Used for popups.
-  bool opened_by_user_gesture_;
-
   // Timer used to delay the updating of nav state (see
   // StartNavStateSyncTimerIfNecessary).
   base::OneShotTimer nav_state_sync_timer_;
@@ -776,13 +691,7 @@ class CONTENT_EXPORT RenderViewImpl
   // messages to send when the next |nav_state_sync_timer_| fires.
   std::set<int> frames_with_pending_state_;
 
-  // Page IDs ------------------------------------------------------------------
-  // See documentation in RenderView.
-  int32_t page_id_;
-
-  // The next available page ID to use for this RenderView.  These IDs are
-  // specific to a given RenderView and the frames within it.
-  int32_t next_page_id_;
+  // History list --------------------------------------------------------------
 
   // The offset of the current item in the history list.
   int history_list_offset_;
@@ -834,16 +743,16 @@ class CONTENT_EXPORT RenderViewImpl
   bool uses_temporary_zoom_level_;
 
 #if defined(OS_ANDROID)
-  // Cache the old top controls state constraints. Used when updating
+  // Cache the old browser controls state constraints. Used when updating
   // current value only without altering the constraints.
-  TopControlsState top_controls_constraints_;
+  BrowserControlsState top_controls_constraints_;
 #endif
 
   // Whether or not Blink's viewport size should be shrunk by the height of the
   // URL-bar.
-  bool top_controls_shrink_blink_size_;
+  bool browser_controls_shrink_blink_size_;
 
-  // The height of the top controls.
+  // The height of the browser controls.
   float top_controls_height_;
 
   // View ----------------------------------------------------------------------
@@ -898,6 +807,8 @@ class CONTENT_EXPORT RenderViewImpl
   // A date/time picker object for date and time related input elements.
   std::unique_ptr<RendererDateTimePicker> date_time_picker_client_;
 
+  // Whether this was a renderer-created or browser-created RenderView.
+  bool was_created_by_renderer_;
 #endif
 
   // Misc ----------------------------------------------------------------------
@@ -911,19 +822,10 @@ class CONTENT_EXPORT RenderViewImpl
   // uses it whenever asking the browser process to allocate new storage areas.
   int64_t session_storage_namespace_id_;
 
-  // Stores edit commands associated to the next key event.
-  // Shall be cleared as soon as the next key event is processed.
-  EditCommands edit_commands_;
-
   // All the registered observers.  We expect this list to be small, so vector
   // is fine.
   base::ObserverList<RenderViewObserver> observers_;
 
-  // This field stores drag/drop related info for the event that is currently
-  // being handled. If the current event results in starting a drag/drop
-  // session, this info is sent to the browser along with other drag/drop info.
-  DragEventSourceInfo possible_drag_event_info_;
-
   // NOTE: stats_collection_observer_ should be the last members because their
   // constructors call the AddObservers method of RenderViewImpl.
   std::unique_ptr<StatsCollectionObserver> stats_collection_observer_;
