@@ -7,23 +7,24 @@ package org.chromium.content_browsertests_apk;
 import android.content.Context;
 
 import org.chromium.base.BaseChromiumApplication;
+import org.chromium.base.ContextUtils;
 import org.chromium.base.PathUtils;
 
 /**
  * A basic content browser tests {@link android.app.Application}.
  */
 public class ContentBrowserTestsApplication extends BaseChromiumApplication {
-
     static final String PRIVATE_DATA_DIRECTORY_SUFFIX = "content_shell";
 
+    @Override
+    protected void attachBaseContext(Context base) {
+        super.attachBaseContext(base);
+        ContextUtils.initApplicationContext(this);
+    }
+
     @Override
     public void onCreate() {
         super.onCreate();
-        initializeApplicationParameters(this);
+        PathUtils.setPrivateDataDirectorySuffix(PRIVATE_DATA_DIRECTORY_SUFFIX);
     }
-
-    public static void initializeApplicationParameters(Context context) {
-        PathUtils.setPrivateDataDirectorySuffix(PRIVATE_DATA_DIRECTORY_SUFFIX, context);
-    }
-
-}
+}
\ No newline at end of file
