@@ -6,35 +6,36 @@
 
 #include <string>
 
+#include "base/memory/ptr_util.h"
 #include "content/public/test/layouttest_support.h"
 #include "content/shell/browser/layout_test/layout_test_bluetooth_adapter_provider.h"
+#include "device/bluetooth/bluetooth_adapter_factory_wrapper.h"
+#include "mojo/public/cpp/bindings/strong_binding.h"
 
 namespace content {
 
-// static
-void LayoutTestBluetoothFakeAdapterSetterImpl::Create(
-    int render_process_id,
-    mojom::LayoutTestBluetoothFakeAdapterSetterRequest request) {
-  new LayoutTestBluetoothFakeAdapterSetterImpl(render_process_id,
-                                               std::move(request));
-}
-
 LayoutTestBluetoothFakeAdapterSetterImpl::
-    LayoutTestBluetoothFakeAdapterSetterImpl(
-        int render_process_id,
-        mojom::LayoutTestBluetoothFakeAdapterSetterRequest request)
-    : render_process_id_(render_process_id),
-      binding_(this, std::move(request)) {}
+    LayoutTestBluetoothFakeAdapterSetterImpl() {}
 
 LayoutTestBluetoothFakeAdapterSetterImpl::
     ~LayoutTestBluetoothFakeAdapterSetterImpl() {}
 
+// static
+void LayoutTestBluetoothFakeAdapterSetterImpl::Create(
+    mojom::LayoutTestBluetoothFakeAdapterSetterRequest request) {
+  mojo::MakeStrongBinding(
+      base::MakeUnique<LayoutTestBluetoothFakeAdapterSetterImpl>(),
+      std::move(request));
+}
+
 void LayoutTestBluetoothFakeAdapterSetterImpl::Set(
-    const mojo::String& adapter_name,
+    const std::string& adapter_name,
     const SetCallback& callback) {
-  SetBluetoothAdapter(render_process_id_,
-                      LayoutTestBluetoothAdapterProvider::GetBluetoothAdapter(
-                          adapter_name));
+  SetTestBluetoothScanDuration();
+
+  device::BluetoothAdapterFactoryWrapper::Get().SetBluetoothAdapterForTesting(
+      LayoutTestBluetoothAdapterProvider::GetBluetoothAdapter(adapter_name));
+
   callback.Run();
 }
 
