@@ -31,12 +31,13 @@
 #include "base/values.h"
 #include "build/build_config.h"
 #include "components/plugins/renderer/plugin_placeholder.h"
+#include "components/test_runner/app_banner_service.h"
 #include "components/test_runner/gamepad_controller.h"
 #include "components/test_runner/layout_and_paint_async_then.h"
 #include "components/test_runner/pixel_dump.h"
 #include "components/test_runner/web_test_interfaces.h"
-#include "components/test_runner/web_test_proxy.h"
 #include "components/test_runner/web_test_runner.h"
+#include "components/test_runner/web_view_test_proxy.h"
 #include "content/common/content_switches_internal.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/common/url_constants.h"
@@ -55,25 +56,27 @@
 #include "content/shell/renderer/layout_test/leak_detector.h"
 #include "media/base/audio_capturer_source.h"
 #include "media/base/audio_parameters.h"
-#include "media/base/video_capturer_source.h"
+#include "media/capture/video_capturer_source.h"
+#include "media/media_features.h"
 #include "net/base/filename_util.h"
 #include "net/base/net_errors.h"
-#include "services/shell/public/cpp/interface_provider.h"
+#include "services/service_manager/public/cpp/interface_provider.h"
+#include "services/service_manager/public/cpp/interface_registry.h"
 #include "skia/ext/platform_canvas.h"
 #include "third_party/WebKit/public/platform/FilePathConversion.h"
 #include "third_party/WebKit/public/platform/Platform.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
 #include "third_party/WebKit/public/platform/WebPoint.h"
 #include "third_party/WebKit/public/platform/WebRect.h"
 #include "third_party/WebKit/public/platform/WebSize.h"
 #include "third_party/WebKit/public/platform/WebString.h"
-#include "third_party/WebKit/public/platform/WebTaskRunner.h"
 #include "third_party/WebKit/public/platform/WebThread.h"
 #include "third_party/WebKit/public/platform/WebTraceLocation.h"
 #include "third_party/WebKit/public/platform/WebURL.h"
 #include "third_party/WebKit/public/platform/WebURLError.h"
 #include "third_party/WebKit/public/platform/WebURLRequest.h"
 #include "third_party/WebKit/public/platform/WebURLResponse.h"
-#include "third_party/WebKit/public/platform/modules/app_banner/WebAppBannerPromptReply.h"
+#include "third_party/WebKit/public/platform/modules/app_banner/app_banner.mojom.h"
 #include "third_party/WebKit/public/web/WebArrayBufferView.h"
 #include "third_party/WebKit/public/web/WebContextMenuData.h"
 #include "third_party/WebKit/public/web/WebDataSource.h"
@@ -81,6 +84,7 @@
 #include "third_party/WebKit/public/web/WebDocument.h"
 #include "third_party/WebKit/public/web/WebElement.h"
 #include "third_party/WebKit/public/web/WebFrame.h"
+#include "third_party/WebKit/public/web/WebFrameWidget.h"
 #include "third_party/WebKit/public/web/WebHistoryItem.h"
 #include "third_party/WebKit/public/web/WebKit.h"
 #include "third_party/WebKit/public/web/WebLeakDetector.h"
@@ -89,6 +93,7 @@
 #include "third_party/WebKit/public/web/WebTestingSupport.h"
 #include "third_party/WebKit/public/web/WebView.h"
 #include "ui/gfx/geometry/rect.h"
+#include "ui/gfx/icc_profile.h"
 
 using blink::Platform;
 using blink::WebArrayBufferView;
@@ -237,7 +242,7 @@ WebURL RewriteAbsolutePathInWPT(const std::string& utf8_url) {
   base::FilePath new_path =
       LayoutTestRenderThreadObserver::GetInstance()
           ->webkit_source_dir()
-          .Append(FILE_PATH_LITERAL("LayoutTests/imported/wpt/"))
+          .Append(FILE_PATH_LITERAL("LayoutTests/external/wpt/"))
           .AppendASCII(path);
   return WebURL(net::FilePathToFileURL(new_path));
 }
@@ -290,15 +295,16 @@ void BlinkTestRunner::PrintMessage(const std::string& message) {
   Send(new ShellViewHostMsg_PrintMessage(routing_id(), message));
 }
 
-void BlinkTestRunner::PostTask(blink::WebTaskRunner::Task* task) {
-  Platform::current()->currentThread()->getWebTaskRunner()->postTask(
-      WebTraceLocation(__FUNCTION__, __FILE__), task);
+void BlinkTestRunner::PostTask(const base::Closure& task) {
+  Platform::current()->currentThread()->getSingleThreadTaskRunner()->PostTask(
+      FROM_HERE, task);
 }
 
-void BlinkTestRunner::PostDelayedTask(blink::WebTaskRunner::Task* task,
-                                      long long ms) {
-  Platform::current()->currentThread()->getWebTaskRunner()->postDelayedTask(
-      WebTraceLocation(__FUNCTION__, __FILE__), task, ms);
+void BlinkTestRunner::PostDelayedTask(const base::Closure& task, long long ms) {
+  Platform::current()
+      ->currentThread()
+      ->getSingleThreadTaskRunner()
+      ->PostDelayedTask(FROM_HERE, task, base::TimeDelta::FromMilliseconds(ms));
 }
 
 WebString BlinkTestRunner::RegisterIsolatedFileSystem(
@@ -475,10 +481,12 @@ void BlinkTestRunner::SetDatabaseQuota(int quota) {
   Send(new LayoutTestHostMsg_SetDatabaseQuota(routing_id(), quota));
 }
 
-void BlinkTestRunner::SimulateWebNotificationClick(const std::string& title,
-                                                   int action_index) {
+void BlinkTestRunner::SimulateWebNotificationClick(
+    const std::string& title,
+    int action_index,
+    const base::NullableString16& reply) {
   Send(new LayoutTestHostMsg_SimulateWebNotificationClick(routing_id(), title,
-                                                          action_index));
+                                                          action_index, reply));
 }
 
 void BlinkTestRunner::SimulateWebNotificationClose(const std::string& title,
@@ -495,6 +503,19 @@ float BlinkTestRunner::GetWindowToViewportScale() {
   return content::GetWindowToViewportScale(render_view());
 }
 
+std::unique_ptr<blink::WebInputEvent>
+BlinkTestRunner::TransformScreenToWidgetCoordinates(
+    test_runner::WebWidgetTestProxyBase* web_widget_test_proxy_base,
+    const blink::WebInputEvent& event) {
+  return content::TransformScreenToWidgetCoordinates(web_widget_test_proxy_base,
+                                                     event);
+}
+
+test_runner::WebWidgetTestProxyBase* BlinkTestRunner::GetWebWidgetTestProxyBase(
+    blink::WebLocalFrame* frame) {
+  return content::GetWebWidgetTestProxyBase(frame);
+}
+
 void BlinkTestRunner::EnableUseZoomForDSF() {
   base::CommandLine::ForCurrentProcess()->
       AppendSwitch(switches::kEnableUseZoomForDSF);
@@ -505,7 +526,7 @@ bool BlinkTestRunner::IsUseZoomForDSFEnabled() {
 }
 
 void BlinkTestRunner::SetDeviceColorProfile(const std::string& name) {
-  content::SetDeviceColorProfile(render_view(), name);
+  content::SetDeviceColorProfile(render_view(), GetTestingICCProfile(name));
 }
 
 void BlinkTestRunner::SetBluetoothFakeAdapter(const std::string& adapter_name,
@@ -536,25 +557,26 @@ void BlinkTestRunner::SetFocus(blink::WebView* web_view, bool focus) {
     SetFocusAndActivate(render_view, focus);
 }
 
-void BlinkTestRunner::SetAcceptAllCookies(bool accept) {
-  Send(new LayoutTestHostMsg_AcceptAllCookies(routing_id(), accept));
+void BlinkTestRunner::SetBlockThirdPartyCookies(bool block) {
+  Send(new LayoutTestHostMsg_BlockThirdPartyCookies(routing_id(), block));
 }
 
 std::string BlinkTestRunner::PathToLocalResource(const std::string& resource) {
 #if defined(OS_WIN)
-  if (resource.find("/tmp/") == 0) {
+  if (base::StartsWith(resource, "/tmp/", base::CompareCase::SENSITIVE)) {
     // We want a temp file.
     GURL base_url = net::FilePathToFileURL(test_config_.temp_path);
-    return base_url.Resolve(resource.substr(strlen("/tmp/"))).spec();
+    return base_url.Resolve(resource.substr(sizeof("/tmp/") - 1)).spec();
   }
 #endif
 
   // Some layout tests use file://// which we resolve as a UNC path. Normalize
   // them to just file:///.
   std::string result = resource;
-  while (base::ToLowerASCII(result).find("file:////") == 0) {
-    result = result.substr(0, strlen("file:///")) +
-             result.substr(strlen("file:////"));
+  static const size_t kFileLen = sizeof("file:///") - 1;
+  while (base::StartsWith(base::ToLowerASCII(result), "file:////",
+                          base::CompareCase::SENSITIVE)) {
+    result = result.substr(0, kFileLen) + result.substr(kFileLen + 1);
   }
   return RewriteLayoutTestsURL(result, false /* is_wpt_mode */).string().utf8();
 }
@@ -588,6 +610,10 @@ void BlinkTestRunner::TestFinished() {
     RenderView::ForEach(&visitor);
     Send(new ShellViewHostMsg_CaptureSessionHistory(routing_id()));
   } else {
+    // clean out the lifecycle if needed before capturing the layout tree
+    // dump and pixels from the compositor.
+    render_view()->GetWebView()->mainFrame()->toWebLocalFrame()
+        ->frameWidget()->updateAllLifecyclePhases();
     CaptureDump();
   }
 }
@@ -656,13 +682,13 @@ void BlinkTestRunner::SetPermission(const std::string& name,
                                     const GURL& origin,
                                     const GURL& embedding_origin) {
   blink::mojom::PermissionStatus status;
-  if (value == "granted")
+  if (value == "granted") {
     status = blink::mojom::PermissionStatus::GRANTED;
-  else if (value == "prompt")
+  } else if (value == "prompt") {
     status = blink::mojom::PermissionStatus::ASK;
-  else if (value == "denied")
+  } else if (value == "denied") {
     status = blink::mojom::PermissionStatus::DENIED;
-  else {
+  } else {
     NOTREACHED();
     status = blink::mojom::PermissionStatus::DENIED;
   }
@@ -680,22 +706,24 @@ cc::SharedBitmapManager* BlinkTestRunner::GetSharedBitmapManager() {
 }
 
 void BlinkTestRunner::DispatchBeforeInstallPromptEvent(
-    int request_id,
     const std::vector<std::string>& event_platforms,
     const base::Callback<void(bool)>& callback) {
-  // Send the event to the frame.
-  blink::WebAppBannerPromptReply reply;
-  std::vector<blink::WebString> blink_web_strings;
-  for (const auto& platform : event_platforms)
-    blink_web_strings.push_back(blink::WebString::fromUTF8(platform));
-  blink::WebVector<blink::WebString> blink_event_platforms(blink_web_strings);
+  app_banner_service_.reset(new test_runner::AppBannerService());
 
-  WebLocalFrame* main_frame =
-      render_view()->GetWebView()->mainFrame()->toWebLocalFrame();
-  main_frame->willShowInstallBannerPrompt(request_id, blink_event_platforms,
-                                          &reply);
+  service_manager::InterfaceRegistry::TestApi test_api(
+      render_view()->GetMainRenderFrame()->GetInterfaceRegistry());
+  test_api.GetLocalInterface(
+      mojo::MakeRequest(&app_banner_service_->controller()));
 
-  callback.Run(reply == blink::WebAppBannerPromptReply::Cancel);
+  app_banner_service_->SendBannerPromptRequest(event_platforms, callback);
+}
+
+void BlinkTestRunner::ResolveBeforeInstallPromptPromise(
+    const std::string& platform) {
+  if (app_banner_service_) {
+    app_banner_service_->ResolvePromise(platform);
+    app_banner_service_.reset(nullptr);
+  }
 }
 
 blink::WebPlugin* BlinkTestRunner::CreatePluginPlaceholder(
@@ -709,23 +737,26 @@ blink::WebPlugin* BlinkTestRunner::CreatePluginPlaceholder(
   return placeholder->plugin();
 }
 
-float BlinkTestRunner::GetDeviceScaleFactorForTest() const {
-  return render_view()->GetDeviceScaleFactorForTest();
+float BlinkTestRunner::GetDeviceScaleFactor() const {
+  return render_view()->GetDeviceScaleFactor();
 }
 
 void BlinkTestRunner::RunIdleTasks(const base::Closure& callback) {
     SchedulerRunIdleTasks(callback);
 }
 
+void BlinkTestRunner::ForceTextInputStateUpdate(WebFrame* frame) {
+  ForceTextInputStateUpdateForRenderFrame(RenderFrame::FromWebFrame(frame));
+}
+
 bool BlinkTestRunner::AddMediaStreamVideoSourceAndTrack(
     blink::WebMediaStream* stream) {
   DCHECK(stream);
-#if defined(ENABLE_WEBRTC)
-  return AddVideoTrackToMediaStream(
-      base::WrapUnique(new MockVideoCapturerSource()),
-      false,  // is_remote
-      false,  // is_readonly
-      stream);
+#if BUILDFLAG(ENABLE_WEBRTC)
+  return AddVideoTrackToMediaStream(base::MakeUnique<MockVideoCapturerSource>(),
+                                    false,  // is_remote
+                                    false,  // is_readonly
+                                    stream);
 #else
   return false;
 #endif
@@ -734,7 +765,7 @@ bool BlinkTestRunner::AddMediaStreamVideoSourceAndTrack(
 bool BlinkTestRunner::AddMediaStreamAudioSourceAndTrack(
     blink::WebMediaStream* stream) {
   DCHECK(stream);
-#if defined(ENABLE_WEBRTC)
+#if BUILDFLAG(ENABLE_WEBRTC)
   return AddAudioTrackToMediaStream(
       make_scoped_refptr(new MockAudioCapturerSource()),
       48000,  // sample rate
@@ -873,20 +904,7 @@ void BlinkTestRunner::CaptureDumpContinued() {
     return;
   }
 
-#ifndef NDEBUG
-  // Force a layout/paint by the end of the test to ensure test coverage of
-  // incremental painting.
-  test_runner::LayoutAndPaintAsyncThen(
-      render_view()
-          ->GetWebView()
-          ->mainFrame()
-          ->toWebLocalFrame()
-          ->frameWidget(),
-      base::Bind(&BlinkTestRunner::CaptureDumpComplete,
-                 base::Unretained(this)));
-#else
   CaptureDumpComplete();
-#endif
 }
 
 void BlinkTestRunner::OnPixelsDumpCompleted(const SkBitmap& snapshot) {
@@ -924,7 +942,7 @@ mojom::LayoutTestBluetoothFakeAdapterSetter&
 BlinkTestRunner::GetBluetoothFakeAdapterSetter() {
   if (!bluetooth_fake_adapter_setter_) {
     RenderThread::Get()->GetRemoteInterfaces()->GetInterface(
-        mojo::GetProxy(&bluetooth_fake_adapter_setter_));
+        mojo::MakeRequest(&bluetooth_fake_adapter_setter_));
   }
   return *bluetooth_fake_adapter_setter_;
 }
@@ -982,8 +1000,8 @@ void BlinkTestRunner::OnReset() {
   Reset(true /* for_new_test */);
   // Navigating to about:blank will make sure that no new loads are initiated
   // by the renderer.
-  render_view()->GetWebView()->mainFrame()->loadRequest(
-      WebURLRequest(GURL(url::kAboutBlankURL)));
+  WebURLRequest request = WebURLRequest(GURL(url::kAboutBlankURL));
+  render_view()->GetWebView()->mainFrame()->loadRequest(request);
   Send(new ShellViewHostMsg_ResetDone(routing_id()));
 }
 
