@@ -3,16 +3,15 @@
 // found in the LICENSE file.
 
 define('main', [
-    'mojo/public/js/router',
+    'mojo/public/js/bindings',
     'content/test/data/web_ui_test_mojo_bindings.mojom',
-    'content/public/renderer/frame_service_registry',
-], function (router, bindings, serviceProvider) {
+    'content/public/renderer/frame_interfaces',
+], function (bindings, webUITest, interfaceProvider) {
   var browserTarget;
 
   return function() {
-    browserTarget = new bindings.BrowserTarget.proxyClass(
-        new router.Router(
-            serviceProvider.connectToService(bindings.BrowserTarget.name)));
+    browserTarget = new webUITest.BrowserTargetPtr(
+        interfaceProvider.getInterface(webUITest.BrowserTarget.name));
 
     browserTarget.start().then(function() {
       browserTarget.stop();
