@@ -7,6 +7,7 @@
 #include <utility>
 
 #include "content/browser/loader/navigation_url_loader.h"
+#include "content/public/browser/navigation_ui_data.h"
 #include "content/test/test_navigation_url_loader.h"
 
 namespace content {
@@ -23,7 +24,8 @@ std::unique_ptr<NavigationURLLoader>
 TestNavigationURLLoaderFactory::CreateLoader(
     BrowserContext* browser_context,
     std::unique_ptr<NavigationRequestInfo> request_info,
-    ServiceWorkerContextWrapper* service_worker_context_wrapper,
+    std::unique_ptr<NavigationUIData> navigation_ui_data,
+    ServiceWorkerNavigationHandle* service_worker_handle,
     NavigationURLLoaderDelegate* delegate) {
   return std::unique_ptr<NavigationURLLoader>(
       new TestNavigationURLLoader(std::move(request_info), delegate));
