@@ -6,8 +6,8 @@
 
 #include <algorithm>
 #include <iterator>
-#include <utility>
 
+#include "base/bind.h"
 #include "base/logging.h"
 #include "courgette/assembly_program.h"
 #include "courgette/courgette.h"
@@ -43,14 +43,13 @@ RVA DisassemblerElf32::Elf32RvaVisitor_Rel32::Get() const {
   return (*it_)->rva() + (*it_)->relative_target();
 }
 
-DisassemblerElf32::DisassemblerElf32(const void* start, size_t length)
+DisassemblerElf32::DisassemblerElf32(const uint8_t* start, size_t length)
     : Disassembler(start, length),
       header_(nullptr),
       section_header_table_size_(0),
       program_header_table_(nullptr),
       program_header_table_size_(0),
-      default_string_section_(nullptr) {
-}
+      default_string_section_(nullptr) {}
 
 RVA DisassemblerElf32::FileOffsetToRVA(FileOffset offset) const {
   // File offsets can be 64-bit values, but we are dealing with 32-bit
@@ -164,24 +163,23 @@ bool DisassemblerElf32::ParseHeader() {
   return Good();
 }
 
-bool DisassemblerElf32::Disassemble(AssemblyProgram* target) {
+bool DisassemblerElf32::Disassemble(AssemblyProgram* program) {
   if (!ok())
     return false;
 
-  // The Image Base is always 0 for ELF Executables
-  target->set_image_base(0);
-
   if (!ParseAbs32Relocs())
     return false;
 
   if (!ParseRel32RelocsFromSections())  // Does not sort rel32 locations.
     return false;
 
-  PrecomputeLabels(target);
-  RemoveUnusedRel32Locations(target);
+  PrecomputeLabels(program);
+  RemoveUnusedRel32Locations(program);
 
-  if (!ParseFile(target))
+  if (!program->GenerateInstructions(
+          base::Bind(&DisassemblerElf32::ParseFile, base::Unretained(this)))) {
     return false;
+  }
 
   // Finally sort rel32 locations.
   std::sort(rel32_locations_.begin(),
@@ -190,7 +188,7 @@ bool DisassemblerElf32::Disassemble(AssemblyProgram* target) {
   DCHECK(rel32_locations_.empty() ||
          rel32_locations_.back()->rva() != kUnassignedRVA);
 
-  target->DefaultAssignIndexes();
+  program->DefaultAssignIndexes();
   return true;
 }
 
@@ -216,6 +214,32 @@ CheckBool DisassemblerElf32::IsValidTargetRVA(RVA rva) const {
   return false;
 }
 
+// static
+bool DisassemblerElf32::QuickDetect(const uint8_t* start,
+                                    size_t length,
+                                    e_machine_values elf_em) {
+  if (length < sizeof(Elf32_Ehdr))
+    return false;
+
+  const Elf32_Ehdr* header = reinterpret_cast<const Elf32_Ehdr*>(start);
+
+  // Have magic for ELF header?
+  if (header->e_ident[0] != 0x7f || header->e_ident[1] != 'E' ||
+      header->e_ident[2] != 'L' || header->e_ident[3] != 'F')
+    return false;
+
+  if (header->e_type != ET_EXEC && header->e_type != ET_DYN)
+    return false;
+  if (header->e_machine != elf_em)
+    return false;
+  if (header->e_version != 1)
+    return false;
+  if (header->e_shentsize != sizeof(Elf32_Shdr))
+    return false;
+
+  return true;
+}
+
 bool DisassemblerElf32::UpdateLength() {
   Elf32_Off result = 0;
 
@@ -276,7 +300,7 @@ CheckBool DisassemblerElf32::SectionName(const Elf32_Shdr& shdr,
 
 CheckBool DisassemblerElf32::RVAsToFileOffsets(
     const std::vector<RVA>& rvas,
-    std::vector<FileOffset>* file_offsets) {
+    std::vector<FileOffset>* file_offsets) const {
   file_offsets->clear();
   file_offsets->reserve(rvas.size());
   for (RVA rva : rvas) {
@@ -289,7 +313,7 @@ CheckBool DisassemblerElf32::RVAsToFileOffsets(
 }
 
 CheckBool DisassemblerElf32::RVAsToFileOffsets(
-    std::vector<std::unique_ptr<TypedRVA>>* typed_rvas) {
+    std::vector<std::unique_ptr<TypedRVA>>* typed_rvas) const {
   for (auto& typed_rva : *typed_rvas) {
     FileOffset file_offset = RVAToFileOffset(typed_rva->rva());
     if (file_offset == kNoFileOffset)
@@ -325,7 +349,8 @@ void DisassemblerElf32::RemoveUnusedRel32Locations(AssemblyProgram* program) {
   rel32_locations_.resize(std::distance(rel32_locations_.begin(), tail_it));
 }
 
-CheckBool DisassemblerElf32::ParseFile(AssemblyProgram* program) {
+CheckBool DisassemblerElf32::ParseFile(AssemblyProgram* program,
+                                       InstructionReceptor* receptor) const {
   // Walk all the bytes in the file, whether or not in a section.
   FileOffset file_offset = 0;
 
@@ -359,24 +384,21 @@ CheckBool DisassemblerElf32::ParseFile(AssemblyProgram* program) {
     if (section_header->sh_type == SHT_NOBITS)
       continue;
 
-    if (!ParseSimpleRegion(file_offset, section_header->sh_offset, program))
+    if (!ParseSimpleRegion(file_offset, section_header->sh_offset, receptor))
       return false;
 
     file_offset = section_header->sh_offset;
 
     switch (section_header->sh_type) {
       case SHT_REL:
-        if (!ParseRelocationSection(section_header, program))
+        if (!ParseRelocationSection(section_header, receptor))
           return false;
         file_offset = section_header->sh_offset + section_header->sh_size;
         break;
       case SHT_PROGBITS:
-        if (!ParseProgbitsSection(section_header,
-                                  &current_abs_offset,
-                                  end_abs_offset,
-                                  &current_rel,
-                                  end_rel,
-                                  program)) {
+        if (!ParseProgbitsSection(section_header, &current_abs_offset,
+                                  end_abs_offset, &current_rel, end_rel,
+                                  program, receptor)) {
           return false;
         }
         file_offset = section_header->sh_offset + section_header->sh_size;
@@ -407,7 +429,7 @@ CheckBool DisassemblerElf32::ParseFile(AssemblyProgram* program) {
   }
 
   // Rest of the file past the last section
-  if (!ParseSimpleRegion(file_offset, length(), program))
+  if (!ParseSimpleRegion(file_offset, length(), receptor))
     return false;
 
   // Make certain we consume all of the relocations as expected
@@ -420,14 +442,15 @@ CheckBool DisassemblerElf32::ParseProgbitsSection(
     std::vector<FileOffset>::iterator end_abs_offset,
     std::vector<std::unique_ptr<TypedRVA>>::iterator* current_rel,
     std::vector<std::unique_ptr<TypedRVA>>::iterator end_rel,
-    AssemblyProgram* program) {
+    AssemblyProgram* program,
+    InstructionReceptor* receptor) const {
   // Walk all the bytes in the file, whether or not in a section.
   FileOffset file_offset = section_header->sh_offset;
   FileOffset section_end = section_header->sh_offset + section_header->sh_size;
 
   Elf32_Addr origin = section_header->sh_addr;
   FileOffset origin_offset = section_header->sh_offset;
-  if (!program->EmitOriginInstruction(origin))
+  if (!receptor->EmitOrigin(origin))
     return false;
 
   while (file_offset < section_end) {
@@ -454,7 +477,7 @@ CheckBool DisassemblerElf32::ParseProgbitsSection(
       next_relocation = (**current_rel)->file_offset();
 
     if (next_relocation > file_offset) {
-      if (!ParseSimpleRegion(file_offset, next_relocation, program))
+      if (!ParseSimpleRegion(file_offset, next_relocation, receptor))
         return false;
 
       file_offset = next_relocation;
@@ -468,7 +491,7 @@ CheckBool DisassemblerElf32::ParseProgbitsSection(
 
       Label* label = program->FindAbs32Label(target_rva);
       CHECK(label);
-      if (!program->EmitAbs32(label))
+      if (!receptor->EmitAbs32(label))
         return false;
       file_offset += sizeof(RVA);
       ++(*current_abs_offset);
@@ -488,7 +511,7 @@ CheckBool DisassemblerElf32::ParseProgbitsSection(
       Label* label = program->FindRel32Label(target_rva);
       CHECK(label);
 
-      if (!(**current_rel)->EmitInstruction(program, label))
+      if (!(**current_rel)->EmitInstruction(label, receptor))
         return false;
       file_offset += (**current_rel)->op_size();
       ++(*current_rel);
@@ -497,20 +520,21 @@ CheckBool DisassemblerElf32::ParseProgbitsSection(
   }
 
   // Rest of the section (if any)
-  return ParseSimpleRegion(file_offset, section_end, program);
+  return ParseSimpleRegion(file_offset, section_end, receptor);
 }
 
-CheckBool DisassemblerElf32::ParseSimpleRegion(FileOffset start_file_offset,
-                                               FileOffset end_file_offset,
-                                               AssemblyProgram* program) {
+CheckBool DisassemblerElf32::ParseSimpleRegion(
+    FileOffset start_file_offset,
+    FileOffset end_file_offset,
+    InstructionReceptor* receptor) const {
   // Callers don't guarantee start < end
   if (start_file_offset >= end_file_offset)
     return true;
 
   const size_t len = end_file_offset - start_file_offset;
 
-  if (!program->EmitBytesInstruction(FileOffsetToPointer(start_file_offset),
-                                     len)) {
+  if (!receptor->EmitMultipleBytes(FileOffsetToPointer(start_file_offset),
+                                   len)) {
     return false;
   }
 
