@@ -6,11 +6,13 @@
 
 #include <stdio.h>
 
+#include <algorithm>
 #include <memory>
 #include <utility>
 
 #include "base/bind.h"
-#include "base/metrics/histogram.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "dbus/bus.h"
@@ -156,9 +158,15 @@ BluetoothDeviceBlueZ::BluetoothDeviceBlueZ(
       weak_ptr_factory_(this) {
   bluez::BluezDBusManager::Get()->GetBluetoothGattServiceClient()->AddObserver(
       this);
-  bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->AddObserver(this);
 
-  InitializeGattServiceMap();
+  // If GATT Services have already been discovered update the list of Gatt
+  // Services.
+  if (IsGattServicesDiscoveryComplete()) {
+    UpdateGattServices(object_path_);
+  } else {
+    VLOG(2) << "Gatt services have not been fully resolved for device "
+            << object_path_.value();
+  }
 }
 
 BluetoothDeviceBlueZ::~BluetoothDeviceBlueZ() {
@@ -166,8 +174,6 @@ BluetoothDeviceBlueZ::~BluetoothDeviceBlueZ() {
       ->GetBluetoothGattServiceClient()
       ->RemoveObserver(this);
 
-  bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->RemoveObserver(
-      this);
   // Copy the GATT services list here and clear the original so that when we
   // send GattServiceRemoved(), GetGattServices() returns no services.
   GattServiceMap gatt_services_swapped;
@@ -175,7 +181,7 @@ BluetoothDeviceBlueZ::~BluetoothDeviceBlueZ() {
   for (const auto& iter : gatt_services_swapped) {
     DCHECK(adapter());
     adapter()->NotifyGattServiceRemoved(
-        static_cast<BluetoothRemoteGattServiceBlueZ*>(iter.second));
+        static_cast<BluetoothRemoteGattServiceBlueZ*>(iter.second.get()));
   }
 }
 
@@ -210,15 +216,6 @@ device::BluetoothTransport BluetoothDeviceBlueZ::GetType() const {
   return device::BLUETOOTH_TRANSPORT_INVALID;
 }
 
-std::string BluetoothDeviceBlueZ::GetDeviceName() const {
-  bluez::BluetoothDeviceClient::Properties* properties =
-      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
-          object_path_);
-  DCHECK(properties);
-
-  return properties->alias.value();
-}
-
 void BluetoothDeviceBlueZ::CreateGattConnectionImpl() {
   // BlueZ implementation does not use the default CreateGattConnection
   // implementation.
@@ -240,6 +237,28 @@ bool BluetoothDeviceBlueZ::IsGattServicesDiscoveryComplete() const {
 }
 
 void BluetoothDeviceBlueZ::DisconnectGatt() {
+  // There isn't currently a good way to manage the ownership of a connection
+  // between Chrome and bluetoothd plugins/profiles. Until a proper reference
+  // count is kept in bluetoothd, we might unwittingly kill a connection to a
+  // device the user is still interested in, e.g. a mouse. A device's paired
+  // status is usually a good indication that the device is being used by other
+  // parts of the system and therefore we leak these connections.
+  // TODO(crbug.com/630586): Call disconnect for all devices.
+
+  // IsPaired() returns true if we've connected to the device before. So we
+  // check the dbus property directly.
+  // TODO(crbug.com/649651): Use IsPaired once it returns true only for paired
+  // devices.
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+
+  if (properties->paired.value()) {
+    LOG(WARNING) << "Leaking connection to paired device.";
+    return;
+  }
+
   Disconnect(base::Bind(&base::DoNothing), base::Bind(&base::DoNothing));
 }
 
@@ -289,6 +308,18 @@ uint16_t BluetoothDeviceBlueZ::GetAppearance() const {
   return properties->appearance.value();
 }
 
+base::Optional<std::string> BluetoothDeviceBlueZ::GetName() const {
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+
+  if (properties->name.is_valid())
+    return properties->name.value();
+  else
+    return base::nullopt;
+}
+
 bool BluetoothDeviceBlueZ::IsPaired() const {
   bluez::BluetoothDeviceClient::Properties* properties =
       bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
@@ -332,45 +363,50 @@ bool BluetoothDeviceBlueZ::IsConnecting() const {
   return num_connecting_calls_ > 0;
 }
 
-BluetoothDeviceBlueZ::UUIDList BluetoothDeviceBlueZ::GetUUIDs() const {
+BluetoothDevice::UUIDSet BluetoothDeviceBlueZ::GetUUIDs() const {
   bluez::BluetoothDeviceClient::Properties* properties =
       bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
           object_path_);
   DCHECK(properties);
 
-  std::vector<device::BluetoothUUID> uuids;
+  UUIDSet uuids;
   const std::vector<std::string>& dbus_uuids = properties->uuids.value();
-  for (std::vector<std::string>::const_iterator iter = dbus_uuids.begin();
-       iter != dbus_uuids.end(); ++iter) {
-    device::BluetoothUUID uuid(*iter);
+  for (const std::string& dbus_uuid : dbus_uuids) {
+    device::BluetoothUUID uuid(dbus_uuid);
     DCHECK(uuid.IsValid());
-    uuids.push_back(uuid);
+    uuids.insert(std::move(uuid));
   }
   return uuids;
 }
 
-int16_t BluetoothDeviceBlueZ::GetInquiryRSSI() const {
+base::Optional<int8_t> BluetoothDeviceBlueZ::GetInquiryRSSI() const {
   bluez::BluetoothDeviceClient::Properties* properties =
       bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
           object_path_);
   DCHECK(properties);
 
   if (!properties->rssi.is_valid())
-    return kUnknownPower;
+    return base::nullopt;
 
-  return properties->rssi.value();
+  // BlueZ uses int16_t because there is no int8_t for DBus, so we should never
+  // get an int16_t that cannot be represented by an int8_t. But just in case
+  // clamp the value.
+  return ClampPower(properties->rssi.value());
 }
 
-int16_t BluetoothDeviceBlueZ::GetInquiryTxPower() const {
+base::Optional<int8_t> BluetoothDeviceBlueZ::GetInquiryTxPower() const {
   bluez::BluetoothDeviceClient::Properties* properties =
       bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
           object_path_);
   DCHECK(properties);
 
   if (!properties->tx_power.is_valid())
-    return kUnknownPower;
+    return base::nullopt;
 
-  return properties->tx_power.value();
+  // BlueZ uses int16_t because there is no int8_t for DBus, so we should never
+  // get an int16_t that cannot be represented by an int8_t. But just in case
+  // clamp the value.
+  return ClampPower(properties->tx_power.value());
 }
 
 bool BluetoothDeviceBlueZ::ExpectingPinCode() const {
@@ -547,8 +583,8 @@ void BluetoothDeviceBlueZ::CreateGattConnection(
     return;
   }
 
-  // TODO(armansito): Until there is a way to create a reference counted GATT
-  // connection in bluetoothd, simply do a regular connect.
+  // TODO(crbug.com/630586): Until there is a way to create a reference counted
+  // GATT connection in bluetoothd, simply do a regular connect.
   Connect(NULL, base::Bind(&BluetoothDeviceBlueZ::OnCreateGattConnection,
                            weak_ptr_factory_.GetWeakPtr(), callback),
           error_callback);
@@ -563,6 +599,46 @@ void BluetoothDeviceBlueZ::GetServiceRecords(
                  weak_ptr_factory_.GetWeakPtr(), error_callback));
 }
 
+void BluetoothDeviceBlueZ::UpdateServiceData() {
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+  DCHECK(properties->service_data.is_valid());
+
+  service_data_.clear();
+  for (const auto& pair : properties->service_data.value())
+    service_data_[BluetoothUUID(pair.first)] = pair.second;
+}
+
+void BluetoothDeviceBlueZ::UpdateManufacturerData() {
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+  manufacturer_data_.clear();
+
+  if (properties->manufacturer_data.is_valid()) {
+    for (const auto& pair : properties->manufacturer_data.value())
+      manufacturer_data_[pair.first] = pair.second;
+  }
+}
+
+void BluetoothDeviceBlueZ::UpdateAdvertisingDataFlags() {
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+  advertising_data_flags_ = base::nullopt;
+
+  // The advertising data flags property is a vector<uint8> because the
+  // Supplement to Bluetooth Core Specification Version 6 page 13 said that
+  // "The Flags field may be zero or more octets long." However, only the first
+  // byte of that is needed because there is only 5 bits of data defined there.
+  if (properties->advertising_data_flags.is_valid())
+    advertising_data_flags_ = properties->advertising_data_flags.value()[0];
+}
+
 BluetoothPairingBlueZ* BluetoothDeviceBlueZ::BeginPairing(
     BluetoothDevice::PairingDelegate* pairing_delegate) {
   pairing_.reset(new BluetoothPairingBlueZ(this, pairing_delegate));
@@ -581,30 +657,6 @@ BluetoothAdapterBlueZ* BluetoothDeviceBlueZ::adapter() const {
   return static_cast<BluetoothAdapterBlueZ*>(adapter_);
 }
 
-void BluetoothDeviceBlueZ::DevicePropertyChanged(
-    const dbus::ObjectPath& object_path,
-    const std::string& property_name) {
-  if (object_path != object_path_)
-    return;
-
-  bluez::BluetoothDeviceClient::Properties* properties =
-      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
-          object_path);
-  DCHECK(properties);
-
-  if (property_name == properties->services_resolved.name() &&
-      properties->services_resolved.value()) {
-    VLOG(3) << "All services were discovered for device: "
-            << object_path.value();
-
-    for (const auto iter : newly_discovered_gatt_services_) {
-      adapter()->NotifyGattDiscoveryComplete(
-          static_cast<BluetoothRemoteGattService*>(iter));
-    }
-    newly_discovered_gatt_services_.clear();
-  }
-}
-
 void BluetoothDeviceBlueZ::GattServiceAdded(
     const dbus::ObjectPath& object_path) {
   if (GetGattService(object_path.value())) {
@@ -627,11 +679,7 @@ void BluetoothDeviceBlueZ::GattServiceAdded(
   BluetoothRemoteGattServiceBlueZ* service =
       new BluetoothRemoteGattServiceBlueZ(adapter(), this, object_path);
 
-  newly_discovered_gatt_services_.push_back(
-      static_cast<BluetoothRemoteGattServiceBlueZ*>(service));
-
-  gatt_services_.set(service->GetIdentifier(),
-                     std::unique_ptr<BluetoothRemoteGattService>(service));
+  gatt_services_[service->GetIdentifier()] = base::WrapUnique(service);
   DCHECK(service->object_path() == object_path);
   DCHECK(service->GetUUID().IsValid());
 
@@ -641,15 +689,14 @@ void BluetoothDeviceBlueZ::GattServiceAdded(
 
 void BluetoothDeviceBlueZ::GattServiceRemoved(
     const dbus::ObjectPath& object_path) {
-  GattServiceMap::const_iterator iter =
-      gatt_services_.find(object_path.value());
+  auto iter = gatt_services_.find(object_path.value());
   if (iter == gatt_services_.end()) {
     VLOG(3) << "Unknown GATT service removed: " << object_path.value();
     return;
   }
 
   BluetoothRemoteGattServiceBlueZ* service =
-      static_cast<BluetoothRemoteGattServiceBlueZ*>(iter->second);
+      static_cast<BluetoothRemoteGattServiceBlueZ*>(iter->second.get());
 
   VLOG(1) << "Removing remote GATT service with UUID: '"
           << service->GetUUID().canonical_value()
@@ -657,37 +704,47 @@ void BluetoothDeviceBlueZ::GattServiceRemoved(
 
   DCHECK(service->object_path() == object_path);
   std::unique_ptr<BluetoothRemoteGattService> scoped_service =
-      gatt_services_.take_and_erase(iter->first);
+      std::move(gatt_services_[object_path.value()]);
+  gatt_services_.erase(iter);
 
   DCHECK(adapter());
+  discovery_complete_notified_.erase(service);
   adapter()->NotifyGattServiceRemoved(service);
 }
 
-void BluetoothDeviceBlueZ::InitializeGattServiceMap() {
-  DCHECK(gatt_services_.empty());
-
-  if (!IsGattServicesDiscoveryComplete()) {
-    VLOG(2) << "Gatt services have not been fully resolved for device "
-            << object_path_.value();
+void BluetoothDeviceBlueZ::UpdateGattServices(
+    const dbus::ObjectPath& object_path) {
+  if (object_path != object_path_) {
+    // No need to update map if update is for a different device.
     return;
   }
 
-  VLOG(3) << "Initializing the list of GATT services associated with device "
+  DCHECK(IsGattServicesDiscoveryComplete());
+
+  VLOG(3) << "Updating the list of GATT services associated with device "
           << object_path_.value();
 
-  // Add all known GATT services associated with the device.
-  const std::vector<dbus::ObjectPath> gatt_services =
+  const std::vector<dbus::ObjectPath> service_paths =
       bluez::BluezDBusManager::Get()
           ->GetBluetoothGattServiceClient()
           ->GetServices();
-  for (const auto& it : gatt_services)
-    GattServiceAdded(it);
+  for (const auto& service_path : service_paths) {
+    // Add all previously unknown GATT services associated with the device.
+    GattServiceAdded(service_path);
 
-  // Notify on the discovery complete for each service which is found in the
-  // first discovery.
-  DCHECK(adapter());
-  for (const auto& iter : gatt_services_)
-    adapter()->NotifyGattDiscoveryComplete(iter.second);
+    // If the service does not belong in this device, there is nothing left to
+    // do.
+    BluetoothRemoteGattService* service = GetGattService(service_path.value());
+    if (service == nullptr) {
+      return;
+    }
+
+    // Notify of GATT discovery complete if we haven't before.
+    auto notified_pair = discovery_complete_notified_.insert(service);
+    if (notified_pair.second) {
+      adapter()->NotifyGattDiscoveryComplete(service);
+    }
+  }
 }
 
 void BluetoothDeviceBlueZ::OnGetConnInfo(const ConnectionInfoCallback& callback,
@@ -871,6 +928,9 @@ void BluetoothDeviceBlueZ::OnSetTrusted(bool success) {
 
 void BluetoothDeviceBlueZ::OnDisconnect(const base::Closure& callback) {
   VLOG(1) << object_path_.value() << ": Disconnected";
+  // Do not notify about changed device since this is already done by
+  // the dbus::PropertySet and the property change callback for BlueZ.
+  DidDisconnectGatt(false /* notifyDeviceChanged */);
   callback.Run();
 }
 
