@@ -225,7 +225,7 @@ void CastChannelAsyncApiFunction::SetResultFromError(int channel_id,
   channel_info.error_state = error;
   channel_info.connect_info.ip_address = "";
   channel_info.connect_info.port = 0;
-  channel_info.connect_info.auth = cast_channel::CHANNEL_AUTH_TYPE_SSL;
+  channel_info.connect_info.auth = cast_channel::CHANNEL_AUTH_TYPE_SSL_VERIFIED;
   SetResultFromChannelInfo(channel_info);
   SetError("Channel error = " + base::IntToString(error));
 }
@@ -271,12 +271,12 @@ bool CastChannelOpenFunction::Prepare() {
   } else {
     // Parse timeout parameters if they are set.
     if (connect_info.liveness_timeout.get()) {
-      liveness_timeout_ = base::TimeDelta::FromMilliseconds(
-          *connect_info.liveness_timeout.get());
+      liveness_timeout_ =
+          base::TimeDelta::FromMilliseconds(*connect_info.liveness_timeout);
     }
     if (connect_info.ping_interval.get()) {
       ping_interval_ =
-          base::TimeDelta::FromMilliseconds(*connect_info.ping_interval.get());
+          base::TimeDelta::FromMilliseconds(*connect_info.ping_interval);
     }
 
     // Validate timeout parameters.
@@ -313,10 +313,10 @@ void CastChannelOpenFunction::AsyncWorkStart() {
         extension_->id(), *ip_endpoint_, channel_auth_,
         ExtensionsBrowserClient::Get()->GetNetLog(),
         base::TimeDelta::FromMilliseconds(connect_info.timeout.get()
-                                              ? *connect_info.timeout.get()
+                                              ? *connect_info.timeout
                                               : kDefaultConnectTimeoutMillis),
         liveness_timeout_ > base::TimeDelta(), api_->GetLogger(),
-        connect_info.capabilities.get() ? *connect_info.capabilities.get()
+        connect_info.capabilities.get() ? *connect_info.capabilities
                                         : CastDeviceCapability::NONE);
   }
   new_channel_id_ = AddSocket(socket);
@@ -324,9 +324,9 @@ void CastChannelOpenFunction::AsyncWorkStart() {
 
   // Construct read delegates.
   std::unique_ptr<api::cast_channel::CastTransport::Delegate> delegate(
-      base::WrapUnique(new CastMessageHandler(
+      base::MakeUnique<CastMessageHandler>(
           base::Bind(&CastChannelAPI::SendEvent, api_->AsWeakPtr()), socket,
-          api_->GetLogger())));
+          api_->GetLogger()));
   if (socket->keep_alive()) {
     // Wrap read delegate in a KeepAliveDelegate for timeout handling.
     api::cast_channel::KeepAliveDelegate* keep_alive =
@@ -336,9 +336,8 @@ void CastChannelOpenFunction::AsyncWorkStart() {
     std::unique_ptr<base::Timer> injected_timer =
         api_->GetInjectedTimeoutTimerForTest();
     if (injected_timer) {
-      keep_alive->SetTimersForTest(
-          base::WrapUnique(new base::Timer(false, false)),
-          std::move(injected_timer));
+      keep_alive->SetTimersForTest(base::MakeUnique<base::Timer>(false, false),
+                                   std::move(injected_timer));
     }
     delegate.reset(keep_alive);
   }
@@ -380,8 +379,8 @@ bool CastChannelSendFunction::Prepare() {
     return false;
   }
   switch (params_->message.data->GetType()) {
-    case base::Value::TYPE_STRING:
-    case base::Value::TYPE_BINARY:
+    case base::Value::Type::STRING:
+    case base::Value::Type::BINARY:
       break;
     default:
       SetError("Invalid type of message_info.data");
