@@ -10,11 +10,14 @@
 #include "base/callback.h"
 #include "base/json/json_writer.h"
 #include "base/location.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/sparse_histogram.h"
 #include "base/sequenced_task_runner.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_piece.h"
+#include "base/strings/string_split.h"
+#include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/task_runner_util.h"
 #include "base/values.h"
@@ -90,7 +93,8 @@ void AttachProperties(const Properties& properties,
 
   base::ListValue* const properties_value = new base::ListValue;
   for (const auto& property : properties) {
-    base::DictionaryValue* const property_value = new base::DictionaryValue;
+    std::unique_ptr<base::DictionaryValue> property_value(
+        new base::DictionaryValue);
     std::string visibility_as_string;
     switch (property.visibility()) {
       case Property::VISIBILITY_PRIVATE:
@@ -103,7 +107,7 @@ void AttachProperties(const Properties& properties,
     property_value->SetString("visibility", visibility_as_string);
     property_value->SetString("key", property.key());
     property_value->SetString("value", property.value());
-    properties_value->Append(property_value);
+    properties_value->Append(std::move(property_value));
   }
   request_body->Set("properties", properties_value);
 }
@@ -144,37 +148,6 @@ std::string CreateMultipartUploadMetadataJson(
   return json_string;
 }
 
-// Splits |string| into lines by |kHttpBr|.
-// Each line does not include |kHttpBr|.
-void SplitIntoLines(const std::string& string,
-                    std::vector<base::StringPiece>* output) {
-  const size_t br_size = std::string(kHttpBr).size();
-  std::string::const_iterator it = string.begin();
-  std::vector<base::StringPiece> lines;
-  while (true) {
-    const std::string::const_iterator next_pos =
-        std::search(it, string.end(), kHttpBr, kHttpBr + br_size);
-    lines.push_back(base::StringPiece(it, next_pos));
-    if (next_pos == string.end())
-      break;
-    it = next_pos + br_size;
-  }
-  output->swap(lines);
-}
-
-// Remove transport padding (spaces and tabs at the end of line) from |piece|.
-base::StringPiece TrimTransportPadding(const base::StringPiece& piece) {
-  size_t trim_size = 0;
-  while (trim_size < piece.size() &&
-         (piece[piece.size() - 1 - trim_size] == ' ' ||
-          piece[piece.size() - 1 - trim_size] == '\t')) {
-    ++trim_size;
-  }
-  return piece.substr(0, piece.size() - trim_size);
-}
-
-void EmptyClosure(std::unique_ptr<BatchableDelegate>) {}
-
 }  // namespace
 
 MultipartHttpResponse::MultipartHttpResponse() : code(HTTP_SUCCESS) {
@@ -219,10 +192,9 @@ bool ParseMultipartResponse(const std::string& content_type,
   if (content_type_piece.empty())
     return false;
   if (content_type_piece[0] == '"') {
-    if (content_type_piece.size() <= 2 ||
-        content_type_piece[content_type_piece.size() - 1] != '"') {
+    if (content_type_piece.size() <= 2 || content_type_piece.back() != '"')
       return false;
-    }
+
     content_type_piece =
         content_type_piece.substr(1, content_type_piece.size() - 2);
   }
@@ -232,8 +204,8 @@ bool ParseMultipartResponse(const std::string& content_type,
   const std::string header = "--" + boundary;
   const std::string terminator = "--" + boundary + "--";
 
-  std::vector<base::StringPiece> lines;
-  SplitIntoLines(response, &lines);
+  std::vector<base::StringPiece> lines = base::SplitStringPieceUsingSubstr(
+      response, kHttpBr, base::KEEP_WHITESPACE, base::SPLIT_WANT_ALL);
 
   enum {
     STATE_START,
@@ -275,7 +247,8 @@ bool ParseMultipartResponse(const std::string& content_type,
       body.clear();
       continue;
     }
-    const base::StringPiece chopped_line = TrimTransportPadding(line);
+    const base::StringPiece chopped_line =
+        base::TrimString(line, " \t", base::TRIM_TRAILING);
     const bool is_new_part = chopped_line == header;
     const bool was_last_part = chopped_line == terminator;
     if (is_new_part || was_last_part) {
@@ -1169,11 +1142,10 @@ bool PermissionsInsertRequest::GetContentData(std::string* upload_content_type,
 
 SingleBatchableDelegateRequest::SingleBatchableDelegateRequest(
     RequestSender* sender,
-    BatchableDelegate* delegate)
+    std::unique_ptr<BatchableDelegate> delegate)
     : UrlFetchRequestBase(sender),
-      delegate_(delegate),
-      weak_ptr_factory_(this) {
-}
+      delegate_(std::move(delegate)),
+      weak_ptr_factory_(this) {}
 
 SingleBatchableDelegateRequest::~SingleBatchableDelegateRequest() {
 }
@@ -1255,7 +1227,7 @@ void BatchUploadRequest::AddRequest(BatchableDelegate* request) {
   DCHECK(request);
   DCHECK(GetChildEntry(request) == child_requests_.end());
   DCHECK(!committed_);
-  child_requests_.push_back(new BatchUploadChildEntry(request));
+  child_requests_.push_back(base::MakeUnique<BatchUploadChildEntry>(request));
   request->Prepare(base::Bind(&BatchUploadRequest::OnChildRequestPrepared,
                               weak_ptr_factory_.GetWeakPtr(), request));
 }
@@ -1299,8 +1271,8 @@ void BatchUploadRequest::Cancel() {
 
 // Obtains corresponding child entry of |request_id|. Returns NULL if the
 // entry is not found.
-ScopedVector<BatchUploadChildEntry>::iterator BatchUploadRequest::GetChildEntry(
-    RequestID request_id) {
+std::vector<std::unique_ptr<BatchUploadChildEntry>>::iterator
+BatchUploadRequest::GetChildEntry(RequestID request_id) {
   for (auto it = child_requests_.begin(); it != child_requests_.end(); ++it) {
     if ((*it)->request.get() == request_id)
       return it;
@@ -1319,7 +1291,7 @@ void BatchUploadRequest::MayCompletePrepare() {
   // Build multipart body here.
   int64_t total_size = 0;
   std::vector<ContentTypeAndData> parts;
-  for (auto& child : child_requests_) {
+  for (const auto& child : child_requests_) {
     std::string type;
     std::string data;
     const bool result = child->request->GetContentData(&type, &data);
@@ -1347,10 +1319,7 @@ void BatchUploadRequest::MayCompletePrepare() {
     child->data_size = data.size();
     total_size += data.size();
 
-    parts.push_back(ContentTypeAndData());
-    parts.back().type = kHttpContentType;
-    parts.back().data = header;
-    parts.back().data.append(data);
+    parts.push_back(ContentTypeAndData({kHttpContentType, header + data}));
   }
 
   UMA_HISTOGRAM_COUNTS_100(kUMADriveTotalFileCountInBatchUpload, parts.size());
@@ -1428,12 +1397,12 @@ void BatchUploadRequest::ProcessURLFetchResults(const net::URLFetcher* source) {
   }
 
   for (size_t i = 0; i < parts.size(); ++i) {
-    BatchableDelegate* const delegate = child_requests_[i]->request.get();
-    // Pass onwership of |delegate| so that child_requests_.clear() won't
+    BatchableDelegate* delegate = child_requests_[i]->request.get();
+    // Pass ownership of |delegate| so that child_requests_.clear() won't
     // kill the delegate. It has to be deleted after the notification.
-    delegate->NotifyResult(
-        parts[i].code, parts[i].body,
-        base::Bind(&EmptyClosure, base::Passed(&child_requests_[i]->request)));
+    delegate->NotifyResult(parts[i].code, parts[i].body,
+                           base::Bind(&base::DeletePointer<BatchableDelegate>,
+                                      child_requests_[i]->request.release()));
   }
   child_requests_.clear();
 
@@ -1441,16 +1410,15 @@ void BatchUploadRequest::ProcessURLFetchResults(const net::URLFetcher* source) {
 }
 
 void BatchUploadRequest::RunCallbackOnPrematureFailure(DriveApiErrorCode code) {
-  for (auto child : child_requests_) {
+  for (const auto& child : child_requests_)
     child->request->NotifyError(code);
-  }
   child_requests_.clear();
 }
 
 void BatchUploadRequest::OnURLFetchUploadProgress(const net::URLFetcher* source,
                                                   int64_t current,
                                                   int64_t total) {
-  for (auto child : child_requests_) {
+  for (const auto& child : child_requests_) {
     if (child->data_offset <= current &&
         current <= child->data_offset + child->data_size) {
       child->request->NotifyUploadProgress(source, current - child->data_offset,
