@@ -10,10 +10,6 @@
 #include "build/build_config.h"
 #include "mojo/edk/embedder/embedder.h"
 
-#if defined(OS_ANDROID)
-#include "gpu/ipc/client/android/in_process_surface_texture_manager.h"
-#endif
-
 namespace {
 
 class GpuTestSuite : public base::TestSuite {
@@ -21,9 +17,6 @@ class GpuTestSuite : public base::TestSuite {
   GpuTestSuite(int argc, char** argv);
   ~GpuTestSuite() override;
 
- protected:
-  void Initialize() override;
-
  private:
   DISALLOW_COPY_AND_ASSIGN(GpuTestSuite);
 };
@@ -33,14 +26,6 @@ GpuTestSuite::GpuTestSuite(int argc, char** argv)
 
 GpuTestSuite::~GpuTestSuite() {}
 
-void GpuTestSuite::Initialize() {
-  base::TestSuite::Initialize();
-#if defined(OS_ANDROID)
-  gpu::SurfaceTextureManager::SetInstance(
-      gpu::InProcessSurfaceTextureManager::GetInstance());
-#endif
-}
-
 }  // namespace
 
 int main(int argc, char** argv) {
