@@ -95,7 +95,7 @@ class FeatureInfoTest
       const char* extensions, const char* renderer, const char* version) {
     GpuServiceTest::SetUpWithGLVersion(version, extensions);
     TestHelper::SetupFeatureInfoInitExpectationsWithGLVersion(
-        gl_.get(), extensions, renderer, version);
+        gl_.get(), extensions, renderer, version, GetContextType());
     info_ = new FeatureInfo();
     info_->Initialize(GetContextType(), DisallowedFeatures());
   }
@@ -107,7 +107,7 @@ class FeatureInfoTest
       const DisallowedFeatures& disallowed_features) {
     GpuServiceTest::SetUpWithGLVersion(version, extensions);
     TestHelper::SetupFeatureInfoInitExpectationsWithGLVersion(
-        gl_.get(), extensions, renderer, version);
+        gl_.get(), extensions, renderer, version, GetContextType());
     info_ = new FeatureInfo();
     info_->Initialize(GetContextType(), disallowed_features);
   }
@@ -119,7 +119,7 @@ class FeatureInfoTest
       const base::CommandLine& command_line) {
     GpuServiceTest::SetUpWithGLVersion(version, extensions);
     TestHelper::SetupFeatureInfoInitExpectationsWithGLVersion(
-        gl_.get(), extensions, renderer, version);
+        gl_.get(), extensions, renderer, version, GetContextType());
     GpuDriverBugWorkarounds gpu_driver_bug_workaround(&command_line);
     info_ = new FeatureInfo(command_line, gpu_driver_bug_workaround);
     info_->Initialize(GetContextType(), DisallowedFeatures());
@@ -136,7 +136,7 @@ class FeatureInfoTest
       const base::CommandLine& command_line) {
     GpuServiceTest::SetUpWithGLVersion("2.0", extensions);
     TestHelper::SetupFeatureInfoInitExpectationsWithGLVersion(
-        gl_.get(), extensions, "", "");
+        gl_.get(), extensions, "", "", GetContextType());
     GpuDriverBugWorkarounds gpu_driver_bug_workaround(&command_line);
     info_ = new FeatureInfo(command_line, gpu_driver_bug_workaround);
     info_->Initialize(GetContextType(), DisallowedFeatures());
@@ -246,8 +246,11 @@ TEST_P(FeatureInfoTest, InitializeNoExtensions) {
       NOTREACHED();
       break;
   }
+  // Note that because GL_EXT_sRGB is a substring of GL_EXT_sRGB_write_control,
+  // which is not part of the ES3 core, we have to be careful to search for
+  // "GL_EXT_sRGB ", and append a space to the end of the extension string.
   if (expect_ext_srgb) {
-    EXPECT_THAT(info_->extensions(), HasSubstr("GL_EXT_sRGB"));
+    EXPECT_THAT(info_->extensions() + " ", HasSubstr("GL_EXT_sRGB "));
     EXPECT_TRUE(info_->validators()->texture_format.IsValid(GL_SRGB_EXT));
     EXPECT_TRUE(info_->validators()->texture_format.IsValid(GL_SRGB_ALPHA_EXT));
     EXPECT_TRUE(info_->validators()->texture_internal_format.IsValid(
@@ -256,10 +259,10 @@ TEST_P(FeatureInfoTest, InitializeNoExtensions) {
         GL_SRGB_ALPHA_EXT));
     EXPECT_TRUE(info_->validators()->render_buffer_format.IsValid(
         GL_SRGB8_ALPHA8_EXT));
-    EXPECT_TRUE(info_->validators()->frame_buffer_parameter.IsValid(
+    EXPECT_TRUE(info_->validators()->framebuffer_parameter.IsValid(
         GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT));
   } else {
-    EXPECT_THAT(info_->extensions(), Not(HasSubstr("GL_EXT_sRGB")));
+    EXPECT_THAT(info_->extensions() + " ", Not(HasSubstr("GL_EXT_sRGB ")));
     EXPECT_FALSE(info_->validators()->texture_format.IsValid(GL_SRGB_EXT));
     EXPECT_FALSE(info_->validators()->texture_format.IsValid(
         GL_SRGB_ALPHA_EXT));
@@ -269,7 +272,7 @@ TEST_P(FeatureInfoTest, InitializeNoExtensions) {
         GL_SRGB_ALPHA_EXT));
     EXPECT_FALSE(info_->validators()->render_buffer_format.IsValid(
         GL_SRGB8_ALPHA8_EXT));
-    EXPECT_FALSE(info_->validators()->frame_buffer_parameter.IsValid(
+    EXPECT_FALSE(info_->validators()->framebuffer_parameter.IsValid(
         GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT));
   }
 
@@ -291,6 +294,8 @@ TEST_P(FeatureInfoTest, InitializeNoExtensions) {
               Not(HasSubstr("GL_AMD_compressed_ATC_texture")));
   EXPECT_THAT(info_->extensions(),
               Not(HasSubstr("GL_IMG_texture_compression_pvrtc")));
+  EXPECT_THAT(info_->extensions(),
+              Not(HasSubstr("GL_EXT_texture_compression_s3tc_srgb")));
   EXPECT_FALSE(info_->feature_flags().npot_ok);
   EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
       GL_COMPRESSED_RGB_S3TC_DXT1_EXT));
@@ -300,6 +305,14 @@ TEST_P(FeatureInfoTest, InitializeNoExtensions) {
       GL_COMPRESSED_RGBA_S3TC_DXT3_EXT));
   EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
       GL_COMPRESSED_RGBA_S3TC_DXT5_EXT));
+  EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_S3TC_DXT1_EXT));
+  EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT));
+  EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT));
+  EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT));
   EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
       GL_ETC1_RGB8_OES));
   EXPECT_FALSE(info_->validators()->compressed_texture_format.IsValid(
@@ -320,9 +333,9 @@ TEST_P(FeatureInfoTest, InitializeNoExtensions) {
       GL_TEXTURE_MAX_ANISOTROPY_EXT));
   EXPECT_FALSE(info_->validators()->g_l_state.IsValid(
       GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT));
-  EXPECT_FALSE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_FALSE(info_->validators()->framebuffer_target.IsValid(
       GL_READ_FRAMEBUFFER_EXT));
-  EXPECT_FALSE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_FALSE(info_->validators()->framebuffer_target.IsValid(
       GL_DRAW_FRAMEBUFFER_EXT));
   EXPECT_FALSE(info_->validators()->g_l_state.IsValid(
       GL_READ_FRAMEBUFFER_BINDING_EXT));
@@ -410,6 +423,83 @@ TEST_P(FeatureInfoTest, InitializeDXTExtensionGL) {
       GL_COMPRESSED_RGBA_S3TC_DXT5_EXT));
 }
 
+TEST_P(FeatureInfoTest, InitializeEXT_texture_compression_s3tc_srgb) {
+  SetupInitExpectationsWithGLVersion("GL_NV_sRGB_formats", "",
+                                     "OpenGL ES 2.0");
+  EXPECT_THAT(info_->extensions(),
+              HasSubstr("GL_EXT_texture_compression_s3tc_srgb"));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_S3TC_DXT1_EXT));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT));
+}
+
+TEST_P(FeatureInfoTest, InitializeEXT_texture_compression_s3tc_srgbGL) {
+  SetupInitExpectations("GL_EXT_texture_sRGB GL_EXT_texture_compression_s3tc");
+  EXPECT_THAT(info_->extensions(),
+              HasSubstr("GL_EXT_texture_compression_s3tc_srgb"));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_S3TC_DXT1_EXT));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT));
+}
+
+TEST_P(FeatureInfoTest, InitializeCHROMIUM_compressed_texture_etc) {
+  SetupInitExpectationsWithGLVersion("", "",
+                                     "OpenGL ES 3.0");
+  EXPECT_THAT(info_->extensions(),
+              HasSubstr("GL_CHROMIUM_compressed_texture_etc"));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_R11_EAC));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SIGNED_R11_EAC));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_RGB8_ETC2));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB8_ETC2));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_RG11_EAC));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SIGNED_RG11_EAC));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_RGBA8_ETC2_EAC));
+  EXPECT_TRUE(info_->validators()->compressed_texture_format.IsValid(
+      GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC));
+
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_R11_EAC));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_SIGNED_R11_EAC));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_RGB8_ETC2));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_SRGB8_ETC2));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_RG11_EAC));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_SIGNED_RG11_EAC));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_RGBA8_ETC2_EAC));
+  EXPECT_TRUE(info_->validators()->texture_internal_format_storage.IsValid(
+      GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC));
+}
+
 TEST_P(FeatureInfoTest, InitializeEXT_texture_format_BGRA8888GLES2) {
   SetupInitExpectationsWithGLVersion("GL_EXT_texture_format_BGRA8888", "",
                                      "OpenGL ES 2.0");
@@ -506,7 +596,7 @@ TEST_P(FeatureInfoTest, InitializeEXT_sRGB) {
   SetupInitExpectations("GL_EXT_sRGB GL_OES_rgb8_rgba8");
 
   if (GetContextType() == CONTEXT_TYPE_OPENGLES3) {
-    EXPECT_THAT(info_->extensions(), Not(HasSubstr("GL_EXT_sRGB")));
+    EXPECT_THAT(info_->extensions() + " ", Not(HasSubstr("GL_EXT_sRGB ")));
     EXPECT_FALSE(info_->validators()->texture_format.IsValid(GL_SRGB_EXT));
     EXPECT_FALSE(
         info_->validators()->texture_format.IsValid(GL_SRGB_ALPHA_EXT));
@@ -516,10 +606,10 @@ TEST_P(FeatureInfoTest, InitializeEXT_sRGB) {
         GL_SRGB_ALPHA_EXT));
     EXPECT_FALSE(
         info_->validators()->render_buffer_format.IsValid(GL_SRGB8_ALPHA8_EXT));
-    EXPECT_FALSE(info_->validators()->frame_buffer_parameter.IsValid(
+    EXPECT_FALSE(info_->validators()->framebuffer_parameter.IsValid(
         GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT));
   } else {
-    EXPECT_THAT(info_->extensions(), HasSubstr("GL_EXT_sRGB"));
+    EXPECT_THAT(info_->extensions() + " ", HasSubstr("GL_EXT_sRGB "));
     EXPECT_TRUE(info_->validators()->texture_format.IsValid(GL_SRGB_EXT));
     EXPECT_TRUE(info_->validators()->texture_format.IsValid(GL_SRGB_ALPHA_EXT));
     EXPECT_TRUE(
@@ -528,7 +618,7 @@ TEST_P(FeatureInfoTest, InitializeEXT_sRGB) {
         GL_SRGB_ALPHA_EXT));
     EXPECT_TRUE(
         info_->validators()->render_buffer_format.IsValid(GL_SRGB8_ALPHA8_EXT));
-    EXPECT_TRUE(info_->validators()->frame_buffer_parameter.IsValid(
+    EXPECT_TRUE(info_->validators()->framebuffer_parameter.IsValid(
         GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT));
   }
 }
@@ -835,7 +925,7 @@ TEST_P(FeatureInfoTest, Initialize_texture_floatGLES3) {
 
 TEST_P(FeatureInfoTest, Initialize_sRGBGLES3) {
   SetupInitExpectationsWithGLVersion("", "", "OpenGL ES 3.0");
-  EXPECT_THAT(info_->extensions(), Not(HasSubstr("GL_EXT_sRGB")));
+  EXPECT_THAT(info_->extensions() + " ", Not(HasSubstr("GL_EXT_sRGB ")));
   EXPECT_FALSE(info_->validators()->texture_format.IsValid(
       GL_SRGB_EXT));
   EXPECT_FALSE(info_->validators()->texture_format.IsValid(
@@ -846,7 +936,7 @@ TEST_P(FeatureInfoTest, Initialize_sRGBGLES3) {
       GL_SRGB_ALPHA_EXT));
   EXPECT_FALSE(info_->validators()->render_buffer_format.IsValid(
       GL_SRGB8_ALPHA8_EXT));
-  EXPECT_FALSE(info_->validators()->frame_buffer_parameter.IsValid(
+  EXPECT_FALSE(info_->validators()->framebuffer_parameter.IsValid(
       GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT));
 }
 
@@ -911,9 +1001,9 @@ TEST_P(FeatureInfoTest, InitializeEXT_framebuffer_multisample) {
   EXPECT_TRUE(info_->feature_flags().chromium_framebuffer_multisample);
   EXPECT_THAT(info_->extensions(),
               HasSubstr("GL_CHROMIUM_framebuffer_multisample"));
-  EXPECT_TRUE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_TRUE(info_->validators()->framebuffer_target.IsValid(
       GL_READ_FRAMEBUFFER_EXT));
-  EXPECT_TRUE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_TRUE(info_->validators()->framebuffer_target.IsValid(
       GL_DRAW_FRAMEBUFFER_EXT));
   EXPECT_TRUE(info_->validators()->g_l_state.IsValid(
       GL_READ_FRAMEBUFFER_BINDING_EXT));
@@ -929,9 +1019,9 @@ TEST_P(FeatureInfoTest, InitializeANGLE_framebuffer_multisample) {
   EXPECT_TRUE(info_->feature_flags().chromium_framebuffer_multisample);
   EXPECT_THAT(info_->extensions(),
               HasSubstr("GL_CHROMIUM_framebuffer_multisample"));
-  EXPECT_TRUE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_TRUE(info_->validators()->framebuffer_target.IsValid(
       GL_READ_FRAMEBUFFER_EXT));
-  EXPECT_TRUE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_TRUE(info_->validators()->framebuffer_target.IsValid(
       GL_DRAW_FRAMEBUFFER_EXT));
   EXPECT_TRUE(info_->validators()->g_l_state.IsValid(
       GL_READ_FRAMEBUFFER_BINDING_EXT));
@@ -949,9 +1039,9 @@ TEST_P(FeatureInfoTest, InitializeANGLE_framebuffer_multisampleWithoutANGLE) {
   EXPECT_FALSE(info_->feature_flags().chromium_framebuffer_multisample);
   EXPECT_THAT(info_->extensions(),
               Not(HasSubstr("GL_CHROMIUM_framebuffer_multisample")));
-  EXPECT_FALSE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_FALSE(info_->validators()->framebuffer_target.IsValid(
       GL_READ_FRAMEBUFFER_EXT));
-  EXPECT_FALSE(info_->validators()->frame_buffer_target.IsValid(
+  EXPECT_FALSE(info_->validators()->framebuffer_target.IsValid(
       GL_DRAW_FRAMEBUFFER_EXT));
   EXPECT_FALSE(info_->validators()->g_l_state.IsValid(
       GL_READ_FRAMEBUFFER_BINDING_EXT));
@@ -973,7 +1063,7 @@ TEST_P(FeatureInfoTest, InitializeEXT_multisampled_render_to_texture) {
       GL_MAX_SAMPLES_EXT));
   EXPECT_TRUE(info_->validators()->render_buffer_parameter.IsValid(
       GL_RENDERBUFFER_SAMPLES_EXT));
-  EXPECT_TRUE(info_->validators()->frame_buffer_parameter.IsValid(
+  EXPECT_TRUE(info_->validators()->framebuffer_parameter.IsValid(
       GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SAMPLES_EXT));
 }
 
@@ -989,7 +1079,7 @@ TEST_P(FeatureInfoTest, InitializeIMG_multisampled_render_to_texture) {
       GL_MAX_SAMPLES_EXT));
   EXPECT_TRUE(info_->validators()->render_buffer_parameter.IsValid(
       GL_RENDERBUFFER_SAMPLES_EXT));
-  EXPECT_TRUE(info_->validators()->frame_buffer_parameter.IsValid(
+  EXPECT_TRUE(info_->validators()->framebuffer_parameter.IsValid(
       GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SAMPLES_EXT));
 }
 
@@ -1191,8 +1281,10 @@ TEST_P(FeatureInfoTest, InitializeIMG_texture_compression_pvrtc) {
 
 TEST_P(FeatureInfoTest, InitializeEXT_occlusion_query_boolean) {
   SetupInitExpectations("GL_EXT_occlusion_query_boolean");
-  EXPECT_THAT(info_->extensions(),
-              HasSubstr("GL_EXT_occlusion_query_boolean"));
+  if (GetContextType() == CONTEXT_TYPE_OPENGLES2) {
+    EXPECT_THAT(info_->extensions(),
+                HasSubstr("GL_EXT_occlusion_query_boolean"));
+  }
   EXPECT_TRUE(info_->feature_flags().occlusion_query_boolean);
   EXPECT_FALSE(info_->feature_flags(
       ).use_arb_occlusion_query2_for_occlusion_query_boolean);
@@ -1202,8 +1294,10 @@ TEST_P(FeatureInfoTest, InitializeEXT_occlusion_query_boolean) {
 
 TEST_P(FeatureInfoTest, InitializeARB_occlusion_query) {
   SetupInitExpectations("GL_ARB_occlusion_query");
-  EXPECT_THAT(info_->extensions(),
-              HasSubstr("GL_EXT_occlusion_query_boolean"));
+  if (GetContextType() == CONTEXT_TYPE_OPENGLES2) {
+    EXPECT_THAT(info_->extensions(),
+                HasSubstr("GL_EXT_occlusion_query_boolean"));
+  }
   EXPECT_TRUE(info_->feature_flags().occlusion_query_boolean);
   EXPECT_FALSE(info_->feature_flags(
       ).use_arb_occlusion_query2_for_occlusion_query_boolean);
@@ -1213,8 +1307,10 @@ TEST_P(FeatureInfoTest, InitializeARB_occlusion_query) {
 
 TEST_P(FeatureInfoTest, InitializeARB_occlusion_query2) {
   SetupInitExpectations("GL_ARB_occlusion_query2 GL_ARB_occlusion_query2");
-  EXPECT_THAT(info_->extensions(),
-              HasSubstr("GL_EXT_occlusion_query_boolean"));
+  if (GetContextType() == CONTEXT_TYPE_OPENGLES2) {
+    EXPECT_THAT(info_->extensions(),
+                HasSubstr("GL_EXT_occlusion_query_boolean"));
+  }
   EXPECT_TRUE(info_->feature_flags().occlusion_query_boolean);
   EXPECT_TRUE(info_->feature_flags(
       ).use_arb_occlusion_query2_for_occlusion_query_boolean);
@@ -1222,6 +1318,37 @@ TEST_P(FeatureInfoTest, InitializeARB_occlusion_query2) {
       ).use_arb_occlusion_query_for_occlusion_query_boolean);
 }
 
+TEST_P(FeatureInfoTest, InitializeGLES3_occlusion_query_boolean) {
+  SetupInitExpectationsWithGLVersion("", "", "OpenGL ES 3.0");
+  if (GetContextType() == CONTEXT_TYPE_OPENGLES2) {
+    EXPECT_THAT(info_->extensions(),
+                HasSubstr("GL_EXT_occlusion_query_boolean"));
+  }
+  EXPECT_TRUE(info_->feature_flags().occlusion_query_boolean);
+}
+
+TEST_P(FeatureInfoTest, InitializeGL33_occlusion_query2) {
+  SetupInitExpectationsWithGLVersion("", "", "3.3");
+  if (GetContextType() == CONTEXT_TYPE_OPENGLES2) {
+    EXPECT_THAT(info_->extensions(),
+                HasSubstr("GL_EXT_occlusion_query_boolean"));
+  }
+  EXPECT_TRUE(info_->feature_flags().occlusion_query_boolean);
+  EXPECT_TRUE(info_->feature_flags(
+      ).use_arb_occlusion_query2_for_occlusion_query_boolean);
+}
+
+TEST_P(FeatureInfoTest, InitializeGL43_occlusion_query2) {
+  SetupInitExpectationsWithGLVersion("", "", "4.3");
+  if (GetContextType() == CONTEXT_TYPE_OPENGLES2) {
+    EXPECT_THAT(info_->extensions(),
+                HasSubstr("GL_EXT_occlusion_query_boolean"));
+  }
+  EXPECT_TRUE(info_->feature_flags().occlusion_query_boolean);
+  EXPECT_FALSE(info_->feature_flags(
+      ).use_arb_occlusion_query2_for_occlusion_query_boolean);
+}
+
 TEST_P(FeatureInfoTest, InitializeOES_vertex_array_object) {
   SetupInitExpectations("GL_OES_vertex_array_object");
   EXPECT_THAT(info_->extensions(),
@@ -1311,6 +1438,39 @@ TEST_P(FeatureInfoTest, InitializeWithES3) {
   EXPECT_TRUE(info_->feature_flags().use_async_readpixels);
   EXPECT_TRUE(info_->feature_flags().oes_standard_derivatives);
   EXPECT_TRUE(info_->feature_flags().oes_depth24);
+  EXPECT_THAT(info_->extensions(), Not(HasSubstr("GL_GOOGLE_depth_texture")));
+  EXPECT_THAT(info_->extensions(), Not(HasSubstr("GL_CHROMIUM_depth_texture")));
+  EXPECT_FALSE(
+      info_->validators()->texture_internal_format.IsValid(GL_DEPTH_COMPONENT));
+  EXPECT_FALSE(
+      info_->validators()->texture_internal_format.IsValid(GL_DEPTH_STENCIL));
+  EXPECT_FALSE(info_->validators()->texture_format.IsValid(GL_DEPTH_COMPONENT));
+  EXPECT_FALSE(info_->validators()->texture_format.IsValid(GL_DEPTH_STENCIL));
+  EXPECT_FALSE(info_->validators()->pixel_type.IsValid(GL_UNSIGNED_SHORT));
+  EXPECT_FALSE(info_->validators()->pixel_type.IsValid(GL_UNSIGNED_INT));
+  EXPECT_FALSE(info_->validators()->pixel_type.IsValid(GL_UNSIGNED_INT_24_8));
+  EXPECT_TRUE(info_->feature_flags().packed_depth24_stencil8);
+  EXPECT_THAT(info_->extensions(), HasSubstr("GL_OES_depth24"));
+  EXPECT_TRUE(
+      info_->validators()->render_buffer_format.IsValid(GL_DEPTH_COMPONENT24));
+  EXPECT_TRUE(
+      info_->validators()->render_buffer_format.IsValid(GL_DEPTH24_STENCIL8));
+  EXPECT_FALSE(
+      info_->validators()->texture_internal_format.IsValid(GL_DEPTH_STENCIL));
+  EXPECT_FALSE(info_->validators()->texture_format.IsValid(GL_DEPTH_STENCIL));
+  EXPECT_TRUE(info_->feature_flags().npot_ok);
+  EXPECT_TRUE(info_->feature_flags().native_vertex_array_object);
+  EXPECT_TRUE(info_->feature_flags().enable_samplers);
+  EXPECT_TRUE(info_->feature_flags().map_buffer_range);
+  EXPECT_TRUE(info_->feature_flags().ext_discard_framebuffer);
+  EXPECT_THAT(info_->extensions(), HasSubstr("GL_EXT_discard_framebuffer"));
+  EXPECT_TRUE(info_->feature_flags().chromium_sync_query);
+  EXPECT_TRUE(gl::GLFence::IsSupported());
+}
+
+TEST_P(FeatureInfoTest, InitializeWithES3AndDepthTexture) {
+  SetupInitExpectationsWithGLVersion(
+      "GL_ANGLE_depth_texture", "", "OpenGL ES 3.0");
   EXPECT_THAT(info_->extensions(), HasSubstr("GL_GOOGLE_depth_texture"));
   EXPECT_THAT(info_->extensions(), HasSubstr("GL_CHROMIUM_depth_texture"));
   EXPECT_TRUE(
@@ -1323,22 +1483,9 @@ TEST_P(FeatureInfoTest, InitializeWithES3) {
   EXPECT_TRUE(info_->validators()->pixel_type.IsValid(GL_UNSIGNED_INT));
   EXPECT_TRUE(info_->validators()->pixel_type.IsValid(GL_UNSIGNED_INT_24_8));
   EXPECT_TRUE(info_->feature_flags().packed_depth24_stencil8);
-  EXPECT_THAT(info_->extensions(), HasSubstr("GL_OES_depth24"));
-  EXPECT_TRUE(
-      info_->validators()->render_buffer_format.IsValid(GL_DEPTH_COMPONENT24));
-  EXPECT_TRUE(
-      info_->validators()->render_buffer_format.IsValid(GL_DEPTH24_STENCIL8));
   EXPECT_TRUE(
       info_->validators()->texture_internal_format.IsValid(GL_DEPTH_STENCIL));
   EXPECT_TRUE(info_->validators()->texture_format.IsValid(GL_DEPTH_STENCIL));
-  EXPECT_TRUE(info_->feature_flags().npot_ok);
-  EXPECT_TRUE(info_->feature_flags().native_vertex_array_object);
-  EXPECT_TRUE(info_->feature_flags().enable_samplers);
-  EXPECT_TRUE(info_->feature_flags().map_buffer_range);
-  EXPECT_TRUE(info_->feature_flags().ext_discard_framebuffer);
-  EXPECT_THAT(info_->extensions(), HasSubstr("GL_EXT_discard_framebuffer"));
-  EXPECT_TRUE(info_->feature_flags().chromium_sync_query);
-  EXPECT_TRUE(gl::GLFence::IsSupported());
 }
 
 TEST_P(FeatureInfoTest, InitializeWithoutSamplers) {
