@@ -31,9 +31,11 @@
 #include "gpu/command_buffer/service/context_group.h"
 #include "gpu/command_buffer/service/gl_context_virtual.h"
 #include "gpu/command_buffer/service/gles2_cmd_decoder.h"
+#include "gpu/command_buffer/service/gpu_switches.h"
 #include "gpu/command_buffer/service/image_manager.h"
 #include "gpu/command_buffer/service/mailbox_manager_impl.h"
 #include "gpu/command_buffer/service/memory_tracking.h"
+#include "gpu/command_buffer/service/service_utils.h"
 #include "gpu/command_buffer/service/sync_point_manager.h"
 #include "gpu/command_buffer/service/transfer_buffer_manager.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -55,6 +57,14 @@ namespace {
 
 uint64_t g_next_command_buffer_id = 0;
 
+void InitializeGpuPreferencesForTestingFromCommandLine(
+    const base::CommandLine& command_line,
+    GpuPreferences* preferences) {
+  // Only initialize specific GpuPreferences members used for testing.
+  preferences->use_passthrough_cmd_decoder =
+      command_line.HasSwitch(switches::kUsePassthroughCmdDecoder);
+}
+
 class GpuMemoryBufferImpl : public gfx::GpuMemoryBuffer {
  public:
   GpuMemoryBufferImpl(base::RefCountedBytes* bytes,
@@ -147,6 +157,7 @@ class IOSurfaceGpuMemoryBuffer : public gfx::GpuMemoryBuffer {
     return IOSurfaceGetWidthOfPlane(iosurface_, plane);
   }
   gfx::GpuMemoryBufferId GetId() const override {
+    NOTREACHED();
     return gfx::GpuMemoryBufferId(0);
   }
   gfx::GpuMemoryBufferHandle GetHandle() const override {
@@ -188,7 +199,7 @@ GLManager::Options::Options()
       multisampled(false),
       backbuffer_alpha(true),
       image_factory(nullptr),
-      enable_arb_texture_rectangle(false) {}
+      preserve_backbuffer(false) {}
 
 GLManager::GLManager()
     : sync_point_manager_(nullptr),
@@ -248,6 +259,9 @@ void GLManager::InitializeWithCommandLine(
 
   context_lost_allowed_ = options.context_lost_allowed;
 
+  InitializeGpuPreferencesForTestingFromCommandLine(command_line,
+                                                    &gpu_preferences_);
+
   gles2::MailboxManager* mailbox_manager = NULL;
   if (options.share_mailbox_manager) {
     mailbox_manager = options.share_mailbox_manager->mailbox_manager();
@@ -271,7 +285,8 @@ void GLManager::InitializeWithCommandLine(
   }
 
   gl::GLContext* real_gl_context = NULL;
-  if (options.virtual_manager) {
+  if (options.virtual_manager &&
+      !gpu_preferences_.use_passthrough_cmd_decoder) {
     real_gl_context = options.virtual_manager->context();
   }
 
@@ -293,22 +308,18 @@ void GLManager::InitializeWithCommandLine(
   attribs.should_use_native_gmb_for_backbuffer =
       options.image_factory != nullptr;
   attribs.offscreen_framebuffer_size = options.size;
+  attribs.buffer_preserved = options.preserve_backbuffer;
+  attribs.bind_generates_resource = options.bind_generates_resource;
 
   if (!context_group) {
     GpuDriverBugWorkarounds gpu_driver_bug_workaround(&command_line);
     scoped_refptr<gles2::FeatureInfo> feature_info =
         new gles2::FeatureInfo(command_line, gpu_driver_bug_workaround);
-    if (options.enable_arb_texture_rectangle) {
-      gles2::FeatureInfo::FeatureFlags& flags =
-          const_cast<gles2::FeatureInfo::FeatureFlags&>(
-              feature_info->feature_flags());
-      flags.arb_texture_rectangle = true;
-    }
     context_group = new gles2::ContextGroup(
-        gpu_preferences_, mailbox_manager_.get(), NULL,
+        gpu_preferences_, mailbox_manager_.get(), nullptr,
         new gpu::gles2::ShaderTranslatorCache(gpu_preferences_),
         new gpu::gles2::FramebufferCompletenessCache, feature_info,
-        options.bind_generates_resource, options.image_factory);
+        options.bind_generates_resource, options.image_factory, nullptr);
   }
 
   decoder_.reset(::gpu::gles2::GLES2Decoder::Create(context_group));
@@ -329,16 +340,20 @@ void GLManager::InitializeWithCommandLine(
   if (base_context_) {
     context_ = scoped_refptr<gl::GLContext>(new gpu::GLContextVirtual(
         share_group_.get(), base_context_->get(), decoder_->AsWeakPtr()));
-    ASSERT_TRUE(context_->Initialize(surface_.get(), attribs.gpu_preference));
+    ASSERT_TRUE(context_->Initialize(
+        surface_.get(),
+        GenerateGLContextAttribs(attribs, context_group->gpu_preferences())));
   } else {
     if (real_gl_context) {
       context_ = scoped_refptr<gl::GLContext>(new gpu::GLContextVirtual(
           share_group_.get(), real_gl_context, decoder_->AsWeakPtr()));
-      ASSERT_TRUE(
-          context_->Initialize(surface_.get(), attribs.gpu_preference));
+      ASSERT_TRUE(context_->Initialize(
+          surface_.get(),
+          GenerateGLContextAttribs(attribs, context_group->gpu_preferences())));
     } else {
-      context_ = gl::init::CreateGLContext(share_group_.get(), surface_.get(),
-                                           attribs.gpu_preference);
+      context_ = gl::init::CreateGLContext(
+          share_group_.get(), surface_.get(),
+          GenerateGLContextAttribs(attribs, context_group->gpu_preferences()));
     }
   }
   ASSERT_TRUE(context_.get() != NULL) << "could not create GL context";
@@ -402,9 +417,9 @@ void GLManager::SetupBaseContext() {
     gfx::Size size(4, 4);
     base_surface_ = new scoped_refptr<gl::GLSurface>(
         gl::init::CreateOffscreenGLSurface(size));
-    gl::GpuPreference gpu_preference(gl::PreferDiscreteGpu);
     base_context_ = new scoped_refptr<gl::GLContext>(gl::init::CreateGLContext(
-        base_share_group_->get(), base_surface_->get(), gpu_preference));
+        base_share_group_->get(), base_surface_->get(),
+        gl::GLContextAttribs()));
     #endif
   }
   ++use_count_;
@@ -413,6 +428,7 @@ void GLManager::SetupBaseContext() {
 void GLManager::OnFenceSyncRelease(uint64_t release) {
   DCHECK(sync_point_client_);
   DCHECK(!sync_point_client_->client_state()->IsFenceSyncReleased(release));
+  command_buffer_->SetReleaseCount(release);
   sync_point_client_->ReleaseFenceSync(release);
 }
 
@@ -440,6 +456,10 @@ void GLManager::SetSurface(gl::GLSurface* surface) {
   decoder_->SetSurface(surface);
 }
 
+void GLManager::PerformIdleWork() {
+  executor_->PerformIdleWork();
+}
+
 void GLManager::Destroy() {
   if (gles2_implementation_.get()) {
     MakeCurrent();
@@ -527,7 +547,6 @@ int32_t GLManager::CreateImage(ClientBuffer buffer,
   gfx::Size size(width, height);
   scoped_refptr<gl::GLImage> gl_image;
 
-  int gmb_id = -1;
 #if defined(OS_MACOSX)
   if (use_iosurface_memory_buffers_) {
     IOSurfaceGpuMemoryBuffer* gpu_memory_buffer =
@@ -540,7 +559,6 @@ int32_t GLManager::CreateImage(ClientBuffer buffer,
       return -1;
     }
     gl_image = image;
-    gmb_id = gpu_memory_buffer->GetId().id;
   }
 #endif  // defined(OS_MACOSX)
   if (!gl_image) {
@@ -561,12 +579,6 @@ int32_t GLManager::CreateImage(ClientBuffer buffer,
   gpu::gles2::ImageManager* image_manager = decoder_->GetImageManager();
   DCHECK(image_manager);
   image_manager->AddImage(gl_image.get(), new_id);
-
-  if (gmb_id != -1) {
-    DCHECK(image_gmb_ids_map_.find(new_id) == image_gmb_ids_map_.end());
-    image_gmb_ids_map_[new_id] = gmb_id;
-  }
-
   return new_id;
 }
 
@@ -580,18 +592,7 @@ int32_t GLManager::CreateGpuMemoryBufferImage(size_t width,
   return CreateImage(buffer->AsClientBuffer(), width, height, internalformat);
 }
 
-int32_t GLManager::GetImageGpuMemoryBufferId(unsigned image_id) {
-  auto it = image_gmb_ids_map_.find(image_id);
-  if (it != image_gmb_ids_map_.end())
-    return it->second;
-  return -1;
-}
-
 void GLManager::DestroyImage(int32_t id) {
-  auto it = image_gmb_ids_map_.find(id);
-  if (it != image_gmb_ids_map_.end())
-    image_gmb_ids_map_.erase(it);
-
   gpu::gles2::ImageManager* image_manager = decoder_->GetImageManager();
   DCHECK(image_manager);
   image_manager->RemoveImage(id);
@@ -637,6 +638,10 @@ bool GLManager::IsFenceSyncFlushReceived(uint64_t release) {
   return IsFenceSyncRelease(release);
 }
 
+bool GLManager::IsFenceSyncReleased(uint64_t release) {
+  return release <= command_buffer_->GetLastState().release_count;
+}
+
 void GLManager::SignalSyncToken(const gpu::SyncToken& sync_token,
                                 const base::Closure& callback) {
   if (sync_point_manager_) {
