@@ -7,7 +7,7 @@
 namespace gpu {
 
 VideoMemoryUsageStats::VideoMemoryUsageStats()
-    : bytes_allocated(0), bytes_allocated_historical_max(0) {}
+    : bytes_allocated(0) {}
 
 VideoMemoryUsageStats::VideoMemoryUsageStats(
     const VideoMemoryUsageStats& other) = default;
