@@ -11,10 +11,9 @@ namespace gpu {
 
 // static
 scoped_refptr<gl::GLSurface> ImageTransportSurface::CreateNativeSurface(
-    GpuChannelManager* manager,
-    GpuCommandBufferStub* stub,
+    base::WeakPtr<ImageTransportSurfaceDelegate> delegate,
     SurfaceHandle surface_handle,
-    gl::GLSurface::Format format) {
+    gl::GLSurfaceFormat format) {
   DCHECK_NE(surface_handle, kNullSurfaceHandle);
   scoped_refptr<gl::GLSurface> surface;
 #if defined(USE_OZONE)
@@ -25,7 +24,7 @@ scoped_refptr<gl::GLSurface> ImageTransportSurface::CreateNativeSurface(
   if (!surface)
     return surface;
   return scoped_refptr<gl::GLSurface>(
-      new PassThroughImageTransportSurface(manager, stub, surface.get()));
+      new PassThroughImageTransportSurface(delegate, surface.get()));
 }
 
 }  // namespace gpu
