@@ -609,7 +609,20 @@ SetupPnaclTranslatorFast1ThreadArmOptSzHW() {
   SUFFIX=pnacl_translator_fast_1thread.opt.hw.sz.arm
 }
 
-
+#@
+#@ SetupWasm
+#@   use emcc with wasm backend.
+SetupWasm() {
+  if [ -z $WASM_INSTALL_DIR ]; then
+    echo 'error: WASM_INSTALL_DIR is not set'
+    exit 1
+  fi
+  PREFIX=../run_wasm.sh
+  SUFFIX=js
+  # When running in d8, emcc does not support persistent output files.
+  # So we don't verify the results.
+  VERIFY=no
+}
 
 ConfigInfo() {
   SubBanner "Config Info"
@@ -999,7 +1012,7 @@ PopulateFromSpecHarness() {
     # patch if necessary
     if [[ -e $i/diff ]] ; then
       echo "PATCH"
-      patch -d $i --verbose -p0 < $i/diff
+      patch -l -d $i --verbose -p0 < $i/diff
     fi
 
     echo "COMPLETE"
