@@ -10,19 +10,21 @@ import static android.net.NetworkCapabilities.TRANSPORT_VPN;
 import static android.net.NetworkCapabilities.TRANSPORT_WIFI;
 
 import android.annotation.SuppressLint;
+import android.content.BroadcastReceiver;
 import android.content.Context;
+import android.content.ContextWrapper;
 import android.content.Intent;
+import android.content.IntentFilter;
 import android.net.ConnectivityManager;
 import android.net.ConnectivityManager.NetworkCallback;
 import android.net.Network;
 import android.net.NetworkCapabilities;
 import android.net.NetworkRequest;
-import android.net.wifi.WifiManager;
 import android.os.Build;
+import android.support.test.filters.MediumTest;
 import android.telephony.TelephonyManager;
 import android.test.InstrumentationTestCase;
 import android.test.UiThreadTest;
-import android.test.suitebuilder.annotation.MediumTest;
 
 import org.chromium.base.ApplicationState;
 import org.chromium.base.ThreadUtils;
@@ -31,6 +33,7 @@ import org.chromium.base.library_loader.LibraryProcessType;
 import org.chromium.base.test.util.Feature;
 import org.chromium.net.NetworkChangeNotifierAutoDetect.ConnectivityManagerDelegate;
 import org.chromium.net.NetworkChangeNotifierAutoDetect.NetworkState;
+import org.chromium.net.NetworkChangeNotifierAutoDetect.WifiManagerDelegate;
 import org.chromium.net.test.util.NetworkChangeNotifierTestUtil;
 
 import java.lang.reflect.Constructor;
@@ -105,6 +108,16 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         }
     }
 
+    private static void triggerApplicationStateChange(
+            final RegistrationPolicyApplicationStatus policy, final int applicationState) {
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                policy.onApplicationStateChange(applicationState);
+            }
+        });
+    }
+
     /**
      * Mocks out calls to the ConnectivityManager.
      */
@@ -135,7 +148,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         }
 
         // List of networks we're pretending are currently connected.
-        private final ArrayList<MockNetwork> mMockNetworks = new ArrayList<MockNetwork>();
+        private final ArrayList<MockNetwork> mMockNetworks = new ArrayList<>();
 
         private boolean mActiveNetworkExists;
         private int mNetworkType;
@@ -143,13 +156,16 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         private NetworkCallback mLastRegisteredNetworkCallback;
 
         @Override
-        public NetworkState getNetworkState() {
-            return new NetworkState(mActiveNetworkExists, mNetworkType, mNetworkSubtype);
+        public NetworkState getNetworkState(WifiManagerDelegate wifiManagerDelegate) {
+            return new NetworkState(mActiveNetworkExists, mNetworkType, mNetworkSubtype,
+                    mNetworkType == ConnectivityManager.TYPE_WIFI
+                            ? wifiManagerDelegate.getWifiSsid()
+                            : null);
         }
 
         @Override
         protected NetworkCapabilities getNetworkCapabilities(Network network) {
-            int netId = NetworkChangeNotifierAutoDetect.networkToNetId(network);
+            int netId = demungeNetId(NetworkChangeNotifierAutoDetect.networkToNetId(network));
             for (MockNetwork mockNetwork : mMockNetworks) {
                 if (netId == mockNetwork.mNetId) {
                     return mockNetwork.getCapabilities();
@@ -160,7 +176,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
 
         @Override
         protected boolean vpnAccessible(Network network) {
-            int netId = NetworkChangeNotifierAutoDetect.networkToNetId(network);
+            int netId = demungeNetId(NetworkChangeNotifierAutoDetect.networkToNetId(network));
             for (MockNetwork mockNetwork : mMockNetworks) {
                 if (netId == mockNetwork.mNetId) {
                     return mockNetwork.mVpnAccessible;
@@ -181,13 +197,13 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         // Dummy implementations to avoid NullPointerExceptions in default implementations:
 
         @Override
-        public int getDefaultNetId() {
+        public long getDefaultNetId() {
             return NetId.INVALID;
         }
 
         @Override
-        public NetworkState getNetworkState(Network network) {
-            return new NetworkState(false, -1, -1);
+        public int getConnectionType(Network network) {
+            return ConnectionType.CONNECTION_NONE;
         }
 
         @Override
@@ -245,28 +261,27 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     /**
      * Mocks out calls to the WifiManager.
      */
-    private static class MockWifiManagerDelegate
-            extends NetworkChangeNotifierAutoDetect.WifiManagerDelegate {
+    private static class MockWifiManagerDelegate extends WifiManagerDelegate {
         private String mWifiSSID;
-        private int mLinkSpeedMbps;
 
         @Override
-        public String getWifiSSID() {
+        public String getWifiSsid() {
             return mWifiSSID;
         }
 
         public void setWifiSSID(String wifiSSID) {
             mWifiSSID = wifiSSID;
         }
+    }
 
-        @Override
-        public int getLinkSpeedInMbps() {
-            return mLinkSpeedMbps;
-        }
-
-        public void setLinkSpeedInMbps(int linkSpeedInMbps) {
-            mLinkSpeedMbps = linkSpeedInMbps;
+    private static int demungeNetId(long netId) {
+        // On Marshmallow, demunge the NetID to undo munging done in Network.getNetworkHandle().
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
+            netId >>= 32;
         }
+        // Now that the NetID has been demunged it is a true NetID which means it's only a 16-bit
+        // value (see ConnectivityService.MAX_NET_ID) so it should be safe to cast to int.
+        return (int) netId;
     }
 
     // Types of network changes. Each is associated with a NetworkChangeNotifierAutoDetect.Observer
@@ -284,9 +299,9 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
          * @param changeType the type of change.
          * @param netId the network identifier of the network changing.
          */
-        ChangeInfo(ChangeType changeType, int netId) {
+        ChangeInfo(ChangeType changeType, long netId) {
             mChangeType = changeType;
-            mNetId = netId;
+            mNetId = demungeNetId(netId);
         }
     }
 
@@ -295,7 +310,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     private static class TestNetworkChangeNotifierAutoDetectObserver
             implements NetworkChangeNotifierAutoDetect.Observer {
         // The list of network changes that have been witnessed.
-        final ArrayList<ChangeInfo> mChanges = new ArrayList<ChangeInfo>();
+        final ArrayList<ChangeInfo> mChanges = new ArrayList<>();
 
         @Override
         public void onConnectionTypeChanged(int newConnectionType) {}
@@ -303,25 +318,25 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         public void onMaxBandwidthChanged(double maxBandwidthMbps) {}
 
         @Override
-        public void onNetworkConnect(int netId, int connectionType) {
+        public void onNetworkConnect(long netId, int connectionType) {
             ThreadUtils.assertOnUiThread();
             mChanges.add(new ChangeInfo(ChangeType.CONNECT, netId));
         }
 
         @Override
-        public void onNetworkSoonToDisconnect(int netId) {
+        public void onNetworkSoonToDisconnect(long netId) {
             ThreadUtils.assertOnUiThread();
             mChanges.add(new ChangeInfo(ChangeType.SOON_TO_DISCONNECT, netId));
         }
 
         @Override
-        public void onNetworkDisconnect(int netId) {
+        public void onNetworkDisconnect(long netId) {
             ThreadUtils.assertOnUiThread();
             mChanges.add(new ChangeInfo(ChangeType.DISCONNECT, netId));
         }
 
         @Override
-        public void purgeActiveNetworkList(int[] activeNetIds) {
+        public void purgeActiveNetworkList(long[] activeNetIds) {
             ThreadUtils.assertOnUiThread();
             if (activeNetIds.length == 1) {
                 mChanges.add(new ChangeInfo(ChangeType.PURGE_LIST, activeNetIds[0]));
@@ -359,7 +374,22 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
      *            it is in the foreground.
      */
     private void createTestNotifier(WatchForChanges watchForChanges) {
-        Context context = getInstrumentation().getTargetContext();
+        Context context = new ContextWrapper(getInstrumentation().getTargetContext()) {
+            // Mock out to avoid unintended system interaction.
+            @Override
+            public Intent registerReceiver(BroadcastReceiver receiver, IntentFilter filter) {
+                return null;
+            }
+
+            @Override
+            public void unregisterReceiver(BroadcastReceiver receiver) {}
+
+            // Don't allow escaping the mock via the application context.
+            @Override
+            public Context getApplicationContext() {
+                return this;
+            }
+        };
         mNotifier = new TestNetworkChangeNotifier(context);
         NetworkChangeNotifier.resetInstanceForTests(mNotifier);
         if (watchForChanges == WatchForChanges.ALWAYS) {
@@ -389,7 +419,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     private int getCurrentConnectionType() {
         final NetworkChangeNotifierAutoDetect.NetworkState networkState =
                 mReceiver.getCurrentNetworkState();
-        return mReceiver.getCurrentConnectionType(networkState);
+        return NetworkChangeNotifierAutoDetect.convertToConnectionType(networkState);
     }
 
     // Create Network object given a NetID.
@@ -404,13 +434,13 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     @Override
     protected void setUp() throws Exception {
         super.setUp();
-        LibraryLoader.get(LibraryProcessType.PROCESS_BROWSER)
-                .ensureInitialized(getInstrumentation().getTargetContext());
+        LibraryLoader.get(LibraryProcessType.PROCESS_BROWSER).ensureInitialized();
         if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
             // Find Network.Network(int netId) using reflection.
             mNetworkConstructor = Network.class.getConstructor(Integer.TYPE);
         }
         ThreadUtils.postOnUiThread(new Runnable() {
+            @Override
             public void run() {
                 createTestNotifier(WatchForChanges.ONLY_WHEN_APP_IN_FOREGROUND);
             }
@@ -462,47 +492,16 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     public void testNetworkChangeNotifierRegistersForIntents() throws InterruptedException {
         RegistrationPolicyApplicationStatus policy =
                 (RegistrationPolicyApplicationStatus) mReceiver.getRegistrationPolicy();
-        policy.onApplicationStateChange(ApplicationState.HAS_RUNNING_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_RUNNING_ACTIVITIES);
         assertTrue(mReceiver.isReceiverRegisteredForTesting());
 
-        policy.onApplicationStateChange(ApplicationState.HAS_PAUSED_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_PAUSED_ACTIVITIES);
         assertFalse(mReceiver.isReceiverRegisteredForTesting());
 
-        policy.onApplicationStateChange(ApplicationState.HAS_RUNNING_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_RUNNING_ACTIVITIES);
         assertTrue(mReceiver.isReceiverRegisteredForTesting());
     }
 
-    /**
-     * Tests that changing the RSSI_CHANGED_ACTION intent updates MaxBandwidth.
-     */
-    @UiThreadTest
-    @MediumTest
-    @Feature({"Android-AppBase"})
-    public void testNetworkChangeNotifierRSSIEventUpdatesMaxBandwidthForWiFi()
-            throws InterruptedException {
-        NetworkChangeNotifier notifier = NetworkChangeNotifier.getInstance();
-        mConnectivityDelegate.setNetworkType(ConnectivityManager.TYPE_WIFI);
-        mWifiDelegate.setLinkSpeedInMbps(42);
-        Intent intent = new Intent(WifiManager.RSSI_CHANGED_ACTION);
-        mReceiver.onReceive(getInstrumentation().getTargetContext(), intent);
-
-        assertEquals(42.0, notifier.getCurrentMaxBandwidthInMbps());
-
-        // Changing the link speed has no effect until the intent fires.
-        mWifiDelegate.setLinkSpeedInMbps(80);
-        assertEquals(42.0, notifier.getCurrentMaxBandwidthInMbps());
-
-        // Fire the intent.
-        mReceiver.onReceive(getInstrumentation().getTargetContext(), intent);
-        assertEquals(80.0, notifier.getCurrentMaxBandwidthInMbps());
-
-        // Firing a network type change intent also causes max bandwidth to update.
-        mWifiDelegate.setLinkSpeedInMbps(20);
-        intent = new Intent(ConnectivityManager.CONNECTIVITY_ACTION);
-        mReceiver.onReceive(getInstrumentation().getTargetContext(), intent);
-        assertEquals(20.0, notifier.getCurrentMaxBandwidthInMbps());
-    }
-
     /**
      * Tests that changing the network type changes the maxBandwidth.
      */
@@ -520,11 +519,10 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     @MediumTest
     @Feature({"Android-AppBase"})
     public void testNetworkChangeNotifierMaxBandwidthWifi() throws InterruptedException {
-        // Test that for wifi types the link speed is read from the WifiManager.
-        mWifiDelegate.setLinkSpeedInMbps(42);
+        // Show that for WiFi the link speed is unknown (+Infinity).
         mConnectivityDelegate.setNetworkType(ConnectivityManager.TYPE_WIFI);
         assertEquals(ConnectionType.CONNECTION_WIFI, getCurrentConnectionType());
-        assertEquals(42.0, getCurrentMaxBandwidthInMbps());
+        assertEquals(Double.POSITIVE_INFINITY, getCurrentMaxBandwidthInMbps());
     }
 
     @UiThreadTest
@@ -606,14 +604,14 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         // Pretend we got moved to the background.
         final RegistrationPolicyApplicationStatus policy =
                 (RegistrationPolicyApplicationStatus) mReceiver.getRegistrationPolicy();
-        policy.onApplicationStateChange(ApplicationState.HAS_PAUSED_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_PAUSED_ACTIVITIES);
         // Change the state.
         mConnectivityDelegate.setActiveNetworkExists(true);
         mConnectivityDelegate.setNetworkType(ConnectivityManager.TYPE_WIFI);
         // The NetworkChangeNotifierAutoDetect doesn't receive any notification while we are in the
         // background, but when we get back to the foreground the state changed should be detected
         // and a notification sent.
-        policy.onApplicationStateChange(ApplicationState.HAS_RUNNING_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_RUNNING_ACTIVITIES);
         assertTrue(observer.hasReceivedNotification());
     }
 
@@ -628,7 +626,6 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         // Initialize the NetworkChangeNotifier with a connection.
         mConnectivityDelegate.setActiveNetworkExists(true);
         mConnectivityDelegate.setNetworkType(ConnectivityManager.TYPE_WIFI);
-        mWifiDelegate.setLinkSpeedInMbps(1);
         Intent connectivityIntent = new Intent(ConnectivityManager.CONNECTIVITY_ACTION);
         mReceiver.onReceive(getInstrumentation().getTargetContext(), connectivityIntent);
         assertTrue(mNotifier.hasReceivedMaxBandwidthNotification());
@@ -640,12 +637,6 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         mReceiver.onReceive(getInstrumentation().getTargetContext(), connectivityIntent);
         assertFalse(mNotifier.hasReceivedMaxBandwidthNotification());
 
-        // We should be notified if the bandwidth changed but not the connection type.
-        mWifiDelegate.setLinkSpeedInMbps(2);
-        mReceiver.onReceive(getInstrumentation().getTargetContext(), connectivityIntent);
-        assertTrue(mNotifier.hasReceivedMaxBandwidthNotification());
-        mNotifier.resetHasReceivedMaxBandwidthNotification();
-
         // We should be notified if bandwidth and connection type changed.
         mConnectivityDelegate.setNetworkType(ConnectivityManager.TYPE_ETHERNET);
         mReceiver.onReceive(getInstrumentation().getTargetContext(), connectivityIntent);
@@ -690,18 +681,16 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
     public void testConnectivityManagerDelegateDoesNotCrash() {
         ConnectivityManagerDelegate delegate =
                 new ConnectivityManagerDelegate(getInstrumentation().getTargetContext());
-        delegate.getNetworkState();
+        delegate.getNetworkState(new WifiManagerDelegate(getInstrumentation().getTargetContext()));
         if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
-            // getNetworkState(Network) doesn't crash upon invalid Network argument.
+            // getConnectionType(Network) doesn't crash upon invalid Network argument.
             Network invalidNetwork = netIdToNetwork(NetId.INVALID);
-            NetworkState invalidNetworkState = delegate.getNetworkState(invalidNetwork);
-            assertFalse(invalidNetworkState.isConnected());
-            assertEquals(-1, invalidNetworkState.getNetworkType());
-            assertEquals(-1, invalidNetworkState.getNetworkSubType());
+            assertEquals(
+                    ConnectionType.CONNECTION_NONE, delegate.getConnectionType(invalidNetwork));
 
             Network[] networks = delegate.getAllNetworksUnfiltered();
             if (networks.length >= 1) {
-                delegate.getNetworkState(networks[0]);
+                delegate.getConnectionType(networks[0]);
             }
             delegate.getDefaultNetId();
             NetworkCallback networkCallback = new NetworkCallback();
@@ -766,7 +755,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
             }
 
             @Override
-            int getDefaultNetId() {
+            long getDefaultNetId() {
                 return Integer.parseInt(mNetworks[1].toString());
             }
 
@@ -776,8 +765,8 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
             }
 
             @Override
-            public NetworkState getNetworkState(Network network) {
-                return new NetworkState(false, -1, -1);
+            public int getConnectionType(Network network) {
+                return ConnectionType.CONNECTION_NONE;
             }
         });
 
@@ -789,9 +778,9 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
         // returns an array of a repeated sequence of: (NetID, ConnectionType).
         // There are 4 entries in the array, two for each network.
         assertEquals(4, ncn.getNetworksAndTypes().length);
-        assertEquals(111, ncn.getNetworksAndTypes()[0]);
+        assertEquals(111, demungeNetId(ncn.getNetworksAndTypes()[0]));
         assertEquals(ConnectionType.CONNECTION_NONE, ncn.getNetworksAndTypes()[1]);
-        assertEquals(333, ncn.getNetworksAndTypes()[2]);
+        assertEquals(333, demungeNetId(ncn.getNetworksAndTypes()[2]));
         assertEquals(ConnectionType.CONNECTION_NONE, ncn.getNetworksAndTypes()[3]);
     }
 
@@ -811,6 +800,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
                 new TestNetworkChangeNotifierAutoDetectObserver();
         Callable<NetworkChangeNotifierAutoDetect> callable =
                 new Callable<NetworkChangeNotifierAutoDetect>() {
+                    @Override
                     public NetworkChangeNotifierAutoDetect call() {
                         return new NetworkChangeNotifierAutoDetect(
                                 observer, context, new RegistrationPolicyApplicationStatus() {
@@ -826,8 +816,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
                                 });
                     }
                 };
-        FutureTask<NetworkChangeNotifierAutoDetect> task =
-                new FutureTask<NetworkChangeNotifierAutoDetect>(callable);
+        FutureTask<NetworkChangeNotifierAutoDetect> task = new FutureTask<>(callable);
         ThreadUtils.postOnUiThread(task);
         NetworkChangeNotifierAutoDetect ncn = task.get();
 
@@ -840,7 +829,7 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
 
         RegistrationPolicyApplicationStatus policy =
                 (RegistrationPolicyApplicationStatus) ncn.getRegistrationPolicy();
-        policy.onApplicationStateChange(ApplicationState.HAS_RUNNING_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_RUNNING_ACTIVITIES);
         assertTrue(ncn.isReceiverRegisteredForTesting());
 
         // Find NetworkChangeNotifierAutoDetect's NetworkCallback, which should have been registered
@@ -865,9 +854,9 @@ public class NetworkChangeNotifierTest extends InstrumentationTestCase {
 
         // Simulate app backgrounding then foregrounding.
         assertTrue(ncn.isReceiverRegisteredForTesting());
-        policy.onApplicationStateChange(ApplicationState.HAS_PAUSED_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_PAUSED_ACTIVITIES);
         assertFalse(ncn.isReceiverRegisteredForTesting());
-        policy.onApplicationStateChange(ApplicationState.HAS_RUNNING_ACTIVITIES);
+        triggerApplicationStateChange(policy, ApplicationState.HAS_RUNNING_ACTIVITIES);
         assertTrue(ncn.isReceiverRegisteredForTesting());
         // Verify network list purged.
         observer.assertLastChange(ChangeType.PURGE_LIST, NetId.INVALID);
