@@ -5,13 +5,11 @@
 #include "net/nqe/network_quality.h"
 
 namespace net {
-
 namespace nqe {
-
 namespace internal {
 
 base::TimeDelta InvalidRTT() {
-  return base::TimeDelta::Max();
+  return base::TimeDelta::FromMilliseconds(INVALID_RTT_THROUGHPUT);
 }
 
 NetworkQuality::NetworkQuality()
@@ -23,7 +21,7 @@ NetworkQuality::NetworkQuality(const base::TimeDelta& http_rtt,
     : http_rtt_(http_rtt),
       transport_rtt_(transport_rtt),
       downstream_throughput_kbps_(downstream_throughput_kbps) {
-  DCHECK_GE(downstream_throughput_kbps_, 0);
+  DCHECK_GE(downstream_throughput_kbps_, kInvalidThroughput);
 }
 
 NetworkQuality::NetworkQuality(const NetworkQuality& other)
@@ -40,8 +38,23 @@ NetworkQuality& NetworkQuality::operator=(const NetworkQuality& other) {
   return *this;
 }
 
+bool NetworkQuality::operator==(const NetworkQuality& other) const {
+  return http_rtt_ == other.http_rtt_ &&
+         transport_rtt_ == other.transport_rtt_ &&
+         downstream_throughput_kbps_ == other.downstream_throughput_kbps_;
+}
+
+bool NetworkQuality::IsFaster(const NetworkQuality& other) const {
+  return (http_rtt() == InvalidRTT() || other.http_rtt() == InvalidRTT() ||
+          http_rtt() <= other.http_rtt()) &&
+         (transport_rtt() == InvalidRTT() ||
+          other.transport_rtt() == InvalidRTT() ||
+          transport_rtt() <= other.transport_rtt()) &&
+         (downstream_throughput_kbps() == kInvalidThroughput ||
+          other.downstream_throughput_kbps() == kInvalidThroughput ||
+          downstream_throughput_kbps() >= other.downstream_throughput_kbps());
+}
+
 }  // namespace internal
-
 }  // namespace nqe
-
-}  // namespace net
\ No newline at end of file
+}  // namespace net
