@@ -10,7 +10,6 @@
 #include <map>
 #include <memory>
 #include <string>
-#include <tuple>
 
 #include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
@@ -23,10 +22,13 @@
 #include "net/base/net_export.h"
 #include "net/base/network_change_notifier.h"
 #include "net/nqe/cached_network_quality.h"
+#include "net/nqe/effective_connection_type.h"
 #include "net/nqe/external_estimate_provider.h"
+#include "net/nqe/network_id.h"
 #include "net/nqe/network_quality.h"
 #include "net/nqe/network_quality_observation.h"
 #include "net/nqe/network_quality_observation_source.h"
+#include "net/nqe/network_quality_store.h"
 #include "net/nqe/observation_buffer.h"
 #include "net/socket/socket_performance_watcher_factory.h"
 
@@ -52,32 +54,12 @@ class URLRequest;
 // A single instance of NQE can be attached to multiple URLRequestContexts,
 // thereby increasing the single NQE instance's accuracy by providing more
 // observed traffic characteristics.
-class NET_EXPORT_PRIVATE NetworkQualityEstimator
+class NET_EXPORT NetworkQualityEstimator
     : public NetworkChangeNotifier::ConnectionTypeObserver,
       public ExternalEstimateProvider::UpdatedEstimateDelegate {
  public:
-  // EffectiveConnectionType is the connection type whose typical performance is
-  // most similar to the measured performance of the network in use. In many
-  // cases, the "effective" connection type and the actual type of connection in
-  // use are the same, but often a network connection performs significantly
-  // different, usually worse, from its expected capabilities.
-  // EffectiveConnectionType of a network is independent of if the current
-  // connection is metered or not. For example, an unmetered slow connection may
-  // have EFFECTIVE_CONNECTION_TYPE_SLOW_2G as its effective connection type.
-  enum EffectiveConnectionType {
-    // The connection types should be in increasing order of quality.
-    EFFECTIVE_CONNECTION_TYPE_UNKNOWN = 0,
-    EFFECTIVE_CONNECTION_TYPE_OFFLINE,
-    EFFECTIVE_CONNECTION_TYPE_SLOW_2G,
-    EFFECTIVE_CONNECTION_TYPE_2G,
-    EFFECTIVE_CONNECTION_TYPE_3G,
-    EFFECTIVE_CONNECTION_TYPE_4G,
-    EFFECTIVE_CONNECTION_TYPE_BROADBAND,
-    EFFECTIVE_CONNECTION_TYPE_LAST,
-  };
-
   // Observes changes in effective connection type.
-  class NET_EXPORT_PRIVATE EffectiveConnectionTypeObserver {
+  class NET_EXPORT EffectiveConnectionTypeObserver {
    public:
     // Notifies the observer of a change in the effective connection type.
     // NetworkQualityEstimator computes the effective connection type once in
@@ -102,6 +84,36 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
     DISALLOW_COPY_AND_ASSIGN(EffectiveConnectionTypeObserver);
   };
 
+  // Observes changes in the network quality.
+  class NET_EXPORT RTTAndThroughputEstimatesObserver {
+   public:
+    // Notifies the observer when estimated HTTP RTT, estimated transport RTT or
+    // estimated downstream throughput is computed. NetworkQualityEstimator
+    // computes the RTT and throughput estimates at regular intervals.
+    // Additionally, when there is a change in the connection type of the
+    // device, then the estimates are immediately computed. The observer must
+    // register and unregister itself on the IO thread. All the observers would
+    // be notified on the IO thread.
+    //
+    // |http_rtt|, |transport_rtt| and |downstream_throughput_kbps| are the
+    // computed estimates of the HTTP RTT, transport RTT and downstream
+    // throughput (in kilobits per second), respectively. If an estimate of the
+    // HTTP or transport RTT is unavailable, it will be set to
+    // nqe::internal::InvalidRTT(). If the throughput estimate is unavailable,
+    // it will be set to nqe::internal::kInvalidThroughput.
+    virtual void OnRTTOrThroughputEstimatesComputed(
+        base::TimeDelta http_rtt,
+        base::TimeDelta transport_rtt,
+        int32_t downstream_throughput_kbps) = 0;
+
+   protected:
+    RTTAndThroughputEstimatesObserver() {}
+    virtual ~RTTAndThroughputEstimatesObserver() {}
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(RTTAndThroughputEstimatesObserver);
+  };
+
   // Observes measurements of round trip time.
   class NET_EXPORT_PRIVATE RTTObserver {
    public:
@@ -138,6 +150,29 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
     DISALLOW_COPY_AND_ASSIGN(ThroughputObserver);
   };
 
+  // Provides simple interface to obtain the effective connection type.
+  class NET_EXPORT NetworkQualityProvider {
+   public:
+    // Returns the current effective connection type.
+    virtual EffectiveConnectionType GetEffectiveConnectionType() const = 0;
+
+    // Adds |observer| to a list of effective connection type observers.
+    virtual void AddEffectiveConnectionTypeObserver(
+        EffectiveConnectionTypeObserver* observer) = 0;
+
+    // Removes |observer| from a list of effective connection type observers.
+    virtual void RemoveEffectiveConnectionTypeObserver(
+        EffectiveConnectionTypeObserver* observer) = 0;
+
+    virtual ~NetworkQualityProvider() {}
+
+   protected:
+    NetworkQualityProvider() {}
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(NetworkQualityProvider);
+  };
+
   // Creates a new NetworkQualityEstimator.
   // |variation_params| is the map containing all field trial parameters
   // related to NetworkQualityEstimator field trial.
@@ -166,8 +201,10 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
 
   ~NetworkQualityEstimator() override;
 
-  // Returns the effective type of the current connection. Virtualized for
-  // testing.
+  // Returns the last computed effective type of the current connection. The
+  // effective connection type is computed by the network quality estimator at
+  // regular intervals and at certain events (e.g., connection change).
+  // Virtualized for testing.
   virtual EffectiveConnectionType GetEffectiveConnectionType() const;
 
   // Returns the effective type of the current connection based on only the
@@ -177,7 +214,8 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
       const base::TimeTicks& start_time) const;
 
   // Adds |observer| to the list of effective connection type observers. Must be
-  // called on the IO thread.
+  // called on the IO thread. |observer| would be notified of the current
+  // effective connection type in the next message pump.
   void AddEffectiveConnectionTypeObserver(
       EffectiveConnectionTypeObserver* observer);
 
@@ -186,6 +224,17 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   void RemoveEffectiveConnectionTypeObserver(
       EffectiveConnectionTypeObserver* observer);
 
+  // Adds |observer| to the list of RTT and throughput estimate observers. Must
+  // be called on the IO thread. |observer| would be notified of the current RTT
+  // and throughput estimates in the next message pump.
+  void AddRTTAndThroughputEstimatesObserver(
+      RTTAndThroughputEstimatesObserver* observer);
+
+  // Removes |observer| from the list of RTT and throughput estimate observers.
+  // Must be called on the IO thread.
+  void RemoveRTTAndThroughputEstimatesObserver(
+      RTTAndThroughputEstimatesObserver* observer);
+
   // Notifies NetworkQualityEstimator that the response header of |request| has
   // been received.
   void NotifyHeadersReceived(const URLRequest& request);
@@ -196,7 +245,7 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
 
   // Notifies NetworkQualityEstimator that the response body of |request| has
   // been received.
-  void NotifyRequestCompleted(const URLRequest& request);
+  void NotifyRequestCompleted(const URLRequest& request, int net_error);
 
   // Notifies NetworkQualityEstimator that |request| will be destroyed.
   void NotifyURLRequestDestroyed(const URLRequest& request);
@@ -219,14 +268,6 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
 
   SocketPerformanceWatcherFactory* GetSocketPerformanceWatcherFactory();
 
-  // Returns a string equivalent to |type|.
-  static const char* GetNameForEffectiveConnectionType(
-      EffectiveConnectionType type);
-
-  // Returns an EffectiveConnectionType equivalent to |connection_type_name|.
-  static EffectiveConnectionType GetEffectiveConnectionTypeForName(
-      const std::string& connection_type_name);
-
   // |use_localhost_requests| should only be true when testing against local
   // HTTP server and allows the requests to local host to be used for network
   // quality estimation.
@@ -237,46 +278,39 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // network quality estimation.
   void SetUseSmallResponsesForTesting(bool use_small_responses);
 
+  // If |disable_offline_check| is set to true, then the device offline check is
+  // disabled when computing the effective connection type or when writing the
+  // prefs.
+  void DisableOfflineCheckForTesting(bool disable_offline_check);
+
+  // Reports |effective_connection_type| to all
+  // EffectiveConnectionTypeObservers.
+  void ReportEffectiveConnectionTypeForTesting(
+      EffectiveConnectionType effective_connection_type);
+
+  // Adds and removes |observer| from the list of cache observers.
+  void AddNetworkQualitiesCacheObserver(
+      nqe::internal::NetworkQualityStore::NetworkQualitiesCacheObserver*
+          observer);
+  void RemoveNetworkQualitiesCacheObserver(
+      nqe::internal::NetworkQualityStore::NetworkQualitiesCacheObserver*
+          observer);
+
+  // Called when the persistent prefs have been read. |read_prefs| contains the
+  // parsed prefs as a map between NetworkIDs and CachedNetworkQualities.
+  void OnPrefsRead(
+      const std::map<nqe::internal::NetworkID,
+                     nqe::internal::CachedNetworkQuality> read_prefs);
+
  protected:
-  // NetworkID is used to uniquely identify a network.
-  // For the purpose of network quality estimation and caching, a network is
-  // uniquely identified by a combination of |type| and
-  // |id|. This approach is unable to distinguish networks with
-  // same name (e.g., different Wi-Fi networks with same SSID).
-  // This is a protected member to expose it to tests.
-  struct NET_EXPORT_PRIVATE NetworkID {
-    NetworkID(NetworkChangeNotifier::ConnectionType type, const std::string& id)
-        : type(type), id(id) {}
-    NetworkID(const NetworkID& other) : type(other.type), id(other.id) {}
-    ~NetworkID() {}
-
-    NetworkID& operator=(const NetworkID& other) {
-      type = other.type;
-      id = other.id;
-      return *this;
-    }
-
-    // Overloaded because NetworkID is used as key in a map.
-    bool operator<(const NetworkID& other) const {
-      return std::tie(type, id) < std::tie(other.type, other.id);
-    }
-
-    // Connection type of the network.
-    NetworkChangeNotifier::ConnectionType type;
-
-    // Name of this network. This is set to:
-    // - Wi-Fi SSID if the device is connected to a Wi-Fi access point and the
-    //   SSID name is available, or
-    // - MCC/MNC code of the cellular carrier if the device is connected to a
-    //   cellular network, or
-    // - "Ethernet" in case the device is connected to ethernet.
-    // - An empty string in all other cases or if the network name is not
-    //   exposed by platform APIs.
-    std::string id;
-  };
-
-  // Returns true if the cached network quality estimate was successfully read.
-  bool ReadCachedNetworkQualityEstimate();
+  // A protected constructor for testing that allows setting the value of
+  // |add_default_platform_observations_|.
+  NetworkQualityEstimator(
+      std::unique_ptr<ExternalEstimateProvider> external_estimates_provider,
+      const std::map<std::string, std::string>& variation_params,
+      bool use_local_host_requests_for_tests,
+      bool use_smaller_responses_for_tests,
+      bool add_default_platform_observations);
 
   // NetworkChangeNotifier::ConnectionTypeObserver implementation:
   void OnConnectionTypeChanged(
@@ -287,46 +321,30 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
                                   int32_t downstream_throughput_kbps,
                                   int32_t upstream_throughput_kbps) override;
 
-  // Returns true if the RTT is available and sets |rtt| to the RTT estimated at
-  // the HTTP layer. Virtualized for testing. |rtt| should not be null. The RTT
-  // at the HTTP layer measures the time from when the request was sent (this
-  // happens after the connection is established) to the time when the response
-  // headers were received.
-  virtual bool GetHttpRTTEstimate(base::TimeDelta* rtt) const
-      WARN_UNUSED_RESULT;
-
-  // Returns true if the RTT is available and sets |rtt| to the RTT estimated at
-  // the transport layer. |rtt| should not be null. Virtualized for testing.
-  virtual bool GetTransportRTTEstimate(base::TimeDelta* rtt) const
-      WARN_UNUSED_RESULT;
-
-  // Returns true if downlink throughput is available and sets |kbps| to
-  // estimated downlink throughput (in kilobits per second).
-  // Virtualized for testing. |kbps| should not be null.
-  virtual bool GetDownlinkThroughputKbpsEstimate(int32_t* kbps) const;
-
   // Returns true if median RTT at the HTTP layer is available and sets |rtt|
   // to the median of RTT observations since |start_time|.
   // Virtualized for testing. |rtt| should not be null. The RTT at the HTTP
   // layer measures the time from when the request was sent (this happens after
   // the connection is established) to the time when the response headers were
   // received.
-  virtual bool GetRecentHttpRTTMedian(const base::TimeTicks& start_time,
-                                      base::TimeDelta* rtt) const
-      WARN_UNUSED_RESULT;
+  // TODO(tbansal): Change it to return HTTP RTT as base::TimeDelta.
+  virtual bool GetRecentHttpRTT(const base::TimeTicks& start_time,
+                                base::TimeDelta* rtt) const WARN_UNUSED_RESULT;
 
   // Returns true if the median RTT at the transport layer is available and sets
   // |rtt| to the median of transport layer RTT observations since
   // |start_time|. |rtt| should not be null. Virtualized for testing.
-  virtual bool GetRecentTransportRTTMedian(const base::TimeTicks& start_time,
-                                           base::TimeDelta* rtt) const
+  // TODO(tbansal): Change it to return transport RTT as base::TimeDelta.
+  virtual bool GetRecentTransportRTT(const base::TimeTicks& start_time,
+                                     base::TimeDelta* rtt) const
       WARN_UNUSED_RESULT;
 
   // Returns true if median downstream throughput is available and sets |kbps|
   // to the median of downstream throughput (in kilobits per second)
   // observations since |start_time|. Virtualized for testing. |kbps|
   // should not be null. Virtualized for testing.
-  virtual bool GetRecentMedianDownlinkThroughputKbps(
+  // TODO(tbansal): Change it to return throughput as int32.
+  virtual bool GetRecentDownlinkThroughputKbps(
       const base::TimeTicks& start_time,
       int32_t* kbps) const WARN_UNUSED_RESULT;
 
@@ -338,20 +356,42 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // Overrides the tick clock used by |this| for testing.
   void SetTickClockForTesting(std::unique_ptr<base::TickClock> tick_clock);
 
+  // Returns a random double in the range [0.0, 1.0). Virtualized for testing.
+  virtual double RandDouble() const;
+
+  // Returns the effective type of the current connection based on only the
+  // observations received after |start_time|. |http_rtt|, |transport_rtt| and
+  // |downstream_throughput_kbps| must be non-null. |http_rtt|, |transport_rtt|
+  // and |downstream_throughput_kbps| are set to the expected HTTP RTT,
+  // transport RTT and downstream throughput (in kilobits per second) based on
+  // observations taken since |start_time|. Virtualized for testing.
+  virtual EffectiveConnectionType
+  GetRecentEffectiveConnectionTypeAndNetworkQuality(
+      const base::TimeTicks& start_time,
+      base::TimeDelta* http_rtt,
+      base::TimeDelta* transport_rtt,
+      int32_t* downstream_throughput_kbps) const;
+
  private:
+  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
+                           AdaptiveRecomputationEffectiveConnectionType);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, StoreObservations);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, TestAddObservation);
-  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, ObtainOperatingParams);
+  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
+                           DefaultObservationsOverridden);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
                            ObtainAlgorithmToUseFromParams);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, HalfLifeParam);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, ComputedPercentiles);
-  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, TestCaching);
-  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
-                           TestLRUCacheMaximumSize);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest, TestGetMetricsSince);
   FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
                            TestExternalEstimateProviderMergeEstimates);
+  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
+                           UnknownEffectiveConnectionType);
+  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
+                           TypicalNetworkQualities);
+  FRIEND_TEST_ALL_PREFIXES(NetworkQualityEstimatorTest,
+                           OnPrefsReadWithReadingDisabled);
 
   // Value of round trip time observations is in base::TimeDelta.
   typedef nqe::internal::Observation<base::TimeDelta> RttObservation;
@@ -362,20 +402,31 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   typedef nqe::internal::Observation<int32_t> ThroughputObservation;
   typedef nqe::internal::ObservationBuffer<int32_t> ThroughputObservationBuffer;
 
-  // This does not use a unordered_map or hash_map for code simplicity (key just
-  // implements operator<, rather than hash and equality) and because the map is
-  // tiny.
-  typedef std::map<NetworkID, nqe::internal::CachedNetworkQuality>
-      CachedNetworkQualities;
-
   // Algorithms supported by network quality estimator for computing effective
   // connection type.
   enum class EffectiveConnectionTypeAlgorithm {
     HTTP_RTT_AND_DOWNSTREAM_THROUGHOUT = 0,
+    TRANSPORT_RTT_OR_DOWNSTREAM_THROUGHOUT,
     EFFECTIVE_CONNECTION_TYPE_ALGORITHM_LAST
   };
 
+  // Defines how a metric (e.g, transport RTT) should be used when computing
+  // the effective connection type.
+  enum class MetricUsage {
+    // The metric should not be used when computing the effective connection
+    // type.
+    DO_NOT_USE = 0,
+    // If the metric is available, then it should be used when computing the
+    // effective connection type.
+    USE_IF_AVAILABLE,
+    // The metric is required when computing the effective connection type.
+    // If the value of the metric is unavailable, effective connection type
+    // should be set to |EFFECTIVE_CONNECTION_TYPE_UNKNOWN|.
+    MUST_BE_USED,
+  };
+
   // Map from algorithm names to EffectiveConnectionTypeAlgorithm.
+  // TODO(tbansal): Consider using an autogenerated enum using macros.
   const std::map<std::string, EffectiveConnectionTypeAlgorithm>
       algorithm_name_to_enum_;
 
@@ -385,26 +436,21 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
       kDefaultEffectiveConnectionTypeAlgorithm =
           EffectiveConnectionTypeAlgorithm::HTTP_RTT_AND_DOWNSTREAM_THROUGHOUT;
 
-  // Minimum valid value of the variation parameter that holds RTT (in
-  // milliseconds) values.
-  static const int kMinimumRTTVariationParameterMsec = 1;
-
-  // Minimum valid value of the variation parameter that holds throughput (in
-  // kilobits per second) values.
-  static const int kMinimumThroughputVariationParameterKbps = 1;
-
-  // Maximum size of the cache that holds network quality estimates.
-  // Smaller size may reduce the cache hit rate due to frequent evictions.
-  // Larger size may affect performance.
-  static const size_t kMaximumNetworkQualityCacheSize = 10;
-
   // Returns the RTT value to be used when the valid RTT is unavailable. Readers
   // should discard RTT if it is set to the value returned by |InvalidRTT()|.
   static const base::TimeDelta InvalidRTT();
 
+  // Queries external estimate provider for network quality. When the network
+  // quality is available, OnUpdatedEstimateAvailable() is called.
+  void MaybeQueryExternalEstimateProvider() const;
+
   // Records UMA when there is a change in connection type.
   void RecordMetricsOnConnectionTypeChanged() const;
 
+  // Records UMA on whether the NetworkID was available or not. Called right
+  // after a network change event.
+  void RecordNetworkIDAvailability() const;
+
   // Records UMA on main frame requests.
   void RecordMetricsOnMainFrameRequest() const;
 
@@ -417,15 +463,11 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   void OnUpdatedRTTAvailable(SocketPerformanceWatcherFactory::Protocol protocol,
                              const base::TimeDelta& rtt);
 
-  // Obtains operating parameters from the field trial parameters.
-  void ObtainOperatingParams(
-      const std::map<std::string, std::string>& variation_params);
-
   // Obtains the model parameters for different effective connection types from
   // the field trial parameters. For each effective connection type, a model
   // (currently composed of a RTT threshold and a downlink throughput threshold)
   // is provided by the field trial.
-  void ObtainEffectiveConnectionTypeModelParams(
+  void ObtainOperatingParams(
       const std::map<std::string, std::string>& variation_params);
 
   // Adds the default median RTT and downstream throughput estimate for the
@@ -452,13 +494,14 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
 
   // Returns the current network ID checking by calling the platform APIs.
   // Virtualized for testing.
-  virtual NetworkID GetCurrentNetworkID() const;
-
-  // Writes the estimated quality of the current network to the cache.
-  void CacheNetworkQualityEstimate();
+  virtual nqe::internal::NetworkID GetCurrentNetworkID() const;
 
+  // Notifies RTT observers of |observation|. May also trigger recomputation
+  // of effective connection type.
   void NotifyObserversOfRTT(const RttObservation& observation);
 
+  // Notifies throughput observers of |observation|. May also trigger
+  // recomputation of effective connection type.
   void NotifyObserversOfThroughput(const ThroughputObservation& observation);
 
   // Returns true only if the |request| can be used for RTT estimation.
@@ -466,24 +509,52 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
 
   // Recomputes effective connection type, if it was computed more than the
   // specified duration ago, or if there has been a connection change recently.
-  void MaybeRecomputeEffectiveConnectionType();
+  void MaybeComputeEffectiveConnectionType();
 
-  // Notify observers of a change in effective connection type.
+  // Notifies observers of a change in effective connection type.
   void NotifyObserversOfEffectiveConnectionTypeChanged();
 
+  // Notifies the observers of RTT or throughput estimates computation.
+  void NotifyObserversOfRTTOrThroughputComputed() const;
+
+  // Notifies |observer| of the current effective connection type if |observer|
+  // is still registered as an observer.
+  void NotifyEffectiveConnectionTypeObserverIfPresent(
+      EffectiveConnectionTypeObserver* observer) const;
+
+  // Notifies |observer| of the current RTT and throughput if |observer| is
+  // still registered as an observer.
+  void NotifyRTTAndThroughputEstimatesObserverIfPresent(
+      RTTAndThroughputEstimatesObserver* observer) const;
+
   // Records NQE accuracy metrics. |measuring_duration| should belong to the
   // vector returned by AccuracyRecordingIntervals().
   // RecordAccuracyAfterMainFrame should be called |measuring_duration| after a
   // main frame request is observed.
   void RecordAccuracyAfterMainFrame(base::TimeDelta measuring_duration) const;
 
+  // Obtains the current cellular signal strength value and updates
+  // |min_signal_strength_since_connection_change_| and
+  // |max_signal_strength_since_connection_change_|.
+  void UpdateSignalStrength();
+
   // Returns the effective type of the current connection based on only the
-  // samples observed after |start_time|. Uses HTTP RTT and downstream
-  // throughput to compute the effective connection type, and requires both of
-  // them to have a valid value.
-  EffectiveConnectionType
-  GetRecentEffectiveConnectionTypeHttpRTTAndDownstreamThroughput(
-      const base::TimeTicks& start_time) const;
+  // samples observed after |start_time|. May use HTTP RTT, transport RTT and
+  // downstream throughput to compute the effective connection type based on
+  // |http_rtt_metric|, |transport_rtt_metric| and
+  // |downstream_throughput_kbps_metric|, respectively. |http_rtt|,
+  // |transport_rtt| and |downstream_throughput_kbps| must be non-null.
+  // |http_rtt|, |transport_rtt| and |downstream_throughput_kbps| are
+  // set to the expected HTTP RTT, transport RTT and downstream throughput (in
+  // kilobits per second) based on observations taken since |start_time|.
+  EffectiveConnectionType GetRecentEffectiveConnectionTypeUsingMetrics(
+      const base::TimeTicks& start_time,
+      MetricUsage http_rtt_metric,
+      MetricUsage transport_rtt_metric,
+      MetricUsage downstream_throughput_kbps_metric,
+      base::TimeDelta* http_rtt,
+      base::TimeDelta* transport_rtt,
+      int32_t* downstream_throughput_kbps) const;
 
   // Values of external estimate provider status. This enum must remain
   // synchronized with the enum of the same name in
@@ -503,6 +574,29 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   void RecordExternalEstimateProviderMetrics(
       NQEExternalEstimateProviderStatus status) const;
 
+  // Returns true if the cached network quality estimate was successfully read.
+  bool ReadCachedNetworkQualityEstimate();
+
+  // Records a correlation metric that can be used for computing the correlation
+  // between HTTP-layer RTT, transport-layer RTT, throughput and the time
+  // taken to complete |request|.
+  void RecordCorrelationMetric(const URLRequest& request, int net_error) const;
+
+  // Returns true if transport RTT should be used for computing the effective
+  // connection type.
+  bool UseTransportRTT() const;
+
+  // Forces computation of effective connection type, and notifies observers
+  // if there is a change in its value.
+  void ComputeEffectiveConnectionType();
+
+  // May update the network quality of the current network if |network_id|
+  // matches the ID of the current network. |cached_network_quality| is the
+  // cached network quality of the network with id |network_id|.
+  void MaybeUpdateNetworkQualityFromCache(
+      const nqe::internal::NetworkID& network_id,
+      const nqe::internal::CachedNetworkQuality& cached_network_quality);
+
   // Determines if the requests to local host can be used in estimating the
   // network quality. Set to true only for tests.
   bool use_localhost_requests_;
@@ -512,9 +606,21 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // network quality. Set to true only for tests.
   bool use_small_responses_;
 
+  // When set to true, the device offline check is disabled when computing the
+  // effective connection type or when writing the prefs.
+  bool disable_offline_check_;
+
+  // If true, default values provided by the platform are used for estimation.
+  const bool add_default_platform_observations_;
+
   // The factor by which the weight of an observation reduces every second.
   const double weight_multiplier_per_second_;
 
+  // The factor by which the weight of an observation reduces for every dBm
+  // difference between the current signal strength (in dBm), and the signal
+  // strength at the time when the observation was taken.
+  const double weight_multiplier_per_dbm_;
+
   // Algorithm to use for computing effective connection type. The value is
   // obtained from field trial parameters. If the value from field trial
   // parameters is unavailable, it is set to
@@ -524,13 +630,6 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // Tick clock used by the network quality estimator.
   std::unique_ptr<base::TickClock> tick_clock_;
 
-  // Minimum duration between two consecutive computations of effective
-  // connection type. Set to non-zero value as a performance optimization.
-  const base::TimeDelta effective_connection_type_recomputation_interval_;
-
-  // Time when the effective connection type was last computed.
-  base::TimeTicks last_effective_connection_type_computation_;
-
   // Intervals after the main frame request arrives at which accuracy of network
   // quality prediction is recorded.
   std::vector<base::TimeDelta> accuracy_recording_intervals_;
@@ -539,7 +638,7 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   base::TimeTicks last_connection_change_;
 
   // ID of the current network.
-  NetworkID current_network_id_;
+  nqe::internal::NetworkID current_network_id_;
 
   // Peak network quality (fastest round-trip-time (RTT) and highest
   // downstream throughput) measured since last connectivity change. RTT is
@@ -549,9 +648,6 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // 2) Includes server processing time.
   nqe::internal::NetworkQuality peak_network_quality_;
 
-  // Cache that stores quality of previously seen networks.
-  CachedNetworkQualities cached_network_qualities_;
-
   // Buffer that holds throughput observations (in kilobits per second) sorted
   // by timestamp.
   ThroughputObservationBuffer downstream_throughput_kbps_observations_;
@@ -569,17 +665,25 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // trial variation params. These thresholds encode how different connection
   // types behave in general. In future, complex encodings (e.g., curve
   // fitting) may be used.
-  nqe::internal::NetworkQuality
-      connection_thresholds_[EFFECTIVE_CONNECTION_TYPE_LAST];
+  nqe::internal::NetworkQuality connection_thresholds_
+      [EffectiveConnectionType::EFFECTIVE_CONNECTION_TYPE_LAST];
 
-  // Latest time when the headers for a main frame request were received.
+  // Typical network quality for different effective connection types.
+  nqe::internal::NetworkQuality typical_network_quality_
+      [EffectiveConnectionType::EFFECTIVE_CONNECTION_TYPE_LAST];
+
+  // Time when the transaction for the last main frame request was started.
   base::TimeTicks last_main_frame_request_;
 
-  // Estimated network quality when the response headers for the last mainframe
-  // request were received.
+  // Estimated network quality when the transaction for the last main frame
+  // request was started.
   nqe::internal::NetworkQuality estimated_quality_at_last_main_frame_;
   EffectiveConnectionType effective_connection_type_at_last_main_frame_;
 
+  // Estimated network quality obtained from external estimate provider when the
+  // external estimate provider was last queried.
+  nqe::internal::NetworkQuality external_estimate_provider_quality_;
+
   // ExternalEstimateProvider that provides network quality using operating
   // system APIs. May be NULL.
   const std::unique_ptr<ExternalEstimateProvider> external_estimate_provider_;
@@ -588,6 +692,10 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   base::ObserverList<EffectiveConnectionTypeObserver>
       effective_connection_type_observer_list_;
 
+  // Observer list for RTT or throughput estimates.
+  base::ObserverList<RTTAndThroughputEstimatesObserver>
+      rtt_and_throughput_estimates_observer_list_;
+
   // Observer lists for round trip times and throughput measurements.
   base::ObserverList<RTTObserver> rtt_observer_list_;
   base::ObserverList<ThroughputObserver> throughput_observer_list_;
@@ -600,12 +708,56 @@ class NET_EXPORT_PRIVATE NetworkQualityEstimator
   // estimating the throughput.
   std::unique_ptr<nqe::internal::ThroughputAnalyzer> throughput_analyzer_;
 
+  // Minimum duration between two consecutive computations of effective
+  // connection type. Set to non-zero value as a performance optimization.
+  const base::TimeDelta effective_connection_type_recomputation_interval_;
+
+  // Time when the effective connection type was last computed.
+  base::TimeTicks last_effective_connection_type_computation_;
+
+  // Number of RTT and bandwidth samples available when effective connection
+  // type was last recomputed.
+  size_t rtt_observations_size_at_last_ect_computation_;
+  size_t throughput_observations_size_at_last_ect_computation_;
+
+  // Current estimate of the network quality.
+  nqe::internal::NetworkQuality network_quality_;
+
   // Current effective connection type. It is updated on connection change
   // events. It is also updated every time there is network traffic (provided
   // the last computation was more than
   // |effective_connection_type_recomputation_interval_| ago).
   EffectiveConnectionType effective_connection_type_;
 
+  // Last known value of the wireless signal strength. Set to INT32_MIN if
+  // unavailable. |signal_strength_dbm_| is reset to INT32_MIN on connection
+  // change events.
+  int32_t signal_strength_dbm_;
+
+  // Minimum and maximum signal strength (in dBm) observed since last connection
+  // change. Updated on connection change and main frame requests.
+  int32_t min_signal_strength_since_connection_change_;
+  int32_t max_signal_strength_since_connection_change_;
+
+  // It is costlier to add values to a sparse histogram. So, the correlation UMA
+  // is recorded with |correlation_uma_logging_probability_| since recording it
+  // in a sparse histogram for each request is unnecessary and cost-prohibitive.
+  // e.g., if it is 0.0, then the UMA will never be recorded. On the other hand,
+  // if it is 1.0, then it will be recorded for all valid HTTP requests.
+  const double correlation_uma_logging_probability_;
+
+  // Stores the qualities of different networks.
+  std::unique_ptr<nqe::internal::NetworkQualityStore> network_quality_store_;
+
+  // True if effective connection type value has been forced via variation
+  // parameters. If set to true, GetEffectiveConnectionType() will always return
+  // |forced_effective_connection_type_|.
+  const bool forced_effective_connection_type_set_;
+  const EffectiveConnectionType forced_effective_connection_type_;
+
+  // Set to true if reading of the network quality prefs is enabled.
+  const bool persistent_cache_reading_enabled_;
+
   base::ThreadChecker thread_checker_;
 
   base::WeakPtrFactory<NetworkQualityEstimator> weak_ptr_factory_;
