@@ -7,14 +7,22 @@
 #include <utility>
 
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/stl_util.h"
+#include "base/strings/stringprintf.h"
+#include "base/trace_event/memory_allocator_dump.h"
+#include "base/trace_event/process_memory_dump.h"
 #include "base/trace_event/trace_event.h"
 #include "base/values.h"
 #include "net/base/address_list.h"
+#include "net/base/trace_constants.h"
 #include "net/http/http_network_session.h"
 #include "net/http/http_server_properties.h"
+#include "net/log/net_log_event_type.h"
+#include "net/log/net_log_source.h"
+#include "net/log/net_log_with_source.h"
 #include "net/spdy/spdy_session.h"
 
 namespace net {
@@ -37,31 +45,22 @@ SpdySessionPool::SpdySessionPool(
     HttpServerProperties* http_server_properties,
     TransportSecurityState* transport_security_state,
     bool enable_ping_based_connection_checking,
-    bool enable_priority_dependencies,
-    NextProto default_protocol,
     size_t session_max_recv_window_size,
-    size_t stream_max_recv_window_size,
+    const SettingsMap& initial_settings,
     SpdySessionPool::TimeFunc time_func,
     ProxyDelegate* proxy_delegate)
     : http_server_properties_(http_server_properties),
       transport_security_state_(transport_security_state),
       ssl_config_service_(ssl_config_service),
       resolver_(resolver),
-      verify_domain_authentication_(true),
       enable_sending_initial_data_(true),
       enable_ping_based_connection_checking_(
           enable_ping_based_connection_checking),
-      enable_priority_dependencies_(enable_priority_dependencies),
-      // TODO(akalin): Force callers to have a valid value of
-      // |default_protocol_|.
-      default_protocol_((default_protocol == kProtoUnknown) ? kProtoSPDY31
-                                                            : default_protocol),
       session_max_recv_window_size_(session_max_recv_window_size),
-      stream_max_recv_window_size_(stream_max_recv_window_size),
+      initial_settings_(initial_settings),
       time_func_(time_func),
+      push_delegate_(nullptr),
       proxy_delegate_(proxy_delegate) {
-  DCHECK(default_protocol_ >= kProtoSPDYMinimumVersion &&
-         default_protocol_ <= kProtoSPDYMaximumVersion);
   NetworkChangeNotifier::AddIPAddressObserver(this);
   if (ssl_config_service_.get())
     ssl_config_service_->AddObserver(this);
@@ -86,33 +85,28 @@ SpdySessionPool::~SpdySessionPool() {
 base::WeakPtr<SpdySession> SpdySessionPool::CreateAvailableSessionFromSocket(
     const SpdySessionKey& key,
     std::unique_ptr<ClientSocketHandle> connection,
-    const BoundNetLog& net_log,
-    int certificate_error_code,
+    const NetLogWithSource& net_log,
     bool is_secure) {
-  TRACE_EVENT0("net", "SpdySessionPool::CreateAvailableSessionFromSocket");
-  DCHECK_GE(default_protocol_, kProtoSPDYMinimumVersion);
-  DCHECK_LE(default_protocol_, kProtoSPDYMaximumVersion);
+  TRACE_EVENT0(kNetTracingCategory,
+               "SpdySessionPool::CreateAvailableSessionFromSocket");
 
   UMA_HISTOGRAM_ENUMERATION(
       "Net.SpdySessionGet", IMPORTED_FROM_SOCKET, SPDY_SESSION_GET_MAX);
 
-  std::unique_ptr<SpdySession> new_session(new SpdySession(
+  auto new_session = base::MakeUnique<SpdySession>(
       key, http_server_properties_, transport_security_state_,
-      verify_domain_authentication_, enable_sending_initial_data_,
-      enable_ping_based_connection_checking_, enable_priority_dependencies_,
-      default_protocol_, session_max_recv_window_size_,
-      stream_max_recv_window_size_, time_func_, proxy_delegate_,
-      net_log.net_log()));
+      enable_sending_initial_data_, enable_ping_based_connection_checking_,
+      session_max_recv_window_size_, initial_settings_, time_func_,
+      push_delegate_, proxy_delegate_, net_log.net_log());
 
-  new_session->InitializeWithSocket(std::move(connection), this, is_secure,
-                                    certificate_error_code);
+  new_session->InitializeWithSocket(std::move(connection), this, is_secure);
 
   base::WeakPtr<SpdySession> available_session = new_session->GetWeakPtr();
   sessions_.insert(new_session.release());
   MapKeyToAvailableSession(key, available_session);
 
   net_log.AddEvent(
-      NetLog::TYPE_HTTP2_SESSION_POOL_IMPORTED_SESSION_FROM_SOCKET,
+      NetLogEventType::HTTP2_SESSION_POOL_IMPORTED_SESSION_FROM_SOCKET,
       available_session->net_log().source().ToEventParametersCallback());
 
   // Look up the IP address for this session so that we can match
@@ -132,7 +126,7 @@ base::WeakPtr<SpdySession> SpdySessionPool::CreateAvailableSessionFromSocket(
 base::WeakPtr<SpdySession> SpdySessionPool::FindAvailableSession(
     const SpdySessionKey& key,
     const GURL& url,
-    const BoundNetLog& net_log) {
+    const NetLogWithSource& net_log) {
   UnclaimedPushedStreamMap::iterator url_it =
       unclaimed_pushed_streams_.find(url);
   if (!url.is_empty() && url_it != unclaimed_pushed_streams_.end()) {
@@ -167,7 +161,7 @@ base::WeakPtr<SpdySession> SpdySessionPool::FindAvailableSession(
     UMA_HISTOGRAM_ENUMERATION(
         "Net.SpdySessionGet", FOUND_EXISTING, SPDY_SESSION_GET_MAX);
     net_log.AddEvent(
-        NetLog::TYPE_HTTP2_SESSION_POOL_FOUND_EXISTING_SESSION,
+        NetLogEventType::HTTP2_SESSION_POOL_FOUND_EXISTING_SESSION,
         it->second->net_log().source().ToEventParametersCallback());
     return it->second;
   }
@@ -206,7 +200,7 @@ base::WeakPtr<SpdySession> SpdySessionPool::FindAvailableSession(
 
     const base::WeakPtr<SpdySession>& available_session =
         available_session_it->second;
-    DCHECK(ContainsKey(sessions_, available_session.get()));
+    DCHECK(base::ContainsKey(sessions_, available_session.get()));
     // If the session is a secure one, we need to verify that the
     // server is authenticated to serve traffic for |host_port_proxy_pair| too.
     if (!available_session->VerifyDomainAuthentication(
@@ -220,7 +214,7 @@ base::WeakPtr<SpdySession> SpdySessionPool::FindAvailableSession(
                               FOUND_EXISTING_FROM_IP_POOL,
                               SPDY_SESSION_GET_MAX);
     net_log.AddEvent(
-        NetLog::TYPE_HTTP2_SESSION_POOL_FOUND_EXISTING_SESSION_FROM_IP_POOL,
+        NetLogEventType::HTTP2_SESSION_POOL_FOUND_EXISTING_SESSION_FROM_IP_POOL,
         available_session->net_log().source().ToEventParametersCallback());
     // Add this session to the map so that we can find it next time.
     MapKeyToAvailableSession(key, available_session);
@@ -249,7 +243,7 @@ void SpdySessionPool::RemoveUnavailableSession(
   DCHECK(!IsSessionAvailable(unavailable_session));
 
   unavailable_session->net_log().AddEvent(
-      NetLog::TYPE_HTTP2_SESSION_POOL_REMOVE_SESSION,
+      NetLogEventType::HTTP2_SESSION_POOL_REMOVE_SESSION,
       unavailable_session->net_log().source().ToEventParametersCallback());
 
   SessionSet::iterator it = sessions_.find(unavailable_session.get());
@@ -285,7 +279,7 @@ void SpdySessionPool::RegisterUnclaimedPushedStream(
     base::WeakPtr<SpdySession> spdy_session) {
   DCHECK(!url.is_empty());
   // This SpdySessionPool  must own |spdy_session|.
-  DCHECK(ContainsKey(sessions_, spdy_session.get()));
+  DCHECK(base::ContainsKey(sessions_, spdy_session.get()));
   UnclaimedPushedStreamMap::iterator url_it =
       unclaimed_pushed_streams_.lower_bound(url);
   if (url_it == unclaimed_pushed_streams_.end() || url_it->first != url) {
@@ -364,23 +358,57 @@ void SpdySessionPool::OnIPAddressChanged() {
 #endif  // defined(OS_ANDROID) || defined(OS_WIN) || defined(OS_IOS)
     DCHECK(!IsSessionAvailable(*it));
   }
-  http_server_properties_->ClearAllSpdySettings();
 }
 
 void SpdySessionPool::OnSSLConfigChanged() {
   CloseCurrentSessions(ERR_NETWORK_CHANGED);
 }
 
-void SpdySessionPool::OnCertAdded(const X509Certificate* cert) {
+void SpdySessionPool::OnCertDBChanged(const X509Certificate* cert) {
   CloseCurrentSessions(ERR_CERT_DATABASE_CHANGED);
 }
 
-void SpdySessionPool::OnCACertChanged(const X509Certificate* cert) {
-  // Per wtc, we actually only need to CloseCurrentSessions when trust is
-  // reduced. CloseCurrentSessions now because OnCACertChanged does not
-  // tell us this.
-  // See comments in ClientSocketPoolManager::OnCACertChanged.
-  CloseCurrentSessions(ERR_CERT_DATABASE_CHANGED);
+void SpdySessionPool::DumpMemoryStats(
+    base::trace_event::ProcessMemoryDump* pmd,
+    const std::string& parent_dump_absolute_name) const {
+  std::string dump_name = base::StringPrintf("%s/spdy_session_pool",
+                                             parent_dump_absolute_name.c_str());
+  base::trace_event::MemoryAllocatorDump* dump =
+      pmd->CreateAllocatorDump(dump_name);
+  size_t total_size = 0;
+  size_t buffer_size = 0;
+  size_t cert_count = 0;
+  size_t serialized_cert_size = 0;
+  size_t num_active_sessions = 0;
+  for (const auto& session : sessions_) {
+    StreamSocket::SocketMemoryStats stats;
+    bool is_session_active = false;
+    session->DumpMemoryStats(&stats, &is_session_active);
+    total_size += stats.total_size;
+    buffer_size += stats.buffer_size;
+    cert_count += stats.cert_count;
+    serialized_cert_size += stats.serialized_cert_size;
+    if (is_session_active)
+      num_active_sessions++;
+  }
+  dump->AddScalar(base::trace_event::MemoryAllocatorDump::kNameSize,
+                  base::trace_event::MemoryAllocatorDump::kUnitsBytes,
+                  total_size);
+  dump->AddScalar(base::trace_event::MemoryAllocatorDump::kNameObjectCount,
+                  base::trace_event::MemoryAllocatorDump::kUnitsObjects,
+                  sessions_.size());
+  dump->AddScalar("active_session_count",
+                  base::trace_event::MemoryAllocatorDump::kUnitsObjects,
+                  num_active_sessions);
+  dump->AddScalar("buffer_size",
+                  base::trace_event::MemoryAllocatorDump::kUnitsBytes,
+                  buffer_size);
+  dump->AddScalar("cert_count",
+                  base::trace_event::MemoryAllocatorDump::kUnitsObjects,
+                  cert_count);
+  dump->AddScalar("serialized_cert_size",
+                  base::trace_event::MemoryAllocatorDump::kUnitsBytes,
+                  serialized_cert_size);
 }
 
 bool SpdySessionPool::IsSessionAvailable(
@@ -396,7 +424,7 @@ bool SpdySessionPool::IsSessionAvailable(
 void SpdySessionPool::MapKeyToAvailableSession(
     const SpdySessionKey& key,
     const base::WeakPtr<SpdySession>& session) {
-  DCHECK(ContainsKey(sessions_, session.get()));
+  DCHECK(base::ContainsKey(sessions_, session.get()));
   std::pair<AvailableSessionMap::iterator, bool> result =
       available_sessions_.insert(std::make_pair(key, session));
   CHECK(result.second);
