@@ -7,12 +7,10 @@
 #include <errno.h>
 #include <sys/socket.h>
 
-#include "base/logging.h"
 #include "base/stl_util.h"
-#include "net/base/ip_endpoint.h"
-#include "net/quic/crypto/quic_random.h"
+#include "net/quic/core/crypto/quic_random.h"
 #include "net/tools/epoll_server/epoll_server.h"
-#include "net/tools/quic/quic_socket_utils.h"
+#include "net/tools/quic/platform/impl/quic_socket_utils.h"
 
 namespace net {
 
