@@ -18,7 +18,7 @@ QuicConnectionId QuicSimpleServerSessionHelper::GenerateConnectionIdForReject(
 
 bool QuicSimpleServerSessionHelper::CanAcceptClientHello(
     const CryptoHandshakeMessage& message,
-    const IPEndPoint& self_address,
+    const QuicSocketAddress& self_address,
     std::string* error_details) const {
   return true;
 }
