@@ -20,7 +20,7 @@
 
 #include <QtGlobal>
 
-#include "qt/core/browser/oxide_qt_web_preferences.h"
+#include "qt/core/browser/web_preferences.h"
 
 class OxideQWebPreferences;
 
@@ -28,14 +28,20 @@ QT_BEGIN_NAMESPACE
 class QObject;
 QT_END_NAMESPACE
 
-class OxideQWebPreferencesPrivate final {
+namespace oxide {
+class WebPreferences;
+}
+
+class OxideQWebPreferencesPrivate {
  public:
   ~OxideQWebPreferencesPrivate();
 
-  oxide::qt::WebPreferences* preferences() { return &preferences_; }
-
   static OxideQWebPreferencesPrivate* get(OxideQWebPreferences* q);
 
+  oxide::WebPreferences* GetPrefs() const;
+
+  void AdoptPrefs(oxide::WebPreferences* prefs);
+
  private:
   friend class OxideQWebPreferences;
   OxideQWebPreferencesPrivate(OxideQWebPreferences* q);
