@@ -18,6 +18,7 @@
 #include <QtGlobal>
 
 #include "ui/events/keycodes/dom/dom_key.h"
+#include "ui/events/keycodes/keyboard_codes.h"
 
 QT_BEGIN_NAMESPACE
 class QKeyEvent;
@@ -26,6 +27,8 @@ QT_END_NAMESPACE
 namespace oxide {
 namespace qt {
 
+ui::KeyboardCode GetKeyboardCodeFromQKeyEvent(QKeyEvent* event);
+
 ui::DomKey GetDomKeyFromQKeyEvent(QKeyEvent* event);
 
 } // namespace qt
