@@ -26,8 +26,10 @@
 #include <Qt>
 #include <QWheelEvent>
 
+#include "base/environment.h"
 #include "base/logging.h"
 #include "base/time/time.h"
+#include "third_party/WebKit/public/platform/WebPointerProperties.h"
 #include "ui/events/keycodes/dom/keycode_converter.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/gfx/geometry/point_f.h"
@@ -88,7 +90,15 @@ blink::WebUChar GetControlCharacter(int key_code, bool shift) {
 }
 
 double QInputEventTimeToWebEventTime(QInputEvent* qevent) {
-  if (getenv("OXIDE_TESTING_MODE")) {
+  static bool g_testing_mode = []() {
+    std::unique_ptr<base::Environment> env = base::Environment::Create();
+    std::string result;
+    return env->GetVar("OXIDE_TESTING_MODE", &result) &&
+               result.size() > 0 &&
+               result[0] != '0';
+  }();
+
+  if (g_testing_mode) {
     // We don't have timestamps in testing
     return base::TimeDelta(base::TimeTicks::Now() - base::TimeTicks())
         .InSecondsF();
@@ -120,305 +130,6 @@ int QInputEventStateToWebEventModifiers(QInputEvent* qevent) {
   return modifiers;
 }
 
-int QKeyEventKeyCodeToWebEventKeyCode(QKeyEvent* qevent) {
-  int qkeycode = qevent->key();
-
-  // 1) Keypad
-  if (qevent->modifiers() & Qt::KeypadModifier) {
-    if (qkeycode >= Qt::Key_0 && qkeycode <= Qt::Key_9) {
-      return (qkeycode - Qt::Key_0) + ui::VKEY_NUMPAD0;
-    }
-
-    switch (qkeycode) {
-    case Qt::Key_Enter:
-    case Qt::Key_Return:
-      return ui::VKEY_RETURN;
-    case Qt::Key_PageUp:
-      return ui::VKEY_PRIOR;
-    case Qt::Key_PageDown:
-      return ui::VKEY_NEXT;
-    case Qt::Key_End:
-      return ui::VKEY_END;
-    case Qt::Key_Home:
-      return ui::VKEY_HOME;
-    case Qt::Key_Left:
-      return ui::VKEY_LEFT;
-    case Qt::Key_Up:
-      return ui::VKEY_UP;
-    case Qt::Key_Right:
-      return ui::VKEY_RIGHT;
-    case Qt::Key_Down:
-      return ui::VKEY_DOWN;
-    case Qt::Key_Asterisk:
-      return ui::VKEY_MULTIPLY;
-    case Qt::Key_Plus:
-      return ui::VKEY_ADD;
-    case Qt::Key_Minus:
-      return ui::VKEY_SUBTRACT;
-    case Qt::Key_Period:
-      return ui::VKEY_DECIMAL;
-    case Qt::Key_Slash:
-      return ui::VKEY_DIVIDE;
-    case Qt::Key_Insert:
-      return ui::VKEY_INSERT;
-    case Qt::Key_Delete:
-      return ui::VKEY_DELETE;
-    default:
-      return 0;
-    }
-  }
-
-  // 2) VKEY_A - VKEY_Z
-  if (qkeycode >= Qt::Key_A && qkeycode <= Qt::Key_Z) {
-    return (qkeycode - Qt::Key_A) + ui::VKEY_A;
-  }
-
-  // 3) VKEY_0 - VKEY_9
-  if (qkeycode >= Qt::Key_0 && qkeycode <= Qt::Key_9) {
-    return (qkeycode - Qt::Key_0) + ui::VKEY_0;
-  }
-  switch (qkeycode) {
-  case Qt::Key_ParenRight:
-    return ui::VKEY_0;
-  case Qt::Key_Exclam:
-    return ui::VKEY_1;
-  case Qt::Key_At:
-    return ui::VKEY_2;
-  case Qt::Key_NumberSign:
-    return ui::VKEY_3;
-  case Qt::Key_Dollar:
-    return ui::VKEY_4;
-  case Qt::Key_Percent:
-    return ui::VKEY_5;
-  case Qt::Key_AsciiCircum:
-    return ui::VKEY_6;
-  case Qt::Key_Ampersand:
-    return ui::VKEY_7;
-  case Qt::Key_Asterisk:
-    return ui::VKEY_8;
-  case Qt::Key_ParenLeft:
-    return ui::VKEY_9;
-  default:
-    break;
-  }
-
-  // 4) VKEY_F1 - VKEY_F24
-  if (qkeycode >= Qt::Key_F1 && qkeycode <= Qt::Key_F24) {
-    // We miss Qt::Key_F25 - Qt::Key_F35
-    return (qkeycode - Qt::Key_F1) + ui::VKEY_F1;
-  }
-
-  switch (qkeycode) {
-  case Qt::Key_Backspace:
-    return ui::VKEY_BACK;
-  case Qt::Key_Tab:
-  case Qt::Key_Backtab:
-    return ui::VKEY_TAB;
-  // VKEY_BACKTAB - not used in Chromium X11
-  case Qt::Key_Clear:
-    return ui::VKEY_CLEAR;
-  case Qt::Key_Return:
-  case Qt::Key_Enter:
-    return ui::VKEY_RETURN;
-  case Qt::Key_Shift:
-    return ui::VKEY_SHIFT;
-  case Qt::Key_Control:
-    return ui::VKEY_CONTROL;
-  case Qt::Key_Meta:
-    return ui::VKEY_MENU;
-  case Qt::Key_Pause:
-    return ui::VKEY_PAUSE;
-  case Qt::Key_CapsLock:
-    return ui::VKEY_CAPITAL;
-  case Qt::Key_Kana_Lock:
-    return ui::VKEY_KANA;
-  case Qt::Key_Hangul:
-    return ui::VKEY_HANGUL;
-  // VKEY_JUNJA
-  // VKEY_FINAL
-  case Qt::Key_Hangul_Hanja:
-    return ui::VKEY_HANJA;
-  case Qt::Key_Kanji:
-    return ui::VKEY_KANJI;
-  case Qt::Key_Escape:
-    return ui::VKEY_ESCAPE;
-  case Qt::Key_Henkan:
-    return ui::VKEY_CONVERT;
-  case Qt::Key_Muhenkan:
-    return ui::VKEY_NONCONVERT;
-  // VKEY_ACCEPT
-  // VKEY_MODECHANGE
-  case Qt::Key_Space:
-    return ui::VKEY_SPACE;
-  case Qt::Key_PageUp:
-    return ui::VKEY_PRIOR;
-  case Qt::Key_PageDown:
-    return ui::VKEY_NEXT;
-  case Qt::Key_End:
-    return ui::VKEY_END;
-  case Qt::Key_Home:
-    return ui::VKEY_HOME;
-  case Qt::Key_Left:
-    return ui::VKEY_LEFT;
-  case Qt::Key_Up:
-    return ui::VKEY_UP;
-  case Qt::Key_Right:
-    return ui::VKEY_RIGHT;
-  case Qt::Key_Down:
-    return ui::VKEY_DOWN;
-  case Qt::Key_Select:
-    return ui::VKEY_SELECT;
-  case Qt::Key_Print:
-    return ui::VKEY_PRINT;
-  case Qt::Key_Execute:
-    return ui::VKEY_EXECUTE;
-  // VKEY_SNAPSHOT
-  case Qt::Key_Insert:
-    return ui::VKEY_INSERT;
-  case Qt::Key_Delete:
-    return ui::VKEY_DELETE;
-  case Qt::Key_Help:
-    return ui::VKEY_HELP;
-  // VKEY_0 - VKEY_Z handled above
-  case Qt::Key_Super_L:
-    return ui::VKEY_LWIN;
-  case Qt::Key_Super_R:
-    return ui::VKEY_RWIN;
-  case Qt::Key_Menu:
-    return ui::VKEY_APPS;
-  // VKEY_SLEEP
-  // VKEY_NUMPAD0 - VKEY_NUMPAD9 handled in keypad section
-  case Qt::Key_multiply:
-    return ui::VKEY_MULTIPLY;
-  // VKEY_ADD handled in keypad section
-  // VKEY_SEPARATOR
-  // VKEY_SUBTRACT handled in keypad section
-  // VKEY_DECIMAL handled in keypad section
-  // VKEY_DIVIDE handled in keypad section
-  // VKEY_F1 - VKEY_F24 handled above
-  case Qt::Key_NumLock:
-    return ui::VKEY_NUMLOCK;
-  case Qt::Key_ScrollLock:
-    return ui::VKEY_SCROLL;
-  // VKEY_LSHIFT
-  // VKEY_RSHIFT
-  // VKEY_LCONTROL
-  // VKEY_RCONTROL
-  case Qt::Key_Alt:
-    return ui::VKEY_LMENU;
-  // VKEY_RMENU
-  case Qt::Key_Back:
-    return ui::VKEY_BROWSER_BACK;
-  case Qt::Key_Forward:
-    return ui::VKEY_BROWSER_FORWARD;
-  case Qt::Key_Refresh:
-    return ui::VKEY_BROWSER_REFRESH;
-  case Qt::Key_Stop:
-    return ui::VKEY_BROWSER_STOP;
-  case Qt::Key_Search:
-    return ui::VKEY_BROWSER_SEARCH;
-  case Qt::Key_Favorites:
-    return ui::VKEY_BROWSER_FAVORITES;
-  case Qt::Key_HomePage:
-    return ui::VKEY_BROWSER_HOME;
-  case Qt::Key_VolumeMute:
-    return ui::VKEY_VOLUME_MUTE;
-  case Qt::Key_VolumeDown:
-    return ui::VKEY_VOLUME_DOWN;
-  case Qt::Key_VolumeUp:
-    return ui::VKEY_VOLUME_UP;
-  case Qt::Key_MediaNext:
-    return ui::VKEY_MEDIA_NEXT_TRACK;
-  case Qt::Key_MediaPrevious:
-    return ui::VKEY_MEDIA_PREV_TRACK;
-  case Qt::Key_MediaStop:
-    return ui::VKEY_MEDIA_STOP;
-  case Qt::Key_MediaPlay:
-  case Qt::Key_MediaPause:
-  case Qt::Key_MediaTogglePlayPause:
-    return ui::VKEY_MEDIA_PLAY_PAUSE;
-  case Qt::Key_LaunchMail:
-    return ui::VKEY_MEDIA_LAUNCH_MAIL;
-  case Qt::Key_LaunchMedia:
-    return ui::VKEY_MEDIA_LAUNCH_MEDIA_SELECT;
-  case Qt::Key_Launch0:
-    return ui::VKEY_MEDIA_LAUNCH_APP1;
-  case Qt::Key_Launch1:
-    return ui::VKEY_MEDIA_LAUNCH_APP2;
-  case Qt::Key_Colon:
-  case Qt::Key_Semicolon:
-    return ui::VKEY_OEM_1;
-  case Qt::Key_Plus:
-  case Qt::Key_Equal:
-    return ui::VKEY_OEM_PLUS;
-  case Qt::Key_Comma:
-  case Qt::Key_Less:
-    return ui::VKEY_OEM_COMMA;
-  case Qt::Key_Minus:
-  case Qt::Key_Underscore:
-    return ui::VKEY_OEM_MINUS;
-  case Qt::Key_Period:
-  case Qt::Key_Greater:
-    return ui::VKEY_OEM_PERIOD;
-  case Qt::Key_Slash:
-  case Qt::Key_Question:
-    return ui::VKEY_OEM_2;
-  case Qt::Key_QuoteLeft:
-  case Qt::Key_AsciiTilde:
-    return ui::VKEY_OEM_3;
-  case Qt::Key_BracketLeft:
-  case Qt::Key_BraceLeft:
-    return ui::VKEY_OEM_4;
-  case Qt::Key_Backslash:
-  case Qt::Key_Bar:
-    return ui::VKEY_OEM_5;
-  case Qt::Key_BracketRight:
-  case Qt::Key_BraceRight:
-    return ui::VKEY_OEM_6;
-  case Qt::Key_QuoteDbl:
-  case Qt::Key_Apostrophe:
-    return ui::VKEY_OEM_7;
-  // VKEY_OEM_8
-  case Qt::Key_Ugrave:
-  case Qt::Key_brokenbar:
-    return ui::VKEY_OEM_102;
-  // VKEY_OEM_103
-  // VKEY_OEM_104
-  // VKEY_PROCESSKEY
-  // VKEY_PACKET
-  // VKEY_DBE_SBCSCHAR
-  case Qt::Key_Zenkaku_Hankaku:
-    return ui::VKEY_DBE_DBCSCHAR;
-  // VKEY_ATTN
-  // VKEY_CRSEL
-  // VKEY_EXSEL
-  // VKEY_EREOF
-  // VKEY_PLAY
-  // VKEY_ZOOM
-  // VKEY_NONAME
-  // VKEY_PA1
-  // VKEY_OEM_CLEAR
-  case Qt::Key_WLAN:
-    return ui::VKEY_WLAN;
-  case Qt::Key_PowerOff:
-    return ui::VKEY_POWER;
-  case Qt::Key_MonBrightnessDown:
-    return ui::VKEY_BRIGHTNESS_DOWN;
-  case Qt::Key_MonBrightnessUp:
-    return ui::VKEY_BRIGHTNESS_UP;
-  case Qt::Key_KeyboardBrightnessDown:
-    return ui::VKEY_KBD_BRIGHTNESS_DOWN;
-  case Qt::Key_KeyboardBrightnessUp:
-    return ui::VKEY_KBD_BRIGHTNESS_UP;
-  case Qt::Key_AltGr:
-    return ui::VKEY_ALTGR;
-  default:
-    break;
-  }
-
-  return 0;
-}
-
 int QMouseEventStateToWebEventModifiers(QMouseEvent* qevent) {
   Qt::MouseButtons buttons = qevent->buttons();
 
@@ -439,6 +150,20 @@ int QMouseEventStateToWebEventModifiers(QMouseEvent* qevent) {
   return modifiers;
 }
 
+blink::WebInputEvent::Type QInputEventTypeToWebEventType(QInputEvent* event,
+                                                         bool is_char = false) {
+  switch (event->type()) {
+    case QEvent::KeyPress:
+      return is_char ?
+          blink::WebInputEvent::Char : blink::WebInputEvent::RawKeyDown;
+    case QEvent::KeyRelease:
+      return blink::WebInputEvent::KeyUp;
+    default:
+      NOTREACHED();
+      return blink::WebInputEvent::Undefined;
+  }
+}
+
 void ReleaseKeyEvent(void* event) {
   delete reinterpret_cast<QKeyEvent*>(event);
 }
@@ -451,42 +176,30 @@ void* CopyKeyEvent(void* event) {
 
 content::NativeWebKeyboardEvent MakeNativeWebKeyboardEvent(QKeyEvent* event,
                                                            bool is_char) {
-  content::NativeWebKeyboardEvent result;
+  content::NativeWebKeyboardEvent result(
+      QInputEventTypeToWebEventType(event, is_char),
+      QInputEventStateToWebEventModifiers(event),
+      QInputEventTimeToWebEventTime(event));
 
   QKeyEvent* os_event = new QKeyEvent(*event);
   os_event->setAccepted(false);
   result.SetExtraData(os_event, ReleaseKeyEvent, CopyKeyEvent);
 
-  result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
-  result.modifiers = QInputEventStateToWebEventModifiers(event);
-
   if (event->isAutoRepeat()) {
-    result.modifiers |= blink::WebInputEvent::IsAutoRepeat;
+    result.setModifiers(
+        result.modifiers() | blink::WebInputEvent::IsAutoRepeat);
   }
 
-  switch (event->type()) {
-    case QEvent::KeyPress: {
-      result.type = is_char ?
-          blink::WebInputEvent::Char : blink::WebInputEvent::RawKeyDown;
-      break;
-    }
-    case QEvent::KeyRelease:
-      result.type = blink::WebInputEvent::KeyUp;
-      break;
-    default:
-      NOTREACHED();
-  }
-
-  if (result.modifiers & blink::WebInputEvent::AltKey) {
+  if (result.modifiers() & blink::WebInputEvent::AltKey) {
     result.isSystemKey = true;
   }
 
-  int windowsKeyCode = QKeyEventKeyCodeToWebEventKeyCode(event);
+  int windowsKeyCode = GetKeyboardCodeFromQKeyEvent(event);
   result.windowsKeyCode = oxide::WindowsKeyCodeWithoutLocation(windowsKeyCode);
-  result.modifiers |=
-      oxide::LocationModifiersFromWindowsKeyCode(windowsKeyCode);
+  result.setModifiers(
+      result.modifiers()
+      | oxide::LocationModifiersFromWindowsKeyCode(windowsKeyCode));
   result.nativeKeyCode = event->nativeVirtualKey();
-  result.setKeyIdentifierFromWindowsKeyCode();
 
   result.domCode = static_cast<int>(
       ui::KeycodeConverter::NativeKeycodeToDomCode(event->nativeScanCode()));
@@ -499,10 +212,10 @@ content::NativeWebKeyboardEvent MakeNativeWebKeyboardEvent(QKeyEvent* event,
                 "blink::WebKeyboardEvent::text and "
                 "blink::WebKeyboardEvent::unmodifiedText sizes don't match");
 
-  if (result.modifiers & blink::WebInputEvent::ControlKey) {
+  if (result.modifiers() & blink::WebInputEvent::ControlKey) {
     result.text[0] = GetControlCharacter(
         result.windowsKeyCode,
-        result.modifiers & blink::WebInputEvent::ShiftKey);
+        result.modifiers() & blink::WebInputEvent::ShiftKey);
   } else {
     memcpy(result.text, result.unmodifiedText, sizeof(result.unmodifiedText));
   }
@@ -515,8 +228,9 @@ blink::WebMouseEvent MakeWebMouseEvent(QMouseEvent* event,
                                        float location_bar_content_offset) {
   blink::WebMouseEvent result;
 
-  result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
-  result.modifiers = QMouseEventStateToWebEventModifiers(event);
+  result.setTimeStampSeconds(QInputEventTimeToWebEventTime(event));
+  result.setModifiers(QMouseEventStateToWebEventModifiers(event));
+  result.pointerType = blink::WebPointerProperties::PointerType::Mouse;
 
   gfx::Point pos =
       DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->pos()), screen);
@@ -539,14 +253,14 @@ blink::WebMouseEvent MakeWebMouseEvent(QMouseEvent* event,
 
   switch (event->type()) {
     case QEvent::MouseButtonPress:
-      result.type = blink::WebInputEvent::MouseDown;
+      result.setType(blink::WebInputEvent::MouseDown);
       result.clickCount = 1;
       break;
     case QEvent::MouseButtonRelease:
-      result.type = blink::WebInputEvent::MouseUp;
+      result.setType(blink::WebInputEvent::MouseUp);
       break;
     case QEvent::MouseMove:
-      result.type = blink::WebInputEvent::MouseMove;
+      result.setType(blink::WebInputEvent::MouseMove);
       break;
     default:
       NOTREACHED();
@@ -555,26 +269,26 @@ blink::WebMouseEvent MakeWebMouseEvent(QMouseEvent* event,
   if (event->type() != QEvent::MouseMove) {
     switch(event->button()) {
       case Qt::LeftButton:
-        result.button = blink::WebMouseEvent::ButtonLeft;
+        result.button = blink::WebPointerProperties::Button::Left;
         break;
       case Qt::MidButton:
-        result.button = blink::WebMouseEvent::ButtonMiddle;
+        result.button = blink::WebPointerProperties::Button::Middle;
         break;
       case Qt::RightButton:
-        result.button = blink::WebMouseEvent::ButtonRight;
+        result.button = blink::WebPointerProperties::Button::Right;
         break;
       default:
         NOTREACHED();
     }
   } else {
     if (event->buttons() & Qt::LeftButton) {
-      result.button = blink::WebMouseEvent::ButtonLeft;
+      result.button = blink::WebPointerProperties::Button::Left;
     }
     if (event->buttons() & Qt::MidButton) {
-      result.button = blink::WebMouseEvent::ButtonMiddle;
+      result.button = blink::WebPointerProperties::Button::Middle;
     }
     if (event->buttons() & Qt::RightButton) {
-      result.button = blink::WebMouseEvent::ButtonRight;
+      result.button = blink::WebPointerProperties::Button::Right;
     }
   }
 
@@ -587,7 +301,12 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(QWheelEvent* event,
                                                  float location_bar_content_offset) {
   blink::WebMouseWheelEvent result;
 
-  result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
+  // The timestamp has be referenced to TimeTicks as ui::InputHandlerProxy uses
+  // this on the renderer side to compute a roundtrip delay.
+  result.setTimeStampSeconds(
+      (base::TimeTicks::Now() - base::TimeTicks()).InSecondsF());
+
+  result.pointerType = blink::WebPointerProperties::PointerType::Mouse;
 
   // In Chromium a wheel event is a type of mouse event, but this is not the
   // case in Qt (QWheelEvent is not derived from QMouseEvent). We create a
@@ -598,10 +317,10 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(QWheelEvent* event,
                     Qt::NoButton,
                     event->buttons(),
                     event->modifiers());
-  result.modifiers = QMouseEventStateToWebEventModifiers(&dummy);
+  result.setModifiers(QMouseEventStateToWebEventModifiers(&dummy));
 
-  result.type = blink::WebInputEvent::MouseWheel;
-  result.button = blink::WebMouseEvent::ButtonNone;
+  result.setType(blink::WebInputEvent::MouseWheel);
+  result.button = blink::WebPointerProperties::Button::NoButton;
 
   gfx::Point pos =
       DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->pos()), screen);
@@ -651,8 +370,9 @@ blink::WebMouseEvent MakeWebMouseEvent(
     float location_bar_content_offset) {
   blink::WebMouseEvent result;
 
-  result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
-  result.modifiers = QInputEventStateToWebEventModifiers(event);
+  result.setTimeStampSeconds(QInputEventTimeToWebEventTime(event));
+  result.setModifiers(QInputEventStateToWebEventModifiers(event));
+  result.pointerType = blink::WebPointerProperties::PointerType::Mouse;
 
   gfx::Point pos =
       DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->pos()), screen);
@@ -673,19 +393,19 @@ blink::WebMouseEvent MakeWebMouseEvent(
 
   switch (event->type()) {
     case QEvent::HoverEnter:
-      result.type = blink::WebInputEvent::MouseEnter;
+      result.setType(blink::WebInputEvent::MouseEnter);
       break;
     case QEvent::HoverLeave:
-      result.type = blink::WebInputEvent::MouseLeave;
+      result.setType(blink::WebInputEvent::MouseLeave);
       break;
     case QEvent::HoverMove:
-      result.type = blink::WebInputEvent::MouseMove;
+      result.setType(blink::WebInputEvent::MouseMove);
       break;
     default:
       NOTREACHED();
   }
 
-  result.button = blink::WebMouseEvent::ButtonNone;
+  result.button = blink::WebPointerProperties::Button::NoButton;
 
   return result;
 }
