@@ -18,7 +18,6 @@
 #ifndef _OXIDE_QT_CORE_BROWSER_SCREEN_UTILS_H_
 #define _OXIDE_QT_CORE_BROWSER_SCREEN_UTILS_H_
 
-#include <Qt>
 #include <QtGlobal>
 
 #include "third_party/WebKit/public/platform/WebScreenInfo.h"
@@ -31,7 +30,6 @@ namespace oxide {
 namespace qt {
 
 blink::WebScreenInfo GetWebScreenInfoFromQScreen(QScreen* screen);
-Qt::ScreenOrientation GetNativeOrientation(QScreen* screen);
 
 } // namespace qt
 } // namespace oxide
