@@ -53,7 +53,6 @@
 #include "qt/core/browser/oxide_qt_user_script.h"
 #include "qt/core/glue/oxide_qt_web_context_proxy_client.h"
 #include "shared/browser/media/oxide_media_capture_devices_context.h"
-#include "shared/browser/oxide_browser_context.h"
 #include "shared/browser/oxide_browser_context_delegate.h"
 #include "shared/browser/oxide_browser_process_main.h"
 #include "shared/browser/oxide_devtools_manager.h"
@@ -66,6 +65,7 @@
 namespace oxide {
 namespace qt {
 
+using oxide::BrowserContext;
 using oxide::DevToolsManager;
 using oxide::MediaCaptureDevicesContext;
 using oxide::UserAgentSettings;
@@ -456,7 +456,7 @@ WebContext::~WebContext() {
 }
 
 // static
-WebContext* WebContext::FromBrowserContext(oxide::BrowserContext* context) {
+WebContext* WebContext::FromBrowserContext(BrowserContext* context) {
   BrowserContextDelegate* delegate =
       static_cast<BrowserContextDelegate*>(context->GetDelegate());
   if (!delegate) {
@@ -466,21 +466,21 @@ WebContext* WebContext::FromBrowserContext(oxide::BrowserContext* context) {
   return delegate->context();
 }
 
-oxide::BrowserContext* WebContext::GetContext() {
+BrowserContext* WebContext::GetContext() {
   if (context_.get()) {
     return context_.get();
   }
 
   DCHECK(construct_props_);
 
-  oxide::BrowserContext::Params params(
+  BrowserContext::Params params(
       construct_props_->data_path,
       construct_props_->cache_path,
       construct_props_->max_cache_size_hint,
       construct_props_->session_cookie_mode);
   params.host_mapping_rules = construct_props_->host_mapping_rules;
 
-  context_ = oxide::BrowserContext::Create(params);
+  context_ = BrowserContext::Create(params);
 
   UserAgentSettings* ua_settings = UserAgentSettings::Get(context_.get());
 
@@ -496,10 +496,11 @@ oxide::BrowserContext* WebContext::GetContext() {
   ua_settings->SetUserAgentOverrides(construct_props_->user_agent_overrides);
   ua_settings->SetLegacyUserAgentOverrideEnabled(
       construct_props_->legacy_user_agent_override_enabled);
+  ua_settings->SetIsPopupBlockerEnabled(
+      construct_props_->popup_blocker_enabled);
+  ua_settings->SetDoNotTrack(construct_props_->do_not_track);
 
   context_->SetCookiePolicy(construct_props_->cookie_policy);
-  context_->SetIsPopupBlockerEnabled(construct_props_->popup_blocker_enabled);
-  context_->SetDoNotTrack(construct_props_->do_not_track);
 
   MediaCaptureDevicesContext* dc =
       MediaCaptureDevicesContext::Get(context_.get());
@@ -690,7 +691,7 @@ void WebContext::setSessionCookieMode(SessionCookieMode mode) {
 
 bool WebContext::popupBlockerEnabled() const {
   if (IsInitialized()) {
-    return context_->IsPopupBlockerEnabled();
+    return UserAgentSettings::Get(context_.get())->IsPopupBlockerEnabled();
   }
 
   return construct_props_->popup_blocker_enabled;
@@ -698,7 +699,7 @@ bool WebContext::popupBlockerEnabled() const {
 
 void WebContext::setPopupBlockerEnabled(bool enabled) {
   if (IsInitialized()) {
-    context_->SetIsPopupBlockerEnabled(enabled);
+    UserAgentSettings::Get(context_.get())->SetIsPopupBlockerEnabled(enabled);
   } else {
     construct_props_->popup_blocker_enabled = enabled;
   }
@@ -794,7 +795,7 @@ int WebContext::setCookies(const QUrl& url,
   }
 
   if (ctxt->remaining == 0) {
-    base::MessageLoop::current()->PostTask(
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
         base::Bind(&WebContext::DeliverSetCookiesResponse,
                    weak_factory_.GetWeakPtr(), ctxt));
@@ -1000,7 +1001,7 @@ void WebContext::DefaultVideoDeviceChanged() {
 
 bool WebContext::doNotTrack() const {
     if (IsInitialized()) {
-    return context_->GetDoNotTrack();
+    return UserAgentSettings::Get(context_.get())->GetDoNotTrack();
   }
 
   return construct_props_->do_not_track;
@@ -1008,7 +1009,7 @@ bool WebContext::doNotTrack() const {
 
 void WebContext::setDoNotTrack(bool dnt) {
   if (IsInitialized()) {
-    context_->SetDoNotTrack(dnt);
+    UserAgentSettings::Get(context_.get())->SetDoNotTrack(dnt);
   } else {
     construct_props_->do_not_track = dnt;
   }
