@@ -24,10 +24,9 @@
 #include "base/memory/ptr_util.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/ssl_status.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/referrer.h"
-#include "content/public/common/security_style.h"
-#include "content/public/common/ssl_status.h"
 #include "content/public/test/test_browser_context.h"
 #include "content/public/test/test_web_contents_factory.h"
 #include "net/cert/x509_certificate.h"
@@ -37,7 +36,7 @@
 #include "qt/core/api/oxideqsecuritystatus_p.h"
 #include "qt/core/api/oxideqsslcertificate.h"
 #include "qt/core/api/oxideqsslcertificate_p.h"
-#include "shared/test/oxide_mock_cert_store.h"
+#include "qt/core/browser/web_contents_id_tracker.h"
 #include "shared/test/oxide_test_browser_thread_bundle.h"
 
 #include "oxide_qt_security_status.h"
@@ -45,17 +44,14 @@
 namespace oxide {
 namespace qt {
 
-using oxide::MockCertStore;
 using oxide::TestBrowserThreadBundle;
 
 class SecurityStatusTest : public testing::Test {
  public:
   SecurityStatusTest();
 
-  int cert_id() const { return cert_id_; }
-  int expired_cert_id() const { return expired_cert_id_; }
-
-  content::CertStore* cert_store() { return &cert_store_; }
+  net::X509Certificate* cert() const { return cert_.get(); }
+  net::X509Certificate* expired_cert() const { return expired_cert_.get(); }
 
   OxideQSecurityStatus* status() const { return security_status_.get(); }
 
@@ -64,7 +60,6 @@ class SecurityStatusTest : public testing::Test {
  private:
   void SetUp() override;
 
-  MockCertStore cert_store_;
   TestBrowserThreadBundle browser_thread_bundle_;
   content::TestBrowserContext browser_context_;
   content::TestWebContentsFactory web_contents_factory_;
@@ -72,38 +67,32 @@ class SecurityStatusTest : public testing::Test {
   std::unique_ptr<OxideQSecurityStatus> security_status_;
   content::WebContents* web_contents_;
 
-  int cert_id_;
-  int expired_cert_id_;
+  scoped_refptr<net::X509Certificate> cert_;
+  scoped_refptr<net::X509Certificate> expired_cert_;
 };
 
 SecurityStatusTest::SecurityStatusTest()
-    : web_contents_(nullptr),
-      cert_id_(0),
-      expired_cert_id_(0) {}
+    : web_contents_(nullptr) {}
 
 void SecurityStatusTest::SetUp() {
   security_status_ = base::WrapUnique(OxideQSecurityStatusPrivate::Create());
   web_contents_ = web_contents_factory_.CreateWebContents(&browser_context_);
   oxide::SecurityStatus::CreateForWebContents(web_contents_);
-  oxide::SecurityStatus::FromWebContents(web_contents_)->SetCertStoreForTesting(
-      &cert_store_);
 
-  OxideQSecurityStatusPrivate::get(security_status_.get())->proxy()->Init(
-      web_contents_);
+  OxideQSecurityStatusPrivate::get(security_status_.get())->Init(
+      WebContentsIDTracker::GetInstance()->GetIDForWebContents(web_contents_));
 
-  scoped_refptr<net::X509Certificate> cert(
+  cert_ =
       new net::X509Certificate("https://www.google.com/",
                                "https://www.example.com/",
                                base::Time::UnixEpoch(),
-                               base::Time::Now() + base::TimeDelta::FromDays(1)));
-  cert_id_ = cert_store_.AddCertForTesting(cert.get());
+                               base::Time::Now() + base::TimeDelta::FromDays(1));
 
-  cert =
+  expired_cert_ =
       new net::X509Certificate("https://www.google.com/",
                                "https://www.example.com/",
                                base::Time::UnixEpoch(),
                                base::Time::Now() - base::TimeDelta::FromDays(1));
-  expired_cert_id_ = cert_store_.AddCertForTesting(cert.get());
 }
 
 TEST_F(SecurityStatusTest, Uninitialized) {
@@ -134,10 +123,9 @@ TEST_F(SecurityStatusTest, NoSecurity) {
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_UNAUTHENTICATED;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelNone, status()->securityLevel());
   EXPECT_EQ(OxideQSecurityStatus::ContentStatusNormal,
@@ -150,19 +138,15 @@ TEST_F(SecurityStatusTest, NoSecurity) {
 
 TEST_F(SecurityStatusTest, Secure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelSecure,
             status()->securityLevel());
@@ -173,27 +157,23 @@ TEST_F(SecurityStatusTest, Secure) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
 
 TEST_F(SecurityStatusTest, Broken) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.cert_status = net::CERT_STATUS_COMMON_NAME_INVALID;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelError,
             status()->securityLevel());
@@ -205,27 +185,23 @@ TEST_F(SecurityStatusTest, Broken) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
 
 TEST_F(SecurityStatusTest, RanInsecure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.content_status = content::SSLStatus::RAN_INSECURE_CONTENT;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelError,
             status()->securityLevel());
@@ -236,27 +212,23 @@ TEST_F(SecurityStatusTest, RanInsecure) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
 
 TEST_F(SecurityStatusTest, DisplayedInsecure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.content_status = content::SSLStatus::DISPLAYED_INSECURE_CONTENT;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelWarning,
             status()->securityLevel());
@@ -267,29 +239,25 @@ TEST_F(SecurityStatusTest, DisplayedInsecure) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
 
 TEST_F(SecurityStatusTest, DisplayedAndRanInsecure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.content_status =
       content::SSLStatus::DISPLAYED_INSECURE_CONTENT |
       content::SSLStatus::RAN_INSECURE_CONTENT;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelError,
             status()->securityLevel());
@@ -301,27 +269,23 @@ TEST_F(SecurityStatusTest, DisplayedAndRanInsecure) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
 
 TEST_F(SecurityStatusTest, MinorCertError) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.cert_status = net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelWarning,
             status()->securityLevel());
@@ -333,27 +297,23 @@ TEST_F(SecurityStatusTest, MinorCertError) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
 
 TEST_F(SecurityStatusTest, SecureEV) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.cert_status = net::CERT_STATUS_IS_EV;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(OxideQSecurityStatus::SecurityLevelSecureEV,
             status()->securityLevel());
@@ -364,7 +324,7 @@ TEST_F(SecurityStatusTest, SecureEV) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
@@ -584,25 +544,18 @@ TEST_P(SecurityStatusCertStatusTest, TestCertStatus) {
   const CertStatusRow& row = GetParam();
 
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  if (net::IsCertStatusError(row.cert_status_in) &&
-      !net::IsCertStatusMinorError(row.cert_status_in)) {
-    ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  } else {
-    ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  }
-  ssl_status.cert_id = row.expired_cert ? expired_cert_id() : cert_id();
+  ssl_status.certificate = row.expired_cert ? expired_cert() : cert();
   ssl_status.cert_status = row.cert_status_in;
 
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(ssl_status.cert_id, &cert));
+  scoped_refptr<net::X509Certificate> expected_cert = ssl_status.certificate;
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   OxideQSecurityStatus::SecurityLevel expected_security_level;
   if (net::IsCertStatusError(row.cert_status_in)) {
@@ -623,7 +576,7 @@ TEST_P(SecurityStatusCertStatusTest, TestCertStatus) {
   EXPECT_EQ(QVariant::UserType, status()->certificate().type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(),
             status()->certificate().userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(expected_cert.get(),
             OxideQSslCertificateData::GetX509Certificate(
                 status()->certificate().value<OxideQSslCertificate>()));
 }
@@ -669,41 +622,38 @@ void Observer::OnUpdate() {
 
 TEST_F(SecurityStatusTest, SecurityLevelUpdate) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   std::unique_ptr<Observer> observer = base::WrapUnique(new Observer(status()));
   observer->connect(status(), SIGNAL(securityLevelChanged()),
                     SLOT(OnUpdate()));
 
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
+  ssl_status.certificate = cert();
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(1, observer->update_count);
-  EXPECT_EQ(OxideQSecurityStatus::SecurityLevelError, observer->security_level);
+  EXPECT_EQ(OxideQSecurityStatus::SecurityLevelSecure, observer->security_level);
 }
 
 TEST_F(SecurityStatusTest, ContentStatusUpdate) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   std::unique_ptr<Observer> observer = base::WrapUnique(new Observer(status()));
   observer->connect(status(), SIGNAL(contentStatusChanged()),
@@ -711,7 +661,7 @@ TEST_F(SecurityStatusTest, ContentStatusUpdate) {
 
   ssl_status.content_status = content::SSLStatus::DISPLAYED_INSECURE_CONTENT;
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(1, observer->update_count);
   EXPECT_EQ(OxideQSecurityStatus::ContentStatusDisplayedInsecure,
@@ -720,16 +670,15 @@ TEST_F(SecurityStatusTest, ContentStatusUpdate) {
 
 TEST_F(SecurityStatusTest, CertStatusUpdate) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   std::unique_ptr<Observer> observer = base::WrapUnique(new Observer(status()));
   observer->connect(status(), SIGNAL(certStatusChanged()),
@@ -737,7 +686,7 @@ TEST_F(SecurityStatusTest, CertStatusUpdate) {
 
   ssl_status.cert_status = net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION;
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(1, observer->update_count);
   EXPECT_EQ(OxideQSecurityStatus::CertStatusRevocationCheckFailed,
@@ -746,33 +695,29 @@ TEST_F(SecurityStatusTest, CertStatusUpdate) {
 
 TEST_F(SecurityStatusTest, CertUpdate) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
 
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
+      ->VisibleSecurityStateChanged();
 
   std::unique_ptr<Observer> observer = base::WrapUnique(new Observer(status()));
   observer->connect(status(), SIGNAL(certificateChanged()),
                     SLOT(OnUpdate()));
 
-  ssl_status.cert_id = expired_cert_id();
+  ssl_status.certificate = expired_cert();
   oxide::SecurityStatus::FromWebContents(web_contents())
-      ->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(expired_cert_id(), &cert));
+      ->VisibleSecurityStateChanged();
 
   EXPECT_EQ(1, observer->update_count);
   EXPECT_FALSE(observer->cert.isNull());
   EXPECT_EQ(QVariant::UserType, observer->cert.type());
   EXPECT_EQ(qMetaTypeId<OxideQSslCertificate>(), observer->cert.userType());
-  EXPECT_EQ(cert.get(),
+  EXPECT_EQ(expired_cert(),
             OxideQSslCertificateData::GetX509Certificate(
                 observer->cert.value<OxideQSslCertificate>()));
 }
@@ -780,8 +725,4 @@ TEST_F(SecurityStatusTest, CertUpdate) {
 } // namespace qt
 } // namespace oxide
 
-#if defined(GN_BUILD)
 #include "qt/core/browser/ssl/oxide_qt_security_status_unittest.moc"
-#else
-#include "oxide_qt_security_status_unittest.moc"
-#endif
