@@ -20,23 +20,28 @@
 
 #include <QString>
 
+#include "qt/core/api/oxideqglobal.h"
+
 namespace oxide {
 namespace qt {
 
-struct MenuItem {
-  MenuItem() :
-      index(-1),
-      enabled(false),
-      checked(false),
-      separator(false) {}
+struct OXIDE_QTCORE_EXPORT MenuItem {
+  MenuItem();
+  MenuItem(const MenuItem& other);
+  ~MenuItem();
+
+  enum class Type {
+    Option = 0,
+    Group = 2,
+    Separator = 3
+  };
 
   QString label;
   QString tooltip;
-  QString group;
-  int index;
-  bool enabled;
-  bool checked;
-  bool separator;
+  Type type = Type::Option;
+  unsigned action = 0;
+  bool enabled = false;
+  bool checked = false;
 };
 
 } // namespace qt
