@@ -32,12 +32,15 @@ class ShimApi : public QObject {
  public:
   Q_INVOKABLE void resetScreens();
 
-  Q_INVOKABLE void overrideScreenGeometry(QScreen* screen,
-                                          const QRect& geometry,
-                                          const QRect& available_geometry);
+  Q_INVOKABLE void setScreenGeometry(QScreen* screen,
+                                     const QRect& geometry,
+                                     const QRect& work_area_in_screen);
 
   Q_INVOKABLE void setScreenOrientation(QScreen* screen,
                                         Qt::ScreenOrientation orientation);
+
+  Q_INVOKABLE void setScreenFormFactor(QScreen* screen,
+                                       int form_factor);
 };
 
 #endif // _OXIDE_QT_TESTS_MOCK_QPA_SHIMAPI_H_
