@@ -209,9 +209,6 @@ static void RegisterQmlTypes() {
   qmlRegisterUncreatableType<WebContextTestSupport>(
       "Oxide.testsupport", 1, 0, "WebContextTestSupport",
       "WebContextTestSupport only exists to provide attached properties");
-  qmlRegisterUncreatableType<WebViewTestSupport>(
-      "Oxide.testsupport", 1, 0, "WebViewTestSupport",
-      "WebViewTestSupport only exists to provide attached properties");
   qmlRegisterUncreatableType<TestWindow>(
       "Oxide.testsupport", 1, 0, "TestWindow",
       "TestWindow only exists to provide attached properties");
@@ -511,14 +508,18 @@ int main(int argc, char** argv) {
     qFatal("Didn't specify a test name!");
   }
 
+  if (!options.plugin_path.isEmpty()) {
+    QCoreApplication::addLibraryPath(options.plugin_path);
+  }
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+  QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
   QGuiApplication app(argc, argv);
 
   if (!options.nss_db_path.isEmpty()) {
     oxideSetNSSDbPath(options.nss_db_path);
   }
-  if (!options.plugin_path.isEmpty()) {
-    app.addLibraryPath(options.plugin_path);
-  }
   if (options.single_process) {
     oxideSetProcessModel(OxideProcessModelSingleProcess);
   }
