@@ -25,23 +25,23 @@
 #include "base/trace_event/trace_event.h"
 #include "cc/animation/animation_host.h"
 #include "cc/layers/layer.h"
+#include "cc/output/compositor_frame_sink.h"
 #include "cc/output/context_provider.h"
+#include "cc/output/output_surface.h"
 #include "cc/output/renderer_settings.h"
 #include "cc/output/texture_mailbox_deleter.h"
 #include "cc/scheduler/begin_frame_source.h"
 #include "cc/scheduler/delay_based_time_source.h"
+#include "cc/surfaces/direct_compositor_frame_sink.h"
 #include "cc/surfaces/display.h"
 #include "cc/surfaces/display_scheduler.h"
-#include "cc/surfaces/surface_display_output_surface.h"
-#include "cc/surfaces/surface_id_allocator.h"
+#include "cc/trees/layer_tree.h"
 #include "cc/trees/layer_tree_host.h"
+#include "cc/trees/layer_tree_host_in_process.h"
 #include "cc/trees/layer_tree_settings.h"
 #include "content/browser/gpu/browser_gpu_channel_host_factory.h" // nogncheck
 #include "content/browser/gpu/browser_gpu_memory_buffer_manager.h" // nogncheck
 #include "content/browser/gpu/gpu_data_manager_impl.h" // nogncheck
-#include "content/common/gpu/client/command_buffer_metrics.h" // nogncheck
-#include "content/common/gpu/client/context_provider_command_buffer.h" // nogncheck
-#include "content/common/gpu_process_launch_causes.h" // nogncheck
 #include "content/common/host_shared_bitmap_manager.h" // nogncheck
 #include "gpu/command_buffer/client/context_support.h"
 #include "gpu/command_buffer/client/gles2_interface.h"
@@ -49,6 +49,9 @@
 #include "gpu/command_buffer/common/gles2_cmd_utils.h"
 #include "gpu/GLES2/gl2extchromium.h"
 #include "gpu/ipc/client/gpu_channel_host.h"
+#include "services/ui/public/cpp/gpu/command_buffer_metrics.h"
+#include "services/ui/public/cpp/gpu/context_provider_command_buffer.h"
+#include "ui/gfx/geometry/rect.h"
 #include "url/gurl.h"
 
 #include "oxide_compositor_client.h"
@@ -72,10 +75,8 @@ scoped_refptr<cc::ContextProvider> CreateOffscreenContextProvider() {
     return nullptr;
   }
 
-  content::CauseForGpuLaunch cause =
-      content::CAUSE_FOR_GPU_LAUNCH_DISPLAY_COMPOSITOR_CONTEXT;
   scoped_refptr<gpu::GpuChannelHost> gpu_channel_host(
-      content::BrowserGpuChannelHostFactory::instance()->EstablishGpuChannelSync(cause));
+      content::BrowserGpuChannelHostFactory::instance()->EstablishGpuChannelSync());
   if (!gpu_channel_host.get()) {
     return nullptr;
   }
@@ -90,7 +91,7 @@ scoped_refptr<cc::ContextProvider> CreateOffscreenContextProvider() {
   attrs.lose_context_when_out_of_memory = true;
 
   return make_scoped_refptr(
-      new content::ContextProviderCommandBuffer(
+      new ui::ContextProviderCommandBuffer(
           gpu_channel_host.get(),
           gpu::GPU_STREAM_DEFAULT,
           gpu::GpuStreamPriority::NORMAL,
@@ -101,7 +102,7 @@ scoped_refptr<cc::ContextProvider> CreateOffscreenContextProvider() {
           gpu::SharedMemoryLimits(),
           attrs,
           nullptr,
-          content::command_buffer_metrics::CONTEXT_TYPE_UNKNOWN));
+          ui::command_buffer_metrics::CONTEXT_TYPE_UNKNOWN));
 }
 
 } // namespace
@@ -112,8 +113,12 @@ Compositor::Compositor(CompositorClient* client)
       task_runner_(base::ThreadTaskRunnerHandle::Get()),
       output_surface_(nullptr),
       mailbox_buffer_map_(mode_),
-      surface_id_allocator_(
-          CompositorUtils::GetInstance()->CreateSurfaceIdAllocator()),
+      frame_sink_id_(CompositorUtils::GetInstance()->AllocateFrameSinkId()),
+      begin_frame_source_(
+          new cc::DelayBasedBeginFrameSource(
+              base::MakeUnique<cc::DelayBasedTimeSource>(
+                  base::ThreadTaskRunnerHandle::Get().get()))),
+      animation_host_(cc::AnimationHost::CreateMainInstance()),
       layer_tree_host_eviction_pending_(false),
       can_evict_layer_tree_host_(false),
       num_failed_recreate_attempts_(0),
@@ -124,7 +129,11 @@ Compositor::Compositor(CompositorClient* client)
       pending_swaps_(0),
       frames_waiting_for_completion_(0),
       mailbox_resource_fetches_in_progress_(0),
-      weak_factory_(this) {}
+      weak_factory_(this) {
+  CompositorUtils::GetInstance()
+      ->GetSurfaceManager()
+      ->RegisterFrameSinkId(frame_sink_id_);
+}
 
 bool Compositor::SurfaceIdIsCurrent(uint32_t surface_id) {
   return output_surface_ && output_surface_->surface_id() == surface_id;
@@ -212,9 +221,9 @@ void Compositor::SendSwapCompositorFrameToClient(
   DCHECK(output_surface_);
   DCHECK(!swap_ack_callback_.is_null());
 
-  FOR_EACH_OBSERVER(CompositorObserver,
-                    observers_,
-                    CompositorWillRequestSwapFrame());
+  for (auto& observer : observers_) {
+    observer.CompositorWillRequestSwapFrame();
+  }
 
   // XXX: What if we are hidden?
   TRACE_EVENT_ASYNC_BEGIN1("cc", "oxide::Compositor:pending_swaps",
@@ -231,9 +240,9 @@ void Compositor::SendSwapCompositorFrameToClient(
                                 std::move(frame));
   client_->CompositorSwapFrame(handle.get(), swap_ack_callback_);
 
-  FOR_EACH_OBSERVER(CompositorObserver,
-                    observers_,
-                    CompositorDidRequestSwapFrame());
+  for (auto& observer : observers_) {
+    observer.CompositorDidRequestSwapFrame();
+  }
 }
 
 // static
@@ -298,68 +307,62 @@ void Compositor::OutputSurfaceChanged() {
   }
 }
 
-std::unique_ptr<cc::OutputSurface> Compositor::CreateOutputSurface() {
+std::unique_ptr<cc::CompositorFrameSink>
+Compositor::CreateCompositorFrameSink() {
   uint32_t output_surface_id = next_output_surface_id_++;
 
   scoped_refptr<cc::ContextProvider> context_provider;
-  std::unique_ptr<cc::OutputSurface> display_surface;
+  std::unique_ptr<cc::OutputSurface> output_surface;
   if (CompositorUtils::GetInstance()->CanUseGpuCompositing()) {
     context_provider = CreateOffscreenContextProvider();
     if (!context_provider.get()) {
       return nullptr;
     }
-    display_surface =
+    output_surface =
         base::MakeUnique<CompositorOutputSurfaceGL>(output_surface_id,
                                                     context_provider,
                                                     this);
   } else {
     std::unique_ptr<CompositorSoftwareOutputDevice> output_device(
         new CompositorSoftwareOutputDevice());
-    display_surface =
+    output_surface =
         base::MakeUnique<CompositorOutputSurfaceSoftware>(
             output_surface_id,
             std::move(output_device),
             this);
   }
 
-  std::unique_ptr<cc::BeginFrameSource> begin_frame_source(
-      new cc::DelayBasedBeginFrameSource(
-          base::MakeUnique<cc::DelayBasedTimeSource>(
-              base::ThreadTaskRunnerHandle::Get().get())));
-
   std::unique_ptr<cc::DisplayScheduler> scheduler(
       new cc::DisplayScheduler(
-          begin_frame_source.get(),
           base::ThreadTaskRunnerHandle::Get().get(),
-          display_surface->capabilities().max_frames_pending));
-
-  cc::SurfaceManager* manager =
-      CompositorUtils::GetInstance()->GetSurfaceManager();
+          output_surface->capabilities().max_frames_pending));
 
   display_ =
       base::MakeUnique<cc::Display>(
-          manager,
           content::HostSharedBitmapManager::current(),
           content::BrowserGpuMemoryBufferManager::current(),
           cc::RendererSettings(),
-          surface_id_allocator_->id_namespace(),
-          std::move(begin_frame_source),
-          std::move(display_surface),
+          frame_sink_id_,
+          begin_frame_source_.get(),
+          std::move(output_surface),
           std::move(scheduler),
           base::MakeUnique<cc::TextureMailboxDeleter>(
               base::ThreadTaskRunnerHandle::Get().get()));
 
-  std::unique_ptr<cc::SurfaceDisplayOutputSurface> surface(
-      new cc::SurfaceDisplayOutputSurface(
-          manager,
-          surface_id_allocator_.get(),
+  std::unique_ptr<cc::DirectCompositorFrameSink> sink =
+      base::MakeUnique<cc::DirectCompositorFrameSink>(
+          frame_sink_id_,
+          CompositorUtils::GetInstance()->GetSurfaceManager(),
           display_.get(),
           context_provider,
-          nullptr));
+          nullptr,
+          content::BrowserGpuMemoryBufferManager::current(),
+          content::HostSharedBitmapManager::current());
 
-  display_->Resize(layer_tree_host_->device_viewport_size());
+  display_->Resize(layer_tree_host_->GetLayerTree()->device_viewport_size());
+  display_->SetVisible(layer_tree_host_->IsVisible());
 
-  return std::move(surface);
+  return std::move(sink);
 }
 
 void Compositor::AddObserver(CompositorObserver* observer) {
@@ -378,29 +381,26 @@ void Compositor::EnsureLayerTreeHost() {
   }
 
   cc::LayerTreeSettings settings;
-  settings.use_external_begin_frame_source = false;
   settings.renderer_settings.allow_antialiasing = false;
 
-  cc::LayerTreeHost::InitParams params;
+  cc::LayerTreeHostInProcess::InitParams params;
   params.client = this;
-  params.shared_bitmap_manager = content::HostSharedBitmapManager::current();
-  params.gpu_memory_buffer_manager =
-      content::BrowserGpuMemoryBufferManager::current();
   params.task_graph_runner =
       CompositorUtils::GetInstance()->GetTaskGraphRunner();
   params.settings = &settings;
   params.main_task_runner = base::ThreadTaskRunnerHandle::Get();
-  params.animation_host = cc::AnimationHost::CreateMainInstance();
+  params.mutator_host = animation_host_.get();
 
-  layer_tree_host_ = cc::LayerTreeHost::CreateSingleThreaded(this, &params);
+  layer_tree_host_ =
+      cc::LayerTreeHostInProcess::CreateSingleThreaded(this, &params);
   DCHECK(layer_tree_host_);
 
   can_evict_layer_tree_host_ = true;
 
-  layer_tree_host_->SetRootLayer(root_layer_);
+  layer_tree_host_->GetLayerTree()->SetRootLayer(root_layer_);
   layer_tree_host_->SetVisible(visible_);
-  layer_tree_host_->SetViewportSize(size_);
-  layer_tree_host_->SetDeviceScaleFactor(device_scale_factor_);
+  layer_tree_host_->GetLayerTree()->SetViewportSize(size_);
+  layer_tree_host_->GetLayerTree()->SetDeviceScaleFactor(device_scale_factor_);
 }
 
 void Compositor::MaybeEvictLayerTreeHost() {
@@ -496,43 +496,45 @@ void Compositor::ApplyViewportDeltas(
     float page_scale,
     float top_controls_delta) {}
 
-void Compositor::RequestNewOutputSurface() {
-  std::unique_ptr<cc::OutputSurface> surface(CreateOutputSurface());
-  if (!surface) {
-    DidFailToInitializeOutputSurface();
+void Compositor::RequestNewCompositorFrameSink() {
+  std::unique_ptr<cc::CompositorFrameSink> sink(CreateCompositorFrameSink());
+  if (!sink) {
+    DidFailToInitializeCompositorFrameSink();
     return;
   }
 
-  layer_tree_host_->SetOutputSurface(std::move(surface));
+  layer_tree_host_->SetCompositorFrameSink(std::move(sink));
 }
 
-void Compositor::DidInitializeOutputSurface() {
+void Compositor::DidInitializeCompositorFrameSink() {
   num_failed_recreate_attempts_ = 0;
 }
 
-void Compositor::DidFailToInitializeOutputSurface() {
+void Compositor::DidFailToInitializeCompositorFrameSink() {
   num_failed_recreate_attempts_++;
 
   CHECK_LE(num_failed_recreate_attempts_, 4);
 
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
-      base::Bind(&Compositor::RequestNewOutputSurface,
+      base::Bind(&Compositor::RequestNewCompositorFrameSink,
                  weak_factory_.GetWeakPtr()));
 }
 
 void Compositor::WillCommit() {}
 
 void Compositor::DidCommit() {
-  FOR_EACH_OBSERVER(CompositorObserver, observers_, CompositorDidCommit());
+  for (auto& observer : observers_) {
+    observer.CompositorDidCommit();
+  }
 }
 
 void Compositor::DidCommitAndDrawFrame() {}
-void Compositor::DidCompleteSwapBuffers() {}
+void Compositor::DidReceiveCompositorFrameAck() {}
 void Compositor::DidCompletePageScaleAnimation() {}
 
-void Compositor::DidPostSwapBuffers() {}
-void Compositor::DidAbortSwapBuffers() {}
+void Compositor::DidSubmitCompositorFrame() {}
+void Compositor::DidLoseCompositorFrameSink() {}
 
 void Compositor::OutputSurfaceBound(CompositorOutputSurface* output_surface) {
   DCHECK(!output_surface_);
@@ -680,7 +682,16 @@ std::unique_ptr<Compositor> Compositor::Create(CompositorClient* client) {
 }
 
 Compositor::~Compositor() {
-  FOR_EACH_OBSERVER(CompositorObserver, observers_, OnCompositorDestruction());
+  for (auto& observer : observers_) {
+    observer.OnCompositorDestruction();
+  }
+
+  layer_tree_host_.reset();
+  display_.reset();
+
+  CompositorUtils::GetInstance()
+      ->GetSurfaceManager()
+      ->InvalidateFrameSinkId(frame_sink_id_);
 }
 
 void Compositor::SetVisibility(bool visible) {
@@ -698,12 +709,15 @@ void Compositor::SetVisibility(bool visible) {
   DCHECK(!layer_tree_host_eviction_pending_);
 
   layer_tree_host_->SetVisible(visible);
+  if (display_) {
+    display_->SetVisible(visible);
+  }
 
   if (!visible) {
     layer_tree_host_eviction_pending_ = true;
-    FOR_EACH_OBSERVER(CompositorObserver,
-                      observers_,
-                      CompositorEvictResources());
+    for (auto& observer : observers_) {
+      observer.CompositorEvictResources();
+    }
     MaybeEvictLayerTreeHost();
   }
 }
@@ -716,7 +730,7 @@ void Compositor::SetDeviceScaleFactor(float scale) {
   device_scale_factor_ = scale;
 
   if (layer_tree_host_) {
-    layer_tree_host_->SetDeviceScaleFactor(scale);
+    layer_tree_host_->GetLayerTree()->SetDeviceScaleFactor(scale);
   }
 }
 
@@ -728,7 +742,7 @@ void Compositor::SetViewportSize(const gfx::Size& size) {
   size_ = size;
 
   if (layer_tree_host_) {
-    layer_tree_host_->SetViewportSize(size);
+    layer_tree_host_->GetLayerTree()->SetViewportSize(size);
   }
   if (display_) {
     display_->Resize(size);
@@ -743,4 +757,12 @@ void Compositor::SetRootLayer(scoped_refptr<cc::Layer> layer) {
   }
 }
 
+void Compositor::SetNeedsRedraw() {
+  if (!layer_tree_host_) {
+    return;
+  }
+
+  layer_tree_host_->SetNeedsRedrawRect(gfx::Rect(size_));
+}
+
 } // namespace oxide
