@@ -17,6 +17,8 @@
 
 #include "oxide_compositor_utils.h"
 
+#include <memory>
+
 #include "base/auto_reset.h"
 #include "base/bind.h"
 #include "base/containers/hash_tables.h"
@@ -27,15 +29,14 @@
 #include "base/threading/thread_task_runner_handle.h"
 #include "cc/output/context_provider.h"
 #include "cc/raster/single_thread_task_graph_runner.h"
-#include "cc/surfaces/surface_id_allocator.h"
-#include "cc/surfaces/surface_manager.h"
+#include "content/browser/compositor/surface_utils.h" // nogncheck
 #include "content/browser/gpu/browser_gpu_channel_host_factory.h" // nogncheck
 #include "content/browser/gpu/gpu_data_manager_impl.h" // nogncheck
-#include "content/common/gpu/client/context_provider_command_buffer.h" // nogncheck
 #include "gpu/command_buffer/common/command_buffer_id.h"
 #include "gpu/command_buffer/common/mailbox.h"
 #include "gpu/ipc/client/command_buffer_proxy_impl.h"
 #include "gpu/ipc/client/gpu_channel_host.h"
+#include "services/ui/public/cpp/gpu/context_provider_command_buffer.h"
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/gl_switches.h"
 
@@ -49,8 +50,6 @@ namespace oxide {
 
 namespace {
 
-uint32_t g_surface_id_namespace = 0;
-
 void WakeUpGpuThread() {}
 
 void DestroyEGLImage(EGLImageKHR egl_image) {
@@ -131,16 +130,6 @@ class FetchEGLImageTaskInfo : public FetchTextureResourcesTaskInfo {
   EGLImageKHR egl_image_;
 };
 
-class CompositorThread : public base::Thread {
- public:
-  CompositorThread();
-  ~CompositorThread() override;
-
- private:
-  // base::Thread implementation
-  void Init() override;
-};
-
 class CompositorUtilsImpl : public CompositorUtils,
                             public base::MessageLoop::TaskObserver {
  public:
@@ -163,7 +152,7 @@ class CompositorUtilsImpl : public CompositorUtils,
   CompositingMode GetCompositingMode() const override;
   cc::TaskGraphRunner* GetTaskGraphRunner() const override;
   cc::SurfaceManager* GetSurfaceManager() const override;
-  std::unique_ptr<cc::SurfaceIdAllocator> CreateSurfaceIdAllocator() override;
+  cc::FrameSinkId AllocateFrameSinkId() override;
 
   bool CalledOnMainThread() const;
   bool CalledOnGpuThread() const;
@@ -220,7 +209,6 @@ class CompositorUtilsImpl : public CompositorUtils,
 
   struct MainData {
     std::unique_ptr<cc::SingleThreadTaskGraphRunner> task_graph_runner;
-    std::unique_ptr<cc::SurfaceManager> surface_manager;
   } main_unsafe_access_;
 
   struct GpuData {
@@ -464,12 +452,8 @@ void CompositorUtilsImpl::Initialize(bool has_share_context) {
   main().task_graph_runner->Start("CompositorTileWorker1",
                                   base::SimpleThread::Options());
 
-  main().surface_manager.reset(new cc::SurfaceManager());
-
-  content::CauseForGpuLaunch cause =
-      content::CAUSE_FOR_GPU_LAUNCH_BROWSER_STARTUP;
   scoped_refptr<gpu::GpuChannelHost> gpu_channel_host(
-      content::BrowserGpuChannelHostFactory::instance()->EstablishGpuChannelSync(cause));
+      content::BrowserGpuChannelHostFactory::instance()->EstablishGpuChannelSync());
   if (gpu_channel_host.get()) {
     GpuUtils::GetTaskRunner()->PostTask(
         FROM_HERE,
@@ -484,8 +468,6 @@ void CompositorUtilsImpl::Shutdown() {
   main().task_graph_runner->Shutdown();
   main().task_graph_runner.reset();
 
-  main().surface_manager.reset();
-
   // Detach the GPU thread MessageLoop::TaskObserver, to stop processing
   // and existing incoming requests
   scoped_refptr<base::SingleThreadTaskRunner> gpu_task_runner =
@@ -530,7 +512,7 @@ void CompositorUtilsImpl::GetTextureFromMailbox(
 
   FetchTextureIDTaskInfo* info =
       new FetchTextureIDTaskInfo(
-        static_cast<content::ContextProviderCommandBuffer*>(
+        static_cast<ui::ContextProviderCommandBuffer*>(
             context_provider)->GetCommandBufferProxy()->GetCommandBufferID(),
         mailbox,
         sync_point,
@@ -564,7 +546,7 @@ void CompositorUtilsImpl::CreateEGLImageFromMailbox(
 
   FetchEGLImageTaskInfo* info =
       new FetchEGLImageTaskInfo(
-        static_cast<content::ContextProviderCommandBuffer*>(
+        static_cast<ui::ContextProviderCommandBuffer*>(
             context_provider)->GetCommandBufferProxy()->GetCommandBufferID(),
         mailbox,
         sync_point,
@@ -612,15 +594,11 @@ cc::TaskGraphRunner* CompositorUtilsImpl::GetTaskGraphRunner() const {
 }
 
 cc::SurfaceManager* CompositorUtilsImpl::GetSurfaceManager() const {
-  return main().surface_manager.get();
+  return content::GetSurfaceManager();
 }
 
-std::unique_ptr<cc::SurfaceIdAllocator>
-CompositorUtilsImpl::CreateSurfaceIdAllocator() {
-  std::unique_ptr<cc::SurfaceIdAllocator> allocator(
-      new cc::SurfaceIdAllocator(++g_surface_id_namespace));
-  allocator->RegisterSurfaceIdNamespace(GetSurfaceManager());
-  return allocator;
+cc::FrameSinkId CompositorUtilsImpl::AllocateFrameSinkId() {
+  return content::AllocateFrameSinkId();
 }
 
 bool CompositorUtilsImpl::CalledOnMainThread() const {
