@@ -30,12 +30,13 @@
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/common/media_stream_request.h"
+#include "media/base/video_facing.h"
 
+#if defined(ENABLE_HYBRIS_CAMERA)
+#include "shared/browser/hybris_utils.h"
+#endif
 #include "shared/browser/oxide_browser_context.h"
 #include "shared/browser/oxide_browser_process_main.h"
-#if defined(ENABLE_HYBRIS_CAMERA)
-#include "shared/browser/oxide_hybris_utils.h"
-#endif
 #include "shared/browser/permissions/oxide_permission_request_dispatcher.h"
 #include "shared/browser/permissions/oxide_permission_request_response.h"
 #include "shared/browser/permissions/oxide_temporary_saved_permission_context.h"
@@ -210,7 +211,7 @@ PermissionRequestCallback WrapMediaResponseCallback(
 
 void UpdateMediaStreamDeviceFacing(content::MediaStreamDevices& devices) {
 #ifdef ENABLE_HYBRIS_CAMERA
-  if (!HybrisUtils::IsCameraCompatAvailable()) {
+  if (!HybrisUtils::GetInstance()->IsCameraCompatAvailable()) {
     return;
   }
 
@@ -236,10 +237,10 @@ void UpdateMediaStreamDeviceFacing(content::MediaStreamDevices& devices) {
 
     switch(type) {
       case BACK_FACING_CAMERA_TYPE:
-        device.video_facing = content::MEDIA_VIDEO_FACING_ENVIRONMENT;
+        device.video_facing = media::MEDIA_VIDEO_FACING_ENVIRONMENT;
         break;
       case FRONT_FACING_CAMERA_TYPE:
-        device.video_facing = content::MEDIA_VIDEO_FACING_USER;
+        device.video_facing = media::MEDIA_VIDEO_FACING_USER;
         break;
       default:
         break;
@@ -266,16 +267,16 @@ void MediaCaptureDevicesDispatcher::RemoveObserver(
 
 void MediaCaptureDevicesDispatcher::NotifyAudioCaptureDevicesChanged() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-  FOR_EACH_OBSERVER(MediaCaptureDevicesDispatcherObserver,
-                    observers_,
-                    OnAudioCaptureDevicesChanged());
+  for (auto& observer : observers_) {
+    observer.OnAudioCaptureDevicesChanged();
+  }
 }
 
 void MediaCaptureDevicesDispatcher::NotifyVideoCaptureDevicesChanged() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-  FOR_EACH_OBSERVER(MediaCaptureDevicesDispatcherObserver,
-                    observers_,
-                    OnVideoCaptureDevicesChanged());
+  for (auto& observer : observers_) {
+    observer.OnVideoCaptureDevicesChanged();
+  }
 }
 
 void MediaCaptureDevicesDispatcher::OnAudioCaptureDevicesChanged() {
@@ -357,13 +358,19 @@ MediaCaptureDevicesDispatcher::GetVideoCaptureDevices() {
 }
 
 const content::MediaStreamDevice*
-MediaCaptureDevicesDispatcher::GetFirstAudioCaptureDevice() {
+MediaCaptureDevicesDispatcher::GetPreferredDefaultAudioCaptureDevice() {
   return GetFirstCaptureDevice(GetAudioCaptureDevices());
 }
 
 const content::MediaStreamDevice*
-MediaCaptureDevicesDispatcher::GetFirstVideoCaptureDevice() {
-  return GetFirstCaptureDevice(GetVideoCaptureDevices());
+MediaCaptureDevicesDispatcher::GetPreferredDefaultVideoCaptureDevice() {
+  const auto& devices = GetVideoCaptureDevices();
+  for (auto& device : devices) {
+    if (device.video_facing == media::MEDIA_VIDEO_FACING_USER) {
+      return &device;
+    }
+  }
+  return GetFirstCaptureDevice(devices);
 }
 
 const content::MediaStreamDevice*
@@ -399,7 +406,7 @@ bool MediaCaptureDevicesDispatcher::GetDefaultCaptureDevicesForContext(
   if (need_audio) {
     const content::MediaStreamDevice* device = dc->GetDefaultAudioDevice();
     if (!device) {
-      device = GetFirstAudioCaptureDevice();
+      device = GetPreferredDefaultAudioCaptureDevice();
     }
     if (device) {
       need_audio = false;
@@ -410,7 +417,7 @@ bool MediaCaptureDevicesDispatcher::GetDefaultCaptureDevicesForContext(
   if (need_video) {
     const content::MediaStreamDevice* device = dc->GetDefaultVideoDevice();
     if (!device) {
-      device = GetFirstVideoCaptureDevice();
+      device = GetPreferredDefaultVideoCaptureDevice();
     }
     if (device) {
       need_video = false;
@@ -424,14 +431,6 @@ bool MediaCaptureDevicesDispatcher::GetDefaultCaptureDevicesForContext(
 void MediaCaptureDevicesDispatcher::RequestMediaAccessPermission(
     const content::MediaStreamRequest& request,
     const content::MediaResponseCallback& callback) {
-  if (request.video_type == content::MEDIA_DEVICE_AUDIO_OUTPUT ||
-      request.audio_type == content::MEDIA_DEVICE_AUDIO_OUTPUT) {
-    callback.Run(content::MediaStreamDevices(),
-                 content::MEDIA_DEVICE_INVALID_STATE,
-                 nullptr);
-    return;
-  }
-
   if (request.video_type == content::MEDIA_NO_SERVICE &&
       request.audio_type == content::MEDIA_NO_SERVICE) {
     callback.Run(content::MediaStreamDevices(),
