@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013-2015 Canonical Ltd.
+// Copyright (C) 2013-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -43,7 +43,7 @@
 #include "net/base/host_mapping_rules.h"
 #include "net/base/net_errors.h"
 #include "net/cookies/cookie_monster.h"
-#include "net/ftp/ftp_network_layer.h"
+#include "net/extras/sqlite/sqlite_channel_id_store.h"
 #include "net/http/http_cache.h"
 #include "net/http/http_network_session.h"
 #include "net/http/http_server_properties_impl.h"
@@ -68,7 +68,6 @@
 
 #include "oxide_browser_context_delegate.h"
 #include "oxide_browser_context_destroyer.h"
-#include "oxide_browser_context_observer.h"
 #include "oxide_browser_process_main.h"
 #include "oxide_download_manager_delegate.h"
 #include "oxide_http_user_agent_settings.h"
@@ -91,6 +90,8 @@ const base::FilePath::CharType kCacheDirname2[] = FILE_PATH_LITERAL("Cache2");
 
 const base::FilePath::CharType kCookiesFilename[] =
     FILE_PATH_LITERAL("cookies.sqlite");
+const base::FilePath::CharType kChannelIDFilename[] =
+    FILE_PATH_LITERAL("ChannelID");
 
 const char kDataScheme[] = "data";
 const char kFileScheme[] = "file";
@@ -193,8 +194,7 @@ struct BrowserContextSharedIOData {
         session_cookie_mode(params.session_cookie_mode),
         popup_blocker_enabled(true),
         host_mapping_rules(params.host_mapping_rules),
-        user_agent_settings(new UserAgentSettingsIOData(context)),
-        do_not_track(false) {}
+        user_agent_settings(new UserAgentSettingsIOData(context)) {}
 
   mutable base::Lock lock;
 
@@ -210,8 +210,6 @@ struct BrowserContextSharedIOData {
 
   std::unique_ptr<UserAgentSettingsIOData> user_agent_settings;
 
-  bool do_not_track;
-
   scoped_refptr<BrowserContextDelegate> delegate;
 };
 
@@ -309,10 +307,6 @@ net::StaticCookiePolicy::Type BrowserContextIOData::GetCookiePolicy() const {
   return data.cookie_policy;
 }
 
-bool BrowserContextIOData::IsPopupBlockerEnabled() const {
-  return GetSharedData().popup_blocker_enabled;
-}
-
 base::FilePath BrowserContextIOData::GetPath() const {
   return GetSharedData().path;
 }
@@ -335,35 +329,35 @@ int BrowserContextIOData::GetMaxCacheSizeHint() const {
   return max_cache_size_hint;
 }
 
-bool BrowserContextIOData::GetDoNotTrack() const {
-  const BrowserContextSharedIOData& data = GetSharedData();
-  base::AutoLock lock(data.lock);
-  return data.do_not_track;
-}
-
 URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
     content::ProtocolHandlerMap& protocol_handlers,
     content::URLRequestInterceptorScopedVector request_interceptors) {
   DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
   DCHECK(!main_request_context_);
 
-  base::FilePath cookie_path;
+  std::unique_ptr<net::CookieStore> cookie_store;
+  scoped_refptr<net::SQLiteChannelIDStore> channel_id_store;
   if (!IsOffTheRecord() && !GetPath().empty()) {
-    cookie_path = GetPath().Append(kCookiesFilename);
+    cookie_store =
+        content::CreateCookieStore(
+            content::CookieStoreConfig(GetPath().Append(kCookiesFilename),
+                                       GetSessionCookieMode(),
+                                       nullptr, nullptr));
+    channel_id_store =
+        new net::SQLiteChannelIDStore(
+            GetPath().Append(kChannelIDFilename),
+            content::BrowserThread::GetBlockingPool()->GetSequencedTaskRunner(
+                base::SequencedWorkerPool::GetSequenceToken()));
+  } else {
+    cookie_store = content::CreateCookieStore(content::CookieStoreConfig());
   }
-  std::unique_ptr<net::CookieStore> cookie_store =
-      content::CreateCookieStore(
-          content::CookieStoreConfig(cookie_path,
-                                     GetSessionCookieMode(),
-                                     nullptr, nullptr));
+
   cookie_store_owner_->set_store(std::move(cookie_store));
 
   IOThread::Globals* io_thread_globals = IOThread::instance()->globals();
 
   ssl_config_service_ = new SSLConfigService();
   http_user_agent_settings_.reset(new HttpUserAgentSettings(this));
-  ftp_transaction_factory_.reset(
-      new net::FtpNetworkLayer(io_thread_globals->host_resolver()));
 
   // TODO: We want persistent storage here (for non-incognito), but the
   //       persistent implementation used in Chrome uses the preferences
@@ -377,7 +371,7 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
       new net::TransportSecurityPersister(
         transport_security_state_.get(),
         GetPath(),
-        content::BrowserThread::GetMessageLoopProxyForThread(
+        content::BrowserThread::GetTaskRunnerForThread(
           content::BrowserThread::FILE),
         IsOffTheRecord()));
 
@@ -398,12 +392,10 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   context->set_network_delegate(network_delegate_.get());
   context->set_http_user_agent_settings(http_user_agent_settings_.get());
 
-  // TODO: We want persistent storage here (for non-incognito), but 
-  //       SQLiteChannelIDStore is part of chrome
   storage->set_channel_id_service(
-      base::WrapUnique(new net::ChannelIDService(
-          new net::DefaultChannelIDStore(nullptr),
-          base::WorkerPool::GetTaskRunner(true))));
+      base::MakeUnique<net::ChannelIDService>(
+          new net::DefaultChannelIDStore(channel_id_store.get()),
+          base::WorkerPool::GetTaskRunner(true)));
 
   context->set_http_server_properties(http_server_properties_.get());
 
@@ -427,7 +419,7 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
           net::CACHE_BACKEND_SIMPLE,
           GetCachePath().Append(kCacheDirname2),
           GetMaxCacheSizeHint() * 1024 * 1024, // MB -> bytes
-          content::BrowserThread::GetMessageLoopProxyForThread(
+          content::BrowserThread::GetTaskRunnerForThread(
               content::BrowserThread::CACHE)));
   }
 
@@ -480,7 +472,7 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
       oxide::kFileScheme,
       base::WrapUnique(
           new net::FileProtocolHandler(
-              content::BrowserThread::GetMessageLoopProxyForThread(
+              content::BrowserThread::GetTaskRunnerForThread(
                   content::BrowserThread::FILE))));
   DCHECK(set_protocol);
   set_protocol = job_factory->SetProtocolHandler(
@@ -490,23 +482,19 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
 
   set_protocol = job_factory->SetProtocolHandler(
       oxide::kFtpScheme,
-      base::WrapUnique(new net::FtpProtocolHandler(
-        ftp_transaction_factory_.get())));
+      net::FtpProtocolHandler::Create(io_thread_globals->host_resolver()));
   DCHECK(set_protocol);
 
   std::unique_ptr<net::URLRequestJobFactory> top_job_factory(
       new URLRequestDelegatedJobFactory(std::move(job_factory),
                                         this));
 
-  for (content::URLRequestInterceptorScopedVector::reverse_iterator it =
-          request_interceptors.rbegin();
-       it != request_interceptors.rend();
-       ++it) {
-    top_job_factory.reset(
-        new net::URLRequestInterceptingJobFactory(std::move(top_job_factory),
-                                                  base::WrapUnique(*it)));
+  for (auto& interceptor : request_interceptors) {
+    top_job_factory =
+        base::MakeUnique<net::URLRequestInterceptingJobFactory>(
+            std::move(top_job_factory),
+            std::move(interceptor));
   }
-  request_interceptors.weak_clear();
 
   storage->set_job_factory(std::move(top_job_factory));
 
@@ -549,24 +537,16 @@ class OTRBrowserContextImpl : public BrowserContext {
  public:
   OTRBrowserContextImpl(BrowserContextImpl* original,
                         BrowserContextIODataImpl* original_io_data);
-
-  base::WeakPtr<OTRBrowserContextImpl> GetWeakPtr() {
-    return weak_ptr_factory_.GetWeakPtr();
-  }
-
- private:
   ~OTRBrowserContextImpl() override;
 
-  scoped_refptr<BrowserContext> GetOffTheRecordContext() override {
-    return make_scoped_refptr(this);
-  }
-  BrowserContext* GetOriginalContext() const override;
+ private:
+  BrowserContext* GetOffTheRecordContext() override { return this; }
+
+  BrowserContext* GetOriginalContext() override;
 
   bool HasOffTheRecordContext() const override { return true; }
 
-  scoped_refptr<BrowserContextImpl> original_context_;
-
-  base::WeakPtrFactory<OTRBrowserContextImpl> weak_ptr_factory_;
+  BrowserContextImpl* original_context_;
 };
 
 class BrowserContextImpl : public BrowserContext {
@@ -574,50 +554,49 @@ class BrowserContextImpl : public BrowserContext {
   BrowserContextImpl(const BrowserContext::Params& params);
 
  private:
+  friend class BrowserContext;
+
   ~BrowserContextImpl() override;
 
-  scoped_refptr<BrowserContext> GetOffTheRecordContext() override;
+  BrowserContext* GetOffTheRecordContext() override;
 
-  BrowserContext* GetOriginalContext() const override {
-    return const_cast<BrowserContextImpl*>(this);
-  }
+  BrowserContext* GetOriginalContext() override { return this; }
 
   bool HasOffTheRecordContext() const override {
     return otr_context_ != nullptr;
   }
 
-  base::WeakPtr<OTRBrowserContextImpl> otr_context_;
+  std::unique_ptr<OTRBrowserContextImpl> otr_context_;
 };
 
-OTRBrowserContextImpl::~OTRBrowserContextImpl() {}
-
-BrowserContext* OTRBrowserContextImpl::GetOriginalContext() const {
-  return original_context_.get();
+BrowserContext* OTRBrowserContextImpl::GetOriginalContext() {
+  return original_context_;
 }
 
+OTRBrowserContextImpl::~OTRBrowserContextImpl() {}
+
 OTRBrowserContextImpl::OTRBrowserContextImpl(
     BrowserContextImpl* original,
     BrowserContextIODataImpl* original_io_data)
     : BrowserContext(new OTRBrowserContextIODataImpl(original_io_data)),
-      original_context_(original),
-      weak_ptr_factory_(this) {
+      original_context_(original) {
   BrowserContextDependencyManager::GetInstance()
       ->CreateBrowserContextServices(this);
 }
 
-BrowserContextImpl::~BrowserContextImpl() {
-  CHECK(!otr_context_);
-}
-
-scoped_refptr<BrowserContext> BrowserContextImpl::GetOffTheRecordContext() {
+BrowserContext* BrowserContextImpl::GetOffTheRecordContext() {
   if (!otr_context_) {
-    OTRBrowserContextImpl* context = new OTRBrowserContextImpl(
-        this,
-        static_cast<BrowserContextIODataImpl *>(io_data()));
-    otr_context_ = context->GetWeakPtr();
+    otr_context_ =
+        base::MakeUnique<OTRBrowserContextImpl>(
+            this,
+            static_cast<BrowserContextIODataImpl*>(io_data()));
   }
 
-  return make_scoped_refptr(otr_context_.get());
+  return otr_context_.get();
+}
+
+BrowserContextImpl::~BrowserContextImpl() {
+  CHECK(!otr_context_);
 }
 
 BrowserContextImpl::BrowserContextImpl(const BrowserContext::Params& params)
@@ -634,8 +613,9 @@ BrowserContextImpl::BrowserContextImpl(const BrowserContext::Params& params)
       ->CreateBrowserContextServices(this);
 }
 
-void BrowserContextTraits::Destruct(const BrowserContext* x) {
-  BrowserContextDestroyer::DestroyContext(const_cast<BrowserContext*>(x));
+void BrowserContext::Deleter::operator()(BrowserContext* context) {
+  CHECK(!context->IsOffTheRecord());
+  BrowserContextDestroyer::DestroyContext(base::WrapUnique(context));
 }
 
 std::unique_ptr<content::ZoomLevelDelegate>
@@ -725,18 +705,8 @@ BrowserContext::CreateMediaRequestContextForStoragePartition(
   return nullptr;
 }
 
-void BrowserContext::AddObserver(BrowserContextObserver* observer) {
-  DCHECK(CalledOnValidThread());
-  observers_.AddObserver(observer);
-}
-
-void BrowserContext::RemoveObserver(BrowserContextObserver* observer) {
-  DCHECK(CalledOnValidThread());
-  observers_.RemoveObserver(observer);
-}
-
-BrowserContext::BrowserContext(BrowserContextIOData* io_data) :
-    io_data_(io_data) {
+BrowserContext::BrowserContext(BrowserContextIOData* io_data)
+    : io_data_(io_data) {
   CHECK(BrowserProcessMain::GetInstance()->IsRunning()) <<
       "The main browser process components must be started before " <<
       "creating a context";
@@ -747,9 +717,9 @@ BrowserContext::BrowserContext(BrowserContextIOData* io_data) :
   cookie_store_ =
       base::MakeUnique<CookieStoreProxy>(
           io_data->cookie_store_owner_->GetWeakPtr(),
-          content::BrowserThread::GetMessageLoopProxyForThread(
+          content::BrowserThread::GetTaskRunnerForThread(
               content::BrowserThread::UI),
-          content::BrowserThread::GetMessageLoopProxyForThread(
+          content::BrowserThread::GetTaskRunnerForThread(
               content::BrowserThread::IO));
 
   content::BrowserContext::Initialize(this, io_data->GetPath());
@@ -759,15 +729,13 @@ BrowserContext::BrowserContext(BrowserContextIOData* io_data) :
 BrowserContext::~BrowserContext() {
   DCHECK(CalledOnValidThread());
 
-  FOR_EACH_OBSERVER(BrowserContextObserver,
-                    observers_,
-                    OnBrowserContextDestruction());
-
   g_all_contexts.Get().erase(this);
 
   BrowserContextDependencyManager::GetInstance()
       ->DestroyBrowserContextServices(this);
 
+  ShutdownStoragePartitions();
+
   // Schedule io_data_ to be destroyed on the IO thread
   content::BrowserThread::DeleteSoon(content::BrowserThread::IO,
                                      FROM_HERE, io_data_);
@@ -775,9 +743,8 @@ BrowserContext::~BrowserContext() {
 }
 
 // static
-scoped_refptr<BrowserContext> BrowserContext::Create(const Params& params) {
-  scoped_refptr<BrowserContext> context = new BrowserContextImpl(params);
-  return context;
+BrowserContext::UniquePtr BrowserContext::Create(const Params& params) {
+  return BrowserContext::UniquePtr(new BrowserContextImpl(params));
 }
 
 // static
@@ -789,7 +756,10 @@ void BrowserContext::ForEach(const BrowserContextCallback& callback) {
 
 // static
 void BrowserContext::AssertNoContextsExist() {
-  CHECK_EQ(g_all_contexts.Get().size(), static_cast<size_t>(0));
+  CHECK_EQ(g_all_contexts.Get().size(), static_cast<size_t>(0))
+      << "BrowserContexts still exist at shutdown! This is normally the result "
+      << "of an application leak, but it's possible that there might be an "
+      << "Oxide bug too";
 }
 
 BrowserContextID BrowserContext::GetID() const {
@@ -809,6 +779,14 @@ void BrowserContext::SetDelegate(BrowserContextDelegate* delegate) {
   data.delegate = delegate;
 }
 
+// static
+void BrowserContext::DestroyOffTheRecordContextForContext(
+    BrowserContext* context) {
+  CHECK(!context->IsOffTheRecord() && context->HasOffTheRecordContext());
+  BrowserContextDestroyer::DestroyContext(
+      std::move(static_cast<BrowserContextImpl*>(context)->otr_context_));
+}
+
 bool BrowserContext::IsOffTheRecord() const {
   DCHECK(CalledOnValidThread());
   return io_data()->IsOffTheRecord();
@@ -818,7 +796,7 @@ bool BrowserContext::IsSameContext(BrowserContext* other) const {
   DCHECK(CalledOnValidThread());
   return other->GetOriginalContext() == this ||
          (other->HasOffTheRecordContext() &&
-          other->GetOffTheRecordContext().get() == this);
+          other->GetOffTheRecordContext() == this);
 }
 
 base::FilePath BrowserContext::GetPath() const {
@@ -855,26 +833,6 @@ BrowserContext::GetSessionCookieMode() const {
   return io_data()->GetSessionCookieMode();
 }
 
-bool BrowserContext::IsPopupBlockerEnabled() const {
-  DCHECK(CalledOnValidThread());
-  return io_data()->IsPopupBlockerEnabled();
-}
-
-void BrowserContext::SetIsPopupBlockerEnabled(bool enabled) {
-  DCHECK(CalledOnValidThread());
-
-  io_data()->GetSharedData().popup_blocker_enabled = enabled;
-
-  FOR_EACH_OBSERVER(BrowserContextObserver,
-                    GetOriginalContext()->observers_,
-                    NotifyPopupBlockerEnabledChanged());
-  if (HasOffTheRecordContext()) {
-    FOR_EACH_OBSERVER(BrowserContextObserver,
-                      GetOffTheRecordContext()->observers_,
-                      NotifyPopupBlockerEnabledChanged());
-  }
-}
-
 const std::vector<std::string>& BrowserContext::GetHostMappingRules() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetSharedData().host_mapping_rules;
@@ -900,26 +858,4 @@ BrowserContextIOData* BrowserContext::GetIOData() const {
   return io_data_;
 }
 
-bool BrowserContext::GetDoNotTrack() const {
-  DCHECK(CalledOnValidThread());
-  return io_data()->GetSharedData().do_not_track;
-}
-
-void BrowserContext::SetDoNotTrack(bool dnt) {
-  DCHECK(CalledOnValidThread());
-
-  BrowserContextSharedIOData& data = io_data()->GetSharedData();
-  base::AutoLock lock(data.lock);
-  data.do_not_track = dnt;
-
-  FOR_EACH_OBSERVER(BrowserContextObserver,
-                    GetOriginalContext()->observers_,
-                    NotifyDoNotTrackChanged());
-  if (HasOffTheRecordContext()) {
-    FOR_EACH_OBSERVER(BrowserContextObserver,
-                      GetOffTheRecordContext()->observers_,
-                      NotifyDoNotTrackChanged());
-  }
-}
-
 } // namespace oxide
