@@ -27,6 +27,7 @@
 #include "base/base_paths.h"
 #include "base/base_switches.h"
 #include "base/command_line.h"
+#include "base/environment.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/i18n/icu_util.h"
@@ -34,8 +35,9 @@
 #include "base/logging.h"
 #include "base/memory/ref_counted.h"
 #include "base/path_service.h"
-#include "base/posix/global_descriptors.h"
+#include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
+#include "base/strings/utf_string_conversions.h"
 #include "cc/base/switches.h"
 #include "content/app/mojo/mojo_init.h" // nogncheck
 #include "content/browser/gpu/gpu_process_host.h" // nogncheck
@@ -69,10 +71,8 @@
 #include "shared/common/oxide_content_client.h"
 #include "shared/common/oxide_form_factor.h"
 
-#include "oxide_browser_context.h"
 #include "oxide_form_factor_detection.h"
 #include "oxide_message_pump.h"
-#include "oxide_web_contents_unloader.h"
 
 namespace content {
 
@@ -134,20 +134,30 @@ class BrowserProcessMainImpl : public BrowserProcessMain {
 
 namespace {
 
-bool IsEnvironmentOptionEnabled(base::StringPiece option) {
+bool IsEnvironmentOptionEnabled(base::StringPiece option,
+                                base::Environment* env) {
   std::string name("OXIDE_");
   name += option.data();
 
-  base::StringPiece val(getenv(name.c_str()));
+  std::string result;
+  if (!env->GetVar(name, &result)) {
+    return false;
+  }
 
-  return !val.empty() && val == "1";
+  return result.size() > 0 && result[0] != '0';
 }
 
-base::StringPiece GetEnvironmentOption(base::StringPiece option) {
+std::string GetEnvironmentOption(base::StringPiece option,
+                                 base::Environment* env) {
   std::string name("OXIDE_");
   name += option.data();
 
-  return getenv(name.c_str());
+  // An environment variable can be set to an empty string, but we don't
+  // expose this to callers
+  std::string result;
+  env->GetVar(name, &result);
+
+  return result;
 }
 
 #if defined(OS_POSIX)
@@ -172,12 +182,18 @@ void SetupAndVerifySignalHandlers() {
 }
 #endif
 
-base::FilePath GetSubprocessPath() {
-  base::StringPiece subprocess_path = GetEnvironmentOption("SUBPROCESS_PATH");
-  if (!subprocess_path.empty()) {
+base::FilePath GetSubprocessPath(base::Environment* env) {
+  std::string override_subprocess_path =
+      GetEnvironmentOption("SUBPROCESS_PATH", env);
+  if (!override_subprocess_path.empty()) {
     // Make sure that we have a properly formed absolute path
     // there are some load issues if not.
-    return base::MakeAbsoluteFilePath(base::FilePath(subprocess_path.data()));
+#if defined(OS_POSIX)
+    base::FilePath subprocess_path(override_subprocess_path);
+#else
+    base::FilePath subprocess_path(base::UTF8ToUTF16(override_subprocess_path));
+#endif
+    return base::MakeAbsoluteFilePath(subprocess_path);
   }
 
   base::FilePath subprocess_exe =
@@ -220,10 +236,40 @@ const char* GetGLImplName(gl::GLImplementation impl) {
   }
 }
 
-void InitializeCommandLine(const base::FilePath& subprocess_path,
+#if defined(OS_POSIX)
+base::FilePath GetSharedMemoryPath(base::Environment* env) {
+  // snap packages
+  std::string snap_name;
+  if (env->GetVar("SNAP_NAME", &snap_name)) {
+    return base::FilePath(std::string("/dev/shm/snap.") + snap_name + ".oxide");
+  }
+
+  // click packages
+  // Ref: https://developer.ubuntu.com/en/phone/platform/guides/app-confinement/#Runtime_Environment
+  std::string app_pkgname;
+  if (env->GetVar("APP_ID", &app_pkgname)) {
+    app_pkgname = app_pkgname.substr(0, app_pkgname.find("_"));
+    if (!app_pkgname.empty()) {
+      return base::FilePath(std::string("/dev/shm/") + app_pkgname + ".oxide");
+    }
+  }
+
+  // default
+  return base::FilePath("/dev/shm");
+}
+#endif
+
+void InitializeCommandLine(const std::string& argv0,
+                           const base::FilePath& subprocess_path,
                            ProcessModel process_model,
-                           gl::GLImplementation gl_impl) {
-  CHECK(base::CommandLine::Init(0, nullptr)) <<
+                           gl::GLImplementation gl_impl,
+                           base::Environment* env) {
+  const char* argv0_c = nullptr;
+  if (!argv0.empty()) {
+    argv0_c = argv0.c_str();
+  }
+  CHECK(base::CommandLine::Init(argv0_c ? 1 : 0,
+                                argv0_c ? &argv0_c : nullptr)) <<
       "CommandLine already exists. Did you call BrowserProcessMain::Start "
       "in a child process?";
 
@@ -254,54 +300,59 @@ void InitializeCommandLine(const base::FilePath& subprocess_path,
   command_line->AppendSwitchASCII(switches::kProfilerTiming,
                                   switches::kProfilerTimingDisabledValue);
 
+  // See https://launchpad.net/bugs/1632490
+  command_line->AppendSwitchASCII(switches::kEnableUseZoomForDSF,
+                                  "false");
+
   if (gl_impl == gl::kGLImplementationNone ||
-      IsEnvironmentOptionEnabled("DISABLE_GPU")) {
+      IsEnvironmentOptionEnabled("DISABLE_GPU", env)) {
     command_line->AppendSwitch(switches::kDisableGpu);
   } else {
     command_line->AppendSwitchASCII(switches::kUseGL,
                                     GetGLImplName(gl_impl));
   }
 
-  if (IsEnvironmentOptionEnabled("DISABLE_GPU_COMPOSITING")) {
+  if (IsEnvironmentOptionEnabled("DISABLE_GPU_COMPOSITING", env)) {
     command_line->AppendSwitch(switches::kDisableGpuCompositing);
   }
 
-  base::StringPiece renderer_cmd_prefix =
-      GetEnvironmentOption("RENDERER_CMD_PREFIX");
+  std::string renderer_cmd_prefix =
+      GetEnvironmentOption("RENDERER_CMD_PREFIX", env);
   if (!renderer_cmd_prefix.empty()) {
     command_line->AppendSwitchASCII(switches::kRendererCmdPrefix,
-                                    renderer_cmd_prefix.data());
+                                    renderer_cmd_prefix);
   }
-  if (IsEnvironmentOptionEnabled("NO_SANDBOX")) {
+  if (IsEnvironmentOptionEnabled("NO_SANDBOX", env) ||
+      process_model == PROCESS_MODEL_SINGLE_PROCESS) {
     command_line->AppendSwitch(switches::kNoSandbox);
   } else {
     // See https://launchpad.net/bugs/1447311
     command_line->AppendSwitch(switches::kDisableNamespaceSandbox);
 
-    if (IsEnvironmentOptionEnabled("DISABLE_SETUID_SANDBOX")) {
-      command_line->AppendSwitch(switches::kDisableSetuidSandbox);
-    }
-    if (IsEnvironmentOptionEnabled("DISABLE_SECCOMP_FILTER_SANDBOX")) {
+    if (IsEnvironmentOptionEnabled("DISABLE_SECCOMP_FILTER_SANDBOX",
+                                   env)) {
       command_line->AppendSwitch(switches::kDisableSeccompFilterSandbox);
     }
   }
 
-  if (IsEnvironmentOptionEnabled("IGNORE_GPU_BLACKLIST")) {
+  if (IsEnvironmentOptionEnabled("IGNORE_GPU_BLACKLIST", env)) {
     command_line->AppendSwitch(switches::kIgnoreGpuBlacklist);
   }
-  if (IsEnvironmentOptionEnabled("DISABLE_GPU_DRIVER_BUG_WORKAROUNDS")) {
+  if (IsEnvironmentOptionEnabled("DISABLE_GPU_DRIVER_BUG_WORKAROUNDS",
+                                 env)) {
     command_line->AppendSwitch(switches::kDisableGpuDriverBugWorkarounds);
   }
 
-  if (IsEnvironmentOptionEnabled("ENABLE_GPU_SERVICE_LOGGING")) {
+  if (IsEnvironmentOptionEnabled("ENABLE_GPU_SERVICE_LOGGING", env)) {
     command_line->AppendSwitch(switches::kEnableGPUServiceLogging);
   }
-  if (IsEnvironmentOptionEnabled("ENABLE_GPU_DEBUGGING")) {
+  if (IsEnvironmentOptionEnabled("ENABLE_GPU_DEBUGGING", env)) {
     command_line->AppendSwitch(switches::kEnableGPUDebugging);
   }
 
   if (process_model == PROCESS_MODEL_SINGLE_PROCESS) {
     command_line->AppendSwitch(switches::kSingleProcess);
+    command_line->AppendSwitch(switches::kNoZygote);
   } else if (process_model == PROCESS_MODEL_PROCESS_PER_VIEW) {
     command_line->AppendSwitch(switches::kProcessPerTab);
   } else if (process_model == PROCESS_MODEL_PROCESS_PER_SITE) {
@@ -313,26 +364,56 @@ void InitializeCommandLine(const base::FilePath& subprocess_path,
            process_model == PROCESS_MODEL_MULTI_PROCESS);
   }
 
-  if (IsEnvironmentOptionEnabled("ALLOW_SANDBOX_DEBUGGING")) {
+  if (IsEnvironmentOptionEnabled("ALLOW_SANDBOX_DEBUGGING", env)) {
     command_line->AppendSwitch(switches::kAllowSandboxDebugging);
   }
 
-  if (IsEnvironmentOptionEnabled("ENABLE_MEDIA_HUB_AUDIO")) {
+  if (IsEnvironmentOptionEnabled("ENABLE_MEDIA_HUB_AUDIO", env)) {
     command_line->AppendSwitch(switches::kEnableMediaHubAudio);
   }
-  base::StringPiece mediahub_fixed_session_domains =
-      GetEnvironmentOption("MEDIA_HUB_FIXED_SESSION_DOMAINS");
+  std::string mediahub_fixed_session_domains =
+      GetEnvironmentOption("MEDIA_HUB_FIXED_SESSION_DOMAINS", env);
   if (!mediahub_fixed_session_domains.empty()) {
     command_line->AppendSwitchASCII(switches::kMediaHubFixedSessionDomains,
-                                    mediahub_fixed_session_domains.data());
-    if (!IsEnvironmentOptionEnabled("ENABLE_MEDIA_HUB_AUDIO")) {
+                                    mediahub_fixed_session_domains);
+    if (!IsEnvironmentOptionEnabled("ENABLE_MEDIA_HUB_AUDIO", env)) {
       command_line->AppendSwitch(switches::kEnableMediaHubAudio);
     }
   }
 
-  if (IsEnvironmentOptionEnabled("TESTING_MODE")) {
+  if (IsEnvironmentOptionEnabled("TESTING_MODE", env)) {
     command_line->AppendSwitch(switches::kUseFakeDeviceForMediaStream);
   }
+
+#if defined(OS_POSIX)
+  command_line->AppendSwitchPath(switches::kSharedMemoryOverridePath,
+                                 GetSharedMemoryPath(env));
+#endif
+
+  // verbose logging
+  const std::string& verbose_log_level =
+      GetEnvironmentOption("VERBOSE_LOG_LEVEL", env);
+  if (!verbose_log_level.empty()) {
+    command_line->AppendSwitch(switches::kEnableLogging);
+    command_line->AppendSwitchASCII(switches::kV, verbose_log_level);
+  }
+
+  const std::string& extra_cmd_arg_list =
+      GetEnvironmentOption("EXTRA_CMD_ARGS", env);
+  if (!extra_cmd_arg_list.empty()) {
+    std::vector<std::string> args =
+        base::SplitString(extra_cmd_arg_list,
+                          base::kWhitespaceASCII,
+                          base::KEEP_WHITESPACE,
+                          base::SPLIT_WANT_NONEMPTY);
+
+    base::CommandLine::StringVector new_args;
+    new_args.push_back(command_line->argv()[0]);
+    new_args.insert(new_args.end(), args.begin(), args.end());
+
+    base::CommandLine extra_cmd_line(new_args);
+    command_line->AppendArguments(extra_cmd_line, false);
+  }
 }
 
 void AddFormFactorSpecificCommandLineArguments() {
@@ -341,36 +422,11 @@ void AddFormFactorSpecificCommandLineArguments() {
   }
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
-  command_line->AppendSwitch(switches::kEnableViewport);
-  command_line->AppendSwitch(switches::kMainFrameResizesAreOrientationChanges);
-  command_line->AppendSwitch(switches::kEnablePinch);
   // Note, overlay scrollbars do not work properly on desktop yet
   // see https://launchpad.net/bugs/1426567
   command_line->AppendSwitch(switches::kEnableOverlayScrollbar);
 }
 
-base::FilePath GetSharedMemoryPath() {
-  // snap packages
-  const char* tmp = getenv("SNAP_NAME");
-  if (tmp) {
-    return base::FilePath(std::string("/dev/shm/snap.") + tmp + ".oxide");
-  }
-
-  // click packages
-  // Ref: https://developer.ubuntu.com/en/phone/platform/guides/app-confinement/#Runtime_Environment
-  tmp = getenv("APP_ID");
-  if (tmp) {
-    std::string app_pkgname(tmp);
-    app_pkgname = app_pkgname.substr(0, app_pkgname.find("_"));
-    if (!app_pkgname.empty()) {
-      return base::FilePath(std::string("/dev/shm/") + app_pkgname + ".oxide");
-    }
-  }
-
-  // default
-  return base::FilePath("/dev/shm");
-}
-
 bool IsUnsupportedProcessModel(ProcessModel process_model) {
   switch (process_model) {
     case PROCESS_MODEL_MULTI_PROCESS:
@@ -404,16 +460,9 @@ BrowserProcessMain::StartParams::StartParams(
       gl_implementation(gl::kGLImplementationNone),
       process_model(PROCESS_MODEL_MULTI_PROCESS) {}
 
-BrowserProcessMain::StartParams::~StartParams() {}
+BrowserProcessMain::StartParams::~StartParams() = default;
 
-BrowserProcessMain::StartParams::StartParams(StartParams&& other)
-    : delegate(std::move(other.delegate)),
-#if defined(OS_LINUX)
-      nss_db_path(std::move(other.nss_db_path)),
-#endif
-      gl_implementation(std::move(other.gl_implementation)),
-      process_model(std::move(other.process_model)),
-      primary_screen_size(std::move(other.primary_screen_size)) {}
+BrowserProcessMain::StartParams::StartParams(StartParams&& other) = default;
 
 BrowserProcessMainImpl::BrowserProcessMainImpl()
     : state_(STATE_NOT_STARTED),
@@ -443,17 +492,18 @@ void BrowserProcessMainImpl::Start(StartParams params) {
 
 #if defined(OS_POSIX)
   SetupAndVerifySignalHandlers();
-
-  base::GlobalDescriptors::GetInstance()->Set(
-      kPrimaryIPCChannel,
-      kPrimaryIPCChannel + base::GlobalDescriptors::kBaseDescriptor);
 #endif
 
   exit_manager_.reset(new base::AtExitManager());
 
-  base::FilePath subprocess_exe = GetSubprocessPath();
-  InitializeCommandLine(subprocess_exe, process_model_,
-                        params.gl_implementation);
+  std::unique_ptr<base::Environment> env = base::Environment::Create();
+
+  base::FilePath subprocess_exe = GetSubprocessPath(env.get());
+  InitializeCommandLine(params.argv0,
+                        subprocess_exe,
+                        process_model_,
+                        params.gl_implementation,
+                        env.get());
 
   FormFactor form_factor =
       DetectFormFactorHint(params.primary_screen_size);
@@ -467,10 +517,6 @@ void BrowserProcessMainImpl::Start(StartParams params) {
   PathService::Override(base::FILE_EXE, subprocess_exe);
   PathService::Override(base::FILE_MODULE, subprocess_exe);
 
-  base::CommandLine::ForCurrentProcess()
-      ->AppendSwitchPath(switches::kSharedMemoryOverridePath,
-                         GetSharedMemoryPath());
-
   int exit_code;
   CHECK(!main_delegate_->BasicStartupComplete(&exit_code));
 
@@ -531,14 +577,6 @@ void BrowserProcessMainImpl::Shutdown() {
 
   MessagePump::Get()->Stop();
 
-  WebContentsUnloader::GetInstance()->WaitForPendingUnloadsToFinish();
-
-  if (process_model_ != PROCESS_MODEL_SINGLE_PROCESS) {
-    // In single process mode, we do this check after destroying
-    // threads, as we hold the single BrowserContext alive until then
-    BrowserContext::AssertNoContextsExist();
-  }
-
   browser_main_runner_->Shutdown();
   browser_main_runner_.reset();
 
@@ -565,25 +603,27 @@ ProcessModel BrowserProcessMain::GetProcessModelOverrideFromEnv() {
 
   g_initialized = true;
 
-  if (IsEnvironmentOptionEnabled("SINGLE_PROCESS")) {
+  std::unique_ptr<base::Environment> env = base::Environment::Create();
+
+  if (IsEnvironmentOptionEnabled("SINGLE_PROCESS", env.get())) {
     g_process_model = PROCESS_MODEL_SINGLE_PROCESS;
   } else {
-    base::StringPiece env = GetEnvironmentOption("PROCESS_MODEL");
-    if (!env.empty()) {
-      if (env == "multi-process") {
+    std::string model = GetEnvironmentOption("PROCESS_MODEL", env.get());
+    if (!model.empty()) {
+      if (model == "multi-process") {
         g_process_model = PROCESS_MODEL_MULTI_PROCESS;
-      } else if (env == "single-process") {
+      } else if (model == "single-process") {
         g_process_model = PROCESS_MODEL_SINGLE_PROCESS;
-      } else if (env == "process-per-site-instance") {
+      } else if (model == "process-per-site-instance") {
         g_process_model = PROCESS_MODEL_PROCESS_PER_SITE_INSTANCE;
-      } else if (env == "process-per-view") {
+      } else if (model == "process-per-view") {
         g_process_model = PROCESS_MODEL_PROCESS_PER_VIEW;
-      } else if (env == "process-per-site") {
+      } else if (model == "process-per-site") {
         g_process_model = PROCESS_MODEL_PROCESS_PER_SITE;
-      } else if (env == "site-per-process") {
+      } else if (model == "site-per-process") {
         g_process_model = PROCESS_MODEL_SITE_PER_PROCESS;
       } else {
-        LOG(WARNING) << "Invalid process mode: " << env.data();
+        LOG(WARNING) << "Invalid process mode: " << model;
       }
     }
   }
