@@ -17,7 +17,8 @@
 
 #include "oxide_geolocation_delegate.h"
 
-#include "content/public/browser/location_provider.h"
+#include "device/geolocation/access_token_store.h"
+#include "device/geolocation/location_provider.h"
 
 #include "oxide_browser_platform_integration.h"
 
@@ -27,14 +28,14 @@ bool GeolocationDelegate::UseNetworkLocationProviders() {
   return false;
 }
 
-content::AccessTokenStore* GeolocationDelegate::CreateAccessTokenStore() {
+scoped_refptr<device::AccessTokenStore>
+GeolocationDelegate::CreateAccessTokenStore() {
   return nullptr;
 }
 
-content::LocationProvider*
+std::unique_ptr<device::LocationProvider>
 GeolocationDelegate::OverrideSystemLocationProvider() {
-  return BrowserPlatformIntegration::GetInstance()
-      ->CreateLocationProvider().release();
+  return BrowserPlatformIntegration::GetInstance()->CreateLocationProvider();
 }
 
 } // namespace oxide
