@@ -26,11 +26,12 @@
 #include "base/memory/scoped_vector.h"
 #include "cc/layers/surface_layer.h"
 #include "cc/output/compositor_frame.h"
-#include "cc/output/compositor_frame_ack.h"
-#include "cc/output/delegated_frame_data.h"
 #include "cc/quads/render_pass.h"
 #include "cc/surfaces/surface.h"
+#include "cc/surfaces/surface_factory.h"
+#include "cc/surfaces/surface_id.h"
 #include "cc/surfaces/surface_id_allocator.h"
+#include "cc/surfaces/surface_info.h"
 #include "cc/surfaces/surface_manager.h"
 #include "content/browser/renderer_host/render_widget_host_delegate.h" // nogncheck
 #include "content/browser/renderer_host/render_widget_host_impl.h" // nogncheck
@@ -42,7 +43,7 @@
 #include "content/public/common/context_menu_params.h"
 #include "third_party/WebKit/public/platform/WebCursorInfo.h"
 #include "third_party/WebKit/public/platform/WebGestureDevice.h"
-#include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
 #include "ui/events/gesture_detection/motion_event.h"
 #include "ui/gfx/geometry/point_conversions.h"
 #include "ui/gfx/geometry/rect_conversions.h"
@@ -65,15 +66,6 @@ namespace {
 
 const float kMobileViewportWidthEpsilon = 0.15f;
 
-bool ShouldSendPinchGesture() {
-  static bool pinch_allowed =
-      base::CommandLine::ForCurrentProcess()->HasSwitch(
-        switches::kEnableViewport) ||
-      base::CommandLine::ForCurrentProcess()->HasSwitch(
-        switches::kEnablePinch);
-  return pinch_allowed;
-}
-
 bool HasFixedPageScale(const cc::CompositorFrameMetadata& frame_metadata) {
   return frame_metadata.min_page_scale_factor ==
          frame_metadata.max_page_scale_factor;
@@ -87,31 +79,12 @@ bool HasMobileViewport(const cc::CompositorFrameMetadata& frame_metadata) {
   return content_width_css <= window_width_dip + kMobileViewportWidthEpsilon;
 }
 
-void SatisfyCallback(cc::SurfaceManager* manager,
-                     const cc::SurfaceSequence& sequence) {
-  std::vector<uint32_t> sequences;
-  sequences.push_back(sequence.sequence);
-  manager->DidSatisfySequences(sequence.id_namespace, &sequences);
-}
-
-void RequireCallback(cc::SurfaceManager* manager,
-                     const cc::SurfaceId& id,
-                     const cc::SurfaceSequence& sequence) {
-  cc::Surface* surface = manager->GetSurfaceForId(id);
-  if (!surface) {
-    LOG(ERROR) << "Attempting to require callback on nonexistent surface";
-    return;
-  }
-  surface->AddDestructionDependency(sequence);
-}
-
 bool HasLocationBarOffsetChanged(const cc::CompositorFrameMetadata& old,
                                  const cc::CompositorFrameMetadata& current) {
-  if (old.location_bar_offset.y() != current.location_bar_offset.y()) {
+  if (old.top_controls_height != current.top_controls_height) {
     return true;
   }
-  if (old.location_bar_content_translation.y() !=
-      current.location_bar_content_translation.y()) {
+  if (old.top_controls_shown_ratio != current.top_controls_shown_ratio) {
     return true;
   }
   return false;
@@ -123,22 +96,27 @@ void RenderWidgetHostView::OnSelectionBoundsChanged(
     const gfx::Rect& anchor_rect,
     const gfx::Rect& focus_rect,
     bool is_anchor_first) {
-  gfx::Rect caret_rect = gfx::UnionRects(anchor_rect, focus_rect);
+  gfx::Rect caret_rect;
+  if (anchor_rect == focus_rect) {
+    caret_rect = anchor_rect;
+  }
 
   size_t selection_cursor_position = 0;
   size_t selection_anchor_position = 0;
 
-  if (selection_range_.IsValid()) {
+  const content::TextInputManager::TextSelection* selection =
+      GetTextInputManager()->GetTextSelection();
+  if (selection && selection->range.IsValid()) {
     if (is_anchor_first) {
       selection_cursor_position =
-          selection_range_.GetMax() - selection_text_offset_;
+          selection->range.GetMax() - selection->offset;
       selection_anchor_position =
-          selection_range_.GetMin() - selection_text_offset_;
+          selection->range.GetMin() - selection->offset;
     } else {
       selection_cursor_position =
-          selection_range_.GetMin() - selection_text_offset_;
+          selection->range.GetMin() - selection->offset;
       selection_anchor_position =
-          selection_range_.GetMax() - selection_text_offset_;
+          selection->range.GetMax() - selection->offset;
     }
   }
 
@@ -180,8 +158,8 @@ gfx::Size RenderWidgetHostView::GetPhysicalBackingSize() const {
   return container_->GetViewSizeInPixels();
 }
 
-bool RenderWidgetHostView::DoTopControlsShrinkBlinkSize() const {
-  return top_controls_shrink_blink_size_;
+bool RenderWidgetHostView::DoBrowserControlsShrinkBlinkSize() const {
+  return browser_controls_shrink_blink_size_;
 }
 
 float RenderWidgetHostView::GetTopControlsHeight() const {
@@ -193,26 +171,21 @@ float RenderWidgetHostView::GetTopControlsHeight() const {
     return 0.0f;
   }
 
-  return container_->GetLocationBarHeight();
+  return container_->GetTopControlsHeight();
 }
 
-void RenderWidgetHostView::FocusedNodeChanged(bool is_editable_node) {
+void RenderWidgetHostView::FocusedNodeChanged(
+    bool is_editable_node,
+    const gfx::Rect& node_bounds_in_screen) {
   ime_bridge_.FocusedNodeChanged(is_editable_node);
 }
 
 void RenderWidgetHostView::OnSwapCompositorFrame(uint32_t output_surface_id,
                                                  cc::CompositorFrame frame) {
-  if (!frame.delegated_frame_data) {
-    DLOG(ERROR) << "Non delegated renderer path is not supported";
-    host_->GetProcess()->ShutdownForBadMessage();
-    return;
-  }
-
-  cc::DelegatedFrameData* frame_data = frame.delegated_frame_data.get();
-
-  if (frame_data->render_pass_list.empty()) {
+  if (frame.render_pass_list.empty()) {
     DLOG(ERROR) << "Invalid delegated frame";
-    host_->GetProcess()->ShutdownForBadMessage();
+    host_->GetProcess()->ShutdownForBadMessage(
+        content::RenderProcessHost::CrashReportMode::GENERATE_CRASH_DUMP);
     return;
   }
 
@@ -234,7 +207,7 @@ void RenderWidgetHostView::OnSwapCompositorFrame(uint32_t output_surface_id,
   cc::CompositorFrameMetadata metadata = frame.metadata.Clone();
 
   float device_scale_factor = metadata.device_scale_factor;
-  cc::RenderPass* root_pass = frame_data->render_pass_list.back().get();
+  cc::RenderPass* root_pass = frame.render_pass_list.back().get();
 
   gfx::Size frame_size = root_pass->output_rect.size();
   gfx::Size frame_size_dip = gfx::Size(
@@ -253,24 +226,23 @@ void RenderWidgetHostView::OnSwapCompositorFrame(uint32_t output_surface_id,
     if (!surface_factory_) {
       DCHECK(manager);
       surface_factory_ =
-          base::WrapUnique(new cc::SurfaceFactory(manager, this));
+          base::MakeUnique<cc::SurfaceFactory>(frame_sink_id_, manager, this);
     }
 
-    if (surface_id_.is_null() ||
+    if (!local_frame_id_.is_valid() ||
         frame_size_dip != last_frame_size_dip_) {
       DestroyDelegatedContent();
 
-      surface_id_ = id_allocator_->GenerateId();
-      DCHECK(!surface_id_.is_null());
-      surface_factory_->Create(surface_id_);
+      local_frame_id_ = id_allocator_->GenerateId();
+      DCHECK(local_frame_id_.is_valid());
 
-      layer_ =
-          cc::SurfaceLayer::Create(base::Bind(&SatisfyCallback,
-                                              base::Unretained(manager)),
-                                   base::Bind(&RequireCallback,
-                                              base::Unretained(manager)));
+      layer_ = cc::SurfaceLayer::Create(manager->reference_factory());
       DCHECK(layer_);
-      layer_->SetSurfaceId(surface_id_, device_scale_factor, frame_size);
+
+      layer_->SetSurfaceInfo(
+          cc::SurfaceInfo(cc::SurfaceId(frame_sink_id_, local_frame_id_),
+                          device_scale_factor,
+                          frame_size));
       layer_->SetBounds(frame_size_dip);
       layer_->SetIsDrawable(true);
       layer_->SetContentsOpaque(true);
@@ -282,7 +254,7 @@ void RenderWidgetHostView::OnSwapCompositorFrame(uint32_t output_surface_id,
     cc::SurfaceFactory::DrawCallback ack_callback =
         base::Bind(&RenderWidgetHostView::RunAckCallbacks,
                    weak_ptr_factory_.GetWeakPtr());
-    surface_factory_->SubmitCompositorFrame(surface_id_,
+    surface_factory_->SubmitCompositorFrame(local_frame_id_,
                                             std::move(frame),
                                             ack_callback);
   }
@@ -292,10 +264,10 @@ void RenderWidgetHostView::OnSwapCompositorFrame(uint32_t output_surface_id,
   }
 
   bool shrink =
-      metadata.location_bar_offset.y() == 0.0f &&
-      metadata.location_bar_content_translation.y() > 0.0f;
-  if (shrink != top_controls_shrink_blink_size_) {
-    top_controls_shrink_blink_size_ = shrink;
+      metadata.top_controls_height > 0.f &&
+      metadata.top_controls_shown_ratio == 1.f;
+  if (shrink != browser_controls_shrink_blink_size_) {
+    browser_controls_shrink_blink_size_ = shrink;
     host_->WasResized();
   }
 
@@ -305,7 +277,7 @@ void RenderWidgetHostView::OnSwapCompositorFrame(uint32_t output_surface_id,
       !has_fixed_page_scale && !has_mobile_viewport);
 
   if (host_->is_hidden()) {
-    RunAckCallbacks(cc::SurfaceDrawStatus::DRAW_SKIPPED);
+    RunAckCallbacks();
   }
 
   const cc::Selection<gfx::SelectionBound>& selection = metadata.selection;
@@ -330,6 +302,10 @@ void RenderWidgetHostView::ProcessAckedTouchEvent(
       ack_result == content::INPUT_EVENT_ACK_STATE_CONSUMED);
 }
 
+cc::FrameSinkId RenderWidgetHostView::GetFrameSinkId() {
+  return frame_sink_id_;
+}
+
 void RenderWidgetHostView::InitAsPopup(
     content::RenderWidgetHostView* parent_host_view,
     const gfx::Rect& pos) {
@@ -364,19 +340,6 @@ void RenderWidgetHostView::SetIsLoading(bool is_loading) {
   UpdateCurrentCursor();
 }
 
-void RenderWidgetHostView::TextInputStateChanged(
-    const content::TextInputState& params) {
-  ime_bridge_.TextInputStateChanged(params.type, params.show_ime_if_needed);
-}
-
-void RenderWidgetHostView::ImeCancelComposition() {
-  if (!ime_bridge_.context()) {
-    return;
-  }
-
-  ime_bridge_.context()->CancelComposition();
-}
-
 void RenderWidgetHostView::RenderProcessGone(base::TerminationStatus status,
                                              int error_code) {
   Destroy();
@@ -416,15 +379,6 @@ bool RenderWidgetHostView::HasAcceleratedSurface(
   return false;
 }
 
-void RenderWidgetHostView::GetScreenInfo(blink::WebScreenInfo* result) {
-  if (!container_) {
-    RenderWidgetHostViewOxide::GetDefaultScreenInfo(result);
-    return;
-  }
-
-  *result = container_->GetScreenInfo();
-}
-
 gfx::Rect RenderWidgetHostView::GetBoundsInRootWindow() {
   return GetViewBounds();
 }
@@ -441,10 +395,6 @@ void RenderWidgetHostView::UnlockCompositingSurface() {
   NOTIMPLEMENTED();
 }
 
-void RenderWidgetHostView::ImeCompositionRangeChanged(
-    const gfx::Range& range,
-    const std::vector<gfx::Rect>& character_bounds) {}
-
 void RenderWidgetHostView::InitAsChild(gfx::NativeView parent_view) {}
 
 gfx::Vector2dF RenderWidgetHostView::GetLastScrollOffset() const {
@@ -485,7 +435,7 @@ gfx::Rect RenderWidgetHostView::GetViewBounds() const {
     bounds = container_->GetViewBounds();
   }
 
-  if (DoTopControlsShrinkBlinkSize()) {
+  if (DoBrowserControlsShrinkBlinkSize()) {
     bounds.Inset(0, GetTopControlsHeight(), 0, 0);
   }
 
@@ -498,9 +448,11 @@ bool RenderWidgetHostView::LockMouse() {
 
 void RenderWidgetHostView::UnlockMouse() {}
 
+void RenderWidgetHostView::SetNeedsBeginFrames(bool needs_begin_frames) {}
+
 void RenderWidgetHostView::CompositorDidCommit() {
   committed_frame_metadata_ = std::move(last_submitted_frame_metadata_);
-  RunAckCallbacks(cc::SurfaceDrawStatus::DRAWN);
+  RunAckCallbacks();
 }
 
 void RenderWidgetHostView::CompositorWillRequestSwapFrame() {
@@ -533,27 +485,20 @@ void RenderWidgetHostView::OnGestureEvent(
     return;
   }
 
-  if ((event.type == blink::WebInputEvent::GesturePinchBegin ||
-       event.type == blink::WebInputEvent::GesturePinchUpdate ||
-       event.type == blink::WebInputEvent::GesturePinchEnd) &&
-      !ShouldSendPinchGesture()) {
-    return;
-  }
-
   if (HandleGestureForTouchSelection(event)) {
     return;
   }
 
-  if (event.type == blink::WebInputEvent::GestureTapDown) {
+  if (event.type() == blink::WebInputEvent::GestureTapDown) {
     // Webkit does not stop a fling-scroll on tap-down. So explicitly send an
     // event to stop any in-progress flings.
     blink::WebGestureEvent fling_cancel = event;
-    fling_cancel.type = blink::WebInputEvent::GestureFlingCancel;
+    fling_cancel.setType(blink::WebInputEvent::GestureFlingCancel);
     fling_cancel.sourceDevice = blink::WebGestureDeviceTouchpad;
     host_->ForwardGestureEvent(fling_cancel);
   }
 
-  if (event.type == blink::WebInputEvent::Undefined) {
+  if (event.type() == blink::WebInputEvent::Undefined) {
     return;
   }
 
@@ -581,10 +526,10 @@ bool RenderWidgetHostView::HandleContextMenu(
 
 bool RenderWidgetHostView::HandleGestureForTouchSelection(
     const blink::WebGestureEvent& event) const {
-  switch (event.type) {
+  switch (event.type()) {
     case blink::WebInputEvent::GestureLongPress: {
       base::TimeTicks event_time = base::TimeTicks() +
-          base::TimeDelta::FromSecondsD(event.timeStampSeconds);
+          base::TimeDelta::FromSecondsD(event.timeStampSeconds());
       gfx::PointF location(event.x, event.y);
       if (selection_controller_->WillHandleLongPressEvent(
               event_time, location)) {
@@ -642,6 +587,32 @@ void RenderWidgetHostView::ReturnResources(
 void RenderWidgetHostView::SetBeginFrameSource(
     cc::BeginFrameSource* begin_frame_source) {}
 
+void RenderWidgetHostView::OnUpdateTextInputStateCalled(
+    content::TextInputManager* text_input_manager,
+    content::RenderWidgetHostViewBase* updated_view,
+    bool did_update_state) {
+  if (updated_view != this) {
+    return;
+  }
+
+  const content::TextInputState* state =
+      GetTextInputManager()->GetTextInputState();
+
+  ime_bridge_.TextInputStateChanged(
+      state ? state->type : ui::TEXT_INPUT_TYPE_NONE,
+      state ? state->show_ime_if_needed : false);
+}
+
+void RenderWidgetHostView::OnImeCancelComposition(
+    content::TextInputManager* text_input_manager,
+    content::RenderWidgetHostViewBase* updated_view) {
+  if (ime_bridge_.context() || updated_view != this) {
+    return;
+  }
+
+  ime_bridge_.context()->CancelComposition();
+}
+
 bool RenderWidgetHostView::SupportsAnimation() const {
   return false;
 }
@@ -723,40 +694,37 @@ void RenderWidgetHostView::UpdateCurrentCursor() {
 
 void RenderWidgetHostView::DestroyDelegatedContent() {
   DetachLayer();
-  if (!surface_id_.is_null()) {
+  if (local_frame_id_.is_valid()) {
     DCHECK(surface_factory_.get());
-    surface_factory_->Destroy(surface_id_);
-    surface_id_ = cc::SurfaceId();
+    surface_factory_->EvictSurface();
+    local_frame_id_ = cc::LocalFrameId();
   }
   layer_ = nullptr;
 }
 
 void RenderWidgetHostView::SendDelegatedFrameAck(uint32_t surface_id) {
-  cc::CompositorFrameAck ack;
-  ack.resources.swap(surface_returned_resources_);
-
-  content::RenderWidgetHostImpl::SendSwapCompositorFrameAck(
+  content::RenderWidgetHostImpl::SendReclaimCompositorResources(
       host_->GetRoutingID(),
       surface_id,
       host_->GetProcess()->GetID(),
-      ack);
+      true, // is_swap_ack
+      surface_returned_resources_);
+  surface_returned_resources_.clear();
 }
 
 void RenderWidgetHostView::SendReturnedDelegatedResources() {
   DCHECK(host_);
 
-  cc::CompositorFrameAck ack;
-  DCHECK(!surface_returned_resources_.empty());
-  ack.resources.swap(surface_returned_resources_);
-
   content::RenderWidgetHostImpl::SendReclaimCompositorResources(
       host_->GetRoutingID(),
       last_output_surface_id_,
       host_->GetProcess()->GetID(),
-      ack);
+      false, // is_swap_ack
+      surface_returned_resources_);
+  surface_returned_resources_.clear();
 }
 
-void RenderWidgetHostView::RunAckCallbacks(cc::SurfaceDrawStatus status) {
+void RenderWidgetHostView::RunAckCallbacks() {
   while (!ack_callbacks_.empty()) {
     ack_callbacks_.front().Run();
     ack_callbacks_.pop();
@@ -789,13 +757,13 @@ RenderWidgetHostView::RenderWidgetHostView(
     content::RenderWidgetHostImpl* host)
     : host_(host),
       container_(nullptr),
-      id_allocator_(
-          CompositorUtils::GetInstance()->CreateSurfaceIdAllocator()),
+      frame_sink_id_(CompositorUtils::GetInstance()->AllocateFrameSinkId()),
+      id_allocator_(new cc::SurfaceIdAllocator()),
       last_output_surface_id_(0),
       ime_bridge_(this),
       is_loading_(false),
       is_showing_(!host->is_hidden()),
-      top_controls_shrink_blink_size_(false),
+      browser_controls_shrink_blink_size_(false),
       gesture_provider_(GestureProvider::Create(this)),
       handle_drag_in_progress_(false),
       weak_ptr_factory_(this) {
@@ -813,12 +781,28 @@ RenderWidgetHostView::RenderWidgetHostView(
   tsc_config.enable_longpress_drag_selection = false;
   selection_controller_.reset(
       new ui::TouchSelectionController(this, tsc_config));
+
+  CompositorUtils::GetInstance()
+      ->GetSurfaceManager()
+      ->RegisterFrameSinkId(frame_sink_id_);
+
+  if (GetTextInputManager()) {
+    GetTextInputManager()->AddObserver(this);
+  }
 }
 
 RenderWidgetHostView::~RenderWidgetHostView() {
   DCHECK(!layer_);
   DCHECK(!surface_factory_);
-  DCHECK(surface_id_.is_null());
+  DCHECK(!local_frame_id_.is_valid());
+
+  if (text_input_manager_) {
+    text_input_manager_->RemoveObserver(this);
+  }
+
+  CompositorUtils::GetInstance()
+      ->GetSurfaceManager()
+      ->InvalidateFrameSinkId(frame_sink_id_);
 }
 
 void RenderWidgetHostView::SetContainer(
@@ -842,6 +826,24 @@ void RenderWidgetHostView::SetContainer(
   host_->WasResized();
 }
 
+base::string16 RenderWidgetHostView::GetSelectionText() const {
+  auto* self = const_cast<RenderWidgetHostView*>(this);
+  if (!self->GetTextInputManager() ||
+      !self->GetTextInputManager()->GetTextSelection(self)) {
+    return base::string16();
+  }
+  return self->GetTextInputManager()->GetTextSelection(self)->text;
+}
+
+gfx::Range RenderWidgetHostView::GetSelectionRange() const {
+  auto* self = const_cast<RenderWidgetHostView*>(this);
+  if (!self->GetTextInputManager() ||
+      !self->GetTextInputManager()->GetTextSelection(self)) {
+    return gfx::Range();
+  }
+  return self->GetTextInputManager()->GetTextSelection(self)->range;
+}
+
 void RenderWidgetHostView::Blur() {
   host_->SetActive(false);
   host_->Blur();
@@ -893,7 +895,7 @@ void RenderWidgetHostView::SetBounds(const gfx::Rect& rect) {
 }
 
 void RenderWidgetHostView::Focus() {
-  host_->Focus();
+  host_->GotFocus();
   host_->SetActive(true);
 }
 
@@ -924,7 +926,7 @@ void RenderWidgetHostView::Hide() {
     layer_->SetHideLayerAndSubtree(true);
   }
 
-  RunAckCallbacks(cc::SurfaceDrawStatus::DRAW_SKIPPED);
+  RunAckCallbacks();
 
   if (!host_ || host_->is_hidden()) {
     return;
