@@ -29,8 +29,8 @@
 #include "content/public/browser/web_contents.h"
 #include "third_party/skia/include/core/SkImage.h"
 #include "third_party/WebKit/public/platform/WebDragOperation.h"
-#include "third_party/WebKit/public/platform/WebScreenInfo.h"
-#include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebMouseEvent.h"
 #include "ui/base/clipboard/clipboard.h"
 #include "ui/base/ime/text_input_type.h"
 #include "ui/gfx/geometry/rect_conversions.h"
@@ -42,20 +42,20 @@
 #include "shared/browser/compositor/oxide_compositor.h"
 #include "shared/browser/compositor/oxide_compositor_frame_data.h"
 #include "shared/browser/compositor/oxide_compositor_frame_handle.h"
+#include "shared/browser/context_menu/web_context_menu_host.h"
 #include "shared/browser/input/oxide_input_method_context.h"
 #include "shared/common/oxide_enum_flags.h"
 #include "shared/common/oxide_messages.h"
 #include "shared/common/oxide_unowned_user_data.h"
 
+#include "chrome_controller.h"
 #include "oxide_browser_platform_integration.h"
 #include "oxide_drag_source.h"
 #include "oxide_fullscreen_helper.h"
 #include "oxide_render_widget_host_view.h"
-#include "oxide_screen_client.h"
 #include "oxide_web_contents_view_client.h"
-#include "oxide_web_context_menu.h"
-#include "oxide_web_popup_menu_impl.h"
-#include "oxide_web_view.h"
+#include "screen.h"
+#include "web_popup_menu_host.h"
 
 namespace oxide {
 
@@ -72,7 +72,7 @@ WebContentsView::WebContentsView(content::WebContents* web_contents)
       root_layer_(cc::SolidColorLayer::Create()),
       current_drag_allowed_ops_(blink::WebDragOperationNone),
       current_drag_op_(blink::WebDragOperationNone),
-      render_frame_message_source_(nullptr) {
+      chrome_controller_(nullptr) {
   web_contents->SetUserData(&kUserDataKey,
                             new UnownedUserData<WebContentsView>(this));
 
@@ -83,6 +83,9 @@ WebContentsView::WebContentsView(content::WebContents* web_contents)
   compositor_->SetRootLayer(root_layer_);
 
   CompositorObserver::Observe(compositor_.get());
+
+  ChromeController::CreateForWebContents(web_contents);
+  chrome_controller_ = ChromeController::FromWebContents(web_contents);
 }
 
 content::WebContentsImpl* WebContentsView::web_contents_impl() const {
@@ -160,7 +163,7 @@ gfx::RectF WebContentsView::GetBoundsF() const {
   }
 
   if (ViewSizeShouldBeScreenSize()) {
-    return gfx::RectF(GetScreenInfo().rect);
+    return gfx::RectF(GetDisplay().bounds());
   }
 
   return client_->GetBounds();
@@ -196,7 +199,7 @@ bool WebContentsView::ViewSizeShouldBeScreenSize() const {
 }
 
 void WebContentsView::ResizeCompositorViewport() {
-  compositor_->SetDeviceScaleFactor(GetScreenInfo().deviceScaleFactor);
+  compositor_->SetDeviceScaleFactor(GetDisplay().device_scale_factor());
   compositor_->SetViewportSize(GetSizeInPixels());
 }
 
@@ -211,26 +214,19 @@ void WebContentsView::UpdateContentsSize() {
   }
 }
 
-void WebContentsView::OnShowPopup(const OxideHostMsg_ShowPopup_Params& params) {
-  // XXX: We should do better than this
-  DCHECK(!active_popup_menu_);
+void WebContentsView::DidCloseContextMenu() {
+  DCHECK(active_context_menu_);
 
-  WebPopupMenuImpl* menu =
-      new WebPopupMenuImpl(render_frame_message_source_,
-                           params.popup_items,
-                           params.selected_item,
-                           params.allow_multiple_selection);
-  active_popup_menu_ = menu->GetWeakPtr();
-
-  menu->Show(params.bounds);
+  base::ThreadTaskRunnerHandle::Get()->DeleteSoon(
+      FROM_HERE,
+      active_context_menu_.release());
 }
 
-void WebContentsView::OnHidePopup() {
-  if (!active_popup_menu_) {
-    return;
-  }
+void WebContentsView::DidHidePopupMenu() {
+  DCHECK(active_popup_menu_);
 
-  active_popup_menu_->Hide();
+  base::ThreadTaskRunnerHandle::Get()->DeleteSoon(FROM_HERE,
+                                                  active_popup_menu_.release());
 }
 
 gfx::NativeView WebContentsView::GetNativeView() const {
@@ -245,6 +241,11 @@ gfx::NativeWindow WebContentsView::GetTopLevelNativeWindow() const {
   return nullptr;
 }
 
+void WebContentsView::GetScreenInfo(content::ScreenInfo* screen_info) const {
+  content::WebContentsViewOxide::GetScreenInfoForDisplay(GetDisplay(),
+                                                         screen_info);
+}
+
 void WebContentsView::GetContainerBounds(gfx::Rect* out) const {
   *out = GetBounds();
 }
@@ -328,16 +329,13 @@ void WebContentsView::ShowContextMenu(
     return;
   }
 
-  if (!client_) {
-    return;
-  }
-
-  WebContextMenu* menu = client_->CreateContextMenu(render_frame_host, params);
-  if (!menu) {
-    return;
-  }
-
-  menu->Show();
+  active_context_menu_ =
+      base::MakeUnique<WebContextMenuHost>(
+          render_frame_host,
+          params,
+          base::Bind(&WebContentsView::DidCloseContextMenu,
+                     base::Unretained(this)));
+  active_context_menu_->Show();
 }
 
 void WebContentsView::StartDragging(
@@ -345,22 +343,26 @@ void WebContentsView::StartDragging(
     blink::WebDragOperationsMask allowed_ops,
     const gfx::ImageSkia& image,
     const gfx::Vector2d& image_offset,
-    const content::DragEventSourceInfo& event_info) {
+    const content::DragEventSourceInfo& event_info,
+    content::RenderWidgetHostImpl* source_rwh) {
   if (drag_source_) {
     LOG(WARNING) <<
         "Rejecting request to start a drag when one is already in progress";
-    web_contents()->SystemDragEnded();
+    web_contents()->SystemDragEnded(source_rwh);
     return;
   }
 
+  DCHECK(!drag_source_rwh_.IsValid());
+
   drag_source_ =
       BrowserPlatformIntegration::GetInstance()->CreateDragSource(this);
   if (!drag_source_) {
     LOG(WARNING) <<
         "Rejecting request to start a drag - not supported";
-    web_contents()->SystemDragEnded();
+    web_contents()->SystemDragEnded(source_rwh);
     return;
   }
+  drag_source_rwh_ = RenderWidgetHostID(source_rwh);
 
   HideTouchSelectionController();
 
@@ -369,7 +371,7 @@ void WebContentsView::StartDragging(
   // Recreate it with the correct scale and dimenstions
   gfx::ImageSkia new_image =
       gfx::ImageSkia(gfx::ImageSkiaRep(image.GetRepresentation(1).sk_bitmap(),
-                                       GetScreenInfo().deviceScaleFactor));
+                                       GetDisplay().device_scale_factor()));
 
   drag_source_->StartDragging(web_contents(),
                               drop_data,
@@ -382,6 +384,47 @@ void WebContentsView::UpdateDragCursor(blink::WebDragOperation operation) {
   current_drag_op_ = operation;
 }
 
+void WebContentsView::GotFocus() {
+  web_contents_impl()->NotifyWebContentsFocused();
+}
+
+void WebContentsView::ShowPopupMenu(content::RenderFrameHost* render_frame_host,
+                                    const gfx::Rect& bounds,
+                                    int item_height,
+                                    double item_font_size,
+                                    int selected_item,
+                                    const std::vector<content::MenuItem>& items,
+                                    bool right_aligned,
+                                    bool allow_multiple_selection) {
+  active_popup_menu_ =
+      base::MakeUnique<WebPopupMenuHost>(
+          render_frame_host,
+          items,
+          selected_item,
+          allow_multiple_selection,
+          bounds,
+          base::Bind(&WebContentsView::DidHidePopupMenu,
+                     base::Unretained(this)));
+  active_popup_menu_->Show();
+}
+
+void WebContentsView::HidePopupMenu() {
+  active_popup_menu_.reset();
+}
+
+void WebContentsView::RenderFrameDeleted(
+    content::RenderFrameHost* render_frame_host) {
+  if (active_popup_menu_ &&
+      active_popup_menu_->GetRenderFrameHost() == render_frame_host) {
+    active_popup_menu_.reset();
+  }
+
+  if (active_context_menu_ &&
+      active_context_menu_->GetRenderFrameHost() == render_frame_host) {
+    active_context_menu_.reset();
+  }
+}
+
 void WebContentsView::RenderViewHostChanged(
     content::RenderViewHost* old_host,
     content::RenderViewHost* new_host) {
@@ -539,25 +582,6 @@ void WebContentsView::DidDetachInterstitialPage() {
   TouchSelectionChanged(orig_rwhv, false, false);
 }
 
-bool WebContentsView::OnMessageReceived(
-    const IPC::Message& message,
-    content::RenderFrameHost* render_frame_host) {
-  DCHECK(!render_frame_message_source_);
-  render_frame_message_source_ = render_frame_host;
-
-  bool handled = true;
-  IPC_BEGIN_MESSAGE_MAP(WebContentsView, message)
-    IPC_MESSAGE_HANDLER(OxideHostMsg_ShowPopup, OnShowPopup)
-    IPC_MESSAGE_HANDLER(OxideHostMsg_HidePopup, OnHidePopup)
-    IPC_MESSAGE_UNHANDLED(handled = false)
-  IPC_END_MESSAGE_MAP()
-
-  DCHECK_EQ(render_frame_message_source_, render_frame_host);
-  render_frame_message_source_ = nullptr;
-
-  return handled;
-}
-
 void WebContentsView::CompositorSwapFrame(CompositorFrameHandle* handle,
                                           const SwapAckCallback& callback) {
   compositor_ack_callbacks_.push(callback);
@@ -596,20 +620,28 @@ void WebContentsView::CompositorEvictResources() {
 void WebContentsView::EndDrag(blink::WebDragOperation operation) {
   DCHECK(drag_source_);
 
+  content::RenderWidgetHost* source_rwh = drag_source_rwh_.ToInstance();
+  if (!source_rwh) {
+    return;
+  }
+
   gfx::Point screen_point =
       BrowserPlatformIntegration::GetInstance()
-        ->GetScreenClient()
+        ->GetScreen()
         ->GetCursorScreenPoint();
   gfx::Point view_point =
       screen_point - gfx::Vector2d(GetBounds().origin().x(),
                                    GetBounds().origin().y());
-  web_contents_impl()->DragSourceEndedAt(view_point.x(), view_point.y(),
-                                         screen_point.x(), screen_point.y(),
-                                         operation);
+  web_contents_impl()->DragSourceEndedAt(
+      view_point.x(), view_point.y(),
+      screen_point.x(), screen_point.y(),
+      operation,
+      content::RenderWidgetHostImpl::From(source_rwh));
 
-  web_contents()->SystemDragEnded();
+  web_contents()->SystemDragEnded(source_rwh);
 
   drag_source_.reset();
+  drag_source_rwh_ = RenderWidgetHostID();
 }
 
 void WebContentsView::InputPanelVisibilityChanged() {
@@ -656,14 +688,8 @@ bool WebContentsView::IsFullscreen() const {
   return helper->IsFullscreen();
 }
 
-float WebContentsView::GetLocationBarHeight() const {
-  // TODO: Add LocationBarController class
-  WebView* view = WebView::FromWebContents(web_contents());
-  if (!view) {
-    return 0.f;
-  }
-
-  return view->GetLocationBarHeight();
+float WebContentsView::GetTopControlsHeight() {
+  return chrome_controller_->top_controls_height();
 }
 
 ui::TouchHandleDrawable* WebContentsView::CreateTouchHandleDrawable() const {
@@ -674,10 +700,9 @@ ui::TouchHandleDrawable* WebContentsView::CreateTouchHandleDrawable() const {
   return client_->CreateTouchHandleDrawable();
 }
 
-void WebContentsView::TouchSelectionChanged(
-    RenderWidgetHostView* view,
-    bool handle_drag_in_progress,
-    bool insertion_handle_tapped) const {
+void WebContentsView::TouchSelectionChanged(RenderWidgetHostView* view,
+                                            bool handle_drag_in_progress,
+                                            bool insertion_handle_tapped) {
   if (!client_) {
     return;
   }
@@ -697,13 +722,7 @@ void WebContentsView::TouchSelectionChanged(
     bounds = controller->GetRectBetweenBounds();
   }
 
-  // TODO: Add LocationBarController class
-  float offset = 0.f;
-  WebView* web_view = WebView::FromWebContents(web_contents());
-  if (web_view) {
-    offset = web_view->GetLocationBarContentOffset();
-  }
-  bounds.Offset(0, offset);
+  bounds.Offset(0, chrome_controller_->GetTopContentOffset());
 
   client_->TouchSelectionChanged(status,
                                  bounds,
@@ -721,6 +740,15 @@ void WebContentsView::EditingCapabilitiesChanged(RenderWidgetHostView* view) {
   }
 }
 
+void WebContentsView::OnDisplayPropertiesChanged(
+    const display::Display& display) {
+  if (display.id() != GetDisplay().id()) {
+    return;
+  }
+
+  ScreenChanged();
+}
+
 WebContentsView::~WebContentsView() {
   if (client_) {
     DCHECK_EQ(client_->view_, this);
@@ -777,7 +805,7 @@ void WebContentsView::SetClient(WebContentsViewClient* client) {
   WasResized();
   VisibilityChanged();
   FocusChanged();
-  ScreenUpdated();
+  ScreenChanged();
 
   // Update client from view
   CursorChanged(GetRenderWidgetHostView());
@@ -806,21 +834,22 @@ gfx::Size WebContentsView::GetSize() const {
 
 gfx::Size WebContentsView::GetSizeInPixels() const {
   return gfx::ToRoundedSize(
-      gfx::ScaleSize(GetBoundsF().size(), GetScreenInfo().deviceScaleFactor));
+      gfx::ScaleSize(GetBoundsF().size(), GetDisplay().device_scale_factor()));
 }
 
 gfx::Rect WebContentsView::GetBounds() const {
   return gfx::ToEnclosingRect(GetBoundsF());
 }
 
-blink::WebScreenInfo WebContentsView::GetScreenInfo() const {
-  if (!client_) {
-    blink::WebScreenInfo result;
-    content::RenderWidgetHostViewBase::GetDefaultScreenInfo(&result);
-    return result;
+display::Display WebContentsView::GetDisplay() const {
+  if (client_) {
+    display::Display display = client_->GetDisplay();
+    if (display.is_valid()) {
+      return display;
+    }
   }
 
-  return client_->GetScreenInfo();
+  return Screen::GetInstance()->GetPrimaryDisplay();
 }
 
 void WebContentsView::HandleKeyEvent(
@@ -879,14 +908,14 @@ void WebContentsView::HandleDragEnter(
   current_drop_data_.reset(new content::DropData(drop_data));
   current_drag_allowed_ops_ = allowed_ops;
 
-  content::RenderViewHost* rvh = web_contents()->GetRenderViewHost();
-  current_drag_target_ = RenderWidgetHostID(rvh->GetWidget());
+  content::RenderWidgetHost* rwh = GetRenderWidgetHost();
+  current_drag_target_ = RenderWidgetHostID(rwh);
 
   gfx::Point screen_location =
       BrowserPlatformIntegration::GetInstance()
-        ->GetScreenClient()
-        ->GetCursorScreenPoint();
-  rvh->DragTargetDragEnter(*current_drop_data_,
+          ->GetScreen()
+          ->GetCursorScreenPoint();
+  rwh->DragTargetDragEnter(*current_drop_data_,
                            location,
                            screen_location,
                            current_drag_allowed_ops_,
@@ -900,8 +929,8 @@ blink::WebDragOperation WebContentsView::HandleDragMove(
     return blink::WebDragOperationNone;
   }
 
-  content::RenderViewHost* rvh = web_contents()->GetRenderViewHost();
-  if (RenderWidgetHostID(rvh->GetWidget()) != current_drag_target_) {
+  content::RenderWidgetHost* rwh = GetRenderWidgetHost();
+  if (RenderWidgetHostID(rwh) != current_drag_target_) {
     HandleDragEnter(*current_drop_data_,
                     location,
                     current_drag_allowed_ops_,
@@ -910,9 +939,9 @@ blink::WebDragOperation WebContentsView::HandleDragMove(
 
   gfx::Point screen_location =
       BrowserPlatformIntegration::GetInstance()
-        ->GetScreenClient()
+        ->GetScreen()
         ->GetCursorScreenPoint();
-  rvh->DragTargetDragOver(location,
+  rwh->DragTargetDragOver(location,
                           screen_location,
                           current_drag_allowed_ops_,
                           key_modifiers);
@@ -927,12 +956,12 @@ void WebContentsView::HandleDragLeave() {
 
   current_drop_data_.reset();
 
-  content::RenderViewHost* rvh = web_contents()->GetRenderViewHost();
-  if (RenderWidgetHostID(rvh->GetWidget()) != current_drag_target_) {
+  content::RenderWidgetHost* rwh = GetRenderWidgetHost();
+  if (RenderWidgetHostID(rwh) != current_drag_target_) {
     return;
   }
 
-  rvh->DragTargetDragLeave();
+  rwh->DragTargetDragLeave();
 }
 
 blink::WebDragOperation WebContentsView::HandleDrop(const gfx::Point& location,
@@ -941,8 +970,8 @@ blink::WebDragOperation WebContentsView::HandleDrop(const gfx::Point& location,
     return blink::WebDragOperationNone;
   }
 
-  content::RenderViewHost* rvh = web_contents()->GetRenderViewHost();
-  if (RenderWidgetHostID(rvh->GetWidget()) != current_drag_target_) {
+  content::RenderWidgetHost* rwh = GetRenderWidgetHost();
+  if (RenderWidgetHostID(rwh) != current_drag_target_) {
     HandleDragEnter(*current_drop_data_,
                     location,
                     current_drag_allowed_ops_,
@@ -951,9 +980,9 @@ blink::WebDragOperation WebContentsView::HandleDrop(const gfx::Point& location,
 
   gfx::Point screen_location =
       BrowserPlatformIntegration::GetInstance()
-        ->GetScreenClient()
+        ->GetScreen()
         ->GetCursorScreenPoint();
-  rvh->DragTargetDrop(*current_drop_data_,
+  rwh->DragTargetDrop(*current_drop_data_,
                       location,
                       screen_location,
                       key_modifiers);
@@ -1021,7 +1050,7 @@ void WebContentsView::FocusChanged() {
   MaybeScrollFocusedEditableNodeIntoView();
 }
 
-void WebContentsView::ScreenUpdated() {
+void WebContentsView::ScreenChanged() {
   // If the device scale has changed, then the compositor viewport size
   // and scale needs updating
   // See https://launchpad.net/bugs/1575216
