@@ -20,13 +20,13 @@
 #include <queue>
 #include <utility>
 
-#include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/supports_user_data.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "components/sessions/content/content_serialized_navigation_builder.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/interstitial_page.h"
@@ -39,7 +39,9 @@
 #include "content/public/browser/notification_source.h"
 #include "content/public/browser/notification_types.h"
 #include "content/public/browser/page_navigator.h"
+#include "content/public/browser/reload_type.h"
 #include "content/public/browser/render_frame_host.h"
+#include "content/public/browser/render_process_host.h"
 #include "content/public/browser/render_view_host.h"
 #include "content/public/browser/render_widget_host.h"
 #include "content/public/browser/resource_request_details.h"
@@ -48,8 +50,8 @@
 #include "content/public/common/file_chooser_params.h"
 #include "content/public/common/menu_item.h"
 #include "content/public/common/renderer_preferences.h"
+#include "content/public/common/result_codes.h"
 #include "content/public/common/url_constants.h"
-#include "content/public/common/web_preferences.h"
 #include "ipc/ipc_message_macros.h"
 #include "net/base/net_errors.h"
 #include "net/ssl/ssl_info.h"
@@ -63,6 +65,7 @@
 
 #include "shared/browser/clipboard/oxide_clipboard.h"
 #include "shared/browser/input/oxide_ime_bridge.h"
+#include "shared/browser/javascript_dialogs/javascript_dialog_contents_helper.h"
 #include "shared/browser/media/oxide_media_capture_devices_dispatcher.h"
 #include "shared/browser/permissions/oxide_permission_request_dispatcher.h"
 #include "shared/browser/permissions/oxide_temporary_saved_permission_context.h"
@@ -73,6 +76,7 @@
 #include "shared/common/oxide_messages.h"
 #include "shared/common/oxide_unowned_user_data.h"
 
+#include "navigation_controller_observer.h"
 #include "oxide_browser_context.h"
 #include "oxide_browser_process_main.h"
 #include "oxide_content_browser_client.h"
@@ -81,17 +85,17 @@
 #include "oxide_file_picker.h"
 #include "oxide_find_controller.h"
 #include "oxide_fullscreen_helper.h"
-#include "oxide_javascript_dialog_manager.h"
 #include "oxide_render_widget_host_view.h"
 #include "oxide_script_message_contents_helper.h"
-#include "oxide_web_contents_unloader.h"
+#include "oxide_user_agent_settings.h"
 #include "oxide_web_contents_view.h"
 #include "oxide_web_contents_view_client.h"
 #include "oxide_web_frame_tree.h"
 #include "oxide_web_frame.h"
-#include "oxide_web_preferences.h"
 #include "oxide_web_view_client.h"
-#include "oxide_web_view_contents_helper.h"
+#include "web_contents_client.h"
+#include "web_contents_helper.h"
+#include "web_process_status_monitor.h"
 
 #if defined(ENABLE_MEDIAHUB)
 #include "shared/browser/media/oxide_media_web_contents_observer.h"
@@ -123,9 +127,7 @@ void FillLoadURLParamsFromOpenURLParams(
   }
 }
 
-void CreateHelpers(content::WebContents* contents,
-                   content::WebContents* opener = nullptr) {
-  new WebViewContentsHelper(contents, opener);
+void CreateHelpers(content::WebContents* contents) {
   CertificateErrorDispatcher::CreateForWebContents(contents);
   SecurityStatus::CreateForWebContents(contents);
   PermissionRequestDispatcher::CreateForWebContents(contents);
@@ -137,80 +139,40 @@ void CreateHelpers(content::WebContents* contents,
   WebFrameTree::CreateForWebContents(contents);
   FaviconHelper::CreateForWebContents(contents);
   FullscreenHelper::CreateForWebContents(contents);
+  WebProcessStatusMonitor::CreateForWebContents(contents);
+  JavaScriptDialogContentsHelper::CreateForWebContents(contents);
 }
 
 OXIDE_MAKE_ENUM_BITWISE_OPERATORS(ui::PageTransition)
 OXIDE_MAKE_ENUM_BITWISE_OPERATORS(ContentType)
 OXIDE_MAKE_ENUM_BITWISE_OPERATORS(blink::WebContextMenuData::EditFlags)
 
-base::LazyInstance<std::vector<WebView*> > g_all_web_views;
-
 }
 
-void WebView::WebContentsDeleter::operator()(content::WebContents* contents) {
-  WebContentsUnloader::GetInstance()->Unload(base::WrapUnique(contents));
-};
+WebView::CommonParams::CommonParams() = default;
 
-WebViewIterator::WebViewIterator(const std::vector<WebView*>& views) {
-  for (std::vector<WebView*>::const_iterator it = views.begin();
-       it != views.end(); ++it) {
-    views_.push_back((*it)->AsWeakPtr());
-  }
-  current_ = views_.begin();
-}
-
-WebViewIterator::~WebViewIterator() {}
-
-bool WebViewIterator::HasMore() const {
-  return current_ != views_.end();
-}
-
-WebView* WebViewIterator::GetNext() {
-  while (current_ != views_.end()) {
-    base::WeakPtr<WebView>& view = *current_;
-    current_++;
-    if (view.get()) {
-      return view.get();
-    }
-  }
-
-  return nullptr;
-}
-
-WebView::CommonParams::CommonParams()
-    : client(nullptr),
-      view_client(nullptr) {}
-
-WebView::CommonParams::~CommonParams() {}
+WebView::CommonParams::~CommonParams() = default;
 
 WebView::CreateParams::CreateParams()
     : context(nullptr),
       incognito(false),
-      restore_type(content::NavigationController::RESTORE_CURRENT_SESSION),
+      restore_type(content::RestoreType::NONE),
       restore_index(0) {}
 
 WebView::CreateParams::~CreateParams() {}
 
-// static
-WebViewIterator WebView::GetAllWebViews() {
-  return WebViewIterator(g_all_web_views.Get());
-}
-
 WebView::WebView(WebViewClient* client)
     : client_(client),
-      web_contents_helper_(nullptr),
       blocked_content_(CONTENT_TYPE_NONE),
-      location_bar_height_(0),
-      location_bar_constraints_(blink::WebTopControlsBoth),
-      location_bar_animated_(true),
       edit_flags_(blink::WebContextMenuData::CanDoNone),
       weak_factory_(this) {
   CHECK(client) << "Didn't specify a client";
 }
 
-void WebView::CommonInit(std::unique_ptr<content::WebContents> contents,
-                         WebContentsViewClient* view_client) {
-  web_contents_.reset(contents.release());
+void WebView::CommonInit(WebContentsUniquePtr contents,
+                         WebContentsViewClient* view_client,
+                         WebContentsClient* contents_client) {
+  web_contents_ = std::move(contents);
 
   // Attach ourself to the WebContents
   web_contents_->SetDelegate(this);
@@ -219,31 +181,16 @@ void WebView::CommonInit(std::unique_ptr<content::WebContents> contents,
 
   content::WebContentsObserver::Observe(web_contents_.get());
 
-  // Set the initial WebPreferences. This has to happen after attaching
-  // ourself to the WebContents, as the pref update needs to call back in
-  // to us (via CanCreateWindows)
-  web_contents_helper_ =
-      WebViewContentsHelper::FromWebContents(web_contents_.get());
-  web_contents_helper_->WebContentsAdopted();
-
-  registrar_.Add(this, content::NOTIFICATION_NAV_LIST_PRUNED,
-                 content::NotificationService::AllBrowserContextsAndSources());
-  registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_CHANGED,
-                 content::NotificationService::AllBrowserContextsAndSources());
-
   WebContentsView* view = WebContentsView::FromWebContents(web_contents_.get());
   view->SetClient(view_client);
   view->set_editing_capabilities_changed_callback(
       base::Bind(&WebView::EditingCapabilitiesChanged,
                  base::Unretained(this)));
 
-  CompositorObserver::Observe(view->GetCompositor());
+  DCHECK(GetWebContentsHelper());
+  GetWebContentsHelper()->set_client(contents_client);
 
-  DCHECK(std::find(g_all_web_views.Get().begin(),
-                   g_all_web_views.Get().end(),
-                   this) ==
-         g_all_web_views.Get().end());
-  g_all_web_views.Get().push_back(this);
+  CompositorObserver::Observe(view->GetCompositor());
 }
 
 RenderWidgetHostView* WebView::GetRenderWidgetHostView() const {
@@ -256,10 +203,6 @@ RenderWidgetHostView* WebView::GetRenderWidgetHostView() const {
   return static_cast<RenderWidgetHostView *>(rwhv);
 }
 
-content::RenderViewHost* WebView::GetRenderViewHost() const {
-  return web_contents_->GetRenderViewHost();
-}
-
 content::RenderWidgetHost* WebView::GetRenderWidgetHost() const {
   RenderWidgetHostView* rwhv = GetRenderWidgetHostView();
   if (!rwhv) {
@@ -323,23 +266,6 @@ void WebView::OnDidBlockRunningInsecureContent() {
   client_->ContentBlocked();
 }
 
-void WebView::InitializeTopControlsForHost(content::RenderViewHost* rvh,
-                                           bool initial_host) {
-  // Show the location bar if this is the initial RVH and the constraints
-  // are set to blink::WebTopControlsBoth
-  blink::WebTopControlsState current = location_bar_constraints_;
-  if (initial_host &&
-      location_bar_constraints_ == blink::WebTopControlsBoth) {
-    current = blink::WebTopControlsShown;
-  }
-
-  rvh->Send(
-      new OxideMsg_UpdateTopControlsState(rvh->GetRoutingID(),
-                                          location_bar_constraints_,
-                                          current,
-                                          false));
-}
-
 void WebView::DispatchPrepareToCloseResponse(bool proceed) {
   client_->PrepareToCloseResponseReceived(proceed);
 }
@@ -365,7 +291,7 @@ void WebView::EditingCapabilitiesChanged() {
   ui::TextInputType text_input_type = rwhv->ime_bridge()->text_input_type();
   bool editable = (text_input_type != ui::TEXT_INPUT_TYPE_NONE);
   bool readable = (text_input_type != ui::TEXT_INPUT_TYPE_PASSWORD);
-  bool has_selection = !rwhv->selection_range().is_empty();
+  bool has_selection = !rwhv->GetSelectionRange().is_empty();
 
   // XXX: if editable, can we determine whether undo/redo is available?
   if (editable && readable && has_selection) {
@@ -404,87 +330,46 @@ void WebView::CompositorWillRequestSwapFrame() {
       WebContentsView::FromWebContents(web_contents_.get())
         ->committed_frame_metadata().Clone();
 
-  if (IsFullscreen()) {
-    // Ensure that the location bar is always hidden in fullscreen. This
-    // is required because fullscreen RenderWidgets don't have a mechanism
-    // to control the location bar height
-    compositor_frame_metadata_.location_bar_content_translation =
-        gfx::Vector2dF();
-    compositor_frame_metadata_.location_bar_offset =
-        gfx::Vector2dF(0.0f, -GetLocationBarHeight());
-  }
-
   client_->FrameMetadataUpdated(old);
 }
 
-void WebView::WebPreferencesDestroyed() {
-  client_->WebPreferencesDestroyed();
-}
-
-void WebView::Observe(int type,
-                      const content::NotificationSource& source,
-                      const content::NotificationDetails& details) {
-  if (content::Source<content::NavigationController>(source).ptr() !=
-      &GetWebContents()->GetController()) {
-    return;
-  }
-  if (type == content::NOTIFICATION_NAV_LIST_PRUNED) {
-    content::PrunedDetails* pruned_details =
-        content::Details<content::PrunedDetails>(details).ptr();
-    client_->NavigationListPruned(pruned_details->from_front,
-                                  pruned_details->count);
-  } else if (type == content::NOTIFICATION_NAV_ENTRY_CHANGED) {
-    int index =
-        content::Details<content::EntryChangedDetails>(details).ptr()->index;
-    client_->NavigationEntryChanged(index);
-  }
-}
-
 content::WebContents* WebView::OpenURLFromTab(
     content::WebContents* source,
     const content::OpenURLParams& params) {
   DCHECK_VALID_SOURCE_CONTENTS
 
-  // We get here from the following places:
-  // FIXME(chrisccoulson): 1 isn't really true anymore, and re-evaluate 2
-  //  1) RenderFrameHostManager::OnCrossSiteResponse. In this case, disposition
-  //     is always CURRENT_TAB and we always want to perform the navigation.
-  //     Asking the embedder whether to proceed is done via
-  //     NavigationInterceptResourceThrottle.
-  //  2) Non CURRENT_TAB navigations as a result of keyboard modifiers. In
-  //     this case, we want to ask the embedder whether to perform the
-  //     navigation unless we change it to CURRENT_TAB (in which case, we ask
-  //     the embedder via NavigationInterceptResourceThrottle)
-  //  3) CURRENT_TAB navigations in new webviews. Asking the embedder whether
-  //     to proceed is done via NavigationInterceptResourceThrottle
-
-  if (params.disposition != CURRENT_TAB &&
-      params.disposition != NEW_FOREGROUND_TAB &&
-      params.disposition != NEW_BACKGROUND_TAB &&
-      params.disposition != NEW_POPUP &&
-      params.disposition != NEW_WINDOW) {
+  if (params.disposition != WindowOpenDisposition::CURRENT_TAB &&
+      params.disposition != WindowOpenDisposition::NEW_FOREGROUND_TAB &&
+      params.disposition != WindowOpenDisposition::NEW_BACKGROUND_TAB &&
+      params.disposition != WindowOpenDisposition::NEW_POPUP &&
+      params.disposition != WindowOpenDisposition::NEW_WINDOW) {
     return nullptr;
   }
 
   // Block popups
-  if ((params.disposition == NEW_FOREGROUND_TAB ||
-       params.disposition == NEW_BACKGROUND_TAB ||
-       params.disposition == NEW_WINDOW ||
-       params.disposition == NEW_POPUP) &&
-      !params.user_gesture && GetBrowserContext()->IsPopupBlockerEnabled()) {
+  UserAgentSettings* ua_settings =
+      UserAgentSettings::Get(web_contents_->GetBrowserContext());
+  if ((params.disposition == WindowOpenDisposition::NEW_FOREGROUND_TAB ||
+       params.disposition == WindowOpenDisposition::NEW_BACKGROUND_TAB ||
+       params.disposition == WindowOpenDisposition::NEW_WINDOW ||
+       params.disposition == WindowOpenDisposition::NEW_POPUP) &&
+      !params.user_gesture && ua_settings->IsPopupBlockerEnabled()) {
     return nullptr;
   }
 
   WindowOpenDisposition disposition = params.disposition;
 
+  WebContentsClient* contents_client = GetWebContentsHelper()->client();
+
   // If we can't create new windows, this should be a CURRENT_TAB navigation
   // in the top-level frame
-  if (!CanCreateWindows() && disposition != CURRENT_TAB) {
-    disposition = CURRENT_TAB;
+  if ((!contents_client || !contents_client->CanCreateWindows()) &&
+      disposition != WindowOpenDisposition::CURRENT_TAB) {
+    disposition = WindowOpenDisposition::CURRENT_TAB;
   }
 
   // Handle the CURRENT_TAB case now
-  if (disposition == CURRENT_TAB) {
+  if (disposition == WindowOpenDisposition::CURRENT_TAB) {
     // XXX: We have no way to propagate OpenURLParams::user_gesture here, so
     // ResourceRequestInfo::HasUserGesture will always return false in
     // NavigationInterceptResourceThrottle
@@ -497,22 +382,26 @@ content::WebContents* WebView::OpenURLFromTab(
     return web_contents_.get();
   }
 
-  // At this point, we expect all navigations to be for the main frame and
-  // to be renderer initiated
+  // At this point, we expect all navigations to be for the main frame
   DCHECK_EQ(params.frame_tree_node_id, -1);
-  DCHECK(params.is_renderer_initiated);
 
   // Coerce all non CURRENT_TAB navigations that don't come from a user
   // gesture to NEW_POPUP
   if (!params.user_gesture) {
-    disposition = NEW_POPUP;
+    disposition = WindowOpenDisposition::NEW_POPUP;
+  }
+
+  // If there's no WebContentsClient, then we can't open a new view
+  if (!contents_client) {
+    return nullptr;
   }
 
   // Give the application a chance to block the navigation if it is
   // renderer initiated
-  if (!client_->ShouldHandleNavigation(params.url,
-                                       disposition,
-                                       params.user_gesture)) {
+  if (params.is_renderer_initiated &&
+      !contents_client->ShouldCreateNewWebContents(params.url,
+                                                   disposition,
+                                                   params.user_gesture)) {
     return nullptr;
   }
 
@@ -524,7 +413,8 @@ content::WebContents* WebView::OpenURLFromTab(
       GetBrowserContext(),
       opener_suppressed ? nullptr : web_contents_->GetSiteInstance());
   contents_params.initial_size = GetViewSizeDip();
-  contents_params.initially_hidden = disposition == NEW_BACKGROUND_TAB;
+  contents_params.initially_hidden =
+      disposition == WindowOpenDisposition::NEW_BACKGROUND_TAB;
   contents_params.opener_render_process_id =
       web_contents_->GetRenderProcessHost()->GetID();
   // XXX(chrisccoulson): This is probably wrong, but we're going to revisit
@@ -534,35 +424,36 @@ content::WebContents* WebView::OpenURLFromTab(
       web_contents_->GetMainFrame()->GetRoutingID();
   contents_params.opener_suppressed = opener_suppressed;
 
-  std::unique_ptr<content::WebContents> contents(
-      content::WebContents::Create(contents_params));
-  if (!contents) {
-    LOG(ERROR) << "Failed to create new WebContents for navigation";
-    return nullptr;
-  }
+  WebContentsUniquePtr contents =
+      WebContentsHelper::CreateWebContents(contents_params);
+  CreateHelpers(contents.get());
 
-  CreateHelpers(contents.get(), web_contents_.get());
+  WebContentsHelper* helper = WebContentsHelper::FromWebContents(contents.get());
 
-  WebView* new_view =
-      client_->CreateNewWebView(GetViewBoundsDip(),
-                                disposition,
-                                std::move(contents));
-  if (!new_view) {
+  bool created =
+      GetWebContentsHelper()->client()->AdoptNewWebContents(GetViewBoundsDip(),
+                                                            disposition,
+                                                            std::move(contents));
+  if (!created) {
     return nullptr;
   }
 
   content::NavigationController::LoadURLParams load_params(params.url);
   FillLoadURLParamsFromOpenURLParams(&load_params, params);
 
-  new_view->GetWebContents()->GetController().LoadURLWithParams(load_params);
+  helper->GetWebContents()->GetController().LoadURLWithParams(load_params);
 
-  return new_view->GetWebContents();
+  return helper->GetWebContents();
 }
 
 void WebView::NavigationStateChanged(content::WebContents* source,
                                      content::InvalidateTypes changed_flags) {
   DCHECK_VALID_SOURCE_CONTENTS
 
+  NavigationControllerObserver::NotifyNavigationStateChanged(
+      web_contents_->GetController(),
+      changed_flags);
+
   if (changed_flags & content::INVALIDATE_TYPE_URL) {
     client_->URLChanged();
   }
@@ -570,25 +461,23 @@ void WebView::NavigationStateChanged(content::WebContents* source,
   if (changed_flags & content::INVALIDATE_TYPE_TITLE) {
     client_->TitleChanged();
   }
-
-  if (changed_flags & (content::INVALIDATE_TYPE_URL |
-                       content::INVALIDATE_TYPE_LOAD)) {
-    client_->CommandsUpdated();
-  }
 }
 
-void WebView::VisibleSSLStateChanged(const content::WebContents* source) {
+void WebView::VisibleSecurityStateChanged(content::WebContents* source) {
   DCHECK_VALID_SOURCE_CONTENTS
 
-  SecurityStatus::FromWebContents(web_contents_.get())->VisibleSSLStateChanged();
+  SecurityStatus::FromWebContents(web_contents_.get())
+      ->VisibleSecurityStateChanged();
 }
 
 bool WebView::ShouldCreateWebContents(
     content::WebContents* source,
-    int route_id,
-    int main_frame_route_id,
-    int main_frame_widget_route_id,
+    content::SiteInstance* source_site_instance,
+    int32_t route_id,
+    int32_t main_frame_route_id,
+    int32_t main_frame_widget_route_id,
     WindowContainerType window_container_type,
+    const GURL& opener_url,
     const std::string& frame_name,
     const GURL& target_url,
     const std::string& partition_id,
@@ -597,22 +486,26 @@ bool WebView::ShouldCreateWebContents(
     bool user_gesture) {
   DCHECK_VALID_SOURCE_CONTENTS
 
-  if (disposition != NEW_FOREGROUND_TAB &&
-      disposition != NEW_BACKGROUND_TAB &&
-      disposition != NEW_POPUP &&
-      disposition != NEW_WINDOW) {
+  if (disposition != WindowOpenDisposition::NEW_FOREGROUND_TAB &&
+      disposition != WindowOpenDisposition::NEW_BACKGROUND_TAB &&
+      disposition != WindowOpenDisposition::NEW_POPUP &&
+      disposition != WindowOpenDisposition::NEW_WINDOW) {
     return false;
   }
 
   // Note that popup blocking was done on the IO thread
 
-  if (!CanCreateWindows()) {
+  if (!GetWebContentsHelper()->client()) {
     return false;
   }
 
-  return client_->ShouldHandleNavigation(
+  if (!GetWebContentsHelper()->client()->CanCreateWindows()) {
+    return false;
+  }
+
+  return GetWebContentsHelper()->client()->ShouldCreateNewWebContents(
       target_url,
-      user_gesture ? disposition : NEW_POPUP,
+      user_gesture ? disposition : WindowOpenDisposition::NEW_POPUP,
       user_gesture);
 }
 
@@ -627,14 +520,30 @@ void WebView::HandleKeyboardEvent(
 }
 
 void WebView::WebContentsCreated(content::WebContents* source,
-                                 int source_frame_id,
+                                 int opener_render_process_id,
+                                 int opener_render_frame_id,
                                  const std::string& frame_name,
                                  const GURL& target_url,
                                  content::WebContents* new_contents) {
   DCHECK_VALID_SOURCE_CONTENTS
   DCHECK(!WebView::FromWebContents(new_contents));
 
-  CreateHelpers(new_contents, web_contents_.get());
+  WebContentsHelper::CreateForWebContents(new_contents, web_contents_.get());
+  CreateHelpers(new_contents);
+}
+
+void WebView::RendererUnresponsive(
+    content::WebContents* source,
+    const content::WebContentsUnresponsiveState& unresponsive_state) {
+  DCHECK_VALID_SOURCE_CONTENTS
+  WebProcessStatusMonitor::FromWebContents(web_contents_.get())
+      ->RendererIsUnresponsive();
+}
+
+void WebView::RendererResponsive(content::WebContents* source) {
+  DCHECK_VALID_SOURCE_CONTENTS
+  WebProcessStatusMonitor::FromWebContents(web_contents_.get())
+      ->RendererIsResponsive();
 }
 
 void WebView::AddNewContents(content::WebContents* source,
@@ -644,10 +553,11 @@ void WebView::AddNewContents(content::WebContents* source,
                              bool user_gesture,
                              bool* was_blocked) {
   DCHECK_VALID_SOURCE_CONTENTS
-  DCHECK(disposition == NEW_FOREGROUND_TAB ||
-         disposition == NEW_BACKGROUND_TAB ||
-         disposition == NEW_POPUP ||
-         disposition == NEW_WINDOW) << "Invalid disposition";
+  DCHECK(disposition == WindowOpenDisposition::NEW_FOREGROUND_TAB ||
+         disposition == WindowOpenDisposition::NEW_BACKGROUND_TAB ||
+         disposition == WindowOpenDisposition::NEW_POPUP ||
+         disposition == WindowOpenDisposition::NEW_WINDOW) <<
+             "Invalid disposition";
   DCHECK_EQ(GetBrowserContext(),
             BrowserContext::FromContent(new_contents->GetBrowserContext()));
 
@@ -655,13 +565,18 @@ void WebView::AddNewContents(content::WebContents* source,
     *was_blocked = true;
   }
 
-  std::unique_ptr<content::WebContents> contents(new_contents);
+  WebContentsUniquePtr contents(new_contents);
 
-  WebView* new_view =
-      client_->CreateNewWebView(initial_pos,
-                                user_gesture ? disposition : NEW_POPUP,
-                                std::move(contents));
-  if (!new_view) {
+  if (!GetWebContentsHelper()->client()) {
+    return;
+  }
+
+  bool created =
+      GetWebContentsHelper()->client()->AdoptNewWebContents(
+          initial_pos,
+          user_gesture ? disposition : WindowOpenDisposition::NEW_POPUP,
+          std::move(contents));
+  if (!created) {
     return;
   }
 
@@ -693,11 +608,11 @@ void WebView::UpdateTargetURL(content::WebContents* source, const GURL& url) {
   }
 }
 
-bool WebView::AddMessageToConsole(content::WebContents* source,
-                                  int32_t level,
-                                  const base::string16& message,
-                                  int32_t line_no,
-                                  const base::string16& source_id) {
+bool WebView::DidAddMessageToConsole(content::WebContents* source,
+                                     int32_t level,
+                                     const base::string16& message,
+                                     int32_t line_no,
+                                     const base::string16& source_id) {
   DCHECK_VALID_SOURCE_CONTENTS
 
   return client_->AddMessageToConsole(level, message, line_no, source_id);
@@ -717,7 +632,7 @@ void WebView::BeforeUnloadFired(content::WebContents* source,
 content::JavaScriptDialogManager* WebView::GetJavaScriptDialogManager(
     content::WebContents* source) {
   DCHECK_VALID_SOURCE_CONTENTS
-  return JavaScriptDialogManager::GetInstance();
+  return JavaScriptDialogContentsHelper::FromWebContents(web_contents_.get());
 }
 
 void WebView::RunFileChooser(content::RenderFrameHost* render_frame_host,
@@ -801,32 +716,6 @@ bool WebView::CheckMediaAccessPermission(content::WebContents* source,
   return status == TEMPORARY_SAVED_PERMISSION_STATUS_ALLOWED;
 }
 
-void WebView::RenderFrameForInterstitialPageCreated(
-    content::RenderFrameHost* render_frame_host) {
-  if (render_frame_host->GetParent()) {
-    return;
-  }
-
-  InitializeTopControlsForHost(render_frame_host->GetRenderViewHost(), false);
-}
-
-void WebView::RenderViewReady() {
-  client_->CrashedStatusChanged();
-}
-
-void WebView::RenderProcessGone(base::TerminationStatus status) {
-  client_->CrashedStatusChanged();
-}
-
-void WebView::RenderViewHostChanged(content::RenderViewHost* old_host,
-                                    content::RenderViewHost* new_host) {
-  if (!new_host) {
-    return;
-  }
-
-  InitializeTopControlsForHost(new_host, !old_host);
-}
-
 void WebView::DidStartLoading() {
   client_->LoadingChanged();
 }
@@ -868,8 +757,7 @@ void WebView::DidFailLoad(content::RenderFrameHost* render_frame_host,
 void WebView::DidStartProvisionalLoadForFrame(
     content::RenderFrameHost* render_frame_host,
     const GURL& validated_url,
-    bool is_error_frame,
-    bool is_iframe_srcdoc) {
+    bool is_error_frame) {
   if (is_error_frame) {
     return;
   }
@@ -923,7 +811,6 @@ void WebView::DidNavigateMainFrame(
 }
 
 void WebView::DidGetRedirectForResourceRequest(
-      content::RenderFrameHost* render_frame_host,
       const content::ResourceRedirectDetails& details) {
   if (details.resource_type != content::RESOURCE_TYPE_MAIN_FRAME) {
     return;
@@ -934,11 +821,6 @@ void WebView::DidGetRedirectForResourceRequest(
                           details.http_response_code);
 }
 
-void WebView::NavigationEntryCommitted(
-    const content::LoadCommittedDetails& load_details) {
-  client_->NavigationEntryCommitted();
-}
-
 void WebView::DidShowFullscreenWidget() {
   if (IsFullscreen()) {
     return;
@@ -950,22 +832,11 @@ void WebView::DidShowFullscreenWidget() {
   // We do this asynchronously to avoid a UAF in
   // WebContentsImpl::ShowCreatedWidget
   // See https://launchpad.net/bugs/1510973
-  base::MessageLoop::current()->PostTask(
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&WebView::MaybeCancelFullscreenMode, AsWeakPtr()));
 }
 
-void WebView::TitleWasSet(content::NavigationEntry* entry, bool explicit_set) {
-  const content::NavigationController& controller = web_contents_->GetController();
-  int count = controller.GetEntryCount();
-  for (int i = 0; i < count; ++i) {
-    if (controller.GetEntryAtIndex(i) == entry) {
-      client_->NavigationEntryChanged(i);
-      return;
-    }
-  }
-}
-
 bool WebView::OnMessageReceived(const IPC::Message& msg,
                                 content::RenderFrameHost* render_frame_host) {
   bool handled = true;
@@ -993,26 +864,27 @@ WebView::WebView(const CommonParams& common_params,
     : WebView(common_params.client) {
   CHECK(create_params.context) << "Didn't specify a BrowserContext";
 
-  scoped_refptr<BrowserContext> context = create_params.incognito ?
+  BrowserContext* context = create_params.incognito ?
       create_params.context->GetOffTheRecordContext() :
       create_params.context->GetOriginalContext();
 
-  content::WebContents::CreateParams content_params(context.get());
+  content::WebContents::CreateParams content_params(context);
   content_params.initial_size =
       gfx::ToEnclosingRect(common_params.view_client->GetBounds()).size();
   content_params.initially_hidden = !common_params.view_client->IsVisible();
 
-  std::unique_ptr<content::WebContents> contents(
-      content::WebContents::Create(content_params));
-  CHECK(contents.get()) << "Failed to create WebContents";
+  WebContentsUniquePtr contents =
+      WebContentsHelper::CreateWebContents(content_params);
 
   CreateHelpers(contents.get());
-  CommonInit(std::move(contents), common_params.view_client);
+  CommonInit(std::move(contents),
+             common_params.view_client,
+             common_params.contents_client);
 
   if (create_params.restore_entries.size() > 0) {
     std::vector<std::unique_ptr<content::NavigationEntry>> entries =
         sessions::ContentSerializedNavigationBuilder::ToNavigationEntries(
-            create_params.restore_entries, context.get());
+            create_params.restore_entries, context);
     web_contents_->GetController().Restore(
         create_params.restore_index,
         create_params.restore_type,
@@ -1022,35 +894,26 @@ WebView::WebView(const CommonParams& common_params,
 }
 
 WebView::WebView(const CommonParams& common_params,
-                 std::unique_ptr<content::WebContents> contents)
+                 WebContentsUniquePtr contents)
     : WebView(common_params.client) {
   CHECK(contents);
   DCHECK(contents->GetBrowserContext()) <<
          "Specified WebContents doesn't have a BrowserContext";
-  CHECK(WebViewContentsHelper::FromWebContents(contents.get())) <<
-       "Specified WebContents should already have a WebViewContentsHelper";
   CHECK(!FromWebContents(contents.get())) <<
         "Specified WebContents already belongs to a WebView";
 
-  CommonInit(std::move(contents), common_params.view_client);
-
-  content::RenderViewHost* rvh = GetRenderViewHost();
-  if (rvh) {
-    InitializeTopControlsForHost(rvh, true);
-  }
+  CommonInit(std::move(contents),
+             common_params.view_client,
+             common_params.contents_client);
 }
 
 WebView::~WebView() {
-  g_all_web_views.Get().erase(
-      std::remove(g_all_web_views.Get().begin(),
-                  g_all_web_views.Get().end(),
-                  this),
-      g_all_web_views.Get().end());
-
   WebContentsView* view = WebContentsView::FromWebContents(web_contents_.get());
   view->SetClient(nullptr);
   view->set_editing_capabilities_changed_callback(base::Closure());
 
+  GetWebContentsHelper()->set_client(nullptr);
+
   // Stop WebContents from calling back in to us
   content::WebContentsObserver::Observe(nullptr);
 
@@ -1062,6 +925,11 @@ WebView::~WebView() {
   web_contents_->RemoveUserData(&kUserDataKey);
 }
 
+void WebView::SetJavaScriptDialogFactory(JavaScriptDialogFactory* factory) {
+  JavaScriptDialogContentsHelper::FromWebContents(web_contents_.get())
+      ->set_factory(factory);
+}
+
 // static
 WebView* WebView::FromWebContents(const content::WebContents* web_contents) {
   UnownedUserData<WebView>* data =
@@ -1189,7 +1057,7 @@ void WebView::Stop() {
 }
 
 void WebView::Reload() {
-  web_contents_->GetController().Reload(true);
+  web_contents_->GetController().Reload(content::ReloadType::NORMAL, true);
 }
 
 bool WebView::IsIncognito() const {
@@ -1200,15 +1068,6 @@ bool WebView::IsLoading() const {
   return web_contents_->IsLoading();
 }
 
-void WebView::UpdateWebPreferences() {
-  content::RenderViewHost* rvh = GetRenderViewHost();
-  if (!rvh) {
-    return;
-  }
-
-  rvh->OnWebkitPreferencesChanged();
-}
-
 BrowserContext* WebView::GetBrowserContext() const {
   return BrowserContext::FromContent(web_contents_->GetBrowserContext());
 }
@@ -1217,55 +1076,14 @@ content::WebContents* WebView::GetWebContents() const {
   return web_contents_.get();
 }
 
-int WebView::GetNavigationEntryCount() const {
-  return web_contents_->GetController().GetEntryCount();
-}
-
-int WebView::GetNavigationCurrentEntryIndex() const {
-  return web_contents_->GetController().GetCurrentEntryIndex();
-}
-
-void WebView::SetNavigationCurrentEntryIndex(int index) {
-  web_contents_->GetController().GoToIndex(index);
-}
-
-int WebView::GetNavigationEntryUniqueID(int index) const {
-  const content::NavigationController& controller = web_contents_->GetController();
-  content::NavigationEntry* entry = controller.GetEntryAtIndex(index);
-  return entry->GetUniqueID();
-}
-
-const GURL& WebView::GetNavigationEntryUrl(int index) const {
-  const content::NavigationController& controller = web_contents_->GetController();
-  content::NavigationEntry* entry = controller.GetEntryAtIndex(index);
-  return entry->GetURL();
-}
-
-std::string WebView::GetNavigationEntryTitle(int index) const {
-  const content::NavigationController& controller = web_contents_->GetController();
-  content::NavigationEntry* entry = controller.GetEntryAtIndex(index);
-  return base::UTF16ToUTF8(entry->GetTitle());
-}
-
-base::Time WebView::GetNavigationEntryTimestamp(int index) const {
-  const content::NavigationController& controller = web_contents_->GetController();
-  content::NavigationEntry* entry = controller.GetEntryAtIndex(index);
-  return entry->GetTimestamp();
+WebContentsHelper* WebView::GetWebContentsHelper() const {
+  return WebContentsHelper::FromWebContents(web_contents_.get());
 }
 
 WebFrame* WebView::GetRootFrame() const {
   return WebFrameTree::FromWebContents(web_contents_.get())->root_frame();
 }
 
-WebPreferences* WebView::GetWebPreferences() {
-  return web_contents_helper_->GetWebPreferences();
-}
-
-void WebView::SetWebPreferences(WebPreferences* prefs) {
-  WebPreferencesObserver::Observe(prefs);
-  web_contents_helper_->SetWebPreferences(prefs);
-}
-
 gfx::Point WebView::GetCompositorFrameScrollOffset() {
   // See https://launchpad.net/bugs/1336730
   const gfx::SizeF& viewport_size =
@@ -1306,81 +1124,6 @@ gfx::Size WebView::GetCompositorFrameViewportSize() {
   return gfx::Size(std::round(size.width()), std::round(size.height()));
 }
 
-float WebView::GetLocationBarOffset() const {
-  return compositor_frame_metadata().location_bar_offset.y();
-}
-
-float WebView::GetLocationBarContentOffset() const {
-  return compositor_frame_metadata().location_bar_content_translation.y();
-}
-
-float WebView::GetLocationBarHeight() const {
-  return location_bar_height_;
-}
-
-void WebView::SetLocationBarHeight(float height) {
-  DCHECK_GE(height, 0);
-
-  if (height == location_bar_height_) {
-    return;
-  }
-
-  location_bar_height_ = height;
-
-  content::RenderWidgetHost* host = GetRenderWidgetHost();
-  if (!host) {
-    return;
-  }
-
-  host->WasResized();
-}
-
-void WebView::SetLocationBarConstraints(blink::WebTopControlsState constraints) {
-  if (constraints == location_bar_constraints_) {
-    return;
-  }
-
-  location_bar_constraints_ = constraints;
-
-  content::RenderViewHost* rvh = GetRenderViewHost();
-  if (!rvh) {
-    return;
-  }
-
-  rvh->Send(new OxideMsg_UpdateTopControlsState(rvh->GetRoutingID(),
-                                                location_bar_constraints_,
-                                                blink::WebTopControlsBoth,
-                                                location_bar_animated_));
-}
-
-void WebView::ShowLocationBar(bool animate) {
-  DCHECK_EQ(location_bar_constraints_, blink::WebTopControlsBoth);
-
-  content::RenderViewHost* rvh = GetRenderViewHost();
-  if (!rvh) {
-    return;
-  }
-
-  rvh->Send(new OxideMsg_UpdateTopControlsState(rvh->GetRoutingID(),
-                                                location_bar_constraints_,
-                                                blink::WebTopControlsShown,
-                                                animate));
-}
-
-void WebView::HideLocationBar(bool animate) {
-  DCHECK_EQ(location_bar_constraints_, blink::WebTopControlsBoth);
-
-  content::RenderViewHost* rvh = GetRenderViewHost();
-  if (!rvh) {
-    return;
-  }
-
-  rvh->Send(new OxideMsg_UpdateTopControlsState(rvh->GetRoutingID(),
-                                                location_bar_constraints_,
-                                                blink::WebTopControlsHidden,
-                                                animate));
-}
-
 void WebView::SetCanTemporarilyDisplayInsecureContent(bool allow) {
   if (!(blocked_content_ & CONTENT_TYPE_MIXED_DISPLAY) && allow) {
     return;
@@ -1424,7 +1167,7 @@ void WebView::PrepareToClose() {
                  AsWeakPtr(), true);
 
   if (!web_contents_->NeedToFireBeforeUnload()) {
-    base::MessageLoop::current()->PostTask(
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
         base::Bind(&WebView::DispatchPrepareToCloseResponse,
                    AsWeakPtr(), true));
@@ -1436,58 +1179,23 @@ void WebView::PrepareToClose() {
   web_contents_->DispatchBeforeUnload();
 }
 
-void WebView::DownloadRequested(
-    const GURL& url,
-    const std::string& mime_type,
-    const bool should_prompt,
-    const base::string16& suggested_filename,
-    const std::string& cookies,
-    const std::string& referrer,
-    const std::string& user_agent) {
-  client_->DownloadRequested(url,
-                             mime_type,
-                             should_prompt,
-                             suggested_filename,
-                             cookies,
-                             referrer,
-                             user_agent);
-}
-
-void WebView::HttpAuthenticationRequested(
-    ResourceDispatcherHostLoginDelegate* login_delegate) {
-  client_->HttpAuthenticationRequested(login_delegate);
-}
-
-blink::WebScreenInfo WebView::GetScreenInfo() const {
-  return WebContentsView::FromWebContents(web_contents_.get())->GetScreenInfo();
-}
-
-JavaScriptDialog* WebView::CreateJavaScriptDialog(
-    content::JavaScriptMessageType javascript_message_type) {
-  return client_->CreateJavaScriptDialog(javascript_message_type);
-}
-
-JavaScriptDialog* WebView::CreateBeforeUnloadDialog() {
-  return client_->CreateBeforeUnloadDialog();
-}
-
-bool WebView::ShouldHandleNavigation(const GURL& url, bool has_user_gesture) {
-  if (web_contents_->GetController().IsInitialNavigation()) {
-    return true;
-  }
-
-  return client_->ShouldHandleNavigation(url,
-                                         CURRENT_TAB,
-                                         has_user_gesture);
-}
-
-bool WebView::CanCreateWindows() const {
-  return client_->CanCreateWindows();
-}
-
 blink::WebContextMenuData::EditFlags WebView::GetEditFlags() const {
   return edit_flags_;
 }
 
-} // namespace oxide
+void WebView::TerminateWebProcess() {
+  content::RenderProcessHost* host = web_contents_->GetRenderProcessHost();
+  if (!host) {
+    return;
+  }
 
+  bool is_hung =
+      WebProcessStatusMonitor::FromWebContents(web_contents_.get())
+          ->GetStatus() == WebProcessStatusMonitor::Status::Unresponsive;
+
+  host->Shutdown(is_hung ?
+                     content::RESULT_CODE_HUNG : content::RESULT_CODE_KILLED,
+                 false);
+}
+
+} // namespace oxide
