@@ -19,13 +19,11 @@
 
 #include "base/bind.h"
 #include "base/memory/ref_counted.h"
-#include "content/public/browser/cert_store.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/ssl_status.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/referrer.h"
-#include "content/public/common/security_style.h"
-#include "content/public/common/ssl_status.h"
 #include "content/public/test/test_browser_context.h"
 #include "content/public/test/test_web_contents_factory.h"
 #include "net/cert/x509_certificate.h"
@@ -34,7 +32,6 @@
 #include "url/gurl.h"
 
 #include "shared/common/oxide_enum_flags.h"
-#include "shared/test/oxide_mock_cert_store.h"
 #include "shared/test/oxide_test_browser_thread_bundle.h"
 
 #include "oxide_security_status.h"
@@ -55,44 +52,37 @@ class SecurityStatusTest : public testing::Test {
  protected:
   content::WebContents* web_contents() const { return web_contents_; }
 
-  content::CertStore* cert_store() { return &cert_store_; }
-
-  int cert_id() const { return cert_id_; }
-  int expired_cert_id() const { return expired_cert_id_; }
+  net::X509Certificate* cert() const { return cert_.get(); }
+  net::X509Certificate* expired_cert() const { return expired_cert_.get(); }
 
  private:
   void SetUp() override;
 
-  MockCertStore cert_store_;
   TestBrowserThreadBundle browser_thread_bundle_;
   content::TestBrowserContext browser_context_;
   content::TestWebContentsFactory web_contents_factory_;
 
   content::WebContents* web_contents_;
 
-  int cert_id_;
-  int expired_cert_id_;
+  scoped_refptr<net::X509Certificate> cert_;
+  scoped_refptr<net::X509Certificate> expired_cert_;
 };
 
 void SecurityStatusTest::SetUp() {
   web_contents_ = web_contents_factory_.CreateWebContents(&browser_context_);
   SecurityStatus::CreateForWebContents(web_contents_);
-  SecurityStatus::FromWebContents(web_contents_)->SetCertStoreForTesting(
-      &cert_store_);
 
-  scoped_refptr<net::X509Certificate> cert(
+  cert_ =
       new net::X509Certificate("https://www.google.com/",
                                "https://www.example.com/",
                                base::Time::UnixEpoch(),
-                               base::Time::Now() + base::TimeDelta::FromDays(1)));
-  cert_id_ = cert_store_.AddCertForTesting(cert.get());
+                               base::Time::Now() + base::TimeDelta::FromDays(1));
 
-  cert =
+  expired_cert_ =
       new net::X509Certificate("https://www.google.com/",
                                "https://www.example.com/",
                                base::Time::UnixEpoch(),
                                base::Time::Now() - base::TimeDelta::FromDays(1));
-  expired_cert_id_ = cert_store_.AddCertForTesting(cert.get());
 }
 
 SecurityStatusTest::SecurityStatusTest() {}
@@ -111,11 +101,8 @@ TEST_F(SecurityStatusTest, NoSecurity) {
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
-  content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_UNAUTHENTICATED;
-
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_NONE, status->security_level());
   EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
@@ -125,167 +112,147 @@ TEST_F(SecurityStatusTest, NoSecurity) {
 
 TEST_F(SecurityStatusTest, Secure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
-  content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
+  status->VisibleSecurityStateChanged();
 
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  EXPECT_EQ(SECURITY_LEVEL_NONE, status->security_level());
+  EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
+  EXPECT_EQ(CERT_STATUS_OK, status->cert_status());
+  EXPECT_EQ(nullptr, status->cert());
+
+  content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
+  ssl_status.certificate = cert();
+
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_SECURE, status->security_level());
   EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
   EXPECT_EQ(CERT_STATUS_OK, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 TEST_F(SecurityStatusTest, Broken) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.cert_status = net::CERT_STATUS_COMMON_NAME_INVALID;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_ERROR, status->security_level());
   EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
   EXPECT_EQ(CERT_STATUS_BAD_IDENTITY, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 TEST_F(SecurityStatusTest, RanInsecure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.content_status = content::SSLStatus::RAN_INSECURE_CONTENT;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_ERROR, status->security_level());
   EXPECT_EQ(content::SSLStatus::RAN_INSECURE_CONTENT, status->content_status());
   EXPECT_EQ(CERT_STATUS_OK, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 TEST_F(SecurityStatusTest, DisplayedInsecure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.content_status = content::SSLStatus::DISPLAYED_INSECURE_CONTENT;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_WARNING, status->security_level());
   EXPECT_EQ(content::SSLStatus::DISPLAYED_INSECURE_CONTENT,
             status->content_status());
   EXPECT_EQ(CERT_STATUS_OK, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 TEST_F(SecurityStatusTest, DisplayedAndRanInsecure) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.content_status =
       content::SSLStatus::DISPLAYED_INSECURE_CONTENT |
       content::SSLStatus::RAN_INSECURE_CONTENT;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_ERROR, status->security_level());
   EXPECT_EQ(content::SSLStatus::DISPLAYED_INSECURE_CONTENT |
                 content::SSLStatus::RAN_INSECURE_CONTENT,
             status->content_status());
   EXPECT_EQ(CERT_STATUS_OK, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 TEST_F(SecurityStatusTest, MinorCertError) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.cert_status = net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_WARNING, status->security_level());
   EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
   EXPECT_EQ(CERT_STATUS_REVOCATION_CHECK_FAILED, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 TEST_F(SecurityStatusTest, SecureEV) {
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  ssl_status.cert_id = cert_id();
+  ssl_status.certificate = cert();
   ssl_status.cert_status = net::CERT_STATUS_IS_EV;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
-
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(cert_id(), &cert));
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(SECURITY_LEVEL_SECURE_EV, status->security_level());
   EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
   EXPECT_EQ(CERT_STATUS_OK, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(cert(), status->cert());
 }
 
 struct CertStatusRow {
@@ -489,25 +456,18 @@ TEST_P(SecurityStatusCertStatusTest, TestCertStatus) {
   const CertStatusRow& row = GetParam();
 
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("http://www.google.com/"),
+  controller.LoadURL(GURL("https://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  if (net::IsCertStatusError(row.cert_status_in) &&
-      !net::IsCertStatusMinorError(row.cert_status_in)) {
-    ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATION_BROKEN;
-  } else {
-    ssl_status.security_style = content::SECURITY_STYLE_AUTHENTICATED;
-  }
-  ssl_status.cert_id = row.expired_cert ? expired_cert_id() : cert_id();
+  ssl_status.certificate = row.expired_cert ? expired_cert() : cert();
   ssl_status.cert_status = row.cert_status_in;
 
-  scoped_refptr<net::X509Certificate> cert;
-  ASSERT_TRUE(cert_store()->RetrieveCert(ssl_status.cert_id, &cert));
+  scoped_refptr<net::X509Certificate> expected_cert = ssl_status.certificate;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
+  status->VisibleSecurityStateChanged();
 
   SecurityLevel expected_security_level;
   if (net::IsCertStatusError(row.cert_status_in)) {
@@ -523,21 +483,21 @@ TEST_P(SecurityStatusCertStatusTest, TestCertStatus) {
   EXPECT_EQ(expected_security_level, status->security_level());
   EXPECT_EQ(content::SSLStatus::NORMAL_CONTENT, status->content_status());
   EXPECT_EQ(row.cert_status_out, status->cert_status());
-  EXPECT_EQ(cert.get(), status->cert());
+  EXPECT_EQ(expected_cert.get(), status->cert());
 }
 
 struct ObserverCallbackRow {
   struct InputData {
-    InputData(content::SecurityStyle security_style,
+    InputData(bool https,
               bool expired_cert,
               net::CertStatus cert_status,
               content::SSLStatus::ContentStatusFlags content_status)
-        : security_style(security_style),
+        : https(https),
           expired_cert(expired_cert),
           cert_status(cert_status),
           content_status(content_status) {}
 
-    content::SecurityStyle security_style;
+    bool https;
     bool expired_cert;
     net::CertStatus cert_status;
     content::SSLStatus::ContentStatusFlags content_status;
@@ -579,11 +539,9 @@ INSTANTIATE_TEST_CASE_P(
     testing::Values(
         ObserverCallbackRow(
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
+                true, false, 0, content::SSLStatus::NORMAL_CONTENT),
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
+                true, false, 0, content::SSLStatus::NORMAL_CONTENT),
             SECURITY_LEVEL_SECURE,
             content::SSLStatus::NORMAL_CONTENT,
             CERT_STATUS_OK,
@@ -595,12 +553,10 @@ INSTANTIATE_TEST_CASE_P(
     testing::Values(
         ObserverCallbackRow(
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
+                false, false, 0, content::SSLStatus::NORMAL_CONTENT),
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATION_BROKEN,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
-            SECURITY_LEVEL_ERROR,
+                true, false, 0, content::SSLStatus::NORMAL_CONTENT),
+            SECURITY_LEVEL_SECURE,
             content::SSLStatus::NORMAL_CONTENT,
             CERT_STATUS_OK,
             false,
@@ -611,11 +567,9 @@ INSTANTIATE_TEST_CASE_P(
     testing::Values(
         ObserverCallbackRow(
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
+                true, false, 0, content::SSLStatus::NORMAL_CONTENT),
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::DISPLAYED_INSECURE_CONTENT),
+                true, false, 0, content::SSLStatus::DISPLAYED_INSECURE_CONTENT),
             SECURITY_LEVEL_WARNING,
             content::SSLStatus::DISPLAYED_INSECURE_CONTENT,
             CERT_STATUS_OK,
@@ -628,11 +582,9 @@ INSTANTIATE_TEST_CASE_P(
     testing::Values(
         ObserverCallbackRow(
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
+                true, false, 0, content::SSLStatus::NORMAL_CONTENT),
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false,
+                true, false,
                 net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION,
                 content::SSLStatus::NORMAL_CONTENT),
             SECURITY_LEVEL_WARNING,
@@ -647,11 +599,9 @@ INSTANTIATE_TEST_CASE_P(
     testing::Values(
         ObserverCallbackRow(
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                false, 0, content::SSLStatus::NORMAL_CONTENT),
+                true, false, 0, content::SSLStatus::NORMAL_CONTENT),
             ObserverCallbackRow::InputData(
-                content::SECURITY_STYLE_AUTHENTICATED,
-                true, 0, content::SSLStatus::NORMAL_CONTENT),
+                true, true, 0, content::SSLStatus::NORMAL_CONTENT),
             SECURITY_LEVEL_SECURE,
             content::SSLStatus::NORMAL_CONTENT,
             CERT_STATUS_OK,
@@ -678,19 +628,20 @@ TEST_P(SecurityStatusObserverCallbackTest, TestObserver) {
   const ObserverCallbackRow& params = GetParam();
 
   content::NavigationController& controller = web_contents()->GetController();
-  controller.LoadURL(GURL("https://www.google.com/"),
+  controller.LoadURL(params.initial_data.https ?
+                         GURL("https://www.google.com/")
+                         : GURL("http://www.google.com/"),
                      content::Referrer(),
                      ui::PAGE_TRANSITION_TYPED,
                      std::string());
   content::SSLStatus& ssl_status = controller.GetVisibleEntry()->GetSSL();
-  ssl_status.security_style = params.initial_data.security_style;
-  ssl_status.cert_id =
-      params.initial_data.expired_cert ? expired_cert_id() : cert_id();
+  ssl_status.certificate =
+      params.initial_data.expired_cert ? expired_cert() : cert();
   ssl_status.cert_status = params.initial_data.cert_status;
   ssl_status.content_status = params.initial_data.content_status;
 
   SecurityStatus* status = SecurityStatus::FromWebContents(web_contents());
-  status->VisibleSSLStateChanged();
+  status->VisibleSecurityStateChanged();
 
   SecurityStatus::ChangedFlags flags = SecurityStatus::CHANGED_FLAG_NONE;
   SecurityLevel security_level = status->security_level();
@@ -708,19 +659,17 @@ TEST_P(SecurityStatusObserverCallbackTest, TestObserver) {
                                            &cert,
                                            &flags));
 
-  ssl_status.security_style = params.test_data.security_style;
-  ssl_status.cert_id =
-      params.test_data.expired_cert ? expired_cert_id() : cert_id();
+  controller.GetVisibleEntry()->SetURL(
+      params.test_data.https ?
+          GURL("https://www.google.com") : GURL("http://www.google.com/"));
+  ssl_status.certificate =
+      params.test_data.expired_cert ? expired_cert() : this->cert();
   ssl_status.cert_status = params.test_data.cert_status;
   ssl_status.content_status = params.test_data.content_status;
 
-  scoped_refptr<net::X509Certificate> expected_cert;
-  ASSERT_TRUE(
-      cert_store()->RetrieveCert(params.cert_expected_expired ?
-                                     expired_cert_id() : cert_id(),
-                                 &expected_cert));
+  scoped_refptr<net::X509Certificate> expected_cert = ssl_status.certificate;
 
-  status->VisibleSSLStateChanged();
+  status->VisibleSecurityStateChanged();
 
   EXPECT_EQ(params.expected_flags, flags);
   EXPECT_EQ(params.expected_security_level, security_level);
