@@ -4,7 +4,6 @@
 
 #include "ppapi/proxy/ppapi_proxy_test.h"
 
-#include <sstream>
 #include <tuple>
 
 #include "base/bind.h"
@@ -68,10 +67,8 @@ PPB_Proxy_Private ppb_proxy_private = {
 base::ObserverList<ProxyTestHarnessBase> get_interface_handlers_;
 
 const void* MockGetInterface(const char* name) {
-  base::ObserverList<ProxyTestHarnessBase>::Iterator it(
-      &get_interface_handlers_);
-  while (ProxyTestHarnessBase* observer = it.GetNext()) {
-    const void* interface = observer->GetInterface(name);
+  for (auto& observer : get_interface_handlers_) {
+    const void* interface = observer.GetInterface(name);
     if (interface)
       return interface;
   }
@@ -550,21 +547,18 @@ void TwoWayTest::SetUp() {
   io_thread_.StartWithOptions(options);
   plugin_thread_.Start();
 
-  // Construct the IPC handle name using the process ID so we can safely run
-  // multiple |TwoWayTest|s concurrently.
-  std::ostringstream handle_name;
-  handle_name << "TwoWayTestChannel" << base::GetCurrentProcId();
-  IPC::ChannelHandle handle(handle_name.str());
+  mojo::MessagePipe pipe;
   base::WaitableEvent remote_harness_set_up(
       base::WaitableEvent::ResetPolicy::MANUAL,
       base::WaitableEvent::InitialState::NOT_SIGNALED);
   plugin_thread_.task_runner()->PostTask(
-      FROM_HERE, base::Bind(&SetUpRemoteHarness, remote_harness_, handle,
-                            base::RetainedRef(io_thread_.task_runner()),
-                            &shutdown_event_, &remote_harness_set_up));
+      FROM_HERE,
+      base::Bind(&SetUpRemoteHarness, remote_harness_, pipe.handle0.release(),
+                 base::RetainedRef(io_thread_.task_runner()), &shutdown_event_,
+                 &remote_harness_set_up));
   remote_harness_set_up.Wait();
   local_harness_->SetUpHarnessWithChannel(
-      handle, io_thread_.task_runner().get(), &shutdown_event_,
+      pipe.handle1.release(), io_thread_.task_runner().get(), &shutdown_event_,
       true);  // is_client
 }
 
