@@ -7,20 +7,21 @@ package org.chromium.chromoting;
 import android.view.MotionEvent;
 
 /**
- * An {@link Event} parameter to represent two {@link MotionEvent}s with a {@link #dx} and a
+ * An {@link Event} parameter to represent two {@link TapEventParameter}s with a {@link #dx} and a
  * {@link #dy} to indicate the distance between the points.
  *
  * {@link android.graphics.Point} and {@link android.graphics.PointF} are both mutable, so this
  * class uses two floats instead.
+ * handled field in Both {@link #event1} and {@link #event2} are ignored.
  */
 public final class TwoPointsEventParameter {
-    public final MotionEvent event1;
-    public final MotionEvent event2;
+    public final TapEventParameter event1;
+    public final TapEventParameter event2;
     public final float dx;
     public final float dy;
 
-    public TwoPointsEventParameter(MotionEvent event1,
-                                   MotionEvent event2,
+    public TwoPointsEventParameter(TapEventParameter event1,
+                                   TapEventParameter event2,
                                    float dx,
                                    float dy) {
         this.event1 = event1;
@@ -28,4 +29,14 @@ public final class TwoPointsEventParameter {
         this.dx = dx;
         this.dy = dy;
     }
+
+    public TwoPointsEventParameter(MotionEvent event1,
+                                   MotionEvent event2,
+                                   float dx,
+                                   float dy) {
+        this(new TapEventParameter(event1),
+             new TapEventParameter(event2),
+             dx,
+             dy);
+    }
 }
