@@ -15,14 +15,13 @@
 #include "jingle/glue/thread_wrapper.h"
 #include "net/socket/client_socket_factory.h"
 #include "remoting/base/chromium_url_request.h"
+#include "remoting/base/chromoting_event.h"
 #include "remoting/client/audio_player_android.h"
 #include "remoting/client/client_telemetry_logger.h"
 #include "remoting/client/jni/android_keymap.h"
 #include "remoting/client/jni/chromoting_jni_runtime.h"
-#include "remoting/client/jni/display_updater_factory.h"
 #include "remoting/client/jni/jni_client.h"
 #include "remoting/client/jni/jni_pairing_secret_fetcher.h"
-#include "remoting/client/jni/jni_video_renderer.h"
 #include "remoting/protocol/chromium_port_allocator_factory.h"
 #include "remoting/protocol/chromium_socket_factory.h"
 #include "remoting/protocol/client_authentication_config.h"
@@ -63,7 +62,10 @@ ChromotingJniInstance::ChromotingJniInstance(
     const std::string& pairing_id,
     const std::string& pairing_secret,
     const std::string& capabilities,
-    const std::string& flags)
+    const std::string& flags,
+    const std::string& host_version,
+    const std::string& host_os,
+    const std::string& host_os_version)
     : jni_runtime_(jni_runtime),
       jni_client_(jni_client),
       secret_fetcher_(secret_fetcher),
@@ -72,8 +74,12 @@ ChromotingJniInstance::ChromotingJniInstance(
       cursor_shape_stub_(std::move(cursor_shape_stub)),
       video_renderer_(std::move(video_renderer)),
       capabilities_(capabilities),
+      host_version_(host_version),
+      host_os_(host_os),
+      host_os_version_(host_os_version),
       weak_factory_(this) {
   DCHECK(jni_runtime_->ui_task_runner()->BelongsToCurrentThread());
+  weak_ptr_ = weak_factory_.GetWeakPtr();
 
   // Initialize XMPP config.
   xmpp_config_.host = kXmppServer;
@@ -369,12 +375,16 @@ void ChromotingJniInstance::InjectClipboardEvent(
 }
 
 base::WeakPtr<ChromotingJniInstance> ChromotingJniInstance::GetWeakPtr() {
-  return weak_factory_.GetWeakPtr();
+  return weak_ptr_;
 }
 
 void ChromotingJniInstance::ConnectToHostOnNetworkThread() {
   DCHECK(jni_runtime_->network_task_runner()->BelongsToCurrentThread());
 
+  jni_runtime_->logger()->SetHostInfo(
+      host_version_, ChromotingEvent::ParseOsFromString(host_os_),
+      host_os_version_);
+
   jingle_glue::JingleThreadWrapper::EnsureForCurrentMessageLoop();
 
   client_context_.reset(new ClientContext(jni_runtime_->network_task_runner()));
@@ -400,9 +410,9 @@ void ChromotingJniInstance::ConnectToHostOnNetworkThread() {
   scoped_refptr<protocol::TransportContext> transport_context =
       new protocol::TransportContext(
           signaling_.get(),
-          base::WrapUnique(new protocol::ChromiumPortAllocatorFactory()),
-          base::WrapUnique(
-              new ChromiumUrlRequestFactory(jni_runtime_->url_requester())),
+          base::MakeUnique<protocol::ChromiumPortAllocatorFactory>(),
+          base::MakeUnique<ChromiumUrlRequestFactory>(
+              jni_runtime_->url_requester()),
           protocol::NetworkSettings(
               protocol::NetworkSettings::NAT_TRAVERSAL_FULL),
           protocol::TransportRole::CLIENT);
