@@ -44,9 +44,6 @@ const int kVp9AqModeCyclicRefresh = 3;
 
 const int kDefaultTargetBitrateKbps = 1000;
 
-// Target quantizer at which stop the encoding top-off.
-const int kTargetQuantizerForVp8TopOff = 30;
-
 void SetCommonCodecParameters(vpx_codec_enc_cfg_t* config,
                               const webrtc::DesktopSize& size) {
   // Use millisecond granularity time base.
@@ -276,67 +273,56 @@ void WebrtcVideoEncoderVpx::SetLosslessColor(bool want_lossless) {
   }
 }
 
-void WebrtcVideoEncoderVpx::UpdateTargetBitrate(int new_bitrate_kbps) {
-  target_bitrate_kbps_ = new_bitrate_kbps;
-  // Configuration not initialized.
-  if (config_.g_timebase.den == 0)
-    return;
+std::unique_ptr<WebrtcVideoEncoder::EncodedFrame> WebrtcVideoEncoderVpx::Encode(
+    const webrtc::DesktopFrame* frame,
+    const FrameParams& params) {
+  webrtc::DesktopSize previous_frame_size =
+      image_ ? webrtc::DesktopSize(image_->w, image_->h)
+             : webrtc::DesktopSize();
 
-  if (config_.rc_target_bitrate == static_cast<unsigned int>(new_bitrate_kbps))
-    return;
-  config_.rc_target_bitrate = new_bitrate_kbps;
+  webrtc::DesktopSize frame_size = frame ? frame->size() : previous_frame_size;
 
-  // Update encoder context.
-  if (vpx_codec_enc_config_set(codec_.get(), &config_))
-    NOTREACHED() << "Unable to set encoder config";
-
-  VLOG(1) << "New rc_target_bitrate: " << new_bitrate_kbps << " kbps";
-}
-
-std::unique_ptr<VideoPacket> WebrtcVideoEncoderVpx::Encode(
-    const webrtc::DesktopFrame& frame,
-    uint32_t flags) {
-  DCHECK_LE(32, frame.size().width());
-  DCHECK_LE(32, frame.size().height());
-
-  // Based on information fetching active map, we return here if there is
-  // nothing to top-off.
-  if (frame.updated_region().is_empty() && !encode_unchanged_frame_)
+  // Don't need to send anything until we get the first non-null frame.
+  if (frame_size.is_empty()) {
     return nullptr;
+  }
+
+  DCHECK_GE(frame_size.width(), 32);
+  DCHECK_GE(frame_size.height(), 32);
 
   // Create or reconfigure the codec to match the size of |frame|.
-  if (!codec_ ||
-      (image_ &&
-       !frame.size().equals(webrtc::DesktopSize(image_->w, image_->h)))) {
-    Configure(frame.size());
+  if (!codec_ || !frame_size.equals(previous_frame_size)) {
+    Configure(frame_size);
   }
 
+  UpdateConfig(params);
+
   vpx_active_map_t act_map;
   act_map.rows = active_map_size_.height();
   act_map.cols = active_map_size_.width();
   act_map.active_map = active_map_.get();
 
   webrtc::DesktopRegion updated_region;
-  if (!frame.updated_region().is_empty()) {
-    // Convert the updated capture data ready for encode.
-    PrepareImage(frame, &updated_region);
+  // Convert the updated capture data ready for encode.
+  PrepareImage(frame, &updated_region);
 
-    // Update active map based on updated region.
-    SetActiveMapFromRegion(updated_region);
+  // Update active map based on updated region.
+  if (params.clear_active_map)
+    ClearActiveMap();
 
-    // Apply active map to the encoder.
+  if (params.key_frame)
+    updated_region.SetRect(webrtc::DesktopRect::MakeSize(frame_size));
 
-    if (vpx_codec_control(codec_.get(), VP8E_SET_ACTIVEMAP, &act_map)) {
-      LOG(ERROR) << "Unable to apply active map";
-    }
+  SetActiveMapFromRegion(updated_region);
+
+  // Apply active map to the encoder.
+  if (vpx_codec_control(codec_.get(), VP8E_SET_ACTIVEMAP, &act_map)) {
+    LOG(ERROR) << "Unable to apply active map";
   }
 
-  // Frame rate is adapted based on how well the encoder meets the target
-  // bandwidth requirement. We specify a target rate of 1 / 15 fps here.
-  // TODO(isheriff): Investigate if it makes sense to increase the target FPS.
   vpx_codec_err_t ret = vpx_codec_encode(
-      codec_.get(), image_.get(), 0, 66666,
-      (flags & REQUEST_KEY_FRAME) ? VPX_EFLAG_FORCE_KF : 0, VPX_DL_REALTIME);
+      codec_.get(), image_.get(), 0, params.duration.InMicroseconds(),
+      (params.key_frame) ? VPX_EFLAG_FORCE_KF : 0, VPX_DL_REALTIME);
   DCHECK_EQ(ret, VPX_CODEC_OK)
       << "Encoding error: " << vpx_codec_err_to_string(ret) << "\n"
       << "Details: " << vpx_codec_error(codec_.get()) << "\n"
@@ -350,13 +336,6 @@ std::unique_ptr<VideoPacket> WebrtcVideoEncoderVpx::Encode(
       DCHECK_EQ(ret, VPX_CODEC_OK)
           << "Failed to fetch active map: " << vpx_codec_err_to_string(ret)
           << "\n";
-
-      // If the encoder output no changes then there's nothing left to top-off.
-      encode_unchanged_frame_ = !updated_region.is_empty();
-    } else {
-      // Always set |encode_unchanged_frame_| when using VP8. It will be reset
-      // below once the target quantizer value is reached.
-      encode_unchanged_frame_ = true;
     }
 
     UpdateRegionFromActiveMap(&updated_region);
@@ -366,11 +345,8 @@ std::unique_ptr<VideoPacket> WebrtcVideoEncoderVpx::Encode(
   vpx_codec_iter_t iter = nullptr;
   bool got_data = false;
 
-  // TODO(hclam): Make sure we get exactly one frame from the packet.
-  // TODO(hclam): We should provide the output buffer to avoid one copy.
-  std::unique_ptr<VideoPacket> packet(
-      helper_.CreateVideoPacketWithUpdatedRegion(frame, updated_region));
-  packet->mutable_format()->set_encoding(VideoPacketFormat::ENCODING_VP8);
+  std::unique_ptr<EncodedFrame> encoded_frame(new EncodedFrame());
+  encoded_frame->size = frame_size;
 
   while (!got_data) {
     const vpx_codec_cx_pkt_t* vpx_packet =
@@ -381,15 +357,15 @@ std::unique_ptr<VideoPacket> WebrtcVideoEncoderVpx::Encode(
     switch (vpx_packet->kind) {
       case VPX_CODEC_CX_FRAME_PKT: {
         got_data = true;
-        packet->set_data(vpx_packet->data.frame.buf, vpx_packet->data.frame.sz);
-        packet->set_key_frame(vpx_packet->data.frame.flags & VPX_FRAME_IS_KEY);
-        int quantizer = -1;
+        // TODO(sergeyu): Avoid copying the data here..
+        encoded_frame->data.assign(
+            reinterpret_cast<const char*>(vpx_packet->data.frame.buf),
+            vpx_packet->data.frame.sz);
+        encoded_frame->key_frame =
+            vpx_packet->data.frame.flags & VPX_FRAME_IS_KEY;
         CHECK_EQ(vpx_codec_control(codec_.get(), VP8E_GET_LAST_QUANTIZER_64,
-                                   &quantizer),
+                                   &(encoded_frame->quantizer)),
                  VPX_CODEC_OK);
-        // VP8: Stop top-off as soon as the target quantizer value is reached.
-        if (!use_vp9_ && quantizer <= kTargetQuantizerForVp8TopOff)
-          encode_unchanged_frame_ = false;
         break;
       }
       default:
@@ -397,13 +373,11 @@ std::unique_ptr<VideoPacket> WebrtcVideoEncoderVpx::Encode(
     }
   }
 
-  return packet;
+  return encoded_frame;
 }
 
 WebrtcVideoEncoderVpx::WebrtcVideoEncoderVpx(bool use_vp9)
     : use_vp9_(use_vp9),
-      target_bitrate_kbps_(kDefaultTargetBitrateKbps),
-      encode_unchanged_frame_(false),
       clock_(&default_tick_clock_) {
   // Indicates config is still uninitialized.
   config_.g_timebase.den = 0;
@@ -424,6 +398,7 @@ void WebrtcVideoEncoderVpx::Configure(const webrtc::DesktopSize& size) {
       (size.height() + kMacroBlockSize - 1) / kMacroBlockSize);
   active_map_.reset(
       new uint8_t[active_map_size_.width() * active_map_size_.height()]);
+  ClearActiveMap();
 
   // TODO(wez): Remove this hack once VPX can handle frame size reconfiguration.
   // See https://code.google.com/p/webm/issues/detail?id=912.
@@ -447,7 +422,8 @@ void WebrtcVideoEncoderVpx::Configure(const webrtc::DesktopSize& size) {
   } else {
     SetVp8CodecParameters(&config_, size);
   }
-  config_.rc_target_bitrate = target_bitrate_kbps_;
+
+  config_.rc_target_bitrate = kDefaultTargetBitrateKbps;
 
   // Initialize or re-configure the codec with the custom configuration.
   if (!codec_) {
@@ -467,18 +443,55 @@ void WebrtcVideoEncoderVpx::Configure(const webrtc::DesktopSize& size) {
   }
 }
 
+void WebrtcVideoEncoderVpx::UpdateConfig(const FrameParams& params) {
+  // Configuration not initialized.
+  if (config_.g_timebase.den == 0)
+    return;
+
+  bool changed = false;
+
+  if (params.bitrate_kbps >= 0 &&
+      config_.rc_target_bitrate !=
+          static_cast<unsigned int>(params.bitrate_kbps)) {
+    config_.rc_target_bitrate = params.bitrate_kbps;
+    changed = true;
+  }
+
+  if (params.vpx_min_quantizer >= 0 &&
+      config_.rc_min_quantizer !=
+          static_cast<unsigned int>(params.vpx_min_quantizer)) {
+    config_.rc_min_quantizer = params.vpx_min_quantizer;
+    changed = true;
+  }
+
+  if (params.vpx_max_quantizer >= 0 &&
+      config_.rc_max_quantizer !=
+          static_cast<unsigned int>(params.vpx_max_quantizer)) {
+    config_.rc_max_quantizer = params.vpx_max_quantizer;
+    changed = true;
+  }
+
+  if (!changed)
+    return;
+
+  // Update encoder context.
+  if (vpx_codec_enc_config_set(codec_.get(), &config_))
+    NOTREACHED() << "Unable to set encoder config";
+
+}
+
 void WebrtcVideoEncoderVpx::PrepareImage(
-    const webrtc::DesktopFrame& frame,
+    const webrtc::DesktopFrame* frame,
     webrtc::DesktopRegion* updated_region) {
-  if (frame.updated_region().is_empty()) {
+  if (!frame || frame->updated_region().is_empty()) {
     updated_region->Clear();
     return;
   }
 
   updated_region->Clear();
   if (image_) {
-    // Pad each rectangle to avoid the block-artefact filters in libvpx from
-    // introducing artefacts; VP9 includes up to 8px either side, and VP8 up to
+    // Pad each rectangle to avoid the block-artifact filters in libvpx from
+    // introducing artifacts; VP9 includes up to 8px either side, and VP8 up to
     // 3px, so unchanged pixels up to that far out may still be affected by the
     // changes in the updated region, and so must be listed in the active map.
     // After padding we align each rectangle to 16x16 active-map macroblocks.
@@ -486,7 +499,7 @@ void WebrtcVideoEncoderVpx::PrepareImage(
     // is required by ConvertRGBToYUVWithRect().
     // TODO(wez): Do we still need 16x16 align, or is even alignment sufficient?
     int padding = use_vp9_ ? 8 : 3;
-    for (webrtc::DesktopRegion::Iterator r(frame.updated_region());
+    for (webrtc::DesktopRegion::Iterator r(frame->updated_region());
          !r.IsAtEnd(); r.Advance()) {
       const webrtc::DesktopRect& rect = r.rect();
       updated_region->AddRect(AlignRect(webrtc::DesktopRect::MakeLTRB(
@@ -501,13 +514,13 @@ void WebrtcVideoEncoderVpx::PrepareImage(
     updated_region->IntersectWith(
         webrtc::DesktopRect::MakeWH(image_->w, image_->h));
   } else {
-    CreateImage(lossless_color_, frame.size(), &image_, &image_buffer_);
+    CreateImage(lossless_color_, frame->size(), &image_, &image_buffer_);
     updated_region->AddRect(webrtc::DesktopRect::MakeWH(image_->w, image_->h));
   }
 
   // Convert the updated region to YUV ready for encoding.
-  const uint8_t* rgb_data = frame.data();
-  const int rgb_stride = frame.stride();
+  const uint8_t* rgb_data = frame->data();
+  const int rgb_stride = frame->stride();
   const int y_stride = image_->stride[0];
   DCHECK_EQ(image_->stride[1], image_->stride[2]);
   const int uv_stride = image_->stride[1];
@@ -549,12 +562,15 @@ void WebrtcVideoEncoderVpx::PrepareImage(
   }
 }
 
-void WebrtcVideoEncoderVpx::SetActiveMapFromRegion(
-    const webrtc::DesktopRegion& updated_region) {
+void WebrtcVideoEncoderVpx::ClearActiveMap() {
+  DCHECK(active_map_);
   // Clear active map first.
   memset(active_map_.get(), 0,
          active_map_size_.width() * active_map_size_.height());
+}
 
+void WebrtcVideoEncoderVpx::SetActiveMapFromRegion(
+    const webrtc::DesktopRegion& updated_region) {
   // Mark updated areas active.
   for (webrtc::DesktopRegion::Iterator r(updated_region); !r.IsAtEnd();
        r.Advance()) {
