@@ -15,11 +15,7 @@
 #include "remoting/base/capabilities.h"
 #include "remoting/base/constants.h"
 #include "remoting/base/logging.h"
-#include "remoting/codec/audio_encoder.h"
-#include "remoting/codec/audio_encoder_opus.h"
-#include "remoting/codec/audio_encoder_verbatim.h"
 #include "remoting/host/audio_capturer.h"
-#include "remoting/host/audio_pump.h"
 #include "remoting/host/desktop_environment.h"
 #include "remoting/host/host_extension_session.h"
 #include "remoting/host/input_injector.h"
@@ -28,6 +24,7 @@
 #include "remoting/host/screen_resolution.h"
 #include "remoting/proto/control.pb.h"
 #include "remoting/proto/event.pb.h"
+#include "remoting/protocol/audio_stream.h"
 #include "remoting/protocol/client_stub.h"
 #include "remoting/protocol/clipboard_thread_proxy.h"
 #include "remoting/protocol/pairing_registry.h"
@@ -43,27 +40,13 @@ namespace {
 // Name of command-line flag to disable use of I444 by default.
 const char kDisableI444SwitchName[] = "disable-i444";
 
-std::unique_ptr<AudioEncoder> CreateAudioEncoder(
-    const protocol::SessionConfig& config) {
-  const protocol::ChannelConfig& audio_config = config.audio_config();
-
-  if (audio_config.codec == protocol::ChannelConfig::CODEC_VERBATIM) {
-    return base::WrapUnique(new AudioEncoderVerbatim());
-  } else if (audio_config.codec == protocol::ChannelConfig::CODEC_OPUS) {
-    return base::WrapUnique(new AudioEncoderOpus());
-  }
-
-  NOTREACHED();
-  return nullptr;
-}
-
 }  // namespace
 
 ClientSession::ClientSession(
     EventHandler* event_handler,
-    scoped_refptr<base::SingleThreadTaskRunner> audio_task_runner,
     std::unique_ptr<protocol::ConnectionToClient> connection,
     DesktopEnvironmentFactory* desktop_environment_factory,
+    const DesktopEnvironmentOptions& desktop_environment_options,
     const base::TimeDelta& max_duration,
     scoped_refptr<protocol::PairingRegistry> pairing_registry,
     const std::vector<HostExtension*>& extensions)
@@ -71,6 +54,7 @@ ClientSession::ClientSession(
       connection_(std::move(connection)),
       client_jid_(connection_->session()->jid()),
       desktop_environment_factory_(desktop_environment_factory),
+      desktop_environment_options_(desktop_environment_options),
       input_tracker_(&host_input_filter_),
       remote_input_filter_(&input_tracker_),
       mouse_clamping_filter_(&remote_input_filter_),
@@ -78,7 +62,6 @@ ClientSession::ClientSession(
       disable_clipboard_filter_(clipboard_echo_filter_.host_filter()),
       client_clipboard_factory_(clipboard_echo_filter_.client_filter()),
       max_duration_(max_duration),
-      audio_task_runner_(audio_task_runner),
       pairing_registry_(pairing_registry),
       // Note that |lossless_video_color_| defaults to true, but actually only
       // controls VP9 video stream color quality.
@@ -99,7 +82,7 @@ ClientSession::ClientSession(
 
 ClientSession::~ClientSession() {
   DCHECK(CalledOnValidThread());
-  DCHECK(!audio_pump_);
+  DCHECK(!audio_stream_);
   DCHECK(!desktop_environment_);
   DCHECK(!input_injector_);
   DCHECK(!screen_controls_);
@@ -120,12 +103,23 @@ void ClientSession::NotifyClientResolution(
   if (!screen_controls_)
     return;
 
-  ScreenResolution client_resolution(
-      webrtc::DesktopSize(resolution.dips_width(), resolution.dips_height()),
-      webrtc::DesktopVector(kDefaultDpi, kDefaultDpi));
+  webrtc::DesktopSize client_size(resolution.dips_width(),
+                                  resolution.dips_height());
+  if (connection_->session()->config().protocol() ==
+      protocol::SessionConfig::Protocol::WEBRTC) {
+    // When using WebRTC round down the dimensions to multiple of 2. Otherwise
+    // the dimensions will be rounded on the receiver, which will cause blurring
+    // due to scaling. The resulting size is still close to the client size and
+    // will fit on the client's screen without scaling.
+    // TODO(sergeyu): Make WebRTC handle odd dimensions properly.
+    // crbug.com/636071
+    client_size.set(client_size.width() & (~1), client_size.height() & (~1));
+  }
 
   // Try to match the client's resolution.
-  screen_controls_->SetScreenResolution(client_resolution);
+  // TODO(sergeyu): Pass clients DPI to the resizer.
+  screen_controls_->SetScreenResolution(ScreenResolution(
+      client_size, webrtc::DesktopVector(kDefaultDpi, kDefaultDpi)));
 }
 
 void ClientSession::ControlVideo(const protocol::VideoControl& video_control) {
@@ -162,8 +156,8 @@ void ClientSession::ControlAudio(const protocol::AudioControl& audio_control) {
   if (audio_control.has_enable()) {
     VLOG(1) << "Received AudioControl (enable="
             << audio_control.enable() << ")";
-    if (audio_pump_)
-      audio_pump_->Pause(!audio_control.enable());
+    if (audio_stream_)
+      audio_stream_->Pause(!audio_control.enable());
   }
 }
 
@@ -178,7 +172,7 @@ void ClientSession::SetCapabilities(
   }
 
   // Compute the set of capabilities supported by both client and host.
-  client_capabilities_ = base::WrapUnique(new std::string());
+  client_capabilities_ = base::MakeUnique<std::string>();
   if (capabilities.has_capabilities())
     *client_capabilities_ = capabilities.capabilities();
   capabilities_ = IntersectCapabilities(*client_capabilities_,
@@ -223,16 +217,13 @@ void ClientSession::DeliverClientMessage(
   }
 }
 
-void ClientSession::OnConnectionAuthenticating(
-    protocol::ConnectionToClient* connection) {
+void ClientSession::OnConnectionAuthenticating() {
   event_handler_->OnSessionAuthenticating(this);
 }
 
-void ClientSession::OnConnectionAuthenticated(
-    protocol::ConnectionToClient* connection) {
+void ClientSession::OnConnectionAuthenticated() {
   DCHECK(CalledOnValidThread());
-  DCHECK_EQ(connection_.get(), connection);
-  DCHECK(!audio_pump_);
+  DCHECK(!audio_stream_);
   DCHECK(!desktop_environment_);
   DCHECK(!input_injector_);
   DCHECK(!screen_controls_);
@@ -252,8 +243,8 @@ void ClientSession::OnConnectionAuthenticated(
 
   // Create the desktop environment. Drop the connection if it could not be
   // created for any reason (for instance the curtain could not initialize).
-  desktop_environment_ =
-      desktop_environment_factory_->Create(weak_factory_.GetWeakPtr());
+  desktop_environment_ = desktop_environment_factory_->Create(
+      weak_factory_.GetWeakPtr(), desktop_environment_options_);
   if (!desktop_environment_) {
     DisconnectSession(protocol::HOST_CONFIGURATION_ERROR);
     return;
@@ -284,15 +275,20 @@ void ClientSession::OnConnectionAuthenticated(
   clipboard_echo_filter_.set_client_stub(connection_->client_stub());
 }
 
-void ClientSession::CreateVideoStreams(
-    protocol::ConnectionToClient* connection) {
+void ClientSession::CreateMediaStreams() {
   DCHECK(CalledOnValidThread());
-  DCHECK_EQ(connection_.get(), connection);
 
   // Create a VideoStream to pump frames from the capturer to the client.
   video_stream_ = connection_->StartVideoStream(
       desktop_environment_->CreateVideoCapturer());
 
+  // Create a AudioStream to pump audio from the capturer to the client.
+  std::unique_ptr<protocol::AudioSource> audio_capturer =
+      desktop_environment_->CreateAudioCapturer();
+  if (audio_capturer) {
+    audio_stream_ = connection_->StartAudioStream(std::move(audio_capturer));
+  }
+
   video_stream_->SetObserver(this);
 
   // Apply video-control parameters to the new stream.
@@ -301,12 +297,13 @@ void ClientSession::CreateVideoStreams(
 
   // Pause capturing if necessary.
   video_stream_->Pause(pause_video_);
+
+  if (event_timestamp_source_for_tests_)
+    video_stream_->SetEventTimestampsSource(event_timestamp_source_for_tests_);
 }
 
-void ClientSession::OnConnectionChannelsConnected(
-    protocol::ConnectionToClient* connection) {
+void ClientSession::OnConnectionChannelsConnected() {
   DCHECK(CalledOnValidThread());
-  DCHECK_EQ(connection_.get(), connection);
 
   DCHECK(!channels_connected_);
   channels_connected_ = true;
@@ -326,15 +323,6 @@ void ClientSession::OnConnectionChannelsConnected(
       new MouseShapePump(desktop_environment_->CreateMouseCursorMonitor(),
                          connection_->client_stub()));
 
-  // Create an AudioPump if audio is enabled, to pump audio samples.
-  if (connection_->session()->config().is_audio_enabled()) {
-    std::unique_ptr<AudioEncoder> audio_encoder =
-        CreateAudioEncoder(connection_->session()->config());
-    audio_pump_.reset(new AudioPump(
-        audio_task_runner_, desktop_environment_->CreateAudioCapturer(),
-        std::move(audio_encoder), connection_->audio_stub()));
-  }
-
   if (pending_video_layout_message_) {
     connection_->client_stub()->SetVideoLayout(*pending_video_layout_message_);
     pending_video_layout_message_.reset();
@@ -344,11 +332,8 @@ void ClientSession::OnConnectionChannelsConnected(
   event_handler_->OnSessionChannelsConnected(this);
 }
 
-void ClientSession::OnConnectionClosed(
-    protocol::ConnectionToClient* connection,
-    protocol::ErrorCode error) {
+void ClientSession::OnConnectionClosed(protocol::ErrorCode error) {
   DCHECK(CalledOnValidThread());
-  DCHECK_EQ(connection_.get(), connection);
 
   HOST_LOG << "Client disconnected: " << client_jid_ << "; error = " << error;
 
@@ -364,7 +349,7 @@ void ClientSession::OnConnectionClosed(
 
   // Stop components access the client, audio or video stubs, which are no
   // longer valid once ConnectionToClient calls OnConnectionClosed().
-  audio_pump_.reset();
+  audio_stream_.reset();
   video_stream_.reset();
   mouse_shape_pump_.reset();
   client_clipboard_factory_.InvalidateWeakPtrs();
@@ -376,22 +361,10 @@ void ClientSession::OnConnectionClosed(
   event_handler_->OnSessionClosed(this);
 }
 
-void ClientSession::OnInputEventReceived(
-    protocol::ConnectionToClient* connection,
-    int64_t event_timestamp) {
-  DCHECK(CalledOnValidThread());
-  DCHECK_EQ(connection_.get(), connection);
-
-  if (video_stream_.get())
-    video_stream_->OnInputEventReceived(event_timestamp);
-}
-
 void ClientSession::OnRouteChange(
-    protocol::ConnectionToClient* connection,
     const std::string& channel_name,
     const protocol::TransportRoute& route) {
   DCHECK(CalledOnValidThread());
-  DCHECK_EQ(connection_.get(), connection);
   event_handler_->OnSessionRouteChange(this, channel_name, route);
 }
 
@@ -436,12 +409,20 @@ ClientSessionControl* ClientSession::session_control() {
   return this;
 }
 
+void ClientSession::SetEventTimestampsSourceForTests(
+    scoped_refptr<protocol::InputEventTimestampsSource>
+        event_timestamp_source) {
+  DCHECK(CalledOnValidThread());
+  event_timestamp_source_for_tests_ = event_timestamp_source;
+  if (video_stream_)
+    video_stream_->SetEventTimestampsSource(event_timestamp_source_for_tests_);
+}
+
 std::unique_ptr<protocol::ClipboardStub> ClientSession::CreateClipboardProxy() {
   DCHECK(CalledOnValidThread());
-
-  return base::WrapUnique(
-      new protocol::ClipboardThreadProxy(client_clipboard_factory_.GetWeakPtr(),
-                                         base::ThreadTaskRunnerHandle::Get()));
+  return base::MakeUnique<protocol::ClipboardThreadProxy>(
+      client_clipboard_factory_.GetWeakPtr(),
+      base::ThreadTaskRunnerHandle::Get());
 }
 
 void ClientSession::OnVideoSizeChanged(protocol::VideoStream* video_stream,
@@ -487,10 +468,4 @@ void ClientSession::OnVideoSizeChanged(protocol::VideoStream* video_stream,
   }
 }
 
-void ClientSession::OnVideoFrameSent(protocol::VideoStream* stream,
-                                     uint32_t frame_id,
-                                     int64_t input_event_timestamp) {
-  // TODO(sergeyu): Send a message to the client to notify about the new frame.
-}
-
 }  // namespace remoting
