@@ -4,6 +4,7 @@
 
 #include "remoting/host/ipc_video_frame_capturer.h"
 
+#include "base/logging.h"
 #include "remoting/host/desktop_session_proxy.h"
 #include "third_party/webrtc/modules/desktop_capture/desktop_frame.h"
 
@@ -27,7 +28,7 @@ void IpcVideoFrameCapturer::Start(Callback* callback) {
   desktop_session_proxy_->SetVideoCapturer(weak_factory_.GetWeakPtr());
 }
 
-void IpcVideoFrameCapturer::Capture(const webrtc::DesktopRegion& region) {
+void IpcVideoFrameCapturer::CaptureFrame() {
   DCHECK(!capture_pending_);
   capture_pending_ = true;
   desktop_session_proxy_->CaptureFrame();
@@ -41,4 +42,14 @@ void IpcVideoFrameCapturer::OnCaptureResult(
   callback_->OnCaptureResult(result, std::move(frame));
 }
 
+bool IpcVideoFrameCapturer::GetSourceList(SourceList* sources) {
+  NOTIMPLEMENTED();
+  return false;
+}
+
+bool IpcVideoFrameCapturer::SelectSource(SourceId id) {
+  NOTIMPLEMENTED();
+  return false;
+}
+
 }  // namespace remoting
