@@ -6,11 +6,10 @@
 #define REMOTING_HOST_IT2ME_IT2ME_CONFIRMATION_DIALOG_H_
 
 #include <memory>
+#include <string>
 
 #include "base/callback_forward.h"
 #include "base/macros.h"
-#include "base/memory/ref_counted.h"
-#include "base/single_thread_task_runner.h"
 
 namespace remoting {
 
@@ -26,22 +25,14 @@ class It2MeConfirmationDialog {
 
   typedef base::Callback<void(Result)> ResultCallback;
 
+  static std::unique_ptr<It2MeConfirmationDialog> Create();
+
   virtual ~It2MeConfirmationDialog() {}
 
   // Shows the dialog. |callback| will be called with the user's selection.
   // |callback| will not be called if the dialog is destroyed.
-  virtual void Show(const ResultCallback& callback) = 0;
-};
-
-// Used to create an platform specific instance of It2MeConfirmationDialog.
-class It2MeConfirmationDialogFactory {
- public:
-  It2MeConfirmationDialogFactory();
-  virtual ~It2MeConfirmationDialogFactory();
-
-  virtual std::unique_ptr<It2MeConfirmationDialog> Create();
-
-  DISALLOW_COPY_AND_ASSIGN(It2MeConfirmationDialogFactory);
+  virtual void Show(const std::string& remote_user_email,
+                    const ResultCallback& callback) = 0;
 };
 
 }  // namespace remoting
