@@ -4,20 +4,25 @@
 
 #include "remoting/host/it2me/it2me_native_messaging_host.h"
 
-#include <stdint.h>
-
+#include <cstdint>
+#include <memory>
+#include <string>
 #include <utility>
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
+#include "base/location.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/stl_util.h"
 #include "base/strings/stringize_macros.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
+#include "components/policy/core/common/mock_policy_service.h"
 #include "net/base/file_stream.h"
 #include "remoting/base/auto_thread_task_runner.h"
 #include "remoting/host/chromoting_host_context.h"
@@ -70,20 +75,67 @@ void VerifyCommonProperties(std::unique_ptr<base::DictionaryValue> response,
   EXPECT_EQ(id, int_value);
 }
 
-}  // namespace
+class FakePolicyService : public policy::PolicyService {
+ public:
+  FakePolicyService();
+  ~FakePolicyService() override;
+
+  // policy::PolicyService overrides.
+  void AddObserver(policy::PolicyDomain domain,
+                   policy::PolicyService::Observer* observer) override;
+  void RemoveObserver(policy::PolicyDomain domain,
+                      policy::PolicyService::Observer* observer) override;
+  const policy::PolicyMap& GetPolicies(
+      const policy::PolicyNamespace& ns) const override;
+  bool IsInitializationComplete(policy::PolicyDomain domain) const override;
+  void RefreshPolicies(const base::Closure& callback) override;
+
+ private:
+  policy::PolicyMap policy_map_;
+
+  DISALLOW_COPY_AND_ASSIGN(FakePolicyService);
+};
+
+FakePolicyService::FakePolicyService() {}
+
+FakePolicyService::~FakePolicyService() {}
+
+void FakePolicyService::AddObserver(policy::PolicyDomain domain,
+                                    policy::PolicyService::Observer* observer) {
+}
+
+void FakePolicyService::RemoveObserver(
+    policy::PolicyDomain domain,
+    policy::PolicyService::Observer* observer) {}
+
+const policy::PolicyMap& FakePolicyService::GetPolicies(
+    const policy::PolicyNamespace& ns) const {
+  return policy_map_;
+}
+
+bool FakePolicyService::IsInitializationComplete(
+    policy::PolicyDomain domain) const {
+  return true;
+}
+
+void FakePolicyService::RefreshPolicies(const base::Closure& callback) {
+  base::ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, callback);
+}
 
 class MockIt2MeHost : public It2MeHost {
  public:
   MockIt2MeHost(std::unique_ptr<ChromotingHostContext> context,
                 std::unique_ptr<PolicyWatcher> policy_watcher,
                 base::WeakPtr<It2MeHost::Observer> observer,
-                const XmppSignalStrategy::XmppServerConfig& xmpp_server_config,
+                std::unique_ptr<SignalStrategy> signal_strategy,
+                const std::string& username,
                 const std::string& directory_bot_jid)
       : It2MeHost(std::move(context),
                   std::move(policy_watcher),
-                  nullptr,
+                  /*confirmation_dialog_factory=*/nullptr,
                   observer,
-                  xmpp_server_config,
+                  std::move(signal_strategy),
+                  username,
                   directory_bot_jid) {}
 
   // It2MeHost overrides
@@ -101,7 +153,6 @@ class MockIt2MeHost : public It2MeHost {
 
 void MockIt2MeHost::Connect() {
   if (!host_context()->ui_task_runner()->BelongsToCurrentThread()) {
-    DCHECK(task_runner()->BelongsToCurrentThread());
     host_context()->ui_task_runner()->PostTask(
         FROM_HERE, base::Bind(&MockIt2MeHost::Connect, this));
     return;
@@ -113,27 +164,24 @@ void MockIt2MeHost::Connect() {
   std::string access_code(kTestAccessCode);
   base::TimeDelta lifetime =
       base::TimeDelta::FromSeconds(kTestAccessCodeLifetimeInSeconds);
-  task_runner()->PostTask(FROM_HERE,
-                          base::Bind(&It2MeHost::Observer::OnStoreAccessCode,
-                                     observer(),
-                                     access_code,
-                                     lifetime));
+  host_context()->ui_task_runner()->PostTask(
+      FROM_HERE, base::Bind(&It2MeHost::Observer::OnStoreAccessCode, observer(),
+                            access_code, lifetime));
 
   RunSetState(kReceivedAccessCode);
+  RunSetState(kConnecting);
 
   std::string client_username(kTestClientUsername);
-  task_runner()->PostTask(
-      FROM_HERE,
-      base::Bind(&It2MeHost::Observer::OnClientAuthenticated,
-                 observer(),
-                 client_username));
+  host_context()->ui_task_runner()->PostTask(
+      FROM_HERE, base::Bind(&It2MeHost::Observer::OnClientAuthenticated,
+                            observer(), client_username));
 
   RunSetState(kConnected);
 }
 
 void MockIt2MeHost::Disconnect() {
   if (!host_context()->network_task_runner()->BelongsToCurrentThread()) {
-    DCHECK(task_runner()->BelongsToCurrentThread());
+    DCHECK(host_context()->ui_task_runner()->BelongsToCurrentThread());
     host_context()->network_task_runner()->PostTask(
         FROM_HERE, base::Bind(&MockIt2MeHost::Disconnect, this));
     return;
@@ -155,19 +203,39 @@ void MockIt2MeHost::RunSetState(It2MeHostState state) {
 
 class MockIt2MeHostFactory : public It2MeHostFactory {
  public:
-  MockIt2MeHostFactory() : It2MeHostFactory() {}
+  MockIt2MeHostFactory();
+  ~MockIt2MeHostFactory() override;
+
   scoped_refptr<It2MeHost> CreateIt2MeHost(
       std::unique_ptr<ChromotingHostContext> context,
+      policy::PolicyService* policy_service,
       base::WeakPtr<It2MeHost::Observer> observer,
-      const XmppSignalStrategy::XmppServerConfig& xmpp_server_config,
-      const std::string& directory_bot_jid) override {
-    return new MockIt2MeHost(std::move(context), nullptr, observer,
-                             xmpp_server_config, directory_bot_jid);
-  }
+      std::unique_ptr<SignalStrategy> signal_strategy,
+      const std::string& username,
+      const std::string& directory_bot_jid) override;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockIt2MeHostFactory);
-};  // MockIt2MeHostFactory
+};
+
+MockIt2MeHostFactory::MockIt2MeHostFactory() : It2MeHostFactory() {}
+
+MockIt2MeHostFactory::~MockIt2MeHostFactory() {}
+
+scoped_refptr<It2MeHost> MockIt2MeHostFactory::CreateIt2MeHost(
+    std::unique_ptr<ChromotingHostContext> context,
+    policy::PolicyService* policy_service,
+    base::WeakPtr<It2MeHost::Observer> observer,
+    std::unique_ptr<SignalStrategy> signal_strategy,
+    const std::string& username,
+    const std::string& directory_bot_jid) {
+  return new MockIt2MeHost(std::move(context),
+                           /*policy_watcher=*/nullptr, observer,
+                           std::move(signal_strategy), username,
+                           directory_bot_jid);
+}
+
+}  // namespace
 
 class It2MeNativeMessagingHostTest : public testing::Test {
  public:
@@ -217,6 +285,8 @@ class It2MeNativeMessagingHostTest : public testing::Test {
   scoped_refptr<AutoThreadTaskRunner> host_task_runner_;
   std::unique_ptr<remoting::NativeMessagingPipe> pipe_;
 
+  std::unique_ptr<policy::PolicyService> fake_policy_service_;
+
   DISALLOW_COPY_AND_ASSIGN(It2MeNativeMessagingHostTest);
 };
 
@@ -224,6 +294,11 @@ void It2MeNativeMessagingHostTest::SetUp() {
   test_message_loop_.reset(new base::MessageLoop());
   test_run_loop_.reset(new base::RunLoop());
 
+#if defined(OS_CHROMEOS)
+  // On Chrome OS, the browser owns the PolicyService so simulate that here.
+  fake_policy_service_.reset(new FakePolicyService());
+#endif  // defined(OS_CHROMEOS)
+
   // Run the host on a dedicated thread.
   host_thread_.reset(new base::Thread("host_thread"));
   host_thread_->Start();
@@ -277,7 +352,7 @@ It2MeNativeMessagingHostTest::ReadMessageFromOutputPipe() {
 
     std::string message_json(length, '\0');
     read_result = output_read_file_.ReadAtCurrentPos(
-        string_as_array(&message_json), length);
+        base::string_as_array(&message_json), length);
     if (read_result != static_cast<int>(length)) {
       LOG(ERROR) << "Message size (" << read_result
                  << ") doesn't match the header (" << length << ").";
@@ -285,7 +360,7 @@ It2MeNativeMessagingHostTest::ReadMessageFromOutputPipe() {
     }
 
     std::unique_ptr<base::Value> message = base::JSONReader::Read(message_json);
-    if (!message || !message->IsType(base::Value::TYPE_DICTIONARY)) {
+    if (!message || !message->IsType(base::Value::Type::DICTIONARY)) {
       LOG(ERROR) << "Malformed message:" << message_json;
       return nullptr;
     }
@@ -327,10 +402,11 @@ void It2MeNativeMessagingHostTest::VerifyConnectResponses(int request_id) {
   bool starting_received = false;
   bool requestedAccessCode_received = false;
   bool receivedAccessCode_received = false;
+  bool connecting_received = false;
   bool connected_received = false;
 
-  // We expect a total of 5 messages: 1 connectResponse and 4 hostStateChanged.
-  for (int i = 0; i < 5; ++i) {
+  // We expect a total of 6 messages: 1 connectResponse and 5 hostStateChanged.
+  for (int i = 0; i < 6; ++i) {
     std::unique_ptr<base::DictionaryValue> response =
         ReadMessageFromOutputPipe();
     ASSERT_TRUE(response);
@@ -366,6 +442,10 @@ void It2MeNativeMessagingHostTest::VerifyConnectResponses(int request_id) {
         EXPECT_TRUE(
             response->GetInteger("accessCodeLifetime", &accessCodeLifetime));
         EXPECT_EQ(kTestAccessCodeLifetimeInSeconds, accessCodeLifetime);
+      } else if (state ==
+                 It2MeNativeMessagingHost::HostStateToString(kConnecting)) {
+        EXPECT_FALSE(connecting_received);
+        connecting_received = true;
       } else if (state ==
                  It2MeNativeMessagingHost::HostStateToString(kConnected)) {
         EXPECT_FALSE(connected_received);
@@ -426,8 +506,9 @@ void It2MeNativeMessagingHostTest::TestBadRequest(const base::Value& message,
 
   VerifyHelloResponse(1);
 
-  if (expect_error_response)
+  if (expect_error_response) {
     VerifyErrorResponse();
+  }
 
   std::unique_ptr<base::DictionaryValue> response = ReadMessageFromOutputPipe();
   EXPECT_FALSE(response);
@@ -451,6 +532,7 @@ void It2MeNativeMessagingHostTest::StartHost() {
   // Creating a native messaging host with a mock It2MeHostFactory.
   std::unique_ptr<extensions::NativeMessageHost> it2me_host(
       new It2MeNativeMessagingHost(
+          /*needs_elevation=*/false, fake_policy_service_.get(),
           ChromotingHostContext::Create(host_task_runner_),
           base::WrapUnique(new MockIt2MeHostFactory())));
   it2me_host->Start(pipe_.get());
@@ -531,8 +613,9 @@ TEST_F(It2MeNativeMessagingHostTest, Connect) {
   // A new It2MeHost instance is created for every it2me session. The native
   // messaging host, on the other hand, is long lived. This test verifies
   // multiple It2Me host startup and shutdowns.
-  for (int i = 0; i < 3; ++i)
+  for (int i = 0; i < 3; ++i) {
     TestConnect();
+  }
 }
 
 // Verify non-Dictionary requests are rejected.
