@@ -22,8 +22,8 @@
 #include "base/mac/mac_logging.h"
 #include "base/mac/scoped_launch_data.h"
 #include "base/posix/eintr_wrapper.h"
-#include "remoting/host/constants_mac.h"
 #include "remoting/host/host_config.h"
+#include "remoting/host/mac/constants_mac.h"
 #import "remoting/host/mac/me2me_preference_pane_confirm_pin.h"
 #import "remoting/host/mac/me2me_preference_pane_disable.h"
 #include "remoting/host/pin_hash.h"
