@@ -5,8 +5,8 @@
 // This file implements a standalone host process for Me2Me.
 
 #include <stddef.h>
-#include <stdint.h>
 
+#include <cstdint>
 #include <memory>
 #include <string>
 #include <utility>
@@ -20,23 +20,25 @@
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringize_macros.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
-#include "ipc/attachment_broker_unprivileged.h"
+#include "components/policy/policy_constants.h"
 #include "ipc/ipc_channel.h"
 #include "ipc/ipc_channel_proxy.h"
 #include "ipc/ipc_listener.h"
 #include "jingle/glue/thread_wrapper.h"
+#include "mojo/edk/embedder/embedder.h"
+#include "mojo/edk/embedder/platform_channel_pair.h"
+#include "mojo/edk/embedder/scoped_ipc_support.h"
 #include "net/base/network_change_notifier.h"
 #include "net/base/url_util.h"
 #include "net/socket/client_socket_factory.h"
-#include "net/socket/ssl_server_socket.h"
 #include "net/url_request/url_fetcher.h"
-#include "policy/policy_constants.h"
 #include "remoting/base/auto_thread_task_runner.h"
 #include "remoting/base/breakpad.h"
 #include "remoting/base/chromium_url_request.h"
@@ -51,6 +53,7 @@
 #include "remoting/host/config_file_watcher.h"
 #include "remoting/host/config_watcher.h"
 #include "remoting/host/desktop_environment.h"
+#include "remoting/host/desktop_environment_options.h"
 #include "remoting/host/desktop_session_connector.h"
 #include "remoting/host/dns_blackhole_checker.h"
 #include "remoting/host/gcd_rest_client.h"
@@ -61,6 +64,7 @@
 #include "remoting/host/host_event_logger.h"
 #include "remoting/host/host_exit_codes.h"
 #include "remoting/host/host_main.h"
+#include "remoting/host/host_power_save_blocker.h"
 #include "remoting/host/host_status_logger.h"
 #include "remoting/host/input_injector.h"
 #include "remoting/host/ipc_desktop_environment.h"
@@ -71,8 +75,8 @@
 #include "remoting/host/pairing_registry_delegate.h"
 #include "remoting/host/pin_hash.h"
 #include "remoting/host/policy_watcher.h"
-#include "remoting/host/security_key/gnubby_auth_handler.h"
-#include "remoting/host/security_key/gnubby_extension.h"
+#include "remoting/host/security_key/security_key_auth_handler.h"
+#include "remoting/host/security_key/security_key_extension.h"
 #include "remoting/host/service_urls.h"
 #include "remoting/host/shutdown_watchdog.h"
 #include "remoting/host/signaling_connector.h"
@@ -148,12 +152,14 @@ const char kStdinConfigPath[] = "-";
 // The command line switch used to pass name of the pipe to capture audio on
 // linux.
 const char kAudioPipeSwitchName[] = "audio-pipe-name";
-
-// The command line switch used to pass name of the unix domain socket used to
-// listen for gnubby requests.
-const char kAuthSocknameSwitchName[] = "ssh-auth-sockname";
 #endif  // defined(OS_LINUX)
 
+#if defined(OS_POSIX)
+// The command line switch used to pass name of the unix domain socket used to
+// listen for security key requests.
+const char kAuthSocknameSwitchName[] = "ssh-auth-sockname";
+#endif  // defined(OS_POSIX)
+
 // The command line switch used by the parent to request the host to signal it
 // when it is successfully started.
 const char kSignalParentSwitchName[] = "signal-parent";
@@ -371,10 +377,10 @@ class HostProcess : public ConfigWatcher::Delegate,
   std::string talkgadget_prefix_;
   bool allow_pairing_ = true;
 
-  bool curtain_required_ = false;
+  DesktopEnvironmentOptions desktop_environment_options_;
   ThirdPartyAuthConfig third_party_auth_config_;
-  bool gnubby_auth_policy_enabled_ = false;
-  bool gnubby_extension_supported_ = false;
+  bool security_key_auth_policy_enabled_ = false;
+  bool security_key_extension_supported_ = true;
 
   // Boolean to change flow, where necessary, if we're
   // capturing a window instead of the entire desktop.
@@ -401,6 +407,7 @@ class HostProcess : public ConfigWatcher::Delegate,
       host_change_notification_listener_;
   std::unique_ptr<HostStatusLogger> host_status_logger_;
   std::unique_ptr<HostEventLogger> host_event_logger_;
+  std::unique_ptr<HostPowerSaveBlocker> power_save_blocker_;
 
   std::unique_ptr<ChromotingHost> host_;
 
@@ -408,6 +415,8 @@ class HostProcess : public ConfigWatcher::Delegate,
   scoped_refptr<HostProcess> self_;
 
 #if defined(REMOTING_MULTI_PROCESS)
+  std::unique_ptr<mojo::edk::ScopedIPCSupport> ipc_support_;
+
   // Accessed on the UI thread.
   std::unique_ptr<IPC::ChannelProxy> daemon_channel_;
 
@@ -429,9 +438,14 @@ HostProcess::HostProcess(std::unique_ptr<ChromotingHostContext> context,
                          int* exit_code_out,
                          ShutdownWatchdog* shutdown_watchdog)
     : context_(std::move(context)),
+      desktop_environment_options_(DesktopEnvironmentOptions::CreateDefault()),
       self_(this),
       exit_code_out_(exit_code_out),
       shutdown_watchdog_(shutdown_watchdog) {
+  // TODO(zijiehe):
+  // desktop_environment_options_.desktop_capture_options()
+  //     ->set_use_update_notifications(true);
+  // And remove the same line from me2me_desktop_environment.cc.
   StartOnUiThread();
 }
 
@@ -452,35 +466,22 @@ HostProcess::~HostProcess() {
 
 bool HostProcess::InitWithCommandLine(const base::CommandLine* cmd_line) {
 #if defined(REMOTING_MULTI_PROCESS)
-  // Parse the handle value and convert it to a handle/file descriptor.
-  std::string channel_name =
-      cmd_line->GetSwitchValueASCII(kDaemonPipeSwitchName);
-
-  int pipe_handle = 0;
-  if (channel_name.empty() ||
-      !base::StringToInt(channel_name, &pipe_handle)) {
-    LOG(ERROR) << "Invalid '" << kDaemonPipeSwitchName
-               << "' value: " << channel_name;
-    return false;
-  }
-
-#if defined(OS_WIN)
-  base::win::ScopedHandle pipe(reinterpret_cast<HANDLE>(pipe_handle));
-  IPC::ChannelHandle channel_handle(pipe.Get());
-#elif defined(OS_POSIX)
-  base::FileDescriptor pipe(pipe_handle, true);
-  IPC::ChannelHandle channel_handle(channel_name, pipe);
-#endif  // defined(OS_POSIX)
+  // Mojo keeps the task runner passed to it alive forever, so an
+  // AutoThreadTaskRunner should not be passed to it. Otherwise, the process may
+  // never shut down cleanly.
+  ipc_support_ = base::MakeUnique<mojo::edk::ScopedIPCSupport>(
+      context_->network_task_runner()->task_runner(),
+      mojo::edk::ScopedIPCSupport::ShutdownPolicy::FAST);
+  mojo::edk::SetParentPipeHandle(
+      mojo::edk::PlatformChannelPair::PassClientHandleFromParentProcess(
+          *cmd_line));
 
   // Connect to the daemon process.
-  daemon_channel_.reset(
-      new IPC::ChannelProxy(this, context_->network_task_runner()));
-  IPC::AttachmentBrokerUnprivileged::CreateBrokerIfNeeded();
-  IPC::AttachmentBroker* broker = IPC::AttachmentBroker::GetGlobal();
-  if (broker && !broker->IsPrivilegedBroker())
-    broker->RegisterBrokerCommunicationChannel(daemon_channel_.get());
-  daemon_channel_->Init(channel_handle, IPC::Channel::MODE_CLIENT,
-                        /*create_pipe_now=*/true);
+  daemon_channel_ = IPC::ChannelProxy::Create(
+      mojo::edk::CreateChildMessagePipe(
+          cmd_line->GetSwitchValueASCII(kMojoPipeToken))
+          .release(),
+      IPC::Channel::MODE_CLIENT, this, context_->network_task_runner());
 
 #else  // !defined(REMOTING_MULTI_PROCESS)
   if (cmd_line->HasSwitch(kHostConfigSwitchName)) {
@@ -813,17 +814,19 @@ void HostProcess::StartOnUiThread() {
     remoting::AudioCapturerLinux::InitializePipeReader(
         context_->audio_task_runner(), audio_pipe_name);
   }
+#endif  // defined(OS_LINUX)
 
-  base::FilePath gnubby_socket_name = base::CommandLine::ForCurrentProcess()->
-      GetSwitchValuePath(kAuthSocknameSwitchName);
-  if (!gnubby_socket_name.empty()) {
-    remoting::GnubbyAuthHandler::SetGnubbySocketName(gnubby_socket_name);
-    gnubby_extension_supported_ = true;
+#if defined(OS_POSIX)
+  base::FilePath security_key_socket_name =
+      base::CommandLine::ForCurrentProcess()->GetSwitchValuePath(
+          kAuthSocknameSwitchName);
+  if (!security_key_socket_name.empty()) {
+    remoting::SecurityKeyAuthHandler::SetSecurityKeySocketName(
+        security_key_socket_name);
+  } else {
+    security_key_extension_supported_ = false;
   }
-#elif defined(OS_WIN)
-  // TODO(joedow): Remove the conditional once this is supported on OSX.
-  gnubby_extension_supported_ = true;
-#endif  // defined(OS_WIN)
+#endif  // defined(OS_POSIX)
 
   // Create a desktop environment factory appropriate to the build type &
   // platform.
@@ -845,8 +848,6 @@ void HostProcess::StartOnUiThread() {
         context_->input_task_runner(), context_->ui_task_runner());
   }
 #endif  // !defined(REMOTING_MULTI_PROCESS)
-  desktop_environment_factory->set_supports_touch_events(
-      InputInjector::SupportsTouchEvents());
 
   desktop_environment_factory_.reset(desktop_environment_factory);
 
@@ -863,10 +864,6 @@ void HostProcess::ShutdownOnUiThread() {
   policy_watcher_.reset();
 
 #if defined(REMOTING_MULTI_PROCESS)
-  IPC::AttachmentBroker* broker = IPC::AttachmentBroker::GetGlobal();
-  if (broker && !broker->IsPrivilegedBroker()) {
-    broker->DeregisterBrokerCommunicationChannel(daemon_channel_.get());
-  }
   daemon_channel_.reset();
 #endif  // defined(REMOTING_MULTI_PROCESS)
 
@@ -1143,7 +1140,7 @@ void HostProcess::ApplyUsernamePolicy() {
     // for each connection, removing the need for an explicit user-name matching
     // check.
 #if defined(OS_WIN) && defined(REMOTING_RDP_SESSION)
-    if (curtain_required_)
+    if (desktop_environment_options_.enable_curtaining())
       return;
 #endif  // defined(OS_WIN) && defined(REMOTING_RDP_SESSION)
 
@@ -1227,13 +1224,15 @@ bool HostProcess::OnCurtainPolicyUpdate(base::DictionaryValue* policies) {
   // Returns true if the host has to be restarted after this policy update.
   DCHECK(context_->network_task_runner()->BelongsToCurrentThread());
 
+  bool curtain_required;
   if (!policies->GetBoolean(policy::key::kRemoteAccessHostRequireCurtain,
-                            &curtain_required_)) {
+                            &curtain_required)) {
     return false;
   }
+  desktop_environment_options_.set_enable_curtaining(curtain_required);
 
 #if defined(OS_MACOSX)
-  if (curtain_required_) {
+  if (curtain_required) {
     // When curtain mode is in effect on Mac, the host process runs in the
     // user's switched-out session, but launchd will also run an instance at
     // the console login screen.  Even if no user is currently logged-on, we
@@ -1244,7 +1243,7 @@ bool HostProcess::OnCurtainPolicyUpdate(base::DictionaryValue* policies) {
     // TODO(jamiewalch): Fix this once we have implemented the multi-process
     // daemon architecture (crbug.com/134894)
     if (getuid() == 0) {
-      LOG(ERROR) << "Running the host in the console login session is yet not "
+      LOG(ERROR) << "Running the host in the console login session is not yet "
                     "supported.";
       ShutdownHost(kLoginScreenNotSupportedExitCode);
       return false;
@@ -1252,15 +1251,13 @@ bool HostProcess::OnCurtainPolicyUpdate(base::DictionaryValue* policies) {
   }
 #endif
 
-  if (curtain_required_) {
+  if (curtain_required) {
     HOST_LOG << "Policy requires curtain-mode.";
   } else {
     HOST_LOG << "Policy does not require curtain-mode.";
   }
 
-  if (host_)
-    host_->SetEnableCurtaining(curtain_required_);
-  return false;
+  return true;
 }
 
 bool HostProcess::OnHostTalkGadgetPrefixPolicyUpdate(
@@ -1315,14 +1312,14 @@ bool HostProcess::OnGnubbyAuthPolicyUpdate(base::DictionaryValue* policies) {
   DCHECK(context_->network_task_runner()->BelongsToCurrentThread());
 
   if (!policies->GetBoolean(policy::key::kRemoteAccessHostAllowGnubbyAuth,
-                            &gnubby_auth_policy_enabled_)) {
+                            &security_key_auth_policy_enabled_)) {
     return false;
   }
 
-  if (gnubby_auth_policy_enabled_) {
-    HOST_LOG << "Policy enables gnubby auth.";
+  if (security_key_auth_policy_enabled_) {
+    HOST_LOG << "Policy enables security key auth.";
   } else {
-    HOST_LOG << "Policy disables gnubby auth.";
+    HOST_LOG << "Policy disables security key auth.";
   }
 
   return true;
@@ -1432,9 +1429,9 @@ void HostProcess::StartHost() {
   scoped_refptr<protocol::TransportContext> transport_context =
       new protocol::TransportContext(
           signal_strategy_.get(),
-          base::WrapUnique(new protocol::ChromiumPortAllocatorFactory()),
-          base::WrapUnique(new ChromiumUrlRequestFactory(
-              context_->url_request_context_getter())),
+          base::MakeUnique<protocol::ChromiumPortAllocatorFactory>(),
+          base::MakeUnique<ChromiumUrlRequestFactory>(
+              context_->url_request_context_getter()),
           network_settings, protocol::TransportRole::SERVER);
   transport_context->set_ice_config_url(
       ServiceUrls::GetInstance()->ice_config_url());
@@ -1454,10 +1451,12 @@ void HostProcess::StartHost() {
   host_.reset(new ChromotingHost(desktop_environment_factory_.get(),
                                  std::move(session_manager), transport_context,
                                  context_->audio_task_runner(),
-                                 context_->video_encode_task_runner()));
+                                 context_->video_encode_task_runner(),
+                                 desktop_environment_options_));
 
-  if (gnubby_auth_policy_enabled_ && gnubby_extension_supported_) {
-    host_->AddExtension(base::WrapUnique(new GnubbyExtension()));
+  if (security_key_auth_policy_enabled_ && security_key_extension_supported_) {
+    host_->AddExtension(
+        base::MakeUnique<SecurityKeyExtension>(context_->file_task_runner()));
   }
 
   // TODO(simonmorris): Get the maximum session duration from a policy.
@@ -1472,6 +1471,11 @@ void HostProcess::StartHost() {
       host_->AsWeakPtr(), ServerLogEntry::ME2ME,
       signal_strategy_.get(), directory_bot_jid_));
 
+  power_save_blocker_.reset(new HostPowerSaveBlocker(
+      host_->AsWeakPtr(),
+      context_->ui_task_runner(),
+      context_->file_task_runner()));
+
   // Set up reporting the host status notifications.
 #if defined(REMOTING_MULTI_PROCESS)
   host_event_logger_.reset(
@@ -1481,7 +1485,6 @@ void HostProcess::StartHost() {
       HostEventLogger::Create(host_->AsWeakPtr(), kApplicationName);
 #endif  // !defined(REMOTING_MULTI_PROCESS)
 
-  host_->SetEnableCurtaining(curtain_required_);
   host_->Start(host_owner_email_);
 
   CreateAuthenticatorFactory();
@@ -1535,6 +1538,7 @@ void HostProcess::GoOffline(const std::string& host_offline_reason) {
   host_.reset();
   host_event_logger_.reset();
   host_status_logger_.reset();
+  power_save_blocker_.reset();
   host_change_notification_listener_.reset();
 
   // Before shutting down HostSignalingManager, send the |host_offline_reason|
@@ -1630,10 +1634,6 @@ int HostProcessMain() {
   base::GetLinuxDistro();
 #endif
 
-  // Enable support for SSL server sockets, which must be done while still
-  // single-threaded.
-  net::EnableSSLServerSockets();
-
   // Create the main message loop and start helper threads.
   base::MessageLoopForUI message_loop;
   std::unique_ptr<ChromotingHostContext> context =
@@ -1656,7 +1656,7 @@ int HostProcessMain() {
   new HostProcess(std::move(context), &exit_code, &shutdown_watchdog);
 
   // Run the main (also UI) message loop until the host no longer needs it.
-  message_loop.Run();
+  base::RunLoop().Run();
 
   return exit_code;
 }
