@@ -14,19 +14,24 @@
 #include "base/files/file_path.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
+#include "base/process/process_handle.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/win/scoped_handle.h"
-#include "ipc/attachment_broker.h"
 #include "ipc/ipc_channel.h"
 #include "ipc/ipc_channel_proxy.h"
 #include "ipc/ipc_listener.h"
 #include "ipc/ipc_message.h"
+#include "mojo/edk/embedder/embedder.h"
+#include "mojo/edk/embedder/named_platform_channel_pair.h"
+#include "mojo/edk/embedder/platform_channel_pair.h"
+#include "mojo/edk/embedder/platform_handle_utils.h"
+#include "mojo/edk/embedder/scoped_platform_handle.h"
 #include "remoting/host/host_main.h"
 #include "remoting/host/ipc_constants.h"
-#include "remoting/host/ipc_util.h"
 #include "remoting/host/switches.h"
 #include "remoting/host/win/launch_process_with_token.h"
 #include "remoting/host/win/security_descriptor.h"
@@ -52,8 +57,7 @@ class WtsSessionProcessDelegate::Core
   Core(scoped_refptr<base::SingleThreadTaskRunner> io_task_runner,
        std::unique_ptr<base::CommandLine> target,
        bool launch_elevated,
-       const std::string& channel_security,
-       const std::string& new_process_security);
+       const std::string& channel_security);
 
   // Initializes the object returning true on success.
   bool Initialize(uint32_t session_id);
@@ -101,8 +105,12 @@ class WtsSessionProcessDelegate::Core
   // Called when the number of processes running in the job reaches zero.
   void OnActiveProcessZero();
 
+  // Called when a process is launched in |job_|.
+  void OnProcessLaunchDetected(base::ProcessId pid);
+
   void ReportFatalError();
-  void ReportProcessLaunched(base::win::ScopedHandle worker_process);
+  void ReportProcessLaunched(base::win::ScopedHandle worker_process,
+                             mojo::edk::ScopedPlatformHandle server_handle);
 
   // The task runner all public methods of this class should be called on.
   scoped_refptr<base::SingleThreadTaskRunner> caller_task_runner_;
@@ -117,15 +125,8 @@ class WtsSessionProcessDelegate::Core
   // Security descriptor (as SDDL) to be applied to |channel_|.
   std::string channel_security_;
 
-  // Security descriptor (as SDDL) to be applied to the newly created process.
-  std::string new_process_security_;
-
   WorkerProcessLauncher* event_handler_;
 
-  // Pointer to GetNamedPipeClientProcessId() API if it is available.
-  typedef BOOL (WINAPI * GetNamedPipeClientProcessIdFn)(HANDLE, DWORD*);
-  GetNamedPipeClientProcessIdFn get_named_pipe_client_pid_;
-
   // The job object used to control the lifetime of child processes.
   base::win::ScopedHandle job_;
 
@@ -135,9 +136,6 @@ class WtsSessionProcessDelegate::Core
   // True if a laucnh attemp is pending.
   bool launch_pending_;
 
-  // The named pipe used as the transport by |channel_|.
-  base::win::ScopedHandle pipe_;
-
   // The token to be used to launch a process in a different session.
   base::win::ScopedHandle session_token_;
 
@@ -147,6 +145,19 @@ class WtsSessionProcessDelegate::Core
   // The handle of the worker process, if launched.
   base::win::ScopedHandle worker_process_;
 
+  // If launching elevated, this holds the server handle after launch, until
+  // the final process launches.
+  mojo::edk::ScopedPlatformHandle elevated_server_handle_;
+
+  // If launching elevated, this is the pid of the launcher process.
+  base::ProcessId elevated_launcher_pid_ = base::kNullProcessId;
+
+  // Tracks the id of the worker process.
+  base::ProcessId worker_process_pid_ = base::kNullProcessId;
+
+  // The mojo child token for the process being launched.
+  std::string mojo_child_token_;
+
   DISALLOW_COPY_AND_ASSIGN(Core);
 };
 
@@ -154,14 +165,11 @@ WtsSessionProcessDelegate::Core::Core(
     scoped_refptr<base::SingleThreadTaskRunner> io_task_runner,
     std::unique_ptr<base::CommandLine> target_command,
     bool launch_elevated,
-    const std::string& channel_security,
-    const std::string& new_process_security)
+    const std::string& channel_security)
     : caller_task_runner_(base::ThreadTaskRunnerHandle::Get()),
       io_task_runner_(io_task_runner),
       channel_security_(channel_security),
-      new_process_security_(new_process_security),
       event_handler_(nullptr),
-      get_named_pipe_client_pid_(nullptr),
       launch_elevated_(launch_elevated),
       launch_pending_(false),
       target_command_(std::move(target_command)) {}
@@ -174,11 +182,6 @@ bool WtsSessionProcessDelegate::Core::Initialize(uint32_t session_id) {
     HMODULE kernel32 = ::GetModuleHandle(L"kernel32.dll");
     CHECK(kernel32 != nullptr);
 
-    get_named_pipe_client_pid_ =
-        reinterpret_cast<GetNamedPipeClientProcessIdFn>(
-            GetProcAddress(kernel32, "GetNamedPipeClientProcessId"));
-    CHECK(get_named_pipe_client_pid_ != nullptr);
-
     ScopedHandle job;
     job.Set(CreateJobObject(nullptr, nullptr));
     if (!job.IsValid()) {
@@ -247,13 +250,17 @@ void WtsSessionProcessDelegate::Core::Send(IPC::Message* message) {
 void WtsSessionProcessDelegate::Core::CloseChannel() {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 
-  if (!channel_)
+  if (!channel_) {
     return;
+  }
 
-  IPC::AttachmentBroker::GetGlobal()->DeregisterCommunicationChannel(
-      channel_.get());
   channel_.reset();
-  pipe_.Close();
+  elevated_server_handle_.reset();
+  elevated_launcher_pid_ = base::kNullProcessId;
+  if (!mojo_child_token_.empty()) {
+    mojo::edk::ChildProcessLaunchFailed(mojo_child_token_);
+    mojo_child_token_.clear();
+  }
 }
 
 void WtsSessionProcessDelegate::Core::KillProcess() {
@@ -265,11 +272,13 @@ void WtsSessionProcessDelegate::Core::KillProcess() {
   launch_pending_ = false;
 
   if (launch_elevated_) {
-    if (job_.IsValid())
+    if (job_.IsValid()) {
       TerminateJobObject(job_.Get(), CONTROL_C_EXIT);
+    }
   } else {
-    if (worker_process_.IsValid())
+    if (worker_process_.IsValid()) {
       TerminateProcess(worker_process_.Get(), CONTROL_C_EXIT);
+    }
   }
 
   worker_process_.Close();
@@ -278,7 +287,6 @@ void WtsSessionProcessDelegate::Core::KillProcess() {
 WtsSessionProcessDelegate::Core::~Core() {
   DCHECK(!channel_);
   DCHECK(!event_handler_);
-  DCHECK(!pipe_.IsValid());
   DCHECK(!worker_process_.IsValid());
 }
 
@@ -289,10 +297,50 @@ void WtsSessionProcessDelegate::Core::OnIOCompleted(
   DCHECK(io_task_runner_->BelongsToCurrentThread());
 
   // |bytes_transferred| is used in job object notifications to supply
-  // the message ID; |context| carries process ID.
-  if (bytes_transferred == JOB_OBJECT_MSG_ACTIVE_PROCESS_ZERO) {
-    caller_task_runner_->PostTask(FROM_HERE,
-                                  base::Bind(&Core::OnActiveProcessZero, this));
+  // the message ID; |context| carries process ID for the events we listen for.
+  base::ProcessId process_id =
+      static_cast<base::ProcessId>(reinterpret_cast<uintptr_t>(context));
+  switch (bytes_transferred) {
+    case JOB_OBJECT_MSG_ACTIVE_PROCESS_ZERO: {
+      caller_task_runner_->PostTask(
+          FROM_HERE, base::Bind(&Core::OnActiveProcessZero, this));
+      break;
+    }
+    case JOB_OBJECT_MSG_NEW_PROCESS: {
+      if (elevated_launcher_pid_ == base::kNullProcessId) {
+        // Ignore process launch events when we don't have a valid launcher pid.
+        return;
+      }
+
+      if (process_id != elevated_launcher_pid_) {
+        DCHECK_EQ(worker_process_pid_, base::kNullProcessId);
+        worker_process_pid_ = process_id;
+      }
+      break;
+    }
+    case JOB_OBJECT_MSG_EXIT_PROCESS: {
+      if (process_id == worker_process_pid_) {
+        // In official builds the first launch of a UiAccess enabled binary
+        // will fail due to 'STATUS_ELEVATION_REQUIRED'.  This is an artifact of
+        // using ShellExecuteEx() to launch the process.  In this scenario, we
+        // will clear out the previously stored value for |worker_process_pid_|
+        // and retry after the subsequent relaunch of the worker process.
+        worker_process_pid_ = base::kNullProcessId;
+      } else if (process_id == elevated_launcher_pid_) {
+        if (worker_process_pid_ == base::kNullProcessId) {
+          // The elevated launcher process can fail to launch without attemping
+          // to launch the worker.  In this scenario, the failure will be
+          // detected outside this method and the elevated launcher will be
+          // launched again.
+          return;
+        }
+
+        caller_task_runner_->PostTask(
+            FROM_HERE, base::Bind(&Core::OnProcessLaunchDetected, this,
+                                  worker_process_pid_));
+      }
+      break;
+    }
   }
 }
 
@@ -306,37 +354,6 @@ bool WtsSessionProcessDelegate::Core::OnMessageReceived(
 void WtsSessionProcessDelegate::Core::OnChannelConnected(int32_t peer_pid) {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 
-  // Report the worker PID now if the worker process is launched indirectly.
-  // Note that in this case the pipe's security descriptor is the only
-  // protection against a malicious processed connecting to the pipe.
-  if (launch_elevated_) {
-    DWORD pid;
-    if (!get_named_pipe_client_pid_(pipe_.Get(), &pid)) {
-      PLOG(ERROR) << "Failed to retrive PID of the client";
-      ReportFatalError();
-      return;
-    }
-
-    if (pid != static_cast<DWORD>(peer_pid)) {
-      LOG(ERROR) << "The actual client PID " << pid
-                 << " does not match the one reported by the client: "
-                 << peer_pid;
-      ReportFatalError();
-      return;
-    }
-
-    DWORD desired_access =
-        SYNCHRONIZE | PROCESS_DUP_HANDLE | PROCESS_QUERY_INFORMATION;
-    ScopedHandle worker_process(OpenProcess(desired_access, false, pid));
-    if (!worker_process.IsValid()) {
-      PLOG(ERROR) << "Failed to open process " << pid;
-      ReportFatalError();
-      return;
-    }
-
-    ReportProcessLaunched(std::move(worker_process));
-  }
-
   if (event_handler_)
     event_handler_->OnChannelConnected(peer_pid);
 }
@@ -350,7 +367,6 @@ void WtsSessionProcessDelegate::Core::OnChannelError() {
 void WtsSessionProcessDelegate::Core::DoLaunchProcess() {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
   DCHECK(!channel_);
-  DCHECK(!pipe_.IsValid());
   DCHECK(!worker_process_.IsValid());
 
   base::CommandLine command_line(target_command_->argv());
@@ -375,55 +391,42 @@ void WtsSessionProcessDelegate::Core::DoLaunchProcess() {
                                   target_command_->GetProgram());
   }
 
-  // Create the server end of the IPC channel.
-  std::string channel_name = IPC::Channel::GenerateUniqueRandomChannelID();
-  ScopedHandle pipe;
-  if (!CreateIpcChannel(channel_name, channel_security_, &pipe)) {
-    ReportFatalError();
-    return;
-  }
+  const std::string mojo_message_pipe_token = mojo::edk::GenerateRandomToken();
+  mojo_child_token_ = mojo::edk::GenerateRandomToken();
+  std::unique_ptr<IPC::ChannelProxy> channel = IPC::ChannelProxy::Create(
+      mojo::edk::CreateParentMessagePipe(mojo_message_pipe_token,
+                                         mojo_child_token_)
+          .release(),
+      IPC::Channel::MODE_SERVER, this, io_task_runner_);
+  command_line.AppendSwitchASCII(kMojoPipeToken, mojo_message_pipe_token);
 
-  // Wrap the pipe into an IPC channel.
-  std::unique_ptr<IPC::ChannelProxy> channel(
-      new IPC::ChannelProxy(this, io_task_runner_));
-  IPC::AttachmentBroker::GetGlobal()->RegisterCommunicationChannel(
-      channel.get(), io_task_runner_);
-  channel->Init(IPC::ChannelHandle(pipe.Get()), IPC::Channel::MODE_SERVER,
-                /*create_pipe_now=*/true);
-
-  // Pass the name of the IPC channel to use.
-  command_line.AppendSwitchNative(kDaemonPipeSwitchName,
-                                  base::UTF8ToWide(channel_name));
-
-  ScopedSd security_descriptor;
-  std::unique_ptr<SECURITY_ATTRIBUTES> security_attributes;
-  if (!new_process_security_.empty()) {
-    security_descriptor = ConvertSddlToSd(new_process_security_);
-    if (!security_descriptor) {
-      PLOG(ERROR) << "ConvertSddlToSd() failed.";
-      ReportFatalError();
-      return;
-    }
-
-    security_attributes.reset(new SECURITY_ATTRIBUTES());
-    security_attributes->nLength = sizeof(SECURITY_ATTRIBUTES);
-    security_attributes->lpSecurityDescriptor = security_descriptor.get();
-    security_attributes->bInheritHandle = FALSE;
+  std::unique_ptr<mojo::edk::PlatformChannelPair> normal_mojo_channel;
+  std::unique_ptr<mojo::edk::NamedPlatformChannelPair> elevated_mojo_channel;
+  base::HandlesToInheritVector handles_to_inherit;
+  if (launch_elevated_) {
+    // Pass the name of the IPC channel to use.
+    mojo::edk::NamedPlatformChannelPair::Options options;
+    options.security_descriptor = base::UTF8ToUTF16(channel_security_);
+    elevated_mojo_channel =
+        base::MakeUnique<mojo::edk::NamedPlatformChannelPair>(options);
+    elevated_mojo_channel->PrepareToPassClientHandleToChildProcess(
+        &command_line);
+  } else {
+    normal_mojo_channel = base::MakeUnique<mojo::edk::PlatformChannelPair>();
+    normal_mojo_channel->PrepareToPassClientHandleToChildProcess(
+        &command_line, &handles_to_inherit);
   }
 
   // Try to launch the process.
   ScopedHandle worker_process;
   ScopedHandle worker_thread;
-  if (!LaunchProcessWithToken(command_line.GetProgram(),
-                              command_line.GetCommandLineString(),
-                              session_token_.Get(),
-                              security_attributes.get(),
-                              nullptr,
-                              false,
-                              CREATE_SUSPENDED | CREATE_BREAKAWAY_FROM_JOB,
-                              base::UTF8ToUTF16(kDefaultDesktopName).c_str(),
-                              &worker_process,
-                              &worker_thread)) {
+  if (!LaunchProcessWithToken(
+          command_line.GetProgram(), command_line.GetCommandLineString(),
+          session_token_.Get(), /*security_attributes=*/nullptr,
+          /* thread_attributes= */ nullptr, handles_to_inherit,
+          /* creation_flags= */ CREATE_SUSPENDED | CREATE_BREAKAWAY_FROM_JOB,
+          base::UTF8ToUTF16(kDefaultDesktopName).c_str(), &worker_process,
+          &worker_thread)) {
     ReportFatalError();
     return;
   }
@@ -443,13 +446,19 @@ void WtsSessionProcessDelegate::Core::DoLaunchProcess() {
   }
 
   channel_ = std::move(channel);
-  pipe_ = std::move(pipe);
 
-  // Report success if the worker process is lauched directly. Otherwise, PID of
-  // the client connected to the pipe will be used later. See
-  // OnChannelConnected().
-  if (!launch_elevated_)
-    ReportProcessLaunched(std::move(worker_process));
+  if (launch_elevated_) {
+    // When launching an elevated worker process, an intermediate launcher
+    // process launches the worker process. Reporting the launch waits until the
+    // worker process launch is detected. Until then, store the values needed in
+    // fields. See OnProcessLaunchDetected for their use.
+    elevated_server_handle_ = elevated_mojo_channel->PassServerHandle();
+    elevated_launcher_pid_ = GetProcessId(worker_process.Get());
+    DCHECK(elevated_server_handle_.is_valid());
+  } else {
+    ReportProcessLaunched(std::move(worker_process),
+                          normal_mojo_channel->PassServerHandle());
+  }
 }
 
 void WtsSessionProcessDelegate::Core::DrainJobNotifications() {
@@ -495,8 +504,9 @@ void WtsSessionProcessDelegate::Core::InitializeJobCompleted(ScopedHandle job) {
 
   job_ = std::move(job);
 
-  if (launch_pending_)
+  if (launch_pending_) {
     DoLaunchProcess();
+  }
 }
 
 void WtsSessionProcessDelegate::Core::OnActiveProcessZero() {
@@ -509,6 +519,29 @@ void WtsSessionProcessDelegate::Core::OnActiveProcessZero() {
   }
 }
 
+void WtsSessionProcessDelegate::Core::OnProcessLaunchDetected(
+    base::ProcessId pid) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+  DCHECK_NE(pid, elevated_launcher_pid_);
+
+  if (!elevated_server_handle_.is_valid()) {
+    return;
+  }
+
+  DWORD desired_access =
+      SYNCHRONIZE | PROCESS_DUP_HANDLE | PROCESS_QUERY_INFORMATION;
+  base::win::ScopedHandle worker_process(
+      OpenProcess(desired_access, false, pid));
+  if (!worker_process.IsValid()) {
+    PLOG(ERROR) << "Failed to open process " << pid;
+    ReportFatalError();
+    return;
+  }
+  elevated_launcher_pid_ = base::kNullProcessId;
+  ReportProcessLaunched(std::move(worker_process),
+                        std::move(elevated_server_handle_));
+}
+
 void WtsSessionProcessDelegate::Core::ReportFatalError() {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 
@@ -520,10 +553,15 @@ void WtsSessionProcessDelegate::Core::ReportFatalError() {
 }
 
 void WtsSessionProcessDelegate::Core::ReportProcessLaunched(
-    base::win::ScopedHandle worker_process) {
+    base::win::ScopedHandle worker_process,
+    mojo::edk::ScopedPlatformHandle server_handle) {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
   DCHECK(!worker_process_.IsValid());
 
+  mojo::edk::ChildProcessLaunched(worker_process.Get(),
+                                  std::move(server_handle),
+                                  mojo_child_token_);
+  mojo_child_token_.clear();
   worker_process_ = std::move(worker_process);
 
   // Report a handle that can be used to wait for the worker process completion,
@@ -547,10 +585,9 @@ WtsSessionProcessDelegate::WtsSessionProcessDelegate(
     scoped_refptr<base::SingleThreadTaskRunner> io_task_runner,
     std::unique_ptr<base::CommandLine> target_command,
     bool launch_elevated,
-    const std::string& channel_security,
-    const std::string& new_process_security_descriptor) {
+    const std::string& channel_security) {
   core_ = new Core(io_task_runner, std::move(target_command), launch_elevated,
-                   channel_security, new_process_security_descriptor);
+                   channel_security);
 }
 
 WtsSessionProcessDelegate::~WtsSessionProcessDelegate() {
