@@ -14,7 +14,7 @@
 namespace remoting {
 namespace protocol {
 
-ChannelDispatcherBase::ChannelDispatcherBase(const char* channel_name)
+ChannelDispatcherBase::ChannelDispatcherBase(const std::string& channel_name)
     : channel_name_(channel_name) {}
 
 ChannelDispatcherBase::~ChannelDispatcherBase() {
@@ -31,15 +31,35 @@ void ChannelDispatcherBase::Init(MessageChannelFactory* channel_factory,
       &ChannelDispatcherBase::OnChannelReady, base::Unretained(this)));
 }
 
+void ChannelDispatcherBase::Init(std::unique_ptr<MessagePipe> message_pipe,
+                                 EventHandler* event_handler) {
+  event_handler_ = event_handler;
+  OnChannelReady(std::move(message_pipe));
+}
+
 void ChannelDispatcherBase::OnChannelReady(
     std::unique_ptr<MessagePipe> message_pipe) {
   channel_factory_ = nullptr;
   message_pipe_ = std::move(message_pipe);
-  message_pipe_->StartReceiving(base::Bind(
-      &ChannelDispatcherBase::OnIncomingMessage, base::Unretained(this)));
+  message_pipe_->Start(this);
+}
 
+void ChannelDispatcherBase::OnMessagePipeOpen() {
+  DCHECK(!is_connected_);
+  is_connected_ = true;
   event_handler_->OnChannelInitialized(this);
 }
 
+void ChannelDispatcherBase::OnMessageReceived(
+    std::unique_ptr<CompoundBuffer> message) {
+  OnIncomingMessage(std::move(message));
+}
+
+void ChannelDispatcherBase::OnMessagePipeClosed() {
+  is_connected_ = false;
+  message_pipe_.reset();
+  event_handler_->OnChannelClosed(this);
+}
+
 }  // namespace protocol
 }  // namespace remoting
