@@ -10,20 +10,24 @@
 namespace remoting {
 
 class VideoPacket;
+class FrameStatsMessage;
 
 namespace protocol {
 
-// Struct used to track timestamp for various events in the video pipeline for a
-// single video frame
-struct FrameStats {
-  FrameStats();
-  FrameStats(const FrameStats&);
-  ~FrameStats();
+struct HostFrameStats {
+  HostFrameStats();
+  HostFrameStats(const HostFrameStats&);
+  ~HostFrameStats();
 
-  // Copies timing fields from the |packet|.
-  static FrameStats GetForVideoPacket(const VideoPacket& packet);
+  // Extracts timing fields from the |packet|.
+  static HostFrameStats GetForVideoPacket(const VideoPacket& packet);
 
-  int frame_size = 0;
+  // Converts FrameStatsMessage protobuf message to HostFrameStats.
+  static HostFrameStats FromFrameStatsMessage(const FrameStatsMessage& message);
+  void ToFrameStatsMessage(FrameStatsMessage* message_out) const;
+
+  // Frame Size.
+  int frame_size {};
 
   // Set to null for frames that were not sent after a fresh input event.
   base::TimeTicks latest_event_timestamp;
@@ -35,12 +39,37 @@ struct FrameStats {
   base::TimeDelta capture_overhead_delay = base::TimeDelta::Max();
   base::TimeDelta encode_pending_delay = base::TimeDelta::Max();
   base::TimeDelta send_pending_delay = base::TimeDelta::Max();
+  base::TimeDelta rtt_estimate = base::TimeDelta::Max();
+  int bandwidth_estimate_kbps = -1;
+};
+
+struct ClientFrameStats {
+  ClientFrameStats();
+  ClientFrameStats(const ClientFrameStats&);
+  ~ClientFrameStats();
+  ClientFrameStats& operator=(const ClientFrameStats&);
 
   base::TimeTicks time_received;
   base::TimeTicks time_decoded;
   base::TimeTicks time_rendered;
 };
 
+struct FrameStats {
+  FrameStats();
+  FrameStats(const FrameStats&);
+  ~FrameStats();
+
+  HostFrameStats host_stats;
+  ClientFrameStats client_stats;
+};
+
+class FrameStatsConsumer {
+ public:
+  virtual void OnVideoFrameStats(const FrameStats& stats) = 0;
+ protected:
+  virtual ~FrameStatsConsumer() {}
+};
+
 }  // namespace protocol
 }  // namespace remoting
 
