@@ -18,16 +18,18 @@
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/task_runner_util.h"
+#include "base/threading/thread_restrictions.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "jingle/glue/thread_wrapper.h"
 #include "remoting/protocol/authenticator.h"
 #include "remoting/protocol/port_allocator_factory.h"
+#include "remoting/protocol/sdp_message.h"
 #include "remoting/protocol/stream_message_pipe_adapter.h"
 #include "remoting/protocol/transport_context.h"
-#include "remoting/protocol/webrtc_video_encoder_factory.h"
+#include "remoting/protocol/webrtc_audio_module.h"
+#include "remoting/protocol/webrtc_dummy_video_encoder.h"
+#include "third_party/libjingle_xmpp/xmllite/xmlelement.h"
 #include "third_party/webrtc/api/test/fakeconstraints.h"
-#include "third_party/webrtc/libjingle/xmllite/xmlelement.h"
-#include "third_party/webrtc/modules/audio_device/include/fake_audio_device.h"
 
 using buzz::QName;
 using buzz::XmlElement;
@@ -56,15 +58,38 @@ bool IsValidSessionDescriptionType(const std::string& type) {
          type == webrtc::SessionDescriptionInterface::kAnswer;
 }
 
-// Normalizes the SDP message to make sure the text used for HMAC signatures
-// verifications is the same that was signed on the sending side. This is
-// necessary because WebRTC generates SDP with CRLF line endings which are
-// sometimes converted to LF after passing the signaling channel.
-std::string NormalizeSessionDescription(const std::string& sdp) {
-  // Split SDP lines. The CR symbols is removed by the TRIM_WHITESPACE flag.
-  std::vector<std::string> lines =
-      SplitString(sdp, "\n", base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
-  return base::JoinString(lines, "\n") + "\n";
+void UpdateCodecParameters(SdpMessage* sdp_message, bool incoming) {
+  // Set bitrate range to 1-100 Mbps.
+  //  - Setting min bitrate here enables padding.
+  //  - The default max bitrate is 600 kbps. Setting it to 100 Mbps allows to
+  //    use higher bandwidth when it's available.
+  //
+  // TODO(sergeyu): Padding needs to be enabled to workaround BW estimator not
+  // handling spiky traffic patterns well. This won't be necessary with a
+  // better bandwidth estimator.
+  // TODO(isheriff): The need for this should go away once we have a proper
+  // API to provide max bitrate for the case of handing over encoded
+  // frames to webrtc.
+  if (sdp_message->has_video() &&
+      !sdp_message->AddCodecParameter(
+          "VP8", "x-google-min-bitrate=1000; x-google-max-bitrate=100000")) {
+    if (incoming) {
+      LOG(WARNING) << "VP8 not found in an incoming SDP.";
+    } else {
+      LOG(FATAL) << "VP8 not found in SDP generated by WebRTC.";
+    }
+  }
+
+  // Update SDP format to use stereo for opus codec.
+  if (sdp_message->has_audio() &&
+      !sdp_message->AddCodecParameter("opus",
+                                      "stereo=1; x-google-min-bitrate=160")) {
+    if (incoming) {
+      LOG(WARNING) << "Opus not found in an incoming SDP.";
+    } else {
+      LOG(FATAL) << "Opus not found in SDP generated by WebRTC.";
+    }
+  }
 }
 
 // A webrtc::CreateSessionDescriptionObserver implementation used to receive the
@@ -135,28 +160,162 @@ class SetSessionDescriptionObserver
   DISALLOW_COPY_AND_ASSIGN(SetSessionDescriptionObserver);
 };
 
-
 }  // namespace
 
+class WebrtcTransport::PeerConnectionWrapper
+    : public webrtc::PeerConnectionObserver {
+ public:
+  PeerConnectionWrapper(
+      rtc::Thread* worker_thread,
+      std::unique_ptr<cricket::WebRtcVideoEncoderFactory> encoder_factory,
+      std::unique_ptr<cricket::PortAllocator> port_allocator,
+      base::WeakPtr<WebrtcTransport> transport)
+      : transport_(transport) {
+    audio_module_ = new rtc::RefCountedObject<WebrtcAudioModule>();
+
+    peer_connection_factory_ = webrtc::CreatePeerConnectionFactory(
+        worker_thread, rtc::Thread::Current(), audio_module_.get(),
+        encoder_factory.release(), nullptr);
+
+    webrtc::FakeConstraints constraints;
+    constraints.AddMandatory(webrtc::MediaConstraintsInterface::kEnableDtlsSrtp,
+                             webrtc::MediaConstraintsInterface::kValueTrue);
+
+    webrtc::PeerConnectionInterface::RTCConfiguration rtc_config;
+
+    // Set bundle_policy and rtcp_mux_policy to ensure that all channels are
+    // multiplexed over a single channel.
+    rtc_config.bundle_policy =
+        webrtc::PeerConnectionInterface::kBundlePolicyMaxBundle;
+    rtc_config.rtcp_mux_policy =
+        webrtc::PeerConnectionInterface::kRtcpMuxPolicyRequire;
+
+    rtc_config.media_config.video.periodic_alr_bandwidth_probing = true;
+
+    peer_connection_ = peer_connection_factory_->CreatePeerConnection(
+        rtc_config, &constraints, std::move(port_allocator), nullptr, this);
+  }
+  virtual ~PeerConnectionWrapper() {
+    // PeerConnection creates threads internally, which are stopped when the
+    // connection is closed. Thread.Stop() is a blocking operation.
+    // See crbug.com/660081.
+    base::ThreadRestrictions::ScopedAllowIO allow_io;
+    peer_connection_->Close();
+  }
+
+  WebrtcAudioModule* audio_module() {
+    return audio_module_.get();
+  }
+
+  webrtc::PeerConnectionInterface* peer_connection() {
+    return peer_connection_.get();
+  }
+
+  webrtc::PeerConnectionFactoryInterface* peer_connection_factory() {
+    return peer_connection_factory_.get();
+  }
+
+  // webrtc::PeerConnectionObserver interface.
+  void OnSignalingChange(
+      webrtc::PeerConnectionInterface::SignalingState new_state) override {
+    if (transport_)
+      transport_->OnSignalingChange(new_state);
+  }
+  void OnAddStream(
+      rtc::scoped_refptr<webrtc::MediaStreamInterface> stream) override {
+    if (transport_)
+      transport_->OnAddStream(stream);
+  }
+  void OnRemoveStream(
+      rtc::scoped_refptr<webrtc::MediaStreamInterface> stream) override {
+    if (transport_)
+      transport_->OnRemoveStream(stream);
+  }
+  void OnDataChannel(
+      rtc::scoped_refptr<webrtc::DataChannelInterface> data_channel) override {
+    if (transport_)
+      transport_->OnDataChannel(data_channel);
+  }
+  void OnRenegotiationNeeded() override {
+    if (transport_)
+      transport_->OnRenegotiationNeeded();
+  }
+  void OnIceConnectionChange(
+      webrtc::PeerConnectionInterface::IceConnectionState new_state) override {
+    if (transport_)
+      transport_->OnIceConnectionChange(new_state);
+  }
+  void OnIceGatheringChange(
+      webrtc::PeerConnectionInterface::IceGatheringState new_state) override {
+    if (transport_)
+      transport_->OnIceGatheringChange(new_state);
+  }
+  void OnIceCandidate(const webrtc::IceCandidateInterface* candidate) override {
+    if (transport_)
+      transport_->OnIceCandidate(candidate);
+  }
+
+ private:
+  scoped_refptr<WebrtcAudioModule> audio_module_;
+  scoped_refptr<webrtc::PeerConnectionFactoryInterface>
+      peer_connection_factory_;
+  scoped_refptr<webrtc::PeerConnectionInterface> peer_connection_;
+
+  base::WeakPtr<WebrtcTransport> transport_;
+
+  DISALLOW_COPY_AND_ASSIGN(PeerConnectionWrapper);
+};
+
 WebrtcTransport::WebrtcTransport(
     rtc::Thread* worker_thread,
     scoped_refptr<TransportContext> transport_context,
     EventHandler* event_handler)
-    : worker_thread_(worker_thread),
-      transport_context_(transport_context),
+    : transport_context_(transport_context),
       event_handler_(event_handler),
       handshake_hmac_(crypto::HMAC::SHA256),
-      outgoing_data_stream_adapter_(
-          true,
-          base::Bind(&WebrtcTransport::Close, base::Unretained(this))),
-      incoming_data_stream_adapter_(
-          false,
-          base::Bind(&WebrtcTransport::Close, base::Unretained(this))),
       weak_factory_(this) {
   transport_context_->set_relay_mode(TransportContext::RelayMode::TURN);
+
+  video_encoder_factory_ = new WebrtcDummyVideoEncoderFactory();
+  std::unique_ptr<cricket::PortAllocator> port_allocator =
+      transport_context_->port_allocator_factory()->CreatePortAllocator(
+          transport_context_);
+
+  // Takes ownership of video_encoder_factory_.
+  peer_connection_wrapper_.reset(new PeerConnectionWrapper(
+      worker_thread, base::WrapUnique(video_encoder_factory_),
+      std::move(port_allocator), weak_factory_.GetWeakPtr()));
 }
 
-WebrtcTransport::~WebrtcTransport() {}
+WebrtcTransport::~WebrtcTransport() {
+  Close(OK);
+}
+
+webrtc::PeerConnectionInterface* WebrtcTransport::peer_connection() {
+  return peer_connection_wrapper_ ? peer_connection_wrapper_->peer_connection()
+                                  : nullptr;
+}
+
+webrtc::PeerConnectionFactoryInterface*
+WebrtcTransport::peer_connection_factory() {
+  return peer_connection_wrapper_
+             ? peer_connection_wrapper_->peer_connection_factory()
+             : nullptr;
+}
+
+WebrtcAudioModule* WebrtcTransport::audio_module() {
+  return peer_connection_wrapper_
+             ? peer_connection_wrapper_->audio_module()
+             : nullptr;
+}
+
+std::unique_ptr<MessagePipe> WebrtcTransport::CreateOutgoingChannel(
+    const std::string& name) {
+  webrtc::DataChannelInit config;
+  config.reliable = true;
+  return base::MakeUnique<WebrtcDataStreamAdapter>(
+      peer_connection()->CreateDataChannel(name, &config));
+}
 
 void WebrtcTransport::Start(
     Authenticator* authenticator,
@@ -175,32 +334,6 @@ void WebrtcTransport::Start(
     LOG(FATAL) << "HMAC::Init() failed.";
   }
 
-  fake_audio_device_module_.reset(new webrtc::FakeAudioDeviceModule());
-  video_encoder_factory_ = new remoting::WebrtcVideoEncoderFactory();
-
-  // Takes ownership of video_encoder_factory_
-  peer_connection_factory_ = webrtc::CreatePeerConnectionFactory(
-      worker_thread_, rtc::Thread::Current(), fake_audio_device_module_.get(),
-      video_encoder_factory_, nullptr);
-
-  webrtc::PeerConnectionInterface::IceServer stun_server;
-  stun_server.urls.push_back("stun:stun.l.google.com:19302");
-  webrtc::PeerConnectionInterface::RTCConfiguration rtc_config;
-  rtc_config.servers.push_back(stun_server);
-
-  webrtc::FakeConstraints constraints;
-  constraints.AddMandatory(webrtc::MediaConstraintsInterface::kEnableDtlsSrtp,
-                           webrtc::MediaConstraintsInterface::kValueTrue);
-
-  std::unique_ptr<cricket::PortAllocator> port_allocator =
-      transport_context_->port_allocator_factory()->CreatePortAllocator(
-          transport_context_);
-  peer_connection_ = peer_connection_factory_->CreatePeerConnection(
-      rtc_config, &constraints, std::move(port_allocator), nullptr, this);
-
-  outgoing_data_stream_adapter_.Initialize(peer_connection_);
-  incoming_data_stream_adapter_.Initialize(peer_connection_);
-
   event_handler_->OnWebrtcTransportConnecting();
 
   if (transport_context_->role() == TransportRole::SERVER)
@@ -213,7 +346,7 @@ bool WebrtcTransport::ProcessTransportInfo(XmlElement* transport_info) {
   if (transport_info->Name() != QName(kTransportNamespace, "transport"))
     return false;
 
-  if (!peer_connection_)
+  if (!peer_connection())
     return false;
 
   XmlElement* session_description = transport_info->FirstNamed(
@@ -223,24 +356,26 @@ bool WebrtcTransport::ProcessTransportInfo(XmlElement* transport_info) {
         transport_context_->role() == TransportRole::CLIENT
             ? webrtc::PeerConnectionInterface::kStable
             : webrtc::PeerConnectionInterface::kHaveLocalOffer;
-    if (peer_connection_->signaling_state() != expected_state) {
+    if (peer_connection()->signaling_state() != expected_state) {
       LOG(ERROR) << "Received unexpected WebRTC session_description.";
       return false;
     }
 
     std::string type = session_description->Attr(QName(std::string(), "type"));
-    std::string sdp =
-        NormalizeSessionDescription(session_description->BodyText());
-    if (!IsValidSessionDescriptionType(type) || sdp.empty()) {
+    std::string raw_sdp = session_description->BodyText();
+    if (!IsValidSessionDescriptionType(type) || raw_sdp.empty()) {
       LOG(ERROR) << "Incorrect session description format.";
       return false;
     }
 
+    SdpMessage sdp_message(raw_sdp);
+
     std::string signature_base64 =
         session_description->Attr(QName(std::string(), "signature"));
     std::string signature;
     if (!base::Base64Decode(signature_base64, &signature) ||
-        !handshake_hmac_.Verify(type + " " + sdp, signature)) {
+        !handshake_hmac_.Verify(type + " " + sdp_message.ToString(),
+                                signature)) {
       LOG(WARNING) << "Received session-description with invalid signature.";
       bool ignore_error = false;
 #if !defined(NDEBUG)
@@ -253,34 +388,18 @@ bool WebrtcTransport::ProcessTransportInfo(XmlElement* transport_info) {
       }
     }
 
-    // Set bitrate range to 1-20 Mbps.
-    //  - Setting min bitrate here enables padding.
-    //  - The default max bitrate is 600 kbps. Setting it to 20 Mbps allows to
-    //    use higher bandwidth when it's available.
-    //
-    // TODO(sergeyu): Padding needs to be enabled to workaround BW estimator not
-    // handling spiky traffic patterns well. This won't be necessary with a
-    // better bandwidth estimator.
-    // TODO(isheriff): The need for this should go away once we have a proper
-    // API to provide max bitrate for the case of handing over encoded
-    // frames to webrtc.
-    std::string vp8line = "a=rtpmap:100 VP8/90000\n";
-    std::string vp8line_with_bitrate =
-        vp8line +
-        "a=fmtp:100 x-google-min-bitrate=1000\n"
-        "a=fmtp:100 x-google-max-bitrate=20000\n";
-    base::ReplaceSubstringsAfterOffset(&sdp, 0, vp8line, vp8line_with_bitrate);
+    UpdateCodecParameters(&sdp_message, /*incoming=*/true);
 
     webrtc::SdpParseError error;
     std::unique_ptr<webrtc::SessionDescriptionInterface> session_description(
-        webrtc::CreateSessionDescription(type, sdp, &error));
+        webrtc::CreateSessionDescription(type, sdp_message.ToString(), &error));
     if (!session_description) {
       LOG(ERROR) << "Failed to parse the session description: "
                  << error.description << " line: " << error.line;
       return false;
     }
 
-    peer_connection_->SetRemoteDescription(
+    peer_connection()->SetRemoteDescription(
         SetSessionDescriptionObserver::Create(
             base::Bind(&WebrtcTransport::OnRemoteDescriptionSet,
                        weak_factory_.GetWeakPtr(),
@@ -315,9 +434,9 @@ bool WebrtcTransport::ProcessTransportInfo(XmlElement* transport_info) {
       return false;
     }
 
-    if (peer_connection_->signaling_state() ==
+    if (peer_connection()->signaling_state() ==
         webrtc::PeerConnectionInterface::kStable) {
-      if (!peer_connection_->AddIceCandidate(candidate.get())) {
+      if (!peer_connection()->AddIceCandidate(candidate.get())) {
         LOG(ERROR) << "Failed to add incoming ICE candidate.";
         return false;
       }
@@ -334,7 +453,7 @@ void WebrtcTransport::OnLocalSessionDescriptionCreated(
     const std::string& error) {
   DCHECK(thread_checker_.CalledOnValidThread());
 
-  if (!peer_connection_)
+  if (!peer_connection())
     return;
 
   if (!description) {
@@ -349,7 +468,10 @@ void WebrtcTransport::OnLocalSessionDescriptionCreated(
     Close(CHANNEL_CONNECTION_ERROR);
     return;
   }
-  description_sdp = NormalizeSessionDescription(description_sdp);
+
+  SdpMessage sdp_message(description_sdp);
+  UpdateCodecParameters(&sdp_message, /*incoming=*/false);
+  description_sdp = sdp_message.ToString();
 
   // Format and send the session description to the peer.
   std::unique_ptr<XmlElement> transport_info(
@@ -371,7 +493,7 @@ void WebrtcTransport::OnLocalSessionDescriptionCreated(
 
   send_transport_info_callback_.Run(std::move(transport_info));
 
-  peer_connection_->SetLocalDescription(
+  peer_connection()->SetLocalDescription(
       SetSessionDescriptionObserver::Create(base::Bind(
           &WebrtcTransport::OnLocalDescriptionSet, weak_factory_.GetWeakPtr())),
       description.release());
@@ -381,7 +503,7 @@ void WebrtcTransport::OnLocalDescriptionSet(bool success,
                                             const std::string& error) {
   DCHECK(thread_checker_.CalledOnValidThread());
 
-  if (!peer_connection_)
+  if (!peer_connection())
     return;
 
   if (!success) {
@@ -398,7 +520,7 @@ void WebrtcTransport::OnRemoteDescriptionSet(bool send_answer,
                                              const std::string& error) {
   DCHECK(thread_checker_.CalledOnValidThread());
 
-  if (!peer_connection_)
+  if (!peer_connection())
     return;
 
   if (!success) {
@@ -409,7 +531,7 @@ void WebrtcTransport::OnRemoteDescriptionSet(bool send_answer,
 
   // Create and send answer on the server.
   if (send_answer) {
-    peer_connection_->CreateAnswer(
+    peer_connection()->CreateAnswer(
         CreateSessionDescriptionObserver::Create(
             base::Bind(&WebrtcTransport::OnLocalSessionDescriptionCreated,
                        weak_factory_.GetWeakPtr())),
@@ -439,7 +561,9 @@ void WebrtcTransport::OnRemoveStream(
 void WebrtcTransport::OnDataChannel(
     rtc::scoped_refptr<webrtc::DataChannelInterface> data_channel) {
   DCHECK(thread_checker_.CalledOnValidThread());
-  incoming_data_stream_adapter_.OnIncomingDataChannel(data_channel);
+  event_handler_->OnWebrtcTransportIncomingDataChannel(
+      data_channel->label(),
+      base::MakeUnique<WebrtcDataStreamAdapter>(data_channel));
 }
 
 void WebrtcTransport::OnRenegotiationNeeded() {
@@ -537,7 +661,7 @@ void WebrtcTransport::SendOffer() {
       webrtc::MediaConstraintsInterface::kValueFalse);
   offer_config.AddMandatory(webrtc::MediaConstraintsInterface::kEnableDtlsSrtp,
                             webrtc::MediaConstraintsInterface::kValueTrue);
-  peer_connection_->CreateOffer(
+  peer_connection()->CreateOffer(
       CreateSessionDescriptionObserver::Create(
           base::Bind(&WebrtcTransport::OnLocalSessionDescriptionCreated,
                      weak_factory_.GetWeakPtr())),
@@ -554,10 +678,10 @@ void WebrtcTransport::SendTransportInfo() {
 void WebrtcTransport::AddPendingCandidatesIfPossible() {
   DCHECK(thread_checker_.CalledOnValidThread());
 
-  if (peer_connection_->signaling_state() ==
+  if (peer_connection()->signaling_state() ==
       webrtc::PeerConnectionInterface::kStable) {
-    for (auto candidate : pending_incoming_candidates_) {
-      if (!peer_connection_->AddIceCandidate(candidate)) {
+    for (const auto& candidate : pending_incoming_candidates_) {
+      if (!peer_connection()->AddIceCandidate(candidate.get())) {
         LOG(ERROR) << "Failed to add incoming candidate";
         Close(INCOMPATIBLE_PROTOCOL);
         return;
@@ -569,13 +693,15 @@ void WebrtcTransport::AddPendingCandidatesIfPossible() {
 
 void WebrtcTransport::Close(ErrorCode error) {
   DCHECK(thread_checker_.CalledOnValidThread());
-  if (!peer_connection_)
+  if (!peer_connection_wrapper_)
     return;
 
   weak_factory_.InvalidateWeakPtrs();
-  peer_connection_->Close();
-  peer_connection_ = nullptr;
-  peer_connection_factory_ = nullptr;
+
+  // Close and delete PeerConnection asynchronously. PeerConnection may be on
+  // the stack and so it must be destroyed later.
+  base::ThreadTaskRunnerHandle::Get()->DeleteSoon(
+      FROM_HERE, peer_connection_wrapper_.release());
 
   if (error != OK)
     event_handler_->OnWebrtcTransportError(error);
