@@ -7,11 +7,12 @@
 #include <stdint.h>
 
 #include <cmath>
+#include <memory>
 #include <utility>
+#include <vector>
 
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
-#include "base/memory/scoped_vector.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/threading/thread_task_runner_handle.h"
@@ -140,8 +141,7 @@ void TestVideoRendererTest::TestVideoPacketProcessing(int screen_width,
       CreateDesktopFrameWithGradient(screen_width, screen_height);
   EXPECT_TRUE(original_frame);
 
-  std::unique_ptr<VideoPacket> packet =
-      encoder_->Encode(*original_frame.get(), 0);
+  std::unique_ptr<VideoPacket> packet = encoder_->Encode(*original_frame.get());
 
   DCHECK(!run_loop_ || !run_loop_->running());
   DCHECK(!timer_->IsRunning());
@@ -233,7 +233,7 @@ void TestVideoRendererTest::TestImagePatternMatch(
       CreateDesktopFrameWithGradient(screen_width, screen_height);
   RGBValue expected_average_color =
       CalculateAverageColorValueForFrame(frame.get(), expected_rect);
-  std::unique_ptr<VideoPacket> packet = encoder_->Encode(*frame.get(), 0);
+  std::unique_ptr<VideoPacket> packet = encoder_->Encode(*frame.get());
 
   if (expect_to_match) {
     EXPECT_TRUE(SendPacketAndWaitForMatch(std::move(packet), expected_rect,
@@ -400,19 +400,16 @@ TEST_F(TestVideoRendererTest, VerifyMultipleVideoProcessing) {
   // more than one task on the video decode thread, while not too large to wait
   // for too long for the unit test to complete.
   const int task_num = 20;
-  ScopedVector<VideoPacket> video_packets;
+  std::vector<std::unique_ptr<VideoPacket>> video_packets;
   for (int i = 0; i < task_num; ++i) {
     std::unique_ptr<webrtc::DesktopFrame> original_frame =
         CreateDesktopFrameWithGradient(kDefaultScreenWidthPx,
                                        kDefaultScreenHeightPx);
-    video_packets.push_back(encoder_->Encode(*original_frame.get(), 0));
+    video_packets.push_back(encoder_->Encode(*original_frame.get()));
   }
 
-  for (int i = 0; i < task_num; ++i) {
-    // Transfer ownership of video packet.
-    VideoPacket* packet = video_packets[i];
-    video_packets[i] = nullptr;
-    test_video_renderer_->ProcessVideoPacket(base::WrapUnique(packet),
+  for (auto& packet : video_packets) {
+    test_video_renderer_->ProcessVideoPacket(std::move(packet),
                                              base::Bind(&base::DoNothing));
   }
 }
@@ -454,7 +451,7 @@ TEST_F(TestVideoRendererTest, VerifySetExpectedImagePattern) {
       kDefaultExpectedRect, black_color, base::Bind(&base::DoNothing));
 
   // Post test video packet.
-  test_video_renderer_->ProcessVideoPacket(encoder_->Encode(*frame.get(), 0),
+  test_video_renderer_->ProcessVideoPacket(encoder_->Encode(*frame.get()),
                                            base::Bind(&base::DoNothing));
 }
 
