@@ -2,6 +2,7 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
+from common.chrome_proxy_shared_page_state import ChromeProxySharedPageState
 from telemetry.page import page as page_module
 from telemetry import story
 
@@ -9,7 +10,8 @@ from telemetry import story
 class CorsBypassPage(page_module.Page):
 
   def __init__(self, url, page_set):
-    super(CorsBypassPage, self).__init__(url=url, page_set=page_set)
+    super(CorsBypassPage, self).__init__(url=url, page_set=page_set,
+        shared_page_state_class=ChromeProxySharedPageState)
 
 
 class CorsBypassStorySet(story.StorySet):
