@@ -22,10 +22,6 @@ import sys
 
 # Paths from the root of the tree to directories to skip.
 PRUNE_PATHS = set([
-    # Same module occurs in crypto/third_party/nss and net/third_party/nss, so
-    # skip this one.
-    os.path.join('third_party','nss'),
-
     # Placeholder directory only, not third-party code.
     os.path.join('third_party','adobe'),
 
@@ -63,12 +59,9 @@ PRUNE_PATHS = set([
     os.path.join('third_party','pyelftools'),
     os.path.join('third_party','pylib'),
     os.path.join('third_party','pywebsocket'),
-    os.path.join('third_party','qunit'),
-    os.path.join('third_party','sinonjs'),
     os.path.join('third_party','syzygy'),
 
-    # Chromium code in third_party.
-    os.path.join('third_party','fuzzymatch'),
+    # Chromium code.
     os.path.join('tools', 'swarming_client'),
 
     # Stuff pulled in from chrome-internal for official builds/tools.
@@ -85,8 +78,6 @@ PRUNE_PATHS = set([
 
     # Redistribution does not require attribution in documentation.
     os.path.join('third_party','directxsdk'),
-    os.path.join('third_party','platformsdk_win2008_6_1'),
-    os.path.join('third_party','platformsdk_win7'),
 
     # For testing only, presents on some bots.
     os.path.join('isolate_deps_dir'),
@@ -107,7 +98,6 @@ ADDITIONAL_PATHS = (
     os.path.join('chrome', 'test', 'chromeos', 'autotest'),
     os.path.join('chrome', 'test', 'data'),
     os.path.join('native_client'),
-    os.path.join('net', 'tools', 'spdyshark'),
     os.path.join('sdch', 'open-vcdiff'),
     os.path.join('testing', 'gmock'),
     os.path.join('testing', 'gtest'),
@@ -195,7 +185,7 @@ SPECIAL_CASES = {
     os.path.join('third_party', 'WebKit'): {
         "Name": "WebKit",
         "URL": "http://webkit.org/",
-        "License": "BSD and GPL v2",
+        "License": "BSD and LGPL v2 and LGPL v2.1",
         # Absolute path here is resolved as relative to the source root.
         "License File": "/third_party/WebKit/LICENSE_FOR_ABOUT_CREDITS",
     },
@@ -228,7 +218,7 @@ SPECIAL_CASES = {
         "URL": "http://www.netlib.org/fdlibm/",
         "License": "Freely Distributable",
         # Absolute path here is resolved as relative to the source root.
-        "License File" : "/v8/src/third_party/fdlibm/LICENSE",
+        "License File" : "/v8/LICENSE.fdlibm",
         "License Android Compatible" : "yes",
     },
     os.path.join('third_party', 'khronos_glcts'): {
@@ -245,6 +235,13 @@ SPECIAL_CASES = {
         "License": "Apache 2.0",
         "License File": "NOT_SHIPPED",
     },
+    os.path.join('third_party', 'swiftshader'): {
+        "Name": "SwiftShader",
+        "URL": "https://swiftshader.googlesource.com/SwiftShader",
+        "License": "Apache 2.0 and compatible licenses",
+        "License Android Compatible": "yes",
+        "License File": "/third_party/swiftshader/LICENSE.txt",
+    },
 }
 
 # Special value for 'License File' field used to indicate that the license file
