@@ -6,7 +6,12 @@ _JS_FLAGS_SWITCH = '--js-flags='
 
 
 def EnableIgnition(options):
-  AppendJSFlags(options, '--ignition')
+  AppendJSFlags(options, '--ignition-staging')
+
+
+def EnableTurbo(options):
+  AppendJSFlags(options, '--ignition-staging')
+  AppendJSFlags(options, '--turbo')
 
 
 def AppendJSFlags(options, js_flags):
