@@ -8,12 +8,14 @@
 #include "base/android/jni_registrar.h"
 #include "base/macros.h"
 #include "ui/android/resources/resource_manager_impl.h"
+#include "ui/android/screen_android.h"
 #include "ui/android/view_android.h"
 #include "ui/android/window_android.h"
 
 namespace ui {
 
 static base::android::RegistrationMethod kAndroidRegisteredMethods[] = {
+    {"DisplayAndroidManager", ui::RegisterScreenAndroid},
     {"ResourceManager", ui::ResourceManagerImpl::RegisterResourceManager},
     {"WindowAndroid", WindowAndroid::RegisterWindowAndroid},
 };
