@@ -15,11 +15,6 @@ namespace switches {
 APP_LIST_EXPORT extern const char kCustomLauncherPage[];
 APP_LIST_EXPORT extern const char kDisableAppListDismissOnBlur[];
 APP_LIST_EXPORT extern const char kEnableAppList[];
-APP_LIST_EXPORT extern const char kEnableCenteredAppList[];
-APP_LIST_EXPORT extern const char kEnableExperimentalAppList[];
-APP_LIST_EXPORT extern const char kDisableExperimentalAppList[];
-APP_LIST_EXPORT extern const char kEnableNewAppListMixer[];
-APP_LIST_EXPORT extern const char kDisableNewAppListMixer[];
 APP_LIST_EXPORT extern const char kEnableSyncAppList[];
 APP_LIST_EXPORT extern const char kDisableSyncAppList[];
 APP_LIST_EXPORT extern const char kEnableDriveSearchInChromeLauncher[];
@@ -32,14 +27,6 @@ bool APP_LIST_EXPORT IsFolderUIEnabled();
 
 bool APP_LIST_EXPORT IsVoiceSearchEnabled();
 
-bool APP_LIST_EXPORT IsExperimentalAppListEnabled();
-
-// Determines whether either command-line switch was given for enabling the
-// centered app list position. Do not use this when positioning the app list;
-// instead use AppListViewDelegate::ShouldCenterWindow. It checks a superset of
-// the conditions that trigger the position.
-bool APP_LIST_EXPORT IsCenteredAppListEnabled();
-
 // Determines whether the app list should not be dismissed on focus loss.
 bool APP_LIST_EXPORT ShouldNotDismissOnBlur();
 
