@@ -12,7 +12,7 @@
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/macros.h"
-#include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
@@ -28,6 +28,7 @@
 #include "ui/app_list/views/apps_grid_view_folder_delegate.h"
 #include "ui/app_list/views/test/apps_grid_view_test_api.h"
 #include "ui/events/event_utils.h"
+#include "ui/views/controls/label.h"
 #include "ui/views/test/views_test_base.h"
 
 namespace app_list {
@@ -41,8 +42,8 @@ const int kTilesPerPage = kCols * kRows;
 
 class PageFlipWaiter : public PaginationModelObserver {
  public:
-  PageFlipWaiter(base::MessageLoopForUI* ui_loop, PaginationModel* model)
-      : ui_loop_(ui_loop), model_(model), wait_(false) {
+  explicit PageFlipWaiter(PaginationModel* model)
+      : model_(model), wait_(false) {
     model_->AddObserver(this);
   }
 
@@ -52,7 +53,8 @@ class PageFlipWaiter : public PaginationModelObserver {
     DCHECK(!wait_);
     wait_ = true;
 
-    ui_loop_->Run();
+    ui_run_loop_.reset(new base::RunLoop);
+    ui_run_loop_->Run();
     wait_ = false;
   }
 
@@ -69,12 +71,12 @@ class PageFlipWaiter : public PaginationModelObserver {
     selected_pages_ += base::IntToString(new_selected);
 
     if (wait_)
-      ui_loop_->QuitWhenIdle();
+      ui_run_loop_->QuitWhenIdle();
   }
   void TransitionStarted() override {}
   void TransitionChanged() override {}
 
-  base::MessageLoopForUI* ui_loop_;
+  std::unique_ptr<base::RunLoop> ui_run_loop_;
   PaginationModel* model_;
   bool wait_;
   std::string selected_pages_;
@@ -145,10 +147,10 @@ class AppsGridViewTest : public views::ViewsTestBase {
     AppListItemView* view = GetItemViewForPoint(from);
     DCHECK(view);
 
-    gfx::Point translated_from = gfx::PointAtOffsetFromOrigin(
-        from - view->bounds().origin());
-    gfx::Point translated_to = gfx::PointAtOffsetFromOrigin(
-        to - view->bounds().origin());
+    gfx::Point translated_from =
+        gfx::PointAtOffsetFromOrigin(from - view->origin());
+    gfx::Point translated_to =
+        gfx::PointAtOffsetFromOrigin(to - view->origin());
 
     ui::MouseEvent pressed_event(ui::ET_MOUSE_PRESSED, translated_from, from,
                                  ui::EventTimeForNow(), 0, 0);
@@ -439,7 +441,7 @@ TEST_F(AppsGridViewTest, MouseDragMaxItemsInFolderWithMovement) {
   // Move onto the folder and end the drag.
   to = GetItemTileRectAt(0, 1).CenterPoint();
   gfx::Point translated_to =
-      gfx::PointAtOffsetFromOrigin(to - dragged_view->bounds().origin());
+      gfx::PointAtOffsetFromOrigin(to - dragged_view->origin());
   ui::MouseEvent drag_event(ui::ET_MOUSE_DRAGGED, translated_to, to,
                             ui::EventTimeForNow(), 0, 0);
   apps_grid_view_->UpdateDragFromItem(AppsGridView::MOUSE, drag_event);
@@ -568,7 +570,7 @@ TEST_F(AppsGridViewTest, MouseDragFlipPage) {
   test_api_->SetPageFlipDelay(10);
   GetPaginationModel()->SetTransitionDurations(10, 10);
 
-  PageFlipWaiter page_flip_waiter(message_loop(), GetPaginationModel());
+  PageFlipWaiter page_flip_waiter(GetPaginationModel());
 
   const int kPages = 3;
   model_->PopulateApps(kPages * kTilesPerPage);
