@@ -18,7 +18,7 @@
 #include "base/macros.h"
 #include "base/memory/ref_counted_memory.h"
 #include "base/memory/singleton.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "third_party/skia/include/core/SkBitmap.h"
@@ -28,6 +28,7 @@
 #include "ui/base/x/selection_requestor.h"
 #include "ui/base/x/selection_utils.h"
 #include "ui/base/x/x11_util.h"
+#include "ui/base/x/x11_window_event_manager.h"
 #include "ui/events/platform/platform_event_dispatcher.h"
 #include "ui/events/platform/platform_event_observer.h"
 #include "ui/events/platform/platform_event_source.h"
@@ -301,6 +302,9 @@ class ClipboardAuraX11::AuraX11Details : public PlatformEventDispatcher {
   // Input-only window used as a selection owner.
   ::Window x_window_;
 
+  // Events selected on |x_window_|.
+  std::unique_ptr<XScopedEventSelector> x_window_events_;
+
   X11AtomCache atom_cache_;
 
   // Object which requests and receives selection data.
@@ -339,7 +343,8 @@ ClipboardAuraX11::AuraX11Details::AuraX11Details()
   atom_cache_.allow_uncached_atoms();
 
   XStoreName(x_display_, x_window_, "Chromium clipboard");
-  XSelectInput(x_display_, x_window_, PropertyChangeMask);
+  x_window_events_.reset(
+      new XScopedEventSelector(x_window_, PropertyChangeMask));
 
   if (PlatformEventSource::GetInstance())
     PlatformEventSource::GetInstance()->AddPlatformEventDispatcher(this);
@@ -673,7 +678,10 @@ ClipboardAuraX11::ClipboardAuraX11() : aurax11_details_(new AuraX11Details) {
 
 ClipboardAuraX11::~ClipboardAuraX11() {
   DCHECK(CalledOnValidThread());
+}
 
+void ClipboardAuraX11::OnPreShutdown() {
+  DCHECK(CalledOnValidThread());
   aurax11_details_->StoreCopyPasteDataAndWait();
 }
 
