@@ -215,9 +215,8 @@ bool IsDuplicateName(const std::string& locale_name) {
   if (base::StartsWith(locale_name, "es_",
                        base::CompareCase::INSENSITIVE_ASCII))
     return !base::EndsWith(locale_name, "419", base::CompareCase::SENSITIVE);
-
-  for (size_t i = 0; i < arraysize(kDuplicateNames); ++i) {
-    if (base::EqualsCaseInsensitiveASCII(kDuplicateNames[i], locale_name))
+  for (const char* duplicate_name : kDuplicateNames) {
+    if (base::EqualsCaseInsensitiveASCII(duplicate_name, locale_name))
       return true;
   }
   return false;
@@ -358,6 +357,10 @@ bool CheckAndResolveLocale(const std::string& locale,
     if (base::LowerCaseEqualsASCII(lang, "es") &&
         !base::LowerCaseEqualsASCII(region, "es")) {
       tmp_locale.append("-419");
+    } else if (base::LowerCaseEqualsASCII(lang, "pt")) {
+      // Map pt-RR other than pt-BR to pt-PT. Note that "pt" by itself maps to
+      // pt-BR (logic below).
+      tmp_locale.append("-PT");
     } else if (base::LowerCaseEqualsASCII(lang, "zh")) {
       // Map zh-HK and zh-MO to zh-TW. Otherwise, zh-FOO is mapped to zh-CN.
       if (base::LowerCaseEqualsASCII(region, "hk") ||
@@ -388,19 +391,17 @@ bool CheckAndResolveLocale(const std::string& locale,
   }
 
   // Google updater uses no, tl, iw and en for our nb, fil, he, and en-US.
+  // Note that pt-RR is mapped to pt-PT above, but we want pt -> pt-BR here.
   struct {
     const char* source;
     const char* dest;
   } alias_map[] = {
-      {"no", "nb"},
-      {"tl", "fil"},
-      {"iw", "he"},
-      {"en", "en-US"},
+      {"en", "en-US"}, {"iw", "he"},  {"no", "nb"},
+      {"pt", "pt-BR"}, {"tl", "fil"}, {"zh", "zh-CN"},
   };
-
-  for (size_t i = 0; i < arraysize(alias_map); ++i) {
-    if (base::LowerCaseEqualsASCII(lang, alias_map[i].source)) {
-      std::string tmp_locale(alias_map[i].dest);
+  for (const auto& alias : alias_map) {
+    if (base::LowerCaseEqualsASCII(lang, alias.source)) {
+      std::string tmp_locale(alias.dest);
       if (IsLocaleAvailable(tmp_locale)) {
         resolved_locale->swap(tmp_locale);
         return true;
@@ -459,7 +460,7 @@ std::string GetApplicationLocaleInternal(const std::string& pref_locale) {
 #elif defined(OS_ANDROID)
 
   // On Android, query java.util.Locale for the default locale.
-  candidates.push_back(base::android::GetDefaultLocale());
+  candidates.push_back(base::android::GetDefaultLocaleString());
 
 #elif defined(USE_GLIB) && !defined(OS_CHROMEOS)
 
@@ -862,14 +863,13 @@ const std::vector<std::string>& GetAvailableLocales() {
 
 void GetAcceptLanguagesForLocale(const std::string& display_locale,
                                  std::vector<std::string>* locale_codes) {
-  for (size_t i = 0; i < arraysize(kAcceptLanguageList); ++i) {
-    if (!l10n_util::IsLocaleNameTranslated(kAcceptLanguageList[i],
-                                           display_locale)) {
+  for (const char* accept_language : kAcceptLanguageList) {
+    if (!l10n_util::IsLocaleNameTranslated(accept_language, display_locale)) {
       // TODO(jungshik) : Put them at the end of the list with language codes
       // enclosed by brackets instead of skipping.
       continue;
     }
-    locale_codes->push_back(kAcceptLanguageList[i]);
+    locale_codes->push_back(accept_language);
   }
 }
 
