@@ -17,21 +17,17 @@
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted_memory.h"
-#include "base/metrics/histogram.h"
 #include "base/path_service.h"
 #include "base/stl_util.h"
 #include "base/strings/string_piece.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/synchronization/lock.h"
-#include "base/sys_byteorder.h"
 #include "build/build_config.h"
 #include "skia/ext/image_operations.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkColor.h"
-#include "third_party/zlib/zlib.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/layout.h"
-#include "ui/base/material_design/material_design_controller.h"
 #include "ui/base/resource/data_pack.h"
 #include "ui/base/ui_base_paths.h"
 #include "ui/base/ui_base_switches.h"
@@ -48,7 +44,6 @@
 
 #if defined(OS_ANDROID)
 #include "ui/base/resource/resource_bundle_android.h"
-#include "ui/gfx/android/device_display_info.h"
 #endif
 
 #if defined(OS_CHROMEOS)
@@ -114,39 +109,6 @@ SkBitmap CreateEmptyBitmap() {
   return bitmap;
 }
 
-// Decodes given gzip input via zlib.
-base::RefCountedBytes* DecodeGzipData(const unsigned char* input_buffer,
-                                      size_t input_size) {
-  z_stream inflateStream;
-  memset(&inflateStream, 0, sizeof(inflateStream));
-  inflateStream.zalloc = Z_NULL;
-  inflateStream.zfree = Z_NULL;
-  inflateStream.opaque = Z_NULL;
-
-  inflateStream.avail_in = input_size;
-  inflateStream.next_in = const_cast<Bytef*>(input_buffer);
-
-  CHECK(input_size >= 4);
-  // Size of output comes from footer of gzip file format, found as the last 4
-  // bytes in the compressed file, which are stored little endian.
-  inflateStream.avail_out = base::ByteSwapToLE32(
-      *reinterpret_cast<const uint32_t*>(&input_buffer[input_size - 4]));
-
-  std::vector<unsigned char> output(inflateStream.avail_out);
-  inflateStream.next_out = reinterpret_cast<Bytef*>(&output[0]);
-
-  CHECK(inflateInit2(&inflateStream, 16) == Z_OK);
-  CHECK(inflate(&inflateStream, Z_FINISH) == Z_STREAM_END);
-  CHECK(inflateEnd(&inflateStream) == Z_OK);
-
-  // Cannot use TakeVector since it puts the RefCounted* into a scoped_refptr,
-  // and callers of this function return a raw pointer (not a scoped_refptr), so
-  // the memory will be deallocated upon exit of the calling function.
-  base::RefCountedBytes* returnVal = new base::RefCountedBytes();
-  returnVal->data().swap(output);
-  return returnVal;
-}
-
 }  // namespace
 
 // An ImageSkiaSource that loads bitmaps for the requested scale factor from
@@ -252,7 +214,7 @@ void ResourceBundle::InitSharedInstanceWithPakFileRegion(
     NOTREACHED() << "failed to load pak file";
     return;
   }
-  g_shared_instance_->locale_resources_data_.reset(data_pack.release());
+  g_shared_instance_->locale_resources_data_ = std::move(data_pack);
   g_shared_instance_->InitDefaultFontList();
 }
 
@@ -292,24 +254,12 @@ bool ResourceBundle::LocaleDataPakExists(const std::string& locale) {
 
 void ResourceBundle::AddDataPackFromPath(const base::FilePath& path,
                                          ScaleFactor scale_factor) {
-  AddDataPackFromPathInternal(path, scale_factor, false, false);
+  AddDataPackFromPathInternal(path, scale_factor, false);
 }
 
 void ResourceBundle::AddOptionalDataPackFromPath(const base::FilePath& path,
-                                         ScaleFactor scale_factor) {
-  AddDataPackFromPathInternal(path, scale_factor, true, false);
-}
-
-void ResourceBundle::AddMaterialDesignDataPackFromPath(
-    const base::FilePath& path,
-    ScaleFactor scale_factor) {
-  AddDataPackFromPathInternal(path, scale_factor, false, true);
-}
-
-void ResourceBundle::AddOptionalMaterialDesignDataPackFromPath(
-    const base::FilePath& path,
-    ScaleFactor scale_factor) {
-  AddDataPackFromPathInternal(path, scale_factor, true, true);
+                                                 ScaleFactor scale_factor) {
+  AddDataPackFromPathInternal(path, scale_factor, true);
 }
 
 void ResourceBundle::AddDataPackFromFile(base::File file,
@@ -380,14 +330,12 @@ std::string ResourceBundle::LoadLocaleResources(
 
   std::unique_ptr<DataPack> data_pack(new DataPack(SCALE_FACTOR_100P));
   if (!data_pack->LoadFromPath(locale_file_path)) {
-    UMA_HISTOGRAM_ENUMERATION("ResourceBundle.LoadLocaleResourcesError",
-                              logging::GetLastSystemErrorCode(), 16000);
     LOG(ERROR) << "failed to load locale.pak";
     NOTREACHED();
     return std::string();
   }
 
-  locale_resources_data_.reset(data_pack.release());
+  locale_resources_data_ = std::move(data_pack);
   return app_locale;
 }
 #endif  // defined(OS_ANDROID)
@@ -404,10 +352,13 @@ void ResourceBundle::LoadTestResources(const base::FilePath& path,
 
   data_pack.reset(new DataPack(ui::SCALE_FACTOR_NONE));
   if (!locale_path.empty() && data_pack->LoadFromPath(locale_path)) {
-    locale_resources_data_.reset(data_pack.release());
+    locale_resources_data_ = std::move(data_pack);
   } else {
     locale_resources_data_.reset(new DataPack(ui::SCALE_FACTOR_NONE));
   }
+  // This is necessary to initialize ICU since we won't be calling
+  // LoadLocaleResources in this case.
+  l10n_util::GetApplicationLocale(std::string());
 }
 
 void ResourceBundle::UnloadLocaleResources() {
@@ -512,17 +463,9 @@ base::RefCountedMemory* ResourceBundle::LoadDataResourceBytesForScale(
 
   if (!bytes) {
     base::StringPiece data =
-        GetRawDataResourceForScaleImpl(resource_id, scale_factor);
-    if (!data.empty()) {
-      if (data.starts_with(CUSTOM_GZIP_HEADER)) {
-        // Jump past special identification byte prepended to header
-        const unsigned char* gzip_start =
-            reinterpret_cast<const unsigned char*>(data.data()) + 1;
-        bytes = DecodeGzipData(gzip_start, data.length() - 1);
-      } else {
-        bytes = new base::RefCountedStaticMemory(data.data(), data.length());
-      }
-    }
+        GetRawDataResourceForScale(resource_id, scale_factor);
+    if (!data.empty())
+      bytes = new base::RefCountedStaticMemory(data.data(), data.length());
   }
 
   return bytes;
@@ -535,13 +478,31 @@ base::StringPiece ResourceBundle::GetRawDataResource(int resource_id) const {
 base::StringPiece ResourceBundle::GetRawDataResourceForScale(
     int resource_id,
     ScaleFactor scale_factor) const {
-  base::StringPiece data =
-      GetRawDataResourceForScaleImpl(resource_id, scale_factor);
+  base::StringPiece data;
+  if (delegate_ &&
+      delegate_->GetRawDataResource(resource_id, scale_factor, &data))
+    return data;
 
-  // Do not allow this function to retrieve gzip compressed resources.
-  CHECK(!data.starts_with(CUSTOM_GZIP_HEADER));
+  if (scale_factor != ui::SCALE_FACTOR_100P) {
+    for (size_t i = 0; i < data_packs_.size(); i++) {
+      if (data_packs_[i]->GetScaleFactor() == scale_factor &&
+          data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
+                                         &data))
+        return data;
+    }
+  }
 
-  return data;
+  for (size_t i = 0; i < data_packs_.size(); i++) {
+    if ((data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_100P ||
+         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_200P ||
+         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_300P ||
+         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_NONE) &&
+        data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
+                                       &data))
+      return data;
+  }
+
+  return base::StringPiece();
 }
 
 base::string16 ResourceBundle::GetLocalizedString(int message_id) {
@@ -594,6 +555,22 @@ base::string16 ResourceBundle::GetLocalizedString(int message_id) {
   return msg;
 }
 
+base::RefCountedMemory* ResourceBundle::LoadLocalizedResourceBytes(
+    int resource_id) {
+  {
+    base::AutoLock lock_scope(*locale_resources_data_lock_);
+    base::StringPiece data;
+    if (locale_resources_data_.get() &&
+        locale_resources_data_->GetStringPiece(
+            static_cast<uint16_t>(resource_id), &data) &&
+        !data.empty()) {
+      return new base::RefCountedStaticMemory(data.data(), data.length());
+    }
+  }
+  // Release lock_scope and fall back to main data pack.
+  return LoadDataResourceBytes(resource_id);
+}
+
 const gfx::FontList& ResourceBundle::GetFontListWithDelta(
     int size_delta,
     gfx::Font::FontStyle style,
@@ -639,14 +616,12 @@ const gfx::Font& ResourceBundle::GetFontWithDelta(int size_delta,
 
 const gfx::FontList& ResourceBundle::GetFontList(FontStyle legacy_style) {
   gfx::Font::Weight font_weight = gfx::Font::Weight::NORMAL;
-  if (legacy_style == BoldFont || legacy_style == SmallBoldFont ||
-      legacy_style == MediumBoldFont || legacy_style == LargeBoldFont)
+  if (legacy_style == BoldFont || legacy_style == MediumBoldFont)
     font_weight = gfx::Font::Weight::BOLD;
 
   int size_delta = 0;
   switch (legacy_style) {
     case SmallFont:
-    case SmallBoldFont:
       size_delta = kSmallFontDelta;
       break;
     case MediumFont:
@@ -654,7 +629,6 @@ const gfx::FontList& ResourceBundle::GetFontList(FontStyle legacy_style) {
       size_delta = kMediumFontDelta;
       break;
     case LargeFont:
-    case LargeBoldFont:
       size_delta = kLargeFontDelta;
       break;
     case BaseFont:
@@ -718,8 +692,7 @@ void ResourceBundle::InitSharedInstance(Delegate* delegate) {
   if (display::Display::HasForceDeviceScaleFactor()) {
     display_density = display::Display::GetForcedDeviceScaleFactor();
   } else {
-    gfx::DeviceDisplayInfo device_info;
-    display_density = device_info.GetDIPScale();
+    display_density = GetPrimaryDisplayScale();
   }
   const ScaleFactor closest = FindClosestScaleFactorUnsafe(display_density);
   if (closest != SCALE_FACTOR_100P)
@@ -747,28 +720,6 @@ void ResourceBundle::FreeImages() {
 }
 
 void ResourceBundle::LoadChromeResources() {
-// TODO(estade): remove material design specific resources.
-// See crbug.com/613593
-#if defined(OS_MACOSX)
-  // The material design data packs contain some of the same asset IDs as in
-  // the non-material design data packs. Add these to the ResourceBundle
-  // first so that they are searched first when a request for an asset is
-  // made.
-  if (MaterialDesignController::IsModeMaterial()) {
-    if (IsScaleFactorSupported(SCALE_FACTOR_100P)) {
-      AddMaterialDesignDataPackFromPath(
-          GetResourcesPakFilePath("chrome_material_100_percent.pak"),
-          SCALE_FACTOR_100P);
-    }
-
-    if (IsScaleFactorSupported(SCALE_FACTOR_200P)) {
-      AddOptionalMaterialDesignDataPackFromPath(
-          GetResourcesPakFilePath("chrome_material_200_percent.pak"),
-          SCALE_FACTOR_200P);
-    }
-  }
-#endif
-
   // Always load the 1x data pack first as the 2x data pack contains both 1x and
   // 2x images. The 1x data pack only has 1x images, thus passes in an accurate
   // scale factor to gfx::ImageSkia::AddRepresentation.
@@ -786,8 +737,7 @@ void ResourceBundle::LoadChromeResources() {
 void ResourceBundle::AddDataPackFromPathInternal(
     const base::FilePath& path,
     ScaleFactor scale_factor,
-    bool optional,
-    bool has_only_material_assets) {
+    bool optional) {
   // Do not pass an empty |path| value to this method. If the absolute path is
   // unknown pass just the pack file name.
   DCHECK(!path.empty());
@@ -801,7 +751,6 @@ void ResourceBundle::AddDataPackFromPathInternal(
     return;
 
   std::unique_ptr<DataPack> data_pack(new DataPack(scale_factor));
-  data_pack->set_has_only_material_design_assets(has_only_material_assets);
   if (data_pack->LoadFromPath(pack_path)) {
     AddDataPack(data_pack.release());
   } else if (!optional) {
@@ -871,64 +820,33 @@ bool ResourceBundle::LoadBitmap(int resource_id,
                                 SkBitmap* bitmap,
                                 bool* fell_back_to_1x) const {
   DCHECK(fell_back_to_1x);
-  ResourceHandle* data_handle_100_percent = nullptr;
-  for (size_t i = 0; i < data_packs_.size(); ++i) {
-    if (data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_NONE &&
-        LoadBitmap(*data_packs_[i], resource_id, bitmap, fell_back_to_1x)) {
+  for (const auto& pack : data_packs_) {
+    if (pack->GetScaleFactor() == ui::SCALE_FACTOR_NONE &&
+        LoadBitmap(*pack, resource_id, bitmap, fell_back_to_1x)) {
       DCHECK(!*fell_back_to_1x);
       *scale_factor = ui::SCALE_FACTOR_NONE;
       return true;
     }
-    if (data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_100P)
-      data_handle_100_percent = data_packs_[i];
 
-    if (data_packs_[i]->GetScaleFactor() == *scale_factor &&
-        LoadBitmap(*data_packs_[i], resource_id, bitmap, fell_back_to_1x)) {
+    if (pack->GetScaleFactor() == *scale_factor &&
+        LoadBitmap(*pack, resource_id, bitmap, fell_back_to_1x)) {
       return true;
     }
   }
 
   // Unit tests may only have 1x data pack. Allow them to fallback to 1x
   // resources.
-  if (*scale_factor != ui::SCALE_FACTOR_100P && is_test_resources_ &&
-      data_handle_100_percent &&
-      LoadBitmap(*data_handle_100_percent, resource_id, bitmap,
-                 fell_back_to_1x)) {
-    *fell_back_to_1x = true;
-    return true;
-  }
-
-  return false;
-}
-
-base::StringPiece ResourceBundle::GetRawDataResourceForScaleImpl(
-    int resource_id,
-    ScaleFactor scale_factor) const {
-  base::StringPiece data;
-  if (delegate_ &&
-      delegate_->GetRawDataResource(resource_id, scale_factor, &data))
-    return data;
-
-  if (scale_factor != ui::SCALE_FACTOR_100P) {
-    for (size_t i = 0; i < data_packs_.size(); i++) {
-      if (data_packs_[i]->GetScaleFactor() == scale_factor &&
-          data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
-                                         &data))
-        return data;
+  if (is_test_resources_ && *scale_factor != ui::SCALE_FACTOR_100P) {
+    for (const auto& pack : data_packs_) {
+      if (pack->GetScaleFactor() == ui::SCALE_FACTOR_100P &&
+          LoadBitmap(*pack, resource_id, bitmap, fell_back_to_1x)) {
+        *fell_back_to_1x = true;
+        return true;
+      }
     }
   }
 
-  for (size_t i = 0; i < data_packs_.size(); i++) {
-    if ((data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_100P ||
-         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_200P ||
-         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_300P ||
-         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_NONE) &&
-        data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
-                                       &data))
-      return data;
-  }
-
-  return base::StringPiece();
+  return false;
 }
 
 gfx::Image& ResourceBundle::GetEmptyImage() {
