@@ -7,7 +7,11 @@
 #include <math.h>
 #include <stddef.h>
 
+#include <memory>
+#include <vector>
+
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/test/simple_test_tick_clock.h"
 #include "base/time/time.h"
 #include "ui/aura/client/cursor_client.h"
@@ -23,6 +27,8 @@
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/test/gl_surface_test_support.h"
 
+using EventList = std::vector<std::unique_ptr<ui::Event>>;
+
 namespace ui {
 
 namespace {
@@ -52,10 +58,10 @@ class EventCapturer : public ui::EventHandler {
     ASSERT_LT(events_.size(), 100u);
   }
 
-  const ScopedVector<ui::Event>& captured_events() const { return events_; }
+  const EventList& captured_events() const { return events_; }
 
  private:
-  ScopedVector<ui::Event> events_;
+  EventList events_;
 
   DISALLOW_COPY_AND_ASSIGN(EventCapturer);
 };
@@ -176,14 +182,14 @@ class TouchExplorationControllerTestApi {
     return touch_exploration_controller_->kSlopDistanceFromEdge;
   }
 
-  void SetTickClockForTesting(base::TickClock* simulated_clock) {
-    touch_exploration_controller_->tick_clock_ = simulated_clock;
-  }
-
   void SetTouchAccessibilityAnchorPoint(const gfx::Point& location) {
     touch_exploration_controller_->SetTouchAccessibilityAnchorPoint(location);
   }
 
+  void SetExcludeBounds(const gfx::Rect& bounds) {
+    touch_exploration_controller_->SetExcludeBounds(bounds);
+  }
+
  private:
   std::unique_ptr<TouchExplorationController> touch_exploration_controller_;
 
@@ -192,10 +198,7 @@ class TouchExplorationControllerTestApi {
 
 class TouchExplorationTest : public aura::test::AuraTestBase {
  public:
-  TouchExplorationTest() : simulated_clock_(new base::SimpleTestTickClock()) {
-    // Tests fail if time is ever 0.
-    simulated_clock_->Advance(base::TimeDelta::FromMilliseconds(10));
-  }
+  TouchExplorationTest() : simulated_clock_(nullptr) {}
   ~TouchExplorationTest() override {}
 
   void SetUp() override {
@@ -205,14 +208,20 @@ class TouchExplorationTest : public aura::test::AuraTestBase {
     cursor_client_.reset(new aura::test::TestCursorClient(root_window()));
     root_window()->AddPreTargetHandler(&event_capturer_);
     generator_.reset(new test::EventGenerator(root_window()));
-    // The generator takes ownership of the tick clock.
-    generator_->SetTickClock(
+
+    simulated_clock_ = new base::SimpleTestTickClock();
+    // Tests fail if time is ever 0.
+    simulated_clock_->Advance(base::TimeDelta::FromMilliseconds(10));
+    // ui takes ownership of the tick clock.
+    ui::SetEventTickClockForTesting(
         std::unique_ptr<base::TickClock>(simulated_clock_));
+
     cursor_client()->ShowCursor();
     cursor_client()->DisableMouseEvents();
   }
 
   void TearDown() override {
+    ui::SetEventTickClockForTesting(nullptr);
     root_window()->RemovePreTargetHandler(&event_capturer_);
     SwitchTouchExplorationMode(false);
     cursor_client_.reset();
@@ -222,29 +231,30 @@ class TouchExplorationTest : public aura::test::AuraTestBase {
  protected:
   aura::client::CursorClient* cursor_client() { return cursor_client_.get(); }
 
-  const ScopedVector<ui::Event>& GetCapturedEvents() {
+  const EventList& GetCapturedEvents() {
     return event_capturer_.captured_events();
   }
 
   std::vector<ui::LocatedEvent*> GetCapturedLocatedEvents() {
-    const ScopedVector<ui::Event>& all_events = GetCapturedEvents();
+    const EventList& all_events = GetCapturedEvents();
     std::vector<ui::LocatedEvent*> located_events;
     for (size_t i = 0; i < all_events.size(); ++i) {
       if (all_events[i]->IsMouseEvent() ||
           all_events[i]->IsTouchEvent() ||
           all_events[i]->IsGestureEvent()) {
-        located_events.push_back(static_cast<ui::LocatedEvent*>(all_events[i]));
+        located_events.push_back(
+            static_cast<ui::LocatedEvent*>(all_events[i].get()));
       }
     }
     return located_events;
   }
 
   std::vector<ui::Event*> GetCapturedEventsOfType(int type) {
-    const ScopedVector<ui::Event>& all_events = GetCapturedEvents();
+    const EventList& all_events = GetCapturedEvents();
     std::vector<ui::Event*> events;
     for (size_t i = 0; i < all_events.size(); ++i) {
       if (type == all_events[i]->type())
-        events.push_back(all_events[i]);
+        events.push_back(all_events[i].get());
     }
     return events;
   }
@@ -283,18 +293,14 @@ class TouchExplorationTest : public aura::test::AuraTestBase {
     touch_exploration_controller_->SuppressVLOGsForTesting(suppress);
   }
 
-  void SetTickClock() {
-    touch_exploration_controller_->SetTickClockForTesting(
-        static_cast<base::TickClock*>(simulated_clock_));
-  }
-
   void SwitchTouchExplorationMode(bool on) {
     if (!on && touch_exploration_controller_.get()) {
       touch_exploration_controller_.reset();
     } else if (on && !touch_exploration_controller_.get()) {
       touch_exploration_controller_.reset(
           new ui::TouchExplorationControllerTestApi(
-              new TouchExplorationController(root_window(), &delegate_)));
+              new TouchExplorationController(root_window(), &delegate_,
+                                             nullptr)));
       cursor_client()->ShowCursor();
       cursor_client()->DisableMouseEvents();
     }
@@ -392,19 +398,19 @@ class TouchExplorationTest : public aura::test::AuraTestBase {
     return touch_exploration_controller_->GetSlopDistanceFromEdge();
   }
 
-  base::TimeTicks Now() {
-    // This is the same as what EventTimeForNow() does, but here we do it
-    // with our simulated clock.
-    return simulated_clock_->NowTicks();
-  }
+  base::TimeTicks Now() { return ui::EventTimeForNow(); }
 
   void SetTouchAccessibilityAnchorPoint(const gfx::Point& location) {
     touch_exploration_controller_->SetTouchAccessibilityAnchorPoint(location);
   }
 
+  void SetExcludeBounds(const gfx::Rect& bounds) {
+    touch_exploration_controller_->SetExcludeBounds(bounds);
+  }
+
   std::unique_ptr<test::EventGenerator> generator_;
   ui::GestureDetector::Config gesture_detector_config_;
-  // Owned by |generator_|.
+  // Owned by |ui|.
   base::SimpleTestTickClock* simulated_clock_;
   MockTouchExplorationControllerDelegate delegate_;
 
@@ -589,8 +595,10 @@ TEST_F(TouchExplorationTest, TurnOnMidTouch) {
   captured_events = GetCapturedLocatedEvents();
   std::vector<ui::LocatedEvent*>::const_iterator it;
   for (it = captured_events.begin(); it != captured_events.end(); ++it) {
-    if ((*it)->type() == ui::ET_MOUSE_MOVED)
+    if ((*it)->type() == ui::ET_MOUSE_MOVED) {
+      EXPECT_TRUE((*it)->flags() & ui::EF_TOUCH_ACCESSIBILITY);
       break;
+    }
   }
   EXPECT_NE(captured_events.end(), it);
   ClearCapturedEvents();
@@ -613,7 +621,9 @@ TEST_F(TouchExplorationTest, TurnOnMidTouch) {
   captured_events = GetCapturedLocatedEvents();
   ASSERT_EQ(2u, captured_events.size());
   EXPECT_EQ(ui::ET_MOUSE_MOVED, captured_events[0]->type());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_MOUSE_MOVED, captured_events[1]->type());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_TRUE(IsInNoFingersDownState());
 }
 
@@ -709,8 +719,10 @@ TEST_F(TouchExplorationTest, DoubleTap) {
   ASSERT_EQ(2U, captured_events.size());
   EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[0]->type());
   EXPECT_EQ(tap_location, captured_events[0]->location());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_TOUCH_RELEASED, captured_events[1]->type());
   EXPECT_EQ(tap_location, captured_events[1]->location());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_TRUE(IsInNoFingersDownState());
 }
 
@@ -795,6 +807,7 @@ TEST_F(TouchExplorationTest, DoubleTapPassthrough) {
   EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[0]->type());
   EXPECT_EQ(second_tap_location - passthrough_offset,
             captured_events[0]->location());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
 
   ClearCapturedEvents();
 
@@ -813,6 +826,7 @@ TEST_F(TouchExplorationTest, DoubleTapPassthrough) {
   EXPECT_EQ(ui::ET_TOUCH_MOVED, captured_events[1]->type());
   EXPECT_EQ(second_move_location - passthrough_offset,
             captured_events[1]->location());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
 
   ClearCapturedEvents();
 
@@ -833,6 +847,7 @@ TEST_F(TouchExplorationTest, DoubleTapPassthrough) {
   EXPECT_EQ(ui::ET_TOUCH_MOVED, captured_events[0]->type());
   EXPECT_EQ(third_move_location - passthrough_offset,
             captured_events[0]->location());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
 
   // No fingers down state is only reached when every finger is lifted.
   generator_->ReleaseTouchId(1);
@@ -846,7 +861,6 @@ TEST_F(TouchExplorationTest, DoubleTapPassthrough) {
 // to the location of the last successful touch exploration.
 TEST_F(TouchExplorationTest, DoubleTapLongPress) {
   SwitchTouchExplorationMode(true);
-  SetTickClock();
   // Tap at one location, and get a mouse move event.
   gfx::Point tap_location(11, 12);
   generator_->set_current_location(tap_location);
@@ -882,9 +896,11 @@ TEST_F(TouchExplorationTest, DoubleTapLongPress) {
   ASSERT_EQ(2U, captured_events.size());
   EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[0]->type());
   EXPECT_EQ(tap_location, captured_events[0]->location());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   base::TimeTicks pressed_time = captured_events[0]->time_stamp();
   EXPECT_EQ(ui::ET_TOUCH_RELEASED, captured_events[1]->type());
   EXPECT_EQ(tap_location, captured_events[1]->location());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   base::TimeTicks released_time = captured_events[1]->time_stamp();
   EXPECT_EQ(released_time - pressed_time,
             gesture_detector_config_.longpress_timeout);
@@ -919,11 +935,15 @@ TEST_F(TouchExplorationTest, SingleTap) {
   std::vector<ui::LocatedEvent*> captured_events = GetCapturedLocatedEvents();
   ASSERT_EQ(4U, captured_events.size());
   EXPECT_EQ(ui::ET_MOUSE_MOVED, captured_events[0]->type());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_MOUSE_MOVED, captured_events[1]->type());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[2]->type());
   EXPECT_EQ(tap_location, captured_events[2]->location());
+  EXPECT_TRUE(captured_events[2]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_TOUCH_RELEASED, captured_events[3]->type());
   EXPECT_EQ(tap_location, captured_events[3]->location());
+  EXPECT_TRUE(captured_events[3]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
 }
 
 // Double-tapping without coming from touch exploration (no previous touch
@@ -998,8 +1018,10 @@ TEST_F(TouchExplorationTest, SplitTap) {
   ASSERT_EQ(2U, captured_events.size());
   EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[0]->type());
   EXPECT_EQ(initial_touch_location, captured_events[0]->location());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_TOUCH_RELEASED, captured_events[1]->type());
   EXPECT_EQ(initial_touch_location, captured_events[1]->location());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   ClearCapturedEvents();
 
   ui::TouchEvent touch_explore_release(
@@ -1025,7 +1047,7 @@ TEST_F(TouchExplorationTest, SplitTapRelease) {
   std::vector<ui::LocatedEvent*> events =
       GetCapturedLocatedEventsOfType(ui::ET_MOUSE_MOVED);
   ASSERT_EQ(1U, events.size());
-
+  EXPECT_TRUE(events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   ClearCapturedEvents();
 
   // Now tap at a different location. Release at the first location,
@@ -1047,8 +1069,10 @@ TEST_F(TouchExplorationTest, SplitTapRelease) {
   ASSERT_EQ(2U, captured_events.size());
   EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[0]->type());
   EXPECT_EQ(initial_touch_location, captured_events[0]->location());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_TOUCH_RELEASED, captured_events[1]->type());
   EXPECT_EQ(initial_touch_location, captured_events[1]->location());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
 }
 
 TEST_F(TouchExplorationTest, SplitTapMultiFinger) {
@@ -1186,7 +1210,7 @@ TEST_F(TouchExplorationTest, EnterGestureInProgressState) {
   generator_->MoveTouch(third_location);
   EXPECT_TRUE(IsInGestureInProgressState());
   EXPECT_FALSE(IsInTouchToMouseMode());
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   // Exit out of gesture mode once grace period is over and enter touch
@@ -1196,7 +1220,9 @@ TEST_F(TouchExplorationTest, EnterGestureInProgressState) {
   generator_->MoveTouch(touch_exploration_location);
   ASSERT_EQ(2U, captured_events.size());
   EXPECT_EQ(ui::ET_MOUSE_MOVED, captured_events[0]->type());
+  EXPECT_TRUE(captured_events[0]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_EQ(ui::ET_MOUSE_MOVED, captured_events[1]->type());
+  EXPECT_TRUE(captured_events[1]->flags() & ui::EF_TOUCH_ACCESSIBILITY);
   EXPECT_TRUE(IsInTouchToMouseMode());
   EXPECT_FALSE(IsInGestureInProgressState());
 }
@@ -1272,16 +1298,23 @@ TEST_F(TouchExplorationTest, DISABLED_AllFingerPermutations) {
   // We will test all permutations of events from three different fingers
   // to ensure that we return to NO_FINGERS_DOWN when fingers have been
   // released.
-  ScopedVector<ui::TouchEvent> all_events;
+  std::vector<std::unique_ptr<ui::TouchEvent>> all_events;
+
+  // A copy of all events list which can be modified without destrying events.
+  std::vector<ui::TouchEvent*> queued_events;
+
   for (int touch_id = 0; touch_id < 3; touch_id++){
     int x = 10*touch_id + 1;
     int y = 10*touch_id + 2;
-    all_events.push_back(new TouchEvent(
+    all_events.push_back(base::MakeUnique<TouchEvent>(
         ui::ET_TOUCH_PRESSED, gfx::Point(x++, y++), touch_id, Now()));
-    all_events.push_back(new TouchEvent(
+    queued_events.push_back(all_events.back().get());
+    all_events.push_back(base::MakeUnique<TouchEvent>(
         ui::ET_TOUCH_MOVED, gfx::Point(x++, y++), touch_id, Now()));
-    all_events.push_back(new TouchEvent(
+    queued_events.push_back(all_events.back().get());
+    all_events.push_back(base::MakeUnique<TouchEvent>(
         ui::ET_TOUCH_RELEASED, gfx::Point(x, y), touch_id, Now()));
+    queued_events.push_back(all_events.back().get());
   }
 
   // I'm going to explain this algorithm, and use an example in parentheses.
@@ -1291,7 +1324,6 @@ TEST_F(TouchExplorationTest, DISABLED_AllFingerPermutations) {
   const int num_permutations = Factorial(num_events);
 
   for (int p = 0; p < num_permutations; p++) {
-    std::vector<ui::TouchEvent*> queued_events = all_events.get();
     std::vector<bool> fingers_pressed(3, false);
 
     int current_num_permutations = num_permutations;
@@ -1392,7 +1424,7 @@ TEST_F(TouchExplorationTest, GestureAddedFinger) {
   generator_->MoveTouch(second_location);
   EXPECT_TRUE(IsInGestureInProgressState());
   EXPECT_FALSE(IsInTouchToMouseMode());
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   // Generate a second press, but time out past the gesture period so that
@@ -1438,7 +1470,7 @@ TEST_F(TouchExplorationTest, EnterSlideGestureState) {
 
   // Now that we are in slide gesture, we can adjust the volume.
   generator_->MoveTouch(fourth_location);
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   // Since we are at the right edge of the screen, but the sound timer has not
@@ -1495,7 +1527,7 @@ TEST_F(TouchExplorationTest, AvoidEnteringSlideGesture) {
   EXPECT_TRUE(IsInGestureInProgressState());
   EXPECT_FALSE(IsInSlideGestureState());
   EXPECT_FALSE(IsInTouchToMouseMode());
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   generator_->ReleaseTouch();
@@ -1576,7 +1608,7 @@ TEST_F(TouchExplorationTest, TestingBoundaries) {
   ASSERT_EQ(2U, num_adjust_sounds);
   ASSERT_EQ(3U, delegate_.VolumeChanges().size());
 
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   generator_->ReleaseTouch();
@@ -1617,7 +1649,7 @@ TEST_F(TouchExplorationTest, TwoFingerTap) {
   generator_->PressTouchId(2);
   EXPECT_TRUE(IsInTwoFingerTapState());
 
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   generator_->ReleaseTouchId(1);
@@ -1639,7 +1671,7 @@ TEST_F(TouchExplorationTest, TwoFingerTapAndHold) {
   generator_->PressTouchId(2);
   EXPECT_TRUE(IsInTwoFingerTapState());
 
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   AdvanceSimulatedTimePastTapDelay();
@@ -1683,7 +1715,7 @@ TEST_F(TouchExplorationTest, TwoFingerTapAndMoveFirstFinger) {
   generator_->Dispatch(&first_press_id_2);
   EXPECT_TRUE(IsInTwoFingerTapState());
 
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   // The presses have not moved out of slop yet so it should still be in
@@ -1723,7 +1755,7 @@ TEST_F(TouchExplorationTest, TwoFingerTapAndMoveSecondFinger) {
   generator_->Dispatch(&first_press_id_2);
   EXPECT_TRUE(IsInTwoFingerTapState());
 
-  const ScopedVector<ui::Event>& captured_events = GetCapturedEvents();
+  const EventList& captured_events = GetCapturedEvents();
   ASSERT_EQ(0U, captured_events.size());
 
   generator_->Dispatch(&out_slop_id_2);
@@ -1830,4 +1862,83 @@ TEST_F(TouchExplorationTest, ExitEarconPlays) {
   }
 }
 
+TEST_F(TouchExplorationTest, ExclusionArea) {
+  SwitchTouchExplorationMode(true);
+
+  gfx::Rect window = BoundsOfRootWindowInDIP();
+  gfx::Rect exclude = window;
+  exclude.Inset(0, 0, 0, 30);
+  SetExcludeBounds(exclude);
+
+  gfx::Point in_pt = exclude.CenterPoint();
+  gfx::Point in_mv_pt(in_pt.x(), (in_pt.y() + exclude.bottom()) / 2);
+  gfx::Point out_pt(in_pt.x(), exclude.bottom() + 20);
+  gfx::Point out_mv_pt(in_pt.x(), exclude.bottom() + 10);
+
+  // Motion starting in exclusion bounds is passed-through unchanged.
+  {
+    generator_->set_current_location(in_pt);
+    generator_->PressTouchId(0);
+    AdvanceSimulatedTimePastPotentialTapDelay();
+    generator_->MoveTouchId(out_mv_pt, 0);
+    generator_->ReleaseTouchId(0);
+    EXPECT_TRUE(IsInNoFingersDownState());
+    const EventList& captured_events = GetCapturedEvents();
+    ASSERT_EQ(3U, captured_events.size());
+    EXPECT_EQ(ui::ET_TOUCH_PRESSED, captured_events[0]->type());
+    EXPECT_EQ(ui::ET_TOUCH_MOVED, captured_events[1]->type());
+    EXPECT_EQ(ui::ET_TOUCH_RELEASED, captured_events[2]->type());
+    ClearCapturedEvents();
+  }
+
+  // Complete motion outside exclusion is rewritten.
+  {
+    generator_->set_current_location(out_pt);
+    generator_->PressTouchId(0);
+    AdvanceSimulatedTimePastTapDelay();
+    generator_->MoveTouchId(out_mv_pt, 0);
+    generator_->ReleaseTouchId(0);
+    AdvanceSimulatedTimePastTapDelay();
+    EXPECT_TRUE(IsInNoFingersDownState());
+    const EventList& captured_events = GetCapturedEvents();
+    ASSERT_EQ(3U, captured_events.size());
+    for (const std::unique_ptr<ui::Event>& e : captured_events) {
+      EXPECT_EQ(ui::ET_MOUSE_MOVED, e->type());
+    }
+    ClearCapturedEvents();
+  }
+
+  // For a motion starting outside: outside events are rewritten, inside
+  // events are discarded unless they end the motion.
+  {
+    // finger 0 down outside, moves inside.
+    generator_->set_current_location(out_pt);
+    generator_->PressTouchId(0);
+    AdvanceSimulatedTimePastTapDelay();
+    generator_->MoveTouchId(out_mv_pt, 0);
+    generator_->MoveTouchId(in_mv_pt, 0);
+    ASSERT_EQ(2U, GetCapturedEvents().size());
+    for (const std::unique_ptr<ui::Event>& e : GetCapturedEvents()) {
+      EXPECT_EQ(ui::ET_MOUSE_MOVED, e->type());
+    }
+    ClearCapturedEvents();
+
+    // finger 1 down inside, moves outside
+    generator_->set_current_location(in_pt);
+    generator_->PressTouchId(1);
+    generator_->MoveTouchId(out_mv_pt, 1);
+    generator_->ReleaseTouchId(1);
+    ASSERT_EQ(0U, GetCapturedEvents().size());
+    EXPECT_FALSE(IsInNoFingersDownState());
+
+    generator_->ReleaseTouchId(0);
+    AdvanceSimulatedTimePastTapDelay();
+    EXPECT_TRUE(IsInNoFingersDownState());
+
+    ASSERT_EQ(1U, GetCapturedEvents().size());
+    EXPECT_EQ(ui::ET_MOUSE_MOVED, GetCapturedEvents()[0]->type());
+    ClearCapturedEvents();
+  }
+}
+
 }  // namespace ui
