@@ -9,41 +9,29 @@
 
 #include "base/macros.h"
 #include "ui/display/display.h"
-#include "ui/display/screen.h"
+#include "ui/display/screen_base.h"
 
 namespace display {
 namespace test {
 
-// A dummy implementation of display::Screen that contains a single
-// display::Display only. The contained Display can be accessed and modified via
+// A dummy implementation of Screen that contains a single
+// Display only. The contained Display can be accessed and modified via
 // TestScreen::display().
 //
-// NOTE: Adding and removing display::DisplayOberver's are no-ops and observers
+// NOTE: Adding and removing DisplayOberver's are no-ops and observers
 // will NOT be notified ever.
-class TestScreen : public Screen {
+class TestScreen : public ScreenBase {
  public:
   TestScreen();
   ~TestScreen() override;
 
-  Display* display() { return &display_; }
-
-  // display::Screen:
+  // Screen:
   gfx::Point GetCursorScreenPoint() override;
   bool IsWindowUnderCursor(gfx::NativeWindow window) override;
   gfx::NativeWindow GetWindowAtScreenPoint(const gfx::Point& point) override;
-  int GetNumDisplays() const override;
-  std::vector<Display> GetAllDisplays() const override;
   Display GetDisplayNearestWindow(gfx::NativeView view) const override;
-  Display GetDisplayNearestPoint(const gfx::Point& point) const override;
-  Display GetDisplayMatching(const gfx::Rect& match_rect) const override;
-  Display GetPrimaryDisplay() const override;
-  void AddObserver(DisplayObserver* observer) override;
-  void RemoveObserver(DisplayObserver* observer) override;
 
  private:
-  // The only display.
-  Display display_;
-
   DISALLOW_COPY_AND_ASSIGN(TestScreen);
 };
 
