@@ -6,7 +6,8 @@
 
 #include "cc/input/input_handler.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebMouseWheelEvent.h"
 
 namespace ui {
 namespace {
@@ -87,13 +88,14 @@ class ScrollElasticityControllerTest : public testing::Test {
 
 
   void SendGestureScrollBegin(InertialPhaseState inertialPhase) {
-    blink::WebGestureEvent event;
+    TickCurrentTime();
+    blink::WebGestureEvent event(
+        blink::WebInputEvent::GestureScrollBegin,
+        blink::WebInputEvent::NoModifiers,
+        (current_time_ - base::TimeTicks()).InSecondsF());
     event.sourceDevice = blink::WebGestureDeviceTouchpad;
-    event.type = blink::WebInputEvent::GestureScrollBegin;
     event.data.scrollBegin.inertialPhase =
         static_cast<blink::WebGestureEvent::InertialPhaseState>(inertialPhase);
-    TickCurrentTime();
-    event.timeStampSeconds = (current_time_ - base::TimeTicks()).InSecondsF();
 
     controller_.ObserveGestureEventAndResult(event,
                                              cc::InputHandlerScrollResult());
@@ -104,15 +106,16 @@ class ScrollElasticityControllerTest : public testing::Test {
       InertialPhaseState inertialPhase,
       const gfx::Vector2dF& event_delta = gfx::Vector2dF(),
       const gfx::Vector2dF& overscroll_delta = gfx::Vector2dF()) {
-    blink::WebGestureEvent event;
+    TickCurrentTime();
+    blink::WebGestureEvent event(
+        blink::WebInputEvent::GestureScrollUpdate,
+        blink::WebInputEvent::NoModifiers,
+        (current_time_ - base::TimeTicks()).InSecondsF());
     event.sourceDevice = blink::WebGestureDeviceTouchpad;
-    event.type = blink::WebInputEvent::GestureScrollUpdate;
     event.data.scrollUpdate.inertialPhase =
         static_cast<blink::WebGestureEvent::InertialPhaseState>(inertialPhase);
     event.data.scrollUpdate.deltaX = -event_delta.x();
     event.data.scrollUpdate.deltaY = -event_delta.y();
-    TickCurrentTime();
-    event.timeStampSeconds = (current_time_ - base::TimeTicks()).InSecondsF();
 
     cc::InputHandlerScrollResult scroll_result;
     scroll_result.did_overscroll_root = !overscroll_delta.IsZero();
@@ -123,12 +126,12 @@ class ScrollElasticityControllerTest : public testing::Test {
   }
 
   void SendGestureScrollEnd() {
-    blink::WebGestureEvent event;
-    event.sourceDevice = blink::WebGestureDeviceTouchpad;
-    event.type = blink::WebInputEvent::GestureScrollEnd;
-
     TickCurrentTime();
-    event.timeStampSeconds = (current_time_ - base::TimeTicks()).InSecondsF();
+    blink::WebGestureEvent event(
+        blink::WebInputEvent::GestureScrollEnd,
+        blink::WebInputEvent::NoModifiers,
+        (current_time_ - base::TimeTicks()).InSecondsF());
+    event.sourceDevice = blink::WebGestureDeviceTouchpad;
 
     controller_.ObserveGestureEventAndResult(event,
                                              cc::InputHandlerScrollResult());
