@@ -12,7 +12,7 @@
 #include "base/callback.h"
 #include "base/macros.h"
 #include "base/time/time.h"
-#include "ui/events/event_constants.h"
+#include "ui/events/event.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/gfx/geometry/point.h"
 #include "ui/gfx/native_widget_types.h"
@@ -26,14 +26,8 @@ class PointF;
 }
 
 namespace ui {
-class Event;
-class EventProcessor;
 class EventSource;
 class EventTarget;
-class KeyEvent;
-class MouseEvent;
-class ScrollEvent;
-class TouchEvent;
 
 namespace test {
 
@@ -138,12 +132,23 @@ class EventGenerator {
   void set_async(bool async) { async_ = async; }
   bool async() const { return async_; }
 
-  // Dispatch events through the application instead of directly to the
-  // target window. Currently only supported on Mac.
-  void set_targeting_application(bool targeting_application) {
-    targeting_application_ = targeting_application;
-  }
-  bool targeting_application() const { return targeting_application_; }
+  // Events could be dispatched using different methods. The choice is a
+  // tradeoff between test robustness and coverage of OS internals that affect
+  // event dispatch.
+  // Currently only supported on Mac.
+  enum class Target {
+    // Dispatch through the application. Least robust.
+    APPLICATION,
+    // Dispatch directly to target NSWindow via -sendEvent:.
+    WINDOW,
+    // Default. Emulates default NSWindow dispatch: calls specific event handler
+    // based on event type. Most robust.
+    WIDGET,
+  };
+
+  // Selects dispatch method. Currently only supported on Mac.
+  void set_target(Target target) { target_ = target; }
+  Target target() const { return target_; }
 
   // Resets the event flags bitmask.
   void set_flags(int flags) { flags_ = flags; }
@@ -170,6 +175,9 @@ class EventGenerator {
   // Moves the mouse wheel by |delta_x|, |delta_y|.
   void MoveMouseWheel(int delta_x, int delta_y);
 
+  // Generates a mouse enter event.
+  void SendMouseEnter();
+
   // Generates a mouse exit.
   void SendMouseExit();
 
@@ -220,6 +228,28 @@ class EventGenerator {
   // Generates events to move the mouse to the center of the window.
   void MoveMouseToCenterOf(EventTarget* window);
 
+  // Enter pen-pointer mode, which will cause any generated mouse events to have
+  // a pointer type ui::EventPointerType::POINTER_TYPE_PEN.
+  void EnterPenPointerMode();
+
+  // Exit pen-pointer mode. Generated mouse events will use the default pointer
+  // type event.
+  void ExitPenPointerMode();
+
+  // Set radius of touch PointerDetails.
+  void SetTouchRadius(float x, float y);
+
+  // Set tilt of touch PointerDetails.
+  void SetTouchTilt(float x, float y);
+
+  // Set pointer type of touch PointerDetails.
+  void SetTouchPointerType(ui::EventPointerType type) {
+    touch_pointer_details_.pointer_type = type;
+  }
+
+  // Set force of touch PointerDetails.
+  void SetTouchForce(float force) { touch_pointer_details_.force = force; }
+
   // Generates a touch press event.
   void PressTouch();
 
@@ -229,9 +259,20 @@ class EventGenerator {
   // Generates a ET_TOUCH_MOVED event to |point|.
   void MoveTouch(const gfx::Point& point);
 
+  // Generates a ET_TOUCH_MOVED event moving by (x, y) from current location.
+  void MoveTouchBy(int x, int y) {
+    MoveTouch(current_location_ + gfx::Vector2d(x, y));
+  }
+
   // Generates a ET_TOUCH_MOVED event to |point| with |touch_id|.
   void MoveTouchId(const gfx::Point& point, int touch_id);
 
+  // Generates a ET_TOUCH_MOVED event moving (x, y) from current location with
+  // |touch_id|.
+  void MoveTouchIdBy(int touch_id, int x, int y) {
+    MoveTouchId(current_location_ + gfx::Vector2d(x, y), touch_id);
+  }
+
   // Generates a touch release event.
   void ReleaseTouch();
 
@@ -254,11 +295,6 @@ class EventGenerator {
   // to the center of the window.
   void PressMoveAndReleaseTouchToCenterOf(EventTarget* window);
 
-  // Generates and dispatches a Win8 edge-swipe event (swipe up from bottom or
-  // swipe down from top).  Note that it is not possible to distinguish between
-  // the two edges with this event.
-  void GestureEdgeSwipe();
-
   // Generates and dispatches touch-events required to generate a TAP gesture.
   // Note that this can generate a number of other gesture events at the same
   // time (e.g. GESTURE_BEGIN, TAP_DOWN, END).
@@ -348,6 +384,15 @@ class EventGenerator {
                       const std::vector<gfx::PointF>& offsets,
                       int num_fingers);
 
+  // Generate a TrackPad "rest" event. That is, a user resting fingers on the
+  // trackpad without moving. This may then be followed by a ScrollSequence(),
+  // or a CancelTrackpadRest().
+  void GenerateTrackpadRest();
+
+  // Cancels a previous GenerateTrackpadRest(). That is, a user lifting fingers
+  // from the trackpad without having moved them in any direction.
+  void CancelTrackpadRest();
+
   // Generates a key press event. On platforms except Windows and X11, a key
   // event without native_event() is generated. Note that ui::EF_ flags should
   // be passed as |flags|, not the native ones like 'ShiftMask' in <X11/X.h>.
@@ -367,12 +412,6 @@ class EventGenerator {
     current_target_ = target;
   }
 
-  // Specify an alternative tick clock to be used for simulating time in tests.
-  void SetTickClock(std::unique_ptr<base::TickClock> tick_clock);
-
-  // Get the current time from the tick clock.
-  base::TimeTicks Now();
-
   // Default delegate set by a platform-specific GeneratorDelegate singleton.
   static EventGeneratorDelegate* default_delegate;
 
@@ -401,10 +440,12 @@ class EventGenerator {
   EventTarget* current_target_;
   int flags_;
   bool grab_;
+  ui::PointerDetails touch_pointer_details_;
+
   std::list<std::unique_ptr<Event>> pending_events_;
   // Set to true to cause events to be posted asynchronously.
   bool async_;
-  bool targeting_application_;
+  Target target_;
   std::unique_ptr<base::TickClock> tick_clock_;
 
   DISALLOW_COPY_AND_ASSIGN(EventGenerator);
