@@ -2,13 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-/**
- * This definition is required by
- * ui/file_manager/file_manager/common/js/util.js.
- * @type {string}
- */
-Window.prototype.appID;
-
 /**
  * @typedef {?{
  *   position: (number|undefined),
