@@ -88,6 +88,17 @@ DeviceHandler.Notification.DEVICE_NAVIGATION = new DeviceHandler.Notification(
     'REMOVABLE_DEVICE_NAVIGATION_MESSAGE',
     'REMOVABLE_DEVICE_NAVIGATION_BUTTON_LABEL');
 
+/**
+ * @type {DeviceHandler.Notification}
+ * @const
+ */
+DeviceHandler.Notification.DEVICE_NAVIGATION_READONLY_POLICY =
+    new DeviceHandler.Notification(
+        'deviceNavigation',
+        'REMOVABLE_DEVICE_DETECTION_TITLE',
+        'REMOVABLE_DEVICE_NAVIGATION_MESSAGE_READONLY_POLICY',
+        'REMOVABLE_DEVICE_NAVIGATION_BUTTON_LABEL');
+
 /**
  * @type {DeviceHandler.Notification}
  * @const
@@ -117,6 +128,16 @@ DeviceHandler.Notification.DEVICE_FAIL_UNKNOWN = new DeviceHandler.Notification(
     'DEVICE_UNKNOWN_DEFAULT_MESSAGE',
     'DEVICE_UNKNOWN_BUTTON_LABEL');
 
+/**
+ * @type {DeviceHandler.Notification}
+ * @const
+ */
+DeviceHandler.Notification.DEVICE_FAIL_UNKNOWN_READONLY =
+    new DeviceHandler.Notification(
+        'deviceFail',
+        'REMOVABLE_DEVICE_DETECTION_TITLE',
+        'DEVICE_UNKNOWN_DEFAULT_MESSAGE');
+
 /**
  * @type {DeviceHandler.Notification}
  * @const
@@ -391,9 +412,15 @@ DeviceHandler.prototype.onMountCompleted_ = function(event) {
         message = volume.deviceLabel ?
             strf('DEVICE_UNKNOWN_MESSAGE', volume.deviceLabel) :
             str('DEVICE_UNKNOWN_DEFAULT_MESSAGE');
-        DeviceHandler.Notification.DEVICE_FAIL_UNKNOWN.show(
-            /** @type {string} */ (volume.devicePath),
-            message);
+        if (event.volumeMetadata.isReadOnly) {
+          DeviceHandler.Notification.DEVICE_FAIL_UNKNOWN_READONLY.show(
+              /** @type {string} */ (volume.devicePath),
+              message);
+        } else {
+          DeviceHandler.Notification.DEVICE_FAIL_UNKNOWN.show(
+              /** @type {string} */ (volume.devicePath),
+              message);
+        }
       }
   }
 };
@@ -407,7 +434,7 @@ DeviceHandler.prototype.onMount_ = function(event) {
   // If this is remounting, which happens when resuming Chrome OS, the device
   // has already inserted to the computer. So we suppress the notification.
   var metadata = event.volumeMetadata;
-  VolumeManager.getInstance()
+  volumeManagerFactory.getInstance()
       .then(
           /**
            * @param {!VolumeManager} volumeManager
@@ -470,8 +497,14 @@ DeviceHandler.prototype.onMount_ = function(event) {
       .catch(
         function(error) {
           if (metadata.deviceType && metadata.devicePath) {
+            if (metadata.isReadOnly &&
+                !metadata.isReadOnlyRemovableDevice) {
+              DeviceHandler.Notification.DEVICE_NAVIGATION_READONLY_POLICY.show(
+                  /** @type {string} */ (metadata.devicePath));
+            } else {
               DeviceHandler.Notification.DEVICE_NAVIGATION.show(
                   /** @type {string} */ (metadata.devicePath));
+            }
           }
         });
 };
