@@ -10,11 +10,17 @@ window.Polymer.dom = 'shadow';
  *   elements for file manager UI are loaded.
  */
 window.importElementsPromise = new Promise(function(resolve, reject) {
+  var startTime = Date.now();
+
   var link = document.createElement('link');
   link.rel = 'import';
   link.href = 'foreground/elements/elements_bundle.html';
   link.setAttribute('async', '');
-  link.onload = resolve;
+  link.onload = function() {
+    chrome.metricsPrivate.recordTime(
+        'FileBrowser.Load.ImportElements', Date.now() - startTime);
+    resolve();
+  };
   link.onerror = reject;
   document.head.appendChild(link);
 });
