@@ -506,6 +506,7 @@ Gallery.prototype.setCurrentMode_ = function(mode) {
 
   this.container_.setAttribute('mode', this.currentMode_.getName());
   this.updateSelectionAndState_();
+  this.updateModeButtonAttribute_();
 };
 
 /**
@@ -537,6 +538,17 @@ Gallery.prototype.onChangeToSlideMode_ = function() {
   this.changeCurrentMode_(this.slideMode_);
 };
 
+/**
+ * Update attributes of slide/thumbnail toggle button
+ * @private
+ */
+Gallery.prototype.updateModeButtonAttribute_ = function() {
+  if (this.currentMode_ === this.slideMode_)
+    this.modeSwitchButton_.setAttribute("aria-label", str("GALLERY_THUMBNAIL"));
+  else
+    this.modeSwitchButton_.setAttribute("aria-label", str("GALLERY_SLIDE"));
+};
+
 /**
  * Change current mode.
  * @param {!(SlideMode|ThumbnailMode)} mode Target mode.
@@ -776,7 +788,9 @@ Gallery.prototype.onKeyDown_ = function(event) {
   switch (keyString) {
     case 'Backspace':
       // The default handler would call history.back and close the Gallery.
-      event.preventDefault();
+      // Except while typing into text.
+      if(!event.target.classList.contains('text'))
+        event.preventDefault();
       break;
 
     case 'm':  // 'm' switches between Slide and Mosaic mode.
@@ -1050,7 +1064,7 @@ var loadTimeDataPromise = new Promise(function(fulfill, reject) {
  * @type {!Promise}
  */
 var volumeManagerPromise = new Promise(function(fulfill, reject) {
-  var volumeManager = new VolumeManagerWrapper(AllowedPaths.ANY_PATH);
+  var volumeManager = new VolumeManagerWrapper(AllowedPaths.ANY_PATH, false);
   volumeManager.ensureInitialized(fulfill.bind(null, volumeManager));
 });
 
