@@ -518,6 +518,11 @@ ImageEditor.Mode.prototype.markUpdated = function() {
  */
 ImageEditor.Mode.prototype.isUpdated = function() { return this.updated_; };
 
+/**
+ * @return {boolean} True if a key event should be consumed by the mode.
+ */
+ImageEditor.Mode.prototype.isConsumingKeyEvents = function() { return false; };
+
 /**
  * Resets the mode to a clean state.
  */
@@ -614,6 +619,7 @@ ImageEditor.prototype.enterMode = function(mode) {
 ImageEditor.prototype.setUpMode_ = function(mode) {
   this.currentTool_ = mode.button_;
   this.currentMode_ = mode;
+  this.rootContainer_.setAttribute('editor-mode', mode.name);
 
   // Activate toggle ripple if button is toggleable.
   var filesToggleRipple =
@@ -679,7 +685,19 @@ ImageEditor.prototype.leaveModeInternal_ = function(commit, leaveToSwitchMode) {
   if (!this.currentMode_)
     return;
 
+  // If the current mode is 'Resize', and commit is required,
+  // leaving mode should be stopped when an input value is not valid.
+  if(commit && this.currentMode_.name === 'resize') {
+    var resizeMode = /** @type {!ImageEditor.Mode.Resize} */
+                              (this.currentMode_);
+    if(!resizeMode.isInputValid()) {
+      resizeMode.showAlertDialog();
+      return;
+    }
+  }
+
   this.modeToolbar_.show(false);
+  this.rootContainer_.removeAttribute('editor-mode');
 
   // If it leaves to switch mode, do not restore screen size since the next mode
   // might change screen size. We should avoid to show intermediate animation
@@ -748,6 +766,9 @@ ImageEditor.prototype.enterModeByName_ = function(name) {
  * @return {boolean} True if handled.
  */
 ImageEditor.prototype.onKeyDown = function(event) {
+  if (this.currentMode_ && this.currentMode_.isConsumingKeyEvents())
+    return false;
+
   switch (util.getKeyModifiers(event) + event.key) {
     case 'Escape':
     case 'Enter':
@@ -1121,7 +1142,14 @@ ImageEditor.Toolbar = function(
    */
   this.updateCallback_ = opt_updateCallback || null;
 
-  // Create action buttons.
+  /**
+   * @private {!HTMLElement}
+   */
+  this.container_ = /** @type {!HTMLElement} */ (document.createElement('div'));
+  this.container_.classList.add('container');
+  this.wrapper_.appendChild(this.container_);
+
+    // Create action buttons.
   if (opt_showActionButtons) {
     var actionButtonsLayer = document.createElement('div');
     actionButtonsLayer.classList.add('action-buttons');
@@ -1138,13 +1166,6 @@ ImageEditor.Toolbar = function(
 
     this.wrapper_.appendChild(actionButtonsLayer);
   }
-
-  /**
-   * @private {!HTMLElement}
-   */
-  this.container_ = /** @type {!HTMLElement} */ (document.createElement('div'));
-  this.container_.classList.add('container');
-  this.wrapper_.appendChild(this.container_);
 };
 
 ImageEditor.Toolbar.prototype.__proto__ = cr.EventTarget.prototype;
@@ -1297,6 +1318,53 @@ ImageEditor.Toolbar.prototype.addButton = function(
   return button;
 };
 
+/**
+ * Add a input field.
+ *
+ * @param {string} name Input name
+ * @param {string} title Input title
+ * @param {function(Event)} handler onInput and onChange handler
+ * @param {string|number} value Default value
+ * @param {string=} opt_unit Unit for an input field
+ * @return {!HTMLElement} Input Element
+ */
+ImageEditor.Toolbar.prototype.addInput = function(
+    name, title, handler, value, opt_unit) {
+
+  var input = /** @type {!HTMLElement} */ (document.createElement('div'));
+  input.classList.add('input', name);
+
+  var text = document.createElement('paper-input');
+  text.setAttribute('label', strf(title));
+  text.classList.add('text', name);
+  text.value = value;
+
+  // We should listen to not only 'change' event, but also 'input' because we
+  // want to update values as soon as the user types characters.
+  text.addEventListener('input', handler, false);
+  text.addEventListener('change', handler, false);
+  input.appendChild(text);
+
+  if(opt_unit) {
+    var unit_label = document.createElement('span');
+    unit_label.textContent = opt_unit;
+    unit_label.classList.add('unit_label');
+    input.appendChild(unit_label);
+  }
+
+  input.name = name;
+  input.getValue = function(text) {
+    return text.value;
+  }.bind(this, text);
+  input.setValue = function(text, value) {
+    text.value = value;
+  }.bind(this, text);
+
+  this.add(input);
+
+  return input;
+};
+
 /**
  * Add a range control (scalar value picker).
  *
@@ -1386,6 +1454,14 @@ ImageEditor.Toolbar.prototype.show = function(on) {
     return;  // Do not show empty toolbar;
 
   this.wrapper_.hidden = !on;
+
+  // Focus the first input on the toolbar.
+  if (on) {
+    var input = this.container_.querySelector(
+       'button, paper-button, input, paper-input, paper-slider');
+    if (input)
+      input.focus();
+  }
 };
 
 /** A prompt panel for the editor.
