@@ -6,6 +6,7 @@
 
 #include <cmath>
 
+#include "base/numerics/saturated_arithmetic.h"
 #include "base/strings/stringprintf.h"
 
 namespace gfx {
@@ -15,13 +16,13 @@ bool Vector2d::IsZero() const {
 }
 
 void Vector2d::Add(const Vector2d& other) {
-  x_ += other.x_;
-  y_ += other.y_;
+  x_ = base::SaturatedAddition(other.x_, x_);
+  y_ = base::SaturatedAddition(other.y_, y_);
 }
 
 void Vector2d::Subtract(const Vector2d& other) {
-  x_ -= other.x_;
-  y_ -= other.y_;
+  x_ = base::SaturatedSubtraction(x_, other.x_);
+  y_ = base::SaturatedSubtraction(y_, other.y_);
 }
 
 int64_t Vector2d::LengthSquared() const {
