@@ -408,29 +408,33 @@ Image::Image(const std::vector<ImagePNGRep>& image_reps) {
     return;
 
   storage_ = new internal::ImageStorage(Image::kImageRepPNG);
-  AddRepresentation(base::WrapUnique(new internal::ImageRepPNG(filtered)));
+  AddRepresentation(base::MakeUnique<internal::ImageRepPNG>(filtered));
 }
 
 Image::Image(const ImageSkia& image) {
   if (!image.isNull()) {
     storage_ = new internal::ImageStorage(Image::kImageRepSkia);
     AddRepresentation(
-        base::WrapUnique(new internal::ImageRepSkia(new ImageSkia(image))));
+        base::MakeUnique<internal::ImageRepSkia>(new ImageSkia(image)));
   }
 }
 
 #if defined(OS_IOS)
-Image::Image(UIImage* image)
+Image::Image(UIImage* image) : Image(image, base::scoped_policy::RETAIN) {}
+
+Image::Image(UIImage* image, base::scoped_policy::OwnershipPolicy policy)
     : storage_(new internal::ImageStorage(Image::kImageRepCocoaTouch)) {
-  if (image)
-    AddRepresentation(
-        base::WrapUnique(new internal::ImageRepCocoaTouch(image)));
+  if (image) {
+    if (policy == base::scoped_policy::RETAIN)
+      base::mac::NSObjectRetain(image);
+    AddRepresentation(base::MakeUnique<internal::ImageRepCocoaTouch>(image));
+  }
 }
 #elif defined(OS_MACOSX)
 Image::Image(NSImage* image) {
   if (image) {
     storage_ = new internal::ImageStorage(Image::kImageRepCocoa);
-    AddRepresentation(base::WrapUnique(new internal::ImageRepCocoa(image)));
+    AddRepresentation(base::MakeUnique<internal::ImageRepCocoa>(image));
   }
 }
 #endif
