@@ -20,6 +20,7 @@
 #include "base/strings/sys_string_conversions.h"
 #include "skia/ext/skia_utils_mac.h"
 #include "third_party/skia/include/ports/SkTypeface_mac.h"
+#include "ui/gfx/decorated_text.h"
 
 namespace {
 
@@ -29,7 +30,7 @@ namespace {
 // 10.10.
 base::ScopedCFTypeRef<CTFontRef> CopyFontWithSymbolicTraits(CTFontRef font,
                                                             int sym_traits) {
-  if (base::mac::IsOSElCapitanOrLater()) {
+  if (base::mac::IsAtLeastOS10_11()) {
     return base::ScopedCFTypeRef<CTFontRef>(CTFontCreateCopyWithSymbolicTraits(
         font, 0, nullptr, sym_traits, sym_traits));
   }
@@ -60,6 +61,30 @@ base::ScopedCFTypeRef<CTFontRef> CopyFontWithSymbolicTraits(CTFontRef font,
       CTFontCreateWithFontDescriptor(desc, 0.0, nullptr));
 }
 
+// Returns whether |font_list| has a valid primary native font.
+bool HasValidPrimaryNativeFont(const gfx::FontList& font_list) {
+  return font_list.GetPrimaryFont().GetNativeFont();
+}
+
+// Checks whether |font_list| is valid. If it isn't, returns the default font
+// list (or a font list derived from it). The returned font list will have a
+// valid primary native font.
+gfx::FontList GetValidFontList(const gfx::FontList& font_list) {
+  if (HasValidPrimaryNativeFont(font_list))
+    return font_list;
+
+  const gfx::FontList default_font_list;
+  const int size_delta =
+      font_list.GetFontSize() - default_font_list.GetFontSize();
+  const gfx::FontList derived_font_list = default_font_list.Derive(
+      size_delta, font_list.GetFontStyle(), font_list.GetFontWeight());
+  if (HasValidPrimaryNativeFont(derived_font_list))
+    return derived_font_list;
+
+  DCHECK(HasValidPrimaryNativeFont(default_font_list));
+  return default_font_list;
+}
+
 }  // namespace
 
 namespace gfx {
@@ -89,6 +114,11 @@ std::unique_ptr<RenderText> RenderTextMac::CreateInstanceOfSameType() const {
   return base::WrapUnique(new RenderTextMac);
 }
 
+void RenderTextMac::SetFontList(const FontList& font_list) {
+  // Ensure the font list used has a valid native font.
+  RenderText::SetFontList(GetValidFontList(font_list));
+}
+
 bool RenderTextMac::MultilineSupported() const {
   return false;
 }
@@ -112,6 +142,10 @@ SelectionModel RenderTextMac::FindCursorPosition(const Point& point) {
   return SelectionModel();
 }
 
+bool RenderTextMac::IsSelectionSupported() const {
+  return false;
+}
+
 std::vector<RenderText::FontSpan> RenderTextMac::GetFontSpansForTesting() {
   EnsureLayout();
   if (!runs_valid_)
@@ -278,6 +312,7 @@ base::ScopedCFTypeRef<CTLineRef> RenderTextMac::EnsureLayoutInternal(
     base::ScopedCFTypeRef<CFMutableArrayRef>* attributes_owner) {
   CTFontRef ct_font =
       base::mac::NSToCFCast(font_list().GetPrimaryFont().GetNativeFont());
+  DCHECK(ct_font);
 
   const void* keys[] = {kCTFontAttributeName};
   const void* values[] = {ct_font};
@@ -448,4 +483,15 @@ void RenderTextMac::InvalidateStyle() {
   runs_valid_ = false;
 }
 
+bool RenderTextMac::GetDecoratedTextForRange(const Range& range,
+                                             DecoratedText* decorated_text) {
+  // TODO(karandeepb): This is not invoked on any codepath currently. Style the
+  // returned text if need be.
+  if (obscured())
+    return false;
+
+  decorated_text->text = GetTextFromRange(range);
+  return true;
+}
+
 }  // namespace gfx
