@@ -9,6 +9,7 @@ extern "C" {
 }
 #include <memory>
 
+#include "base/command_line.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/macros.h"
@@ -28,6 +29,7 @@ extern "C" {
 #include "ui/gfx/x/x11_types.h"
 #include "ui/gl/gl_bindings.h"
 #include "ui/gl/gl_implementation.h"
+#include "ui/gl/gl_visual_picker_glx.h"
 #include "ui/gl/sync_control_vsync_provider.h"
 
 #if defined(OXIDE_BUILD)
@@ -41,6 +43,8 @@ namespace {
 Display* g_display = nullptr;
 bool g_glx_context_create = false;
 bool g_glx_create_context_robustness_supported = false;
+bool g_glx_create_context_profile_supported = false;
+bool g_glx_create_context_profile_es2_supported = false;
 bool g_glx_texture_from_pixmap_supported = false;
 bool g_glx_oml_sync_control_supported = false;
 
@@ -51,18 +55,10 @@ bool g_glx_get_msc_rate_oml_supported = false;
 
 bool g_glx_sgi_video_sync_supported = false;
 
-static const int kGetVSyncParametersMinSeconds =
-#if defined(OS_LINUX)
-    // See crbug.com/373489
-    // On Linux, querying the vsync parameters might burn CPU for up to an
-    // entire vsync, so we only query periodically to reduce CPU usage.
-    // 5 seconds is chosen somewhat abitrarily as a balance between:
-    //  a) Drift in the phase of our signal.
-    //  b) Potential janks from periodically pegging the CPU.
-    5;
-#else
-    0;
-#endif
+// A 24-bit RGB visual and colormap to use when creating offscreen surfaces.
+Visual* g_visual = nullptr;
+int g_depth = CopyFromParent;
+Colormap g_colormap = CopyFromParent;
 
 GLXFBConfig GetConfigForWindow(Display* display,
                                gfx::AcceleratedWidget window) {
@@ -119,12 +115,33 @@ GLXFBConfig GetConfigForWindow(Display* display,
   return nullptr;
 }
 
+bool CreateDummyWindow(Display* display) {
+  DCHECK(display);
+  gfx::AcceleratedWidget parent_window =
+      RootWindow(display, DefaultScreen(display));
+  gfx::AcceleratedWidget window =
+      XCreateWindow(display, parent_window, 0, 0, 1, 1, 0, CopyFromParent,
+                    InputOutput, CopyFromParent, 0, nullptr);
+  if (!window) {
+    LOG(ERROR) << "XCreateWindow failed";
+    return false;
+  }
+  GLXFBConfig config = GetConfigForWindow(display, window);
+  GLXWindow glx_window = glXCreateWindow(display, config, window, nullptr);
+  if (!glx_window) {
+    LOG(ERROR) << "glXCreateWindow failed";
+    XDestroyWindow(display, window);
+    return false;
+  }
+  glXDestroyWindow(display, glx_window);
+  XDestroyWindow(display, window);
+  return true;
+}
+
 class OMLSyncControlVSyncProvider : public SyncControlVSyncProvider {
  public:
   explicit OMLSyncControlVSyncProvider(GLXWindow glx_window)
-      : SyncControlVSyncProvider(),
-        glx_window_(glx_window) {
-  }
+      : SyncControlVSyncProvider(), glx_window_(glx_window) {}
 
   ~OMLSyncControlVSyncProvider() override {}
 
@@ -156,10 +173,9 @@ class OMLSyncControlVSyncProvider : public SyncControlVSyncProvider {
   DISALLOW_COPY_AND_ASSIGN(OMLSyncControlVSyncProvider);
 };
 
-class SGIVideoSyncThread
-     : public base::Thread,
-       public base::NonThreadSafe,
-       public base::RefCounted<SGIVideoSyncThread> {
+class SGIVideoSyncThread : public base::Thread,
+                           public base::NonThreadSafe,
+                           public base::RefCounted<SGIVideoSyncThread> {
  public:
   static scoped_refptr<SGIVideoSyncThread> Create() {
     if (!g_video_sync_thread) {
@@ -189,41 +205,57 @@ class SGIVideoSyncThread
 
 class SGIVideoSyncProviderThreadShim {
  public:
-  explicit SGIVideoSyncProviderThreadShim(GLXFBConfig config,
-                                          GLXWindow glx_window)
-      : config_(config),
-        glx_window_(glx_window),
-        context_(nullptr),
+  explicit SGIVideoSyncProviderThreadShim(gfx::AcceleratedWidget parent_window)
+      : parent_window_(parent_window),
+        window_(0),
+        glx_window_(0),
         task_runner_(base::ThreadTaskRunnerHandle::Get()),
         cancel_vsync_flag_(),
         vsync_lock_() {
-    // This ensures that creation of |window_| has occured when this shim
-    // is executing in the same process as the call to create |window_|.
+    // This ensures that creation of |parent_window_| has occured when this shim
+    // is executing in the same thread as the call to create |parent_window_|.
     XSync(g_display, False);
   }
 
   virtual ~SGIVideoSyncProviderThreadShim() {
-    if (context_) {
-      glXDestroyContext(display_, context_);
-      context_ = nullptr;
-    }
+    if (glx_window_)
+      glXDestroyWindow(display_, glx_window_);
+
+    if (window_)
+      XDestroyWindow(display_, window_);
   }
 
-  base::CancellationFlag* cancel_vsync_flag() {
-    return &cancel_vsync_flag_;
-  }
+  base::CancellationFlag* cancel_vsync_flag() { return &cancel_vsync_flag_; }
 
-  base::Lock* vsync_lock() {
-    return &vsync_lock_;
-  }
+  base::Lock* vsync_lock() { return &vsync_lock_; }
 
   void Initialize() {
     DCHECK(display_);
 
-    context_ =
-        glXCreateNewContext(display_, config_, GLX_RGBA_TYPE, nullptr, True);
+    window_ =
+        XCreateWindow(display_, parent_window_, 0, 0, 1, 1, 0, CopyFromParent,
+                      InputOutput, CopyFromParent, 0, nullptr);
+    if (!window_) {
+      LOG(ERROR) << "video_sync: XCreateWindow failed";
+      return;
+    }
 
-    DCHECK(nullptr != context_);
+    GLXFBConfig config = GetConfigForWindow(display_, window_);
+    DCHECK(config);
+
+    glx_window_ = glXCreateWindow(display_, config, window_, nullptr);
+    if (!glx_window_) {
+      LOG(ERROR) << "video_sync: glXCreateWindow failed";
+      return;
+    }
+
+    // Create the context only once for all vsync providers.
+    if (!context_) {
+      context_ =
+        glXCreateNewContext(display_, config, GLX_RGBA_TYPE, nullptr, True);
+      if (!context_)
+        LOG(ERROR) << "video_sync: glXCreateNewContext failed";
+    }
   }
 
   void GetVSyncParameters(
@@ -251,8 +283,8 @@ class SGIVideoSyncProviderThreadShim {
     const base::TimeDelta kDefaultInterval =
         base::TimeDelta::FromSeconds(1) / 60;
 
-    task_runner_->PostTask(
-        FROM_HERE, base::Bind(callback, now, kDefaultInterval));
+    task_runner_->PostTask(FROM_HERE,
+                           base::Bind(callback, now, kDefaultInterval));
   }
 
  private:
@@ -260,11 +292,16 @@ class SGIVideoSyncProviderThreadShim {
   // the sandbox goes up.
   friend class gl::GLSurfaceGLX;
 
+  // We only need one Display and GLXContext because we only use one thread for
+  // SGI_video_sync. The display is created in GLSurfaceGLX::InitializeOneOff
+  // and the context is created the first time a vsync provider is initialized.
   static Display* display_;
+  static GLXContext context_;
 
-  GLXFBConfig config_;
+  gfx::AcceleratedWidget parent_window_;
+
+  gfx::AcceleratedWidget window_;
   GLXWindow glx_window_;
-  GLXContext context_;
 
   scoped_refptr<base::SingleThreadTaskRunner> task_runner_;
 
@@ -278,15 +315,14 @@ class SGIVideoSyncVSyncProvider
     : public gfx::VSyncProvider,
       public base::SupportsWeakPtr<SGIVideoSyncVSyncProvider> {
  public:
-  explicit SGIVideoSyncVSyncProvider(GLXFBConfig config, GLXWindow glx_window)
+  explicit SGIVideoSyncVSyncProvider(gfx::AcceleratedWidget parent_window)
       : vsync_thread_(SGIVideoSyncThread::Create()),
-        shim_(new SGIVideoSyncProviderThreadShim(config, glx_window)),
+        shim_(new SGIVideoSyncProviderThreadShim(parent_window)),
         cancel_vsync_flag_(shim_->cancel_vsync_flag()),
         vsync_lock_(shim_->vsync_lock()) {
-    vsync_thread_->message_loop()->PostTask(
-        FROM_HERE,
-        base::Bind(&SGIVideoSyncProviderThreadShim::Initialize,
-                   base::Unretained(shim_.get())));
+    vsync_thread_->task_runner()->PostTask(
+        FROM_HERE, base::Bind(&SGIVideoSyncProviderThreadShim::Initialize,
+                              base::Unretained(shim_.get())));
   }
 
   ~SGIVideoSyncVSyncProvider() override {
@@ -296,31 +332,21 @@ class SGIVideoSyncVSyncProvider
     }
 
     // Hand-off |shim_| to be deleted on the |vsync_thread_|.
-    vsync_thread_->message_loop()->DeleteSoon(
-        FROM_HERE,
-        shim_.release());
+    vsync_thread_->task_runner()->DeleteSoon(FROM_HERE, shim_.release());
   }
 
   void GetVSyncParameters(
       const gfx::VSyncProvider::UpdateVSyncCallback& callback) override {
-    if (kGetVSyncParametersMinSeconds > 0) {
-      base::TimeTicks now = base::TimeTicks::Now();
-      base::TimeDelta delta = now - last_get_vsync_parameters_time_;
-      if (delta.InSeconds() < kGetVSyncParametersMinSeconds)
-        return;
-      last_get_vsync_parameters_time_ = now;
-    }
-
     // Only one outstanding request per surface.
     if (!pending_callback_) {
       pending_callback_.reset(
           new gfx::VSyncProvider::UpdateVSyncCallback(callback));
-      vsync_thread_->message_loop()->PostTask(
+      vsync_thread_->task_runner()->PostTask(
           FROM_HERE,
-          base::Bind(&SGIVideoSyncProviderThreadShim::GetVSyncParameters,
-                     base::Unretained(shim_.get()),
-                     base::Bind(
-                         &SGIVideoSyncVSyncProvider::PendingCallbackRunner,
+          base::Bind(
+              &SGIVideoSyncProviderThreadShim::GetVSyncParameters,
+              base::Unretained(shim_.get()),
+              base::Bind(&SGIVideoSyncVSyncProvider::PendingCallbackRunner,
                          AsWeakPtr())));
     }
   }
@@ -346,8 +372,6 @@ class SGIVideoSyncVSyncProvider
   base::CancellationFlag* cancel_vsync_flag_;
   base::Lock* vsync_lock_;
 
-  base::TimeTicks last_get_vsync_parameters_time_;
-
   DISALLOW_COPY_AND_ASSIGN(SGIVideoSyncVSyncProvider);
 };
 
@@ -357,6 +381,7 @@ SGIVideoSyncThread* SGIVideoSyncThread::g_video_sync_thread = nullptr;
 // for use on a separate thread. We must allocate this before the sandbox
 // goes up (rather than on-demand when we start the thread).
 Display* SGIVideoSyncProviderThreadShim::display_ = nullptr;
+GLXContext SGIVideoSyncProviderThreadShim::context_ = 0;
 
 }  // namespace
 
@@ -395,20 +420,45 @@ bool GLSurfaceGLX::InitializeOneOff() {
     return false;
   }
 
-  g_glx_context_create =
-      HasGLXExtension("GLX_ARB_create_context");
+  g_glx_context_create = HasGLXExtension("GLX_ARB_create_context");
   g_glx_create_context_robustness_supported =
       HasGLXExtension("GLX_ARB_create_context_robustness");
+  g_glx_create_context_profile_supported =
+      HasGLXExtension("GLX_ARB_create_context_profile");
+  g_glx_create_context_profile_es2_supported =
+      HasGLXExtension("GLX_ARB_create_context_es2_profile");
   g_glx_texture_from_pixmap_supported =
       HasGLXExtension("GLX_EXT_texture_from_pixmap");
-  g_glx_oml_sync_control_supported =
-      HasGLXExtension("GLX_OML_sync_control");
+  g_glx_oml_sync_control_supported = HasGLXExtension("GLX_OML_sync_control");
   g_glx_get_msc_rate_oml_supported = g_glx_oml_sync_control_supported;
-  g_glx_sgi_video_sync_supported =
-      HasGLXExtension("GLX_SGI_video_sync");
+  g_glx_sgi_video_sync_supported = HasGLXExtension("GLX_SGI_video_sync");
 
-  if (!g_glx_get_msc_rate_oml_supported && g_glx_sgi_video_sync_supported)
-    SGIVideoSyncProviderThreadShim::display_ = gfx::OpenNewXDisplay();
+  const XVisualInfo& visual_info =
+      gl::GLVisualPickerGLX::GetInstance()->system_visual();
+  g_visual = visual_info.visual;
+  g_depth = visual_info.depth;
+  g_colormap = XCreateColormap(g_display, DefaultRootWindow(g_display),
+                               visual_info.visual, AllocNone);
+
+  // We create a dummy unmapped window for both the main Display and the video
+  // sync Display so that the Nvidia driver can initialize itself before the
+  // sandbox is set up.
+  // Unfortunately some fds e.g. /dev/nvidia0 are cached per thread and because
+  // we can't start threads before the sandbox is set up, these are accessed
+  // through the broker process. See GpuProcessPolicy::InitGpuBrokerProcess.
+  if (!CreateDummyWindow(g_display)) {
+    LOG(ERROR) << "CreateDummyWindow(g_display) failed";
+    return false;
+  }
+
+  if (!g_glx_get_msc_rate_oml_supported && g_glx_sgi_video_sync_supported) {
+    Display* video_sync_display = gfx::OpenNewXDisplay();
+    if (!CreateDummyWindow(video_sync_display)) {
+      LOG(ERROR) << "CreateDummyWindow(video_sync_display) failed";
+      return false;
+    }
+    SGIVideoSyncProviderThreadShim::display_ = video_sync_display;
+  }
 
   initialized = true;
   return true;
@@ -434,6 +484,16 @@ bool GLSurfaceGLX::IsCreateContextRobustnessSupported() {
   return g_glx_create_context_robustness_supported;
 }
 
+// static
+bool GLSurfaceGLX::IsCreateContextProfileSupported() {
+  return g_glx_create_context_profile_supported;
+}
+
+// static
+bool GLSurfaceGLX::IsCreateContextES2ProfileSupported() {
+  return g_glx_create_context_profile_es2_supported;
+}
+
 // static
 bool GLSurfaceGLX::IsTextureFromPixmapSupported() {
   return g_glx_texture_from_pixmap_supported;
@@ -452,21 +512,25 @@ GLSurfaceGLX::~GLSurfaceGLX() {}
 
 #ifndef OXIDE_BUILD
 NativeViewGLSurfaceGLX::NativeViewGLSurfaceGLX(gfx::AcceleratedWidget window)
-    : parent_window_(window), window_(0), glx_window_(0), config_(nullptr) {
-}
+    : parent_window_(window),
+      window_(0),
+      glx_window_(0),
+      config_(nullptr),
+      visual_id_(CopyFromParent) {}
 
 GLXDrawable NativeViewGLSurfaceGLX::GetDrawableHandle() const {
   return glx_window_;
 }
 
-bool NativeViewGLSurfaceGLX::Initialize(GLSurface::Format format) {
+bool NativeViewGLSurfaceGLX::Initialize(GLSurfaceFormat format) {
   XWindowAttributes attributes;
   if (!XGetWindowAttributes(g_display, parent_window_, &attributes)) {
     LOG(ERROR) << "XGetWindowAttributes failed for window " << parent_window_
-        << ".";
+               << ".";
     return false;
   }
   size_ = gfx::Size(attributes.width, attributes.height);
+  visual_id_ = XVisualIDFromVisual(attributes.visual);
   // Create a child window, with a CopyFromParent visual (to avoid inducing
   // extra blits in the driver), that we can resize exactly in Resize(),
   // correctly ordered with GL, so that we don't have invalid transient states.
@@ -480,23 +544,29 @@ bool NativeViewGLSurfaceGLX::Initialize(GLSurface::Format format) {
                           CopyFromParent, CWBackPixmap | CWBitGravity, &swa);
   XMapWindow(g_display, window_);
 
-  ui::PlatformEventSource* event_source =
-      ui::PlatformEventSource::GetInstance();
-  // Can be nullptr in tests, when we don't care about Exposes.
-  if (event_source) {
-    XSelectInput(g_display, window_, ExposureMask);
-    ui::PlatformEventSource::GetInstance()->AddPlatformEventDispatcher(this);
-  }
+  RegisterEvents();
   XFlush(g_display);
 
   GetConfig();
   DCHECK(config_);
   glx_window_ = glXCreateWindow(g_display, config_, window_, NULL);
 
-  if (g_glx_oml_sync_control_supported)
+  if (g_glx_oml_sync_control_supported) {
     vsync_provider_.reset(new OMLSyncControlVSyncProvider(glx_window_));
-  else if (g_glx_sgi_video_sync_supported)
-    vsync_provider_.reset(new SGIVideoSyncVSyncProvider(config_, glx_window_));
+  } else if (g_glx_sgi_video_sync_supported) {
+    vsync_provider_.reset(new SGIVideoSyncVSyncProvider(parent_window_));
+  } else {
+    // Assume a refresh rate of 59.9 Hz, which will cause us to skip
+    // 1 frame every 10 seconds on a 60Hz monitor, but will prevent us
+    // from blocking the GPU service due to back pressure. This would still
+    // encounter backpressure on a <60Hz monitor, but hopefully that is
+    // not common.
+    const base::TimeTicks kDefaultTimebase;
+    const base::TimeDelta kDefaultInterval =
+        base::TimeDelta::FromSeconds(1) / 59.9;
+    vsync_provider_.reset(
+        new gfx::FixedVSyncProvider(kDefaultTimebase, kDefaultInterval));
+  }
 
   return true;
 }
@@ -508,29 +578,13 @@ void NativeViewGLSurfaceGLX::Destroy() {
     glx_window_ = 0;
   }
   if (window_) {
-    ui::PlatformEventSource* event_source =
-        ui::PlatformEventSource::GetInstance();
-    if (event_source)
-      event_source->RemovePlatformEventDispatcher(this);
+    UnregisterEvents();
     XDestroyWindow(g_display, window_);
     window_ = 0;
     XFlush(g_display);
   }
 }
 
-bool NativeViewGLSurfaceGLX::CanDispatchEvent(const ui::PlatformEvent& event) {
-  return event->type == Expose && event->xexpose.window == window_;
-}
-
-uint32_t NativeViewGLSurfaceGLX::DispatchEvent(const ui::PlatformEvent& event) {
-  XEvent forwarded_event = *event;
-  forwarded_event.xexpose.window = parent_window_;
-  XSendEvent(g_display, parent_window_, False, ExposureMask,
-             &forwarded_event);
-  XFlush(g_display);
-  return ui::POST_DISPATCH_STOP_PROPAGATION;
-}
-
 bool NativeViewGLSurfaceGLX::Resize(const gfx::Size& size,
                                     float scale_factor,
                                     bool has_alpha) {
@@ -546,9 +600,8 @@ bool NativeViewGLSurfaceGLX::IsOffscreen() {
 }
 
 gfx::SwapResult NativeViewGLSurfaceGLX::SwapBuffers() {
-  TRACE_EVENT2("gpu", "NativeViewGLSurfaceGLX:RealSwapBuffers",
-      "width", GetSize().width(),
-      "height", GetSize().height());
+  TRACE_EVENT2("gpu", "NativeViewGLSurfaceGLX:RealSwapBuffers", "width",
+               GetSize().width(), "height", GetSize().height());
 
   glXSwapBuffers(g_display, GetDrawableHandle());
   return gfx::SwapResult::SWAP_ACK;
@@ -572,6 +625,10 @@ void* NativeViewGLSurfaceGLX::GetConfig() {
   return config_;
 }
 
+unsigned long NativeViewGLSurfaceGLX::GetCompatibilityKey() {
+  return visual_id_;
+}
+
 gfx::SwapResult NativeViewGLSurfaceGLX::PostSubBuffer(int x,
                                                       int y,
                                                       int width,
@@ -588,6 +645,18 @@ gfx::VSyncProvider* NativeViewGLSurfaceGLX::GetVSyncProvider() {
 NativeViewGLSurfaceGLX::~NativeViewGLSurfaceGLX() {
   Destroy();
 }
+
+void NativeViewGLSurfaceGLX::ForwardExposeEvent(XEvent* event) {
+  XEvent forwarded_event = *event;
+  forwarded_event.xexpose.window = parent_window_;
+  XSendEvent(g_display, parent_window_, False, ExposureMask, &forwarded_event);
+  XFlush(g_display);
+}
+
+bool NativeViewGLSurfaceGLX::CanHandleEvent(XEvent* event) {
+  return event->type == Expose &&
+         event->xexpose.window == static_cast<Window>(window_);
+}
 #endif
 
 UnmappedNativeViewGLSurfaceGLX::UnmappedNativeViewGLSurfaceGLX(
@@ -598,18 +667,17 @@ UnmappedNativeViewGLSurfaceGLX::UnmappedNativeViewGLSurfaceGLX(
     size_.SetSize(1, 1);
 }
 
-bool UnmappedNativeViewGLSurfaceGLX::Initialize(GLSurface::Format format) {
+bool UnmappedNativeViewGLSurfaceGLX::Initialize(GLSurfaceFormat format) {
   DCHECK(!window_);
 
-  gfx::AcceleratedWidget parent_window =
-      RootWindow(g_display, DefaultScreen(g_display));
+  gfx::AcceleratedWidget parent_window = DefaultRootWindow(g_display);
 
-  // We create a window with CopyFromParent visual so that we have the same
-  // visual as NativeViewGLSurfaceGLX (i.e. same GLXFBConfig), to ensure
-  // contexts are compatible and can be made current with either.
+  XSetWindowAttributes attrs;
+  attrs.border_pixel = 0;
+  attrs.colormap = g_colormap;
   window_ = XCreateWindow(g_display, parent_window, 0, 0, size_.width(),
-                          size_.height(), 0, CopyFromParent, InputOutput,
-                          CopyFromParent, 0, nullptr);
+                          size_.height(), 0, g_depth, InputOutput, g_visual,
+                          CWBorderPixel | CWColormap, &attrs);
   GetConfig();
   DCHECK(config_);
   glx_window_ = glXCreateWindow(g_display, config_, window_, NULL);
@@ -651,6 +719,10 @@ void* UnmappedNativeViewGLSurfaceGLX::GetConfig() {
   return config_;
 }
 
+unsigned long UnmappedNativeViewGLSurfaceGLX::GetCompatibilityKey() {
+  return XVisualIDFromVisual(g_visual);
+}
+
 UnmappedNativeViewGLSurfaceGLX::~UnmappedNativeViewGLSurfaceGLX() {
   Destroy();
 }
