@@ -22,6 +22,7 @@
 #include "ui/base/ime/text_input_client.h"
 #include "ui/compositor/layer_animation_observer.h"
 #include "ui/compositor/scoped_layer_animation_settings.h"
+#include "ui/display/types/display_constants.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/path.h"
 #include "ui/gfx/skia_util.h"
@@ -165,12 +166,14 @@ void CallbackAnimationObserver::OnLayerAnimationAborted(
 // static
 KeyboardController* KeyboardController::instance_ = NULL;
 
-KeyboardController::KeyboardController(KeyboardUI* ui)
+KeyboardController::KeyboardController(KeyboardUI* ui,
+                                       KeyboardLayoutDelegate* delegate)
     : ui_(ui),
+      layout_delegate_(delegate),
       input_method_(NULL),
       keyboard_visible_(false),
       show_on_resize_(false),
-      lock_keyboard_(false),
+      keyboard_locked_(false),
       keyboard_mode_(FULL_WIDTH),
       type_(ui::TEXT_INPUT_TYPE_NONE),
       weak_factory_(this) {
@@ -188,6 +191,8 @@ KeyboardController::~KeyboardController() {
   }
   if (input_method_)
     input_method_->RemoveObserver(this);
+  for (KeyboardControllerObserver& observer : observer_list_)
+    observer.OnKeyboardClosed();
   ui_->SetController(nullptr);
 }
 
@@ -219,9 +224,8 @@ void KeyboardController::NotifyKeyboardBoundsChanging(
     const gfx::Rect& new_bounds) {
   current_keyboard_bounds_ = new_bounds;
   if (ui_->HasKeyboardWindow() && ui_->GetKeyboardWindow()->IsVisible()) {
-    FOR_EACH_OBSERVER(KeyboardControllerObserver,
-                      observer_list_,
-                      OnKeyboardBoundsChanging(new_bounds));
+    for (KeyboardControllerObserver& observer : observer_list_)
+      observer.OnKeyboardBoundsChanging(new_bounds);
     if (keyboard::IsKeyboardOverscrollEnabled())
       ui_->InitInsets(new_bounds);
     else
@@ -242,7 +246,7 @@ void KeyboardController::HideKeyboard(HideReason reason) {
 
   NotifyKeyboardBoundsChanging(gfx::Rect());
 
-  set_lock_keyboard(false);
+  set_keyboard_locked(false);
 
   ui::LayerAnimator* container_animator = container_->layer()->GetAnimator();
   animation_observer_.reset(new CallbackAnimationObserver(
@@ -279,22 +283,24 @@ void KeyboardController::SetKeyboardMode(KeyboardMode mode) {
   if (keyboard_mode_ == FLOATING) {
     NotifyKeyboardBoundsChanging(gfx::Rect());
   } else if (keyboard_mode_ == FULL_WIDTH) {
-    // TODO(bshe): revisit this logic after we decide to support resize virtual
-    // keyboard.
-    int keyboard_height = GetContainerWindow()->bounds().height();
-    const gfx::Rect& root_bounds = container_->GetRootWindow()->bounds();
-    gfx::Rect new_bounds = root_bounds;
-    new_bounds.set_y(root_bounds.height() - keyboard_height);
-    new_bounds.set_height(keyboard_height);
-    GetContainerWindow()->SetBounds(new_bounds);
+    AdjustKeyboardBounds();
     // No animation added, so call ShowAnimationFinished immediately.
     ShowAnimationFinished();
   }
 }
 
 void KeyboardController::ShowKeyboard(bool lock) {
-  set_lock_keyboard(lock);
-  ShowKeyboardInternal();
+  set_keyboard_locked(lock);
+  ShowKeyboardInternal(display::kInvalidDisplayId);
+}
+
+void KeyboardController::ShowKeyboardInDisplay(int64_t display_id) {
+  set_keyboard_locked(true);
+  ShowKeyboardInternal(display_id);
+}
+
+bool KeyboardController::IsKeyboardWindowCreated() {
+  return keyboard_container_initialized() && ui_->HasKeyboardWindow();
 }
 
 void KeyboardController::OnWindowHierarchyChanged(
@@ -306,6 +312,7 @@ void KeyboardController::OnWindowHierarchyChanged(
 void KeyboardController::OnWindowAddedToRootWindow(aura::Window* window) {
   if (!window->GetRootWindow()->HasObserver(this))
     window->GetRootWindow()->AddObserver(this);
+  AdjustKeyboardBounds();
 }
 
 void KeyboardController::OnWindowRemovingFromRootWindow(aura::Window* window,
@@ -358,7 +365,7 @@ void KeyboardController::OnTextInputStateChanged(
 
   type_ = client ? client->GetTextInputType() : ui::TEXT_INPUT_TYPE_NONE;
 
-  if (type_ == ui::TEXT_INPUT_TYPE_NONE && !lock_keyboard_) {
+  if (type_ == ui::TEXT_INPUT_TYPE_NONE && !keyboard_locked_) {
     if (keyboard_visible_) {
       // Set the visibility state here so that any queries for visibility
       // before the timer fires returns the correct future value.
@@ -391,11 +398,12 @@ void KeyboardController::OnInputMethodDestroyed(
 }
 
 void KeyboardController::OnShowImeIfNeeded() {
-  if (IsKeyboardEnabled())
-    ShowKeyboardInternal();
+  // Calling |ShowKeyboardInternal| may move the keyboard to another display.
+  if (IsKeyboardEnabled() && !keyboard_locked())
+    ShowKeyboardInternal(display::kInvalidDisplayId);
 }
 
-void KeyboardController::ShowKeyboardInternal() {
+void KeyboardController::ShowKeyboardInternal(int64_t display_id) {
   if (!container_.get())
     return;
 
@@ -409,6 +417,13 @@ void KeyboardController::ShowKeyboardInternal() {
 
   ui_->ReloadKeyboardIfNeeded();
 
+  if (layout_delegate_ != nullptr) {
+    if (display_id != display::kInvalidDisplayId)
+      layout_delegate_->MoveKeyboardToDisplay(display_id);
+    else
+      layout_delegate_->MoveKeyboardToTouchableDisplay();
+  }
+
   if (keyboard_visible_) {
     return;
   } else if (ui_->GetKeyboardWindow()->bounds().height() == 0) {
@@ -486,6 +501,25 @@ void KeyboardController::ShowAnimationFinished() {
 
 void KeyboardController::HideAnimationFinished() {
   ui_->HideKeyboardContainer(container_.get());
+  for (KeyboardControllerObserver& observer : observer_list_)
+    observer.OnKeyboardHidden();
+}
+
+void KeyboardController::AdjustKeyboardBounds() {
+  // When keyboard is floating, no resize is necessary.
+  if (keyboard_mode_ == FLOATING)
+    return;
+
+  if (keyboard_mode_ == FULL_WIDTH) {
+    // TODO(bshe): revisit this logic after we decide to support resize virtual
+    // keyboard.
+    int keyboard_height = GetContainerWindow()->bounds().height();
+    const gfx::Rect& root_bounds = container_->GetRootWindow()->bounds();
+    gfx::Rect new_bounds = root_bounds;
+    new_bounds.set_y(root_bounds.height() - keyboard_height);
+    new_bounds.set_height(keyboard_height);
+    GetContainerWindow()->SetBounds(new_bounds);
+  }
 }
 
 }  // namespace keyboard
