@@ -62,14 +62,12 @@ void SetViewHierarchyEnabled(views::View* view, bool enabled) {
 MessageCenterView::MessageCenterView(MessageCenter* message_center,
                                      MessageCenterTray* tray,
                                      int max_height,
-                                     bool initially_settings_visible,
-                                     bool top_down)
+                                     bool initially_settings_visible)
     : message_center_(message_center),
       tray_(tray),
       scroller_(NULL),
       settings_view_(NULL),
       button_bar_(NULL),
-      top_down_(top_down),
       settings_visible_(initially_settings_visible),
       source_view_(NULL),
       source_height_(0),
@@ -97,6 +95,7 @@ MessageCenterView::MessageCenterView(MessageCenter* message_center,
   scroller_ = new views::ScrollView();
   scroller_->ClipHeightTo(kMinScrollViewHeight, max_height - button_height);
   scroller_->SetVerticalScrollBar(new views::OverlayScrollBar(false));
+  scroller_->SetHorizontalScrollBar(new views::OverlayScrollBar(true));
   scroller_->set_background(
       views::Background::CreateSolidBackground(kMessageCenterBackgroundColor));
 
@@ -104,8 +103,9 @@ MessageCenterView::MessageCenterView(MessageCenter* message_center,
   scroller_->layer()->SetFillsBoundsOpaquely(false);
   scroller_->layer()->SetMasksToBounds(true);
 
-  message_list_view_.reset(new MessageListView(this, top_down));
+  message_list_view_.reset(new MessageListView());
   message_list_view_->set_owned_by_client();
+  message_list_view_->AddObserver(this);
 
   // We want to swap the contents of the scroll view between the empty list
   // view and the message list view, without constructing them afresh each
@@ -129,6 +129,8 @@ MessageCenterView::MessageCenterView(MessageCenter* message_center,
 }
 
 MessageCenterView::~MessageCenterView() {
+  message_list_view_->RemoveObserver(this);
+
   if (!is_closing_)
     message_center_->RemoveObserver(this);
 }
@@ -212,19 +214,16 @@ void MessageCenterView::Layout() {
   bool animating = settings_transition_animation_ &&
                    settings_transition_animation_->is_animating();
   if (animating && settings_transition_animation_->current_part_index() == 0) {
-    if (!top_down_) {
-      button_bar_->SetBounds(
-          0, height() - button_height, width(), button_height);
-    }
+    button_bar_->SetBounds(0, height() - button_height, width(), button_height);
     return;
   }
 
   scroller_->SetBounds(0,
-                       top_down_ ? button_height : 0,
+                       0,
                        width(),
                        height() - button_height);
   settings_view_->SetBounds(0,
-                            top_down_ ? button_height : 0,
+                            0,
                             width(),
                             height() - button_height);
 
@@ -238,16 +237,16 @@ void MessageCenterView::Layout() {
     if (is_scrollable) {
       // Draw separator line on the top of the button bar if it is on the bottom
       // or draw it at the bottom if the bar is on the top.
-      button_bar_->SetBorder(views::Border::CreateSolidSidedBorder(
-          top_down_ ? 0 : 1, 0, top_down_ ? 1 : 0, 0, kFooterDelimiterColor));
+      button_bar_->SetBorder(views::CreateSolidSidedBorder(
+          1, 0, 0, 0, kFooterDelimiterColor));
     } else {
-      button_bar_->SetBorder(views::Border::CreateEmptyBorder(
-          top_down_ ? 0 : 1, 0, top_down_ ? 1 : 0, 0));
+      button_bar_->SetBorder(
+          views::CreateEmptyBorder(1, 0, 0, 0));
     }
     button_bar_->SchedulePaint();
   }
   button_bar_->SetBounds(0,
-                         top_down_ ? 0 : height() - button_height,
+                         height() - button_height,
                          width(),
                          button_height);
   if (GetWidget())
@@ -344,8 +343,7 @@ void MessageCenterView::OnNotificationRemoved(const std::string& id,
     // Moves the keyboard focus to the next notification if the removed
     // notification is focused so that the user can dismiss notifications
     // without re-focusing by tab key.
-    if (view->IsCloseButtonFocused() ||
-        view == GetFocusManager()->GetFocusedView()) {
+    if (view->IsCloseButtonFocused() || view->HasFocus()) {
       views::View* next_focused_view = NULL;
       if (message_list_view_->child_count() > index + 1)
         next_focused_view = message_list_view_->child_at(index + 1);
@@ -398,9 +396,15 @@ void MessageCenterView::OnNotificationUpdated(const std::string& id) {
     if ((*iter)->id() == id) {
       int old_width = view->width();
       int old_height = view->height();
+      bool old_pinned = view->IsPinned();
       message_list_view_->UpdateNotification(view, **iter);
-      if (view->GetHeightForWidth(old_width) != old_height)
+      if (view->GetHeightForWidth(old_width) != old_height) {
         Update(true /* animate */);
+      } else if (view->IsPinned() != old_pinned) {
+        // Animate flag is false, since the pinned flag transition doesn't need
+        // animation.
+        Update(false /* animate */);
+      }
       break;
     }
   }
