@@ -25,12 +25,6 @@ intptr_t SurfaceFactoryOzoneOxide::GetNativeDisplay() {
   return gfx::GetOxideNativeDisplay();
 }
 
-bool SurfaceFactoryOzoneOxide::LoadEGLGLES2Bindings(
-    AddGLLibraryCallback add_gl_library,
-    SetGLGetProcAddressProcCallback set_gl_get_proc_address) {
-  return false;
-}
-
-SurfaceFactoryOzoneOxide::SurfaceFactoryOzoneOxide() {}
+SurfaceFactoryOzoneOxide::SurfaceFactoryOzoneOxide() = default;
 
 } // namespace ui
