@@ -11,9 +11,10 @@
 #include "base/logging.h"
 #include "base/macros.h"
 #include "build/build_config.h"
-#include "ui/accessibility/ax_view_state.h"
+#include "ui/accessibility/ax_node_data.h"
 #include "ui/base/default_style.h"
 #include "ui/base/ime/input_method.h"
+#include "ui/base/material_design/material_design_controller.h"
 #include "ui/base/models/combobox_model.h"
 #include "ui/base/models/combobox_model_observer.h"
 #include "ui/base/resource/resource_bundle.h"
@@ -27,10 +28,14 @@
 #include "ui/native_theme/native_theme.h"
 #include "ui/native_theme/native_theme_aura.h"
 #include "ui/resources/grit/ui_resources.h"
+#include "ui/views/animation/flood_fill_ink_drop_ripple.h"
+#include "ui/views/animation/ink_drop_highlight.h"
+#include "ui/views/animation/ink_drop_impl.h"
 #include "ui/views/background.h"
 #include "ui/views/controls/button/custom_button.h"
 #include "ui/views/controls/button/label_button.h"
 #include "ui/views/controls/combobox/combobox_listener.h"
+#include "ui/views/controls/focus_ring.h"
 #include "ui/views/controls/focusable_border.h"
 #include "ui/views/controls/menu/menu_config.h"
 #include "ui/views/controls/menu/menu_runner.h"
@@ -87,6 +92,10 @@ const int kFocusedPressedMenuButtonImages[] =
 
 #undef MENU_IMAGE_GRID
 
+bool UseMd() {
+  return ui::MaterialDesignController::IsSecondaryUiMaterial();
+}
+
 gfx::Rect PositionArrowWithinContainer(const gfx::Rect& container_bounds,
                                        const gfx::Size& arrow_size,
                                        Combobox::Style style) {
@@ -106,26 +115,53 @@ gfx::Rect PositionArrowWithinContainer(const gfx::Rect& container_bounds,
 // The transparent button which holds a button state but is not rendered.
 class TransparentButton : public CustomButton {
  public:
-  TransparentButton(ButtonListener* listener)
+  TransparentButton(ButtonListener* listener, bool animate_state_change)
       : CustomButton(listener) {
-    SetAnimationDuration(LabelButton::kHoverAnimationDurationMs);
+    set_animate_on_state_change(animate_state_change);
+    if (animate_state_change)
+      SetAnimationDuration(LabelButton::kHoverAnimationDurationMs);
     SetFocusBehavior(FocusBehavior::NEVER);
+    set_notify_action(PlatformStyle::kMenuNotifyActivationAction);
+
+    if (UseMd()) {
+      SetInkDropMode(PlatformStyle::kUseRipples ? InkDropMode::ON
+                                                : InkDropMode::OFF);
+      set_has_ink_drop_action_on_click(true);
+    }
   }
   ~TransparentButton() override {}
 
   bool OnMousePressed(const ui::MouseEvent& mouse_event) override {
-    parent()->RequestFocus();
-    return true;
+    if (!UseMd())
+      parent()->RequestFocus();
+    return CustomButton::OnMousePressed(mouse_event);
   }
 
   double GetAnimationValue() const {
     return hover_animation().GetCurrentValue();
   }
 
+  // Overridden from InkDropHost:
+  std::unique_ptr<InkDrop> CreateInkDrop() override {
+    std::unique_ptr<views::InkDropImpl> ink_drop = CreateDefaultInkDropImpl();
+    ink_drop->SetShowHighlightOnHover(false);
+    return std::move(ink_drop);
+  }
+
+  std::unique_ptr<InkDropRipple> CreateInkDropRipple() const override {
+    return std::unique_ptr<views::InkDropRipple>(
+        new views::FloodFillInkDropRipple(
+            size(), GetInkDropCenterBasedOnLastEvent(),
+            GetNativeTheme()->GetSystemColor(
+                ui::NativeTheme::kColorId_LabelEnabledColor),
+            ink_drop_visible_opacity()));
+  }
+
  private:
   DISALLOW_COPY_AND_ASSIGN(TransparentButton);
 };
 
+#if !defined(OS_MACOSX)
 // Returns the next or previous valid index (depending on |increment|'s value).
 // Skips separator or disabled indices. Returns -1 if there is no valid adjacent
 // index.
@@ -140,6 +176,7 @@ int GetAdjacentIndex(ui::ComboboxModel* model, int increment, int index) {
   }
   return kNoSelection;
 }
+#endif
 
 // Returns the image resource ids of an array for the body button.
 //
@@ -244,15 +281,15 @@ void PaintArrowButton(
 const char Combobox::kViewClassName[] = "views/Combobox";
 
 // Adapts a ui::ComboboxModel to a ui::MenuModel.
-class Combobox::ComboboxMenuModelAdapter : public ui::MenuModel,
-                                           public ui::ComboboxModelObserver {
+class Combobox::ComboboxMenuModel : public ui::MenuModel,
+                                    public ui::ComboboxModelObserver {
  public:
-  ComboboxMenuModelAdapter(Combobox* owner, ui::ComboboxModel* model)
+  ComboboxMenuModel(Combobox* owner, ui::ComboboxModel* model)
       : owner_(owner), model_(model) {
     model_->AddObserver(this);
   }
 
-  ~ComboboxMenuModelAdapter() override { model_->RemoveObserver(this); }
+  ~ComboboxMenuModel() override { model_->RemoveObserver(this); }
 
  private:
   bool UseCheckmarks() const {
@@ -351,7 +388,7 @@ class Combobox::ComboboxMenuModelAdapter : public ui::MenuModel,
   Combobox* owner_;           // Weak. Owns this.
   ui::ComboboxModel* model_;  // Weak.
 
-  DISALLOW_COPY_AND_ASSIGN(ComboboxMenuModelAdapter);
+  DISALLOW_COPY_AND_ASSIGN(ComboboxMenuModel);
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -360,12 +397,12 @@ class Combobox::ComboboxMenuModelAdapter : public ui::MenuModel,
 Combobox::Combobox(ui::ComboboxModel* model, Style style)
     : model_(model),
       style_(style),
-      listener_(NULL),
+      listener_(nullptr),
       selected_index_(style == STYLE_ACTION ? 0 : model_->GetDefaultIndex()),
       invalid_(false),
-      menu_model_adapter_(new ComboboxMenuModelAdapter(this, model)),
-      text_button_(new TransparentButton(this)),
-      arrow_button_(new TransparentButton(this)),
+      menu_model_(new ComboboxMenuModel(this, model)),
+      text_button_(new TransparentButton(this, style_ == STYLE_ACTION)),
+      arrow_button_(new TransparentButton(this, style_ == STYLE_ACTION)),
       size_to_largest_label_(style_ == STYLE_NORMAL),
       weak_ptr_factory_(this) {
   ModelChanged();
@@ -376,11 +413,6 @@ Combobox::Combobox(ui::ComboboxModel* model, Style style)
 #endif
 
   UpdateBorder();
-  arrow_image_ = PlatformStyle::CreateComboboxArrow(enabled(), style);
-  // set_background() takes ownership but takes a raw pointer.
-  std::unique_ptr<Background> b =
-      PlatformStyle::CreateComboboxBackground(GetArrowContainerWidth());
-  set_background(b.release());
 
   // Initialize the button images.
   Button::ButtonState button_states[] = {
@@ -396,8 +428,8 @@ Combobox::Combobox(ui::ComboboxModel* model, Style style)
       size_t num;
       bool focused = !!i;
       const int* ids = GetBodyButtonImageIds(focused, state, &num);
-      body_button_painters_[focused][state].reset(
-          Painter::CreateImageGridPainter(ids));
+      body_button_painters_[focused][state] =
+          Painter::CreateImageGridPainter(ids);
       menu_button_images_[focused][state] = GetMenuButtonImages(focused, state);
     }
   }
@@ -406,6 +438,15 @@ Combobox::Combobox(ui::ComboboxModel* model, Style style)
   arrow_button_->SetVisible(true);
   AddChildView(text_button_);
   AddChildView(arrow_button_);
+
+  // A layer is applied to make sure that canvas bounds are snapped to pixel
+  // boundaries (for the sake of drawing the arrow).
+  if (UseMd()) {
+    SetPaintToLayer(true);
+    layer()->SetFillsBoundsOpaquely(false);
+  } else {
+    arrow_image_ = PlatformStyle::CreateComboboxArrow(enabled(), style);
+  }
 }
 
 Combobox::~Combobox() {
@@ -470,12 +511,17 @@ void Combobox::SetInvalid(bool invalid) {
 
   invalid_ = invalid;
 
+  if (HasFocus() && UseMd()) {
+    FocusRing::Install(this, invalid_
+                                 ? ui::NativeTheme::kColorId_AlertSeverityHigh
+                                 : ui::NativeTheme::kColorId_NumColors);
+  }
   UpdateBorder();
   SchedulePaint();
 }
 
 void Combobox::Layout() {
-  PrefixDelegate::Layout();
+  View::Layout();
 
   int text_button_width = 0;
   int arrow_button_width = 0;
@@ -498,8 +544,20 @@ void Combobox::Layout() {
 }
 
 void Combobox::OnEnabledChanged() {
-  PrefixDelegate::OnEnabledChanged();
-  arrow_image_ = PlatformStyle::CreateComboboxArrow(enabled(), style_);
+  View::OnEnabledChanged();
+  if (!UseMd())
+    arrow_image_ = PlatformStyle::CreateComboboxArrow(enabled(), style_);
+}
+
+void Combobox::OnNativeThemeChanged(const ui::NativeTheme* theme) {
+  if (!UseMd())
+    return;
+
+  set_background(
+      Background::CreateBackgroundPainter(Painter::CreateSolidRoundRectPainter(
+          theme->GetSystemColor(
+              ui::NativeTheme::kColorId_TextfieldDefaultBackground),
+          FocusableBorder::kCornerRadiusDp)));
 }
 
 int Combobox::GetRowCount() {
@@ -563,6 +621,16 @@ bool Combobox::OnKeyPressed(const ui::KeyEvent& e) {
   bool show_menu = false;
   int new_index = kNoSelection;
   switch (e.key_code()) {
+#if defined(OS_MACOSX)
+    case ui::VKEY_DOWN:
+    case ui::VKEY_UP:
+    case ui::VKEY_SPACE:
+    case ui::VKEY_HOME:
+    case ui::VKEY_END:
+      // On Mac, navigation keys should always just show the menu first.
+      show_menu = true;
+      break;
+#else
     // Show the menu on F4 without modifiers.
     case ui::VKEY_F4:
       if (e.IsAltDown() || e.IsAltGrDown() || e.IsControlDown())
@@ -595,24 +663,23 @@ bool Combobox::OnKeyPressed(const ui::KeyEvent& e) {
       new_index = GetAdjacentIndex(model(), -1, selected_index_);
       break;
 
-    // Click the button only when the button style mode.
     case ui::VKEY_SPACE:
       if (style_ == STYLE_ACTION) {
         // When pressing space, the click event will be raised after the key is
         // released.
         text_button_->SetState(Button::STATE_PRESSED);
       } else {
-        return false;
+        show_menu = true;
       }
       break;
 
-    // Click the button only when the button style mode.
     case ui::VKEY_RETURN:
-      if (style_ != STYLE_ACTION)
-        return false;
-      OnPerformAction();
+      if (style_ == STYLE_ACTION)
+        OnPerformAction();
+      else
+        show_menu = true;
       break;
-
+#endif  // OS_MACOSX
     default:
       return false;
   }
@@ -633,7 +700,8 @@ bool Combobox::OnKeyReleased(const ui::KeyEvent& e) {
   if (style_ != STYLE_ACTION)
     return false;  // crbug.com/127520
 
-  if (e.key_code() == ui::VKEY_SPACE && style_ == STYLE_ACTION)
+  if (e.key_code() == ui::VKEY_SPACE && style_ == STYLE_ACTION &&
+      text_button_->state() == Button::STATE_PRESSED)
     OnPerformAction();
 
   return false;
@@ -662,6 +730,11 @@ void Combobox::OnFocus() {
   View::OnFocus();
   // Border renders differently when focused.
   SchedulePaint();
+  if (UseMd()) {
+    FocusRing::Install(this, invalid_
+                                 ? ui::NativeTheme::kColorId_AlertSeverityHigh
+                                 : ui::NativeTheme::kColorId_NumColors);
+  }
 }
 
 void Combobox::OnBlur() {
@@ -672,21 +745,28 @@ void Combobox::OnBlur() {
     selector_->OnViewBlur();
   // Border renders differently when focused.
   SchedulePaint();
+  if (UseMd())
+    FocusRing::Uninstall(this);
 }
 
-void Combobox::GetAccessibleState(ui::AXViewState* state) {
-  state->role = ui::AX_ROLE_COMBO_BOX;
-  state->name = accessible_name_;
-  state->value = model_->GetItemAt(selected_index_);
-  state->index = selected_index_;
-  state->count = model_->GetItemCount();
+void Combobox::GetAccessibleNodeData(ui::AXNodeData* node_data) {
+  node_data->role = ui::AX_ROLE_COMBO_BOX;
+  node_data->SetName(accessible_name_);
+  node_data->SetValue(model_->GetItemAt(selected_index_));
+  if (enabled()) {
+    node_data->AddIntAttribute(ui::AX_ATTR_ACTION,
+                               ui::AX_SUPPORTED_ACTION_OPEN);
+  }
+  node_data->AddIntAttribute(ui::AX_ATTR_POS_IN_SET, selected_index_);
+  node_data->AddIntAttribute(ui::AX_ATTR_SET_SIZE, model_->GetItemCount());
 }
 
 void Combobox::ButtonPressed(Button* sender, const ui::Event& event) {
   if (!enabled())
     return;
 
-  RequestFocus();
+  if (!UseMd())
+    RequestFocus();
 
   if (sender == text_button_) {
     OnPerformAction();
@@ -708,12 +788,11 @@ void Combobox::ButtonPressed(Button* sender, const ui::Event& event) {
 }
 
 void Combobox::UpdateBorder() {
-  std::unique_ptr<FocusableBorder> border(
-      PlatformStyle::CreateComboboxBorder());
+  std::unique_ptr<FocusableBorder> border(new FocusableBorder());
   if (style_ == STYLE_ACTION)
     border->SetInsets(5, 10, 5, 10);
   if (invalid_)
-    border->SetColor(gfx::kGoogleRed700);
+    border->SetColorId(ui::NativeTheme::kColorId_AlertSeverityHigh);
   SetBorder(std::move(border));
 }
 
@@ -732,8 +811,10 @@ void Combobox::PaintText(gfx::Canvas* canvas) {
   int y = insets.top();
   int text_height = height() - insets.height();
   SkColor text_color = GetNativeTheme()->GetSystemColor(
-      enabled() ? ui::NativeTheme::kColorId_LabelEnabledColor :
-                  ui::NativeTheme::kColorId_LabelDisabledColor);
+      UseMd() ? (enabled() ? ui::NativeTheme::kColorId_TextfieldDefaultColor
+                           : ui::NativeTheme::kColorId_TextfieldReadOnlyColor)
+              : (enabled() ? ui::NativeTheme::kColorId_LabelEnabledColor
+                           : ui::NativeTheme::kColorId_LabelDisabledColor));
 
   DCHECK_GE(selected_index_, 0);
   DCHECK_LT(selected_index_, model()->GetItemCount());
@@ -758,13 +839,41 @@ void Combobox::PaintText(gfx::Canvas* canvas) {
       PositionArrowWithinContainer(arrow_bounds, ArrowSize(), style_);
   AdjustBoundsForRTLUI(&arrow_bounds);
 
-  canvas->DrawImageInt(arrow_image_, arrow_bounds.x(), arrow_bounds.y());
+  if (UseMd()) {
+    // Since this is a core piece of UI and vector icons don't handle fractional
+    // scale factors particularly well, manually draw an arrow and make sure it
+    // looks good at all scale factors.
+    float dsf = canvas->UndoDeviceScaleFactor();
+    SkScalar x = std::ceil(arrow_bounds.x() * dsf);
+    SkScalar y = std::ceil(arrow_bounds.y() * dsf);
+    SkScalar height = std::floor(arrow_bounds.height() * dsf);
+    SkPath path;
+    // This epsilon makes sure that all the aliasing pixels are slightly more
+    // than half full. Otherwise, rounding issues cause some to be considered
+    // slightly less than half full and come out a little lighter.
+    const SkScalar kEpsilon = 0.0001f;
+    path.moveTo(x - kEpsilon, y);
+    path.rLineTo(height, height);
+    path.rLineTo(2 * kEpsilon, 0);
+    path.rLineTo(height, -height);
+    path.close();
+    SkPaint paint;
+    SkColor arrow_color = GetNativeTheme()->GetSystemColor(
+        ui::NativeTheme::kColorId_ButtonEnabledColor);
+    if (!enabled())
+      arrow_color = SkColorSetA(arrow_color, gfx::kDisabledControlAlpha);
+    paint.setColor(arrow_color);
+    paint.setAntiAlias(true);
+    canvas->DrawPath(path, paint);
+  } else {
+    canvas->DrawImageInt(arrow_image_, arrow_bounds.x(), arrow_bounds.y());
+  }
 }
 
 void Combobox::PaintButtons(gfx::Canvas* canvas) {
   DCHECK(style_ == STYLE_ACTION);
 
-  gfx::ScopedRTLFlipCanvas scoped_canvas(canvas, bounds());
+  gfx::ScopedRTLFlipCanvas scoped_canvas(canvas, width());
 
   bool focused = HasFocus();
   const std::vector<const gfx::ImageSkia*>& arrow_button_images =
@@ -841,22 +950,20 @@ void Combobox::ShowDropDownMenu(ui::MenuSourceType source_type) {
   // Allow |menu_runner_| to be set by the testing API, but if this method is
   // ever invoked recursively, ensure the old menu is closed.
   if (!menu_runner_ || menu_runner_->IsRunning()) {
-    menu_runner_.reset(
-        new MenuRunner(menu_model_adapter_.get(), MenuRunner::COMBOBOX));
-  }
-  if (menu_runner_->RunMenuAt(GetWidget(), nullptr, bounds, anchor_position,
-                              source_type) == MenuRunner::MENU_DELETED) {
-    return;
+    menu_runner_.reset(new MenuRunner(
+        menu_model_.get(), MenuRunner::COMBOBOX | MenuRunner::ASYNC,
+        base::Bind(&Combobox::OnMenuClosed, base::Unretained(this),
+                   original_state)));
   }
+  menu_runner_->RunMenuAt(GetWidget(), nullptr, bounds, anchor_position,
+                          source_type);
+}
+
+void Combobox::OnMenuClosed(Button::ButtonState original_button_state) {
   menu_runner_.reset();
   if (arrow_button_)
-    arrow_button_->SetState(original_state);
+    arrow_button_->SetState(original_button_state);
   closed_time_ = base::Time::Now();
-
-  // Need to explicitly clear mouse handler so that events get sent
-  // properly after the menu finishes running. If we don't do this, then
-  // the first click to other parts of the UI is eaten.
-  SetMouseHandler(NULL);
 }
 
 void Combobox::OnPerformAction() {
@@ -873,7 +980,7 @@ void Combobox::OnPerformAction() {
 }
 
 gfx::Size Combobox::ArrowSize() const {
-  return arrow_image_.size();
+  return UseMd() ? gfx::Size(8, 4) : arrow_image_.size();
 }
 
 gfx::Size Combobox::GetContentSize() const {
@@ -886,8 +993,7 @@ gfx::Size Combobox::GetContentSize() const {
 
     if (size_to_largest_label_ || i == selected_index_) {
       width = std::max(
-          width,
-          gfx::GetStringWidth(menu_model_adapter_->GetLabelAt(i), font_list));
+          width, gfx::GetStringWidth(menu_model_->GetLabelAt(i), font_list));
     }
   }
   return gfx::Size(width, font_list.GetHeight());
@@ -895,13 +1001,16 @@ gfx::Size Combobox::GetContentSize() const {
 
 PrefixSelector* Combobox::GetPrefixSelector() {
   if (!selector_)
-    selector_.reset(new PrefixSelector(this));
+    selector_.reset(new PrefixSelector(this, this));
   return selector_.get();
 }
 
 int Combobox::GetArrowContainerWidth() const {
+  const int kMdPaddingWidth = 8;
+  int arrow_pad = UseMd() ? kMdPaddingWidth
+                          : PlatformStyle::kComboboxNormalArrowPadding;
   int padding = style_ == STYLE_NORMAL
-                    ? PlatformStyle::kComboboxNormalArrowPadding * 2
+                    ? arrow_pad * 2
                     : kActionLeftPadding + kActionRightPadding;
   return ArrowSize().width() + padding;
 }
