@@ -7,20 +7,15 @@
 #include "base/callback.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
+#include "ui/views/widget/widget.h"
+
+#if defined(USE_AURA)
+#include "ui/aura/window.h"
+#endif
 
 namespace views {
 namespace {
 
-class DefaultPlatformTestHelper : public PlatformTestHelper {
- public:
-  DefaultPlatformTestHelper() {}
-
-  ~DefaultPlatformTestHelper() override {}
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(DefaultPlatformTestHelper);
-};
-
 PlatformTestHelper::Factory test_helper_factory;
 bool is_mus = false;
 
@@ -35,7 +30,7 @@ void PlatformTestHelper::set_factory(const Factory& factory) {
 std::unique_ptr<PlatformTestHelper> PlatformTestHelper::Create() {
   return !test_helper_factory.is_null()
              ? test_helper_factory.Run()
-             : base::WrapUnique(new DefaultPlatformTestHelper);
+             : base::WrapUnique(new PlatformTestHelper);
 }
 
 // static
@@ -48,4 +43,10 @@ bool PlatformTestHelper::IsMus() {
   return is_mus;
 }
 
+#if defined(USE_AURA)
+void PlatformTestHelper::SimulateNativeDestroy(Widget* widget) {
+  delete widget->GetNativeView();
+}
+#endif
+
 }  // namespace views
