@@ -14,6 +14,7 @@ class MessageLoopForUI;
 
 namespace ui {
 class ContextFactory;
+class ContextFactoryPrivate;
 }
 
 namespace views {
@@ -26,8 +27,10 @@ class ViewsTestHelper {
   virtual ~ViewsTestHelper();
 
   // Create a platform specific instance.
-  static ViewsTestHelper* Create(base::MessageLoopForUI* message_loop,
-                                 ui::ContextFactory* context_factory);
+  static ViewsTestHelper* Create(
+      base::MessageLoopForUI* message_loop,
+      ui::ContextFactory* context_factory,
+      ui::ContextFactoryPrivate* context_factory_private);
 
   // Creates objects that are needed for tests.
   virtual void SetUp();
