@@ -6,12 +6,15 @@
 
 #include "base/bind.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
+#include "services/ui/public/interfaces/window_manager_constants.mojom.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/client/cursor_client.h"
+#include "ui/aura/client/drag_drop_client.h"
 #include "ui/aura/client/focus_client.h"
-#include "ui/aura/client/window_tree_client.h"
+#include "ui/aura/client/window_parenting_client.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_observer.h"
 #include "ui/aura/window_property.h"
@@ -24,19 +27,18 @@
 #include "ui/gfx/geometry/point_conversions.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size_conversions.h"
-#include "ui/native_theme/native_theme.h"
 #include "ui/views/corewm/tooltip.h"
 #include "ui/views/corewm/tooltip_controller.h"
 #include "ui/views/drag_utils.h"
 #include "ui/views/view_constants_aura.h"
 #include "ui/views/widget/desktop_aura/desktop_capture_client.h"
-#include "ui/views/widget/desktop_aura/desktop_cursor_loader_updater.h"
 #include "ui/views/widget/desktop_aura/desktop_event_client.h"
 #include "ui/views/widget/desktop_aura/desktop_focus_rules.h"
 #include "ui/views/widget/desktop_aura/desktop_native_cursor_manager.h"
 #include "ui/views/widget/desktop_aura/desktop_screen_position_client.h"
 #include "ui/views/widget/desktop_aura/desktop_window_tree_host.h"
 #include "ui/views/widget/drop_helper.h"
+#include "ui/views/widget/focus_manager_event_handler.h"
 #include "ui/views/widget/native_widget_aura.h"
 #include "ui/views/widget/root_view.h"
 #include "ui/views/widget/tooltip_manager_aura.h"
@@ -44,7 +46,6 @@
 #include "ui/views/widget/widget_aura_utils.h"
 #include "ui/views/widget/widget_delegate.h"
 #include "ui/views/widget/window_reorderer.h"
-#include "ui/views/window/native_frame_view.h"
 #include "ui/wm/core/compound_event_filter.h"
 #include "ui/wm/core/cursor_manager.h"
 #include "ui/wm/core/focus_controller.h"
@@ -55,7 +56,6 @@
 #include "ui/wm/core/window_animations.h"
 #include "ui/wm/core/window_modality_controller.h"
 #include "ui/wm/public/activation_client.h"
-#include "ui/wm/public/drag_drop_client.h"
 
 #if defined(OS_WIN)
 #include "ui/base/win/shell.h"
@@ -167,19 +167,19 @@ class DesktopNativeWidgetTopLevelHandler : public aura::WindowObserver {
   DISALLOW_COPY_AND_ASSIGN(DesktopNativeWidgetTopLevelHandler);
 };
 
-class DesktopNativeWidgetAuraWindowTreeClient :
-    public aura::client::WindowTreeClient {
+class DesktopNativeWidgetAuraWindowParentingClient
+    : public aura::client::WindowParentingClient {
  public:
-  explicit DesktopNativeWidgetAuraWindowTreeClient(
+  explicit DesktopNativeWidgetAuraWindowParentingClient(
       aura::Window* root_window)
       : root_window_(root_window) {
-    aura::client::SetWindowTreeClient(root_window_, this);
+    aura::client::SetWindowParentingClient(root_window_, this);
   }
-  ~DesktopNativeWidgetAuraWindowTreeClient() override {
-    aura::client::SetWindowTreeClient(root_window_, NULL);
+  ~DesktopNativeWidgetAuraWindowParentingClient() override {
+    aura::client::SetWindowParentingClient(root_window_, NULL);
   }
 
-  // Overridden from client::WindowTreeClient:
+  // Overridden from client::WindowParentingClient:
   aura::Window* GetDefaultParent(aura::Window* context,
                                  aura::Window* window,
                                  const gfx::Rect& bounds) override {
@@ -203,32 +203,11 @@ class DesktopNativeWidgetAuraWindowTreeClient :
  private:
   aura::Window* root_window_;
 
-  DISALLOW_COPY_AND_ASSIGN(DesktopNativeWidgetAuraWindowTreeClient);
+  DISALLOW_COPY_AND_ASSIGN(DesktopNativeWidgetAuraWindowParentingClient);
 };
 
 }  // namespace
 
-class FocusManagerEventHandler : public ui::EventHandler {
- public:
-  explicit FocusManagerEventHandler(
-      DesktopNativeWidgetAura* desktop_native_widget_aura)
-      : desktop_native_widget_aura_(desktop_native_widget_aura) {}
-
-  // Implementation of ui::EventHandler:
-  void OnKeyEvent(ui::KeyEvent* event) override {
-    Widget* widget = desktop_native_widget_aura_->GetWidget();
-    if (widget && widget->GetFocusManager()->GetFocusedView() &&
-        !widget->GetFocusManager()->OnKeyEvent(*event)) {
-      event->StopPropagation();
-    }
-  }
-
- private:
-  DesktopNativeWidgetAura* desktop_native_widget_aura_;
-
-  DISALLOW_COPY_AND_ASSIGN(FocusManagerEventHandler);
-};
-
 class RootWindowDestructionObserver : public aura::WindowObserver {
  public:
   explicit RootWindowDestructionObserver(DesktopNativeWidgetAura* parent)
@@ -285,6 +264,13 @@ DesktopNativeWidgetAura* DesktopNativeWidgetAura::ForWindow(
   return window->GetProperty(kDesktopNativeWidgetAuraKey);
 }
 
+void DesktopNativeWidgetAura::SetDesktopWindowTreeHost(
+    std::unique_ptr<DesktopWindowTreeHost> desktop_window_tree_host) {
+  DCHECK(!desktop_window_tree_host_);
+  desktop_window_tree_host_ = desktop_window_tree_host.get();
+  host_.reset(desktop_window_tree_host.release()->AsWindowTreeHost());
+}
+
 void DesktopNativeWidgetAura::OnHostClosed() {
   // Don't invoke Widget::OnNativeWidgetDestroying(), its done by
   // DesktopWindowTreeHost.
@@ -312,10 +298,12 @@ void DesktopNativeWidgetAura::OnHostClosed() {
     tooltip_controller_.reset();
   }
 
-  window_tree_client_.reset();  // Uses host_->dispatcher() at destruction.
+  window_parenting_client_.reset();  // Uses host_->dispatcher() at destruction.
 
   capture_client_.reset();  // Uses host_->dispatcher() at destruction.
 
+  focus_manager_event_handler_.reset();
+
   // FocusController uses |content_window_|. Destroy it now so that we don't
   // have to worry about the possibility of FocusController attempting to use
   // |content_window_| after it's been destroyed but before all child windows
@@ -338,14 +326,16 @@ void DesktopNativeWidgetAura::OnHostClosed() {
 
 void DesktopNativeWidgetAura::OnDesktopWindowTreeHostDestroyed(
     aura::WindowTreeHost* host) {
-  // We explicitly do NOT clear the cursor client property. Since the cursor
-  // manager is a singleton, it can outlive any window hierarchy, and it's
-  // important that objects attached to this destroying window hierarchy have
-  // an opportunity to deregister their observers from the cursor manager.
-  // They may want to do this when they are notified that they're being
-  // removed from the window hierarchy, which happens soon after this
-  // function when DesktopWindowTreeHost* calls DestroyDispatcher().
-  native_cursor_manager_->RemoveHost(host);
+  if (use_desktop_native_cursor_manager_) {
+    // We explicitly do NOT clear the cursor client property. Since the cursor
+    // manager is a singleton, it can outlive any window hierarchy, and it's
+    // important that objects attached to this destroying window hierarchy have
+    // an opportunity to deregister their observers from the cursor manager.
+    // They may want to do this when they are notified that they're being
+    // removed from the window hierarchy, which happens soon after this
+    // function when DesktopWindowTreeHost* calls DestroyDispatcher().
+    native_cursor_manager_->RemoveHost(host);
+  }
 
   aura::client::SetScreenPositionClient(host->window(), NULL);
   position_client_.reset();
@@ -368,12 +358,19 @@ void DesktopNativeWidgetAura::HandleActivationChanged(bool active) {
       // This function can be called before the focus manager has had a
       // chance to set the focused view. In which case we should get the
       // last focused view.
-      View* view_for_activation =
-          GetWidget()->GetFocusManager()->GetFocusedView() ?
-              GetWidget()->GetFocusManager()->GetFocusedView() :
-                  GetWidget()->GetFocusManager()->GetStoredFocusView();
-      if (!view_for_activation)
+      views::FocusManager* focus_manager = GetWidget()->GetFocusManager();
+      View* view_for_activation = focus_manager->GetFocusedView()
+                                      ? focus_manager->GetFocusedView()
+                                      : focus_manager->GetStoredFocusView();
+      if (!view_for_activation) {
         view_for_activation = GetWidget()->GetRootView();
+      } else if (view_for_activation == focus_manager->GetStoredFocusView()) {
+        focus_manager->RestoreFocusedView();
+        // Set to false if desktop native widget has activated activation
+        // change, so that aura window activation change focus restore operation
+        // can be ignored.
+        restore_focus_on_activate_ = false;
+      }
       activation_client->ActivateWindow(
           view_for_activation->GetWidget()->GetNativeView());
       // Refreshes the focus info to IMF in case that IMF cached the old info
@@ -412,20 +409,22 @@ void DesktopNativeWidgetAura::InitNativeWidget(
   }
   content_window_->SetType(GetAuraWindowTypeForWidgetType(params.type));
   content_window_->Init(params.layer_type);
-  wm::SetShadowType(content_window_, wm::SHADOW_TYPE_NONE);
+  wm::SetShadowElevation(content_window_, wm::ShadowElevation::NONE);
 
   content_window_container_ = new aura::Window(NULL);
   content_window_container_->Init(ui::LAYER_NOT_DRAWN);
   content_window_container_->Show();
   content_window_container_->AddChild(content_window_);
 
-  desktop_window_tree_host_ = params.desktop_window_tree_host ?
-      params.desktop_window_tree_host :
-      DesktopWindowTreeHost::Create(native_widget_delegate_, this);
-  host_.reset(desktop_window_tree_host_->AsWindowTreeHost());
+  if (!desktop_window_tree_host_) {
+    desktop_window_tree_host_ =
+        params.desktop_window_tree_host
+            ? params.desktop_window_tree_host
+            : DesktopWindowTreeHost::Create(native_widget_delegate_, this);
+    host_.reset(desktop_window_tree_host_->AsWindowTreeHost());
+  }
   desktop_window_tree_host_->Init(content_window_, params);
 
-  host_->InitHost();
   host_->window()->AddChild(content_window_container_);
   host_->window()->SetProperty(kDesktopNativeWidgetAuraKey, this);
 
@@ -449,19 +448,21 @@ void DesktopNativeWidgetAura::InitNativeWidget(
   root_window_event_filter_.reset(new wm::CompoundEventFilter);
   host_->window()->AddPreTargetHandler(root_window_event_filter_.get());
 
-  // The host's dispatcher must be added to |native_cursor_manager_| before
-  // OnNativeWidgetCreated() is called.
-  cursor_reference_count_++;
-  if (!native_cursor_manager_) {
-    native_cursor_manager_ = new DesktopNativeCursorManager(
-        DesktopCursorLoaderUpdater::Create());
+  use_desktop_native_cursor_manager_ =
+      desktop_window_tree_host_->ShouldUseDesktopNativeCursorManager();
+  if (use_desktop_native_cursor_manager_) {
+    // The host's dispatcher must be added to |native_cursor_manager_| before
+    // OnNativeWidgetCreated() is called.
+    cursor_reference_count_++;
+    if (!native_cursor_manager_)
+      native_cursor_manager_ = new DesktopNativeCursorManager();
+    if (!cursor_manager_) {
+      cursor_manager_ = new wm::CursorManager(
+          std::unique_ptr<wm::NativeCursorManager>(native_cursor_manager_));
+    }
+    native_cursor_manager_->AddHost(host());
+    aura::client::SetCursorClient(host_->window(), cursor_manager_);
   }
-  if (!cursor_manager_) {
-    cursor_manager_ = new wm::CursorManager(
-        std::unique_ptr<wm::NativeCursorManager>(native_cursor_manager_));
-  }
-  native_cursor_manager_->AddHost(host());
-  aura::client::SetCursorClient(host_->window(), cursor_manager_);
 
   desktop_window_tree_host_->OnNativeWidgetCreated(params);
 
@@ -480,8 +481,9 @@ void DesktopNativeWidgetAura::InitNativeWidget(
 
   drag_drop_client_ = desktop_window_tree_host_->CreateDragDropClient(
       native_cursor_manager_);
-  aura::client::SetDragDropClient(host_->window(),
-                                  drag_drop_client_.get());
+  // Mus returns null from CreateDragDropClient().
+  if (drag_drop_client_)
+    aura::client::SetDragDropClient(host_->window(), drag_drop_client_.get());
 
   static_cast<aura::client::FocusClient*>(focus_client_.get())->
       FocusWindow(content_window_);
@@ -490,8 +492,8 @@ void DesktopNativeWidgetAura::InitNativeWidget(
 
   host_->AddObserver(this);
 
-  window_tree_client_.reset(
-      new DesktopNativeWidgetAuraWindowTreeClient(host_->window()));
+  window_parenting_client_.reset(
+      new DesktopNativeWidgetAuraWindowParentingClient(host_->window()));
   drop_helper_.reset(new DropHelper(GetWidget()->GetRootView()));
   aura::client::SetDragDropDelegate(content_window_, this);
 
@@ -515,8 +517,8 @@ void DesktopNativeWidgetAura::InitNativeWidget(
   }
 
   if (params.type == Widget::InitParams::TYPE_WINDOW) {
-    focus_manager_event_handler_.reset(new FocusManagerEventHandler(this));
-    host_->window()->AddPreTargetHandler(focus_manager_event_handler_.get());
+    focus_manager_event_handler_ = base::MakeUnique<FocusManagerEventHandler>(
+        GetWidget(), host_->window());
   }
 
   event_client_.reset(new DesktopEventClient);
@@ -535,10 +537,12 @@ void DesktopNativeWidgetAura::InitNativeWidget(
       GetWidget()->GetRootView()));
 }
 
-void DesktopNativeWidgetAura::OnWidgetInitDone() {}
+void DesktopNativeWidgetAura::OnWidgetInitDone() {
+  desktop_window_tree_host_->OnWidgetInitDone();
+}
 
 NonClientFrameView* DesktopNativeWidgetAura::CreateNonClientFrameView() {
-  return ShouldUseNativeFrame() ? new NativeFrameView(GetWidget()) : NULL;
+  return desktop_window_tree_host_->CreateNonClientFrameView();
 }
 
 bool DesktopNativeWidgetAura::ShouldUseNativeFrame() const {
@@ -651,6 +655,9 @@ void DesktopNativeWidgetAura::SetWindowIcons(const gfx::ImageSkia& window_icon,
                                              const gfx::ImageSkia& app_icon) {
   if (content_window_)
     desktop_window_tree_host_->SetWindowIcons(window_icon, app_icon);
+
+  NativeWidgetAura::AssignIconToAuraWindow(content_window_, window_icon,
+                                           app_icon);
 }
 
 void DesktopNativeWidgetAura::InitModalType(ui::ModalType modal_type) {
@@ -686,7 +693,8 @@ void DesktopNativeWidgetAura::SetBounds(const gfx::Rect& bounds) {
   aura::Window* root = host_->window();
   display::Screen* screen = display::Screen::GetScreen();
   gfx::Rect bounds_in_pixels = screen->DIPToScreenRectInWindow(root, bounds);
-  desktop_window_tree_host_->AsWindowTreeHost()->SetBounds(bounds_in_pixels);
+  desktop_window_tree_host_->AsWindowTreeHost()->SetBoundsInPixels(
+      bounds_in_pixels);
 }
 
 void DesktopNativeWidgetAura::SetSize(const gfx::Size& size) {
@@ -704,12 +712,9 @@ void DesktopNativeWidgetAura::StackAtTop() {
     desktop_window_tree_host_->StackAtTop();
 }
 
-void DesktopNativeWidgetAura::StackBelow(gfx::NativeView native_view) {
-}
-
-void DesktopNativeWidgetAura::SetShape(SkRegion* shape) {
+void DesktopNativeWidgetAura::SetShape(std::unique_ptr<SkRegion> shape) {
   if (content_window_)
-    desktop_window_tree_host_->SetShape(shape);
+    desktop_window_tree_host_->SetShape(std::move(shape));
 }
 
 void DesktopNativeWidgetAura::Close() {
@@ -743,6 +748,7 @@ void DesktopNativeWidgetAura::Hide() {
 
 void DesktopNativeWidgetAura::ShowMaximizedWithBounds(
       const gfx::Rect& restored_bounds) {
+  // IsVisible() should check the same objects here for visibility.
   if (!content_window_)
     return;
   desktop_window_tree_host_->ShowMaximizedWithBounds(restored_bounds);
@@ -750,6 +756,7 @@ void DesktopNativeWidgetAura::ShowMaximizedWithBounds(
 }
 
 void DesktopNativeWidgetAura::ShowWithWindowState(ui::WindowShowState state) {
+  // IsVisible() should check the same objects here for visibility.
   if (!content_window_)
     return;
   desktop_window_tree_host_->ShowWindowWithState(state);
@@ -757,7 +764,13 @@ void DesktopNativeWidgetAura::ShowWithWindowState(ui::WindowShowState state) {
 }
 
 bool DesktopNativeWidgetAura::IsVisible() const {
-  return content_window_ && desktop_window_tree_host_->IsVisible();
+  // The objects checked here should be the same objects changed in
+  // ShowWithWindowState and ShowMaximizedWithBounds. For example, MS Windows
+  // platform code might show the desktop window tree host early, meaning we
+  // aren't fully visible as we haven't shown the content window. Callers may
+  // short-circuit a call to show this widget if they think its already visible.
+  return content_window_ && content_window_->IsVisible() &&
+      desktop_window_tree_host_->IsVisible();
 }
 
 void DesktopNativeWidgetAura::Activate() {
@@ -788,6 +801,11 @@ void DesktopNativeWidgetAura::SetVisibleOnAllWorkspaces(bool always_visible) {
     desktop_window_tree_host_->SetVisibleOnAllWorkspaces(always_visible);
 }
 
+bool DesktopNativeWidgetAura::IsVisibleOnAllWorkspaces() const {
+  return content_window_ &&
+         desktop_window_tree_host_->IsVisibleOnAllWorkspaces();
+}
+
 void DesktopNativeWidgetAura::Maximize() {
   if (content_window_)
     desktop_window_tree_host_->Maximize();
@@ -923,27 +941,16 @@ void DesktopNativeWidgetAura::SetVisibilityAnimationTransition(
   wm::SetWindowVisibilityAnimationTransition(content_window_, wm_transition);
 }
 
-ui::NativeTheme* DesktopNativeWidgetAura::GetNativeTheme() const {
-  return DesktopWindowTreeHost::GetNativeTheme(content_window_);
-}
-
-void DesktopNativeWidgetAura::OnRootViewLayout() {
-  if (content_window_)
-    desktop_window_tree_host_->OnRootViewLayout();
-}
-
 bool DesktopNativeWidgetAura::IsTranslucentWindowOpacitySupported() const {
   return content_window_ &&
       desktop_window_tree_host_->IsTranslucentWindowOpacitySupported();
 }
 
 void DesktopNativeWidgetAura::OnSizeConstraintsChanged() {
-  content_window_->SetProperty(aura::client::kCanMaximizeKey,
-                               GetWidget()->widget_delegate()->CanMaximize());
-  content_window_->SetProperty(aura::client::kCanMinimizeKey,
-                               GetWidget()->widget_delegate()->CanMinimize());
-  content_window_->SetProperty(aura::client::kCanResizeKey,
-                               GetWidget()->widget_delegate()->CanResize());
+  int32_t behavior = ui::mojom::kResizeBehaviorNone;
+  if (GetWidget()->widget_delegate())
+    behavior = GetWidget()->widget_delegate()->GetResizeBehavior();
+  content_window_->SetProperty(aura::client::kResizeBehaviorKey, behavior);
   desktop_window_tree_host_->SizeConstraintsChanged();
 }
 
@@ -1086,13 +1093,21 @@ void DesktopNativeWidgetAura::OnWindowActivated(
   DCHECK(content_window_ == gained_active || content_window_ == lost_active);
   if (gained_active == content_window_ && restore_focus_on_activate_) {
     restore_focus_on_activate_ = false;
-    GetWidget()->GetFocusManager()->RestoreFocusedView();
+    // For OS_LINUX, desktop native widget may not be activated when child
+    // widgets gets aura activation changes. Only when desktop native widget is
+    // active, we can rely on aura activation to restore focused view.
+    if (GetWidget()->IsActive())
+      GetWidget()->GetFocusManager()->RestoreFocusedView();
   } else if (lost_active == content_window_ && GetWidget()->HasFocusManager()) {
     DCHECK(!restore_focus_on_activate_);
     restore_focus_on_activate_ = true;
     // Pass in false so that ClearNativeFocus() isn't invoked.
     GetWidget()->GetFocusManager()->StoreFocusedView(false);
   }
+
+  // Give the native widget a chance to handle any specific changes it needs.
+  desktop_window_tree_host_->OnNativeWidgetActivationChanged(
+      content_window_ == gained_active);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1101,10 +1116,8 @@ void DesktopNativeWidgetAura::OnWindowActivated(
 void DesktopNativeWidgetAura::OnWindowFocused(aura::Window* gained_focus,
                                               aura::Window* lost_focus) {
   if (content_window_ == gained_focus) {
-    desktop_window_tree_host_->OnNativeWidgetFocus();
     native_widget_delegate_->OnNativeFocus();
   } else if (content_window_ == lost_focus) {
-    desktop_window_tree_host_->OnNativeWidgetBlur();
     native_widget_delegate_->OnNativeBlur();
   }
 }
@@ -1166,10 +1179,11 @@ void DesktopNativeWidgetAura::OnHostWorkspaceChanged(
   native_widget_delegate_->OnNativeWidgetWorkspaceChanged();
 }
 
-void DesktopNativeWidgetAura::OnHostMoved(const aura::WindowTreeHost* host,
-                                          const gfx::Point& new_origin) {
-  TRACE_EVENT1("views", "DesktopNativeWidgetAura::OnHostMoved",
-               "new_origin", new_origin.ToString());
+void DesktopNativeWidgetAura::OnHostMovedInPixels(
+    const aura::WindowTreeHost* host,
+    const gfx::Point& new_origin_in_pixels) {
+  TRACE_EVENT1("views", "DesktopNativeWidgetAura::OnHostMovedInPixels",
+               "new_origin_in_pixels", new_origin_in_pixels.ToString());
 
   native_widget_delegate_->OnNativeWidgetMove();
 }
@@ -1178,6 +1192,9 @@ void DesktopNativeWidgetAura::OnHostMoved(const aura::WindowTreeHost* host,
 // DesktopNativeWidgetAura, private:
 
 void DesktopNativeWidgetAura::UpdateWindowTransparency() {
+  if (!desktop_window_tree_host_->ShouldUpdateWindowTransparency())
+    return;
+
   content_window_->SetTransparent(
       desktop_window_tree_host_->ShouldWindowContentsBeTransparent());
   // Regardless of transparency or not, this root content window will always
