@@ -5,6 +5,10 @@
 #ifndef UI_VIEWS_WIDGET_DESKTOP_AURA_X11_MOVE_LOOP_DELEGATE_H_
 #define UI_VIEWS_WIDGET_DESKTOP_AURA_X11_MOVE_LOOP_DELEGATE_H_
 
+namespace gfx {
+class Point;
+}
+
 namespace views {
 
 // Receives mouse events while the X11MoveLoop is tracking a drag.
