@@ -9,7 +9,7 @@
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/message_loop/message_loop.h"
-#include "ui/accessibility/ax_view_state.h"
+#include "ui/accessibility/ax_node_data.h"
 #include "ui/base/cursor/cursor.h"
 #include "ui/base/dragdrop/drag_drop_types.h"
 #include "ui/base/ui_base_switches_util.h"
@@ -317,11 +317,6 @@ bool RootView::IsDrawn() const {
   return visible();
 }
 
-void RootView::Layout() {
-  View::Layout();
-  widget_->OnRootViewLayout();
-}
-
 const char* RootView::GetClassName() const {
   return kViewClassName;
 }
@@ -600,9 +595,9 @@ void RootView::SetMouseHandler(View* new_mh) {
   drag_info_.Reset();
 }
 
-void RootView::GetAccessibleState(ui::AXViewState* state) {
-  state->name = widget_->widget_delegate()->GetAccessibleWindowTitle();
-  state->role = widget_->widget_delegate()->GetAccessibleWindowRole();
+void RootView::GetAccessibleNodeData(ui::AXNodeData* node_data) {
+  node_data->SetName(widget_->widget_delegate()->GetAccessibleWindowTitle());
+  node_data->role = widget_->widget_delegate()->GetAccessibleWindowRole();
 }
 
 void RootView::UpdateParentLayer() {
@@ -647,7 +642,7 @@ void RootView::VisibilityChanged(View* /*starting_from*/, bool is_visible) {
 
 void RootView::OnPaint(gfx::Canvas* canvas) {
   if (!layer() || !layer()->fills_bounds_opaquely())
-    canvas->DrawColor(SK_ColorBLACK, SkXfermode::kClear_Mode);
+    canvas->DrawColor(SK_ColorBLACK, SkBlendMode::kClear);
 
   View::OnPaint(canvas);
 }
